/*==========================================================================
//
//      dhcp_support.c
//
//      Support code == friendly API for DHCP client
//
//==========================================================================
//####ECOSPDCOPYRIGHTBEGIN####
//
// Copyright (C) 2000, 2001, 2002 Red Hat, Inc.
// All Rights Reserved.
//
// Permission is granted to use, copy, modify and redistribute this
// file.
//
//####ECOSPDCOPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):   hmt
// Contributors: gthomas
// Date:        2000-07-01
// Purpose:     DHCP support
// Description: 
//
//####DESCRIPTIONEND####
//
//========================================================================*/

#include <pkgconf/system.h>
#include <pkgconf/net.h>

/* Define these locally because we are porting this code from a later package,
 * therefore these are not defined in pkgconf/net.h */
#define CYGPKG_NET_DHCP
#define CYGOPT_NET_DHCP_DHCP_THREAD

#ifdef CYGPKG_NET_DHCP

#include <network.h>
#include <dhcp.h>

typedef struct MIB_DHCP_s
{
    unsigned long IPAddr;
    unsigned long SubnetMask;
    unsigned long GwyAddr;
}
MIB_DHCP;

#ifdef AP5X
MIB_DHCP mib_DHCP;
#define SUBNETMASK  (mib_DHCP.SubnetMask)
#define IPADDR      (mib_DHCP.IPAddr)
#define GWYADDR     (mib_DHCP.GwyAddr)
#else
extern MIB_DHCP *mib_DHCP_p;
#define SUBNETMASK  (mib_DHCP_p->SubnetMask)
#define IPADDR      (mib_DHCP_p->IPAddr)
#define GWYADDR     (mib_DHCP_p->GwyAddr)
#endif
extern unsigned char mib_dhcpenable;
void init_static_network_interfaces(int);
void update_IP_Info(void);
extern int SetUserConfig(void);
extern void Reset(void);

static unsigned long ipaddr;
// ---------------------------------------------------------------------------
#ifdef CYGHWR_NET_DRIVER_ETH0
cyg_uint8 eth0_dhcpstate = 0;
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
cyg_uint8 eth1_dhcpstate = 0;
#endif

cyg_sem_t dhcp_needs_attention;

#ifdef CYGHWR_NET_DRIVER_ETH0
struct dhcp_lease eth0_lease =
    {
        &dhcp_needs_attention, 0, 0, 0, 0, 0, 0
    };
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
struct dhcp_lease eth1_lease =
    {
        &dhcp_needs_attention, 0, 0, 0, 0, 0, 0
    };
#endif


// ---------------------------------------------------------------------------
//
// The point of this module is to deal with all the horrid written out in
// full stuff of having two interfaces; it's ugly but it's also most
// flexible.  The dhcp_prot.c module should do all the work...
//
// ---------------------------------------------------------------------------

// return value: 1 => everything OK, no change.
// 0 => close your connections, then call do_dhcp_halt() to halt the
// interface(s) in question (it knows because the state will be NOTBOUND).
// After that you can return to the start and use
// init_all_network_interfaces(); as usual, or call do_dhcp_bind() by hand,
// or whatever...
int dhcp_bind( void )
{
#ifdef CYGHWR_NET_DRIVER_ETH0
    cyg_uint8 old_eth0_dhcpstate = eth0_dhcpstate;
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1

    cyg_uint8 old_eth1_dhcpstate = eth1_dhcpstate;
#endif

    // If there are no interfaces at all, init it every time, doesn't
    // matter.  In case we are called from elsewhere...
    if ( 1
#ifdef CYGHWR_NET_DRIVER_ETH0
            && eth0_dhcpstate == 0
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
            && eth1_dhcpstate == 0
#endif
       )
        cyg_semaphore_init( &dhcp_needs_attention, 0 );

    // Run the state machine...
#ifdef CYGHWR_NET_DRIVER_ETH0

    if (eth0_up
            && DHCPSTATE_FAILED != eth0_dhcpstate )
        eth0_up = do_dhcp(eth0_name, &eth0_bootp_data, &eth0_dhcpstate, &eth0_lease);
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1

    if (eth1_up
            && DHCPSTATE_FAILED != eth1_dhcpstate )
        eth1_up = do_dhcp(eth1_name, &eth1_bootp_data, &eth1_dhcpstate, &eth1_lease);
#endif

    // If the interface newly came up, initialize it:
    // (this duplicates the code in init_all_network_interfaces() really).
#ifdef CYGHWR_NET_DRIVER_ETH0

    if ( eth0_up
            && eth0_dhcpstate == DHCPSTATE_BOUND
            && old_eth0_dhcpstate != eth0_dhcpstate )
    {
        ipaddr = inet_address(eth0_bootp_data.bp_yiaddr.s_addr);
        if (!init_net(eth0_name, &eth0_bootp_data))
        {
            //if the server give us wrong gateway addr, net mask etc, init_net will fail
            //so we use the ip address and init ourself
            //eth0_up = false;
            init_static_network_interfaces(1);
            ipaddr = 0;
        }
    }
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
    if ( eth1_up
            && eth1_dhcpstate == DHCPSTATE_BOUND
            && old_eth1_dhcpstate != eth1_dhcpstate )
    {
        if (!init_net(eth1_name, &eth1_bootp_data))
        {
            eth1_up = false;
        }
    }
#endif

#ifdef CYGHWR_NET_DRIVER_ETH0
    if ( old_eth0_dhcpstate == DHCPSTATE_BOUND &&
            eth0_dhcpstate == DHCPSTATE_NOTBOUND )
        return 0; // a lease timed out; we became unbound
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1

    if ( old_eth1_dhcpstate == DHCPSTATE_BOUND &&
            eth1_dhcpstate == DHCPSTATE_NOTBOUND )
        return 0; // a lease timed out; we became unbound
#endif

    if (eth0_up)
	{
        init_static_network_interfaces(0);

        return 1; // all is well
    }
    else
        return 0;
}


// Shutdown any interface whose state is DHCPSTATE_NOTBOUND.
int dhcp_halt( void )
{
#ifdef CYGHWR_NET_DRIVER_ETH0
    if ( eth0_up
            && eth0_dhcpstate != DHCPSTATE_FAILED )
    {
        do_dhcp_down_net(eth0_name, &eth0_bootp_data, &eth0_dhcpstate, &eth0_lease);
        eth0_bootp_data.bp_ciaddr.s_addr = 32;
    }
    eth0_up = false;
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1

    if ( eth1_up
            && eth1_dhcpstate != DHCPSTATE_FAILED )
    {
        do_dhcp_down_net(eth1_name, &eth1_bootp_data, &eth1_dhcpstate, &eth1_lease);
    }
    eth1_up = false;
#endif

    return 1;
}


// Release (and set state to DHCPSTATE_NOTBOUND) all interfaces - we are
// closing down.  (unlikely but maybe useful for testing)
int dhcp_release( void )
{
#ifdef CYGHWR_NET_DRIVER_ETH0
    if (eth0_up)
        do_dhcp_release(eth0_name, &eth0_bootp_data, &eth0_dhcpstate, &eth0_lease);
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1

    if (eth1_up)
        do_dhcp_release(eth1_name, &eth1_bootp_data, &eth1_dhcpstate, &eth1_lease);
#endif

    return 1;
}

unsigned int  dhcp_del_old_ip_and_reset_interface(
	const char *if_name,
     int s,
     struct ifreq *ifrp
 /* socket for ioctl operations */
)
{

    strcpy(ifrp->ifr_name, if_name);
    memset(&(ifrp->ifr_addr),0, sizeof(ifrp->ifr_addr));

    if (ioctl(s, SIOCGIFADDR, ifrp))
    {
        if(errno != EADDRNOTAVAIL){
            eth0_up = false;
            return false;
        }
    }
    else{ /* found IP so delete */
        if (ioctl(s, SIOCDIFADDR, ifrp))
        {
            perror("SIOCDIFADDR1");
        }
    }
    // Shut down interface so it can be reinitialized
    ifrp->ifr_flags &= ~(IFF_UP | IFF_RUNNING);
    if (ioctl(s, SIOCSIFFLAGS, ifrp))
    {
        eth0_up = false;
        return false;
    }

    ifrp->ifr_flags = IFF_UP | IFF_BROADCAST | IFF_RUNNING;
    if (ioctl(s, SIOCSIFFLAGS, ifrp))
    { /* set ifnet flags */
        perror("SIOCSIFFLAGS up");
        return false;
    }
    return true;
}

// ------------------------------------------------------------------------
// The management thread function
void dhcp_mgt_entry( cyg_addrword_t loop_on_failure )
{
    struct ifreq ifr;
	int s;
    init_static_network_interfaces(0);
#ifdef DHCPTIMEOUTRETRY

    while (1)
    {
        if (dhcp_bind())  //Do a rebinding due to configuration in ECOS , todo
            break;

        else
        {
#if 0
            static int dhcpretrycnt = 0;
            dhcp_halt(); // tear everything down
            init_all_network_interfaces(); //try again
            if ( dhcpretrycnt++ > DHCPMAXRETRIES )
                break;
#else

            dhcp_halt(); // tear everything down
            init_all_network_interfaces(); //try again
#endif

        }

        //cyg_thread_delay(10000*60);//retry binding every minute
    }
#else
#ifdef AP5X
    cyg_semaphore_post( &dhcp_needs_attention );
#endif
    cyg_semaphore_wait( &dhcp_needs_attention );
    if (mib_dhcpenable)
    {
        if (!dhcp_bind())
        {
            dhcp_halt(); // tear everything down
            init_static_network_interfaces(0);
        }
    }
    init_static_network_interfaces(2);
    if((SUBNETMASK & 0xff000000) != 0xff000000)
    {
        if( (IPADDR & 0xC0000000) == 0xC0000000)
            SUBNETMASK = 0xffffff00;
        else if( (IPADDR & 0xC0000000) == 0x80000000)
            SUBNETMASK = 0xffff0000;
        else
            SUBNETMASK = 0xff000000;
        init_static_network_interfaces(0);
    }
#endif

    while ( 1 )
    {
        while ( 1 )
        {
#if defined (AP_URPTR) || defined(WB_EXCEPTION)
			{
				extern unsigned char ethMac[6];
    			wetUpdateHashEntry(ewbGetEthIp(), ethMac);
			}
#endif            
            cyg_semaphore_wait( &dhcp_needs_attention );
#if 1
			eth0_up = true;
            s = socket(AF_INET, SOCK_DGRAM, 0);

            if(s >= 0){
                 dhcp_del_old_ip_and_reset_interface( eth0_name,
                                                     s,
                                                     &ifr);

                close(s);
            }
            memset(&eth0_bootp_data,0 , sizeof(eth0_bootp_data));
            eth0_dhcpstate = 0;
#endif
            if (mib_dhcpenable)
            {

                dhcp_halt(); // tear everything down
                init_static_network_interfaces(0);
                if ( ! dhcp_bind() ) // a lease expired
                {
                    dhcp_halt(); // tear everything down
                    init_static_network_interfaces(0);
                }
            }
            else
            {
                ipaddr = 0; /* clear ipaddr otherwise init_static_network_interfaces will use old ipaddr */
                init_static_network_interfaces(1);
            }
            init_static_network_interfaces(2);
            if((SUBNETMASK & 0xff000000) != 0xff000000)
            {
                if( (IPADDR & 0xC0000000) == 0xC0000000)
                    SUBNETMASK = 0xffffff00;
                else if( (IPADDR & 0xC0000000) == 0x80000000)
                    SUBNETMASK = 0xffff0000;
                else
                    SUBNETMASK = 0xff000000;
                init_static_network_interfaces(0);
            }
            break; // If we need to re-bind
        }
#if 0
        dhcp_halt(); // tear everything down
        if ( !loop_on_failure )
            return ; // exit the thread/return
        init_static_network_interfaces(1); // re-initialize
        for ( j = 0; j < CYGPKG_NET_NLOOP; j++ )
            init_loopback_interface( j );
#endif
#ifdef CYGPKG_SNMPAGENT

        SnmpdShutDown(0) // Cycle the snmpd state
#endif

    }
}

#ifdef CYGOPT_NET_DHCP_DHCP_THREAD 
// Then we provide such a thread...
cyg_handle_t dhcp_mgt_thread_h = 0;
cyg_thread dhcp_mgt_thread;

/* Need this because this is a port of a later tcpip package */
#define CYGOPT_NET_DHCP_DHCP_THREAD_PARAM 1
#define CYGOPT_NET_DHCP_DHCP_THREAD_PARAM_1
#define CYGPKG_NET_DHCP_THREAD_PRIORITY 8
#define CYGPKG_NET_DHCP_THREAD_PRIORITY_8

#define STACK_SIZE (CYGNUM_HAL_STACK_SIZE_TYPICAL + sizeof(struct bootp))
static cyg_uint8 dhcp_mgt_stack[ STACK_SIZE ];

void dhcp_start_dhcp_mgt_thread( void )
{
    if ( ! dhcp_mgt_thread_h )
    {
        cyg_semaphore_init( &dhcp_needs_attention, 0 );
        cyg_thread_create(
            CYGPKG_NET_DHCP_THREAD_PRIORITY,  /* scheduling info (eg pri) */
            dhcp_mgt_entry,              /* entry point function */
            CYGOPT_NET_DHCP_DHCP_THREAD_PARAM,  /* entry data */
            "DHCP lease mgt",            /* optional thread name */
            dhcp_mgt_stack,              /* stack base, NULL = alloc */
            STACK_SIZE,                  /* stack size, 0 = default */
            &dhcp_mgt_thread_h,          /* returned thread handle */
            &dhcp_mgt_thread           /* put thread here */
        );

        cyg_thread_resume(dhcp_mgt_thread_h);
    }
}
static unsigned char *
add_tag(unsigned char *vp,
        unsigned char tag,
        void *val,
        int len)
{
    int i;
    unsigned char *xp = (unsigned char *)val;
    *vp++ = tag;
    *vp++ = len;
    for (i = 0; i < len; i++)
    {
        *vp++ = *xp++;
    }
    return vp;
}

unsigned long inet_address(unsigned long addr)
{
    return htonl(addr);
}
void init_static_network_interfaces(int flag)
{
    int i;
    in_addr_t addr;
    unsigned char *vp;
    unsigned char cookie[] = VM_RFC1048;
    struct bootp *bp;
    if (flag != 2)
    {
        if (IPADDR == 0xffffffff || IPADDR == 0)
        {
            init_all_network_interfaces(); //initialize network interface first
            return ;
        }
#ifdef CYGHWR_NET_DRIVER_ETH0
        eth0_up = true;
        bp = &eth0_bootp_data;
        bzero(bp, sizeof(struct bootp));
        bp->bp_op = BOOTREPLY;
        bp->bp_htype = HTYPE_ETHERNET;
        bp->bp_hlen = 6;
        for (i = 0; i < bp->bp_hlen; i++)
        {
            bp->bp_chaddr[i] = 0xFF;
        }
        if(ipaddr)
        {
            bp->bp_ciaddr.s_addr = inet_address(ipaddr);                      
            bp->bp_yiaddr.s_addr = inet_address(ipaddr);                      
            bp->bp_siaddr.s_addr = inet_address((ipaddr & 0xffffff00) + 0x64);
            bp->bp_giaddr.s_addr = inet_address((ipaddr & 0xffffff00) + 0x64);
        }else
        {
        bp->bp_ciaddr.s_addr = inet_address(IPADDR);
        bp->bp_yiaddr.s_addr = inet_address(IPADDR);
        bp->bp_siaddr.s_addr = inet_address((IPADDR & 0xffffff00) + 0x64);
        bp->bp_giaddr.s_addr = inet_address((IPADDR & 0xffffff00) + 0x64);
        }
        vp = &bp->bp_vend[0];
        bcopy(&cookie, vp, sizeof(cookie));
        vp += sizeof(cookie);
        addr = inet_address(SUBNETMASK);
        vp = add_tag(vp, TAG_SUBNET_MASK, &addr, sizeof(in_addr_t));
        addr = inet_address((IPADDR & SUBNETMASK) ^ (~SUBNETMASK));
        vp = add_tag(vp, TAG_IP_BROADCAST, &addr, sizeof(in_addr_t));
        *vp = TAG_END;
#endif
#if 0//def CYGHWR_NET_DRIVER_ETH1

        eth1_up = true;
        bp = &eth1_bootp_data;
        bzero(bp, sizeof(struct bootp));
        bp->bp_op = BOOTREPLY;
        bp->bp_htype = HTYPE_ETHERNET;
        bp->bp_hlen = 6;
        for (i = 0; i < bp->bp_hlen; i++)
        {
            bp->bp_chaddr[i] = 0xFF;
        }
        bp->bp_ciaddr.s_addr = inet_address(IPADDR);
        bp->bp_yiaddr.s_addr = inet_address(IPADDR);
        bp->bp_siaddr.s_addr = inet_address(GWYADDR & 0xffffff64);
        bp->bp_giaddr.s_addr = inet_address(GWYADDR & 0xffffff64);
        vp = &bp->bp_vend[0];
        bcopy(&cookie, vp, sizeof(cookie));
        vp += sizeof(cookie);
        addr = inet_address(SUBNETMASK);
        vp = add_tag(vp, TAG_SUBNET_MASK, &addr, sizeof(in_addr_t));
        addr = inet_address((IPADDR & SUBNETMASK) ^ (~SUBNETMASK));
        vp = add_tag(vp, TAG_IP_BROADCAST, &addr, sizeof(in_addr_t));
        *vp = TAG_END;
#endif
#ifdef CYGHWR_NET_DRIVER_ETH0
#ifndef CYGHWR_NET_DRIVER_ETH0_MANUAL

        if (eth0_up)
        {
            if (!init_net(eth0_name, &eth0_bootp_data))
            {
                //diag_printf("Network initialization failed for eth0\n");
                eth0_up = false;
            }
        }
#endif
#endif
#if 0//def CYGHWR_NET_DRIVER_ETH1
#ifndef CYGHWR_NET_DRIVER_ETH1_MANUAL
        if (eth1_up)
        {
            if (!init_net(eth1_name, &eth1_bootp_data))
            {
                //diag_printf("Network initialization failed for eth1\n");
                eth1_up = false;
            }
        }
#endif
#endif

    }
    if (flag)
    {
        update_IP_Info();
        cyg_semaphore_post(&ip_ready);
        cyg_semaphore_post(&ip_ready);
    }
}

void update_IP_Info(void)
{
    struct bootp *bp;
    unsigned char *op, *ap = 0;
    struct in_addr addr[32];
    unsigned char name[128];
    int i, len;
    unsigned long tmpaddr;

    bp = &eth0_bootp_data;
    IPADDR = inet_address(eth0_bootp_data.bp_yiaddr.s_addr);
    if (bp->bp_vend[0])
    {
        op = &bp->bp_vend[4];
        while (*op != TAG_END)
        {
            switch (*op)
            {
            case TAG_SUBNET_MASK:
                ap = (unsigned char *) & tmpaddr;
                len = *(op + 1);
                for (i = 0; i < len; i++)
                {
                    *ap++ = *(op + i + 2);
                }
                while (len > 0)
                {
                    len -= sizeof(struct in_addr);
                    ap += sizeof(struct in_addr);
                }
                SUBNETMASK = inet_address(tmpaddr);
                break;
            case TAG_GATEWAY:
                ap = (unsigned char *) & tmpaddr;
                len = *(op + 1);
                for (i = 0; i < len; i++)
                {
                    *ap++ = *(op + i + 2);
                }
                while (len > 0)
                {
                    len -= sizeof(struct in_addr);
                    ap += sizeof(struct in_addr);
                }
                GWYADDR = inet_address(tmpaddr);
                break;
            case TAG_IP_BROADCAST:
            case TAG_DOMAIN_SERVER:
                ap = (unsigned char *) & addr[0];
                len = *(op + 1);
                for (i = 0; i < len; i++)
                {
                    *ap++ = *(op + i + 2);
                }
                if (*op == TAG_SUBNET_MASK)
                    ap = "  subnet mask";
                if (*op == TAG_GATEWAY)
                    ap = "      gateway";
                if (*op == TAG_IP_BROADCAST)
                    ap = " IP broadcast";
                if (*op == TAG_DOMAIN_SERVER)
                    ap = "domain server";
                ap = (unsigned char *) & addr[0];
                while (len > 0)
                {
                    len -= sizeof(struct in_addr);
                    ap += sizeof(struct in_addr);
                }
                break;
            case TAG_DOMAIN_NAME:
            case TAG_HOST_NAME:
                for (i = 0; i < *(op + 1); i++)
                {
                    name[i] = *(op + i + 2);
                }
                name[*(op + 1)] = '\0';
                if (*op == TAG_DOMAIN_NAME)
                    ap = " domain name";
                if (*op == TAG_HOST_NAME)
                    ap = "   host name";
                break;
            default:
                break;
            }
            op += *(op + 1) + 2;
        }
    }
}
void initiate_dhcp(int force)
{
    static int dhcp_init = 0;
    if ( !dhcp_init || force)
    {
        dhcp_init++;
        cyg_semaphore_post( &dhcp_needs_attention);
    }
}

void wait_ipReady(void)
{
    cyg_semaphore_wait (&ip_ready);
    cyg_semaphore_post(&ip_ready);
}
#endif // CYGOPT_NET_DHCP_DHCP_THREAD

#endif // CYGPKG_NET_DHCP

// EOF dhcp_support.c
