/*==========================================================================
//
//      dhcp_support.c
//
//      Support code == friendly API for DHCP client
//
//==========================================================================
//####ECOSPDCOPYRIGHTBEGIN####
//
// Copyright (C) 2000, 2001, 2002 Red Hat, Inc.
// All Rights Reserved.
//
// Permission is granted to use, copy, modify and redistribute this
// file.
//
//####ECOSPDCOPYRIGHTEND####
//==========================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):   hmt
// Contributors: gthomas
// Date:        2000-07-01
// Purpose:     DHCP support
// Description: 
//
//####DESCRIPTIONEND####
//
//========================================================================*/

#include <pkgconf/system.h>
#include <pkgconf/net.h>

/* Define these locally because we are porting this code from a later package,
 * therefore these are not defined in pkgconf/net.h */

#define DHCP_MIB_VERSION  9


#define CYGPKG_NET_DHCP
#define CYGOPT_NET_DHCP_DHCP_THREAD

#ifdef CYGPKG_NET_DHCP

#include <network.h>
#include <dhcp.h>
#if defined(GATEWAY) && defined(DHCPS)
#if PROD_NAME == 0
#include "timer.h"
Timer LinkStatusTimer;
#endif
                /* Inform DHCP Server application */
extern void dhcpsSetDNS ( unsigned long prim_dns_addr, unsigned long sec_dns_addr);
#endif

//#define CYGDBG_NET_DHCP_CHATTER

#if defined(GATEWAY)
        /* Inform Fast Routing module */
extern void        FR_Main_WanIP_update();
#endif


typedef struct MIB_DHCP_s
{
    unsigned long IPAddr;
    unsigned long SubnetMask;
    unsigned long GwyAddr;
    unsigned long PrimaryDNS;
    unsigned long SecondaryDNS;
}
MIB_DHCP;

#if defined(GATEWAY)
typedef struct MIB_IP_LAN_s
{
    unsigned long IPAddr;
    unsigned long SubnetMask;
}
MIB_IP_LAN;

extern MIB_IP_LAN mib_IP_LAN;
extern in_addr_t         inet_addr (char  *cp);
#define _string(s) #s
#define string(s) _string(s)

#endif
#ifdef AP5X
MIB_DHCP mib_DHCP= {0x34020202, 0xFFFFFF00, 0, 0, 0};
#define MIB_DHCP_P  (&mib_DHCP)
#define SUBNETMASK  (mib_DHCP.SubnetMask)
#define IPADDR      (mib_DHCP.IPAddr)
#define GWYADDR     (mib_DHCP.GwyAddr)
#define PRIMARYDNS      (mib_DHCP.PrimaryDNS)
#define SECONDARYDNS     (mib_DHCP.SecondaryDNS)
#else
extern MIB_DHCP *mib_DHCP_p;
#define MIB_DHCP_P  (mib_DHCP_p)
#define SUBNETMASK  (mib_DHCP_p->SubnetMask)
#define IPADDR      (mib_DHCP_p->IPAddr)
#define GWYADDR     (mib_DHCP_p->GwyAddr)
#endif
extern unsigned char mib_dhcpenable;
extern int SetUserConfig(void);
extern void Reset(void);


// ---------------------------------------------------------------------------
#ifdef CYGHWR_NET_DRIVER_ETH0
cyg_uint8 eth0_dhcpstate = 0;
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
cyg_uint8 eth1_dhcpstate = 0;
#endif

cyg_sem_t dhcp_needs_attention;

#ifdef CYGHWR_NET_DRIVER_ETH0
struct dhcp_lease eth0_lease =
    {
        &dhcp_needs_attention, 0, 0, 0, 0, 0, 0
    };
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
struct dhcp_lease eth1_lease =
    {
        &dhcp_needs_attention, 0, 0, 0, 0, 0, 0
    };
#endif
#if PROD_NAME == 0
unsigned int LinkStatusCheckTimeOut(unsigned char *data)
{ 
	extern int gWanPort;
	#define LINK_NO_CHANGE 0
	#define LINK_TO_UP 1
	#define LINK_TO_DOWN 2
	#define TRUE 1
	#define FALSE 0
	static unsigned char wanPortStatusChange = LINK_NO_CHANGE;
	static unsigned char wanLinkUp = FALSE;
	if(wanPortStatusChange != LINK_NO_CHANGE)
	{
		if(wanPortStatusChange == LINK_TO_UP)
			initiate_dhcp(1);	
		wanPortStatusChange = LINK_NO_CHANGE;
	}
	else
	{
		if(isPortLinkUp(gWanPort))
		{
			if(wanLinkUp == FALSE)
			   wanPortStatusChange = LINK_TO_UP;	
			wanLinkUp = TRUE;
		}
		else
		{
			if(wanLinkUp == TRUE)
			   wanPortStatusChange = LINK_TO_DOWN;	
			wanLinkUp = FALSE;			
		}
	}
    return 0;
}
#endif
static unsigned char *
add_tag(unsigned char *vp,
        unsigned char tag,
        void *val,
        int len)
{
    int i;
    unsigned char *xp = (unsigned char *)val;
    *vp++ = tag;
    *vp++ = len;
    for (i = 0; i < len; i++)
    {
        *vp++ = *xp++;
    }
    return vp;
}
unsigned long inet_address(unsigned long addr)
{
    return htonl(addr);
}
static void
build_bootp_record(struct bootp *bp,
                   const char *addrs_ip,
                   const char *addrs_netmask,
                   const char *addrs_broadcast,
                   const char *addrs_gateway,
                   const char *addrs_server)
{
    int i;
    in_addr_t addr;
    unsigned char *vp;
    unsigned char cookie[] = VM_RFC1048;

    bzero(bp, sizeof(struct bootp));
    bp->bp_op = BOOTREPLY;
    bp->bp_htype = HTYPE_ETHERNET;
    bp->bp_hlen = 6;
    for (i = 0;  i < bp->bp_hlen;  i++) {
        bp->bp_chaddr[i] = 0xFF;
    }
    bp->bp_ciaddr.s_addr = inet_addr(addrs_ip);
    bp->bp_yiaddr.s_addr = inet_addr(addrs_ip);
    bp->bp_siaddr.s_addr = inet_addr(addrs_server);
    bp->bp_giaddr.s_addr = inet_addr(addrs_gateway);
    vp = &bp->bp_vend[0];
    bcopy(&cookie, vp, sizeof(cookie));
    vp += sizeof(cookie);
    addr = inet_addr(addrs_netmask);
    vp = add_tag(vp, TAG_SUBNET_MASK, &addr, sizeof(in_addr_t));
    addr = inet_addr(addrs_broadcast);
    vp = add_tag(vp, TAG_IP_BROADCAST, &addr, sizeof(in_addr_t));
    *vp = TAG_END;
}

/* used at init to BUILD FROM from mib or when moving from dhcp to static*/
void dhcp_build_bootp(struct bootp *bp,
					  cyg_uint32 my_addr,
                      cyg_uint32 my_mask,
                      cyg_uint32 gateway,
                      cyg_uint32 server){
    unsigned int i;
    unsigned char *vp;
    unsigned char cookie[] = VM_RFC1048;

    bzero(bp, sizeof(struct bootp));
    bp->bp_op = BOOTREPLY;
    bp->bp_htype = HTYPE_ETHERNET;
    bp->bp_hlen = 6;
    for (i = 0;  i < bp->bp_hlen;  i++) {
        bp->bp_chaddr[i] = 0xFF;
    }
    bp->bp_ciaddr.s_addr = htonl(my_addr);
    bp->bp_yiaddr.s_addr = htonl(my_addr);
    bp->bp_siaddr.s_addr = htonl(my_addr);
   	bp->bp_giaddr.s_addr = htonl(my_addr);
    vp = &bp->bp_vend[0];
    bcopy(&cookie, vp, sizeof(cookie));
    vp += sizeof(cookie);
    my_mask  = htonl(my_mask);
    vp = add_tag(vp, TAG_SUBNET_MASK, &my_mask, sizeof(in_addr_t));
    if(gateway){/* add default gateway tag */
    	gateway = htonl(gateway);
	    vp = add_tag(vp, TAG_GATEWAY, &gateway, sizeof(in_addr_t));
    }
    *vp = TAG_END;


}
#define INIT_WAN_ONLY 1
#define INIT_LAN_ONLY 2
#define INIT_ALL 0
void
init_my_network_interfaces(int flag)
{
    extern unsigned char mib_version;

#ifdef CYGHWR_NET_DRIVER_ETH0
#ifdef CYGHWR_NET_DRIVER_ETH0_BOOTP
// Perform a complete initialization, using BOOTP/DHCP
    eth0_up = true;
    if (do_bootp(eth0_name, &eth0_bootp_data)) {
#ifdef CYGHWR_NET_DRIVER_ETH0_BOOTP_SHOW
        show_bootp(eth0_name, &eth0_bootp_data);
#endif
    } else {
        //diag_printf("BOOTP failed on eth0\n");
        eth0_up = false;
    }
#elif defined(CYGHWR_NET_DRIVER_ETH0_ADDRS_IP)
    eth0_up = true;
	if(flag != INIT_LAN_ONLY)
	{
    if(mib_version != DHCP_MIB_VERSION )
        build_bootp_record(&eth0_bootp_data,
                       string(CYGHWR_NET_DRIVER_ETH0_ADDRS_IP),
                       string(CYGHWR_NET_DRIVER_ETH0_ADDRS_NETMASK),
                       string(CYGHWR_NET_DRIVER_ETH0_ADDRS_BROADCAST),
                       0/*string(CYGHWR_NET_DRIVER_ETH0_ADDRS_IP)*/,
                       0/*string(CYGHWR_NET_DRIVER_ETH0_ADDRS_IP)*/);
    else
        dhcp_build_bootp(&eth0_bootp_data,
                          IPADDR,
                          SUBNETMASK,
                          GWYADDR,
                          0);
    show_bootp(eth0_name, &eth0_bootp_data);
	}
#endif
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
#ifdef CYGHWR_NET_DRIVER_ETH1_BOOTP
// Perform a complete initialization, using BOOTP/DHCP
    eth1_up = true;
    if (do_bootp(eth1_name, &eth1_bootp_data)) {
#ifdef CYGHWR_NET_DRIVER_ETH1_BOOTP_SHOW
        show_bootp(eth1_name, &eth1_bootp_data);
#endif
    } else {
        //diag_printf("BOOTP failed on eth1\n");
        eth1_up = false;
    }
#elif defined(CYGHWR_NET_DRIVER_ETH1_ADDRS_IP)
    eth1_up = true;
	if(flag != INIT_WAN_ONLY)
	{
    if(mib_version != DHCP_MIB_VERSION )
    {
        build_bootp_record(&eth1_bootp_data,
                       string(CYGHWR_NET_DRIVER_ETH1_ADDRS_IP),
                       string(CYGHWR_NET_DRIVER_ETH1_ADDRS_NETMASK),
                       string(CYGHWR_NET_DRIVER_ETH1_ADDRS_BROADCAST),
                       0/*string(CYGHWR_NET_DRIVER_ETH1_ADDRS_GATEWAY)*/,
                       0/*string(CYGHWR_NET_DRIVER_ETH1_ADDRS_SERVER)*/);
        mib_IP_LAN.IPAddr   =  inet_addr(string(CYGHWR_NET_DRIVER_ETH1_ADDRS_IP));
        mib_IP_LAN.IPAddr   =  htonl(mib_IP_LAN.IPAddr);
        mib_IP_LAN.SubnetMask = inet_addr(string(CYGHWR_NET_DRIVER_ETH1_ADDRS_NETMASK));
        mib_IP_LAN.SubnetMask = htonl(mib_IP_LAN.SubnetMask);
    }
    else
	{
		if(mib_IP_LAN.IPAddr == 0)
		{
        	mib_IP_LAN.IPAddr   =  inet_addr(string(CYGHWR_NET_DRIVER_ETH1_ADDRS_IP));
        	mib_IP_LAN.IPAddr   =  htonl(mib_IP_LAN.IPAddr);
		}
		if(mib_IP_LAN.SubnetMask == 0)
		{
        	mib_IP_LAN.SubnetMask = inet_addr(string(CYGHWR_NET_DRIVER_ETH1_ADDRS_NETMASK));
        	mib_IP_LAN.SubnetMask = htonl(mib_IP_LAN.SubnetMask);
		}
        dhcp_build_bootp(&eth1_bootp_data,
                           mib_IP_LAN.IPAddr,
                           mib_IP_LAN.SubnetMask,
                          0,
                          0);
    }
    show_bootp(eth1_name, &eth1_bootp_data);
	}
#endif
#endif
#ifdef CYGHWR_NET_DRIVER_ETH0
#ifndef CYGHWR_NET_DRIVER_ETH0_MANUAL
    if (eth0_up) {
        if (!init_net(eth0_name, &eth0_bootp_data)) {
            //diag_printf("Network initialization failed for eth0\n");
            eth0_up = false;
        }
    }
#endif
#endif
#ifdef CYGHWR_NET_DRIVER_ETH1
#ifndef CYGHWR_NET_DRIVER_ETH1_MANUAL
    if (eth1_up) {
        if (!init_net(eth1_name, &eth1_bootp_data)) {
            //diag_printf("Network initialization failed for eth1\n");
            eth1_up = false;
        }
    }
#endif
#endif
}
#define DHCP_FIX 1
void update_IP_Info(void)
{
    struct bootp *bp;
    unsigned char *op, *ap = 0;
    struct in_addr addr[32];
    unsigned char name[128];
    int i, len;
    unsigned long tmpaddr;
#ifdef DHCP_FIX
    unsigned long dnsServer = 0, defaultGW =0 ;
#endif
#ifdef CYGDBG_NET_DHCP_CHATTER
     diag_printf("update_IP_Info\n");
#endif
    bp = &eth0_bootp_data;
    IPADDR = inet_address(eth0_bootp_data.bp_yiaddr.s_addr);
    GWYADDR = inet_address(eth0_bootp_data.bp_giaddr.s_addr);
#ifdef CYGDBG_NET_DHCP_CHATTER
     diag_printf("update_IP_Info: set ip %x\n", mib_DHCP_p->IPAddr);
#endif
    if (bp->bp_vend[0])
    {
        op = &bp->bp_vend[4];
        while (*op != TAG_END)
        {
            switch (*op)
            {
            case TAG_SUBNET_MASK:
                ap = (unsigned char *) & tmpaddr;
                len = *(op + 1);
                for (i = 0; i < len; i++)
                {
                    *ap++ = *(op + i + 2);
                }
                while (len > 0)
                {
                    len -= sizeof(struct in_addr);
                    ap += sizeof(struct in_addr);
                }
                SUBNETMASK = inet_address(tmpaddr);
                break;
            case TAG_GATEWAY:
#ifdef DHCP_FIX
                defaultGW = 1;
#endif
                ap = (unsigned char *) & tmpaddr;
                len = *(op + 1);
                for (i = 0; i < len; i++)
                {
                    *ap++ = *(op + i + 2);
                }
                while (len > 0)
                {
                    len -= sizeof(struct in_addr);
                    ap += sizeof(struct in_addr);
                }
                GWYADDR = inet_address(tmpaddr);
                break;
            case TAG_IP_BROADCAST:
                ap = (unsigned char *) & addr[0];
                len = *(op + 1);
                for (i = 0; i < len; i++)
                {
                    *ap++ = *(op + i + 2);
                }
                if (*op == TAG_SUBNET_MASK)
                    ap = "  subnet mask";
                if (*op == TAG_GATEWAY)
                    ap = "      gateway";
                if (*op == TAG_IP_BROADCAST)
                    ap = " IP broadcast";
                ap = (unsigned char *) & addr[0];
                while (len > 0)
                {
                    len -= sizeof(struct in_addr);
                    ap += sizeof(struct in_addr);
                }
                break;
            case TAG_DOMAIN_SERVER:
#ifdef DHCP_FIX
                dnsServer = 1;
#endif
                ap = (unsigned char *) & addr[0];
                len = *(op + 1);
                for (i = 0; i < len; i++)
                {
                    *ap++ = *(op + i + 2);
                }
                if (*op == TAG_DOMAIN_SERVER)
                    ap = "domain server";
                ap = (unsigned char *) & addr[0];
                while (len > 0)
                {
                    len -= sizeof(struct in_addr);
                    ap += sizeof(struct in_addr);
                }
                PRIMARYDNS = inet_address((unsigned long)addr[0].s_addr);
                SECONDARYDNS = inet_address((unsigned long)addr[1].s_addr);
#if defined(GATEWAY) && defined(DHCPS)
#ifndef DHCP_FIX
                /* Inform DHCP Server application */
                dhcpsSetDNS(PRIMARYDNS, SECONDARYDNS);
#endif
#endif
                break;
            case TAG_DOMAIN_NAME:
            case TAG_HOST_NAME:
                for (i = 0; i < *(op + 1); i++)
                {
                    name[i] = *(op + i + 2);
                }
                name[*(op + 1)] = '\0';
                if (*op == TAG_DOMAIN_NAME)
                    ap = " domain name";
                if (*op == TAG_HOST_NAME)
                    ap = "   host name";
                break;
            default:
                break;
            }
            op += *(op + 1) + 2;
        }
    }
#if defined(GATEWAY)
        /* Inform Fast Routing module */
		//if(mib_dhcpenable==0)
		   //GWYADDR =0;	
        FR_Main_WanIP_update();
#ifdef DHCPS
#ifdef DHCP_FIX
    if(defaultGW == 0)
        GWYADDR = 0;
    if(dnsServer == 0 && mib_dhcpenable == 1){
        PRIMARYDNS = 0;
        SECONDARYDNS = 0;
    }
                /* Inform DHCP Server application */
    dhcpsSetDNS(PRIMARYDNS, SECONDARYDNS);
#endif
#endif
#endif
#if !defined(GATEWAY)
    bp = &eth1_bootp_data;
    //mib_IP_LAN.IPAddr = inet_address(eth1_bootp_data.bp_yiaddr.s_addr);
    IPADDR = inet_address(eth1_bootp_data.bp_yiaddr.s_addr);
    if (bp->bp_vend[0])
    {
        op = &bp->bp_vend[4];
        while (*op != TAG_END)
        {
            switch (*op)
            {
            case TAG_SUBNET_MASK:
                ap = (unsigned char *) & tmpaddr;
                len = *(op + 1);
                for (i = 0; i < len; i++)
                {
                    *ap++ = *(op + i + 2);
                }
                while (len > 0)
                {
                    len -= sizeof(struct in_addr);
                    ap += sizeof(struct in_addr);
                }
                /*mib_IP_LAN.*/SUBNETMASK = inet_address(tmpaddr);
                break;
            default:
                break;
            }
            op += *(op + 1) + 2;
         }
    }
#endif

#if defined(DHCPS)
            /* notify DHCP Server about new IP */
    dhcpsSetIpAddress(mib_IP_LAN.IPAddr, mib_IP_LAN.SubnetMask);
#endif
}

typedef enum{
        DHCPC_flag_init_E = 0,       /* first init  */
        DHCPC_flag_set_from_mib = 1,  /* set WAN static data (gate way)  */
        DHCPC_ip_post_config    = 2,/* post config to IP layer */

}DHCPC_flag_ENT;

unsigned int  dhcp_del_old_ip_and_reset_interface(
	const char *if_name,
     int s,
     struct ifreq *ifrp
 /* socket for ioctl operations */
)
{

    strcpy(ifrp->ifr_name, if_name);
    memset(&(ifrp->ifr_addr),0, sizeof(ifrp->ifr_addr));

    if (ioctl(s, SIOCGIFADDR, ifrp))
    {
        if(errno != EADDRNOTAVAIL){
            eth0_up = false;
            return false;
        }
    }
    else{ /* found IP so delete */
        if (ioctl(s, SIOCDIFADDR, ifrp))
        {
            perror("SIOCDIFADDR1");
        }
    }
    // Shut down interface so it can be reinitialized
    ifrp->ifr_flags &= ~(IFF_UP | IFF_RUNNING);
    if (ioctl(s, SIOCSIFFLAGS, ifrp))
    {
        eth0_up = false;
        return false;
    }

    ifrp->ifr_flags = IFF_UP | IFF_BROADCAST | IFF_RUNNING;
    if (ioctl(s, SIOCSIFFLAGS, ifrp))
    { /* set ifnet flags */
        perror("SIOCSIFFLAGS up");
        return false;
    }
    return true;
}

void init_static_network_interfaces(int flag)
{
/* flag 0 init defaults*/

/* flag 2 set IP to current configuration */
/* flag 4 delete old configuration and prepare configuration for static*/
/* flag 5 delete old configuration */

#if defined(GATEWAY)
    int s;
    struct ifreq ifr;
//    struct sockaddr_in *addrpp;
//    struct ecos_rtentry route;
//    struct in_addr netmask, gateway;
#else
    int i;
    in_addr_t addr;
    unsigned char *vp;
    unsigned char cookie[] = VM_RFC1048;
    struct bootp *bp;
#endif
#ifdef CYGDBG_NET_DHCP_CHATTER
     diag_printf("init_static_network_interfaces: flag %d\n",flag);
#endif
#if defined(GATEWAY)
    s = socket(AF_INET, SOCK_DGRAM, 0);
    if (s < 0)
    {
        eth0_up = false;
        return;
    }
    if (flag == DHCPC_flag_init_E)
    {
        init_all_network_interfaces(); //initialize network interface first
        /* if WAN has configuration*/
#ifdef DHCP_FIX
        if (mib_dhcpenable == 1 ) {
            if (!dhcp_del_old_ip_and_reset_interface( eth0_name,
                                                      s,
                                                      &ifr)){
                close(s);
                return;
            }
        }
        else if ( (IPADDR != 0xffffffff) &&
                 (IPADDR != 0 )) {
#else
        if (mib_dhcpenable == 0 &&
            (IPADDR != 0xffffffff && IPADDR != 0 )) {
#endif
            if (!dhcp_del_old_ip_and_reset_interface( eth0_name,
            									      s,
                                                      &ifr)){
            	close(s);
                return;
            }

             dhcp_build_bootp(&eth0_bootp_data,
                               IPADDR,
                               SUBNETMASK,
                               GWYADDR,
                               0);
             /* delete old(default IP) */

#ifdef CYGDBG_NET_DHCP_CHATTER
            diag_printf("Network initialization for eth0 from mib %x\n",mib_DHCP_p->IPAddr);
#endif
            if (!init_net(eth0_name, &eth0_bootp_data)) {
                //diag_printf("Network initialization failed for eth0\n");
                eth0_up = false;
#ifdef CYGDBG_NET_DHCP_CHATTER
                diag_printf("Network initialization failed for eth0\n");
#endif
            }
        }

        /* update eth1 from MIB */
        if ( IPADDR != 0xffffffff && IPADDR != 0) {
             eth1_up = 1;
            if (!dhcp_del_old_ip_and_reset_interface( eth1_name,
                                                      s,
                                                      &ifr)){
                close(s);
                return;
            }
             dhcp_build_bootp(&eth1_bootp_data,
                               IPADDR,
                               SUBNETMASK,
                               0,
                               0);

#ifdef CYGDBG_NET_DHCP_CHATTER
            diag_printf("Network initialization for eth1 from mib %x\n",mib_IP_LAN.IPAddr);
#endif
            if (!init_net(eth1_name, &eth1_bootp_data)) {
                eth1_up = false;
#ifdef CYGDBG_NET_DHCP_CHATTER
                diag_printf("Network initialization failed for eth1\n");
#endif
            }
        }
#ifdef DHCP_FIX
//#endif
//#ifdef CYGHWR_NET_DRIVER_ETH1_ADDRS_IP
                /* default manual IP from
           ecos\packages\net\tcpip\v1_0b1\src\lib\network_support.c */
        else{
                /* update lan from default */
              IPADDR    =  inet_addr(string(CYGHWR_NET_DRIVER_ETH1_ADDRS_IP));
              IPADDR    =  htonl(IPADDR);
              SUBNETMASK = inet_addr(string(CYGHWR_NET_DRIVER_ETH1_ADDRS_NETMASK));
              SUBNETMASK = htonl(SUBNETMASK);
        }
#endif
#ifdef CYGDBG_NET_DHCP_CHATTER
            diag_printf("init_static_network_interfaces: Lan IP %x mask %x\n",
                         IPADDR,
                         SUBNETMASK);
#endif
        close(s);
        return;
    }
    else if (flag == DHCPC_flag_set_from_mib){
    	/* set wan info from new static*/

        // Clean old static configuration
        // get the specific address that was used
        if (!dhcp_del_old_ip_and_reset_interface( eth0_name,
                                                  s,
           										  &ifr)){
            close(s);
            return;
        }
        dhcp_build_bootp(&eth0_bootp_data,
                           IPADDR,
                           SUBNETMASK,
                           GWYADDR,
                           0);
        if (!init_net(eth0_name, &eth0_bootp_data)) {
            diag_printf("Network initialization failed for eth0\n");
            eth0_up = false;
        }

        // All done with socket
    }
    close(s);

#else /* previous code */
   if (flag != 2)
    {



        if (IPADDR == 0xffffffff || IPADDR == 0)

        {
            init_all_network_interfaces(); //initialize network interface first
            return ;
        }
#ifdef CYGHWR_NET_DRIVER_ETH0
        eth0_up = true;
        bp = &eth0_bootp_data;
        bzero(bp, sizeof(struct bootp));
        bp->bp_op = BOOTREPLY;
        bp->bp_htype = HTYPE_ETHERNET;
        bp->bp_hlen = 6;
        for (i = 0; i < bp->bp_hlen; i++)
        {
            bp->bp_chaddr[i] = 0xFF;
        }
        bp->bp_ciaddr.s_addr = inet_address(mib_DHCP_p->IPAddr);
        bp->bp_yiaddr.s_addr = inet_address(mib_DHCP_p->IPAddr);
        bp->bp_siaddr.s_addr = inet_address((mib_DHCP_p->IPAddr & 0xffffff00) + 0x64);
        bp->bp_giaddr.s_addr = inet_address((mib_DHCP_p->IPAddr & 0xffffff00) + 0x64);
        vp = &bp->bp_vend[0];
        bcopy(&cookie, vp, sizeof(cookie));
        vp += sizeof(cookie);
        addr = inet_address(mib_DHCP_p->SubnetMask);
        vp = add_tag(vp, TAG_SUBNET_MASK, &addr, sizeof(in_addr_t));
        addr = inet_address((mib_DHCP_p->IPAddr & mib_DHCP_p->SubnetMask) ^ (~mib_DHCP_p->SubnetMask));
        vp = add_tag(vp, TAG_IP_BROADCAST, &addr, sizeof(in_addr_t));
        *vp = TAG_END;
#endif
#if 0//def CYGHWR_NET_DRIVER_ETH1

        eth1_up = true;
        bp = &eth1_bootp_data;
        bzero(bp, sizeof(struct bootp));
        bp->bp_op = BOOTREPLY;
        bp->bp_htype = HTYPE_ETHERNET;
        bp->bp_hlen = 6;
        for (i = 0; i < bp->bp_hlen; i++)
        {
            bp->bp_chaddr[i] = 0xFF;
        }
        bp->bp_ciaddr.s_addr = inet_address(mib_DHCP_p->IPAddr);
        bp->bp_yiaddr.s_addr = inet_address(mib_DHCP_p->IPAddr);
        bp->bp_siaddr.s_addr = inet_address(mib_DHCP_p->GwyAddr & 0xffffff64);
        bp->bp_giaddr.s_addr = inet_address(mib_DHCP_p->GwyAddr & 0xffffff64);
        vp = &bp->bp_vend[0];
        bcopy(&cookie, vp, sizeof(cookie));
        vp += sizeof(cookie);
        addr = inet_address(mib_DHCP_p->SubnetMask);
        vp = add_tag(vp, TAG_SUBNET_MASK, &addr, sizeof(in_addr_t));
        addr = inet_address((mib_DHCP_p->IPAddr & mib_DHCP_p->SubnetMask) ^ (~mib_DHCP_p->SubnetMask));
        vp = add_tag(vp, TAG_IP_BROADCAST, &addr, sizeof(in_addr_t));
        *vp = TAG_END;
#endif
#ifdef CYGHWR_NET_DRIVER_ETH0
#ifndef CYGHWR_NET_DRIVER_ETH0_MANUAL

        if (eth0_up)
        {
            if (!init_net(eth0_name, &eth0_bootp_data))
            {
                //diag_printf("Network initialization failed for eth0\n");
                eth0_up = false;
            }
        }
#endif
#endif
#if 0//def CYGHWR_NET_DRIVER_ETH1
#ifndef CYGHWR_NET_DRIVER_ETH1_MANUAL
        if (eth1_up)
        {
            if (!init_net(eth1_name, &eth1_bootp_data))
            {
                //diag_printf("Network initialization failed for eth1\n");
                eth1_up = false;
            }
        }
#endif
#endif
    }
#endif

    if (flag != DHCPC_flag_init_E )
    {
        update_IP_Info();
        cyg_semaphore_post(&ip_ready);
    }
}
// ---------------------------------------------------------------------------
//
// The point of this module is to deal with all the horrid written out in
// full stuff of having two interfaces; it's ugly but it's also most
// flexible.  The dhcp_prot.c module should do all the work...
//
// ---------------------------------------------------------------------------

// return value: 1 => everything OK, no change.
// 0 => close your connections, then call do_dhcp_halt() to halt the
// interface(s) in question (it knows because the state will be NOTBOUND).
// After that you can return to the start and use
// init_all_network_interfaces(); as usual, or call do_dhcp_bind() by hand,
// or whatever...
int dhcp_bind( void )
{
#ifdef CYGHWR_NET_DRIVER_ETH0
    cyg_uint8 old_eth0_dhcpstate = eth0_dhcpstate;
#endif

#if !defined(GATEWAY)
#ifdef CYGHWR_NET_DRIVER_ETH1

    cyg_uint8 old_eth1_dhcpstate = eth1_dhcpstate;
#endif
#endif

#ifdef CYGDBG_NET_DHCP_CHATTER
     diag_printf("dhcp_bind\n");
#endif
    // If there are no interfaces at all, init it every time, doesn't
    // matter.  In case we are called from elsewhere...
    if ( 1
#ifdef CYGHWR_NET_DRIVER_ETH0
            && eth0_dhcpstate == 0
#endif
#if !defined(GATEWAY)
#ifdef CYGHWR_NET_DRIVER_ETH1
            && eth1_dhcpstate == 0
#endif
#endif
       )
        cyg_semaphore_init( &dhcp_needs_attention, 0 );

    // Run the state machine...
#ifdef CYGHWR_NET_DRIVER_ETH0

    if (eth0_up
            && DHCPSTATE_FAILED != eth0_dhcpstate )
        eth0_up = do_dhcp(eth0_name, &eth0_bootp_data, &eth0_dhcpstate, &eth0_lease);
#endif
#if !defined(GATEWAY)
#ifdef CYGHWR_NET_DRIVER_ETH1

    if (eth1_up
            && DHCPSTATE_FAILED != eth1_dhcpstate )
        eth1_up = do_dhcp(eth1_name, &eth1_bootp_data, &eth1_dhcpstate, &eth1_lease);
#endif
#endif

    // If the interface newly came up, initialize it:
    // (this duplicates the code in init_all_network_interfaces() really).
#ifdef CYGHWR_NET_DRIVER_ETH0
#if 0
    if ( eth0_up
            && eth0_dhcpstate == DHCPSTATE_BOUND
            && old_eth0_dhcpstate != eth0_dhcpstate )
    {
        if (!init_net(eth0_name, &eth0_bootp_data))
        {
            eth0_up = false;
        }
    }
#endif
#endif
#if !defined(GATEWAY)
#ifdef CYGHWR_NET_DRIVER_ETH1
    if ( eth1_up
            && eth1_dhcpstate == DHCPSTATE_BOUND
            && old_eth1_dhcpstate != eth1_dhcpstate )
    {
        if (!init_net(eth1_name, &eth1_bootp_data))
        {
            eth1_up = false;
        }
    }
#endif
#endif

#ifdef CYGHWR_NET_DRIVER_ETH0
    if ( old_eth0_dhcpstate == DHCPSTATE_BOUND &&
            eth0_dhcpstate == DHCPSTATE_NOTBOUND )
        return 0; // a lease timed out; we became unbound
#endif
#if !defined(GATEWAY)
#ifdef CYGHWR_NET_DRIVER_ETH1

    if ( old_eth1_dhcpstate == DHCPSTATE_BOUND &&
            eth1_dhcpstate == DHCPSTATE_NOTBOUND )
        return 0; // a lease timed out; we became unbound
#endif
#endif

    if (eth0_up)
        return 1; // all is well
    else
        return 0;
}


// Shutdown any interface whose state is DHCPSTATE_NOTBOUND.
int dhcp_halt( void )
{
#ifdef CYGDBG_NET_DHCP_CHATTER
     diag_printf("dhcp_halt\n");
#endif
#ifdef CYGHWR_NET_DRIVER_ETH0
    if ( eth0_up
            && eth0_dhcpstate != DHCPSTATE_FAILED )
    {
        do_dhcp_down_net(eth0_name, &eth0_bootp_data, &eth0_dhcpstate, &eth0_lease);
//        eth0_bootp_data.bp_ciaddr.s_addr = 32;
    }
    eth0_up = false;
#endif
#if !defined(GATEWAY)
#ifdef CYGHWR_NET_DRIVER_ETH1

    if ( eth1_up
            && eth1_dhcpstate != DHCPSTATE_FAILED )
    {
        do_dhcp_down_net(eth1_name, &eth1_bootp_data, &eth1_dhcpstate, &eth1_lease);
    }
    eth1_up = false;
#endif
#endif

    return 1;
}


// Release (and set state to DHCPSTATE_NOTBOUND) all interfaces - we are
// closing down.  (unlikely but maybe useful for testing)
int dhcp_release( void )
{
#ifdef CYGDBG_NET_DHCP_CHATTER
     diag_printf("dhcp_release\n");
#endif
#ifdef CYGHWR_NET_DRIVER_ETH0
    if (eth0_up)
        do_dhcp_release(eth0_name, &eth0_bootp_data, &eth0_dhcpstate, &eth0_lease);
#endif
#if !defined(GATEWAY)
#ifdef CYGHWR_NET_DRIVER_ETH1

    if (eth1_up)
        do_dhcp_release(eth1_name, &eth1_bootp_data, &eth1_dhcpstate, &eth1_lease);
#endif
#endif

    return 1;
}


// ------------------------------------------------------------------------
// The management thread function
void dhcp_mgt_entry( cyg_addrword_t loop_on_failure )
{
#if 1
    struct ifreq ifr;
    int    s;
	unsigned int dhcp_enable_old_val=-1;
#ifdef CYGHWR_NET_DRIVER_ETH0
    cyg_uint8 *cl_stste_PTR = &eth0_dhcpstate;
#endif
    if(mib_dhcpenable)
    {
		dhcp_enable_old_val = mib_dhcpenable;
        if ( !dhcp_bind() ) // a lease expired
        {
            dhcp_halt(); // tear everything down
            init_my_network_interfaces(INIT_WAN_ONLY);
			dhcp_enable_old_val  = !mib_dhcpenable;
        }
    }
    update_IP_Info();
    cyg_semaphore_post(&ip_ready);
#if PROD_NAME == 0
	TimerFireEvery(&LinkStatusTimer, 1, &LinkStatusCheckTimeOut, NULL, 50 );
#endif
    while(1)
    {
        while ( 1 )
        {
			if(dhcp_enable_old_val != mib_dhcpenable){
               	if(dhcp_enable_old_val==1)
               	{
                	dhcp_halt(); // tear everything down
               	}
				else
				{
					eth0_up = true;
#ifdef DHCP_FIX
                    s = socket(AF_INET, SOCK_DGRAM, 0);

                    if(s >= 0){
                         dhcp_del_old_ip_and_reset_interface( eth0_name,
                                                             s,
                                                             &ifr);

                        close(s);
                    }
                    memset(&eth0_bootp_data,0 , sizeof(eth0_bootp_data));
#endif
               }
               *cl_stste_PTR = DHCPSTATE_INIT;
			}
            if (mib_dhcpenable)
            {
				int retries = 3;
				while(retries--)
				{    
                	dhcp_halt(); // tear everything down
                	init_my_network_interfaces(INIT_WAN_ONLY);
                	if ( !dhcp_bind() ) // a lease expired
                	{
                	    dhcp_halt(); // tear everything down
                	    init_my_network_interfaces(INIT_WAN_ONLY);
						dhcp_enable_old_val  = !mib_dhcpenable;
                	}
					else
					{
						dhcp_enable_old_val  = mib_dhcpenable;
						break;
				}
            }
            }
            else
			{
                init_my_network_interfaces(INIT_WAN_ONLY);
				dhcp_enable_old_val  = !mib_dhcpenable;
            }
            update_IP_Info();
            cyg_semaphore_post(&ip_ready);
#ifdef DHCPS
			DHCPSReinit();
#endif
            break; // If we need to re-bind
        }
#ifdef CYGPKG_SNMPAGENT
        SnmpdShutDown(0) // Cycle the snmpd state
#endif
        cyg_semaphore_wait( &dhcp_needs_attention );
    }
#endif
#if 0
	unsigned int dhcp_enable_old_val, retry_flag = 0;
        struct ifreq ifr;
        int    s;
#ifdef CYGHWR_NET_DRIVER_ETH0
    cyg_uint8 *cl_stste_PTR = &eth0_dhcpstate;
#endif
#if !defined(GATEWAY)
#ifdef CYGHWR_NET_DRIVER_ETH1
    cyg_uint8 *cl_stste_PTR = &eth1_dhcpstate;
#endif
#endif
#ifdef CYGDBG_NET_DHCP_CHATTER
     diag_printf("dhcp_mgt_entry\n");
#endif
    init_static_network_interfaces(DHCPC_flag_init_E);

    cyg_semaphore_wait( &dhcp_needs_attention );
    if (mib_dhcpenable)
    {
        /* clear junk mib and update */
        memset(MIB_DHCP_P,0,sizeof(MIB_DHCP));
#ifndef AP31E
        SetUserConfig();
#endif
        /* notify users on change */
        init_static_network_interfaces(DHCPC_ip_post_config);

        if ( !dhcp_bind() )
        {
            retry_flag = 1;
        }
        else
        {
            init_static_network_interfaces(DHCPC_ip_post_config);
        }

    }
    else
    {
        init_static_network_interfaces(DHCPC_ip_post_config);
    }
    while ( 1 )
    {
        while ( 1 )
        {
            dhcp_enable_old_val  = mib_dhcpenable; /* avoid fail for ever
                                                    when re enabling DHCP */
            if(retry_flag)
                cyg_thread_delay( 100 );
            else
            cyg_semaphore_wait( &dhcp_needs_attention );
            retry_flag = 0;
#ifndef AP31E
            SetUserConfig();
#endif
			if(dhcp_enable_old_val != mib_dhcpenable){
               if(dhcp_enable_old_val){
               	   dhcp_halt();
               }
               else{
                   eth0_up = true; /* reset current status */
#ifdef DHCP_FIX
                    s = socket(AF_INET, SOCK_DGRAM, 0);

                    if(s >= 0){
                         dhcp_del_old_ip_and_reset_interface( eth0_name,
                                                             s,
                                                             &ifr);

                        close(s);
                    }
                    memset(&eth0_bootp_data,0 , sizeof(eth0_bootp_data));

                    /* clear configuration */
                    memset(MIB_DHCP_P,0,sizeof(MIB_DHCP));
#ifndef AP31E
            		SetUserConfig();
#endif
                    /* notify users on change */
                    init_static_network_interfaces(DHCPC_ip_post_config);
#endif
               }
               *cl_stste_PTR = DHCPSTATE_INIT;
            }

            if (mib_dhcpenable)
            {

                if(*cl_stste_PTR != DHCPSTATE_BOUND){
                	 *cl_stste_PTR = DHCPSTATE_INIT;
				}
#ifdef CYGDBG_NET_DHCP_CHATTER
                diag_printf("dhcp_mgt_entry :dhcp state = %d\n", eth0_dhcpstate);
                diag_printf("dhcp_mgt_entry: dhcp bind 3\n");
#endif
                if ( ! dhcp_bind() ) // a lease expired
                {
                    dhcp_halt();
	                retry_flag = 1;
                    eth0_up = true;
                    continue; /* cause retry */
                }
#ifdef CYGDBG_NET_DHCP_CHATTER
                diag_printf("dhcp_mgt_entry :new state = %d", eth0_dhcpstate);
#endif
            }
            else
                init_static_network_interfaces(DHCPC_flag_set_from_mib);

            init_static_network_interfaces(DHCPC_ip_post_config);
/*            if((mib_DHCP_p->SubnetMask & 0xff000000) != 0xff000000)
            {
                if( (mib_DHCP_p->IPAddr & 0xC0000000) == 0xC0000000)
                    mib_DHCP_p->SubnetMask = 0xffffff00;
                else if( (mib_DHCP_p->IPAddr & 0xC0000000) == 0x80000000)
                    mib_DHCP_p->SubnetMask = 0xffff0000;
                else
                    mib_DHCP_p->SubnetMask = 0xff000000;
                init_static_network_interfaces(0);
            }
*/
            break; // If we need to re-bind
        }
#if 0
        dhcp_halt(); // tear everything down
        if ( !loop_on_failure )
            return ; // exit the thread/return
        init_static_network_interfaces(1); // re-initialize
        for ( j = 0; j < CYGPKG_NET_NLOOP; j++ )
            init_loopback_interface( j );
#endif
#ifdef CYGPKG_SNMPAGENT

        SnmpdShutDown(0) // Cycle the snmpd state
#endif

    }
#endif
}

#ifdef CYGOPT_NET_DHCP_DHCP_THREAD 
// Then we provide such a thread...
cyg_handle_t dhcp_mgt_thread_h = 0;
cyg_thread dhcp_mgt_thread;

/* Need this because this is a port of a later tcpip package */
#define CYGOPT_NET_DHCP_DHCP_THREAD_PARAM 1
#define CYGOPT_NET_DHCP_DHCP_THREAD_PARAM_1
#define CYGPKG_NET_DHCP_THREAD_PRIORITY 8
#define CYGPKG_NET_DHCP_THREAD_PRIORITY_8

#define STACK_SIZE (CYGNUM_HAL_STACK_SIZE_TYPICAL + sizeof(struct bootp))
static cyg_uint8 dhcp_mgt_stack[ STACK_SIZE ];

void dhcp_start_dhcp_mgt_thread( void )
{
    cyg_semaphore_init (&ip_ready, 0);
    init_my_network_interfaces(INIT_ALL);
    update_IP_Info();
    if ( ! dhcp_mgt_thread_h )
    {
        cyg_semaphore_init( &dhcp_needs_attention, 0 );
        cyg_thread_create(
            CYGPKG_NET_DHCP_THREAD_PRIORITY,  /* scheduling info (eg pri) */
            dhcp_mgt_entry,              /* entry point function */
            CYGOPT_NET_DHCP_DHCP_THREAD_PARAM,  /* entry data */
            "DHCP lease mgt",            /* optional thread name */
            dhcp_mgt_stack,              /* stack base, NULL = alloc */
            STACK_SIZE,                  /* stack size, 0 = default */
            &dhcp_mgt_thread_h,          /* returned thread handle */
            &dhcp_mgt_thread           /* put thread here */
        );

        cyg_thread_resume(dhcp_mgt_thread_h);
    }
}


void initiate_dhcp(int force)
{
    static int dhcp_init = 0;
    if ( !dhcp_init || force)
    {
        dhcp_init++;
        cyg_semaphore_post( &dhcp_needs_attention);
    }
}

void wait_ipReady(void)
{
    cyg_semaphore_wait (&ip_ready);
    cyg_semaphore_post(&ip_ready);
}

#endif // CYGOPT_NET_DHCP_DHCP_THREAD

#endif // CYGPKG_NET_DHCP

// EOF dhcp_support.c
