/*******************************************************************************
*                   Copyright 2002, GALILEO TECHNOLOGY, LTD.                   *
* THIS CODE CONTAINS CONFIDENTIAL INFORMATION OF MARVELL.                      *
* NO RIGHTS ARE GRANTED HEREIN UNDER ANY PATENT, MASK WORK RIGHT OR COPYRIGHT  *
* OF MARVELL OR ANY THIRD PARTY. MARVELL RESERVES THE RIGHT AT ITS SOLE        *
* DISCRETION TO REQUEST THAT THIS CODE BE IMMEDIATELY RETURNED TO MARVELL.     *
* THIS CODE IS PROVIDED "AS IS". MARVELL MAKES NO WARRANTIES, EXPRESSED,       *
* IMPLIED OR OTHERWISE, REGARDING ITS ACCURACY, COMPLETENESS OR PERFORMANCE.   *
*                                                                              *
* MARVELL COMPRISES MARVELL TECHNOLOGY GROUP LTD. (MTGL) AND ITS SUBSIDIARIES, *
* MARVELL INTERNATIONAL LTD. (MIL), MARVELL TECHNOLOGY, INC. (MTI), MARVELL    *
* SEMICONDUCTOR, INC. (MSI), MARVELL ASIA PTE LTD. (MAPL), MARVELL JAPAN K.K.  *
* (MJKK), GALILEO TECHNOLOGY LTD. (GTL) AND GALILEO TECHNOLOGY, INC. (GTI).    *
********************************************************************************
* gtMpsc.h  
*
* DESCRIPTION:
*       Header file for MPCS functions.
*       The header defines are gruoped in x different groups:
*       1. General for all protocols (with defines for fields in mpsc registers
*          - use reg's name to find it's defines)
*       2. Specific defines for HDLC, BISYNC, UART and Transparent (Mainly 
*          defines for fields in MPCR and CHRs).           
*       3. BRG defines.
*
* DEPENDENCIES:
*
*******************************************************************************/

#ifndef _INCgtmpsch 
#define _INCgtmpsch

/* includes */    
#include "MvCore.h"

/* typedefs */
typedef enum _mpscPort {MPSC0
                       ,MPSC1
#ifdef INCLUDE_MPSC_PORTS_2_7
                       ,MPSC2
                       ,MPSC3
                       ,MPSC4
                       ,MPSC5
                       ,MPSC6
                       ,MPSC7
#endif /* INCLUDE_MPSC_PORTS_2_7 */
                       } MPSC_PORT;
typedef enum _mpscProtocol {HDLC, BISYNC, H_BISYNC, UART, TRANSPARENT
                           } MPSC_PROTOCOL;
typedef enum _mpscClockSource {BRG0_SRC = 0
                               ,BRG1_SRC = 1
#ifdef INCLUDE_MPSC_PORTS_2_7
                               ,BRG2_SRC = 2
                               ,BRG3_SRC = 3
                               ,BRG4_SRC = 4
                               ,BRG5_SRC = 5
                               ,BRG6_SRC = 6
                               ,BRG7_SRC = 7
#endif /* INCLUDE_MPSC_PORTS_2_7 */
                               ,SCLK_SRC = 8
                               ,TSCLK_SRC = 9
                               ,NO_SRC = 0xf
                              } MPSC_CLOCK_SOURCE;
typedef enum _mpscBRGNumber {BRG0
                             ,BRG1
#ifdef INCLUDE_MPSC_PORTS_2_7
                             ,BRG2
                             ,BRG3
                             ,BRG4
                             ,BRG5
                             ,BRG6
                             ,BRG7
#endif /*INCLUDE_MPSC_PORTS_2_7*/
                            } MPSC_BRG_NUM;

typedef enum _mpscbrgClockSource {BCLK_IN = 0, SCLK0 = 2, TSCLK0 = 3,
                                  SCLK1 = 6, TSCLK1 = 7, GT_TCLK = 8
                                 } MPSC_BRG_CLOCK_SOURCE;
typedef enum _mpscBisyncMode {BISYNC_NORMAL, BISYNC_TRANSPARENT, 
                              BISYNC_AUTO_TRANSPARENT
                             } MPSC_BISYNC_MODE;
typedef enum _mpscBisyncParityMode {PARITY_NONE, PARITY_EVEN, PARITY_ODD,
                                    PARITY_HIGH, PARITY_LOW
                                   } MPSC_BISYNC_PARITY_MODE;

/* struct returned by gtMpscConfigFor*protocol* functions */
typedef struct _mpscActualRates
{   
    unsigned int    rxClkActualRate; 
    unsigned int    txClkActualRate; 
} MPSC_ACTUAL_RATES;
                                            
/* defines */

/*******************************************************************************
*
*			GENERAL FOR ALL PROTOCOLS
*
*******************************************************************************/

/* Gap between mpsc registers */
/* Consts in use of all protocol */
#define	 MPSC_DEFAULT_NULL_VALUE		            0x0
#define  MPSC_DEFAULT_ALL_1				            0xffff

/* Connect port to serial controller - must do before using port */
#define GT_MPSC_MRR_CONFIG(port) GT_RESET_REG_BITS(MPSC_MAIN_ROUTING_REG,      \
                                              (BIT2 | BIT1 | BIT0) << (port*6));
                       
/* Define source for rx and tx clocks for a given port (use MPSC_CLOCK_SOURCE
   to define sources) */
#define GT_MPSC_CLOCK_ROUTING_CONFIG(port, rxSrc, txSrc)                       \
            GT_REG_WRITE(MPSC_RECEIVE_CLOCK_ROUTING_REG,                       \
                 GTREGREAD(MPSC_RECEIVE_CLOCK_ROUTING_REG) | (rxSrc << port*8));                                   \
            GT_REG_WRITE(MPSC_TRANSMIT_CLOCK_ROUTING_REG,                      \
                 GTREGREAD(MPSC_TRANSMIT_CLOCK_ROUTING_REG) | (txSrc << port*8))  
                                                         
/* Abort MPSC RX immediately */
#define GT_MPSC_ABORT_RX(port)                                                 \
                      GT_REG_WRITE(MPSC_CHANNEL_REG_2(port), MPSC_HDLC_ABORT_RX)  
            /* good for all protocols!!! */

/* Abort MPSC TX immediately */
#define GT_MPSC_ABORT_TX(port)                                                 \
                      GT_REG_WRITE(MPSC_CHANNEL_REG_2(port), MPSC_HDLC_ABORT_TX)  
            /* good for all protocols!!! */

/* Set MPCR for a given port (use MPCR defines, according to protocol) */                       
#define GT_MPSC_MPCR_CONFIG(port, MpcrConfig)						           \
			                GT_REG_WRITE(MPSC_PROTOCOL_CONFIG(port), MpcrConfig)
		   
/* Set CHR1 for a given port (use CHR1 defines, according to protocol) */                       
#define GT_MPSC_CHR1_CONFIG(port, ChrConfig)						           \
			                   GT_REG_WRITE(MPSC_CHANNEL_REG_1(port), ChrConfig)

/* Set CHR2 for a given port (use CHR2 defines, according to protocol) */                       
#define GT_MPSC_CHR2_CONFIG(port, ChrConfig)						           \
                               GT_REG_WRITE(MPSC_CHANNEL_REG_2(port), ChrConfig)

/* set CHR3 for a given port (use CHR3 defines, according to protocol) */                       
#define GT_MPSC_CHR3_CONFIG(port, ChrConfig)						           \
                               GT_REG_WRITE(MPSC_CHANNEL_REG_3(port), ChrConfig)

/* set CHR4 for a given port (use CHR4 defines, according to protocol) */                       
#define GT_MPSC_CHR4_CONFIG(port, ChrConfig)						           \
                               GT_REG_WRITE(MPSC_CHANNEL_REG_4(port), ChrConfig)

/* set CHR5 for a given port (use CHR5 defines, according to protocol) */                       
#define GT_MPSC_CHR5_CONFIG(port, ChrConfig)								   \
                               GT_REG_WRITE(MPSC_CHANNEL_REG_5(port), ChrConfig)

/* set CHR6 for a given port (use CHR6 defines, according to protocol) */                       
#define GT_MPSC_CHR6_CONFIG(port, ChrConfig)								   \
                               GT_REG_WRITE(MPSC_CHANNEL_REG_6(port), ChrConfig)

/* set CHR7 for a given port (use CHR7 defines, according to protocol) */                       
#define GT_MPSC_CHR7_CONFIG(port, ChrConfig)								   \
                               GT_REG_WRITE(MPSC_CHANNEL_REG_7(port), ChrConfig)

/* set CHR8 for a given port (use CHR8 defines, according to protocol) */                       
#define GT_MPSC_CHR8_CONFIG(port, ChrConfig)						    	   \
                               GT_REG_WRITE(MPSC_CHANNEL_REG_8(port), ChrConfig)

/* set CHR9 for a given port (use CHR9 defines, according to protocol) */                       
#define GT_MPSC_CHR9_CONFIG(port, ChrConfig)								   \
                               GT_REG_WRITE(MPSC_CHANNEL_REG_9(port), ChrConfig)
            
/* main configuration register low - MMCRLx */
/*  set MMCRL for a given port (use MMCRL defines) */                       
#define GT_MPSC_MMCRL_CONFIG(port, MmcrlConfig)						           \
		                   GT_REG_WRITE(MPSC_MAIN_CONFIG_LOW(port), MmcrlConfig)
                           
#define MPSC_MODE_HDLC                                  0
#define MPSC_MODE_UART                                  BIT2
#define MPSC_MODE_BISYNC                                (BIT2|BIT0)
#define MPSC_MODE_TRANSPARENT_TRANSMITTER               BIT3
#define MPSC_MODE_TRANSPARENT_RECEIVER                  BIT4
#define MPSC_ENABLE_TRANSMIT                            BIT6
#define MPSC_ENABLE_RECEIVE                             BIT7
#define MPSC_LOOP_BACK_MODE                             BIT8
#define MPSC_ECHO_MODE                                  BIT9
#define MPSC_LOOP_BACK_AND_ECHO_MODE                    (BIT8|BIT9)
#define MPSC_NULL_MODEM                                 BIT10
#define MPSC_TRANSMITTER_SYNC_TO_RECEIVER               BIT12
#define MPSC_TRANSMIT_SENSE_0                           0
#define MPSC_TRANSMIT_SENSE_1                           BIT14
#define MPSC_TRANSMIT_SENSE_2                           BIT15
#define MPSC_TRANSMIT_SENSE_3                           (BIT14|BIT15)
#define MPSC_TRANSMIT_IDLES                             BIT16
#define MPSC_RTS_MODE_0                                 0
#define MPSC_RTS_MODE_1                                 BIT17
#define MPSC_CTS_SAMPLING_MODE_ASYNCHRONOUS             0
#define MPSC_CTS_SAMPLING_MODE_SYNCHRONOUS              BIT19
#define MPSC_CD_SAMPLING_MODE_ASYNCHRONOUS              0
#define MPSC_CD_SAMPLING_MODE_SYNCHRONOUS               BIT20
#define MPSC_CTS_ENVELOP_MODE                           0
#define MPSC_CTS_PULSE_MODE                             BIT21
#define MPSC_CD_ENVELOP_MODE                            0
#define MPSC_CD_PULSE_MODE                              BIT22
#define MPSC_CRC16_CCITT                                0
#define MPSC_CRC16                                      BIT23
#define MPSC_CRC32_CCITT                                BIT24   
#define MPSC_TX_REVERSE_DATA_MODE                       BIT28
#define MPSC_RX_REVERSE_DATA_MODE                       BIT29
#define MPSC_GLITCH_DETECT_ENABLE                       BIT31

/* End of MMCRLx defines */

/* main configuration register high - MMCRHx */
/* set MMCRH for a given port (use MMCRH defines) */                       
#define GT_MPSC_MMCRH_CONFIG(port, MmcrhConfig)					            \
			GT_REG_WRITE(MPSC_MAIN_CONFIG_HIGH(port), MmcrhConfig)       
#define MPSC_TRANSMIT_CLOCK_INVERT                      BIT0
#define MPSC_TRANSMIT_BIT_STREAM_INVERSION              BIT1
#define MPSC_TRANSMIT_PREAMBLE_LENGTH_1B                BIT2
#define MPSC_TRANSMIT_PREAMBLE_LENGTH_2B                BIT3
#define MPSC_TRANSMIT_PREAMBLE_LENGTH_4B                (BIT2|BIT3)
#define MPSC_TRANSMIT_PREAMBLE_LENGTH_6B                BIT4
#define MPSC_TRANSMIT_PREAMBLE_LENGTH_8B                (BIT2|BIT4)
#define MPSC_TRANSMIT_PREAMBLE_LENGTH_16B               (BIT3|BIT4)
#define MPSC_TRANSMIT_CLOCK_DEVIDER_1x                  0
#define MPSC_TRANSMIT_CLOCK_DEVIDER_8x                  BIT9
#define MPSC_TRANSMIT_CLOCK_DEVIDER_16x                 BIT10
#define MPSC_TRANSMIT_CLOCK_DEVIDER_32x                 (BIT9|BIT10)
#define MPSC_TRANSMIT_ENCODER_NRZ                       0
#define MPSC_TRANSMIT_ENCODER_NRZI                      BIT11                
#define MPSC_TRANSMIT_ENCODER_FM0                       BIT12
#define MPSC_TRANSMIT_ENCODER_MANCHESTER                BIT13
#define MPSC_TRANSMIT_ENCODER_DIF_MANCHESTER            (BIT12|BIT13)
#define MPSC_RECEIVE_BIT_STREAM_INVERSION               BIT16
/* BIT22 MUST be set in Bisync and Uart and MUST not be set in HDLC mode */
#define MPSC_RECEIVE_DATA_WIDTH_8BIT                    BIT22 
#define MPSC_RECEIVE_DATA_WIDTH_16BIT                   0     
#define MPSC_RECEIVE_SYNC_LENGTH_4_BIT                  BIT23 
#define MPSC_RECEIVE_SYNC_LENGTH_8_BIT                  BIT24
#define MPSC_RECEIVE_SYNC_LENGTH_16_BIT                 (BIT23|BIT24)
#define MPSC_RECEIVE_CLOCK_DEVIDER_1x                   0
#define MPSC_RECEIVE_CLOCK_DEVIDER_8x                   BIT25
#define MPSC_RECEIVE_CLOCK_DEVIDER_16x                  BIT26
#define MPSC_RECEIVE_CLOCK_DEVIDER_32x                  (BIT25|BIT26)
#define MPSC_RECEIVE_ENCODER_NRZ                        0
#define MPSC_RECEIVE_ENCODER_NRZI                       BIT27                
#define MPSC_RECEIVE_ENCODER_FM0                        BIT28
#define MPSC_RECEIVE_ENCODER_MANCHESTER                 BIT29
#define MPSC_RECEIVE_ENCODER_DIF_MANCHESTER             (BIT28|BIT29)
#define MPSC_SYNCHRONIZE_ON_FALLING_AND_RISING          0          
#define MPSC_SYNCHRONIZE_ON_RISING_EDGE                 BIT30 
#define MPSC_SYNCHRONIZE_ON_FALLING_EDGE                BIT31
#define MPSC_NO_ADJUSTMENT_TO_CLOCK                     (BIT30|BIT31)
/* Define the numer of bits to shift the fileds */
#define MPSC_GDW_SHIFT	    					17
#define MPSC_TPPT_SHIFT							5
/* Fix the GDW shift of bit's */
#define GT_MPSC_GDW(value) (((value) & 0xf) << MPSC_GDW_SHIFT)
/* Fix the TPPT shift of bit's */
#define GT_MPSC_TPPT(value) (((value) & 0xf) << MPSC_TPPT_SHIFT)
			
/* End of MMCRHx defines*/

/***************************************************************************
*
*			HDLC DEFINITIONS
*
***************************************************************************/

/* MPSCx Protocol Configuration Register (MPCRx) for HDLC */
#define  MPSC_HDLC_LOCAL_TALK			                BIT2
#define  MPSC_HDLC_CRC_COMPATIBLE_MODE	                0
#define  MPSC_HDLC_CRC_COMPLIANCE_MODE	                BIT4
#define  MPSC_HDLC_DISABLE_RX_ON_TX		                BIT6
#define  MPSC_HDLC_COLLISION_MODE		                BIT9
/* Bits' shift to NOF field */                       
#define  MPSC_HDLC_NOF_SHIFT			                12
/* Set NOF field */
#define  GT_MPSC_HDLC_NUMBER_OF_FLAGS(value)                                   \
                                        (((value) & 0xf) << MPSC_HDLC_NOF_SHIFT)
            
/* End of MPCR defines */

/* Channel Register CHR1 for HDLC */
#define  MPSC_HDLC_DEFAULT_SYNC_CHAR                    0x7E
#define  MPSC_HDLC_DEFAULT_ABORT_CHAR                   0xFE
/* Bits' shift to Abort field */                      
#define MPSC_HDLC_ABORT_SHIFT			                16
/* Set SYNC field */
#define GT_MPSC_HDLC_SYNC(value)  ((value) & 0xff)
/* Set Abort field */
#define GT_MPSC_HDLC_ABORT(value) (((value) & 0xff) << MPSC_HDLC_ABORT_SHIFT) 
 
/* End of CHR1 defines */

/* Channel Register CHR2 for HDLC */
#define  MPSC_HDLC_ABORT_TX				                BIT7
#define  MPSC_HDLC_ABORT_RX				                BIT23
#define  MPSC_HDLC_ENTER_HUNT			                BIT31

/* End of CHR2 defines */

/* Channel Register CHR3 for HDLC */
/* Set Max Frame Length field */
#define GT_MPSC_HDLC_FRAME_LENGTH(value) ((value) & 0xffff)

/* End of CHR3 defines */

/* Channel Register CHR4 for HDLC */
#define  MPSC_HDLC_NULL_ENABLE			                BIT29
#define  MPSC_HDLC_BROADCAST_ENABLE		                BIT31
/* set BCE field */
#define  GT_MPSC_HDLC_BCE_BITS(value) ((value) & 0xffff)

/* End of CHR4 defines */

/* Channel Register CHR5 for HDLC */
/* set SHFR field */
#define GT_MPSC_HDLC_SHORT_FRAME(value) ((value) & 0x7)

/* End of CHR5 defines */

/* Channel Register CHR6 for HDLC */
/* bits' shift to AD2 field */
#define MPSC_HDLC_ADDRESS_2_SHIFT	                    16
/* set AD1 field */
#define GT_MPSC_HDLC_RX_ADDR_RECOGNITION_1(value) ((value) & 0xffff)
/* set AD2 field */
#define GT_MPSC_HDLC_RX_ADDR_RECOGNITION_2(value)					           \
		                       (((value) & 0xffff) << MPSC_HDLC_ADDRESS_2_SHIFT)
           
/* End of CHR6 defines */

/* Channel Register CHR7 for HDLC */
/* bits' shift to AD4 field */
#define MPSC_HDLC_ADDRESS_4_SHIFT	        16
/* set AD3 field */
#define GT_MPSC_HDLC_RX_ADDR_RECOGNITION_3(value) ((value) & 0xffff)
/* set AD4 field */
#define GT_MPSC_HDLC_RX_ADDR_RECOGNITION_4(value)					           \
		                       (((value) & 0xffff) << MPSC_HDLC_ADDRESS_4_SHIFT)
           
/* End of CHR7 defines */

/* Channel Register CHR10 for HDLC */
#define  MPSC_HDLC_CLEAR_TO_SEND						BIT0
#define  MPSC_HDLC_CARRIER_DETECT						BIT1
#define  MPSC_HDLC_TX_IN_IDLE							BIT3
#define  MPSC_HDLC_RX_IN_HUNT							BIT5
#define  MPSC_HDLC_RX_IN_IDLE							BIT11
#define  MPSC_HDLC_DPLL_CARRIER_SENS					BIT12
#define  MPSC_HDLC_RX_RECEIVING_FLAGS					BIT13

/* End of CHR10 defines */


/***************************************************************************
*
*			BISYNC DEFINITIONS
*
***************************************************************************/

/* MPSCx Protocol Configuration Register (MPCRx) for BISYNC */
#define	 MPSC_BISYNC_AUTO_TRANSPARENT_MODE            	BIT2
#define  MPSC_BISYNC_DISABLE_RX_ON_TX	                BIT6
#define  MPSC_BISYNC_TRAILING_PAD		                BIT10
#define  MPSC_BISYNC_RX_TRANSPARENT_MODE                BIT13
#define  MPSC_BISYNC_RX_DISCARD_BCC		                BIT14
#define  MPSC_BISYNC_TX_32_SYNC_MODE	                BIT15

/* End of MPCRx defines */

/* Channel Register CHR1 for BISYNC */
#define  MPSC_BISYNC_DISCARD_SYNC		                BIT15
#define  MPSC_BISYNC_DISCARD_DLE		                BIT31
/* bits' shift to DLE field */
#define  MPSC_BISYNC_DLE_SHIFT			                16
/* set SYNC field */
#define GT_MPSC_BISYNC_SYNC(value) ((value) & 0xff)
/* set DLE field */
#define GT_MPSC_BISYNC_DLE(value)    (((value) & 0xff) << MPSC_BISYNC_DLE_SHIFT)
            
/* End of CHR1 defines */

/* Channel Register CHR2 for BISYNC */
#define  MPSC_BISYNC_TX_ENABLE_LONGITUDINAL_CHECK		BIT0
#define  MPSC_BISYNC_TX_ENABLE_VERTICAL_PARITY_CHECK	BIT1
#define  MPSC_BISYNC_TX_PARITY_ODD						0
#define  MPSC_BISYNC_TX_PARITY_LOW						BIT2
#define  MPSC_BISYNC_TX_PARITY_EVEN						BIT3
#define  MPSC_BISYNC_TX_PARITY_HIGH						(BIT3 | BIT2)
#define  MPSC_BISYNC_TX_LONGITUDINAL_MODE_ODD			0
#define  MPSC_BISYNC_TX_LONGITUDINAL_MODE_EVEN			BIT4
#define  MPSC_BISYNC_ABORT_TX							BIT7
#define  MPSC_BISYNC_RX_ENABLE_LONGITUDINAL_CHECK		BIT16
#define  MPSC_BISYNC_RX_ENABLE_VERTICAL_PARITY_CHECK	BIT17
#define  MPSC_BISYNC_RX_PARITY_ODD						0
#define  MPSC_BISYNC_RX_PARITY_LOW						BIT18
#define  MPSC_BISYNC_RX_PARITY_EVEN						BIT19
#define  MPSC_BISYNC_RX_PARITY_HIGH						(BIT19 | BIT18)
#define  MPSC_BISYNC_RX_LONGITUDINAL_MODE_ODD			0
#define  MPSC_BISYNC_RX_LONGITUDINAL_MODE_EVEN			BIT20
#define  MPSC_BISYNC_ABORT_RX							BIT23
#define  MPSC_BISYNC_CLOSE_RX_DESCRIPTOR				BIT25
#define  MPSC_BISYNC_RESET_BCC							BIT29
#define  MPSC_BISYNC_ENTER_HUNT							BIT31

/* End of CHR2 defines */

/* Channel Register CHR4 for BISYNC */
/* set BCE field */
#define GT_MPSC_BISYNC_CONTROL_FILTERING(value) ((value) & 0xff)

/* End of CHR4 defines */
		   
/* Channel Registers CHR5-8 for BISYNC */
#define  MPSC_BISYNC_START_OF_HEADER					BIT9
#define  MPSC_BISYNC_START_OF_TEXT						BIT10
#define  MPSC_BISYNC_IGNORE_CHAR_IN_TEXT_MODE			BIT11
#define  MPSC_BISYNC_GENERATE_INTERRUPT_ON_RX			BIT12
#define  MPSC_BISYNC_MOVE_TO_HUNT						BIT13
#define  MPSC_BISYNC_BCC_NEXT							BIT14
#define  MPSC_BISYNC_ENTRY_NOT_VALID					0
#define  MPSC_BISYNC_ENTRY_VALID						BIT15

/* Const for BISYNC comm with none parity */
#define  MPSC_START_OF_TXT					            0x2
#define	 MPSC_END_OF_TXT	   				            0x3
#define	 MPSC_START_OF_HNDLR				            0x1
#define	 MPSC_END_INTERMEDIATE_BLOCK		            0x1F
#define	 MPSC_ENQUIRY						            0x5
#define	 MPSC_NACK							            0x15
#define	 MPSC_NULL_VALUE					            0x0
#define  MPSC_BISYNC_DEFAULT_SYNC_CHAR		            0x16
#define  MPSC_BISYNC_DEFAULT_DELIMETER_CHAR	            0x10

/* Const for BISYNC comm with odd parity */
#define  MPSC_START_OF_TXT_ODD					        0x2
#define	 MPSC_END_OF_TXT_ODD	   				        0x83
#define	 MPSC_START_OF_HNDLR_ODD				        0x1
#define	 MPSC_END_INTERMEDIATE_BLOCK_ODD		        0x1F
#define	 MPSC_ENQUIRY_ODD						        0x85
#define	 MPSC_NACK_ODD							        0x15
#define	 MPSC_NULL_VALUE_ODD					        0x80
#define  MPSC_BISYNC_DEFAULT_SYNC_CHAR_ODD		        0x16
#define  MPSC_BISYNC_DEFAULT_DELIMETER_CHAR_ODD	        0x10
                                                       
/* Const for BISYNC comm with even parity */           
#define  MPSC_START_OF_TXT_EVEN					        0x82
#define	 MPSC_END_OF_TXT_EVEN     				        0x3
#define	 MPSC_START_OF_HNDLR_EVEN				        0x81
#define	 MPSC_END_INTERMEDIATE_BLOCK_EVEN		       	0x9F
#define	 MPSC_ENQUIRY_EVEN						        0x5
#define	 MPSC_NACK_EVEN							        0x95
#define	 MPSC_NULL_VALUE_EVEN					        0x0
#define  MPSC_BISYNC_DEFAULT_SYNC_CHAR_EVEN		        0x96
#define  MPSC_BISYNC_DEFAULT_DELIMETER_CHAR_EVEN        0x90

/* set CHAR field, bits 7:0 */
#define GT_MPSC_BISYNC_CONTROL_CHAR_LOW(value) ((value) & 0xffff)
/* set CHAR field, bits 23:16 */
#define GT_MPSC_BISYNC_CONTROL_CHAR_HIGH(value) (((value) & 0xffff) << 16)

/* End of CHR5-8 defines */

/* Channel Register CHR10 for BISYNC */
#define  MPSC_BISYNC_CLEAR_TO_SEND						BIT0
#define  MPSC_BISYNC_CARRIER_DETECT						BIT1
#define  MPSC_BISYNC_TX_IN_IDLE							BIT3
#define  MPSC_BISYNC_RX_IN_HUNT							BIT5
#define  MPSC_BISYNC_RX_IN_IDLE							BIT11
#define  MPSC_BISYNC_DPLL_CARRIER_SENS					BIT12
#define  MPSC_BISYNC_RX_CONTROL_CHAR_1					BIT16
#define  MPSC_BISYNC_RX_CONTROL_CHAR_2					BIT17
#define  MPSC_BISYNC_RX_CONTROL_CHAR_3					BIT18
#define  MPSC_BISYNC_RX_CONTROL_CHAR_4					BIT19
#define  MPSC_BISYNC_RX_CONTROL_CHAR_5					BIT20
#define  MPSC_BISYNC_RX_CONTROL_CHAR_6					BIT21
#define  MPSC_BISYNC_RX_CONTROL_CHAR_7					BIT22
#define  MPSC_BISYNC_RX_CONTROL_CHAR_8					BIT23

/* End of CHR10 defines */

/***************************************************************************
*
*			UART DEFINITIONS
*
***************************************************************************/

/* MPSCx Protocol Configuration Register (MPCRx) for UART */
#define  MPSC_UART_DISABLE_RX_ON_TX				        BIT6
#define  MPSC_UART_ASYNCHRONOUS_MODE			        0
#define  MPSC_UART_ISOCHRONOUS_MODE  			        BIT7
#define  MPSC_UART_RX_ZERO_STOP_BIT				        BIT8
#define  MPSC_UART_FREEZE_TX					        BIT9
#define  MPSC_UART_NORMAL_MODE					        0
#define  MPSC_UART_MULTI_DROP_MODE				        BIT10
#define  MPSC_UART_CHAR_LENGTH_5_BIT			        0
#define  MPSC_UART_CHAR_LENGTH_6_BIT			        BIT12
#define  MPSC_UART_CHAR_LENGTH_7_BIT			        BIT13
#define  MPSC_UART_CHAR_LENGTH_8_BIT			        (BIT13 | BIT12)
#define  MPSC_UART_STOP_BIT_LENGTH_1			        0
#define  MPSC_UART_STOP_BIT_LENGTH_2			        BIT14
#define  MPSC_UART_FLOW_CONTROL_NORMAL			        0
#define  MPSC_UART_FLOW_CONTROL_ASYNC			        BIT15

/* End of MPCRx defines */

/* Channel Register CHR1 for UART */
/* bits' shift to BRK field */
#define MPSC_UART_BREAK_COUNT_SHIFT				        16
/* set TCS field */
#define GT_MPSC_UART_CONTROL_STUFF_CHAR(value) ((value) & 0xff)
/* set BRK field */
#define GT_MPSC_UART_BREAK_COUNT(value)							            \
		    (((value) & 0xff) << MPSC_UART_BREAK_COUNT_SHIFT)

/* End of CHR1 defines */

/* Channel Register CHR2 for UART */
#define  MPSC_UART_TX_ENABLE_VERTICAL_PARITY_CHECK		BIT1
#define  MPSC_UART_TX_PARITY_ODD						0
#define  MPSC_UART_TX_PARITY_LOW						BIT2
#define  MPSC_UART_TX_PARITY_EVEN						BIT3
#define  MPSC_UART_TX_PARITY_HIGH						(BIT3 | BIT2)
#define  MPSC_UART_ABORT_TX								BIT7
#define  MPSC_UART_TX_TCS_CHAR							BIT9
#define  MPSC_UART_RX_ENABLE_VERTICAL_PARITY_CHECK		BIT17
#define  MPSC_UART_RX_PARITY_ODD						0
#define  MPSC_UART_RX_PARITY_LOW						BIT18
#define  MPSC_UART_RX_PARITY_EVEN						BIT19
#define  MPSC_UART_RX_PARITY_HIGH						(BIT19 | BIT18)
#define  MPSC_UART_ABORT_RX								BIT23
#define  MPSC_UART_CLOSE_RX_DESCRIPTOR					BIT25
#define  MPSC_UART_ENTER_HUNT							BIT31

/* End of CHR2 defines */

/* Channel Register CHR3 for UART */
/* set Max Idle field */
#define GT_MPSC_UART_MAX_IDLE(value) ((value) & 0xffff)

/* End of CHR3 defines */

/* Channel Register CHR4 for UART */
/* set BCE field */
#define GT_MPSC_UART_CONTROL_FILTERING(value) ((value) & 0xff)

/* End of CHR4 defines */

/* Channel Registers CHR5-8 for UART */
#define  MPSC_UART_GENERATE_INTERRUPT_ON_RX				BIT12
#define  MPSC_UART_CONTROL_OCTET						BIT13
#define  MPSC_UART_REJECT_CHAR							BIT14
#define  MPSC_UART_ENTRY_NOT_VALID						0
#define  MPSC_UART_ENTRY_VALID							BIT15
/* set CHAR field, bits 7:0 */
#define GT_MPSC_UART_CONTROL_CHAR_LOW(value) ((value) & 0xffff)
/* set CHAR field, bits 23:16 */
#define GT_MPSC_UART_CONTROL_CHAR_HIGH(value) (((value) & 0xffff) << 16)

/* End of CHR5-8 defines */

/* Channel Register CHR9 for UART */
#define  MPSC_UART_MATCH_ADDRESS_1						BIT15
#define  MPSC_UART_MATCH_ADDRESS_2						BIT31
#define  GT_MPSC_UART_MATCH_ADDR_1(value)               ((value) & 0xff)
#define  GT_MPSC_UART_MATCH_ADDR_2(value)               (((value) & 0xff) << 16)

/* End of CHR9 defines */

/* Channel Register CHR10 for UART */
#define  MPSC_UART_CLEAR_TO_SEND						BIT0
#define  MPSC_UART_CARRIER_DETECT						BIT1
#define  MPSC_UART_TX_IN_IDLE							BIT3
#define  MPSC_UART_RX_IN_HUNT							BIT5
#define  MPSC_UART_RX_LINE_STATUS						BIT7
#define  MPSC_UART_RX_IN_IDLE							BIT11
#define  MPSC_UART_RX_CONTROL_CHAR_1					BIT16
#define  MPSC_UART_RX_CONTROL_CHAR_2					BIT17
#define  MPSC_UART_RX_CONTROL_CHAR_3					BIT18
#define  MPSC_UART_RX_CONTROL_CHAR_4					BIT19
#define  MPSC_UART_RX_CONTROL_CHAR_5					BIT20
#define  MPSC_UART_RX_CONTROL_CHAR_6					BIT21
#define  MPSC_UART_RX_CONTROL_CHAR_7					BIT22
#define  MPSC_UART_RX_CONTROL_CHAR_8					BIT23

/* End of CHR10 defines*/


/***************************************************************************
*
*			TRANSPARENT DEFINITIONS
*
***************************************************************************/

/* Channel Register CHR1 for TRANSPARENT */
#define  MPSC_TRANSPARENT_SYNC_MODE				        BIT15
#define  MPSC_TRANS_DEFAULT_SYNC_CHAR			        0x7E
/* set SYNC field 7:0 */
#define GT_MPSC_TRANSPARENT_SYNC_CHAR(value) ((value) & 0xff)
			
/* End of CHR1 defines */

/* Channel Register CHR2 for TRANSPARENT */
#define  MPSC_TRANSPARENT_ABORT_TX				        BIT7
#define  MPSC_TRANSPARENT_ABORT_RX				        BIT23
#define  MPSC_TRANSPARENT_CLOSE_RX_DESC			        BIT25
#define  MPSC_TRANSPARENT_ENTER_HUNT			        BIT31
                                                     
/* End of CHR2 defines */                             
                                                     
/* Channel Registers CHR10 for TRANSPARENT */        
#define  MPSC_TRANSPARENT_CLEAR_TO_SEND			        BIT0
#define  MPSC_TRANSPARENT_CARRIER_DETECT		        BIT1
#define  MPSC_TRANSPARENT_TX_IN_IDLE			        BIT3
#define  MPSC_TRANSPARENT_RX_IN_HUNT			        BIT5
#define  MPSC_TRANSPARENT_DPLL_CARRIER_SENSE	        BIT12

/* End of CHR10 defines */

/***************************************************************************
*
*			BRG Definitions
*
***************************************************************************/
/* defines for initialze BCR */
#define MPSC_ENABLE_BRG                                 BIT16
#define MPSC_RESET_BRG                                  BIT17
#define MPSC_BRG_BAUD_TUNING                            BIT25
#define MPSC_CDV_MASK                                   0x0000ffff


/* functions */
                     
void         gtMpscConfigForHDLC(MPSC_PORT port, MPSC_CLOCK_SOURCE rxClkSrc, 
                                 unsigned int rxClkRate, 
                                 MPSC_CLOCK_SOURCE txClkSrc,
                                 unsigned int txClkRate, unsigned int tclkFreq,
                                 MPSC_ACTUAL_RATES *rates);
void         gtMpscConfigForBISYNC(MPSC_PORT port, 
                                   MPSC_BISYNC_PARITY_MODE parity, 
                                   MPSC_BISYNC_MODE mode , 
                                   MPSC_CLOCK_SOURCE rxClkSrc, 
                                   unsigned int rxClkRate, 
                                   MPSC_CLOCK_SOURCE txClkSrc,
                                   unsigned int txClkRate, 
                                   unsigned int tclkFreq,
                                   MPSC_ACTUAL_RATES *rates);
void         gtMpscConfigForUART(MPSC_PORT port, MPSC_CLOCK_SOURCE rxClkSrc, 
                                 unsigned int rxClkRate, 
                                 MPSC_CLOCK_SOURCE txClkSrc,
                                 unsigned int txClkRate, unsigned int tclkFreq,
                                 MPSC_ACTUAL_RATES *rates);
void         gtMpscConfigForTRANSPARENT(MPSC_PORT port, 
                                        MPSC_CLOCK_SOURCE rxClkSrc, 
                                        unsigned int rxClkRate, 
                                        MPSC_CLOCK_SOURCE txClkSrc,
                                        unsigned int txClkRate, 
                                        unsigned int tclkFreq,
                                        MPSC_ACTUAL_RATES *rates);
void         gtMpscBrgEnable(MPSC_BRG_NUM brgNum);
void         gtMpscBrgDisable(MPSC_BRG_NUM brgNum);
unsigned int gtMpscBrgInit(MPSC_BRG_NUM brgNum, 
                           MPSC_BRG_CLOCK_SOURCE brgClockSource,
                           unsigned int brgClockSourceFreq, unsigned int freq);
void         gtAbortMpsc(MPSC_PORT port);
void         gtAbortAllMpscPorts(void);

#endif /* _INCgtmpsch */


