#define MAX_GPIO_NUM 16
#define W81_CFG_BASE             (0x80000000) 
#define    W81_LBU_GPU_CFG_BASE         (W81_CFG_BASE+0xD000)      //GP I/O unit
#define GPIO_REG(offset) (W81_LBU_GPU_CFG_BASE | offset)
#define GPIO_REG_OUTPUT_ENABLE  GPIO_REG(0x08)
#define GPIO_REG_SELECT_0_7     W81_LBU_GPU_CFG_BASE
#define GPIO_REG_SELECT_8_15    GPIO_REG(0x04)
#define GPIO_REG_OUTPUTS        GPIO_REG(0x0C)
#define GPIO_REG_INPUTS         GPIO_REG(0x10)


#define W81_REGS32(x)    (*(unsigned long *)(x))
#define W81_WRITE_WORD(reg,val)     (W81_REGS32(reg) = val)    //32 bits
#define W81_READ_WORD(reg,val)     ((val) = W81_REGS32(reg))    //32 bits
#define W81_WRITE_REGS32(reg,val)   (W81_REGS32(reg) = val)

typedef unsigned short       uint16;
typedef unsigned int         uint32;
typedef unsigned long long   uint64;

typedef	enum
{
   FALSE = 0,
   TRUE
} Boolean;

void apctlGPIOInit(unsigned long dirMap);
int apctlGPIORead(int gpioNum);
int apctlGPIOWrite(int gpioNum, int state);
int gpioInOutputMode(int gpioNum);

void apctlGPIOInit(unsigned long dirMap)
{
  uint32 idx;
  uint32 regVal;

  W81_WRITE_REGS32(GPIO_REG_OUTPUT_ENABLE, dirMap & 0xFFFF);
  /* Have two GPIO output source select registers, so do two loops */ 
  regVal = 0;
  for (idx = 0; idx < 8; idx++)
  {
    if (((dirMap >> idx) & 0x01) == 0)
    {
      /* Set to output, so select output source */
      regVal |= (1 << (2*idx));
    }
  }
  W81_WRITE_WORD(GPIO_REG_SELECT_0_7, regVal);
  dirMap = dirMap >> 8;

  regVal = 0;
  for (idx = 0; idx < 8; idx++)
  {
    if (((dirMap >> idx) & 0x01) == 0)
    {
      /* Set to output, so select output source */
      regVal |= (1 << (2*idx));
    }
  }
  W81_WRITE_WORD(GPIO_REG_SELECT_8_15, regVal);
}


int apctlGPIORead(int gpioNum)
{
  uint32 regVal;

  if (gpioNum >= MAX_GPIO_NUM)
  {
    return -1;
  }

  if (gpioInOutputMode(gpioNum))
  {
    /* GPIO pin is output */
    W81_READ_WORD(GPIO_REG_OUTPUTS, regVal);
    regVal = (regVal >> gpioNum) & 0x01;
  }
  else
  {
    /* GPIO pin is input */
    W81_READ_WORD(GPIO_REG_INPUTS, regVal);
    regVal = (regVal >> gpioNum) & 0x01;
  }
  
  return regVal;
}

int apctlGPIOWrite(int gpioNum, int state)
{
  uint32 regVal;

  if (gpioNum >= MAX_GPIO_NUM)
  {
    return -1;
  }

  if (gpioInOutputMode(gpioNum))
  {
    /* GPIO pin is output */
  
    /* Read the current setting and preserve what we do not touch */
    W81_READ_WORD(GPIO_REG_OUTPUTS, regVal);
  
    /* Set the output pin according to state */
    if (state)
    {
      /* Must return state value of 1 or 0 */
      state = 1;
      regVal |= (1 << gpioNum);
    }
    else
    {
      regVal &= ~(1 << gpioNum);
    }
  
    W81_WRITE_WORD(GPIO_REG_OUTPUTS, regVal);
  }
  else
  {
    /* GPIO pin is in input mode */
    return (-2);
  }
  
  return state;
}


int gpioInOutputMode(int gpioNum)
{
  uint32 dirMap;
  int retVal = FALSE;

  if (gpioNum < 8)
  {
    W81_READ_WORD(GPIO_REG_SELECT_0_7, dirMap);
  }
  else
  {
    W81_READ_WORD(GPIO_REG_SELECT_8_15, dirMap);
    /* transform gpio num to a number between 0 and 7 */
    gpioNum -= 8;
  }

  /* Test output mode */
  if (((dirMap >> (gpioNum * 2)) & 0x03) == 0x01)
  {
    retVal = TRUE;
  }
  
  return retVal;
}
