#include "decls.h"
#include "board.h"
#include "flash_types.h"
/*----- Types and Constants Definition ----------------------*/
#define _REG(x) (*(volatile unsigned long *)(x))
#define _REG16(x) (*(volatile unsigned short *)(x))

#define MDU_CSR      0x80006000
#define MDU_CFGR0    0x80006004
#define MDU_TIMR00   0x80006008    
#define MDU_TIMR01   0x8000600C
#define MDU_CFGR1    0x80006010
#define MDU_TIMR10   0x80006014
#define MDU_TIMR11   0x80006018
#define MDU_CFGR2    0x8000601C
#define MDU_TIMR20   0x80006020
#define MDU_TIMR21   0x80006024
#define MDU_CFGR3    0x80006028
#define MDU_TIMR30   0x8000602C
#define MDU_TIMR31   0x80006030
#define MDU_CFGR4    0x80006034
#define MDU_TIMR40   0x80006038
#define MDU_TIMR41   0x8000603C
#define MDU_CFGR5    0x80006040
#define MDU_TIMR50   0x80006044
#define MDU_TIMR51   0x80006048

int split_sector = -1;
int split_sector_data[]={
    0,
    0x8000,
    0xa000,
    0xc000,
};
int split[2][4]={
    { 16, 8, 8, 32 }, 
    { 32, 8, 8, 16 }
};
void enable_cache(void);
void disable_cache(void);

void
interrogate()
{
    disable_cache();
   _REG16(flashbase | 0xAAA) = 0x00AA;
   _REG16(flashbase | 0x555) = 0x0055;
   _REG16(flashbase | 0xAAA) = 0x0090;
   manu = _REG16(flashbase);     // Manu
   part = _REG16(flashbase + 2); // part ID.
   _REG16(flashbase) = 0x00f0; // reset command
   enable_cache();
}
void 
makesplit(unsigned ss)
{
    int top = 0;
    int start = 0;
    int i;
    split_sector = ss;
    if(ss) {
	top = 1;
    }
    for(i=0;i < 4;i++) {
	split_sector_data[i] = start;
	start += split[top][i] * 1024;
    }
}


void ConfigMDU()
{
    int boardtype = BOARD;	
	
    _REG(MDU_TIMR00) = 0x0442A20C;   /* FLASH timing */
    _REG(MDU_TIMR01) = 0x00008080;
    _REG(MDU_CFGR0)  = 0xFFFF4285;   /* 0xffc00000 FLASH configuration pack_en = 1*/
    switch (boardtype)
    {
    case EVAL_BOARD:
        _REG(MDU_CSR)    = 0x04190040;
        _REG(MDU_TIMR20) = 0x00000022;
        _REG(MDU_TIMR21) = 0x03232222;
        _REG(MDU_CFGR2)  = 0x40FE50B5;
        break;
    case AP21_2X1MX16_BOARD:
        _REG(MDU_CSR)    = 0x157E0000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x03232222;
        _REG(MDU_CFGR1)  = 0x40FF7095;
        break;
    case AP21_1X2MX32_BOARD:
        _REG(MDU_CSR)    = 0x157E0000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x03232232;
        _REG(MDU_CFGR1)  = 0x40FE70B7;
        break;
    case AP21_1X4MX16_BOARD:
        _REG(MDU_CSR)    = 0x157E0000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x03232232;
        _REG(MDU_CFGR1)  = 0x40FE50B5;
        break;
    case AP22_2X4MX16_BOARD:
        _REG(MDU_CSR)    = 0x20530000;
        _REG(MDU_TIMR10) = 0x00000022;
        _REG(MDU_TIMR11) = 0x08383632;
        _REG(MDU_CFGR1)  = 0x40FC70B5;
        break;
    case AP22_1X2MX32_BOARD:
        _REG(MDU_CSR)    = 0x20530000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x0A3A3632;
        _REG(MDU_CFGR1)  = 0x40FE70B7;
        break;
    case AP32_2X4MX16_BOARD:
        _REG(MDU_CSR)    = 0x20530000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x0A3A3632;
        _REG(MDU_CFGR1)  = 0x40FC70B5;
        break;
    default:
        _REG(MDU_CSR)    = 0x157E0000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x03232232;
        _REG(MDU_CFGR1)  = 0x40FE70B7;
    }
    // Disable devices 3 4 5
    _REG(MDU_CFGR3) = 0xC0FF0008;
    _REG(MDU_CFGR4) = 0x80FF0008;
    _REG(MDU_CFGR5) = 0x40FF0008;

    // Turn on all clocks for debugging.
//    _REG16(CLOCKS)  = 0x3FFF;
}
void
ConfigFlash()
{
   interrogate();
    // is it a manufacturer we know?
    switch(manu) {
    case AMD:
	// Have we heard of the part?
	switch(part) {
	case 0x22c4: // 160BT
	    flashsize = 2*1024*1024;
	    flashNsect = 32;
	    makesplit(31);
	    break;
	case 0x2249: // 160BB
	    flashsize = 2*1024*1024;
	    flashNsect = 32;
	    makesplit(0);
	    break;
	case 0x22da: // LV800BT
	    flashsize = 1*1024*1024;
	    flashNsect = 16;
	    makesplit(15);
	    break;
	case 0x225b: // LV800BB
	    flashsize = 1*1024*1024;
	    flashNsect = 16;
	    makesplit(0);
	    break;
	default:
	    break;
	}
	break;
    case HYUNDAI: // Hyundai
	switch(part) {
	case 0xD5: // 29F080
	    flashsize = 1*1024*1024;
	    flashNsect = 16;
	    break;
	default:
	    break;
	}
	break;
    case MICRONIX:
	// Have we heard of the part?
	switch(part) {
	case 0x22c4: // 160BT
	    flashsize = 2*1024*1024;
	    flashNsect = 32;
	    makesplit(31);
	    break;
	default:
	    break;
	}
	break;
    default:
	break;
    }
    flashbase = (0 - flashsize);
}
void amdPreamble()
{
   _REG16(flashbase | 0xAAA) = 0x00AA;
   _REG16(flashbase | 0x555) = 0x0055;
   _REG16(flashbase | 0xAAA) = 0x0080;
   _REG16(flashbase | 0xAAA) = 0x00AA;
   _REG16(flashbase | 0x555) = 0x0055;
}
void amdEraseChip(int sblk, int nblks)
{
    unsigned addr;
    int i;
    int j;
    int done;
    
    flashsize = 1*1024*1024;
    flashbase = (0-flashsize);
    makesplit(15);
    
    for(i=sblk;i < (sblk+nblks);i++) {
	if(i == split_sector) {
	    unsigned start = (i << (16));
	    for(j=0;j < sizeof(split_sector_data)/sizeof(int);j++) {
		addr= flashbase + start + split_sector_data[j];
		disable_cache();
		amdPreamble();
		_REG16(addr) = 0x0030;
		for(done=0;!done;) {
		    if(_REG16(addr) == _REG16(addr)) {
			done = 1;
		    }
		}
		enable_cache();
	    }
	} else {
	    addr= flashbase + (i << (16));
	    disable_cache();
	    amdPreamble();
	    _REG16(addr) = 0x0030;
	    for(done=0;!done;) {
		if(_REG16(addr) == _REG16(addr)) {
		    done = 1;
		}
	    }
	    enable_cache();
	}
   }
   invalidate_cache();
}

void amdFlashWriteWord( int offset, unsigned short dat)
{
    int done;
    unsigned addr = flashbase+offset;
    disable_cache();
    _REG16(flashbase | 0xAAA) = 0x00AA;  // word address 555
    _REG16(flashbase | 0x555) = 0x0055;  // word address 2AA
    _REG16(flashbase | 0xAAA) = 0x00A0;  // word address 555

    _REG16(addr) = dat;  // word address 0xFC000000
    do {
	done = 0;
	if(_REG16(addr) == dat) {
	    done++;
	}
	if(_REG16(addr) == dat) {
	    done++;
	}
    } while(done != 2);
    enable_cache();
}

