/*
 * cgi.c -- CGI processing (for the GoAhead Web server
 *
 * Copyright (c) GoAhead Software Inc., 1995-2000. All Rights Reserved.
 *
 * See the file "license.txt" for usage and redistribution license requirements
 *
 * $Id: //depot/MRVL/main/firmware/AP5x/prod/ap/ap5x/sdk/http/src/upload.c#1 $
 */

/********************************** Description *******************************/
/*
 *	This module implements the /cgi-bin handler. CGI processing differs from
 *	goforms processing in that each CGI request is executed as a separate 
 *	process, rather than within the webserver process. For each CGI request the
 *	environment of the new process must be set to include all the CGI variables
 *	and its standard input and output must be directed to the socket.  This
 *	is done using temporary files.
 */

/*********************************** Includes *********************************/
#include "wltypes.h"
#include "oem.h"
#include "IEEE_types.h"
#include	"wsIntrn.h"
#ifdef UEMF
	#include	"uemf.h"
#else
	#include	"basic/basicInternal.h"
#endif

/************************************ Locals **********************************/

sym_fd_t cgiSymtab = -1;
int webCgiDoUpload(webs_t wp, char_t* query, char_t *cgiName);

typedef struct {				/* Struct for CGI tasks which have completed */
	webs_t	wp;					/* pointer to session websRec */
	char_t	*stdIn;				/* file desc. for task's temp input fd */
	char_t	*stdOut;			/* file desc. for task's temp output fd */
	char_t	*cgiPath;			/* path to executable process file */
	char_t	**argp;				/* pointer to buf containing argv tokens */
	char_t	**envp;				/* pointer to array of environment strings */
	int		handle;				/* process handle of the task */
	long	fplacemark;			/* seek location for CGI output file */
} cgiRec;
static cgiRec	**cgiList;		/* hAlloc chain list of wp's to be closed */
static int		cgiMax;			/* Size of hAlloc list */

/************************************* Code ***********************************/

/*
 *	Process a form request. Returns 1 always to indicate it handled the URL
 */
int websCgiHandler(webs_t wp, char_t *urlPrefix, char_t *webDir, int arg, 
		char_t *url, char_t *path, char_t* query)
{
	cgiRec		*cgip;
	sym_t		*s;
	char_t		cgiBuf[FNAMESIZE], *stdIn, *stdOut, cwd[FNAMESIZE];
	char_t		*cp, *cgiName, *cgiPath, **argp, **envp, **ep;
	int			n, envpsize, argpsize, pHandle, cid;
#if 0//def BRIDGE_POLLING_1
    setBridgeTaskToIntDriven();
#endif
	a_assert(websValid(wp));
	a_assert(url && *url);
	a_assert(path && *path == '/');
	websStats.cgiHits++;
/*
 *	Extract the form name and then build the full path name.  The form
 *	name will follow the first '/' in path.
 */
	gstrncpy(cgiBuf, path, TSZ(cgiBuf));
	if ((cgiName = gstrchr(&cgiBuf[1], '/')) == NULL) {
		websError(wp, 200, T("Missing CGI name"));
		return 1;
	}
	cgiName++;
	if ((cp = gstrchr(cgiName, '/')) != NULL) {
		*cp = '\0';
	}
	fmtAlloc(&cgiPath, FNAMESIZE, T("%s/%s/%s"), websGetDefaultDir(),
		CGI_BIN, cgiName);


    webCgiDoUpload(wp, query, cgiName);

	return 1;
}


/******************************************************************************/
/*
 *	Any entry in the cgiList need to be checked to see if it has
 *	completed, and if so, process its output and clean up.
 */
void websCgiCleanup()
{
	cgiRec	*cgip;
	webs_t	wp;
	char_t	**ep;
	int		cid, nTries;
	for (cid = 0; cid < cgiMax; cid++) {
		if ((cgip = cgiList[cid]) != NULL) {
			wp = cgip->wp;
/*bt_disabled			websCgiGatherOutput (cgip); */
			if (websCheckCgiProc(cgip->handle) == 0) {
/*
 *				We get here if the CGI process has terminated.  Clean up.
 */
				nTries = 0;
/*				
 *				Make sure we didn't miss something during a task switch.
 *				Maximum wait is 100 times 10 msecs (1 second).
 */
				while ((cgip->fplacemark == 0) && (nTries < 100)) {
/*bt_disabled					websCgiGatherOutput(cgip); */
/*					
 *					There are some cases when we detect app exit 
 *					before the file is ready. 
 */
					if (cgip->fplacemark == 0) {
#ifdef WIN
						Sleep(10);
#endif /* WIN*/
					}
					nTries++;
				}
				if (cgip->fplacemark == 0) {
					websError(wp, 200, T("CGI generated no output"));
				} else {
					websDone(wp, 200);
				}
/*
 *				Remove the temporary re-direction files
 */
				gunlink(cgip->stdIn);
				gunlink(cgip->stdOut);
/*
 *				Free all the memory buffers pointed to by cgip.
 *				The stdin file name (wp->cgiStdin) gets freed as
 *				part of websFree().
 */
				cgiMax = hFree((void***) &cgiList, cid);
				for (ep = cgip->envp; ep != NULL && *ep != NULL; ep++) {
					bfreeSafe(B_L, *ep);
				}
				bfreeSafe(B_L, cgip->cgiPath);
				bfreeSafe(B_L, cgip->argp);
				bfreeSafe(B_L, cgip->envp);
				bfreeSafe(B_L, cgip->stdOut);
				bfreeSafe(B_L, cgip);
			}
		}
	}
}



/***********************************************************************/

#define BOUNDRY_SIZE_BUF   64
#define BOUNDRY_TAG_SIZE   29
#define DASH                '-'
#define LINE_FEED         '\n'


/*
 *
 *	
 */
int webCgiDoUpload(webs_t wp, char_t* query, char_t *cgiName)
{
    sym_t		*funct;
    int (*pFunct)(void *sock, char_t *path, char_t *query);
 

    if(strncmp(cgiName, "upgrade", 7) == 0)
    {
        upgrade(wp, cgiName, query);
    }
    else if(strncmp(cgiName, "Reboot", 6) == 0)
    {
        apRebooting(wp, "/index.htm");
    }
    else
{
        websError(wp, 200, T("CGI Fail in upload"));
    }
    return 0;
}

/*
 *
 *	
 */
unsigned long webCgiStripBoundry(char *pMemStart, char *pMemEnd, char *boundry_symb)
{
char srchBoundry[BOUNDRY_SIZE_BUF];
int i;

int len;

	for(i=0; i<BOUNDRY_TAG_SIZE; i++)
    {
        srchBoundry[i] = '-';
    }
	srchBoundry[BOUNDRY_TAG_SIZE] = 0x00;
    len = strlen(srchBoundry);
	strcat(srchBoundry, boundry_symb);
    len += strlen(boundry_symb);

	pMemEnd -= len;
	while( pMemEnd > pMemStart )
	{
		if( memcmp(pMemEnd, srchBoundry, len) == 0 )
		{
			pMemEnd -= 2;
			*pMemEnd = 0x00;
			return ( pMemEnd - pMemStart );
		}
		else
			pMemEnd--;
	}
	
	return 0;
}


/*
 *
 *	
 */
int webCgiGetUploadFile(webs_t wp, char_t **pBufStart)
{
	char buf[WEBS_SOCKET_BUFSIZ+1];
    char bufBoundry[BOUNDRY_SIZE_BUF];
    unsigned long len;
	unsigned long lenFile;
	int rdBytes;
    int count;
	int action=0;
	int j=0;
    int i;
	int skip=4;
    char_t *pCurMem;

	
	count = wp->clen;

	*pBufStart = malloc(wp->clen);
	pCurMem = *pBufStart;
	while (count > 0) 
    {
		len = (wp->clen > WEBS_SOCKET_BUFSIZ) ? WEBS_SOCKET_BUFSIZ : wp->clen;
		cyg_thread_delay(2);
		rdBytes = socketRead(wp->sid, buf, len);
		if (rdBytes != 0) {
			count -= rdBytes;
			buf[rdBytes] = '\0';

				for(i=0;i<rdBytes;i++)
                {
					switch(action)
                    {
					case 0:
						if(buf[i] != DASH)
                        {
							bufBoundry[0]=buf[i];
							j=1;
							action=1;
						}
						break;

					case 1:
						if(isalnum(buf[i]))
                        {
							bufBoundry[j++]=buf[i];
                            break;
                        }
						bufBoundry[j]=0;
						action=2;
							j=0;
						break;

					case 2:
						if (buf[i]== LINE_FEED)
                        {
							skip-=1;
						if (skip==0)
                            {
							    action=3;
                            }
                        }
						break;

					case 3:
						*pCurMem++ = buf[i];
					    break;
					}
				}
			} 
	}
 
    /* We need to strip out the boundary pattern */
	if( (lenFile = webCgiStripBoundry(*pBufStart, pCurMem, bufBoundry)) > 0 )
    {
        return lenFile;
    }
		
    return -1;
}

/*
 *
 *	
 */
extern void upgrade(webs_t wp, char_t *path, char_t *query)
{
	char_t *pBufStart;
	unsigned long size;    

	if( (size = webCgiGetUploadFile(wp, &pBufStart) ) == -1 ) 
    {
        apUpgradeFail(wp, path, 0);
		return;
	}

    if(!ApUpgradeFirmware(pBufStart, size))
    {
        apUpgradeFail(wp, path, 1);
        return;
    }

    websRedirect(wp, T("reboot.asp")); 
	//apRequestReboot(wp, "Firmware Upgraded Successful");
}
