#include "wltypes.h"
#include "oem.h"
#include "IEEE_types.h"
#include "apcfg.h"
#include "flashdrvapi.h"
#include "mib.h"
#include "util.h"

#include "os_if.h"
#include "api.h"
#ifdef DOMAIN_SUPPORTED
#include "domain.h"
#endif

#ifdef NEWCALDATA
#include "hal_rfcal.h"
#include "wl_hal.h"
#endif

#ifdef VOIP_GUI
#include "hs_fwk_socket.h"
#include "hs_voip_sys.h"
#include "hs_voip_oam_if.h"
#include "hs_voip_oam_cli_if.h"
#include "hs_voip_oam_config.h"

#include "voip_config.h"
#endif

#ifdef ENABLE_UART
#include "shell.h"
#endif

#define WLAN_FILTER_CONFIGURED_KEY	0xdeadbeef

#if defined(GATEWAY) || defined(DHCPS)
#include "..\..\..\..\marvell\dhcps\incl\dhcps_api.h"
typedef struct MIB_IP_LAN_s
{
    unsigned long IPAddr;
    unsigned long SubnetMask;
}
MIB_IP_LAN;
#ifndef GATEWAY
MIB_IP_LAN mib_IP_LAN = {0xC0A8000A, 0xFFFFFF00} ;
#else
extern MIB_IP_LAN mib_IP_LAN;
#endif
#endif
UINT8 mib_version = MIB_AP_VERSION;
static UINT8 mib_error;

static UINT32 CRC32(unsigned char *buf, int len);

extern UINT32 macErpProtModeEnabled;
extern unsigned char LocalMacAddress[6]; 
extern unsigned char enaddr[6]; 
extern unsigned char enaddr2[6]; 

#ifdef AP5X
extern UINT8 WepType[];
extern UINT8 WepType_G[];
#else            
extern UINT32 WepType;
extern UINT32 WepType_G;
#endif

#ifndef AP21N
extern LOGIN login;
#endif

#ifndef NO_ECOS_TCP_IP_STACK
extern 
#endif
mib_DHCP_t mib_DHCP;

extern unsigned char WanMac[6];
extern unsigned char LanMac[6];
extern unsigned char gwLanMac[6];
extern unsigned char gwWanMac[6];
extern unsigned char WlanMac[6];

#ifdef AP_URPTR
extern UINT8  mib_wipodMode;
extern UINT8  mib_urMode;
extern UINT8  mib_wbMode;
extern UINT16 mib_urRsvd;
extern UINT8  mib_urSsid[33];
extern IEEEtypes_MacAddr_t  mib_urBssid;
extern UINT8 mib_urPrefBssid;
extern UINT8  mib_StaMode;
extern IEEEtypes_MacAddr_t brdcastAddr;
extern IEEEtypes_MacAddr_t macAddrZero;
extern UINT32 msi_wl_CheckURSupport(void);

#ifdef UR_WPA
extern UINT8   PSKValueUr[40];
#endif

#endif
 
int GetUserConfig(void);

static apfdDriver_t gFlashDrv;
static char userSettings[SIZEOF_MIB_USER_AP];
static int flashDriverInitialized = FALSE;
void apfdGetEtherAddr(unsigned char *etherAddr)
{
    unsigned char *flashmacptr;

    if (etherAddr == NULL) return ;

    flashmacptr = (unsigned char *)0xFFFFFFB0;
    if ( ((flashmacptr[0] == 0xff) &&
            (flashmacptr[1] == 0xff) &&
            (flashmacptr[2] == 0xff) &&
            (flashmacptr[3] == 0xff) &&
            (flashmacptr[4] == 0xff) &&
            (flashmacptr[5] == 0xff)) || 
            ((flashmacptr[0] == 0x0) &&
            (flashmacptr[1] == 0x0) &&
            (flashmacptr[2] == 0x0) &&
            (flashmacptr[3] == 0x0) &&
            (flashmacptr[4] == 0x0) &&
            (flashmacptr[5] == 0x0)) )
    {
        /* Keep default MAC address */
        return ;
    }    
    memcpy(enaddr, flashmacptr, 6); 
    memcpy(etherAddr, enaddr, 6);

       flashmacptr = (unsigned char *)0xFFFFFFB6;
    if ( ((flashmacptr[0] == 0xff) &&
            (flashmacptr[1] == 0xff) &&
            (flashmacptr[2] == 0xff) &&
            (flashmacptr[3] == 0xff) &&
            (flashmacptr[4] == 0xff) &&
            (flashmacptr[5] == 0xff)) || 
            ((flashmacptr[0] == 0x0) &&
            (flashmacptr[1] == 0x0) &&
            (flashmacptr[2] == 0x0) &&
            (flashmacptr[3] == 0x0) &&
            (flashmacptr[4] == 0x0) &&
            (flashmacptr[5] == 0x0)) )
    {
        /* Keep default MAC address */
	enaddr2[0]=0xff;
	enaddr2[1]=0xff;
	enaddr2[2]=0xff;
	enaddr2[3]=0xff;
	enaddr2[4]=0xff;
	enaddr2[5]=0xff;
        return ;
    }    
        memcpy(enaddr2, flashmacptr, 6); 


}


int GetDomainCode(void)
{
    {
#ifdef NEWCALDATA
        extern WL_SYS_CFG sSysCfg;
        extern UINT8 mfgRfChip;
        
    if ((msi_wl_rfCalGetHwDataRev(MFG_HW_DATA_ADDRESS) == 0x0B)	|| 
		(msi_wl_rfCalGetHwDataRev(MFG_HW_DATA_ADDRESS) == 0x0A))	
	{
	    if (mfgRfChip == MFG_RF_8030)
            return sSysCfg.mfgHwData->hwInfo->RegionCode;
        else
            return sSysCfg.mfgHwData->hwInfoA->CountryCode;
    }
    else
#endif
    return *(UINT8 *)(0xffffffa0);
    }
}
/**
 * Call OEM supplied function to initialize flash driver object
 */
void flashDrvInit(void)
{
    apfdDriverInit(&gFlashDrv);
    flashDriverInitialized = TRUE;
}

/**
 * Reset user settings to factory settings
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
int ResetToFactorySettings(void)
{
    int rc;

    if (!flashDriverInitialized)
    {
        return FALSE;
    }

    rc = (*gFlashDrv.ResetToFactorySettings)();

    GetUserConfig();

    return rc;
}

#ifdef ENABLE_UART
void dumpMIB(void)
{
    dbgprintf(DUMPMIB, "MAC address = %02x:%02x:%02x:%02x:%02x:%02x\n", enaddr[0], 
                enaddr[1], enaddr[2], enaddr[3], enaddr[4], enaddr[5]);
    dbgprintf(DUMPMIB, "DHCP %s\n", mib_dhcpenable? "enabled" : "disabled");
    dbgprintf(DUMPMIB, "IP address = %d.%d.%d.%d\n", (mib_DHCP.IPAddr >> 24), 
                ((mib_DHCP.IPAddr & 0xffffff) >> 16),
                ((mib_DHCP.IPAddr & 0xffff) >> 8),
                (mib_DHCP.IPAddr & 0xff));
    dbgprintf(DUMPMIB, "SSID \"%s\" Channel %d\n", mib_StaCfg_p->DesiredSsId, PhyDSSSTable_p->CurrChan);
}
#endif

/**
 * Get user settings from flash and populate the MIB
 * 
 * @return TRUE: success
 *         FALSE: failure
 */
int GetUserConfig(void)
{
    mib_AP_t *mib;
    mib_user_AP_t *mibUserPlus;
    int i;
    int ok;


    if (!flashDriverInitialized)
    {
        return FALSE;
    }

    memset(userSettings, 0, SIZEOF_MIB_USER_AP);
    mib = (mib_AP_t *)userSettings;
    mibUserPlus = (mib_user_AP_t *)userSettings;

    /* Read the user configuration from flash memory */
    ok = (*gFlashDrv.ReadUserSettings)(mib);

    if (!ok)
    {
        return FALSE;
    }

    if ( !((enaddr[0] == 0xff) &&
           (enaddr[1] == 0xff) &&
           (enaddr[2] == 0xff) &&
           (enaddr[3] == 0xff) &&
           (enaddr[4] == 0xff) &&
           (enaddr[5] == 0xff)) )
    {
        for ( i = 0; i < 6; ++i )
        {
            LocalMacAddress[i] = enaddr[i];
            mib_OpData_p->StaMacAddr[i] = enaddr[i];
            mib_currentmacaddress[i]    = enaddr[i];
        }

    }
    /* Make sure the version is correct */
    if (mib->mibVersion != mib_version)
    {
        mib_error = 1;
#ifndef GATEWAY
		mib_dhcpenable = 0;
    	mib_DHCP.IPAddr = 0xC0A8000A; /* 192.168.0.10 */
    	mib_DHCP.SubnetMask = 0xFFFFFF00; /* 255.255.255.0 */
    	mib_DHCP.GwyAddr =  0;
#endif
        return FALSE;
    }
    
 
    /* Populate the various MIB structures */
    util_CopyList(mib_StaCfg_p->DesiredSsId, mib->DesiredSsId, IEEEtypes_SSID_SIZE);
    
    for (i = 0; i< 16; i++)
        mib_StaCfg_p->OpRateSet[i] = mib->OpRateSet[i];
    mib_StaCfg_p->BcnPeriod = mib->BcnPeriod;
    mib_StaCfg_p->DtimPeriod = mib->DtimPeriod;
    mib_OpData_p->RtsThresh = mib->RtsThresh;
    mib_OpData_p->ShortRetryLim = mib->RtsRetryLim;
    mib_OpData_p->FragThresh = mib->FragThresh;
    util_CopyList(mib_StaCfg_p->DesiredSsId2, mib->DesiredSsId2, IEEEtypes_SSID_SIZE);

    PhyDSSSTable_p->CurrChan = mib->CurrRFChan;
    PhyDSSSTable_p->CurrChan2 = mib->CurrRFChan2;

    mib_StaCfg_p->PrivOption = mib->PrivOption;
    mib_PrivacyTable_p->PrivInvoked = mib->PrivInvoked;

    mib_AuthAlg_p->Enable = mib->AuthEnable;
    mib_AuthAlg_p->Type = mib->AuthType;
    mib_wep = mib->mib_wep;

    /* WEP configuration */
#ifdef AP5X
	memcpy(WepType, mib->WepType, 4);
#else    
    WepType = mib->WepType;
#endif    
    memcpy(mib_WepDefaultKeys_p[0].WepDefaultKeyValue, mib->WepDefaultKeyValue1, 13);
    memcpy(mib_WepDefaultKeys_p[1].WepDefaultKeyValue, mib->WepDefaultKeyValue2, 13);
    memcpy(mib_WepDefaultKeys_p[2].WepDefaultKeyValue, mib->WepDefaultKeyValue3, 13);
    memcpy(mib_WepDefaultKeys_p[3].WepDefaultKeyValue, mib->WepDefaultKeyValue4, 13);
    mib_defaultkeyindex = mib->DefaultWepKeyindex;

    mib_PhyTxPwrData_p->CurrTxPwrLevel = mib->CurrTxPwrLevel;
    mib_PhyTxPwrData_p->TxPwrLevel1 = mib->TxPwrLevel1;
    mib_PhyTxPwrData_p->TxPwrLevel2 = mib->TxPwrLevel2;
    mib_PhyTxPwrData_p->TxPwrLevel3 = mib->TxPwrLevel3;
    mib_PhyTxPwrData_p->TxPwrLevel4 = mib->TxPwrLevel4;
    mib_PhyTxPwrData_p->TxPwrLevel5 = mib->TxPwrLevel5;
    mib_PhyTxPwrData_p->TxPwrLevel6 = mib->TxPwrLevel6;
    mib_PhyTxPwrData_p->TxPwrLevel7 = mib->TxPwrLevel7;
    mib_PhyTxPwrData_p->TxPwrLevel8 = mib->TxPwrLevel8;

#ifndef PEREGRINE
    mib_txAntenna = mib->CurrTxAnt;
    mib_rxAntenna = mib->CurrRxAnt;
#else
    if(mib->CurrTxAnt == AP_ANT_A)
        mib_txAntenna = AP_ANT_B;
    else
        mib_txAntenna = AP_ANT_A;

    if(mib->CurrRxAnt == AP_ANT_A)
        mib_rxAntenna = AP_ANT_B;
    else
        mib_rxAntenna = AP_ANT_A;        
#endif

    mib_preAmble = mib->PreAmble;

    mib_broadcastssid = mib->broadcastssid;
    
	mib_dhcpenable = mib->EnableDHCP;
    mib_DHCP.IPAddr = mib->IPAddr;
    mib_DHCP.SubnetMask = mib->SubnetMask;
    mib_DHCP.GwyAddr = mib->GwyAddr;

#if defined(GATEWAY)
    mib_DHCP.IPAddr = mib->WanIPAddr;
    mib_DHCP.SubnetMask = mib->WanSubnetMask;
    mib_DHCP.GwyAddr = mib->WanGwyAddr;
    mib_DHCP.PrimaryDNS = mib->PrimaryDNS;
    mib_DHCP.SecondaryDNS = mib->SecondaryDNS;
#endif
#if defined(GATEWAY) || defined(DHCPS)
#ifndef	GATEWAY
    mib_IP_LAN.IPAddr = mib_DHCP.IPAddr;				
    mib_IP_LAN.SubnetMask = mib_DHCP.SubnetMask;
#else
    mib_IP_LAN.IPAddr = mib->LanIPAddr;				
    mib_IP_LAN.SubnetMask = mib->LanSubnetMask;
#endif
    /* DHCP Server */
	mib_DHCPS.DHCPSIsEnable = mibUserPlus->DHCPSIsEnable;
	mib_DHCPS.DHCPSRangeStartAddr = mibUserPlus->DHCPSRangeStartAddr;
	mib_DHCPS.DHCPSRangeEndAddr = mibUserPlus->DHCPSRangeEndAddr;
	mib_DHCPS.DHCPSLeaseTime = mibUserPlus->DHCPSLeaseTime;
    dhcpsEnable(mib_DHCPS.DHCPSIsEnable);
    if ( mib_IP_LAN.IPAddr )
	{
        dhcpsSetIpAddress(mib_IP_LAN.IPAddr, mib_IP_LAN.SubnetMask);
	}
	if((mib_DHCPS.DHCPSRangeStartAddr !=0) && (mib_DHCPS.DHCPSRangeStartAddr !=-1)
	   && (mib_DHCPS.DHCPSRangeEndAddr !=0) && (mib_DHCPS.DHCPSRangeEndAddr !=-1))	
	if((mib_IP_LAN.IPAddr & mib_IP_LAN.SubnetMask) == (mib_DHCPS.DHCPSRangeStartAddr & mib_IP_LAN.SubnetMask))
    	dhcpsSetRange(mib_DHCPS.DHCPSRangeStartAddr, mib_DHCPS.DHCPSRangeEndAddr);
    if((mib_DHCPS.DHCPSLeaseTime!= 0) && (mib_DHCPS.DHCPSLeaseTime!= -1))
    	dhcpsSetLeaseTime(mib_DHCPS.DHCPSLeaseTime);

    for (i = 0;
         mibUserPlus->DHCPSStationsTbl[i].IpAddr != 0 && mibUserPlus->DHCPSStationsTbl[i].IpAddr != -1
         && i < DHCPS_MAX_NUM_OF_STATIONS;
         i++)
    {
        dhcpsSetStation(mibUserPlus->DHCPSStationsTbl[i].IpAddr,
                        mibUserPlus->DHCPSStationsTbl[i].MacAddr,
                        FALSE);
    }

#endif

	mib_enableFixedRateTx = mib->EnableFixedRateTx;
    mib_txDataRate  = util_GetIndexByRate(mib->txDataRate);
    mib_txDataRateG = util_GetIndexByRate(mib->txDataRateG);
    mib_txDataRateA = util_GetIndexByRate(mib->txDataRateA);

#ifndef AP21N
#if 0
    for (i = 0; i < 32; ++i)
    {
        login.username[i] = mib->username[i];
        login.password[i] = mib->password[i];
    }
//    util_CopyList(mib_DeviceName, mib->devName, 16);
#endif
#endif
#ifndef AP_WPA2
    memcpy( mib_RSNConfig_p->PSKPassPhrase, mib->WPA_passphrase, 64 );
    memcpy(mib_RSNConfigAuthSuites_p->AuthSuites, mib->wpaMethod, 4);
    mib_PrivacyTable_p->RSNEnabled = mib->wpaMode;
#else
        mib_PrivacyTable_p->RSNEnabled = mib->wpaMode;
        mib_RSNConfig_p->GroupRekeyTime = mib->GrpRekeyTime;
        strcpy(mib_RSNConfig_p->PSKPassPhrase,mib->WPA_passphrase);
        
        strcpy( mib_RSNConfigWPA2_p->PSKPassPhrase, mib->WPA2_passphrase);
        memcpy(mib_RSNConfigAuthSuites_p->AuthSuites, mib->wpaMethod, 4);
        mib_RSNConfigWPA2_p->WPA2Enabled = mib->wpa2MixedMode;
        mib_RSNConfigWPA2_p->WPA2OnlyEnabled = mib->wpa2OnlyMode;
        mib_RSNConfigUnicastCiphers_p->UnicastCipher[3] = mib->wpaUcastCipher;
        mib_RSNConfigWPA2UnicastCiphers_p->UnicastCipher[3] = mib->wpa2UcastCipher;
        mib_RSNConfigWPA2UnicastCiphers2_p->UnicastCipher[3] = mib->wpa2UcastCipher2;
    
        if (mib_PrivacyTable_p->RSNEnabled && !mib_RSNConfigWPA2_p->WPA2Enabled && !mib_RSNConfigWPA2_p->WPA2OnlyEnabled)
        {
            mib_RSNConfig_p->MulticastCipher[3] = mib->wpaUcastCipher;
        }
        else if (mib_RSNConfigWPA2_p->WPA2Enabled)
        {
            if (mib->wpaUcastCipher == mib->wpa2UcastCipher && 
                mib->wpaUcastCipher == mib->wpa2UcastCipher2) 
            {
                mib_RSNConfig_p->MulticastCipher[3] = mib->wpaUcastCipher;
                mib_RSNConfigWPA2_p->MulticastCipher[3] = mib->wpaUcastCipher;
            }
            else
            {
                mib_RSNConfig_p->MulticastCipher[3] = 2;
                mib_RSNConfigWPA2_p->MulticastCipher[3] = 2;   // TKIP
            }
        }
        else if (mib_RSNConfigWPA2_p->WPA2OnlyEnabled)
        {
            if (mib->wpa2UcastCipher == mib->wpa2UcastCipher2) 
            {
                mib_RSNConfigWPA2_p->MulticastCipher[3] = mib->wpa2UcastCipher;
            }
            else
            {
                mib_RSNConfigWPA2_p->MulticastCipher[3] = 2;   // TKIP
            }
        }
        else
        {
            mib_RSNConfig_p->MulticastCipher[3] = 2;
            mib_RSNConfigWPA2_p->MulticastCipher[3] = 2;   // TKIP
        }

        mib_WPAPSKValueEnabled = mib->APWpaPSKValueEnabled;
        memcpy(mib_RSNConfig_p->PSKValue, mib->APWpaPSKValue, 32);
        mib_WPA2PSKValueEnabled = mib->APWpa2PSKValueEnabled;
        memcpy(mib_RSNConfigWPA2_p->PSKValue, mib->APWpa2PSKValue, 32);
#ifdef UR_WPA
        if ( (mib_RSNConfigWPA2_p->WPA2OnlyEnabled || mib_RSNConfigWPA2_p->WPA2Enabled)
             && mib_WPA2PSKValueEnabled
           )
            memcpy(PSKValueUr, mib->APWpa2PSKValue, 32);
        else if (mib_PrivacyTable_p->RSNEnabled && mib_WPAPSKValueEnabled)
            memcpy(PSKValueUr, mib->APWpaPSKValue, 32);
#endif

#endif


	memcpy(WanMac,mib->WanMacAddr,6);  
	memcpy(LanMac,mib->LanMacAddr,6);
	memcpy(gwWanMac,mib->GwyWanMacAddr,  6);
	memcpy(gwLanMac,mib->GwyLanMacAddr,  6);
	memcpy(WlanMac,mib->WlanMacAddr,  6);     
	if(mib->ApMode != 0xff)
		mib_ApMode = mib->ApMode; 
    if (mib_ApMode == AP_MODE_G_ONLY)
            {
    	macErpProtModeEnabled = FALSE;
	}
	else if(mib_ApMode == AP_MODE_A_ONLY  || mib_ApMode == AP_MODE_AandG)
            {
    	macErpProtModeEnabled = FALSE;
        }
        else
    	macErpProtModeEnabled = TRUE;
        
	mib_StaCfg_p->OpRateSet2[0]  =  2 | 0x80;
   	mib_StaCfg_p->OpRateSet2[1]  =  4 | 0x80;
	mib_StaCfg_p->OpRateSet2[2]  = 11 | 0x80;
	mib_StaCfg_p->OpRateSet2[3]  = 22 | 0x80;
	mib_StaCfg_p->OpRateSet2[4]  = 12 | 0x80;
	mib_StaCfg_p->OpRateSet2[5]  = 18;
	mib_StaCfg_p->OpRateSet2[6]  = 24 | 0x80;
	mib_StaCfg_p->OpRateSet2[7]  = 36;
	mib_StaCfg_p->OpRateSet2[8]  = 48 | 0x80;
	mib_StaCfg_p->OpRateSet2[9]  = 72;
	mib_StaCfg_p->OpRateSet2[10] = 96;
	mib_StaCfg_p->OpRateSet2[11] = 108;
	mib_StaCfg_p->OpRateSet2[12] = 0;
	
	mib_burstmode = mib->BurstMode;
	mib_burstrate = mib->BurstRate;
	mib_forceProtectiondisable=mib->forceprotection;
    mib_StaCfg_p->SelfCts = mib->SelfCts;

#ifdef WDS_FEATURE
    if(mibUserPlus->mib_wds_cfgkey==WDS_CONFIGURED_KEY)
    {
#ifndef PRESET_WDS_DYNAMIC_UNICAST
        mib_wdsEnable = mibUserPlus->mib_wds_Enable;
		mib_disableAssoc = mibUserPlus->mib_disableAssoc;
		mib_wdsBcastMode = mibUserPlus->mib_wdsBcastMode;
        mib_wdsno = mibUserPlus->mib_wds_no;
        memcpy(mib_wdsmac, mibUserPlus->mib_wds_mac, MAX_WDS_PORT*6);
#endif
    }
#endif // WDS_FEATURE

#ifdef AP_URPTR
#ifdef W8515
    if((mib->mib_urMode == 0 || mib->mib_urMode == 1) && msi_wl_CheckURSupport())
#else
    if(mib->mib_urMode == 0 || mib->mib_urMode == 1)
#endif
    {
        mib_urMode = mib->mib_urMode;
        if (mib->mib_wbMode == 0 || mib->mib_wbMode == 1)
            mib_wbMode = mib->mib_wbMode;
        else
            mib->mib_wbMode = 0;
        mib_urRsvd = mib->mib_urRsvd;
        util_CopyList(mib_urSsid, mib->mib_urSsid, IEEEtypes_SSID_SIZE);
        memcpy(mib_urBssid, mib->mib_urBssid, 6);
        if (!memcmp(mib_urBssid, brdcastAddr, sizeof(IEEEtypes_MacAddr_t)) ||
            !memcmp(mib_urBssid, macAddrZero, sizeof(IEEEtypes_MacAddr_t)))
        {
            mib_urPrefBssid = 0;
        }
        else
            mib_urPrefBssid = 1;
    }
    else
    {
        mib_urMode = 0;
        mib_urRsvd = 0;
        mib_urPrefBssid = 0;
    }
    switch(mib_ApMode)
    {
        case AP_MODE_B_ONLY:
            mib_StaMode = CLIENT_MODE_B;
            break;
        case AP_MODE_G_ONLY:
            mib_StaMode = CLIENT_MODE_G;
            break;
        case AP_MODE_MIXED:
            mib_StaMode = CLIENT_MODE_BG;
            break;
        case AP_MODE_A_ONLY:
            mib_StaMode = CLIENT_MODE_A;
            break;
        case AP_MODE_AandG:
            mib_StaMode = CLIENT_MODE_B;
            break;
        default:
            mib_StaMode = CLIENT_MODE_G;
            break;
    }
#endif /* AP_URPTR */
 
#ifdef QOS_FEATURE
	mib_StaCfg_p->QoSOptImpl = mib->mib_QoSOptImpl;
 
	mib_StaCfg_p->NoAckMode = mib->mib_NoAckMode;
#endif
#ifdef IEEE80211H
	mib_StaCfg_p->SpectrumManagementRequired=mib->mib_SpectrumManagementRequired;
#endif

#ifdef AG_MIXED_SECURITY
    if (mib_ApMode == AP_MODE_AandG)
    {
        memcpy(mib_WepDefaultKeys_G[0].WepDefaultKeyValue, mibUserPlus->WepDefaultKeyValue1_G, 13);
        memcpy(mib_WepDefaultKeys_G[1].WepDefaultKeyValue, mibUserPlus->WepDefaultKeyValue2_G, 13);
        memcpy(mib_WepDefaultKeys_G[2].WepDefaultKeyValue, mibUserPlus->WepDefaultKeyValue3_G, 13);
        memcpy(mib_WepDefaultKeys_G[3].WepDefaultKeyValue, mibUserPlus->WepDefaultKeyValue4_G, 13);
        if ((mibUserPlus->DefaultWepKeyindex_G == 0xFF) || (mibUserPlus->AuthEnable_G == 0xFF) 
#ifdef AP5X
             || (mibUserPlus->WepType_G[0] == 0xFF) 
#else             
             || (mibUserPlus->WepType_G == 0xFF) 
#endif
             || (mibUserPlus->PrivInvoked_G == 0xFF))
        {
            mib_PrivacyTable_p->PrivInvoked_G = 0;
            mib_AuthAlg_G->Enable = 0;
            mib_AuthAlg_G->Type = AUTH_OPEN_SYSTEM;
            mib_defaultkeyindex_G = 0;
#ifdef AP5X
            memset(WepType_G, 0, 4);
#else            
            WepType_G = 0;
#endif
    }
        else
        {
            mib_PrivacyTable_p->PrivInvoked_G = mibUserPlus->PrivInvoked_G;
            mib_AuthAlg_G->Enable = mibUserPlus->AuthEnable_G;
            mib_AuthAlg_G->Type = mibUserPlus->AuthType_G;
            mib_defaultkeyindex_G = mibUserPlus->DefaultWepKeyindex_G;
#ifdef AP5X
            memecpy(WepType_G, mibUserPlus->WepType_G, 4);
#else
            WepType_G = mibUserPlus->WepType_G;
#endif
        }
    }
#endif

	if(mibUserPlus->mib_wlanfilter_cfgkey==WLAN_FILTER_CONFIGURED_KEY)
	{
		mib_wlanfilterno = mibUserPlus->mib_wlanfilter_no;
		mib_wlanfiltertype = mibUserPlus->mib_wlanfilter_mode;
		memcpy(mib_wlanfiltermac, mibUserPlus->mib_wlanfilter_mac, 256*6);
	}

#ifdef AUTOLINK
	if (mibUserPlus->ALSecMode == 0xFF)
    	mib_ALSecMode = 1; /*Default to WPA-TKIP*/
    else
        mib_ALSecMode = mibUserPlus->ALSecMode;
    
	mib_ALStatus = mibUserPlus->ALStatus;

	memcpy(mib_ALOperationalKey, mibUserPlus->ALOperationalKey, 16);

	if (mibUserPlus->ALLowPowerEnable == 0xFF)
		mib_ALLowPowerEnable = FALSE;
	else	
		mib_ALLowPowerEnable = mibUserPlus->ALLowPowerEnable;

	if (mibUserPlus->ALEnable == 0xFF)
		mib_ALEnable = FALSE;
	else
		mib_ALEnable = mibUserPlus->ALEnable;

	if (mibUserPlus->ALONTime == 0xFFFFFFFF)
		mib_ALONTime = 60;
	else	
		mib_ALONTime = mibUserPlus->ALONTime;

	if (mibUserPlus->ALPPEnable == 0xFF)
		mib_ALPPEnable = FALSE;
	else
		mib_ALPPEnable = mibUserPlus->ALPPEnable;
#endif

#ifdef DCS
	if (mibUserPlus->dcsMode == 0xFF)
		mib_dcsMode = FALSE;
	else
		mib_dcsMode = mibUserPlus->dcsMode;
#endif
   	mib_AgingTime = ((mibUserPlus->AgingTime == 0xFF) ? 3 : mibUserPlus->AgingTime);

#ifdef GUEST_ACCESS
    if (mibUserPlus->mib_GuestAccessEnable == 0xFF)
    {
        mib_GuestAccessEnable = false;
        sprintf(mib_GuestAccessPassPhrase, "");
        sprintf(mib_GuestAccessPassPhrase2, "");
    }
    else
    {
        mib_GuestAccessEnable = mibUserPlus->mib_GuestAccessEnable;
        memcpy(mib_GuestAccessPassPhrase, mibUserPlus->mib_GuestAccessPassPhrase, 64);
        memcpy(mib_GuestAccessPassPhrase2, mibUserPlus->mib_GuestAccessPassPhrase2, 64);
    }
#endif

    return TRUE;
}

/**
 * Gather the MIB settings and save to flash as user settings
 * 
 * @return TRUE: success
 *         FALSE: error
 */
int SetUserConfig(void)
{
    mib_AP_t *mib;
    mib_user_AP_t *mibUserPlus;
    int i;
    int rc;
    int ok;
    unsigned long crc32;

#if defined(GATEWAY) || defined(DHCPS)
    unsigned long numOfStations;
    DHCPS_STATION_PROPERTIES stationsTbl[DHCPS_MAX_NUM_OF_STATIONS];
#endif
    if (!flashDriverInitialized)
    {
        return FALSE;
    }

    memset(userSettings, 0xff, SIZEOF_MIB_USER_AP);
    mib = (mib_AP_t *)userSettings;
    mibUserPlus = (mib_user_AP_t *)userSettings;

    /* Read the user configuration from flash memory */
    ok = (*gFlashDrv.ReadUserSettings)(mib);

    if (!ok)
    {
        return FALSE;
    }

    crc32 = CRC32((unsigned char *)mibUserPlus, SIZEOF_MIB_USER_AP);

    mib->mibVersion = mib_version;
#ifdef AP_WPA2
    memcpy(mib->wpaMethod, mib_RSNConfigAuthSuites_p->AuthSuites, 4);
    mib->wpaMode = mib_PrivacyTable_p->RSNEnabled;
    mib->GrpRekeyTime = mib_RSNConfig_p->GroupRekeyTime;
    mib->wpa2MixedMode = mib_RSNConfigWPA2_p->WPA2Enabled;
    mib->wpa2OnlyMode = mib_RSNConfigWPA2_p->WPA2OnlyEnabled;
    mib->wpaUcastCipher = mib_RSNConfigUnicastCiphers_p->UnicastCipher[3];
    mib->wpa2UcastCipher = mib_RSNConfigWPA2UnicastCiphers_p->UnicastCipher[3];
    mib->wpa2UcastCipher2 = mib_RSNConfigWPA2UnicastCiphers2_p->UnicastCipher[3];
    strcpy(mib->WPA_passphrase, mib_RSNConfig_p->PSKPassPhrase);
    strcpy(mib->WPA2_passphrase, mib_RSNConfigWPA2_p->PSKPassPhrase);

    mib->APWpaPSKValueEnabled = mib_WPAPSKValueEnabled;
    memcpy(mib->APWpaPSKValue, mib_RSNConfig_p->PSKValue, 32);
    mib->APWpa2PSKValueEnabled = mib_WPA2PSKValueEnabled;
    memcpy(mib->APWpa2PSKValue, mib_RSNConfigWPA2_p->PSKValue, 32);
#else
    /*WPA*/
    memcpy( mib->WPA_passphrase, mib_RSNConfig_p->PSKPassPhrase, 64 );
    memcpy(mib->wpaMethod, mib_RSNConfigAuthSuites_p->AuthSuites, 4);
    //mibUser->RSNEnabled = mib_PrivacyTable_p->RSNEnabled;
    mib->GrpRekeyTime = mib_RSNConfig_p->GroupRekeyTime;
    strcpy(mib->WPA_passphrase, mib_RSNConfig_p->PSKPassPhrase);
    mibUserPlus->WPA_encr_algo = mib_RSNConfigUnicastCiphers_p->UnicastCipher[3];
#endif

    if(mib_PrivacyTable_p->ApiUpdateWpa == TRUE)
    {
    	mib->wpaMode=mib_PrivacyTable_p->ApiUpdateWpa;
        os_EnterCriticalSection;                
        mib_PrivacyTable_p->ApiUpdateWpa = FALSE;
        os_ExitCriticalSection;        		
	} 
	else
	{           
    	mib->wpaMode = mib_PrivacyTable_p->RSNEnabled;
    }

    util_CopyList(mib->DesiredSsId, mib_StaCfg_p->DesiredSsId, IEEEtypes_SSID_SIZE);
    for (i = 0; i < 16; i++)
        mib->OpRateSet[i] = mib_StaCfg_p->OpRateSet[i];

    mib->BcnPeriod = mib_StaCfg_p->BcnPeriod;
    mib->DtimPeriod = mib_StaCfg_p->DtimPeriod;
    mib->RtsThresh = mib_OpData_p->RtsThresh;
    mib->RtsRetryLim = mib_OpData_p->ShortRetryLim;
    mib->FragThresh = mib_OpData_p->FragThresh;
    mib->CurrRFChan = PhyDSSSTable_p->CurrChan;

    util_CopyList(mib->DesiredSsId2, mib_StaCfg_p->DesiredSsId2, IEEEtypes_SSID_SIZE);
    mib->CurrRFChan2 = PhyDSSSTable_p->CurrChan2;
#ifdef AP5X
    memcpy(mib->WepType, WepType, 4);
#else
    mib->WepType = WepType;
#endif
    mib->PrivOption = mib_StaCfg_p->PrivOption;
    mib->PrivInvoked = mib_PrivacyTable_p->PrivInvoked;

    mib->AuthEnable = mib_AuthAlg_p->Enable;
    mib->AuthType = mib_AuthAlg_p->Type;
    mib->mib_wep = mib_wep;
    memcpy(mib->WepDefaultKeyValue1, mib_WepDefaultKeys_p[0].WepDefaultKeyValue, 13);
    memcpy(mib->WepDefaultKeyValue2, mib_WepDefaultKeys_p[1].WepDefaultKeyValue, 13);
    memcpy(mib->WepDefaultKeyValue3, mib_WepDefaultKeys_p[2].WepDefaultKeyValue, 13);
    memcpy(mib->WepDefaultKeyValue4, mib_WepDefaultKeys_p[3].WepDefaultKeyValue, 13);
    mib->DefaultWepKeyindex = mib_defaultkeyindex;

    mib->CurrTxPwrLevel = mib_PhyTxPwrData_p->CurrTxPwrLevel;
    mib->TxPwrLevel1 = mib_PhyTxPwrData_p->TxPwrLevel1;
    mib->TxPwrLevel2 = mib_PhyTxPwrData_p->TxPwrLevel2;
    mib->TxPwrLevel3 = mib_PhyTxPwrData_p->TxPwrLevel3;
    mib->TxPwrLevel4 = mib_PhyTxPwrData_p->TxPwrLevel4;
    mib->TxPwrLevel5 = mib_PhyTxPwrData_p->TxPwrLevel5;
    mib->TxPwrLevel6 = mib_PhyTxPwrData_p->TxPwrLevel6;
    mib->TxPwrLevel7 = mib_PhyTxPwrData_p->TxPwrLevel7;
    mib->TxPwrLevel8 = mib_PhyTxPwrData_p->TxPwrLevel8;

#ifndef PEREGRINE
    mib->CurrTxAnt = mib_txAntenna;
    mib->CurrRxAnt = mib_rxAntenna;
#else
    if(mib_txAntenna == AP_ANT_A)
        mib->CurrTxAnt = AP_ANT_B;
    else
        mib->CurrTxAnt = AP_ANT_A;

    if(mib_rxAntenna == AP_ANT_A)
        mib->CurrRxAnt = AP_ANT_B;
    else
        mib->CurrRxAnt = AP_ANT_A;
#endif


    mib->PreAmble = mib_preAmble;

    mib->broadcastssid = mib_broadcastssid;
    if ((!mib_dhcpenable) || mib_error)
    {
#if defined(GATEWAY)
        if (mib_DHCP.IPAddr)
        {
            mib->WanIPAddr = mib_DHCP.IPAddr;
        }
        if (mib_DHCP.SubnetMask)
        {
            mib->WanSubnetMask = mib_DHCP.SubnetMask;
        }
        mib->WanGwyAddr = mib_DHCP.GwyAddr;
        mib->PrimaryDNS = mib_DHCP.PrimaryDNS;
        mib->SecondaryDNS = mib_DHCP.SecondaryDNS;
#else
        if (mib_DHCP.IPAddr)
        {
            mib->IPAddr = mib_DHCP.IPAddr;
        }
        if (mib_DHCP.SubnetMask)
        {
            mib->SubnetMask = mib_DHCP.SubnetMask;
        }
        mib->GwyAddr = mib_DHCP.GwyAddr;
#endif
    }

#ifdef AP21N
    //we don't know what caused AP21N to mib->IPAddr and SubnetMask been reset to 0
    //so we set to AP21N default IP and Submask
    if (!mib->IPAddr)
        mib->IPAddr = 0xC0A80001;
    if (!mib->SubnetMask)
        mib->SubnetMask = 0xFFFFFF00;
#endif

#if defined(GATEWAY)  || defined(DHCPS)
    mib->LanIPAddr = mib_IP_LAN.IPAddr;
    mib->LanSubnetMask = mib_IP_LAN.SubnetMask;
#endif
    mib->EnableDHCP = mib_dhcpenable;

	mib->EnableFixedRateTx = mib_enableFixedRateTx;
   	mib->txDataRate  = util_GetRateByIndex(mib_txDataRate);
   	mib->txDataRateG = util_GetRateByIndex(mib_txDataRateG);
   	mib->txDataRateA = util_GetRateByIndex(mib_txDataRateA);

    for (i = 0; i < 6; ++i)
    {
        mib->EthMACAddr[i] = mib_currentmacaddress[i];
        mib->RFMACAddr[i] = mib_OpData_p->StaMacAddr[i];
    }
	memcpy(mib->WanMacAddr, WanMac, 6);  
	memcpy(mib->LanMacAddr, LanMac, 6);
	memcpy(mib->GwyWanMacAddr, gwWanMac, 6);
	memcpy(mib->GwyLanMacAddr, gwLanMac, 6);
	memcpy(mib->WlanMacAddr, WlanMac, 6);  

    mib->ApMode=mib_ApMode;
	mib->BurstMode = mib_burstmode;
	mib->BurstRate = mib_burstrate;	
	mib->forceprotection = mib_forceProtectiondisable;
    mib->SelfCts = mib_StaCfg_p->SelfCts;
 
#if 0
#ifndef AP21N
    for (i = 0; i < 32; ++i)
    {
        mib->username[i] = login.username[i];
        mib->password[i] = login.password[i];
    }
//    util_CopyList(mib->devName, mib_DeviceName,  16);
#endif
#endif
#ifdef WDS_FEATURE
    mibUserPlus->mib_wds_cfgkey = WDS_CONFIGURED_KEY;
    mibUserPlus->mib_wds_Enable = mib_wdsEnable;
	mibUserPlus->mib_disableAssoc = mib_disableAssoc;
	mibUserPlus->mib_wdsBcastMode = mib_wdsBcastMode;
    mibUserPlus->mib_wds_no = mib_wdsno;
    memcpy(mibUserPlus->mib_wds_mac, mib_wdsmac, MAX_WDS_PORT*6);
#endif // WDS_FEATURE

#ifdef AP_URPTR
    mib->mib_urMode = mib_urMode;
    mib->mib_wbMode = mib_wbMode;
    mib->mib_urRsvd = mib_urRsvd;
    util_CopyList(mib->mib_urSsid, mib_urSsid, IEEEtypes_SSID_SIZE);
    memcpy(mib->mib_urBssid, mib_urBssid,  6);
#endif

#ifdef QOS_FEATURE
	mib->mib_QoSOptImpl = mib_StaCfg_p->QoSOptImpl;

    mib->mib_NoAckMode = mib_StaCfg_p->NoAckMode;
#endif
#ifdef IEEE80211H

	mib->mib_SpectrumManagementRequired=mib_StaCfg_p->SpectrumManagementRequired;

#endif
#if defined(GATEWAY) || defined(DHCPS)

    /* DHCP Server */

    mibUserPlus->DHCPSIsEnable = dhcpsIsEnable();
    dhcpsGetRange(&(mibUserPlus->DHCPSRangeStartAddr), &(mibUserPlus->DHCPSRangeEndAddr));
    dhcpsGetLeaseTime(&(mibUserPlus->DHCPSLeaseTime));

    dhcpsGetBulkStations(&numOfStations, stationsTbl);

    memset(mibUserPlus->DHCPSStationsTbl, 0, sizeof(mibUserPlus->DHCPSStationsTbl));

    for (i = 0; i < numOfStations; i++)
    {
        if (stationsTbl[i].st_type == DHCPS_STATIC_ENTRY)
        {
            mibUserPlus->DHCPSStationsTbl[i].IpAddr  = stationsTbl[i].ip_address;
            memcpy(mibUserPlus->DHCPSStationsTbl[i].MacAddr, stationsTbl[i].mac_addr, 6);
        }
    }

#endif

#ifdef AG_MIXED_SECURITY
    if (mib_ApMode == AP_MODE_AandG)
        {
	    memcpy(mibUserPlus->WepDefaultKeyValue1_G, mib_WepDefaultKeys_G[0].WepDefaultKeyValue, 13);
	    memcpy(mibUserPlus->WepDefaultKeyValue2_G, mib_WepDefaultKeys_G[1].WepDefaultKeyValue, 13);
	    memcpy(mibUserPlus->WepDefaultKeyValue3_G, mib_WepDefaultKeys_G[2].WepDefaultKeyValue, 13);
	    memcpy(mibUserPlus->WepDefaultKeyValue4_G, mib_WepDefaultKeys_G[3].WepDefaultKeyValue, 13);
	    mibUserPlus->PrivInvoked_G = mib_PrivacyTable_p->PrivInvoked_G;
	    mibUserPlus->AuthEnable_G = mib_AuthAlg_G->Enable;
	    mibUserPlus->AuthType_G = mib_AuthAlg_G->Type;
#ifdef AP5X
	    memcpy(mibUserPlus->WepType_G, WepType_G, 4);
#else
	    mibUserPlus->WepType_G = WepType_G;
#endif
	    mibUserPlus->DefaultWepKeyindex_G = mib_defaultkeyindex_G;
    }
#endif
	mibUserPlus->mib_wlanfilter_cfgkey = WLAN_FILTER_CONFIGURED_KEY;
	mibUserPlus->mib_wlanfilter_no = mib_wlanfilterno; 
	mibUserPlus->mib_wlanfilter_mode = mib_wlanfiltertype;
    memcpy(mibUserPlus->mib_wlanfilter_mac, mib_wlanfiltermac, 256*6);

#ifdef AUTOLINK
    mibUserPlus->ALSecMode = mib_ALSecMode;
	mibUserPlus->ALStatus = mib_ALStatus;
	memcpy(mibUserPlus->ALOperationalKey, mib_ALOperationalKey, 16);
	mibUserPlus->ALLowPowerEnable = mib_ALLowPowerEnable;
	mibUserPlus->ALEnable = mib_ALEnable;
	mibUserPlus->ALONTime = mib_ALONTime;
	mibUserPlus->ALPPEnable = mib_ALPPEnable;
#endif

#ifdef DCS
	mibUserPlus->dcsMode = mib_dcsMode;
#endif
	mibUserPlus->AgingTime = mib_AgingTime;

#ifdef GUEST_ACCESS
    mibUserPlus->mib_GuestAccessEnable = mib_GuestAccessEnable;
    memcpy(mibUserPlus->mib_GuestAccessPassPhrase, mib_GuestAccessPassPhrase, 64);
    memcpy(mibUserPlus->mib_GuestAccessPassPhrase2, mib_GuestAccessPassPhrase2, 64);
#endif

    if (crc32 != CRC32(mibUserPlus, SIZEOF_MIB_USER_AP))
    rc = (*gFlashDrv.WriteUserSettings)(mib);
    return rc;
}

int FirmwareUpgrade(char *pData, unsigned long len)
{
    if (!flashDriverInitialized)
    {
        return FALSE;
    }
    return (*gFlashDrv.UpgradeFirmware)(pData, len);
}

int userConfigModifiedCnt = 0;
int UserConfigModified(void)
{
    return (++userConfigModifiedCnt);
}

int SaveUserConfig(void)
{
        return (SetUserConfig());
}
extern unsigned long crc32_table[];
#define SWAP(a,b) do { unsigned char  t = S[a]; S[a] = S[b]; S[b] = t; } while(0)
static unsigned char S[256] = {0};
static UINT32 CRC32(unsigned char *buf, int len)
{

    unsigned long i, j, k, crc;
    unsigned char  *pos;
    pos = buf;

    /* Compute CRC32 */
    crc = ~0;
    i = j = 0;
    for (k = 0; k < len; k++)
    {
        crc = crc32_table[(crc ^ *pos) & 0xff] ^ (crc >> 8);
        i = (i + 1) & 0xff;
        j = (j + S[i]) & 0xff;
        SWAP(i, j);
        *pos++ ^= S[(S[i] + S[j]) & 0xff];
    }
    crc = ~crc;

    return crc;
}

#ifdef VOIP_GUI

int VOIP_config_set(VOIP_config_struct *config)
{
	//diag_printf("VOIP_config_set()\n");

	return (*gFlashDrv.StoreVOIPConfig)((char *)config, sizeof(VOIP_config_struct));
}
int VOIP_config_get(VOIP_config_struct *config)
{
 	int ret_val;

 	//diag_printf("VOIP_config_get()\n");

 	ret_val = (*gFlashDrv.ReadVOIPConfig)((char *)config, sizeof(VOIP_config_struct));
	if( !ret_val )
	{	// Get default values for configurable parameters
		VOIP_params_defaultCfg(config);
	}
	// Get default values for non-configurable parameters from source code.
	VOIP_params_defaultSrcCode(config);

	return ret_val;
}
#endif
