#include "wltypes.h"
#include "IEEE_types.h"
#include "oem.h"
#include "mib.h" 
#include "api.h"

extern void Bridge_Init(void);
extern void Bridge_Start(void);
 
unsigned char LocalMacAddress[6];
extern mib_DHCP_t mib_DHCP;
extern void etherSwitch_Init( int wanPort );
extern int AP_Init(BOOLEAN);
extern void MfgInit(void);
extern Status_e ApCmdSetProc(UINT16 CmdCode, char *pData);

#ifdef AUTOLINK
#include "autoLink.h"
#include "eth_debug.h"
/*Imports*/
extern UINT8  mib_urMode;
extern void AutoLinkShutOff(void);
extern AutoLinkButtonEvent(void);
#endif /*AUTOLINK*/

/*Prototypes*/
#if defined(AP_URAUTOCFG) || defined(AUTOLINK)
void UtilButtonHandler(ApEvtMsg_t *msg);
#endif

#ifdef MLME_CALLBACK
extern void (*mlmeCallBack_p)(IEEEtypes_MacAddr_t *staAddr, int evt);
void staStateChangedEvent(IEEEtypes_MacAddr_t *sa_ptr, int evt);
#endif

/* Default RF channel */
unsigned char OEMCH = 11;

/* Default SSID name */
unsigned char OEMNAME[] = {"Marvell-AP3X"};

/* Notes: RF-Cal module needs the socket library. The AP22_install ecos package */
/* included in this package does not have TCP/IP and socket library, so we cannot */
/* include MFG task */
#define INCLUDE_RFCAL 1

#include "usrcfg.h"

usr_Cfg_t usrCfg = 
{
   0xFFFFFD00, 5, 6
};

void usr_Cfg_Init(void)
{
#ifdef NEWCALDATA	
	/* This is the new cal struct location for 8030 and 8010 */
	usrCfg.uc_mfgDataLocation = 0xFFFFF000;
#else
    usrCfg.uc_mfgDataLocation = 0xFFFFFD00;
#endif

    usrCfg.uc_gpioSwResetPin = 5;
    usrCfg.uc_gpioUtilPin = 6;
}

/**
 * Initialize hardware and create all the tasks for operating the AP
 */
static void config(void)
{
    systemCfg.prod = PROD_NAME;
	if(systemCfg.prod == AP51)
    {
        #ifdef AP51E
    	systemCfg.feif = EPHY_IF;
        #else
    	systemCfg.feif = PHY_IF;
        #endif
    }
    if(systemCfg.prod == AP52)
    	systemCfg.feif = MII_IF;
}

#ifdef RESET_BUTTON_EVENT
void ResetButtonHandler(ApEvtMsg_t *msg)
{
    static int start_rb = 0;
    static long then_rb;

    if (!start_rb)
    {
        /* Ignore spurious interrupt at reboot */
        if(msg->evtTimeStamp < 3) {
            return;
        }
        start_rb = 1;
        then_rb = msg->evtTimeStamp;
    }
    else
    {
        if ((msg->evtTimeStamp - then_rb) > (5*83)) {
            ResetToFactorySettings();
            Reset();
        }
        else
        {
            Reset();
        }
        start_rb = 0;
        then_rb = msg->evtTimeStamp;
    }    
}                    
#endif

void root( void )
{	
    LocalMacAddress[0] = 0;
    LocalMacAddress[1] = 0x50;
    LocalMacAddress[2] = 0x43;
    LocalMacAddress[3] = 0x03;
    LocalMacAddress[4] = 0x05;
    LocalMacAddress[5] = 0x16;
	usr_Cfg_Init();
    config();
    flashDrvInit();

#if defined(GATEWAY) || defined(DHCPS)
    dhcpsDbInit();
#endif
    
    AP_Init(TRUE);
    if(systemCfg.feif == MII_IF)
        etherSwitch_Init(0/*WAN_PORT*/);
    
#ifdef GATEWAY
    FR_Main_Init();
#endif
#ifdef LED_SUPPORT
    GPIOInit();
#endif
#ifndef NO_ECOS_TCP_IP_STACK
    #ifdef TASKDHCP
        dhcp_start_dhcp_mgt_thread();
    #else
        init_static_network_interfaces(1);
    #endif
#endif
        Bridge_Init();
        Bridge_Start();
#ifndef NO_ECOS_TCP_IP_STACK
        web_Init();
        web_Start();
#endif

#ifndef NO_ECOS_TCP_IP_STACK
#ifdef INCLUDE_RFCAL
    MfgInit();
#endif
#endif    
#ifndef NO_ECOS_TCP_IP_STACK
    winsInit();
    winxStart();
#endif
#if defined(GATEWAY) || defined(DHCPS)
    dhcpsInit();
#endif

#ifdef RESET_BUTTON_EVENT
    {
    unsigned int SubID;
    apEvtSub(APEVT_RST_BTN_PRESSED,(void *)&ResetButtonHandler, &SubID);
    }
#endif

#if (defined(AP_URAUTOCFG) && defined(UTIL_AUTOCFG))
    {
    unsigned int SubID1;
    apEvtSub(APEVT_UTIL_BTN_PRESSED,(void *)&UtilButtonHandler, &SubID1);
    }
#elif (defined(AUTOLINK) && defined(UTIL_AUTOLINK))
    {
    unsigned int SubID1;
    apEvtSub(APEVT_UTIL_BTN_PRESSED,(void *)&UtilButtonHandler, &SubID1);
    }
#endif
#ifdef MLME_CALLBACK
    /* ---- Initialize the callback function for STA's state changes ---- */
    /* ---- mlmeCallBack_p is NULL if not initialized to any routine ---- */
    mlmeCallBack_p = staStateChangedEvent;
#endif
}

/**
 * Called by real-time kernel to start application tasks
 */
extern void cyg_user_start( void )
{
    root();
}

void BSS_Start(void)
{
#ifdef NO_ECOS_TCP_IP_STACK
    APICMDBUF *pBuf;
	pBuf = malloc(sizeof(APICMDBUF));
    if (pBuf)
    {
	    pBuf->cmdCode = APCMD_BSS_START;
	    pBuf->seqNum = 1;
	    pBuf->cmdBodyLen = 0;
	    apctl(pBuf);
	    free(pBuf); 
    }
#else /* When EXCLUDE_WEB is defined, http is not compiled in */
    ApCmdSetProc(APCMD_BSS_START, NULL);
#endif
}

void perror(const char * str)
{}

void GetDebugInfo(unsigned char *str)
{
#ifndef NO_ECOS_TCP_IP_STACK
    sprintf(str, "IP Address = %d.%d.%d.%d A-SSID %s G-SSID %s\n", (mib_DHCP.IPAddr >> 24), 
                ( (mib_DHCP.IPAddr & 0xffffff) >> 16),
                ( (mib_DHCP.IPAddr & 0xffff) >> 8),
                (mib_DHCP.IPAddr & 0xff), mib_StaCfg_p->DesiredSsId, mib_StaCfg_p->DesiredSsId2 );
#else
    sprintf(str, "NO IP Stack Install\n");
#endif
}

#if defined(AP_URAUTOCFG) || defined(AUTOLINK)
extern UINT8 mib_urMode;

extern int urAutoCfg( void (*cbSsid)(char *ssid, int ssidLen, int channel) );

/* Customer specific GPIO PIN */
#define GPIO_UR_LINK  0 /* Pls change this to 8 for customer board! */

/* This sample callback function informed user about 
   UR link status; for LED etc */
void urLinkStatusInd_test(int isConnected)
{
    /* If non-zero, it is connected */
    apctlGPIOWrite(GPIO_UR_LINK, !isConnected);
}

void urCfgStatusInd(char *ssid, int ssidLen, int channel)
{
    extern UINT8  mib_urSsid[33];
    char dummy = 0;

    memset(mib_urSsid,0,IEEEtypes_SSID_SIZE);
    util_CopyList(mib_urSsid, ssid, ssidLen);

    /* Sample code to use same AP ssid as UR */
    ApCmdSetProc(APCMD_SET_SSID, mib_urSsid);
    ApCmdSetProc(APCMD_BSS_START, &dummy);
}

void UtilButtonHandler(ApEvtMsg_t *msg)
{
    static int start_ub = 0;
    static long then_ub;

    if (!start_ub)
    {
        /* Ignore spurious interrupt at reboot */
        if(msg->evtTimeStamp < 3) 
		{
            return;
        }
        start_ub = 1;
        then_ub = msg->evtTimeStamp;
    }
    else
    {
#if (defined(AUTOLINK) && defined(UTIL_AUTOLINK))
		if (mib_ALEnable == TRUE)
		{
			AutoLinkButtonEvent();
		}
#elif (defined(AP_URAUTOCFG) && defined(UTIL_AUTOCFG))
        if( mib_urMode == 1 )
        {
           urAutoCfg(urCfgStatusInd);
        }
#endif 
        start_ub = 0;
        then_ub = msg->evtTimeStamp;
    }    
	
	return;	
}                    
#endif /*AP_URAUTOCFG || AUTOLINK*/
