/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    packet.c
 * \brief   Sample packets processing routines
 */
#include "wltypes.h"
#include "oem.h"
#include "IEEE_types.h" 
#include "mib.h" 
#include "api.h"
#include "packet.h"
#include "ds.h"
#include "StaDb.h"
#include "eth_debug.h"

#ifdef AUTOLINK
extern Timer ALInitiateTimer;
extern void AutoLinkInitiate(void);
extern UINT8 mib_ALStatus;
extern UINT8  *g_pAssociatedFlag;
#endif

#ifdef LED_SUPPORT
#if PROD_NAME == AP52
#define GPIO_WLAN_TX 0
#define GPIO_WLAN_RX 1
#else
#define GPIO_WLAN_TX 2
#define GPIO_WLAN_RX 2
#define GPIO_WLAN_RX_TO_WLAN_TX 10
#endif
#define GPIO_LAN_TX  3
#define GPIO_LAN_RX  3
#define GPIO_UR_LINK_STATUS	 0
#define GPIO_LAN_LINK_STATUS 1

#ifdef AP_URPTR
extern UINT8 mib_wbMode;
#endif

void GPIOInit(void)
{
#if PROD_NAME == AP51
    apctlGPIOInit(~((1<<GPIO_WLAN_TX)|
                    (1<<GPIO_WLAN_RX)|
                    (1<<GPIO_WLAN_RX_TO_WLAN_TX)|
                    (1<<GPIO_LAN_TX)|
                    (1<<GPIO_LAN_RX)|
                    (1<<GPIO_UR_LINK_STATUS)|
                    (1<<GPIO_LAN_LINK_STATUS))); /* LED and SW Reset */
#else
    apctlGPIOInit(~((1<<GPIO_WLAN_TX)|(1<<GPIO_WLAN_RX)));
#endif
}
void LEDoff(void)
{
     apctlSwLedDisable(GPIO_WLAN_TX);
     apctlSwLedDisable(GPIO_WLAN_RX);
#if PROD_NAME == AP51
     apctlSwLedDisable(GPIO_WLAN_RX_TO_WLAN_TX);
     apctlSwLedDisable(GPIO_LAN_TX );
     apctlSwLedDisable(GPIO_LAN_RX );
#endif
}

void LanLinkStatusInd(UINT8 isConnected)
{
    apctlGPIOWrite(GPIO_LAN_LINK_STATUS, !isConnected);
}
#ifdef AP_URPTR_LED
void urLinkStatusInd(int isConnected)
{
#ifdef AUTOLINK
	/* If AutoLink is turned ON then sent AutoLink Initiate message */
	if (mib_ALStatus == 0x02 && ((*g_pAssociatedFlag) != 0) && (isConnected))
	{
		TimerFireEvery(&ALInitiateTimer, 1, &AutoLinkInitiate, 0, 5);
	}
#endif /*AUTOLINK*/	
    apctlGPIOWrite(GPIO_UR_LINK_STATUS, !isConnected);
}
#endif /*AP_URPTR_LED*/
void TxOnWlanLED(void)
{
#if PROD_NAME == AP51
    apctlSwLedEnable(GPIO_WLAN_RX_TO_WLAN_TX);
#endif
}
#endif /*LED_SUPPORT*/

#ifndef GATEWAY
extern apio_handle_t LanHandle, WlanHandle, WanHandle,llcHandle;
extern IEEEtypes_MacAddr_t macStaAddr,macStaAddr2;

extern struct eth_drv_sc *sc0, *sc1;

UINT32 ProcessWirelessData(int pkts)
{
	apio_bufdescr_t BufDescr;
	apio_bufdescr_t* pBufDescrEAPoL;
	ether_hdr_t *EtherHdr_p;
	UINT32 noPkts;
	noPkts = 0;

	while ( apio_read(WlanHandle, &BufDescr) )
	{
#ifdef GUEST_ACCESS
        if (BufDescr.reserved == WLAN_BSS_GUEST)       /* packet from guest account */
        {
            EtherHdr_p = (ether_hdr_t *)BufDescr.framePtr;
            if (EtherHdr_p->type == 0x8E88)
            {
                // Process EAPoL packet by calling key mgmt function
                pBufDescrEAPoL = ProcessEAPoL(&BufDescr, &EtherHdr_p->sa);
            }
            else
            {
#ifdef ECOS_TCP_IP_STACK
                SendToProtStack(BufDescr.framePtr, BufDescr.frameLen);      /* send to upper layer protocol */
#endif
            }
        }
        else
#endif
		if (BufDescr.reserved != WLAN_BSS_PACKET)
		{
#ifndef EZ_WDS
			DSForwardPacket(WDS_Port_CB[BufDescr.reserved].wdsxHandle, &BufDescr);
#else
			DSForwardPacket(WlanHandle, &BufDescr);
#endif
		}
		else
		{
			EtherHdr_p = (ether_hdr_t *)BufDescr.framePtr;
			if (EtherHdr_p->type == 0x8E88)
			{
				// Process EAPoL packet by calling key mgmt function
				pBufDescrEAPoL = ProcessEAPoL(&BufDescr, &EtherHdr_p->sa);
			}
			else
			{
				DSForwardPacket(WlanHandle, &BufDescr);
			}
		}
		if ( ++noPkts >= pkts )  break;
	}
#ifdef LED_SUPPORT
	if(noPkts)
	{
        apctlSwLedEnable(GPIO_WLAN_RX);
#if PROD_NAME == AP51
        apctlSwLedEnable(GPIO_LAN_TX);		
#endif
	}
#endif
	return(noPkts);
}
/****************************************************************************** 
 * 
 * Name: ProcessEtherData
 * 
 * Description: 

 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
UINT32 ProcessEtherData(int pkts)
{
	apio_bufdescr_t BufDescr;
	UINT32 noPkts;
	noPkts = 0;
	while(apio_read(LanHandle, &BufDescr))
	{
		if ( systemCfg.prod==AP51 )
		{
			DSForwardPacket(LanHandle, &BufDescr);
		}
		else
		{
			if ( BufDescr.reserved == 1 )
			{
				// Packet is from WAN port
				DSForwardPacket(LanHandle, &BufDescr);
			}
			else if ( BufDescr.reserved == 2 )
			{
				// Packet is from LAN port
				DSForwardPacket(LanHandle,&BufDescr);
			}
		}
		if ( ++noPkts >= pkts )  break;
	}
#ifdef LED_SUPPORT
	if(noPkts)
	{
#if PROD_NAME == AP51
        apctlSwLedEnable(GPIO_LAN_RX);
#endif
        apctlSwLedEnable(GPIO_WLAN_TX);		
	}
#endif
	return(noPkts);
}
#ifndef NO_ECOS_TCP_IP_STACK
UINT32 ProcessLlcData(int pkts)
{
	apio_bufdescr_t BufDescr;
	dsQ_LlcData_t *llcData_p;
	UINT32 noPkts;
	noPkts = 0;
	while(llcData_p = ReceiveFromProtStack())
	{
		BufDescr.framePtr = (UINT8 *)llcData_p->data;
		BufDescr.frameLen = llcData_p->totalLen;
		DSForwardPacket(llcHandle, &BufDescr);
		if ( ++noPkts >= pkts )  break;
	}
	return(noPkts);
}
#endif
#ifdef AP_URPTR
extern UINT32 urPort_Write( apio_bufdescr_t *pBufDescr);
extern UINT8       mib_urMode;
extern UINT8  *g_pAssociatedFlag;


UINT32 ProcessWirelessDataUr(int pkts)
{
	apio_bufdescr_t BufDescr;
	ether_hdr_t *EtherHdr_p;
	UINT32 noPkts;
	apio_bufdescr_t Duplicated;
	BOOLEAN	brdcst;
	extStaDb_StaInfo_t *StaInfo_p;
	noPkts = 0;

	while ( apio_read(WlanHandle, &BufDescr) )
	{
#ifdef ETHPORT_AS_URCLIENTPORT
		EtherHdr_p = (ether_hdr_t *)BufDescr.framePtr;
		if ( IS_GROUP((UINT8 *)&EtherHdr_p->da ) )        	
			brdcst = 1;
		else
			brdcst = 0;
        if (!mib_wbMode)
        {
		    if ( (StaInfo_p = extStaDb_GetStaInfo(&EtherHdr_p->da, 0)) == NULL  && !brdcst)
		    {
			    urPort_Write(&BufDescr);
		    }
		    else
		    {
			    if ( brdcst )
			    {
				    if ( apio_alloc(WlanHandle,&Duplicated) == APCTL_OK )
				    {
					    Duplicated.frameLen = BufDescr.frameLen;
					    memcpy(Duplicated.framePtr, BufDescr.framePtr, BufDescr.frameLen);
					    urPort_Write(&Duplicated);
				    }
			    }
			    apio_write(LanHandle,&BufDescr);
		    }
        }
        else
            urPort_Write(&BufDescr);
#else
		urPort_Write(&BufDescr);
#endif
		if ( ++noPkts >= pkts )  break;
	}
#ifdef LED_SUPPORT
	if(noPkts)
	{
        apctlSwLedEnable(GPIO_WLAN_RX);
#if PROD_NAME == AP51
        apctlSwLedEnable(GPIO_LAN_TX);		
#endif
	}
#endif
	return(noPkts);
}
/****************************************************************************** 
 * 
 * Name: ProcessEtherData
 * 
 * Description: 

 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
UINT32 ProcessEtherDataUr(int pkts)
{
	apio_bufdescr_t Duplicated;
	apio_bufdescr_t BufDescr;
	UINT32 noPkts;
	BOOLEAN	brdcst=0;
	extStaDb_StaInfo_t *StaInfo_p;
    IEEEtypes_8023_Frame_t  *DataEtherFrame_p;

	noPkts = 0;
	while(apio_read(LanHandle, &BufDescr))
	{
		DataEtherFrame_p = (IEEEtypes_8023_Frame_t*)BufDescr.framePtr;
		brdcst=IS_GROUP((UINT8 *)&DataEtherFrame_p->Hdr.da);
		if ( !memcmp(DataEtherFrame_p->Hdr.da, mib_currentmacaddress,
			sizeof(IEEEtypes_MacAddr_t)) || !memcmp(DataEtherFrame_p->Hdr.da, macStaAddr2,
			sizeof(IEEEtypes_MacAddr_t)) || brdcst)
		{
			SendToProtStack(DataEtherFrame_p,BufDescr.frameLen);
			if( !brdcst)
			{
				apio_free(LanHandle,&BufDescr);
				continue;
			}
			else
            {
                if (!mib_wbMode)
				    Check_ArpRequest_for_Handover(DataEtherFrame_p);
            }
		}
        if(mib_wbMode)
            urPort_Write(&BufDescr);
#ifdef ETHPORT_AS_URCLIENTPORT
        else
        {
		    if ( (StaInfo_p = extStaDb_GetStaInfo(&DataEtherFrame_p->Hdr.sa, 1)) == NULL)
		    {   
			    PsudoStaDbAdd(&DataEtherFrame_p->Hdr.sa);
		    }
		    if(brdcst)
		    {
			    if ( apio_alloc(WlanHandle,&Duplicated) == APCTL_OK )
			    {
				    Duplicated.frameLen = BufDescr.frameLen;
				    memcpy(Duplicated.framePtr, BufDescr.framePtr, BufDescr.frameLen);
				    apio_write(WlanHandle, &Duplicated);
			    }
		    }
		    if ( (StaInfo_p = extStaDb_GetStaInfo(&DataEtherFrame_p->Hdr.da, 0)) != NULL)
		    {
			    apio_write(WlanHandle,&BufDescr);
		    }
		    else
		    {
			    urPort_Write(&BufDescr);
		    }
        }
#endif
		if ( ++noPkts >= pkts )  break;
	}
#ifdef LED_SUPPORT
	if(noPkts)
	{
#if PROD_NAME == AP51
        apctlSwLedEnable(GPIO_LAN_RX);
#endif
        apctlSwLedEnable(GPIO_WLAN_TX);		
	}
#endif
	return(noPkts);
}
#ifndef NO_ECOS_TCP_IP_STACK
UINT32 ProcessLlcDataUr(int pkts)
{
	BOOLEAN BcastFlag;
	IEEEtypes_MacAddr_t da, sa;
	extStaDb_StaInfo_t *StaInfo_p;
	UINT32 Len;

	UINT32 noPkts = 0;
	ether_hdr_t *EtherHdr_p;
	ether_hdr_t EthHdr;

	dsQ_LlcData_t *LlcData_p;
	apio_bufdescr_t BufDescr,BufDescr1, pBufDesc;
	while (LlcData_p = ReceiveFromProtStack() )
	{
		noPkts++;
		EtherHdr_p = (ether_hdr_t *)LlcData_p->data;
		Len = LlcData_p->totalLen;
		MACADDR_CPY(&da, &EtherHdr_p->da);
		MACADDR_CPY(&sa, &EtherHdr_p->sa);
		EthHdr.type = EtherHdr_p->type;
		BcastFlag = FALSE;

		pBufDesc.framePtr = (UINT8 *)LlcData_p->data;
		pBufDesc.frameLen = LlcData_p->totalLen;

		if ( IS_GROUP((UINT8 *)&da) )
			BcastFlag = TRUE;
		if ( ((StaInfo_p = extStaDb_GetStaInfo(&da, 0)) != NULL &&
			StaInfo_p->State == ASSOCIATED) || BcastFlag )
		{
			if(apio_alloc(WlanHandle,&BufDescr) == APCTL_OK)
			{

				BufDescr.frameLen = pBufDesc.frameLen;
				memcpy(BufDescr.framePtr, pBufDesc.framePtr, pBufDesc.frameLen);

				pool_FreeBuf((UINT8 *)LlcData_p);
				apio_write(WlanHandle,&BufDescr);
			}
			else
			{
				/* free the llc msg */
				pool_FreeBuf((UINT8 *)LlcData_p);
				continue;
			}
		}

		if ( (BcastFlag || (StaInfo_p == NULL )
#ifdef ETHPORT_AS_URCLIENTPORT
			|| StaInfo_p->AP
#endif
			))
		{
			/* to ethernet port */
			if(apio_alloc(LanHandle,&BufDescr1) == APCTL_OK)
			{
				BufDescr1.frameLen = pBufDesc.frameLen;
				memcpy(BufDescr1.framePtr, pBufDesc.framePtr, pBufDesc.frameLen);
				pool_FreeBuf((UINT8 *)LlcData_p);
				apio_write(LanHandle,&BufDescr1);

			}
			else
			{
				/*call the tx done and free the llc msg */
				pool_FreeBuf((UINT8 *)LlcData_p);
				continue;
			}
			if ( (mib_urMode == 1) && *g_pAssociatedFlag 
#ifdef ETHPORT_AS_URCLIENTPORT
				&& (BcastFlag || (!StaInfo_p))
#endif
				)
			{
				if(apio_alloc(WlanHandle,&BufDescr) == APCTL_OK)
				{

					BufDescr.frameLen = pBufDesc.frameLen;
					memcpy(BufDescr.framePtr, pBufDesc.framePtr, pBufDesc.frameLen);
					pool_FreeBuf((UINT8 *)LlcData_p);
					apio_urWlanWrite(&BufDescr);
				}
				else
				{
					pool_FreeBuf((UINT8 *)LlcData_p);
					continue;
				}

			}
		}


		if ( ++noPkts >= pkts )  break;

	} /* while end */
	return(noPkts);
}
#endif
#endif
#endif

