/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    packet.c
 * \brief   Sample packets processing routines
 */
#ifdef GATEWAY
#include "wltypes.h"
#include "oem.h"
#include "IEEE_types.h" 
#include "mib.h" 
#include "api.h"
#include "packet.h"
#include "ds.h"
#ifdef GATEWAY
#include "..\..\..\..\marvell\fast_route\fr_main\incl\main_api.h"
#endif
#include "StaDb.h"

extern apio_handle_t LanHandle, WlanHandle, WanHandle,llcHandle;
extern IEEEtypes_MacAddr_t macStaAddr,macStaAddr2;

extern struct eth_drv_sc *sc0, *sc1;

static void dummy(void){}

#ifdef AP_URPTR
extern UINT32 urPort_Write( apio_bufdescr_t *pBufDescr);
extern UINT8       mib_urMode;
extern UINT8  *g_pAssociatedFlag;
#endif

UINT32 ProcessWirelessDataGW(int pkts)
{

    ether_hdr_t *EtherHdr_p;

    UINT32 Len;

    UINT32 noPkts;
    UINT32 noPkts2eth = 0,dummy =0;
    apio_bufdescr_t BufDescr;
#ifdef ETHPORT_AS_URCLIENTPORT
    apio_bufdescr_t Duplicated;
	BOOLEAN	brdcst;
    extStaDb_StaInfo_t *StaInfo_p;
#endif
#ifdef GATEWAY     
    FastRouting_action_ENT action;
#endif
    noPkts = 0;

    while ( apio_read(WlanHandle, &BufDescr) )
    {
#ifdef GATEWAY 
#ifdef WDS_FEATURE
/** Wpa is not support with wds feature 
wmm with wpa2 will not pass if this this enable **/
	if(!mib_PrivacyTable_p->RSNEnabled)
        if(!DSForwardPacketWDS(WlanHandle, &BufDescr))
			continue;
#endif
#else
#ifdef AP_URPTR
        if ( !mib_urMode )
#endif
    	if(!DSForwardPacket(WlanHandle, &BufDescr))
			continue;
#endif
        EtherHdr_p = (ether_hdr_t *)BufDescr.framePtr;
        Len = BufDescr.frameLen;
#ifdef AP_URPTR
        if ( mib_urMode == 1)
        {
#ifdef ETHPORT_AS_URCLIENTPORT
        	if ( IS_GROUP((UINT8 *)&EtherHdr_p->da ) )        	
        		brdcst = 1;
			else
				brdcst = 0;
        	if ( (StaInfo_p = extStaDb_GetStaInfo(&EtherHdr_p->da, 0)) == NULL  && !brdcst)
			{
            	urPort_Write(&BufDescr);
			}
			else
			{
        		if ( brdcst )
				{
            		if ( apio_alloc(WlanHandle,&Duplicated) == APCTL_OK )
            		{
            		    Duplicated.frameLen = BufDescr.frameLen;
            		    memcpy(Duplicated.framePtr, BufDescr.framePtr, BufDescr.frameLen);
            			urPort_Write(&Duplicated);
            		}
				}
            	apio_write(LanHandle,&BufDescr);
			}
#else
            urPort_Write(&BufDescr);
#endif
        }
        else 
#endif
        if ( IS_GROUP((UINT8 *)&EtherHdr_p->da ) )
        {
            if ( EtherHdr_p->type == 0x0608 )/*ARP*/
            {
                /*call fast routing for ARP update (from LAN if)*/
#ifdef GATEWAY                
              FR_Main_ARP_rx(0,BufDescr.framePtr,BufDescr.frameLen);
#endif
            }
#ifndef NO_ECOS_TCP_IP_STACK
            /* send the pkt to protocol stack */
            /* rcvdData is used by the w8100_eth_x functions */
#ifdef GATEWAY
            SendToProtStack((IEEEtypes_8023_Frame_t *)EtherHdr_p, BufDescr.frameLen,0);
#else
            SendToProtStack((IEEEtypes_8023_Frame_t *)EtherHdr_p, BufDescr.frameLen);
#endif
#endif /* ECOS_TCP_IP_STACK */
            /* send the packet over the ethernet */
            /*apio_write(etherHandle,&BufDescr);*/
    		if(systemCfg.feif == MII_IF)
			{
		    	noPkts2eth++;
            	BufDescr.reserved = 0;/*mark frame for TX to LAN ports*/
            	apio_write(LanHandle,&BufDescr);
			}
			else
			{
                apio_free(WlanHandle,&BufDescr);
			}
        }
        /*check for frames with destination address not equal to local mac address*/
        else if ( memcmp(&EtherHdr_p->da , mib_currentmacaddress,
                          sizeof(IEEEtypes_MacAddr_t)) && memcmp(&EtherHdr_p->da, macStaAddr2,
                          sizeof(IEEEtypes_MacAddr_t)) )
        {
    		if(systemCfg.feif == MII_IF)
			{
            	/* send the packet over the ethernet */
            	noPkts2eth++;
            	BufDescr.reserved = 0;/*mark frame for TX to LAN ports*/
            	apio_write(LanHandle,&BufDescr);
			}
			else
			{
	            apio_free(WlanHandle,&BufDescr);
			}
        }
        else /*frames with destination address equal to local mac address*/
        {

            switch(EtherHdr_p->type)
            {
                case    0x8E88:/*EAPoL packet */
                    ProcessEAPoL(&BufDescr, &EtherHdr_p->sa);
                    if ( ++noPkts >= pkts )
                    {
                        if ( noPkts2eth )
                        {
                            //apctlSwLedEnable(GPIO_LAN_TX);
                        }
                        return(noPkts);
                    }
                continue;
                case 0x0008:/*IP frame*/
                    break;
                case 0x0608:/*ARP packet*/
                    /*call fast routing for ARP update (from LAN if)*/
#ifdef GATEWAY                                    
                    FR_Main_ARP_rx(0,BufDescr.framePtr,BufDescr.frameLen);
#endif
                default:
#ifdef GATEWAY                                            
                    SendToProtStack((IEEEtypes_8023_Frame_t *)EtherHdr_p, BufDescr.frameLen,0);
#else
                    SendToProtStack((IEEEtypes_8023_Frame_t *)EtherHdr_p, BufDescr.frameLen);
#endif
                    apio_free(WlanHandle,&BufDescr);
                    //msi_wl_FreePktBuf((&BufDescr)->framePtr);

                    if ( ++noPkts >= pkts )
                    {
                        if ( noPkts2eth )
                        {
                            //apctlSwLedEnable(GPIO_LAN_TX);
                        }
                        return(noPkts);
                    }
                    continue;

            }

            /*Here this is IP frame with destination address equal to the local mac address*/
#ifdef GATEWAY                            
            action = FR_Main_IP_rx(FastRouting_LAN_inf_E, (UINT8**)&BufDescr.framePtr,(UINT32*)&BufDescr.frameLen);
            switch (action)
            {
                case FastRouting_to_Lan_E:
                    sendToLAN(&BufDescr,&noPkts2eth,&dummy);
                    break;
                case FastRouting_to_Wan_E:
                    BufDescr.reserved = 1;/*mark descriptor for TX to WAN*/
                    noPkts2eth++;
                    apio_write(WanHandle,&BufDescr);
                    break;
                case FastRouting_to_Stack_E:
                    SendToProtStack((IEEEtypes_8023_Frame_t *)EtherHdr_p, BufDescr.frameLen,0);
                case FastRouting_discard_E:
                default:
                    //msi_wl_FreePktBuf((&BufDescr)->framePtr);
                    apio_free(WlanHandle,&BufDescr);
                    break;
            }
#endif

        }
        if ( ++noPkts >= pkts )  break;

    } /* End While GetWireless Data */
    if ( noPkts2eth )
    {
        //apctlSwLedEnable(GPIO_LAN_TX);
    }
    return(noPkts);
}

void handle_LAN_broadcast(apio_bufdescr_t *rBufDescr)
{
    IEEEtypes_8023_Frame_t  *DataEtherFrame_p;
	UINT16 reserved;

    DataEtherFrame_p = (IEEEtypes_8023_Frame_t  *)rBufDescr->framePtr;
    if (DataEtherFrame_p->Hdr.type == 0x0608)/*ARP*/
    {
        /*call fast routing for ARP update*/
#ifdef GATEWAY                        
        FR_Main_ARP_rx(rBufDescr->reserved,rBufDescr->framePtr,rBufDescr->frameLen);
#endif
        /*update the wireless station list according to ARP*/
        //Check_ArpRequest_for_Handover(DataEtherFrame_p);
    }
    /*pass frame to TCP/IP stack*/
	reserved = rBufDescr->reserved;
    rBufDescr->reserved = 0;
#ifdef GATEWAY                            
    SendToProtStack(DataEtherFrame_p, rBufDescr->frameLen,rBufDescr->reserved);
#else
    SendToProtStack(DataEtherFrame_p, rBufDescr->frameLen);
#endif
    rBufDescr->reserved = reserved;
    /*TX to wireless port*/
    apio_write(WlanHandle,rBufDescr);
}
void handle_WAN_broadcast(apio_bufdescr_t *rBufDescr)
{
    IEEEtypes_8023_Frame_t  *DataEtherFrame_p;

    DataEtherFrame_p = (IEEEtypes_8023_Frame_t  *)rBufDescr->framePtr;
    if (DataEtherFrame_p->Hdr.type == 0x0608)/*ARP*/
    {
        /*call fast routing for ARP update*/
#ifdef GATEWAY                                        
        FR_Main_ARP_rx (rBufDescr->reserved,rBufDescr->framePtr,rBufDescr->frameLen);
#endif
    }
    /*pass frame to TCP/IP stack*/
#ifdef GATEWAY                                
    SendToProtStack(DataEtherFrame_p,rBufDescr->frameLen,rBufDescr->reserved);
#else
    SendToProtStack(DataEtherFrame_p,rBufDescr->frameLen);
#endif
    /*free buffer*/
    //msi_wl_FreePktBuf(rBufDescr->framePtr);
    apio_free(LanHandle,rBufDescr);
}
/****************************************************************************** 
 * 
 * Name: ProcessEtherData
 * 
 * Description: 

 * 
 * Conditions For Use: 
 *   None
 * 
 * Arguments: 
 *   Arg1 (i  ): 
 * 
 * Return Value: 
 *   None. 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/ 
UINT32 ProcessEtherDataGW(int pkts)
{
    IEEEtypes_8023_Frame_t  *DataEtherFrame_p;
    UINT32 noPkts;
    UINT16 ethType;
    UINT32 noPkts2wlan = 0,dummy =0 ;
    apio_bufdescr_t rBufDescr;
#ifdef GATEWAY                    
    FastRouting_action_ENT action;
#endif
#ifdef ETHPORT_AS_URCLIENTPORT
	BOOLEAN	brdcst=0;
    extStaDb_StaInfo_t *StaInfo_p;
#endif
    /* Retrieve data from the receive buffers one by one
     * and process them. The Frame has ethernet header and the
     * destination address is checked to see how the frame has
     * to be routed */
    noPkts = 0;
    /* retrieve the frame */
    while ( apio_read(LanHandle, &rBufDescr) )
    {
#ifdef GATEWAY 
#ifdef WDS_FEATURE
/** Wpa is not support with wds feature 
wmm with wpa2 will not pass if this this enable **/
		if(!mib_PrivacyTable_p->RSNEnabled)
        	if(!DSForwardPacketWDS(LanHandle, &rBufDescr))
				continue;
#endif
#else
#ifdef AP_URPTR
    	if ( !mib_urMode )
#endif
    	if(!DSForwardPacket(LanHandle, &rBufDescr))
			continue;
#endif
    	if(systemCfg.feif != MII_IF) /* PHY_IF or EPHY_IF */
		{
			rBufDescr.reserved = 1;
		}
        DataEtherFrame_p = (IEEEtypes_8023_Frame_t*)rBufDescr.framePtr;
#ifdef ETHPORT_AS_URCLIENTPORT
        brdcst=IS_GROUP((UINT8 *)&DataEtherFrame_p->Hdr.da);        
        if ( mib_urMode == 1)
        {
        	if ( (StaInfo_p = extStaDb_GetStaInfo(&DataEtherFrame_p->Hdr.sa, 1)) == NULL)
			{
				PsudoStaDbAdd(&DataEtherFrame_p->Hdr.sa);
			}
		}
		if(brdcst)
		{
			apio_bufdescr_t Duplicated;
            if ( apio_alloc(WlanHandle,&Duplicated) == APCTL_OK )
            {
                Duplicated.frameLen = rBufDescr.frameLen;
                memcpy(Duplicated.framePtr, rBufDescr.framePtr, rBufDescr.frameLen);
				urPort_Write(&Duplicated);
				rBufDescr.reserved = 0xbc; /* broadcast packet*/
			}
#else
        /*broadcast handle*/
        if ( IS_GROUP((UINT8 *)&DataEtherFrame_p->Hdr.da) )
        {
#endif
            if(rBufDescr.reserved == 2 || rBufDescr.reserved == 0xbc)/*LAN port*/
            {
                handle_LAN_broadcast(&rBufDescr);
                noPkts2wlan++;
            }
            else
            {
                handle_WAN_broadcast(&rBufDescr);
            }
            if ( ++noPkts >= pkts )
            {
                break;
            }
        }
		
        /*destination address is not local mac address*/
        else if ( memcmp(DataEtherFrame_p->Hdr.da, mib_currentmacaddress,
                      sizeof(IEEEtypes_MacAddr_t)) && memcmp(DataEtherFrame_p->Hdr.da, macStaAddr2,
                      sizeof(IEEEtypes_MacAddr_t)) )
        {
#ifdef ETHPORT_AS_URCLIENTPORT
           if ( (StaInfo_p = extStaDb_GetStaInfo(&DataEtherFrame_p->Hdr.da, 0)) != NULL)
		   {
           		apio_write(WlanHandle,&rBufDescr);
           }
           else
		   {
	           	urPort_Write(&rBufDescr);
		   }
#else
            if(rBufDescr.reserved == 2)/*LAN ports*/
            {
                /*TX to wireless port*/
                noPkts2wlan++;
                apio_write(WlanHandle,&rBufDescr);
            }
            else /*WAN port*/
            {
                //msi_wl_FreePktBuf(rBufDescr.framePtr);
                apio_free(LanHandle,&rBufDescr);
            }
#endif
            if ( ++noPkts >= pkts )
            {
                break;
            }
        }
        else
        {
            /*Here the destination address is local address and now it
            is time to check ether TYPR*/
            rBufDescr.reserved = (rBufDescr.reserved == 2)?0:1;
            switch (DataEtherFrame_p->Hdr.type)
            {
                case 0x0008:/*IP*/
#ifdef GATEWAY                                    
                    break;
#endif
                case 0x0608:/*ARP*/
                    /*call fast routing for ARP update*/
#ifdef GATEWAY                                    
                    FR_Main_ARP_rx (rBufDescr.reserved,rBufDescr.framePtr,rBufDescr.frameLen);
#endif
                default:
#ifdef GATEWAY                                                    
                    SendToProtStack(DataEtherFrame_p,rBufDescr.frameLen,rBufDescr.reserved);
#else
                    SendToProtStack(DataEtherFrame_p,rBufDescr.frameLen);
#endif
                    //msi_wl_FreePktBuf(rBufDescr.framePtr);
                    apio_free(LanHandle,&rBufDescr);
                    if ( ++noPkts >= pkts )
                    {
                        if ( noPkts2wlan )
                        {
                            //apctlSwLedEnable(GPIO_WLAN_TX);
                        }
                        return(noPkts);
                    }
                continue;
            }

            /*Here this is IP frame with destination address equal to the local mac address*/
#ifdef GATEWAY                            
            action = FR_Main_IP_rx(rBufDescr.reserved, (UINT8**)&rBufDescr.framePtr,(UINT32 *)&rBufDescr.frameLen);
            switch (action)
            {
                case FastRouting_to_Lan_E:
                    sendToLAN(&rBufDescr,&dummy,&noPkts2wlan);
                    break;
                case FastRouting_to_Wan_E:
                    rBufDescr.reserved = 1;/*mark descriptor for TX to WAN*/
                    apio_write(WanHandle,&rBufDescr);
                    break;
                case FastRouting_to_Stack_E:
                    SendToProtStack(DataEtherFrame_p, rBufDescr.frameLen,rBufDescr.reserved);
                case FastRouting_discard_E:
                default:
                    //msi_wl_FreePktBuf(rBufDescr.framePtr);
                    apio_free(LanHandle,&rBufDescr);
                    break;
            }
#endif            
        }
        if ( ++noPkts >= pkts )  break;
    }
    if ( noPkts2wlan )
    {
        //apctlSwLedEnable(GPIO_WLAN_TX);
    }
    return(noPkts);
}
#ifndef NO_ECOS_TCP_IP_STACK
UINT32 ProcessLlcDataGW(int pkts)
{
    ether_hdr_t *EtherHdr_p, *EtherHdr2_p;

    BOOLEAN BcastFlag;
    IEEEtypes_MacAddr_t da, sa;
    extStaDb_StaInfo_t *StaInfo_p;
    UINT32 Len;

    UINT32 noPkts = 0;
    ether_hdr_t EthHdr;

    dsQ_LlcData_t *LlcData_p;
    apio_bufdescr_t BufDescr,BufDescr1, pBufDesc;
    while (LlcData_p = ReceiveFromProtStack() )
    {
        noPkts++;
        EtherHdr_p = (ether_hdr_t *)LlcData_p->data;
        Len = LlcData_p->totalLen;
        MACADDR_CPY(&da, &EtherHdr_p->da);
        MACADDR_CPY(&sa, &EtherHdr_p->sa);
        EthHdr.type = EtherHdr_p->type;
        BcastFlag = FALSE;

        pBufDesc.framePtr = (UINT8 *)LlcData_p->data;
        pBufDesc.frameLen = LlcData_p->totalLen;

#ifndef GATEWAY 
#ifdef AP_URPTR
        if ( !mib_urMode )
#endif
    	if(!DSForwardPacket(llcHandle, &pBufDesc))
			continue;
#endif

#ifdef GATEWAY
        if(LlcData_p->interface == 0) /*LAN interface*/
        {
#endif
        if ( IS_GROUP((UINT8 *)&da) )
            BcastFlag = TRUE;
        if ( ((StaInfo_p = extStaDb_GetStaInfo(&da, 0)) != NULL &&
              StaInfo_p->State == ASSOCIATED) || BcastFlag )
        {
            if(apio_alloc(WlanHandle,&BufDescr) == APCTL_OK)
            {

                BufDescr.frameLen = pBufDesc.frameLen;
                memcpy(BufDescr.framePtr, pBufDesc.framePtr, pBufDesc.frameLen);
                
                //memcpy(BufDescr.framePtr, (UINT8*)EtherHdr_p, Len);
                //BufDescr.frameLen = Len;
                apio_write(WlanHandle,&BufDescr);
            }
            else
            {
                /* free the llc msg */
                //eth_drv_tx_done(sc1,LlcData_p->Key,0);
                //msi_wl_FreePktBuf(LlcData_p);
                pool_FreeBuf((UINT8 *)LlcData_p);
                continue;
            }
        }

#ifdef GATEWAY
        if ( (BcastFlag || (StaInfo_p == NULL )) && (systemCfg.feif == MII_IF) )
#else
        if ( (BcastFlag || (StaInfo_p == NULL )
#ifdef ETHPORT_AS_URCLIENTPORT
		|| StaInfo_p->AP
#endif
		))
#endif
        {
            /* to ethernet port */
            if(apio_alloc(LanHandle,&BufDescr1) == APCTL_OK)
            {
                BufDescr1.frameLen = pBufDesc.frameLen;
                memcpy(BufDescr1.framePtr, pBufDesc.framePtr, pBufDesc.frameLen);
    #ifdef GATEWAY
                BufDescr1.reserved = 0; /*send to LAN*/
    #endif
                apio_write(LanHandle,&BufDescr1);

            }
            else
            {
                /*call the tx done and free the llc msg */
                //eth_drv_tx_done(sc1,LlcData_p->Key,0);
                //msi_wl_FreePktBuf(LlcData_p);
                pool_FreeBuf((UINT8 *)LlcData_p);
                continue;
            }
#if defined(AP_URPTR) && (!defined(UR_GW))
            if ( (mib_urMode == 1) && *g_pAssociatedFlag 
#ifdef ETHPORT_AS_URCLIENTPORT
            && (BcastFlag || (!StaInfo_p))
#endif
            )
            {
            	if(apio_alloc(WlanHandle,&BufDescr) == APCTL_OK)
            	{

            	    BufDescr.frameLen = pBufDesc.frameLen;
            	    memcpy(BufDescr.framePtr, pBufDesc.framePtr, pBufDesc.frameLen);
                	apio_urWlanWrite(&BufDescr);
            	}
            	else
            	{
            	    pool_FreeBuf((UINT8 *)LlcData_p);
            	    continue;
            	}
                
            }
#endif /* AP_URPTR */
        }

        //eth_drv_tx_done(sc1, LlcData_p->Key, 0);
        //msi_wl_FreePktBuf(LlcData_p);
        //pool_FreeBuf((UINT8 *)LlcData_p);
#ifdef GATEWAY
        }
        else/*WAN interface*/
        {
            if(apio_alloc(WanHandle,&BufDescr) == APCTL_OK)
            {
                /*memcpy(EtherHdr_p,  LlcData_p->data,   LlcData_p->totalLen); */
                BufDescr.frameLen = pBufDesc.frameLen;
                memcpy(BufDescr.framePtr, pBufDesc.framePtr, pBufDesc.frameLen);

                //memcpy(BufDescr.frameLen,  (UINT8 *)LlcData_p->data,   LlcData_p->totalLen);

                //BufDescr.frameLen = Len;
                BufDescr.reserved = 1;/*send to WAN*/
                apio_write(WanHandle,&BufDescr);
                //eth_drv_tx_done(sc0, LlcData_p->Key, 0);
                //msi_wl_FreePktBuf((char *)LlcData_p);
            }
            else
            {
                /*call the tx done and free the llc msg */
                //eth_drv_tx_done(sc0,LlcData_p->Key,0);
                //msi_wl_FreePktBuf((char *)LlcData_p);
                pool_FreeBuf((UINT8 *)LlcData_p);
                continue;
            }
        }
#endif
        pool_FreeBuf((UINT8 *)LlcData_p);        
        if ( ++noPkts >= pkts )  break;

    } /* while end */
    return (noPkts);
}
#endif

void sendToLAN(apio_bufdescr_t *pBufDescr,UINT32 *noPkts2eth,UINT32 *noPkts2wlan)
{
    ether_hdr_t *EtherHdr_p,*EtherHdr2_p;
    BOOLEAN BcastFlag = FALSE;
    IEEEtypes_MacAddr_t da;
    extStaDb_StaInfo_t *StaInfo_p;
    apio_bufdescr_t tmpBufDescr;


    EtherHdr_p = (ether_hdr_t *)pBufDescr->framePtr;
    /*get destination addr*/
    MACADDR_CPY(&da, &EtherHdr_p->da);
    /*check for broadcast*/
    if ( IS_GROUP((UINT8 *)&da) )
    {
        BcastFlag = TRUE;
    }
    /*if wireless station TX to WLAN*/
    if ( ((StaInfo_p = extStaDb_GetStaInfo(&da, 0)) != NULL &&
            StaInfo_p->State == ASSOCIATED)  )
    {
        /*we can not send the frame yes because we need to check if it needs
        to be send to the LAN*/
        (noPkts2wlan)++;
        apio_write(WlanHandle,pBufDescr);
        return;
    }
    /*broadcast frame is sent to both ETH and WLAN ports*/
    if (  BcastFlag )
    {
        /*we can not send the frame yes because we need to check if it needs
        to be send to the LAN*/
		if(systemCfg.feif == MII_IF)
		{
			if ( (EtherHdr2_p = (ether_hdr_t *) apio_GetEtherTxBuf()) == NULL )
			{
				(*noPkts2wlan)++;
				/*we already has the buffer so TX it to the WLAN*/
				apio_write(WlanHandle,pBufDescr);
				return;
			}
			/*copy the frame to new descriptor for transmition to the ETH*/
			memcpy((UINT8 *)EtherHdr2_p,  (UINT8 *)pBufDescr->framePtr,  pBufDescr->frameLen);
			tmpBufDescr.frameLen = pBufDescr->frameLen;
			tmpBufDescr.framePtr = (UINT8*)EtherHdr2_p;
			tmpBufDescr.reserved = 0; /*send to LAN*/
			(*noPkts2eth)++;
			/*TX second buffer to the ETH port*/
			apio_write(LanHandle,&tmpBufDescr);
		}
        (*noPkts2wlan)++;
        /*TX original buffer to the WLAN port*/
        apio_write(WlanHandle,pBufDescr);
        return;
    }

    /*TX only to the ETH ports*/
    if (StaInfo_p == NULL )
    {
        pBufDescr->reserved = 0; /*send to LAN*/
        (*noPkts2eth)++;
        apio_write(LanHandle,pBufDescr);
    }

}

#endif