;;; Copyright ARM Ltd 1999. All rights reserved.

        AREA Vect, CODE, READONLY
	code32
; These are example exception vectors and exception handlers

; Where there is ROM fixed at 0x0 (build_b & build_c), these are hard-coded at 0x0.
; Where ROM/RAM remapping occurs (build_d), these are copied from ROM to RAM.
; The copying is done automatically by the C library code inside __main.

; *****************
; Exception Vectors
; *****************

; Note: LDR PC instructions are used here because branch (B) instructions
; could not simply be copied (the branch offsets would be wrong).  Also,
; a branch instruction might not reach if the ROM is at an address >32MB).

        ENTRY
	export  Vectors
Vectors
        LDR     PC, Reset_Addr
        LDR     PC, Undefined_Addr
        LDR     PC, SWI_Addr
        LDR     PC, Prefetch_Addr
        LDR     PC, Abort_Addr
        NOP                             ; Reserved vector
        LDR     PC, IRQ_Addr
        LDR     PC, FIQ_Addr
        
Reset_Addr      DCD     Reset_Handler
Undefined_Addr  DCD     Undefined_Handler
SWI_Addr        DCD     SWI_Handler
Prefetch_Addr   DCD     Prefetch_Handler
Abort_Addr      DCD     Abort_Handler
                DCD     0               ; Reserved vector
IRQ_Addr        DCD     IRQ_Handler
FIQ_Addr        DCD     FIQ_Handler

;; Settings for cp15
Protection_Unit_Enable	EQU	(1 << 0)
D_Cache_Enable		EQU	(1 << 2)
Big_Endian_Enable	EQU	(1 << 7)
I_Cache_Enable		EQU	(1 << 12)
HighVectors		EQU	(1 << 13)
DSRAM_Enable		EQU	(1 << 16)
ISRAM_Enable		EQU	(1 << 18)
ISRAM_Load		EQU	(1 << 19)
	

; ************************
; Exception Handlers
; ************************

; The following dummy handlers do not do anything useful in this example.
; They are set up here for completeness.

Undefined_Handler
        B       Reset_Handler
SWI_Handler
        B       Reset_Handler     
Prefetch_Handler
        B       Reset_Handler
Abort_Handler
        B       Reset_Handler
IRQ_Handler
        B       Reset_Handler
FIQ_Handler
        B       Reset_Handler
        
Reset_Handler
        mov	r3, #0x2000   ; Delay in case of clock problem
wait    subs	r3, r3, #1
        bne wait
;	LDR	r0,=0x50078
;	mcr	p15, 0, r0, c1, c0, 0
;

		;; Enable clocks
	ldr	r0,=0x8000200c			; SCLKEN
	mov	r1,#-1
	str	r1,[r0]

	ldr	r2,=0x90008000	; irq enable	
	mov	r3,#0		; off all irqs
	str	r3,[r2,#0x8]	; 0 ->8008
	mov	r1,#-1
	str	r1,[r2,#0xc]		; clr = 1 for all bits
	
	ldr	r0,=0x90009000	; timerbase
	ldr	r1,[r0,#0x24]	; Intr source

	;; Clear GPIO
	ldr	r0,=0x8000d000			; GPIO base addr
	ldr	r1,=0x5555			; 2'b10 repeating. set GPIO to programmed output
	mov	r2,#0				; clear
	mov	r3,#-1				; led's to off
	str	r1,[r0]				; control 5555
	str	r1,[r0,#0x4]			; control2 5555
	str	r2,[r0,#0x8]			; OE->outputs
	str	r3,[r0,#0xc]			; pins should now be 0.

	;; Manipulate cp15 to enable the cache
	mov	r3,#0
	mcr	p15, 0, r3, c7, c5, 0   ; flush I-cache
	mcr	p15, 0, r3, c7, c6, 0   ; flush D-cache

	ldr	r0,=0x8000a800
	mov	r1,#1
	str	r1,[r0]		; soft reset the WL MAC
		
	import	ConfigMDU
	bl	ConfigMDU

;	ldr	r0,=0xfe123457
;	mov	r2,r0
;	mov	r3,r0
;	mov	r4,r0
;	mov	r5,r0
;	mov	r6,r0
;	mov	r7,r0
;	mov	r8,r0
;	mov	r9,r0
;	ldr	r0,=0xd0000000
;	ldr	r1,=0xd0400000
;loop
;	stmia	r0!,{r2-r9}
;	cmp	r0,r1
;	blt	loop
;
;	ldr	r0,=0x0
;	ldr	r1,=0x20000
;loop2
;	stmia	r0!,{r2-r9}
;	cmp	r0,r1
;	blt	loop2
;
;	ldr	r0,=0xc0000000
;	ldr	r1,=0xc000c000
;loop3
;	stmia	r0!,{r2-r9}
;	cmp	r0,r1
;	blt	loop3
		

; region 0
;	ldr	r3,=(0xfff00000 | (0x13 << 1) | 1)	; 1MB & enable @fff00000
;	ldr	r3,=(0xffe00000 | (0x14 << 1) | 1)	; 2MB & enable @ffe00000
	ldr	r3,=(0xffc00000 | (0x15 << 1) | 1)	; 4MB & enable @ffc00000
	mcr	p15, 0, r3, c6, c0, 0   ; protection enabled

; region 1
	ldr	r3,=(0xd0000000 | (0x16 << 1) | 1)	; 8MB & enable @d0000000
;ldr	r3,=(0xd0000000 | (0x15 << 1) | 1)	; 4MB & enable @d0000000
	mcr	p15, 0, r3, c6, c1, 0   ; protection enabled

; region 2
	ldr	r3,=(0x0 | (0x10 << 1) | 1)	        ; 128kB & enable @0
;	ldr	r3,=(0x0 | (0xf << 1) | 1)	        ; 64kB & enable @0
	mcr	p15, 0, r3, c6, c2, 0   		; protection enabled

; region 3
	ldr	r3,=(0x90008000 | (0x10 << 1) | 1)	        ; 128kB & enable @0x90008000
	mcr	p15, 0, r3, c6, c3, 0   ; protection enabled

; region 4
;.ifdef CONDOR2
;hardcode for now , .ifdef don't work for gnu???
	ldr	r3,=(0xc0000000 | (0x10 << 1) | 1)	        ; 128kB & enable @0xc0000000
;.else
;	ldr	r3,=(0xc0000000 | (0xf << 1) | 1)	        ; 64kB & enable @0xc0000000
;.endif

	mcr	p15, 0, r3, c6, c4, 0   ; protection enabled

; region 5
	ldr	r3,=(0x80000000 | (0x10 << 1) | 1)	        ; 128kB & enable @0x80000000
	mcr	p15, 0, r3, c6, c5, 0   ; protection enabled
; region 6
	ldr	r3,=(0x98000000 | (0xc << 1) | 1)	        ; 8kB & enable @0x90008000
	mcr	p15, 0, r3, c6, c6, 0   ; protection enabled
	mov	r3,#0		; disabled
	mcr	p15, 0, r3, c6, c7, 0   ; protection disabled 

	ldr	r3,=(0x3 << 24) | (0x3 << 20) | (0x3 << 16) | (0x3 << 12) | (0x3 << 8) | (0x3 << 4) | 3 	; r6:RW, r5:RW, r4:RW, r3:RW, r2: RW, r1: RW, r0: RW data
	mcr	p15, 0, r3, c5, c0, 2   ; data prot
	ldr	r3,=(0x0 << 24) | (0x0 << 20 ) | (0x6 << 16) | (0x0 << 12) | (0x6 << 8) | (0x6 << 4) | 6 	; r6:- r5:-, r4:R, r3:X, r2: R, r1: R, r0: R code
	mcr	p15, 0, r3, c5, c0, 3   ; inst prot

	mrc	p15, 0, r3, c1, c0, 0
	
	ldr	r0,=0xffff0000
	cmp	pc,r0
;	bhi	high_vectors
	
	ldr	r0,=   ~HighVectors
	and	r3,r3,r0
	b	go_on
;high_vectors				; Do nothing if bge to here
;	orrhi	r3,r3,#HighVectors	; v <- V (high vectors)
go_on
	
	LDR	r2,=(I_Cache_Enable | D_Cache_Enable | ISRAM_Enable | DSRAM_Enable | Protection_Unit_Enable)
	orr	r3, r3, r2
	mcr	p15, 0, r3, c1, c0, 0
	
	ldr	sp,=0xd0380000

		
	mov	r3,#3
	mcr	p15, 0, r3, c2, c0, 0   ; data cache region 0 enable
	mcr	p15, 0, r3, c2, c0, 1   ; inst cache region 0 enable
	
;	import  __rt_entry
;	b 	__rt_entry

	import  __main
	b 	__main

	export	disable_cache
disable_cache
	mrc	p15, 0, r3, c1, c0, 0
	ldr	r0,=(I_Cache_Enable | D_Cache_Enable)
	bic	r3,r3,r0
	mcr	p15, 0, r3, c1, c0, 0
	bx	r14
	export	enable_cache
enable_cache
	mrc	p15, 0, r3, c1, c0, 0
	ldr	r0,=(I_Cache_Enable | D_Cache_Enable)
	orr	r3,r3,r0
	mcr	p15, 0, r3, c1, c0, 0
	bx	r14
	export	invalidate_cache
invalidate_cache
	mov	r3,#0
	mcr	p15, 0, r3, c7, c5, 0   ; flush I-cache
	mcr	p15, 0, r3, c7, c6, 0   ; flush D-cache
	bx	r14

	export	disable_data_cache
disable_data_cache
	mrc	p15, 0, r3, c1, c0, 0
	ldr	r0,=(D_Cache_Enable)
	bic	r3,r3,r0
	mcr	p15, 0, r3, c1, c0, 0
	bx	r14
	export	enable_data_cache
enable_data_cache
	mrc	p15, 0, r3, c1, c0, 0
	ldr	r0,=(D_Cache_Enable)
	orr	r3,r3,r0
	mcr	p15, 0, r3, c1, c0, 0
	bx	r14
	export	invalidate_data_cache
invalidate_data_cache
	mov	r3,#0
	mcr	p15, 0, r3, c7, c6, 0   ; flush D-cache
	bx	r14

	export  trimITCM
trimITCM
	ldr	r0,=(0x0 | (0xc << 1) | 1)	        ; 8kB & enable @0
	mcr	p15, 0, r0, c6, c2, 0   		; protection enabled
	bx	r14
		
	;;
	;; these are needed to support __packed types calling memxxx as arg1,arg2 or both
	export mymemcmp
	import	memcmp
	import	memset
	import  memcpy
mymemcmp
	b	memcmp
	export mymemcpy2
	export mymemcpy3
	export mymemcpy1
	export mymemset
mymemcpy1
mymemcpy2
mymemcpy3
	b	memcpy
mymemset
	b	memset

	code16
	EXPORT	Thumb2ARM
Thumb2ARM
	;; r0 contains the address we wish to go to
	bx	r0
	
	END
	