/*
 *   mem.c
 */

#include "decls.h"
#include "board.h"

/*----- Types and Constants Definition ----------------------*/
#define _REG(x) (*(volatile unsigned long *)(x))
#define _REG16(x) (*(volatile unsigned short *)(x))

//===================================================================
// SDRAM controller registers
//
#define MDU_CSR      0x80006000
#define MDU_CFGR0    0x80006004
#define MDU_TIMR00   0x80006008    
#define MDU_TIMR01   0x8000600C
#define MDU_CFGR1    0x80006010
#define MDU_TIMR10   0x80006014
#define MDU_TIMR11   0x80006018
#define MDU_CFGR2    0x8000601C
#define MDU_TIMR20   0x80006020
#define MDU_TIMR21   0x80006024
#define MDU_CFGR3    0x80006028
#define MDU_TIMR30   0x8000602C
#define MDU_TIMR31   0x80006030
#define MDU_CFGR4    0x80006034
#define MDU_TIMR40   0x80006038
#define MDU_TIMR41   0x8000603C
#define MDU_CFGR5    0x80006040
#define MDU_TIMR50   0x80006044
#define MDU_TIMR51   0x80006048

//=========================================================
// CPU interface registers
//
#define MDU_CLOCK    0x8000200C
#define MDU_CLKCTRL  0x80002030
#define MDU_GPR1     0x80002044

//=========================================================
// Initialize device SDRAM controllers.
//
void ConfigMDU()
{
    int boardtype = BOARD;

    if (boardtype >= LASTBOARD)
    {
        return;
    }

    _REG(MDU_CSR)    = 0x04191001; /* set SDRAM reflesh rate */

    _REG(MDU_CFGR2)  = 0xD0FF0008; /* disable device */
    _REG(MDU_CFGR3)  = 0xC0FF0008;
    _REG(MDU_CFGR4)  = 0x80FF0008;
    _REG(MDU_CFGR5)  = 0x40FF0008;

    _REG(MDU_TIMR00) = 0x0442A20C;   /* FLASH timing */
    _REG(MDU_TIMR01) = 0x00008080;
    _REG(MDU_CFGR0)  = 0xFFFF4285;   /* 0xffc00000 FLASH configuration pack_en = 1*/

    _REG(MDU_TIMR10) = 0x00000037;//33; /* Set to 8-byte burst mode */
    _REG(MDU_TIMR11) = 0x0A3A2632;
    switch (boardtype)
    {
        case DRAM_1_64_32_BOARD:
#ifdef CONDOR2 /** brc and opt setting from venky **/
	        _REG(MDU_CSR)    = 0x04191021; /** new setting from bb **///0x20530000;
#else
	        _REG(MDU_CSR)    = 0x04191021;  /** new setting from bb **///0x20530000;
#endif
            _REG(MDU_CFGR1)  = 0x40FE70B7; /*  8MB = 1 x 64Mbit (32-bit chip, 32-bit bus) */
            break;
        case DRAM_2_16_16_BOARD:
            _REG(MDU_CFGR1)  = 0x40FF7095; /*  4MB = 2 x 16Mbit (16-bit chip, 32-bit bus) */
            break;
    	case DRAM_2_64_16_BOARD:
            _REG(MDU_CFGR1)  = 0x40FC70B5; /* 16MB = 2 x 64Mbit (16-bit chip, 32-bit bus) */
        	break;
        case DRAM_1_16_16_BOARD:
            _REG(MDU_CFGR1)  = 0x40FF5095; /*  2MB = 1 x 16Mbit (16-bit chip, 16-bit bus) */
            break;
        case DRAM_1_64_16_BOARD:
            _REG(MDU_CFGR1)  = 0x40FE50B5; /*  8MB = 1 x 64Mbit (16-bit chip, 16-bit bus) */
            break;
        case EVAL_BOARD:
            _REG(MDU_TIMR20) = 0x00000022;
            _REG(MDU_TIMR21) = 0x03232222;
            _REG(MDU_CFGR2)  = 0x40FE50B5; /*  8MB = 1 x 64Mbit (16-bit chip, 16-bit bus) */
	        break;
        default:
            _REG(MDU_CFGR1)  = 0x40FF5095; /*  2MB = 1 x 16Mbit (16-bit chip, 16-bit bus) */
    }

    _REG(MDU_CLKCTRL)= 0x00007109; /* 44 Mhz */
    _REG(MDU_GPR1)   = 0x00E0001E;

}
