/********************************************************************************
* msApiDefs.h
*
* DESCRIPTION:
*       API definitions for QuarterDeck Device
*
* DEPENDENCIES:
*
* FILE REVISION NUMBER:
*
*******************************************************************************/

#ifndef __msApiDefs_h
#define __msApiDefs_h

#ifdef __cplusplus
extern "C" {
#endif

#ifndef NULL
#define NULL ((void*)0)
#endif

#undef IN
#define IN
#undef OUT
#define OUT
#undef INOUT
#define INOUT

/* Debug function   */
#ifdef DEBUG_QD
#define DBG_INFO(x) gtDbgPrint x
#else
#define DBG_INFO(x)
#endif /* DEBUG_QD */

#define ETHERNET_HEADER_SIZE	6

typedef void  GT_VOID, *GT_VOID_PTR;
typedef char  GT_8,   *GT_8_PTR;
typedef short GT_16,  *GT_16_PTR;
typedef long  GT_32,  *GT_32_PTR;

typedef unsigned char  GT_U8,   *GT_U8_PTR;
typedef unsigned short GT_U16,  *GT_U16_PTR;
typedef unsigned long  GT_U32,  *GT_U32_PTR;

typedef unsigned long  GT_SIZE, *GT_SIZE_PTR;

typedef void   *GT_PTR;

typedef GT_U32 GT_SEM;

typedef union
{
    GT_U8   c[8];
    GT_U16  s[4];
    GT_U32  l[2];
}GT_U64;


typedef struct
{
    GT_U16  u16EtherL;
    GT_U32  u32EtherH;
}GT_MAC1632;

typedef struct
{
    GT_U32  u32EtherL;
    GT_U16  u16EtherH;
}GT_MAC3216;

typedef union
{
    GT_MAC1632  u1632Ether;
    GT_MAC3216  u48Ether;
    GT_U8       arEther[ETHERNET_HEADER_SIZE];
}GT_ETHERADDR;

typedef void          (*GT_VOIDFUNCPTR) (); /* ptr to function returning void */
typedef unsigned int  (*GT_INTFUNCPTR)  (); /* ptr to function returning int  */

/*
 * typedef: enum GT_BOOL
 *
 * Description: Enumeration of boolean.
 *
 * Enumerations:
 *    GT_FALSE - false.
 *    GT_TRUE  - true.
 */
typedef enum
{
    GT_FALSE = 0,
    GT_TRUE  = 1
} GT_BOOL;


/*
 *   Logical Port value based on a Port
 *   3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
 *   1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *  |0|  reserved                                   |    port       |
 *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
 *
 *
 *   The following macros should be used to extract specific info
 *   from a Logical Port index
 */
typedef GT_U32 GT_LPORT;

/* indexes are created internally by the QuarterDeck-driver.       */
typedef enum
{
     GT_ERROR           = (-1),
     GT_OK              = (0x00),   /* Operation succeeded                   */
     GT_FAIL            = (0x01),   /* Operation failed                      */
     GT_BAD_VALUE       = (0x02),   /* Illegal value (general)               */
     GT_OUT_OF_RANGE    = (0x03),   /* The value is out of range             */
     GT_BAD_PARAM       = (0x04),   /* Illegal parameter in function called  */
     GT_BAD_PTR         = (0x05),   /* Illegal pointer value                 */
     GT_BAD_SIZE        = (0x06),   /* Illegal size                          */
     GT_BAD_STATE       = (0x07),   /* Illegal state of state machine        */
     GT_SET_ERROR       = (0x08),   /* Set operation failed                  */
     GT_GET_ERROR       = (0x09),   /* Get operation failed                  */
     GT_CREATE_ERROR    = (0x0A),   /* Fail while creating an item           */
     GT_NOT_FOUND       = (0x0B),   /* Item not found                        */
     GT_NO_MORE         = (0x0C),   /* No more items found                   */
     GT_NO_SUCH         = (0x0D),   /* No such item                          */
     GT_TIMEOUT         = (0x0E),   /* Time Out                              */
     GT_NO_CHANGE       = (0x0F),   /* The parameter(s) is already in this   */
                                    /* value                                 */
     GT_NOT_SUPPORTED   = (0x10),   /* This request is not support           */
     GT_NOT_IMPLEMENTED = (0x11),   /* This request is supported but not     */
                                    /* implemented                           */
     GT_NOT_INITIALIZED = (0x12),   /* The item is not initialized           */
     GT_NO_RESOURCE     = (0x13),   /* Resource not available (memory ...)   */
     GT_FULL            = (0x14),   /* Item is full (Queue or table etc...)  */
     GT_EMPTY           = (0x15),   /* Item is empty (Queue or table etc...) */
     GT_INIT_ERROR      = (0x16),   /* Error occured while INIT process      */
     GT_HW_ERROR        = (0x17),   /* Hardware error                        */
     GT_TX_ERROR        = (0x18),   /* Transmit operation not succeeded      */
     GT_RCV_ERROR       = (0x19),   /* Recieve operation not succeeded       */
     GT_NOT_READY       = (0x1A),   /* The other side is not ready yet       */
     GT_ALREADY_EXIST   = (0x1B),   /* Tried to create existing item         */
     GT_OUT_OF_CPU_MEM  = (0x1C),   /* Cpu memory allocation failed.         */
     GT_OUT_OF_PP_MEM   = (0x1D),   /* PP memory allocation failed.          */
     GT_REDO            = (0x1E),   /* Re-perfrom the interrupt handling.    */
}GT_STATUS;


/* Define the different device type that may exist in system        */
typedef enum
{
    GT_88E6021  = 0x021,
    GT_88E6051  = 0x051,
    GT_88E6052  = 0x052,
    GT_88E6060  = 0x060,
    GT_88E6061	= 0x151,
    GT_88E6062	= 0x152,
    GT_88E6063	= 0x153,
    GT_FH_VPN_L	= 0xF51,
    GT_FH_VPN	= 0xF53,
    GT_FF_XP	= 0xF91,
    GT_FF_EG	= 0xF92,
    GT_FF_HG	= 0xF93
}GT_DEVICE;

typedef enum
{
    INTR_MODE_DISABLE =0,
    INTR_MODE_ENABLE
}INTERRUPT_MODE;


/* Maximal number of ports a switch may have.   */
#define MAX_SWITCH_PORTS    7

/* This macro checks for a multicast mac address    */
#define IS_MULTICAST_MAC(mac)  ((mac.arEther[0] & 0x1) == 1)

/*
GT_ETHERADDR mac;
*/

/* This macro checks for an broadcast mac address     */
#define IS_BROADCAST_MAC(mac)                                             \
            (((mac).arEther[0] == 0xFF) && ((mac).arEther[1] == 0xFF) &&    \
             ((mac).arEther[2] == 0xFF) && ((mac).arEther[3] == 0xFF) &&    \
             ((mac).arEther[4] == 0xFF) && ((mac).arEther[5] == 0xFF))

#define VERSION_MAX_LEN 30

/*
 * Typedef: struct GT_VERSION
 *
 * Description: This struct holds the package version.
 *
 * Fields:
 *      version - string array holding the version.
 *
 */
typedef struct
{
    GT_U8   version[VERSION_MAX_LEN];
}GT_VERSION;



/*
 * Typedef: enum GT_ATU_OPERARION
 *
 * Description: Defines the different ATU and VTU operations
 *
 * Fields:
 *      FLUSH_ALL           - Flush all entries.
 *      FLUSH_UNLOCKED      - Flush all unlocked entries in ATU.
 *      LOAD_PURGE_ENTRY    - Load / Purge entry.
 *      GET_NEXT_ENTRY      - Get next ATU or VTU  entry.
 *      FLUSH_ALL_IN_DB     - Flush all entries in a particular DBNum.
 *      FLUSH_UNLOCKED_IN_DB - Flush all unlocked entries in a particular DBNum.
 *      SERVICE_VIOLATONS   - sevice violations of VTU
 *
 */
typedef enum
{
    FLUSH_ALL = 1,	/* for both atu and vtu */
    FLUSH_UNLOCKED,	/* for atu only */
    LOAD_PURGE_ENTRY,	/* for both atu and vtu */
    GET_NEXT_ENTRY,	/* for both atu and vtu */
	FLUSH_ALL_IN_DB,	/* for atu only */
    FLUSH_UNLOCKED_IN_DB,	/* for atu only */
    SERVICE_VIOLATIONS 	/* for vtu only */
}GT_ATU_OPERARION, GT_VTU_OPERATION;


/*
 * typedef: enum GT_FLUSH_CMD
 *
 * Description: Enumeration of the address translation unit flush operation.
 *
 * Enumerations:
 *   GT_FLUSH_ALL       - flush all entries.
 *   GT_FLUSH_ALL_UNBLK - flush all unblocked.
 */
typedef enum
{
    GT_FLUSH_ALL       = 1,
    GT_FLUSH_ALL_UNBLK
}GT_FLUSH_CMD;


/*
 * typedef: enum GT_ATU_UC_STATE
 *
 * Description:
 *      Enumeration of the address translation unit entry state of unicast
 *      entris.
 *
 * Enumerations:
 *      GT_UC_INVALID   - invalid entry.
 *      GT_UC_DYNAMIC   - unicast dynamic entry.
 *      GT_UC_STATIC    - static unicast entry.
 */
typedef enum
{
    GT_UC_INVALID      = 0,
    GT_UC_DYNAMIC      = 0x1,
    GT_UC_STATIC       = 0xF
} GT_ATU_UC_STATE;


/*
 * typedef: enum GT_ATU_MC_STATE
 *
 * Description:
 *      Enumeration of the address translation unit entry state of multicast
 *      entris.
 *
 * Enumerations:
 *      GT_MC_INVALID         - invalid entry.
 *      GT_MC_MGM_STATIC      - static multicast management entries.
 *      GT_MC_STATIC          - static multicast regular entris.
 *      GT_MC_PRIO_MGM_STATIC - static multicast management entries with
 *                              priority.
 *      GT_MC_PRIO_STATIC     - static multicast regular entris with priority.
 */
typedef enum
{
    GT_MC_INVALID         = 0,
    GT_MC_MGM_STATIC      = 0x6,
    GT_MC_STATIC          = 0x7,
    GT_MC_PRIO_MGM_STATIC = 0xE,
    GT_MC_PRIO_STATIC     = 0xF
} GT_ATU_MC_STATE;



/*
 *  typedef: struct GT_ATU_ENTRY
 *
 *  Description: address tarnslaton unit Entry
 *
 *  Fields:
 *      macAddr    - mac address
 *      portVec    - port Vector.
 *      prio       - entry priority.
 *      entryState - the entry state.
 *		DBNum	   - ATU MAC Address Database number. If multiple address 
 *					databases are not being used, DBNum should be zero.
 *					If multiple address databases are being used, this value
 *					should be set to the desired address database number.
 *
 *  Comment:
 *      The entryState union Type is determine according to the Mac Type.
 */
typedef struct
{
    GT_ETHERADDR  macAddr;
    GT_U8         portVec;
    GT_U8         prio;
    GT_U8         DBNum;
    union
    {
        GT_ATU_UC_STATE ucEntryState;
        GT_ATU_MC_STATE mcEntryState;
    }             entryState;
} GT_ATU_ENTRY;


/*
 *  typedef: struct GT_VTU_DATA
 *
 *  Description: VLAN  tarnslaton unit Data Register
 *
 *  Fields:
 *      memberTagP - Membership and Egress Tagging
 *                   memberTagP[0] is for Port 0, MemberTagP[1] is for port 1, and so on 
 *
 *  Comment:
 * 	MAX_SWITCH_PORTS is 7 for ClipperShip. 
 * 	In the case of FullSail, there are 3 ports. So, the rest 4 is ignored in memeberTagP
 */
typedef struct
{
    GT_U8     memberTagP[MAX_SWITCH_PORTS];
    GT_U8     portStateP[MAX_SWITCH_PORTS];
} GT_VTU_DATA;

/*
 *  definition for MEMBER_TAG
 */
#define MEMBER_EGRESS_UNMODIFIED	0
#define NOT_A_MEMBER				1
#define MEMBER_EGRESS_UNTAGGED		2
#define MEMBER_EGRESS_TAGGED		3
 
/*
 *  typedef: struct GT_VTU_ENTRY
 *
 *  Description: VLAN  tarnslaton unit Entry
 *
 *  Fields:
 *      DBNum      - database number 
 *      vid        - VLAN ID 
 *      vtuData    - VTU data
 */
typedef struct
{
    GT_U8         DBNum;
    GT_U16         vid;
    GT_VTU_DATA   vtuData;
} GT_VTU_ENTRY;

/*
 *  typedef: struct GT_VTU_INT_STATUS
 *
 *  Description: VLAN tarnslaton unit interrupt status
 *
 *  Fields:
 *      intCause  - VTU Interrupt Cause
 *					GT_VTU_FULL_VIOLATION,GT_MEMEBER_VIOLATION,or
 *					GT_MISS_VIOLATION
 *      SPID      - source port number
 * 					if intCause is GT_VTU_FULL_VIOLATION, it means nothing
 *      vid       - VLAN ID 
 * 					if intCause is GT_VTU_FULL_VIOLATION, it means nothing
 */
typedef struct
{
    GT_U16   vtuIntCause;
    GT_U8    spid;
    GT_U16   vid;
} GT_VTU_INT_STATUS;

/*
* Definition for VTU interrupt
*/
#define GT_MEMBER_VIOLATION		0x4
#define GT_MISS_VIOLATION		0x2
#define GT_VTU_FULL_VIOLATION	0x1


/*
 * Typedef:
 *
 * Description: Defines the different sizes of the Mac address table.
 *
 * Fields:
 *      ATU_SIZE_256    -   256 entries Mac address table.
 *      ATU_SIZE_512    -   512 entries Mac address table.
 *      ATU_SIZE_1024   -   1024 entries Mac address table.
 *      ATU_SIZE_2048   -   2048 entries Mac address table.
 *      ATU_SIZE_4096   -   4096 entries Mac address table.
 *
 */
typedef enum
{
    ATU_SIZE_256,
    ATU_SIZE_512,
    ATU_SIZE_1024,
    ATU_SIZE_2048,
    ATU_SIZE_4096
}ATU_SIZE;


/*
 * typedef: enum GT_PORT_STP_STATE
 *
 * Description: Enumeration of the port spanning tree state.
 *
 * Enumerations:
 *   GT_PORT_DISABLE    - port is disabled.
 *   GT_PORT_BLOCKING   - port is in blocking/listening state.
 *   GT_PORT_LEARNING   - port is in learning state.
 *   GT_PORT_FORWARDING - port is in forwarding state.
 */
typedef enum
{
    GT_PORT_DISABLE = 0,
    GT_PORT_BLOCKING,
    GT_PORT_LEARNING,
    GT_PORT_FORWARDING
} GT_PORT_STP_STATE;


/*
 * typedef: enum GT_EGRESS_MODE
 *
 * Description: Enumeration of the port egress mode.
 *
 * Enumerations:
 *   GT_UNMODIFY_EGRESS - frames are transmited unmodified.
 *   GT_TAGGED_EGRESS   - all frames are transmited tagged.
 *   GT_UNTAGGED_EGRESS - all frames are transmited untagged.
 *   GT_ADD_TAG         - always add a tag. (or double tag)
 */
typedef enum
{
    GT_UNMODIFY_EGRESS = 0,
    GT_TAGGED_EGRESS,
    GT_UNTAGGED_EGRESS,
    GT_ADD_TAG
} GT_EGRESS_MODE;

/*  typedef: enum GT_DOT1Q_MODE */

typedef enum
{
	GT_DISABLE = 0,
    GT_FALLBACK,
	GT_CHECK,
	GT_SECURE
} GT_DOT1Q_MODE;


/* typedef: enum GT_SW_MODE */

typedef enum
{
	GT_CPU_ATTATCHED = 0, /* ports come up disabled */
	GT_BACKOFF,           /* EEPROM attac mode with old half duplex backoff mode */
	GT_STAND_ALONE,       /* ports come up enabled, ignore EEPROM */
        GT_EEPROM_ATTATCHED   /* EEPROM defined prot states */
} GT_SW_MODE;

/* typedef: enum GT_RATE_LIMIT_MODE
 * The ingress limit mode in the rate control register (0xA)
 */

typedef enum
{
	GT_LIMT_ALL = 0, 	/* limit and count all frames */
	GT_LIMIT_FLOOD,      	/* limit and count Broadcast, Multicast and flooded unicast frames */
	GT_LIMIT_BRDCST_MLTCST,	/* limit and count Broadcast and Multicast frames */
        GT_LIMIT_BRDCST   	/* limit and count Broadcast frames */
} GT_RATE_LIMIT_MODE;

/* typedef: enum GT_PRI0_RATE
 * The ingress data rate limit for priority 0 frames 
 */

typedef enum
{
	GT_NO_LIMIT = 0, 	/* Not limited   */
	GT_128K,      		/* 128K bits/sec */
	GT_256K,      		/* 256K bits/sec */
	GT_512,      		/* 512 bits/sec */
	GT_2M,      		/* 1M  bits/sec */
	GT_4M,      		/* 4M  bits/sec */
	GT_8M      		/* 8M  bits/sec */
} GT_PRI0_RATE,GT_EGRESS_RATE;

/*
 * typedef: enum GT_EVENT_TYPE
 *
 * Description: Enumeration of the available hardware driven events.
 *
 * Enumerations:
 *   GT_STATS_DONE - Statistics Operation Done interrrupt Enable 
 *   GT_VTU_PORB - VLAN Problem/Violation Interrupt Enable
 *   GT_VTU_DONE - VALN Table Operation Done Interrupt Enable
 *   GT_ATU_FULL - ATU full interrupt enable.
 *   GT_ATU_DONE - ATU Done interrupt enable.
 *   GT_PHY_INT  - PHY interrupt enable.
 *   GT_EE_INT   - EEPROM Done interrupt enable.
 */
#define GT_STATS_DONE           0x40
#define GT_VTU_PROB             0x20
#define GT_VTU_DONE             0x10
#define GT_ATU_FULL 		0x8
#define GT_ATU_DONE		0x4
#define GT_PHY_INTERRUPT	0x2
#define GT_EE_INTERRUPT		0x1

#define GT_INT_MASK		(GT_STATS_DONE | GT_VTU_PROB | GT_VTU_DONE | GT_ATU_FULL | GT_ATU_DONE | GT_PHY_INTERRUPT | GT_EE_INTERRUPT)

/*
* typedef: enum GT_PHY_INT
*
* Description: Enumeration of PHY interrupt
*/

#define GT_SPEED_CHANGED 		0x4000
#define GT_DUPLEX_CHANGED		0x2000
#define GT_PAGE_RECEIVED		0x1000
#define GT_AUTO_NEG_COMPLETED	0x800
#define GT_LINK_STATUS_CHANGED	0x400
#define GT_SYMBOL_ERROR			0x200
#define GT_FALSE_CARRIER		0x100
#define GT_FIFO_FLOW			0x80
#define GT_CROSSOVER_CHANGED	0x40
#define GT_POLARITY_CHANGED		0x2
#define GT_JABBER				0x1

/*
* typedef: enum GT_PHY_AUTO_MODE
*
* Description: Enumeration of Autonegotiation mode.
*	Auto for both speed and duplex.
*	Auto for speed only and Full duplex.
*	Auto for speed only and Half duplex.
*	Auto for duplex only and speed 100Mbps.
*	Auto for duplex only and speed 10Mbps.
*/

typedef enum
{
	SPEED_AUTO_DUPLEX_AUTO,
	SPEED_100_DUPLEX_AUTO,
	SPEED_10_DUPLEX_AUTO,
	SPEED_AUTO_DUPLEX_FULL,
	SPEED_AUTO_DUPLEX_HALF
}GT_PHY_AUTO_MODE;



/*
 * typedef: enum GT_INGRESS_MODE
 *
 * Description: Enumeration of the port ingress mode.
 *
 * Enumerations:
 *   GT_UNMODIFY_INGRESS - frames are receive unmodified.
 *   GT_TRAILER_INGRESS  - all frames are received with trailer.
 *   GT_REM_TAG_INGRESS  - remove tag on receive (for double tagging).
 */
typedef enum
{
    GT_UNMODIFY_INGRESS = 0,  /* 0x00 */
    GT_TRAILER_INGRESS,       /* 0x01 */
    GT_UNTAGGED_INGRESS,      /* 0x10 */
    GT_CPUPORT_INGRESS        /* 0x11 */
} GT_INGRESS_MODE;


/*
 *  typedef: enum GT_INGRESS_MODE
 *
 *  Description: Enumeration of the port ingress mode.
 *
 *  Enumerations:
 *      GT_MC_3_PERCENT_RL   - multicast rate is limited to 3 percent.
 *      GT_MC_6_PERCENT_RL   - multicast rate is limited to 6 percent.
 *      GT_MC_12_PERCENT_RL  - multicast rate is limited to 12 percent.
 *      GT_MC_100_PERCENT_RL - unlimited multicast rate.
 */
typedef enum
{
    GT_MC_3_PERCENT_RL = 0,
    GT_MC_6_PERCENT_RL,
    GT_MC_12_PERCENT_RL,
    GT_MC_100_PERCENT_RL,
} GT_MC_RATE;



/*
 *  typedef: struct GT_PORT_COUNTERS
 *
 *  Description: port statistic struct.
 *
 *  Fields:
 *      rxCtr    - port receive counter.
 *      txCtr    - port transmit counter.
 *
 *  Comment:
 */
typedef struct
{
    GT_U16  rxCtr;
    GT_U16  txCtr;
} GT_PORT_STAT;


/*
 **  typedef: struct GT_PORT_Q_COUNTERS
 **
 **  Description: port queue statistic struct.
 **
 **  Fields:
 **      OutQ_Size - port egress queue size coi
 **      Rsv_Size  - ingress reserved e counter
 **
 **/
typedef struct
{
    GT_U16  OutQ_Size;
    GT_U16  Rsv_Size;
} GT_PORT_Q_STAT;

/*
 * typedef: enum GT_CTR_MODE
 *
 * Description: Enumeration of the port counters mode.
 *
 * Enumerations:
 *   GT_CTR_ALL    - In this mode the counters counts Rx receive and transmit
 *                   frames.
 *   GT_CTR_ERRORS - In this mode the counters counts Rx Errors and collisions.
 */
typedef enum
{
    GT_CTR_ALL = 0,
    GT_CTR_ERRORS,
} GT_CTR_MODE;

typedef struct _GT_QD_DEV GT_QD_DEV;

/*
 * semaphore related definitions.
 * User Applications may register Semaphore functions using following definitions
 */
typedef enum
{
	GT_SEM_EMPTY,
	GT_SEM_FULL
} GT_SEM_BEGIN_STATE;

typedef GT_SEM (*FGT_SEM_CREATE)(
						GT_SEM_BEGIN_STATE state);
typedef GT_STATUS (*FGT_SEM_DELETE)(
						GT_SEM semId);
typedef GT_STATUS (*FGT_SEM_TAKE)(
						GT_SEM semId, GT_U32 timOut);
typedef GT_STATUS (*FGT_SEM_GIVE)(
						GT_SEM semId);

typedef struct
{
	FGT_SEM_CREATE	semCreate; 	/* create semapore */
	FGT_SEM_DELETE	semDelete; 	/* delete the semapore */
	FGT_SEM_TAKE	semTake;	/* try to get a semapore */
	FGT_SEM_GIVE	semGive;	/* return semaphore */
}GT_SEM_ROUTINES;

/*
 * definitions for registering MII access functions.
 *
*/
typedef GT_BOOL (*FGT_READ_MII)(
						GT_QD_DEV*   dev,
						unsigned int phyAddr, 
						unsigned int miiReg, 
						unsigned int* value);
typedef GT_BOOL (*FGT_WRITE_MII)(
						GT_QD_DEV*   dev,
						unsigned int phyAddr, 
						unsigned int miiReg, 
						unsigned int value);
typedef GT_BOOL (*FGT_INT_HANDLER)(
						GT_QD_DEV*   dev,
						GT_U16*);

typedef struct _BSP_FUNCTIONS
{
	FGT_READ_MII 	readMii;	/* read MII Registers */
	FGT_WRITE_MII 	writeMii;	/* write MII Registers */
	FGT_SEM_CREATE	semCreate; 	/* create semapore */
	FGT_SEM_DELETE	semDelete; 	/* delete the semapore */
	FGT_SEM_TAKE	semTake;	/* try to get a semapore */
	FGT_SEM_GIVE	semGive;	/* return semaphore */
}BSP_FUNCTIONS;


/*
 *	Type definition for MIB counter operation
*/
typedef enum 
{
	STATS_FLUSH_ALL,		/* Flush all counters for all ports */
	STATS_FLUSH_PORT,		/* Flush all counters for a port */
	STATS_READ_COUNTER,	/* Read a specific counter from a port */
	STATS_READ_ALL		/* Read all counters from a port */

} GT_STATS_OPERATION;

typedef struct _GT_STATS_COUNTER_SET
{
	GT_U32	InUnicasts;
	GT_U32	InBroadcasts;
	GT_U32	InPause;
	GT_U32	InMulticasts;
	GT_U32	InFCSErr;
	GT_U32	AlignErr;
	GT_U32	InGoodOctets;
	GT_U32	InBadOctets;
	GT_U32	Undersize;
	GT_U32	Fragments;
	GT_U32	In64Octets;
	GT_U32	In127Octets;
	GT_U32	In255Octets;
	GT_U32	In511Octets;
	GT_U32	In1023Octets;
	GT_U32	InMaxOctets;
	GT_U32	Jabber;
	GT_U32	Oversize;
	GT_U32	InDiscards;
	GT_U32	Filtered;
	GT_U32	OutUnicasts;
	GT_U32	OutBroadcasts;
	GT_U32	OutPause;
	GT_U32	OutMulticasts;
	GT_U32	OutFCSErr;
	GT_U32	OutGoodOctets;
	GT_U32	Out64Octets;
	GT_U32	Out127Octets;
	GT_U32	Out255Octets;
	GT_U32	Out511Octets;
	GT_U32	Out1023Octets;
	GT_U32	OutMaxOctets;
	GT_U32	Collisions;
	GT_U32	Late;
	GT_U32	Excessive;
	GT_U32	Multiple;
	GT_U32	Single;
	GT_U32	Deferred;
	GT_U32	OutDiscards;

} GT_STATS_COUNTER_SET;


typedef enum
{
	STATS_InUnicasts = 0,
	STATS_InBroadcasts,
	STATS_InPause,
	STATS_InMulticasts,
	STATS_InFCSErr,
	STATS_AlignErr,
	STATS_InGoodOctets,
	STATS_InBadOctets,
	STATS_Undersize,
	STATS_Fragments,
	STATS_In64Octets,
	STATS_In127Octets,
	STATS_In255Octets,
	STATS_In511Octets,
	STATS_In1023Octets,
	STATS_InMaxOctets,
	STATS_Jabber,
	STATS_Oversize,
	STATS_InDiscards,
	STATS_Filtered,
	STATS_OutUnicasts,
	STATS_OutBroadcasts,
	STATS_OutPause,
	STATS_OutMulticasts,
	STATS_OutFCSErr,
	STATS_OutGoodOctets,
	STATS_Out64Octets,
	STATS_Out127Octets,
	STATS_Out255Octets,
	STATS_Out511Octets,
	STATS_Out1023Octets,
	STATS_OutMaxOctets,
	STATS_Collisions,
	STATS_Late,
	STATS_Excessive,
	STATS_Multiple,
	STATS_Single,
	STATS_Deferred,
	STATS_OutDiscards

} GT_STATS_COUNTERS;

#define GT_MDI_PAIR_NUM     4

/*
 * typedef: enum GT_TEST_STATUS
 *
 * Description: Enumeration of VCT test status
 *
 * Enumerations:
 *      GT_TEST_FAIL    - virtual cable test failed.
 *      GT_NORMAL_CABLE - normal cable.
 *      GT_OPEN_CABLE   - open in cable.
 *      GT_SHORT_CABLE  - short in cable.
 *
 */
typedef enum
{
    GT_TEST_FAIL,
    GT_NORMAL_CABLE,
    GT_OPEN_CABLE,
    GT_SHORT_CABLE,

} GT_TEST_STATUS;


/*
 * typedef: enum GT_NORMAL_CABLE_LEN
 *
 * Description: Enumeration for normal cable length
 *
 * Enumerations:
 *      GT_LESS_THAN_50M - cable length less than 50 meter.
 *      GT_50M_80M       - cable length between 50 - 80 meter.
 *      GT_80M_110M      - cable length between 80 - 110 meter.
 *      GT_110M_140M     - cable length between 110 - 140 meter.
 *      GT_MORE_THAN_140 - cable length more than 140 meter.
 *      GT_UNKNOWN_LEN   - unknown length.
 *
 */
typedef enum
{
    GT_LESS_THAN_50M,
    GT_50M_80M,
    GT_80M_110M,
    GT_110M_140M,
    GT_MORE_THAN_140,
    GT_UNKNOWN_LEN,

} GT_NORMAL_CABLE_LEN;



/*
 * typedef: enum GT_CABLE_LEN
 *
 * Description: Enumeration cable length
 *
 * Enumerations:
 *      normCableLen - cable lenght for normal cable.
 *      errCableLen  - for cable failure the estimate fault distance in meters.
 *
 */
typedef union
{
    GT_NORMAL_CABLE_LEN normCableLen;
    GT_U8               errCableLen;

} GT_CABLE_LEN;



/*
 * typedef: struct GT_CABLE_STATUS
 *
 * Description: virtual cable diagnostic status per MDI pair.
 *
 * Enumerations:
 *      cableStatus - VCT cable status.
 *      cableLen    - VCT cable length.
 *	phyType	    - type of phy (100M phy or Gigabit phy)
 */
typedef struct
{
    GT_TEST_STATUS  cableStatus[GT_MDI_PAIR_NUM];
    GT_CABLE_LEN    cableLen[GT_MDI_PAIR_NUM];
    GT_U16	    phyType;

} GT_CABLE_STATUS;

/*
 * if isGigPhy in GT_CABLE_STATUS is not GT_TRUE, cableStatus and cableLen 
 * will have only 2 pairs available.
 * One is RX Pair and the other is TX Pair.
 */
#define MDI_RX_PAIR		0	/* cableStatus[0] or cableLen[0] */
#define MDI_TX_PAIR		1	/* cableStatus[1] or cableLen[1] */

/* definition for Phy Type */
#define PHY_100M		0 /* 10/100M phy, E3082 or E3083 */
#define PHY_1000M		1 /* Gigabit phy, the rest phys */
#define PHY_10000M		2 /* 10 Gigabit phy, unused */
#define PHY_1000M_B		3 /* Gigabit phy which needs work-around */


/*
 * Typedef: struct GT_SYS_CONFIG
 *
 * Description: System configuration Parameters struct.
 *
 * Fields:
 *	devNum		- Switch Device Number 
 *      cpuPortNum      - The physical port used to connect the device to CPU.
 *                        This is the port to which packets destined to CPU are
 *                        forwarded.
 *      initPorts       - Whether to initialize the ports state.
 *                          GT_FALSE    - leave in default state.
 *                          GT_TRUE     - Initialize to Forwarding state.
 *	BSPFunctions	- Group of BSP specific functions.
 *				SMI Read/Write and Semaphore Related functions.
 */
typedef struct
{
    GT_U8         devNum;
    GT_U8         cpuPortNum;
    GT_BOOL       initPorts;
    BSP_FUNCTIONS BSPFunctions;
}GT_SYS_CONFIG;



/*
 * Typedef: struct GT_QD_DEV
 *
 * Description: Includes Tapi layer switch configuration data.
 *
 * Fields:
 *      deviceId        - The device type identifier.
 *      revision        - The device revision number.
 *      baseRegAddr     - Switch Base Register address.
 *      numOfPorts      - Number of active ports.
 *      cpuPortNum      - The physical port connected to the CPU.
 *      maxPhyNum       - max configurable Phy address.
 *	stpMode         - current switch STP mode (0 none, 1 en, 2 dis)
 *      portsList       - List of all logical ports in system.
 *      devName	        - name of the device
 *	atuRegsSem	- Semaphore for ATU access
 *	vtuRegsSem	- Semaphore for VTU access
 *	statsRegsSem	- Semaphore for RMON counter access
 *	fgtReadMii	- platform specific SMI register Read function
 *	fgtWriteMii	- platform specific SMI register Write function
 *	semCreate	- function to create semapore
 *	semDelete	- function to delete the semapore
 *	semTake		- function to get a semapore
 *	semGive		- function to return semaphore
 *	appData	        - application data that user may use
 */
struct _GT_QD_DEV
{
    GT_DEVICE           deviceId;
    GT_U8   		revision;
    GT_U8		devNum;
    GT_U8		devEnabled;
    GT_U8      		baseRegAddr;
    GT_U8               numOfPorts;
    GT_U8               cpuPortNum;
    GT_U8               maxPhyNum;
    GT_U8		stpMode;
    GT_U32		devName;
    GT_SEM		atuRegsSem;
    GT_SEM		vtuRegsSem;
    GT_SEM		statsRegsSem;

    FGT_READ_MII 	fgtReadMii;
    FGT_WRITE_MII 	fgtWriteMii;

    FGT_SEM_CREATE	semCreate; 	/* create semapore */
    FGT_SEM_DELETE	semDelete; 	/* delete the semapore */
    FGT_SEM_TAKE	semTake;	/* try to get a semapore */
    FGT_SEM_GIVE	semGive;	/* return semaphore */
    void*		appData;

};


#ifdef __cplusplus
}
#endif

#endif /* __msApi_h */
