/**********************************************************
 rp.c
**********************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <setjmp.h>
#define MAIN
#include "decls.h"
#include "types.h"

#ifdef LITTLE
#define SWAPW(x) ((x << 8) | ((x >> 8) & 0xff))
#define SWAPL(x) (((x >> 24) & 0xff) \
		 | ((x >> 8) & 0xff00) \
		 | ((x << 8) & 0xff0000L) \
		 | ((x << 24) & 0xff000000L))
#else
#define SWAPW(x) (x)
#define SWAPL(x) (x)
#endif

/*----- Types and Constants Definition ----------------------*/
#define REG32(x) (*(volatile u32 *)(x))
#define REG16(x) (*(volatile u16 *)(x))
#define REG8(x) (*(volatile u8 *)(x))

int
mymemcmp(__packed void *p1, __packed void *p2, int);
void
mymemset(__packed void *p1, char, int);
void
mymemcpy1(__packed void *p1, void *p2, int);
void
mymemcpy2(void *p1, __packed void *p2, int);
void
mymemcpy3(__packed void *p1, __packed void *p2, int);

#include "transfer.c"

u32 spi_read4(u32 u32addr);
void spi_init(u16 u16strap);

void clear_6063_stats(void);
void capture_all_6063(void);
u32 get_6063_stat(u16 port, u16 stat);
void initcode(void);
void PortChange(int portnum, int en_dis);
void UDPTransmit(EthernetAddress dst, InternetAddress dst_ip, EthernetAddress src, InternetAddress src_ip, u16 dst_port, u16 src_port, void *payload, u32 len);

#define FLASH_MAC_LOCN ((u8 *)0xffffffb0)

EthernetAddress bcMAC={0xff, 0xff, 0xff, 0xff, 0xff, 0xff};    // BROADCAST MAC
//EthernetAddress myMAC={0xde, 0xad, 0xfa, 0xce, 0x01, 0x02};    // MAC of this device
EthernetAddress myMAC={0x00, 0x50, 0x43, 0x02, 0x00, 0xf0};    // MAC of this device
EthernetAddress dstMAC; // BOOTP reply packet will set this
InternetAddress myIP;     // IP of this device. ditto
InternetAddress dstIP;     // IP of remote device. ditto
InternetAddress zeroip={0, 0, 0, 0};
InternetAddress onesip={0xff, 0xff, 0xff, 0xff}; // For the broadcast BOOTP packet

// Ethernet protocol types
#define ETHERTYPE_IP   0x0800
#define ETHERTYPE_ARP  0x0806
#define ETHERTYPE_RARP 0x8035

// BOOTP commands
#define BOOTREQUEST     1
#define BOOTREPLY       2

#define MTU_SIZE	1532

#define ETNA_PORT	5
#define PORT_DISABLE	0
#define PORT_ENABLE	1

/*
 * Protocols
 */
#define	IPPROTO_IP		0			/* dummy for IP */
#define	IPPROTO_ICMP		1			/* control message protocol */
#define	IPPROTO_IGMP		2			/* group mgmt protocol */
#define	IPPROTO_GGP		3			/* gateway^2 (deprecated) */
#define	IPPROTO_TCP		6			/* tcp */
#define	IPPROTO_EGP		8			/* exterior gateway protocol */
#define	IPPROTO_PUP		12			/* pup */
#define	IPPROTO_UDP		17			/* user datagram protocol */
#define	IPPROTO_IDP		22			/* xns idp */
#define	IPPROTO_TP		29 			/* tp-4 w/ class negotiation */
#define	IPPROTO_EON		80			/* ISO cnlp */
#define	IPPROTO_ENCAP		98			/* encapsulation header */

#define	IPPROTO_RAW		255			/* raw IP packet */
#define	IPPROTO_MAX		256

u8 zero[4] = { 0, 0, 0, 0};
u8 fffs[4] = { 0xff, 0xff, 0xff, 0xff };
u8 req_filename[128]; // Override via BOOTP reply

typedef struct _option_handler {
    u8 option_type;
    s8 option_len;
    u8 option_set;
    u8 option_used;
    u8 option_data[4]; // Watch this carefully for overflow!
} option_handler;

option_handler dflt_options[]={
    { DHCP_OPTION_MESSAGE_TYPE, 1, 1, 1, {0} },  // Mark always set on this one
    { DHCP_OPTION_SERVER_IDENTIFIER, 4, 0, 1, {0} }, 
    { DHCP_OPTION_LEASE_TIME, 4, 0, 1, {0} }, 
    { DHCP_OPTION_RENEWAL_TIME, 4, 0, 1, {0} }, 
    { DHCP_OPTION_REBINDING_TIME, 4, 0, 1, {0} }, 
    { DHCP_OPTION_SUBNET_MASK, 4, 0, 1, {0} }, 
    { DHCP_OPTION_DNS_SERVER, 4, 0, 1, {0} }, 
    { DHCP_OPTION_DOMAIN_NAME, 4, 0, 0, {0} }, 
    { DHCP_OPTION_BROADCAST_ADDRESS, 4, 0, 1, {0} }, 
    { DHCP_OPTION_MAX_DHCP_MSG_SIZE, 2, 1, 1, {0x1, 0x6c}}, 
//    { DHCP_OPTION_LOG_SERVER, 4, 0, 0, {0} }, 
//    { DHCP_OPTION_NETBIOS_OVER_TCP, 4, 0, 0, {0} }, 
//    { DHCP_OPTION_BOOT_FILE_SIZE, 2, 0, 0, {0} }, 
//    { DHCP_OPTION_ROUTER, 4, 0, 0, {0} }
};

option_handler options[]={
    { DHCP_OPTION_MESSAGE_TYPE, 1, 1, 1, {0} },  // Mark always set on this one
    { DHCP_OPTION_SERVER_IDENTIFIER, 4, 0, 1, {0} }, 
    { DHCP_OPTION_LEASE_TIME, 4, 0, 1, {0} }, 
    { DHCP_OPTION_RENEWAL_TIME, 4, 0, 1, {0} }, 
    { DHCP_OPTION_REBINDING_TIME, 4, 0, 1, {0} }, 
    { DHCP_OPTION_SUBNET_MASK, 4, 0, 1, {0} }, 
    { DHCP_OPTION_DNS_SERVER, 4, 0, 1, {0} }, 
    { DHCP_OPTION_DOMAIN_NAME, 4, 0, 0, {0} }, 
    { DHCP_OPTION_BROADCAST_ADDRESS, 4, 0, 1, {0} }, 
    { DHCP_OPTION_MAX_DHCP_MSG_SIZE, 2, 1, 1, {0x1, 0x6c}}, 
//    { DHCP_OPTION_LOG_SERVER, 4, 0, 0, {0} }, 
//    { DHCP_OPTION_NETBIOS_OVER_TCP, 4, 0, 0, {0} }, 
//    { DHCP_OPTION_BOOT_FILE_SIZE, 2, 0, 0, {0} }, 
//    { DHCP_OPTION_ROUTER, 4, 0, 0, {0} }
};

/* This is where the zip data is saved (~2-300k)
 * CAREFUL HERE! A bad locn with too big a file coming in can cause big problems!
 * considef this buffer locn and the size of the image carefully. Also
 * remember that a decompressed file can overwrite this too potentially, so it
 * cannot be too close to the app either! The app grows up towards the COMPRESSED_BUFFER_LOCN
 * |0 <low-mem> ..... 0xd0000000 <DRAM>-><APP>>>>>>>>->|<COMPRESSED_BUFFER_LOCN>
 */
#define COMPRESSED_BUFFER_LOCN	0xd0500000
//#define FLASH_START 		0xfff00000  // Fail out to here.
//#define FLASH_START 		0xffe00000  // Fail out to here.  !!! HACK Must interrogate flash first!
//#define FLASH_START 		0xffc00000  // Fail out to here.

#define FACTORY_OFF 0x7000
#define FACTORY_LEN 0x400

#define USER_OFF 0x7400
#define USER_LEN 0xC00

#define	TFTP_DEFAULTSIZE_PACKET	512
//#define	TFTP_MAX_PACKET		1432
#define TFTP_MAX_PACKET	512

#define TFTP_RRQ	1
#define TFTP_WRQ	2
#define TFTP_DATA	3
#define TFTP_ACK	4
#define TFTP_ERROR	5
#define TFTP_OACK	6

typedef struct { 
    unsigned short opcode;
    union {
        char rrq[TFTP_DEFAULTSIZE_PACKET];
        struct {
            unsigned short block;
            char download[TFTP_MAX_PACKET];
        } data;
        struct {
            unsigned short block;
        } ack;
        struct {
            unsigned short errcode;
            char errmsg[TFTP_DEFAULTSIZE_PACKET];
        } err;
        struct {
            char data[TFTP_DEFAULTSIZE_PACKET+2];
        } oack;
    } u;
} TFTP_Hdr;

jmp_buf redo;
u32 flashbase;
u16 tftp_blocknum = 0; 			// start at 0 for the option ACK's ack
u16 tftp_opt_ack = 0;                   // 1 blocksize given. Use 512 otherwise
s32 tftp_blocksize = TFTP_MAX_PACKET;
u32 tftp_filesize;
u16 speak; 				// What port does the TFTP server code want us to talk on
u8 *memory_block = (u8 *)COMPRESSED_BUFFER_LOCN;
u8 *client_ptr;				// uncompressed APP is placed here incrementally
u8 tx_buf[MTU_SIZE];
u8 rx_buf[MTU_SIZE];

u32 transact = (u32)0xd744bf0d;
/*
 * UDP port numbers, server and client.
 */
#define IPPORT_BOOTPS   67
#define IPPORT_BOOTPC   68

#define BOOTREQUEST     1
#define BOOTREPLY       2

#define BOOTP_PAD       0
#define BOOTP_SUBNET    1
#define BOOTP_GATEWAY   3
#define BOOTP_DNS       6
#define BOOTP_HOSTNAME  12
#define BOOTP_END       0xff

union {
    void (*branch) (void);
    s32 hackfest;
} u;
union {
    u8  c[4];
    u16 s[2];
    u32       x;
} cvert;

s32 eth_poll(u8 *buffer, jmp_buf *);
void eth_transmit(u8 *buffer, s32 len);
void EIITransmit(EthernetAddress dst, EthernetAddress src, u16 proto, void *buf, u32 len);
s32 IPValidate(u8 proto, u8 *buf);
void unpack_and_go(u8 *block);
void setclock(void);
s32 do_dhcp_state(u8 mode);

DHCP_STATE dhcp_state = DHCPS_BEGIN;

u32 failed = 0;
//#define FAIL_LIMIT 500
u32 FAIL_LIMIT = 500; // A var now so we can alter the timeouts

/**************************************************************************
 *IPCHKSUM - Calculate Checksum IP Header
 *
 *Inputs - 
 *ip - a pointer to the IP header
 *len - length of the IP header
 *
 *Outpus -
 *The checksum of the IP header
 **************************************************************************/
u16 
ipchksum(u16 *ip, int len)
{
    u32 sum = 0;
    len >>= 1;
    while (len--) 
    {
        sum += *(ip++);
        if (sum > 0xFFFF)
            sum -= 0xFFFF;
    }
    return((~sum) & 0x0000FFFF);
}
// If this is an incoming ARP packet for us, make a reply
int reg6_result=0;
u32 whatval=0;
void
do_arp(u8 *buf)
{
    u32 pos;
    ARP_Hdr *ahdr;
    u16 operation;
    u8 reply[MTU_SIZE+1]; // One extra for implicit 0
    pos = sizeof(etherHdr);
    ahdr = (ARP_Hdr *) &buf[pos];
    if(mymemcmp(&myIP, &ahdr->targetIp, sizeof(InternetAddress))) { // It's not for us
	return;
    }
    // What ARP is this?
    operation = ahdr->operation;
    operation = SWAPW(operation);
    if(operation == ARP_REQUEST) {
	// Make a note of the HW address of the partner
	memset(reply, 0, MTU_SIZE+1);
	pos = sizeof(etherHdr);
	operation = ARP_REPLY;
	ahdr->operation = SWAPW(operation);
	mymemcpy1(&ahdr->senderHw, &myMAC, sizeof(EthernetAddress));
	mymemcpy1(&ahdr->targetHw, &dstMAC, sizeof(EthernetAddress));
	mymemcpy1(&ahdr->senderIp, &myIP, sizeof(InternetAddress));
	mymemcpy1(&ahdr->targetIp, &dstIP, sizeof(InternetAddress));
	mymemcpy2(&reply[pos], ahdr, sizeof(ARP_Hdr));

//	get_rmon();
//	for(;;) {
//	    int i;
//	    EIITransmit(dstMAC, myMAC, ETHERTYPE_ARP, reply, sizeof(ARP_Hdr));
//	    clear_6063_stats();
  	EIITransmit(dstMAC, myMAC, ETHERTYPE_ARP, reply, 100);
//	    get_rmon();
//	    EIITransmit(bcMAC, myMAC, ETHERTYPE_ARP, reply, sizeof(ARP_Hdr));
//	    EIITransmit(bcMAC, myMAC, ETHERTYPE_ARP, reply, 100);
//	}
    }
}
//
// Filter incoming packets looking for EII packets for us. 
// If we see one, filter on the protocol we desire. 
// Copy to the supplied buf and fill in the len
void
EIIReceive(EthernetAddress our, EthernetAddress *partner, u8 proto, void *buf, u32 *len)
{
    etherHdr *eth_II;
    s32 done = 0;
    u16 frame;

    while(!done) {
	*len = eth_poll(buf, &redo);
	eth_II = (etherHdr *)buf;
	// Either !bcast and !ours keep waiting
	if(   mymemcmp(&bcMAC, &eth_II->dst, sizeof(our))
	   && mymemcmp(&our, &eth_II->dst, sizeof(our))) { // It's not for us
	    FAIL_LIMIT = 2;
	    continue;
	}

	// We have a frame for us and it's protocol matches
	// Make a notation of who sent us this, we will use
	// this in further transmissions
	if(partner) { // Only used at the start
	    mymemcpy2(partner, &eth_II->src, 6);
	}

	frame = SWAPW(eth_II->protocol);
	switch(frame) {
	case ETHERTYPE_IP:
	    // Now check the rest
	    if(IPValidate(proto, buf)) {
		done = 1;
	    }
	    break;
	case ETHERTYPE_ARP:
	    // handle ARP
	    do_arp(buf);
	    break;
	}
    }
}
#ifdef PORT_PAD
u8 scratch[MTU_SIZE+2];
#endif
void
EIITransmit(EthernetAddress dst, EthernetAddress src, u16 proto, void *buf, u32 len)
{
    etherHdr eth_II;
    mymemcpy1(&eth_II.dst, &dst, sizeof(dst));
    mymemcpy1(&eth_II.src, &src, sizeof(src));
    eth_II.protocol = (u16)SWAPW(proto);

    mymemcpy2(buf, &eth_II, sizeof(etherHdr));

#ifdef PORT_PAD
    memcpy(scratch+2, buf, sizeof(etherHdr) + len);
    scratch[0] = 0;
    scratch[1] = 0;
    eth_transmit(scratch, sizeof(etherHdr) + len+2);
#else
    eth_transmit(buf, sizeof(etherHdr) + len);
#endif
}
// Return 1 if the requested protocol is seen
s32
IPValidate(u8 proto, u8 *buf)
{
    IPHdr *ip;
    ip = (IPHdr *)(buf+sizeof(etherHdr));
    if(ip->ip_p == proto) {
	return 1;
    } else {
	return 0;
    }
}
void
IPTransmit(EthernetAddress dst, InternetAddress dst_ip, EthernetAddress src, InternetAddress src_ip, void *buf, u32 len)
{
    u16 sum;
    s32 pos;
    IPHdr ip;
    ip.ip_hl = 5;
    ip.ip_v = 4;
    ip.ip_tos = 0;
    ip.ip_len = SWAPW((sizeof(IPHdr) + len));
//    ip.ip_id = 0; // This need only be set when fragmentation is used
    ip.ip_id = 0;
    ip.ip_off = 0;
    ip.ip_ttl = 0x80;
    ip.ip_p = IPPROTO_UDP;
    ip.ip_sum = 0; // ditto

    mymemcpy1(&ip.ip_src, &src_ip, 4);
    mymemcpy1(&ip.ip_dst, &dst_ip, 4);

    sum = ipchksum((u16 *)&ip, sizeof(IPHdr));
    
    ip.ip_sum = sum;

    pos = sizeof(etherHdr);

    mymemcpy2((u8 *)buf+pos, &ip, sizeof(IPHdr));
    
    EIITransmit(dst, src, ETHERTYPE_IP, buf, sizeof(IPHdr) + len);
}
s32
ParseBootPReply(u8 *buf, u32 len)
{
    BOOTP_Hdr *bootp_hdr;
    u32 pos;
    pos = sizeof(etherHdr) + sizeof(IPHdr) + sizeof(UDPHdr);

    bootp_hdr = (BOOTP_Hdr *)(buf + pos);

    mymemcpy2(&myIP, &bootp_hdr->yiaddr, sizeof(InternetAddress));
    mymemcpy2(&dstIP, &bootp_hdr->siaddr, sizeof(InternetAddress));

    // Need to check this really is BOOTP. Hack for now
    if(bootp_hdr->op == BOOTREPLY) {
	if(bootp_hdr->file[0]) {
	    strcpy((char *)req_filename, (char *)bootp_hdr->file);
	}
	return 1;
    } else {
	return 0;
    }
}
void
xyzzy()
{
}
s32
ParseDHCPReply(u8 *buf, u32 len, u8 mode)
{
    DHCP_Hdr *dhcp_hdr;
    u32 pos;
    u32 magic = SWAPL(DHCP_MAGIC);
    u8 *option;
    s32 done = 0;
#ifdef FROM_OUTSIDE
    IPHdr *ip;
    ip = (IPHdr *)(buf + sizeof(etherHdr));
#endif    
    pos = sizeof(etherHdr) + sizeof(IPHdr) + sizeof(UDPHdr);

    dhcp_hdr = (DHCP_Hdr *)(buf + pos);

    // Decode the DHCP message, if so framed
    if(dhcp_hdr->magic == magic) {
	return 0;
    }
    option = (u8 *)(dhcp_hdr + 1);
    while(!done) {
	s32 i, j;
	for(i=0;i < sizeof(options)/sizeof(option_handler);i++) {
	    if((*option == options[i].option_type) && (options[i].option_used)) {
		s32 rec_len;
		option++;
		rec_len = *option++;
		options[i].option_len = rec_len;
		options[i].option_set = 1;
		for(j=0;j < rec_len;j++) {
		    options[i].option_data[j] = *option++;
		}
		break;
	    }
	}
	if(i == sizeof(options)/sizeof(option_handler)) {
	    s32 len;
	    // Hmmm unknown. Skip.
	    option++;
	    len = *option;
	    option += (len + 1);
	}
	if(*option == 0xff) {
	    done = 1;
	}
    }
#ifdef FROM_OUTSIDE
    if (options[0].option_data[0] == DHCPACK) {
        if(ip->ip_src.s_addr != 0xffffffff)
           mymemcpy2(&dstIP, &ip->ip_src, 4);
    }
#endif
    if(options[0].option_data[0] == DHCPOFFER) {
	s32 i;
#ifdef FROM_OUTSIDE
    if(ip->ip_src.s_addr != 0xffffffff)
    mymemcpy2(&dstIP, &ip->ip_src, 4);
	
    if(mode)
#endif
    for(i=0;i < sizeof(options)/sizeof(option_handler);i++) {
	    if(DHCP_OPTION_SERVER_IDENTIFIER == (options[i].option_type)) {
		mymemcpy2(&dstIP, &options[i].option_data, 4);
	    }
	}
	mymemcpy2(&myIP, &dhcp_hdr->yiaddr, sizeof(InternetAddress));
	if(dhcp_hdr->file[0]) {
	    strcpy((char *)req_filename, (char *)dhcp_hdr->file);
	}
    }
    return options[0].option_data[0];
}
s32
UDPDecode(u16 dst_port, u16 *src_port, u8 *buf, u32 *len)
{
    s32 pos1;
    s32 pos2;
    UDPHdr *udp_hdr;
    u16 port;
    u16 wlen;
    IPHdr *ip;
    PseudoIPHdr phdr;
    u16 sum;
    u16 pack_len;

    mymemset(&phdr, 0, sizeof(PseudoIPHdr));
    pos1 = sizeof(etherHdr);
    ip = (IPHdr *)(buf+pos1);
    mymemcpy3(&phdr.ip_src, &ip->ip_src, sizeof(struct in_addr));
    mymemcpy3(&phdr.ip_dst, &ip->ip_dst, sizeof(struct in_addr));
    phdr.ip_pr = ip->ip_p;

    pos2 = sizeof(IPHdr)+sizeof(etherHdr);
    udp_hdr = (UDPHdr *)(buf+pos2);

    pack_len = udp_hdr->length;
    phdr.ip_sum = pack_len;
    // Swap for use in the little endian ARM
    pack_len = SWAPW(pack_len);

    pos1 = (sizeof(etherHdr) + sizeof(IPHdr) + sizeof(UDPHdr) - (sizeof(UDPHdr) + sizeof(PseudoIPHdr)));
    mymemcpy2(buf+pos1, &phdr, sizeof(PseudoIPHdr));

    sum = ipchksum((u16 *)(buf + pos1), pack_len+sizeof(PseudoIPHdr));

    port = SWAPW(udp_hdr->dstPort);
    if(!sum && (dst_port == port)) {
	*src_port = SWAPW(udp_hdr->srcPort);
	wlen = SWAPW(udp_hdr->length);
	*len = wlen;
	return 1;
    } else {
	return 0;
    }
}
void
UDPTransmit(EthernetAddress dst, InternetAddress dst_ip, EthernetAddress src, InternetAddress src_ip, u16 dst_port, u16 src_port, void *payload, u32 len)
{
    u8 buf[MTU_SIZE+1]; // One extra for implicit 0
    PseudoIPHdr phdr;
    UDPHdr udp_hdr;
    s32 pos;
    u16 sum;
    buf[MTU_SIZE] = 0;
    udp_hdr.srcPort = SWAPW(src_port);
    udp_hdr.dstPort = SWAPW(dst_port);
    udp_hdr.length = SWAPW((len + sizeof(UDPHdr)));
    udp_hdr.checksum = 0; // Fill in later;

    memcpy(buf+sizeof(udp_hdr)+sizeof(IPHdr)+sizeof(etherHdr), payload, len);

    // Pseudo header stuff
    mymemcpy1(&phdr.ip_src, &src_ip, sizeof(InternetAddress));
    mymemcpy1(&phdr.ip_dst, &dst_ip, sizeof(InternetAddress));
    phdr.ip_pr = IPPROTO_UDP;
    phdr.ip_zero = 0;
    phdr.ip_sum = SWAPW((len + sizeof(UDPHdr)));

    pos = sizeof(IPHdr)+sizeof(etherHdr);
    mymemcpy2(buf+pos, &udp_hdr, sizeof(UDPHdr));

    pos = (sizeof(etherHdr) + sizeof(IPHdr) + sizeof(UDPHdr) - (sizeof(UDPHdr) + sizeof(PseudoIPHdr)));
    mymemcpy2(buf+pos, &phdr, sizeof(PseudoIPHdr));
    // Psuedo header created and prepended

    // Compute the checksum from the pseudo header
    sum = ipchksum((u16 *)(buf+pos), len+sizeof(PseudoIPHdr)+sizeof(UDPHdr));
    // That is a big endian sum

    udp_hdr.checksum = sum;
    // This copy can be optimized to write just the checksum
    pos = sizeof(IPHdr)+sizeof(etherHdr);
    mymemcpy2(buf+pos, &udp_hdr, sizeof(UDPHdr));

    IPTransmit(dst, dst_ip, src, src_ip, &buf, sizeof(UDPHdr) + len);
}
s32
str2n(s8 *s)
{
    s8 *s_right;
    s32 pos;
    s32 sum=0;
    s32 digit;
    s_right = s;
    while(*s_right) {
	s_right++;
    }
    // s_right points at the trailing 0.
    s_right--;
    s--;
    pos = 1;
    while(s != s_right) {
	digit = *s_right - '0';
	sum = sum + digit*pos;
	s_right--;
	pos *= 10;
    }
    return sum;
}
s32
ParseTFTPReply(u8 *buf, u32 len, s32 *last)
{
    u32 pos;
    TFTP_Hdr *hdr;
    u16 opcode;
    s32 need_ack = 1;
    u16 curr_block;
    s8 *p;
    u32 true_len;

    *last = 0;

    pos = sizeof(etherHdr) + sizeof(IPHdr) + sizeof(UDPHdr);

    hdr = (TFTP_Hdr *)&buf[pos];

    opcode = hdr->opcode;
    opcode = SWAPW(opcode);

    switch(opcode) {
    case TFTP_RRQ:
	break;
    case TFTP_WRQ:
	break;
    case TFTP_DATA:
	curr_block = hdr->u.data.block;
	curr_block = SWAPW(curr_block);
	tftp_blocknum = curr_block;
	true_len = len - (sizeof(UDPHdr) + 4);
	// Second test is a... *HACK* Seems when prepared for eeprom
	// the thing wont fire on the first test... debug this later!!!

	if(true_len == 0) {
        tftp_filesize = (curr_block - 1) * tftp_blocksize;
	    *last = 1;
	} else if(true_len < TFTP_MAX_PACKET) {
	    pos = (curr_block-1)*tftp_blocksize;
	    memcpy(&client_ptr[pos], hdr->u.data.download, true_len);
        tftp_filesize = pos + true_len;
	    *last = 1;
	} else {
	    if(!tftp_opt_ack) {
		tftp_blocksize = TFTP_DEFAULTSIZE_PACKET;
	    }
	    pos = (curr_block-1)*tftp_blocksize;
	    memcpy(&client_ptr[pos], hdr->u.data.download, true_len);
	}
	break;
    case TFTP_ACK:
	break;
    case TFTP_ERROR:
	break;
    case TFTP_OACK:
	// need to parse this
	p = hdr->u.oack.data;
	if(!strcmp(p, "blksize")) {
	    s8 *p2;
	    p2 = p;
	    while(*p2) {
		p2++;
	    }
	    p2++;
	    tftp_blocksize = str2n(p2);
	    tftp_opt_ack = 1;
	}
	break;
    }
    return need_ack;
}
s32
do_tftp_download()
{
    u8 buf[MTU_SIZE];
    TFTP_Hdr hdr;
//    u8 *str=(u8 *)"wlan_firmware_img1mb.bin octet blksize 1432 ";
//    u8 *str=(u8 *)" octet blksize 1432 ";
    u8 *str=(u8 *)" octet blksize 512 ";
    u32 len;
    u32 temp;
    s32 done = 0;
    s32 go_tftp = 0; // 1 when TFTP is operating
    client_ptr = (u8 *)memory_block;
    memset(&hdr, 0, sizeof(TFTP_Hdr));
    // Talk to a TFTP server and get a firmware image
    hdr.opcode = SWAPW(TFTP_RRQ);
    len = sizeof(hdr.opcode) + strlen((char *)req_filename) + strlen((char *)str) + 1;
    strcpy(hdr.u.rrq, (char *)req_filename);
    strcat(hdr.u.rrq, (char *)str);
    for (temp = 0 ; temp < len ; temp ++) {
        if (((char *)hdr.u.rrq)[temp] == ' ') ((char *)hdr.u.rrq)[temp] = 0;
    }

    setjmp(redo);

    UDPTransmit(dstMAC, dstIP, myMAC, myIP, 69, 2001, &hdr, len);

    while(!done) {
	// setjmp here. Inside eth_poll there is a timeout. Should it timeout
	// it will longjmp to here, and continue to wait for packets.
	// We only allow the setjmp if TFTP has started. If TFTP never
	// got going, re-submit the TFTP request
	if(go_tftp) {
	    setjmp(redo);
	}
	
	if(failed == FAIL_LIMIT) {
	    return 0;
	}
	    
	EIIReceive(myMAC, 0, IPPROTO_UDP, &buf, &len);
	if(UDPDecode(2001, &speak, buf, &len) == 0) { // is the port for us?
	    continue;
	}
	// We know this UDP frame is good, 1 the port is for us, the 
	// checksum is good. 

	// First ack goes out with blocknum == 0
	// We derive the blocknum from the data packet. I think this is safe
	// because the server will send 1.. 2.. 3.. etc 
	// we must ACK in order so any drops on the server->us will never
	// get an ack and re-send is forced. We deduce the code location
	// from the blocknum and blocksize.
	if(ParseTFTPReply(buf, len, &done)) { // True when ACK is required
	    hdr.opcode = SWAPW(TFTP_ACK);
	    hdr.u.ack.block = SWAPW(tftp_blocknum);
//	    UDPTransmit(dstMAC, dstIP, myMAC, myIP, speak, 2001, &hdr, 4);
	    UDPTransmit(dstMAC, dstIP, myMAC, myIP, speak, 2001, &hdr, 100);
	    go_tftp = 1;
	}
    }
    return 1;
}
u8 *
setup_dhcp(u8 *buf)
{
    u8 *option;
    u8 *option_start;
    s32 i, j;
    u32 len;
    DHCP_Hdr *dhcp_hdr;
    dhcp_hdr = (DHCP_Hdr *)buf;
    option = (u8 *)(dhcp_hdr + 1);
    memset(buf, 0, MTU_SIZE);
    dhcp_hdr->op = DHCP_BOOTREQUEST;
    dhcp_hdr->htype = DHCP_ETHERNET;
    dhcp_hdr->hlen = 6;
    dhcp_hdr->hops = 0;
    dhcp_hdr->xid = SWAPL(transact); 
//    dhcp_hdr->flags = SWAPW(0x8000); // Set DHCP
    dhcp_hdr->flags = SWAPW(0x0);
    dhcp_hdr->secs = 0;
    dhcp_hdr->magic = SWAPL(DHCP_MAGIC);

    memcpy(&dhcp_hdr->chaddr, &myMAC, sizeof(myMAC));

    option_start = option;

    // set message type DHCPREQUEST
    // Loop over all options, sending the saved values
    for(i=0;i < sizeof(options) / sizeof(option_handler);i++) {
	if(options[i].option_set && options[i].option_used) {
	    *option++ = options[i].option_type;
	    *option++ = len = options[i].option_len;
	    for(j=0;j < len;j++) {
		*option++ = options[i].option_data[j];
	    }
	}
    }
    *option++ = DHCP_OPTION_PARAMETER_LIST_REQUEST;
    *option++ = 8;
    *option++ = DHCP_OPTION_SERVER_IDENTIFIER;
    *option++ = DHCP_OPTION_LEASE_TIME;
    *option++ = DHCP_OPTION_RENEWAL_TIME;
    *option++ = DHCP_OPTION_REBINDING_TIME;
    *option++ = DHCP_OPTION_SUBNET_MASK;
    *option++ = DHCP_OPTION_DNS_SERVER;
    *option++ = DHCP_OPTION_DOMAIN_NAME;
    *option++ = DHCP_OPTION_BROADCAST_ADDRESS;
    // end option

    if(dhcp_state == DHCPS_REQUEST) {
	// One final one if this is a REQUEST
	// set ip address we're requesting
	*option++ = DHCP_OPTION_REQUESTED_IP_ADDRESS;
	*option++ = 4;
	*option++ = myIP.ip1;
	*option++ = myIP.ip2;
	*option++ = myIP.ip3;
	*option++ = myIP.ip4;
    }
    *option++ = DHCP_OPTION_END;
    while (((long)(option-option_start)) < 300) {
	*option++ = DHCP_OPTION_PAD;
    }
    return option;
}
s32
do_dhcp_state(u8 mode)
{
    u8 *option;
    s32 resolved = 0;
    u32 len;
    u16 src_port;
    s32 got;

    while(!resolved) {
	switch(dhcp_state) {
	case DHCPS_BEGIN:
#ifdef FROM_OUTSIDE
        dstIP.ip1 = 0xff;
        dstIP.ip2 = 0xff;
        dstIP.ip3 = 0xff;
        dstIP.ip4 = 0xff;
#endif
	    options[0].option_data[0] = DHCPDISCOVER;
	    option = setup_dhcp(tx_buf);
	    UDPTransmit(bcMAC, onesip, myMAC, zeroip, IPPORT_BOOTPS, IPPORT_BOOTPC, tx_buf, option - tx_buf);
	    dhcp_state = DHCPS_DISCOVER;
	    break;
	case DHCPS_DISCOVER:
	    EIIReceive(bcMAC, &dstMAC, IPPROTO_UDP, &rx_buf, &len);
	    if(UDPDecode(IPPORT_BOOTPC, &src_port, rx_buf, &len) == 0) { // is the port for us?
		continue;
	    }
	    
	    switch((got=ParseDHCPReply(rx_buf, len, mode))) {
	    case DHCPOFFER: // This we get, and process
		// Take the offer and compose a reply
		dhcp_state = DHCPS_REQUEST;
		continue;
		break;
	    default: // We don't understand any other message
		longjmp(redo, 1);
		break;
	    }
	    break;
	case DHCPS_REQUEST: // We are done
	    options[0].option_data[0] = DHCPREQUEST;
	    option = setup_dhcp(tx_buf);
//	    UDPTransmit(dstMAC, dstIP, myMAC, myIP, IPPORT_BOOTPS, IPPORT_BOOTPC, tx_buf, option - tx_buf);
	    UDPTransmit(bcMAC, onesip, myMAC, zeroip, IPPORT_BOOTPS, IPPORT_BOOTPC, tx_buf, option - tx_buf);
	    // Now wait for the DHCP ACK
	    EIIReceive(myMAC, &dstMAC, IPPROTO_UDP, &rx_buf, &len);
	    if(UDPDecode(IPPORT_BOOTPC, &src_port, rx_buf, &len) == 0) { // is the port for us?
		continue;
	    }
	    switch((got=ParseDHCPReply(rx_buf, len, mode))) {
	    case DHCPACK: // This we get, and process
	    case DHCPREQUEST:
		// We are bound. return
		return 1;
		break;
	    }
	    break;
	default:
	    longjmp(redo, 1);
	    break;
	}
    }
    return 1;
}
s32
do_bootp()
{
    BOOTP_Hdr bootp_hdr;
    s32 resolved = 0;
    u32 len;
    u16 src_port;
    u32 x;

    memset(&bootp_hdr, 0, sizeof(bootp_hdr));
    bootp_hdr.op = BOOTREQUEST;
    bootp_hdr.htype = 1;
    bootp_hdr.hlen = 6;
    bootp_hdr.hops = 0;
    x = 0xd744bf0d;
    bootp_hdr.xid = SWAPL(x); 
    bootp_hdr.secs = 0;
    bootp_hdr.unused = 0;
    memcpy(&bootp_hdr.chaddr, &myMAC, sizeof(myMAC));

    while(!resolved) {
	UDPTransmit(bcMAC, onesip, myMAC, zeroip, IPPORT_BOOTPS, IPPORT_BOOTPC, &bootp_hdr, sizeof(bootp_hdr));
	EIIReceive(bcMAC, &dstMAC, IPPROTO_UDP, &rx_buf, &len);
	if(UDPDecode(IPPORT_BOOTPC, &src_port, rx_buf, &len) == 0) { // is the port for us?
	    failed++;
	    if(failed == FAIL_LIMIT) {
		return 0;
	    }
	    continue;
	}
	resolved = ParseBootPReply(rx_buf, len); // Sets our IP address
    }
    return 1;
}
u32 largecount=10000000;

void
send_done()
{
    PkHeader hdr;
    APICMDBUF cmd;
    u8 buf[3400];
    u8 buf1[3400];
    u32 len;
    u8 *ptr;
    u32 i;
    u32 tmp;

    memset(&hdr, 0, sizeof(hdr));
    hdr.id = 1;
    memcpy(&buf, &hdr, sizeof(hdr));
    memset(&cmd, 0, sizeof(cmd));
    cmd.cmdCode = 8888; // Magic reply
    cmd.cmdBodyLen = 1024;
//    cmd.cmdBodyLen = 0;
    cmd.cmdStatus = APCTL_OK;
    memcpy(&buf, &hdr, sizeof(hdr));
    memcpy(buf+sizeof(hdr), &cmd, sizeof(cmd));
#ifdef FLASH
    // Factory settings are here Nth -2 sector.
    memcpy(buf1, (u8 *)0xfffe0400, cmd.cmdBodyLen);
#else
    ptr = buf1;
    for(i=0;i < (USER_LEN/4);i++) {
	u32 result;
	result = spi_read4(USER_OFF + (i << 2));
	memcpy(ptr, &result, 4); // just in case it's misaligned
	ptr += 4;
    }
#endif
    ptr = buf+sizeof(hdr)+sizeof(cmd);
    memset(ptr,0,sizeof(magic_mib_AP_t));
    transfer(buf1, ptr); 

#if 1
// HACK vvvvvvvvvvvv ETNA is off till now. Re-DHCP. I disable PC TFTP/DHCP in the delay here.

    PortChange(ETNA_PORT, PORT_ENABLE);
#ifdef FROM_OUTSIDE
    while(largecount--);
#endif
    memcpy(&options, &dflt_options, sizeof(options));
    memcpy(&myIP, &onesip, sizeof(myIP));
    transact++;

    setjmp(redo);

    dhcp_state = DHCPS_BEGIN;

    do_dhcp_state(1);
// HACK ^^^^^^^^^^^^^^^^^^
#endif

    len = sizeof(hdr)+sizeof(cmd)+cmd.cmdBodyLen;
    UDPTransmit(dstMAC, dstIP, myMAC, myIP, 9931, 9930, &buf, len);

#if 0
    for(i=0;i < 4;i++) {
	PortChange(i, PORT_ENABLE);
    }
#endif
    getstats();

    // Save the IP addr to an address
    
    memcpy(&tmp, &myIP, 4);
    tmp = SWAPL(tmp);
    memcpy((char *)0xc000bf7c, &tmp, 4);

    REG32(0x8000d00c) = 0x8000;
}

/**********************************************************
 flash driver
**********************************************************/
#ifdef FLASH

#define SIZE_FACTORY_CONFIG    0x400
#define SST_FLASH              0xBF
#define GPIO_RESET_TO_DEFAULT  5

#define FALSE 0
#define TRUE  1

s32 split_sector;

u32 flashsize;
u32 flashNsect;

u32 flash_config_sector;
u32 offset_factory_config;
u32 offset_user_config;

u32 manu;
u32 part;

void flashPreamble()
{
    REG16(flashbase | 0xAAAA) = 0x00AA;
    REG16(flashbase | 0x5554) = 0x0055;
    REG16(flashbase | 0xAAAA) = 0x0080;
    REG16(flashbase | 0xAAAA) = 0x00AA;
    REG16(flashbase | 0x5554) = 0x0055;
}

int flashEraseSectors(int sblk, int nblks)
{
    unsigned addr, start;
    int i, j;

    for (i = sblk;i < (sblk + nblks);i++)
    {
        addr = flashbase + (i<<16);
        if (i != split_sector)
        {
            flashPreamble();
            if (manu == SST_FLASH)
                REG16(addr) = 0x0050;
            else
                REG16(addr) = 0x0030;
            while (REG16(addr) != 0xFFFF);
        }
        else
        {
            start = addr;
            for(j=0; j<8; j++)
            {
                addr = start + j * 0x2000; // 8kb
                flashPreamble();
                REG16(addr) = 0x0030;
                while (REG16(addr) != 0xFFFF);
            }
        } // if split
    } // for
    return TRUE;
}

int flashWriteWord( int offset, unsigned short dat)
{
    unsigned addr = flashbase | offset;
    unsigned short tmp;

    /*flags = apctlIntDisable(); */
    REG16(flashbase | 0xAAAA) = 0x00AA;  /* word address 555 */
    REG16(flashbase | 0x5555) = 0x0055;  /* word address 2AA */
    REG16(flashbase | 0xAAAA) = 0x00A0;  /* word address 555 */

    /* if flash is not erase we don't write (avoid infinit loop) */
    /* we can change a bit from 1 to 0 but not from 0 to 1 */
    tmp = REG16 (addr);
    if (~tmp & dat)
        return FALSE;   /* flash not erased */

    REG16(addr) = dat;

    /* spin here until operation is done */
    while (REG16(addr) != dat);

    return TRUE;
}

int flashDetect()
{
    /* get info from the flash */
    REG16(0xfffb0000) = 0xf0; /* reset */
    REG16(0xfffb0000 | 0xAAAA) = 0x00AA;  /* word address 555 */
    REG16(0xfffb0000 | 0x5555) = 0x0055;  /* word address 2AA */
    REG16(0xfffb0000 | 0xAAAA) = 0x0090;  /* word address 555 */
    manu = REG16(0xfffb0000);     /* Manu */
    part = REG16(0xfffb0000 + 2); /* part ID. */
    REG16(0xfffb0000) = 0xf0; /* reset */

    /* is it a manufacturer we know and have we heard of the part? */
    switch (manu)
    {
    case SST_FLASH:  /* SST */
        switch (part)
        {
        case 0x2780: // SST39LF/VF400A
        case 0xd7:   // SST39LF/VF040
            flashNsect   = 8;
            split_sector = -1;
            break;
        case 0x2781: // SST39LF/VF800A
        case 0xd8:   // SST39LF/VF080
            flashNsect   = 16;
            split_sector = 15;
            break;
        case 0x2782: // SST39LF/VF160A
        case 0xd9:   // SST39LF/VF016
        case 0x234A: // SST39VF1602
		case 0x234B: // SST39VF1601
            flashNsect   = 32;
            split_sector = -1;
            break;
        case 0x235A: // SST39VF3202
        case 0x235B: // SST39VF3201
        case 0x2783: // SST39VF320A
            flashNsect   = 64;
            split_sector = -1;
            break;
        case 0x236A:  /* SST39VF6402 */
        case 0x236B:  /* SST39VF6401 */
            flashNsect = 128;
            split_sector = -1;
            break;
        default:
            return FALSE;
        }
        break;
    default:  /* AMD, MXIC, HYNIX, FUJITSU... */
        switch (part)
        {
        case 0x22da:  /* LV800BT */
        case 0x22d6: // F800T
            flashNsect   = 16;
            split_sector = 15;
            break;
        case 0x225b:  /* LV800BB */
        case 0x2258: // F800B
            flashNsect   = 16;
            split_sector = 0;
            break;
        case 0x38: // LV081
        case 0xd5: // F080
            flashNsect   = 16;
            split_sector = -1;
            break;
        case 0x22c4:  /* 160BT */
        case 0x22d2: // F160T
            flashNsect   = 32;
            split_sector = 31;
            break;
        case 0x2249:  /* 160BB */
        case 0x22d7: // LV160EB
        case 0x22d8: // F160B
            flashNsect   = 32;
            split_sector = 0;
            break;
        case 0xc8: // LV017
        case 0xad: // F016
        case 0x3d: // F017
            flashNsect   = 32;
            split_sector = -1;
            break;
        case 0x22f6: // LV320DT
        case 0x22a7: // LV320AT
            flashNsect   = 64;
            split_sector = 63;
            break;
        case 0x22f9: // LV320DB
        case 0x22a8: // LV320AB
            flashNsect   = 64;
            split_sector = 0;
            break;
        default:
            return FALSE;
        }
    }

    flashsize = flashNsect * 0x10000;
    flashbase = (0 - flashsize);
#ifdef ECOS
    flash_config_sector = flashNsect - 2;
#else /* VxWorks settings for now to avoid conflict with armboot. */
    flash_config_sector = flashNsect - 3;
#endif
    offset_factory_config = flash_config_sector * 0x10000;
    offset_user_config = offset_factory_config + SIZE_FACTORY_CONFIG;

    return TRUE;
}

int RestoreFactoryDefault()
{
    unsigned long i, len;
    unsigned short *addr_from, *addr_to;

    // read factory configuration to sdram.
    len = SIZE_FACTORY_CONFIG;
    addr_from = (unsigned short *) (flashbase + offset_factory_config);
    addr_to = (unsigned short *) memory_block;
    memcpy((u8 *)addr_to, (u8 *)addr_from, len);

    // copy factory config to user config
    len = SIZE_FACTORY_CONFIG;
    addr_from = (unsigned short *) (flashbase + offset_factory_config);
    addr_to = (unsigned short *) (memory_block + SIZE_FACTORY_CONFIG);
    memcpy((u8 *)addr_to, (u8 *)addr_from, len);

    // erase configuration sector.
    if (flashEraseSectors(flash_config_sector, 1) == FALSE)
    {
        return FALSE;
    }

    // write factory and user configuration.
    len = SIZE_FACTORY_CONFIG * 2;
    addr_from = (unsigned short *) memory_block;
    for(i=0; i<len; i+=2)
    {
        flashWriteWord(offset_factory_config + i, *addr_from);
    }

    return TRUE;
}

void program_image()
{
    int i;
    unsigned short *addr = (unsigned short *)memory_block;

    /* Image ought to be TFTP to SDRAM by now. */
    // Erase sector
    flashEraseSectors(0, flash_config_sector - 1);

    for(i=0; i<tftp_filesize; i+=2)
    {
        flashWriteWord(i, *addr++);
    }
}
#endif // FALSH

/**********************************************************
 main
**********************************************************/
void write_gpio(unsigned x)
{
    *(volatile unsigned *)0x8000d00c = x;
}

unsigned read_gpio(void)
{
	return (*(volatile unsigned *)(0x8000d010));
}

void ClearMem(void)
{
    memset(0, 0, 128*1024); // clear ITCM.
    memset((char *)0xc0000000, 0, 48*1024 - 132);
    memset((char *)0xd0000000, 0, 1*1024*1024);
}

u32 result=0xfacefeed;

s32
main()
{
    s32 reply;
    u32 result;
    ClearMem();
    flashbase = 0xfff00000;

#if (defined(CONDOR) && defined(CONDOR_FE_CLOCK_OUT))
    setclock();
#endif

    strcpy((char *)req_filename, BOOT_FILE);

    write_gpio(~(0x2));
#ifdef CONDOR2	/** Pls see P4 change history for comments **/
  *(volatile unsigned *)0x80002064 = 0xc201; /** for condor 2 only **/
#endif
	
    initcode();

#if 0
    int i;
    for(i=0;i < 4;i++) {
	PortChange(i, PORT_DISABLE);
    }
#endif

#ifdef W88E6063
    clear_6063_stats();
    capture_all_6063();
#endif

//    seeall();

#ifdef HUB_ONLY
    for(;;); // This dead loop is a hack to make a HUB if initcode
             // has learn disabled
#endif

#ifdef FLASH
    // detect flash size
    flashDetect();

    // if GPIO is depressed when system is being turned on.
    if ((read_gpio() & (1<<GPIO_RESET_TO_DEFAULT)) == 0)
    {
        // restore factory default configuration.
        RestoreFactoryDefault();
    }

    if(*FLASH_MAC_LOCN != 0xff) {
	memcpy((u8 *)&myMAC, FLASH_MAC_LOCN, 6);
    }
//    FAIL_LIMIT = 5;
    FAIL_LIMIT = ~0;
#else
    spi_init(REG16(0x80002000));
    result = spi_read4(0);
    FAIL_LIMIT = ~0; // Never fail

    result = spi_read4(0xc);
    result = SWAPL(result);
    memcpy(&myMAC.mac3, &result, 4);
    result = spi_read4(0x10);
    result = SWAPL(result);
    memcpy(&myMAC.mac1, ((unsigned char *)&result)+2, 2);
#endif

#if defined(DHCP) || defined(BOOTP)
#if defined(SWITCH_INIT) && defined(FROM_OUTSIDE)
    PortChange(ETNA_PORT, PORT_DISABLE);
#endif
    for(;;)
    {
	setjmp(redo);
	failed++;
           if(failed >= FAIL_LIMIT)
           {
	    int tmp;
	    memcpy(&myIP, (char *)0xfffe0504, 4);
               memory_block = (u8 *)flashbase;
	    break;
	}
	reply = 0;
        write_gpio(~(0x1));
#if defined(DHCP)
	dhcp_state = DHCPS_BEGIN;
	reply = do_dhcp_state(0);
#elif defined(BOOTP)
	reply = do_bootp();
#else
#error "Need a boot mode. -DDHCP or -DBOOTP"
#endif
	// Retry for tftp here. Drop in if
        if ( reply )
        {
            write_gpio(~(0x2));
            if ( do_tftp_download() )
            {
                // write image to flash
                //program_image();
                break;
            }
            else
            {
                memory_block = (u8 *)flashbase;
                break;
            }
        }
        else
        {
            memory_block = (u8 *)flashbase;
            break;
        }
    }
#else
    memory_block = (u8 *)flashbase;
#endif // DHCP || BOOTP

    ClearMem();
    // Now process the compressed data @memory_block
    write_gpio(~(0x3));
#ifdef CONDOR2
   *(volatile unsigned *)0x80002030 = 0x00007931; /* 160 Mhz */
#endif

    unpack_and_go(memory_block);
    for(;;);
}
