#include "decls.h"
#include "types.h"

/*----- Types and Constants Definition ----------------------*/
#define REG32(x) (*(volatile u32 *)(x))
#define REG8(x) (*(volatile u8 *)(x))
#define REG16(x) (*(volatile u16 *)(x))

u16 g_u16AddrLen;

#define EAROM_SIZE (32*1024)

#define CON4 (1 << 4)

#define SPI_WREN 0x420
#define SPI_ERASE 0x420
#define SPI_WRDI 0x410
#define SPI_READ_ADDR_DATA(addr_len, data_len)	(0x402 | ((((addr_len) + (data_len)) - 1) << 6))
#define SPI_WRITE_ADDR_DATA(addr_len, data_len)	(0x401 | ((((addr_len) + (data_len)) - 1) << 6))
#define SPI_RSTATUS		0x408	// spi start, len 0, read status

#define READ		          0x3   // Command that the eeprom uses for read
#define RDSR 		5
#define WREN 		6
#define WRDI 		4
#define PP   		2
#define BULK_ERASE 0xC7

void
spi_init( u16 u16strap )
{
    REG32(0x8000c000) = 1; // soft reset p_rst_
    REG32(0x8000c004) = 0; // cs_pclk2 = gclk
    REG32(0x8000c100) = 8; // clock rate
    REG32(0x8000c104) = 0x82; // GCTRL. (bits 7:6-> 10 == SPI mode)
    if(u16strap & CON4) { /* Check CON[4] */
	g_u16AddrLen = 2;
    } else {
	g_u16AddrLen = 3;
    }
}
void
spi_doneclr()
{
    while(REG32(0x8000c108) & 0x400);
}
void
spi_wren()
{
  REG32(0x8000c108) = SPI_WREN;
  REG32(0x8000c124) = WREN;
  spi_doneclr();
}
void
spi_wrdi()
{
  REG32(0x8000c108) = SPI_WRDI;
  REG32(0x8000c124) = WRDI;
  spi_doneclr();
}
u32
spi_rdsr()
{
  u32 result;
  REG32(0x8000c108) = SPI_RSTATUS;
  REG32(0x8000c124) = RDSR;
  result = REG32(0x8000c124);
  spi_doneclr();
  return result;
}
void
spi_write4(u32 u32addr, u32 u32data)
{
  REG32(0x8000c108) = SPI_WRITE_ADDR_DATA(g_u16AddrLen, 4);
  REG32(0x8000c124) = PP;
  REG32(0x8000c124) = (u32addr >> 16) & 0xff;
  REG32(0x8000c124) = (u32addr >>  8) & 0xff;
  REG32(0x8000c124) = u32addr & 0xff;
  REG32(0x8000c124) = (u32data >> 24) & 0xff;
  REG32(0x8000c124) = (u32data >> 16) & 0xff;
  REG32(0x8000c124) = (u32data >>  8) & 0xff;
  REG32(0x8000c124) = u32data & 0xff;
  spi_doneclr();
  while(spi_rdsr() & 1);
}
void
spi_erase()
{
  spi_wren();
  REG32(0x8000c108) = SPI_ERASE;
  REG32(0x8000c124) = BULK_ERASE;
  spi_doneclr();
  while(spi_rdsr() & 1);
  spi_wrdi();
}
u32
spi_read4(u32 u32addr)
{
    s16 i;
    u32 result = 0;
    REG32(0x8000c108) = SPI_READ_ADDR_DATA(g_u16AddrLen, 4); // 24bit addr + 4 return
    REG32(0x8000c124) = READ; // Tell the part what we want to do
    if(g_u16AddrLen == 3) {
	REG32(0x8000c124) = (u32addr>>16) & 0xff;  // addr0 [23:16]
    }
    REG32(0x8000c124) = (u32addr>>8) & 0xff;   // addr1 [16:8]
    REG32(0x8000c124) = u32addr & 0xff;        // addr2 [7:0]
    // Loop to grab all 4 bytes
    for(i=0;i < 4;i++) {
	result <<= 8;
	result |= (REG32(0x8000c124) & 0xff);
    }
    spi_doneclr();
    return result;
}

