typedef unsigned char        uint8;
typedef signed   char        int8;
typedef unsigned short       uint16;
typedef signed   short       int16;
typedef unsigned int         uint32;
typedef signed   int         int32;


typedef unsigned char        UINT8;
typedef signed   char        INT8;
typedef unsigned short       UINT16;
typedef signed   short       INT16;
typedef unsigned int         UINT32;
typedef signed   int         INT32;

#define PACK_END __packed
#define WB_EXCEPTION 1
#define WDS_FEATURE 1

#define MIB_AP_VERSION 2

/* AP configuration data structure */
typedef struct mib_AP_t
{
    UINT8 mibVersion;      /* MIB version number 1..N */
    UINT8 PrivInvoked;     /* BOOLEAN */
    UINT8 PrivOption;
    UINT8 DesiredSsId[32]; /* 32 byte string */
    UINT8 OpRateSet[126];  /* 126 byte string */
    UINT8 DtimPeriod;      /* 1 to 255 */
    /*16 bit alignment before unit16 */
    UINT16 BcnPeriod;       /* 1 to 65535 */
    UINT16 RtsThresh;       /* 0 to 2347 */
    UINT16 FragThresh;      /* 256 to 2346 */
    UINT8 RtsRetryLim;     /* 1 to 255 */
    UINT8 AuthType;
    UINT8 AuthEnable;      /* boolean */
    UINT8 CurrRFChan;      /* 0 to 14 */
    /*32 bit alignment before 32 */
    UINT8 WepType[4]; /*0=not_set, 1=40_bit, 2=104_bit */
    UINT8 WepDefaultKeyValue1[13];
    UINT8 WepDefaultKeyValue2[13];
    UINT8 WepDefaultKeyValue3[13];
    UINT8 WepDefaultKeyValue4[13];
    UINT8 DefaultWepKeyindex;
    UINT8 CurrTxPwrLevel;    /* 1 to 8 */
    /*alignment */
    UINT16 TxPwrLevel1;       /* 0 to 10000 */
    UINT16 TxPwrLevel2;       /* 0 to 10000 */
    UINT16 TxPwrLevel3;       /* 0 to 10000 */
    UINT16 TxPwrLevel4;       /* 0 to 10000 */
    UINT16 TxPwrLevel5;       /* 0 to 10000 */
    UINT16 TxPwrLevel6;       /* 0 to 10000 */
    UINT16 TxPwrLevel7;       /* 0 to 10000 */
    UINT16 TxPwrLevel8;       /* 0 to 10000 */
    UINT8 CurrTxAnt;         /* 1 to 255 */
    UINT8 CurrRxAnt;         /* 1 to 255 */
    UINT8 rxAntenna;
    UINT8 txAntenna;
    UINT8 PreAmble;
    UINT8 broadcastssid;     /* SSID blanking, true or false */
    UINT8 EnableDHCP;        /* True or False */
    UINT8 txDataRate;
    UINT8 mib_wep;           /*special mode */
    UINT8 wipodMode;         /* WiPod mode */
    /*alignment */
    UINT32 IPAddr;            /* IP address */
    UINT32 SubnetMask;        /* Subnet Mask */
    UINT32 GwyAddr;           /* Gateway address */
    UINT8 EthMACAddr[6];     /* Eth MAC addr */
    UINT8 RFMACAddr[6];      /* RF MAC addr */
    /*alignment */
    UINT8 username[32];      /* User name */
    UINT8 password[32];      /* password */
    UINT8 devName[16];
    UINT8 apmode;
    /* For WB31 */
    UINT8 opMode;            // 0 for infrastructure, 1 for ad-hoc
    UINT8 macCloneEnable;    // boolean
    /* For WB31 end */
    UINT8 RSNEnabled;        //WPA mode
    UINT32 GrpRekeyTime;      //GroupRekeyTime
#ifdef OS_VxW
    /* Added for vxWorks version of ap32 - Lan and Wan ports. */
    UINT32 LanIPAddr;            /* Lan IP address */
    UINT32 LanSubnetMask;        /* Lan Subnet Mask */
    UINT32 LanGwyAddr;           /* Lan Gateway address */
    
    UINT32 WanIPAddr;            /* WAN IP address */
    UINT32 WanSubnetMask;        /* WAN Subnet Mask */
    UINT32 WanGwyAddr;           /* WAN Gateway address */
#endif    
//    UINT8 reserved1[5];
#if defined(WIPOD)
    /* The maximum size of this structure is fixed at 512 bytes. 
    Note that changing this structure size will affect many. */
    UINT8 pad[144];
#endif
}
PACK_END mib_AP_t;  /*total 320 bytes */
#ifdef WB_EXCEPTION
/* mfg data struct*/
typedef struct mfg_data_t
{
    UINT8 BrdDscrpt[8];      // 8 byte ASCII to descript the type and version of the board
    UINT8 Rev;
    UINT8 PAOptions;
    UINT8 ExtPA;
    UINT8 Ant;
    UINT16 IntPA[14];
    UINT8 CCA[8];
    UINT16 Domain;            // Wireless domain
    UINT16 CstmrOpts;
    UINT8 Reserved_1[8];
    UINT16 Magic;
    UINT16 ChkSum;
    UINT8 MfgMacAddr[6];     // Mfg mac address
    UINT8 MfgMacAddr2[6];
    UINT8 PID[56];           // Production ID
    UINT8 BootVersion[12];
}
PACK_END mfg_data_t;
#endif /* WB_EXCEPTION */


typedef struct mib_user_AP_t
{
	mib_AP_t	        mib_ap;
#if defined(WIPOD)
    mib_AP_t            mib_wb;
#endif
	UINT32				mib_wlanfilter_cfgkey;
	UINT8				mib_wlanfilter_no;
	UINT8				mib_wlanfilter_mode;
	UINT8				mib_wlanfilter_mac[ 256 * 6];
    UINT8               mib_WPA_passphrase[64];
    UINT8               mib_WPA_encr_algo;
    UINT8               mib_WPA_cfgkey[2];              
    UINT8               mib_forceProtectiondisable;
    UINT8               mib_enableFixedRateTx;
    UINT8               mib_txDataRate;  /* fixed rate to be used for b only clients */
    UINT8               mib_txDataRateG; /* fixed rate to be used for g clients */
    UINT8               reserved0[3];
#ifdef WDS_FEATURE
    UINT32              mib_wds_cfgkey;
    UINT8               mib_wds_Enable; /*0: disable 1: static wds only 2: dynamical wds */
    UINT8               mib_wds_no;
    UINT8               mib_wds_mac[32 * 6]; /* MAX_WDS_PORT*/
    UINT8               mib_disable_assoc; /* TRUE = disable all clients assoc */
    UINT8               reserved1;
#endif /* WDS_FEATURE*/
}
PACK_END mib_user_AP_t;  /*it may be within 0xfffe0400 to 0xffff0000*/
/* Make sizeof(mib_AP_t) 16-bit aligned to simplify writing to flash */
#define SIZEOF_MIB_AP (((sizeof(mib_AP_t)+1)/2)*2)
#define SIZEOF_MIB_USER_AP		(((sizeof(mib_user_AP_t)+1)/2)*2)


typedef struct magic_mib_AP_t
{
    UINT8 mibVersion;      /* MIB version number 1..N */    // 0
    UINT8 PrivInvoked;     /* BOOLEAN */                    // 1
    UINT8 PrivOption;                                       // 2
    UINT8 DesiredSsId[32]; /* 32 byte string */             // 3 .. 34
    UINT8 OpRateSet[16];  /* 16 byte string */              // 35 .. 50
    UINT8 DtimPeriod;      /* 1 to 255 */                   // 51
    UINT16 BcnPeriod;       /* 1 to 65535 */                // 52, 53
    UINT16 RtsThresh;       /* 0 to 2347 */                 // 54, 55
    UINT16 FragThresh;      /* 256 to 2346 */               // 56, 57
    UINT8 AuthType;                                         // 58
    UINT8 AuthEnable;      /* boolean */                    // 59
    UINT8 CurrRFChan;      /* 0 to 14 */                    // 60
    UINT8 WepType[4]; /*0=not_set, 1=40_bit, 2=104_bit */   // 61 .. 64
    UINT8 WepDefaultKeyValue1[13];                          // 65 .. 77
    UINT8 WepDefaultKeyValue2[13];                          // 78 .. 90
    UINT8 WepDefaultKeyValue3[13];                          // 91 .. 103
    UINT8 WepDefaultKeyValue4[13];                          // 104 .. 116
    UINT8 DefaultWepKeyindex;                               // 117
    UINT8 CurrTxPwrLevel;    /* 1 to 8 */                   // 118
    UINT8 CurrTxAnt;         /* 1 to 255 */                 // 119
    UINT8 CurrRxAnt;         /* 1 to 255 */                 // 120
    UINT8 PreAmble;                                         // 121
    UINT8 broadcastssid; /* SSID blanking, true or false */ // 122
    UINT8 mib_wep;           /*special mode */              // 123
    UINT8 apmode;                                           // 124
    UINT8  RSNEnabled;        //WPA mode                    // 125
    UINT8  Reserved[2];                                     // 126 127
    UINT32 GrpRekeyTime;      //GroupRekeyTime              // 128 .. 131
    UINT8  Reserved1[2];                                    // 132 133
    UINT8  mib_WPA_passphrase[64];                          // 134 .. 197
    UINT8  mib_WPA_encr_algo;                               // 222
    UINT8  mib_forceProtectiondisable;                      // 199
    UINT8  mib_enableFixedRateTx;                           // 200
    UINT8  mib_txDataRate;  /* fixed rate to be used for b only clients */  // 225
    UINT8  mib_txDataRateG; /* fixed rate to be used for g clients */       // 226
    
    UINT8 EnableDHCP;         // True or False                  // 227
    UINT32 IPAddr;            // IP address                     // 228.. 207
    UINT32 SubnetMask;        // Subnet Mask                    // 208 .. 207
    UINT32 GwyAddr;           // Gateway address                // 208 .. 211
}
PACK_END magic_mib_AP_t;
UINT8 rate[14] = {2,4,11,22,44,12,18,24,36,48,72,96,108,144};
void transfer(unsigned char *in, unsigned char *out)
{
     mib_AP_t *mib_ap;
     mib_user_AP_t *mib_user_ap;
     magic_mib_AP_t *magic_ap;

     mib_ap = (mib_AP_t *)in;
     mib_user_ap = (mib_user_AP_t *)in;
     magic_ap = (magic_mib_AP_t *)out;
      
     magic_ap->mibVersion                 =  mib_ap->mibVersion;     
     magic_ap->PrivInvoked                =  mib_ap->PrivInvoked;    
     magic_ap->PrivOption                 =  mib_ap->PrivOption; 
     mymemcpy3(magic_ap->DesiredSsId, mib_ap->DesiredSsId, 32); 
     mymemcpy3(magic_ap->OpRateSet, mib_ap->OpRateSet,16); 
     magic_ap->DtimPeriod                 =  mib_ap->DtimPeriod;     
     magic_ap->BcnPeriod                  =  mib_ap->BcnPeriod;      
     magic_ap->RtsThresh                  =  mib_ap->RtsThresh;                 
     magic_ap->FragThresh                 =  mib_ap->FragThresh;                
     magic_ap->AuthType                   =  mib_ap->AuthType;                  
     magic_ap->AuthEnable                 =  mib_ap->AuthEnable;                
     magic_ap->CurrRFChan                 =  mib_ap->CurrRFChan;                
     mymemcpy3(magic_ap->WepType, mib_ap->WepType,4);                
     mymemcpy3(magic_ap->WepDefaultKeyValue1,mib_ap->WepDefaultKeyValue1,13);   
     mymemcpy3(magic_ap->WepDefaultKeyValue2,mib_ap->WepDefaultKeyValue2,13);   
     mymemcpy3(magic_ap->WepDefaultKeyValue3,mib_ap->WepDefaultKeyValue3,13);   
     mymemcpy3(magic_ap->WepDefaultKeyValue4,mib_ap->WepDefaultKeyValue4,13);   
     magic_ap->DefaultWepKeyindex         =  mib_ap->DefaultWepKeyindex;        
     magic_ap->CurrTxPwrLevel             =  mib_ap->CurrTxPwrLevel;            
     magic_ap->CurrTxAnt                  =  mib_ap->CurrTxAnt;                 
     magic_ap->CurrRxAnt                  =  mib_ap->CurrRxAnt;                 
     magic_ap->PreAmble                   =  mib_ap->PreAmble;                  
     magic_ap->broadcastssid              =  mib_ap->broadcastssid;             
     magic_ap->mib_wep                    =  mib_ap->mib_wep;                   
     magic_ap->apmode                     =  mib_ap->apmode;                    
     magic_ap->RSNEnabled                 =  mib_ap->RSNEnabled;                
     magic_ap->GrpRekeyTime               =  mib_ap->GrpRekeyTime;              
     mymemcpy3(magic_ap->mib_WPA_passphrase,mib_user_ap->mib_WPA_passphrase,64);    
     magic_ap->mib_WPA_encr_algo          =  mib_user_ap->mib_WPA_encr_algo;         
     magic_ap->mib_forceProtectiondisable =  mib_user_ap->mib_forceProtectiondisable;
     magic_ap->mib_enableFixedRateTx      =  mib_user_ap->mib_enableFixedRateTx;     
     magic_ap->mib_txDataRate             =  rate[mib_user_ap->mib_txDataRate];            
     magic_ap->mib_txDataRateG            =  rate[mib_user_ap->mib_txDataRateG];           
     magic_ap->EnableDHCP                 =  mib_ap->EnableDHCP;
     magic_ap->IPAddr                     =  mib_ap->IPAddr;
     magic_ap->SubnetMask                 =  mib_ap->SubnetMask;
     magic_ap->GwyAddr                    =  mib_ap->GwyAddr;  
}
