/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
 *
 * $Header: /VOIPDEV/WB31V_SDK/Bootloader/ap31_tftp_boot/setup_uart.h,v 1.1.1.1 2005/04/19 22:31:43 yuany Exp $
 *
 * Purpose:
 *    This file contains the function prototypes and definitions for the
 *    UART setup code.
 *
 * Public Procedures:
 *    getchar                 Read a char from the 16550 UART block. 
 *                            When one is avail.
 *    putchar                 Write a u8 to the 16550.
 *    setup_uart              Prepare the 16550 for 38400 NP 1stop also
 *                            set the pin mux to UART from USB
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

#ifndef _SETUP_UART_H_
#define _SETUP_UART_H_

//=============================================================================
//                                DEFINITIONS
//=============================================================================
#define FAIL -1
#define OK 0

#define REG32(x) (*(volatile unsigned int   *)(x))
#define REG16(x) (*(volatile unsigned short *)(x))
#define REG8(x)  (*(volatile unsigned char  *)(x))
#define bit(x)   (0x0001<<x)

#define SI_DIV       0x8000c008
#define PI_PAD_CTRL  0x80006050

#define UART_BASE          0x8000c840
#define Rx_BUFFER_REG      (UART_BASE + (4*0))
#define Tx_HOLDING_REG     (UART_BASE + (4*0))
#define DIVISOR_LOW        (UART_BASE + (4*0)) /* When LINE_CONTROL[7] == 1 */
#define IRQ_ENABLE         (UART_BASE + (4*1))
#define DIVISOR_HIGH       (UART_BASE + (4*1)) /* When LINE_CONTROL[7] == 1 */
#define PSD                (UART_BASE + (4*2)) /* When LINE_CONTROL[7] == 1 */
#define IRQ_STATUS         (UART_BASE + (4*2))
#define FIFO_CONTROL       (UART_BASE + (4*2))
#define LINE_CONTROL       (UART_BASE + (4*3))
#define  DLAB              bit(7)            /* turns on access to baud rate divisor */
#define  ASSERT_BREAK      bit(6)
#define  STICKY_PARITY     bit(5)
#define  EVEN_PARITY_SEL   bit(4)
#define  PARITY_ENABLE     bit(3)
#define  TWO_STOP_BITS     bit(2)
#define  FIVE_BIT          0                    
#define  SIX_BIT           bit(0)
#define  SEVEN_BIT         bit(1)
#define  EIGHT_BIT         (bit(1) | bit(0))

#define MODEM_CONTROL      (UART_BASE + (4*4))
#define LINE_STATUS        (UART_BASE + (4*5))
#define  Rx_FIFO_ERROR     bit(7)     
#define  Tx_EMPTY          bit(6) 
#define  Tx_HOLD_EMPTY     bit(5) 
#define  BREAK_DETECT      bit(4) 
#define  FRAMING_ERROR     bit(3) 
#define  PARITY_ERROR      bit(2) 
#define  OVERRUN_ERROR     bit(1) 
#define  DATA_READY        bit(0) 
#define MODEM_STATUS       (UART_BASE + (4*6))
#define SCRATCH_REG        (UART_BASE + (4*7))

#define CPU_SPEED          44E6       // Make SURE this is correct!!!

#ifdef CONDOR2
#define SERIAL_SPEED       176E6 / 3  // Make SURE this is correct!!!
#else
#define SERIAL_SPEED       8E6        // Make SURE this is correct!!!
#endif

/* retry loops before we resend the NAK. 10sec. */
/* 4131e-9 is the delay time of the read loop */
#define NAK_TIMEOUT_VAL    (int)((10.0 / (4131.0e-9 * 65536.0)) + 0.5)  

//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================
/******************************************************************************
 *
 * Name: getchar
 *
 * Description:
 *   Return the next char available on UART receive. If no char is
 *   ready, it will wait. Should no char still come after a timeout
 *   passes, then return back to the   uart_boot() code and NAK the
 *   packet. 
 *
 * Conditions For Use:
 *   GPIO Clock should be enabled before attempting to use. (In PAU).
 *
 * Arguments:
 *   Arg1( u16 ): 16 bits of data to appear at the pins.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern int mygetchar( unsigned char *pu8c );

/******************************************************************************
 *
 * Name: putchar
 *
 * Description:
 *   Put a char into the UART transmit buffer. Wait until that buffer
 *   is ready. No timeout here, assume we can always be ready to send
 *   after a wait. 
 *
 * Conditions For Use:
 *   GPIO Clock should be enabled before attempting to use. (In PAU).
 *
 * Arguments:
 *   Arg1( u8 ): 8 bits of data to be placed in the 16550 xmit buffer.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void myputchar( unsigned char u8c );

/******************************************************************************
 *
 * Name: setup_uart
 *
 * Description:
 *   Place the 16550 into a useable state for us. This means the the
 *   baud rate needs to be 38400, no parity, one stop bit. Also the
 *   pad mux needs to get flipped from the default of USB to UART. 
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void setup_uart( int rate );

/*****************************************************************************/
extern void myputs(unsigned char *c);

#endif /* _SETUP_UART_H_ */
