/*
**  mem.c
*/

#include <stdio.h>
#include "board.h"
/*----- Types and Constants Definition ----------------------*/
#define _REG(x) (*(volatile unsigned long *)(x))
#define _REG16(x) (*(volatile unsigned short *)(x))

#define MDU_CSR      0x80006000
#define MDU_CFGR0    0x80006004
#define MDU_TIMR00   0x80006008    
#define MDU_TIMR01   0x8000600C
#define MDU_CFGR1    0x80006010
#define MDU_TIMR10   0x80006014
#define MDU_TIMR11   0x80006018
#define MDU_CFGR2    0x8000601C
#define MDU_TIMR20   0x80006020
#define MDU_TIMR21   0x80006024
#define MDU_CFGR3    0x80006028
#define MDU_TIMR30   0x8000602C
#define MDU_TIMR31   0x80006030
#define MDU_CFGR4    0x80006034
#define MDU_TIMR40   0x80006038
#define MDU_TIMR41   0x8000603C
#define MDU_CFGR5    0x80006040
#define MDU_TIMR50   0x80006044
#define MDU_TIMR51   0x80006048

#define CLOCKS       0x8000200c

void ConfigMDU(int boardtype)
{
    _REG(MDU_TIMR00) = 0x0442A20C;   /* FLASH timing */
    _REG(MDU_TIMR01) = 0x00008080;
    _REG(MDU_CFGR0)  = 0xFFFF4285;   /* 0xffc00000 FLASH configuration pack_en = 1*/
    switch (boardtype)
    {
    case EVAL_BOARD:
        _REG(MDU_CSR)    = 0x04190040;
        _REG(MDU_TIMR20) = 0x00000022;
        _REG(MDU_TIMR21) = 0x03232222;
        _REG(MDU_CFGR2)  = 0x40FE50B5;
        break;
    case AP21_2X1MX16_BOARD:
        _REG(MDU_CSR)    = 0x157E0000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x03232222;
        _REG(MDU_CFGR1)  = 0x40FF7095;
        break;
    case AP21_1X2MX32_BOARD:
        _REG(MDU_CSR)    = 0x157E0000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x03232232;
        _REG(MDU_CFGR1)  = 0x40FE70B7;
        break;
    case AP21_1X4MX16_BOARD:
        _REG(MDU_CSR)    = 0x157E0000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x03232232;
        _REG(MDU_CFGR1)  = 0x40FE50B5;
        break;
    case AP22_2X4MX16_BOARD:
        _REG(MDU_CSR)    = 0x20530000;
        _REG(MDU_TIMR10) = 0x00000022;
        _REG(MDU_TIMR11) = 0x08383632;
        _REG(MDU_CFGR1)  = 0x40FC70B5;
        break;
    case AP22_1X2MX32_BOARD:
        _REG(MDU_CSR)    = 0x20530000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x0A3A3632;
        _REG(MDU_CFGR1)  = 0x40FE70B7;
        break;
    case AP32_2X4MX16_BOARD:
        _REG(MDU_CSR)    = 0x20530000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x0A3A3632;
        _REG(MDU_CFGR1)  = 0x40FC70B5;
        break;
    default:
        _REG(MDU_CSR)    = 0x157E0000;
        _REG(MDU_TIMR10) = 0x00000032;
        _REG(MDU_TIMR11) = 0x03232232;
        _REG(MDU_CFGR1)  = 0x40FE70B7;
    }
    // Disable devices 3 4 5
    _REG(MDU_CFGR3) = 0xC0FF0008;
    _REG(MDU_CFGR4) = 0x80FF0008;
    _REG(MDU_CFGR5) = 0x40FF0008;

    // Turn on all clocks for debugging.
    _REG16(CLOCKS)  = 0x3FFF;
}
