/*
** program to flash factory default settings
** 
*/

#include "wltypes.h"
#include "apcfg.h"
#include "mfg.h"

/*--------------------------------------------*/
/* Various sizes used in IEEE 802.11 messages */
/*--------------------------------------------*/
#define IEEEtypes_ADDRESS_SIZE         6
#define IEEEtypes_BITMAP_SIZE          251
#define IEEEtypes_CHALLENGE_TEXT_SIZE  253
#define IEEEtypes_CHALLENGE_TEXT_LEN   128
#define IEEEtypes_MAX_DATA_RATES       8
#define IEEEtypes_MAX_DATA_BODY_LEN    2312
#define IEEEtypes_MAX_MGMT_BODY_LEN    2312
#define IEEEtypes_SSID_SIZE            32
#define IEEEtypes_TIME_STAMP_SIZE      8
#define IEEEtypes_MAX_CHANNELS         14
#define IEEEtypes_MAX_BSS_DESCRIPTS    10

typedef enum {
  AUTH_OPEN_SYSTEM = 0,
  AUTH_SHARED_KEY,
  AUTH_NOT_SUPPORTED
} IEEEtypes_AuthType_e;

#define WEP_TYPE_40 1
#define WEP_TYPE_104 2

typedef enum {
  AP_ANT_A,
  AP_ANT_B,
  AP_ANT_DIVERSITY,
}ANTENNA_TYPE;

typedef enum {
  PREAMBLE_SHORT,
  PREAMBLE_LONG,
  PREAMBLE_AUTO_SELECT
}PREAMBLE_TYPE;

#define TRUE 1
#define FALSE 0
/* Commented out for main branch merge, if we need this later it can be put back in */
/* #if defined(AP_URPTR) && defined(AUTOLINK) */
/* char *DefaultSSID = "MarvellUR"; */
/* #else */
char *DefaultSSID = "MarvellAP5X";
/* #endif */
char *DefaultUsername = "admin";
char *DefaultPassword = "password";

char Eth_MAC_Address[6] = {
  0x00, 0x12, 0x34, 0x56, 0x78, 0x9A
};

char RF_MAC_Address[6] = {
  0x00, 0x21, 0x43, 0x65, 0x87, 0xA9
};

UINT8
util_CopyList(UINT8 *Dest_p, UINT8 *Src_p, UINT32 MaxSize)
{
  UINT32  i = 0;

  while (i < MaxSize  &&  Src_p[i] != '\0') {
    Dest_p[i] = Src_p[i];
    i++;
  }

  if (i < MaxSize) {
    Dest_p[i] = '\0';
  }

  return(i);

} // End util_CopyList()

int
SetFactoryConfig(void *factorySettings)
{
  mib_AP_t *mib;
  int i;
  int rc;

  mib = (mib_AP_t *)factorySettings;

  memset((char *)mib, 0, SIZEOF_MIB_AP);

  mib->mibVersion = MIB_AP_VERSION;
  mib->PrivOption = 0;
  mib->PrivInvoked = 0;

  util_CopyList((UINT8 *)(mib->DesiredSsId), (UINT8 *)DefaultSSID, IEEEtypes_SSID_SIZE);
  util_CopyList((UINT8 *)(mib->username), (UINT8 *)DefaultUsername, 32);
  util_CopyList((UINT8 *)(mib->password), (UINT8 *)DefaultPassword, 32);
  mib->OpRateSet[0] = 0x82;
  mib->OpRateSet[1] = 0x84;
  mib->OpRateSet[2] = 0x8B;
  mib->OpRateSet[3] = 0x96;
  mib->OpRateSet[4] = 12;
  mib->OpRateSet[5] = 18;
  mib->OpRateSet[6] = 24;
  mib->OpRateSet[7] = 36;
  mib->OpRateSet[8] = 48;
  mib->OpRateSet[9] = 72;
  mib->OpRateSet[10] = 96;
  mib->OpRateSet[11] = 108;
  mib->OpRateSet[12] = 0;
  mib->OpRateSet[13] = 0;

  mib->BcnPeriod = 100;
  mib->DtimPeriod = 1;
  mib->RtsThresh = 0x92B;
  mib->RtsRetryLim = 0x03;
  mib->FragThresh = 0x92a;
  mib->AuthEnable = 0;
  mib->AuthType = AUTH_OPEN_SYSTEM;

  mib->CurrRFChan = 6;
#ifdef AP5X
  mib->WepType[0] = 0;
  mib->WepType[1] = 0;
  mib->WepType[2] = 0;
  mib->WepType[3] = 0;
#else
  mib->WepType = 0;
#endif
  mib->CurrTxPwrLevel = 0;
  mib->TxPwrLevel1 = 0;
  mib->TxPwrLevel2 = 0;
  mib->TxPwrLevel3 = 0;
  mib->TxPwrLevel4 = 0;
  mib->TxPwrLevel5 = 0;
  mib->TxPwrLevel6 = 0;
  mib->TxPwrLevel7 = 0;
  mib->TxPwrLevel8 = 0;

  mib->CurrTxAnt = AP_ANT_B; // Ant A or B
  mib->CurrRxAnt = AP_ANT_B; // diversity

  mib->PreAmble = PREAMBLE_LONG;

  mib->broadcastssid = TRUE;

  mib->EnableDHCP = FALSE;
/* Commented out for main branch merge, if we need this later it can be put back in */
/* #if defined(AP_URPTR) && defined(AUTOLINK) */
/*   mib->IPAddr = 0xC0A8000B; */
/* #else */
  mib->IPAddr = 0xC0A8000A; /* 192.168.0.10 */
/* #endif */
  mib->SubnetMask = 0xFFFFFF00; /* 255.255.255.0 */
  mib->GwyAddr = 0;
#if defined(GATEWAY)
  mib->EnableDHCP = TRUE;
#endif    
  mib->WanIPAddr = 0x34020202; /* 52.2.2.2 */
  mib->WanSubnetMask = 0xFFFFFF00; /* 255.255.255.0 */
  mib->WanGwyAddr = 0;
  mib->PrimaryDNS = 0;           // Gateway address
  mib->SecondaryDNS = 0;         // Gateway address
  mib->LanIPAddr = 0xC0A8000A; /* 192.168.0.10 */
  mib->LanSubnetMask = 0xFFFFFF00; /* 255.255.255.0 */

  /* default in auto state */
#ifdef WB51  
  mib->txDataRate = 0xFF;
#else
  mib->txDataRate = 0;
#endif

  for (i = 0; i < 6; ++i) {
    mib->EthMACAddr[i] = Eth_MAC_Address[i];
    mib->RFMACAddr[i] = RF_MAC_Address[i];
  }

  mib->devName[0] = 0xff;
  mib->devName[15] = 0xff;
  mib->BurstRate = 8;
  mib->ApMode = 2; //mixed mode
  mib->BurstMode = 0;
  mib->BurstRate = 8;
  mib->txDataRateG = 12;
  mib->EnableFixedRateTx = 0;
  mib->forceprotection = 1;
  mib->apmode = 4;

  strcpy(mib->WPA_passphrase, "12345678");

/* #if defined(AP_URPTR) && defined(AUTOLINK) */
/*    mib->mib_urMode = 1; */
/*    mib->mib_wbMode = 0; */
/*    util_CopyList((UINT8 *)mib->mib_urSsid, "MarvellUR", IEEEtypes_SSID_SIZE); */
/* #endif */
  return rc;
}
void MfgInit(void *mfgSetting)
{	
	static int mfgInit = 0;
	mfg_data_t *mfg;
	mfg = (mfg_data_t *)mfgSetting;
	if(!mfgInit)
	{
  		memset((char *)mfg, 0xff, sizeof(mfg_data_t));
		mfgInit = 1;
	}
}

void MfgSetCalibData(void *mfgSetting, char *calibData)
{
	mfg_data_t *mfg;
	mfg = (mfg_data_t *)mfgSetting;
	MfgInit(mfgSetting);
	memcpy(mfg->BrdDscrpt, calibData, 48);//used to be 12*8
}

void MfgSetMacAddr(void *mfgSetting, char *macAddr, char flag)
{
	mfg_data_t *mfg;
	mfg = (mfg_data_t *)mfgSetting;
	//MfgInit(mfgSetting);
	if(flag == 1)
		memcpy(mfg->MfgMacAddr, macAddr, 6);
	else if(flag == 2)
		memcpy(mfg->MfgMacAddr2, macAddr, 6);
}

void MfgSetDomain(void *mfgSetting, UINT16 domain)
{
	mfg_data_t *mfg;
	mfg = (mfg_data_t *)mfgSetting;
	//MfgInit(mfgSetting);
	mfg->Domain = domain;
}
