/*
** program to flash with mac address and factory default settings
** 
*/
/*----- Types and Constants Definition ----------------------*/
#define _REG(x) (*(volatile unsigned long *)(x))
#define _REG16(x) (*(volatile unsigned short *)(x))

#define MDU_CSR      0x80006000
#define MDU_CFGR0    0x80006004
#define MDU_TIMR00   0x80006008    
#define MDU_TIMR01   0x8000600C
#define MDU_CFGR1    0x80006010
#define MDU_TIMR10   0x80006014
#define MDU_TIMR11   0x80006018
#define MDU_CFGR2    0x8000601C
#define MDU_TIMR20   0x80006020
#define MDU_TIMR21   0x80006024
#define MDU_CFGR3    0x80006028
#define MDU_TIMR30   0x8000602C
#define MDU_TIMR31   0x80006030
#define MDU_CFGR4    0x80006034
#define MDU_TIMR40   0x80006038
#define MDU_TIMR41   0x8000603C
#define MDU_CFGR5    0x80006040
#define MDU_TIMR50   0x80006044
#define MDU_TIMR51   0x80006048

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <rt_misc.h>
#include "board.h"

void ConfigMDU(int);
void program_image(int what);
int macAddrProcess(char *src, char *dst);
int stringtoNum(char *src);

int total_size;
int errors = 0;

struct _board_map {
    char *prog;
    int what;
    int board;
} board_map[]={
    {"flash21.axf", ALL, AP21_BOARD}, 
    {"flash21_boot.axf", BOOT, AP21_BOARD}, 
    {"flash21_no_boot.axf", NO_BOOT, AP21_BOARD}, 
    {"flash21_no_boot2.axf", NO_BOOT2, AP21_BOARD}, 
    {"flash21_boot_factory.axf", BOOT_FACTORY, AP21_BOARD}, 
    {"flash21_no_boot_factory.axf", NO_BOOT_FACTORY, AP21_BOARD}, 
    {"flash21m.axf", ALL, AP21M_BOARD}, 
    {"flash21m_boot.axf", BOOT, AP21M_BOARD}, 
    {"flash21m_no_boot.axf", NO_BOOT, AP21M_BOARD}, 
    {"flash21m_no_boot2.axf", NO_BOOT2, AP21M_BOARD}, 
    {"flash21m_boot_factory.axf", BOOT_FACTORY, AP21M_BOARD}, 
    {"flash21m_no_boot_factory.axf", NO_BOOT_FACTORY, AP21M_BOARD}, 
    {"flash22.axf", ALL, AP22_BOARD}, 
    {"flash22_boot.axf", BOOT, AP22_BOARD}, 
    {"flash22_no_boot.axf", NO_BOOT, AP22_BOARD}, 
    {"flash22_no_boot2.axf", NO_BOOT2, AP22_BOARD}, 
    {"flash22_boot_factory.axf", BOOT_FACTORY, AP22_BOARD}, 
    {"flash22_no_boot_factory.axf", NO_BOOT_FACTORY, AP22_BOARD}, 
    {"flash22N.axf", ALL, AP22N_BOARD}, 
    {"flash22N_boot.axf", BOOT, AP22N_BOARD}, 
    {"flash22N_no_boot.axf", NO_BOOT, AP22N_BOARD}, 
    {"flash22N_no_boot2.axf", NO_BOOT2, AP22N_BOARD}, 
    {"flash22N_boot_factory.axf", BOOT_FACTORY, AP22N_BOARD}, 
    {"flash22N_no_boot_factory.axf", NO_BOOT_FACTORY, AP22N_BOARD}, 
    {"flash22C.axf", ALL, AP22C_BOARD}, 
    {"flash22C_boot.axf", BOOT, AP22C_BOARD}, 
    {"flash22C_no_boot.axf", NO_BOOT, AP22C_BOARD}, 
    {"flash22C_no_boot2.axf", NO_BOOT2, AP22C_BOARD}, 
    {"flash22C_boot_factory.axf", BOOT_FACTORY, AP22C_BOARD}, 
    {"flash22C_no_boot_factory.axf", NO_BOOT_FACTORY, AP22C_BOARD}, 
    {"flash_eval.axf", ALL, EVAL_BOARD}, 
    {"flash_eval_boot.axf", BOOT, EVAL_BOARD}, 
    {"flash_eval_no_boot.axf", NO_BOOT, EVAL_BOARD}, 
    {"flash_eval_no_boot2.axf", NO_BOOT2, EVAL_BOARD}, 
    {"flash_eval_boot_factory.axf", BOOT_FACTORY, EVAL_BOARD}, 
    {"flash_eval_no_boot_factory.axf", NO_BOOT_FACTORY, EVAL_BOARD}, 
    {"flash22N2.axf", ALL, AP22N2_BOARD}, 
    {"flash22N2_boot.axf", BOOT, AP22N2_BOARD}, 
    {"flash22N2_no_boot.axf", NO_BOOT, AP22N2_BOARD}, 
    {"flash22N2_no_boot2.axf", NO_BOOT2, AP22N2_BOARD}, 
    {"flash22N2_boot_factory.axf", BOOT_FACTORY, AP22N2_BOARD}, 
    {"flash22N2_no_boot_factory.axf", NO_BOOT_FACTORY, AP22N2_BOARD}, 
    {"flash31_no_boot.axf", NO_BOOT, CONDOR_TEST_BOARD}, 
    {"flash31_no_boot2.axf", NO_BOOT2, CONDOR_TEST_BOARD}, 
    {"flash31_boot_factory.axf", BOOT_FACTORY, CONDOR_TEST_BOARD}, 
    {"flash31_no_boot_factory.axf", NO_BOOT_FACTORY, CONDOR_TEST_BOARD}, 
    {"flash3x_no_boot.axf", NO_BOOT, CONDOR_BOARD}, 
    {"flash3x_no_boot2.axf", NO_BOOT2, CONDOR_BOARD}, 
    {"flash3x_boot_factory.axf", BOOT_FACTORY, CONDOR_BOARD}, 
    {"flash3x_no_boot_factory.axf", NO_BOOT_FACTORY, CONDOR_BOARD}, 
    {"flash5x_no_boot.axf", NO_BOOT, CONDOR_BOARD}, 
    {"flash5x_no_boot2.axf", NO_BOOT2, CONDOR_BOARD}, 
    {"flash5x_boot_factory.axf", BOOT_FACTORY, CONDOR_BOARD}, 
    {"flash5x_no_boot_factory.axf", NO_BOOT_FACTORY, CONDOR_BOARD}, 
    {"flash5x_boot_clear_factory.axf", BOOT_NO_FACTORY, CONDOR_BOARD}, 
    {"flash5x_boot.axf", BOOT, CONDOR_BOARD},     
    {"flash5x_all.axf", ALL, CONDOR_BOARD}, 
    {0, 0, 0}
};
char *progname;
int 
main(int argc, char *argv[])
{
   FILE *fp;
   int ch;
   int addr;
   unsigned short val;
   unsigned short *sptr = (unsigned short *)0xd0000000;
   unsigned short *calibptr = (unsigned short *)0xFFFFFF70;
   char *p;
   char buffer[200];
   int i;
   progname = argv[0];

   memset((char *)0xd0000000, 0xff, 1*1024*1024);

   p = strrchr(progname, '\\');
   if(p) {
       strcpy(buffer, p+1);
   } else {
       strcpy(buffer, progname);
   }
   for(i=0;i < sizeof(board_map) / sizeof(struct _board_map);i++) {
       if(!strcmp(buffer, board_map[i].prog)) {
	   break;
       }
   }
   if(i == (sizeof(board_map) / sizeof(struct _board_map))) {
       fprintf(stderr, "Cannot find this program/flash %s\n", buffer);
       exit(1);
   }

   //
   // Config MDU
   //

   fprintf(stderr, "File to flash: ");
   gets(buffer);
   fp = fopen(buffer, "rb");

   if(!fp) {
     printf("Cannot open %s for reading\n", buffer);
     exit(1);
   }

   printf("Config MDU for %s\n", board_map[i].prog);
   ConfigMDU(board_map[i].board);

   // AMD FLASH test
   // Erase sector
   if( board_map[i].what == BOOT_FACTORY)
   {
        fprintf(stderr, "Default factory setting read\n\n");
        SetFactoryConfig((void *)0xd0000000);
        //duplicated for user setting
        SetFactoryConfig((void *)0xd0000400);
        sptr += (64*1024/2);    //unsigned short pointer increase
   }

   if( board_map[i].what == BOOT_NO_FACTORY)
   {
       fprintf(stderr, "Clear Factory Setting\n\n");
       memset((void *)0xd0000000, 0xff,64*1024 );
        sptr += (64*1024/2);    //unsigned short pointer increase
   }
   
   printf("read image\n");
   
   addr =0;
   total_size = 0;

   while((ch=fgetc(fp)) != EOF) {
       if(!(addr & 0x7fe)) {
	   printf("@addr %x\n", addr);
       }
      val = ch;
      if((ch=fgetc(fp)) == EOF)
      	break;
      val |= ch<<8;	 
      *sptr = val;
      if(*sptr != val) {
	  printf("Mismatch in DRAM @%x got %x want %x\n", addr, *sptr, val);
	  errors++;
	  if(errors > ERROR_LIMIT) {
	      printf("Too many errors, bailing\n");
	      exit(1);
	  }
      }
      addr += 2;
      total_size += 2;
      sptr++;
   }
   fclose(fp);
   if( (board_map[i].what == BOOT_FACTORY) 
        || (board_map[i].what == BOOT_NO_FACTORY)
        || (board_map[i].what == BOOT))
   {
      char ptr[6];
      char calibdata[48];
	  char *macaddr, *macaddr2;
	  short *domain;   
       
      if (board_map[i].board == CONDOR_BOARD)
      {
#if 0 /* New cal struct starts at 0xfffff000 instead of d00. 
          This change will not harm the old cal struct */
          memcpy((char *)0xD001FD00, (char *)0xFFFFFD00, 704);
		  printf("Read board data from 0xfffffd00 to 0xffffffc0\n");
#else
          memcpy((char *)0xD001F000, (char *)0xFFFFF000, 0xFF0); /* Last 12 bytes rsvd for boot version; others 4byte rsvd */
		  printf("Read board data from 0xfffff000 to 0xfffffff0\n");
#endif

		  macaddr = (char *)0xd001ffb0;
		  macaddr2 = (char *)0xd001ffb6;
		  domain = (short *)0xd001ffa0;
		  printf("mac address 1 %02x-%02x-%02x-%02x-%02x-%02x\nmac address 2 %02x-%02x-%02x-%02x-%02x-%02x\ndomain = %x\n", 
		  macaddr[0], macaddr[1],macaddr[2],macaddr[3],macaddr[4],macaddr[5], 
		  macaddr2[0], macaddr2[1],macaddr2[2],macaddr2[3],macaddr2[4],macaddr2[5], *domain);
      }
      else
      {
      	  memcpy(calibdata, (unsigned char *)calibptr , 48);
      	  MfgSetCalibData((void *)0xd001ff70, calibdata);
      }
      total_size = 2*64*1024;

      fprintf(stderr, "Write Mac Address 1 to Flash? (y/n) ");
      gets(buffer);
	  if(buffer[0] == 'y')
	  {
      	fprintf(stderr, "\nformat xx-xx-xx-xx-xx-xx ");
      	gets(buffer);
	  	if (macAddrProcess(buffer, ptr))
      	{
      	  MfgSetMacAddr((void *)0xd001ff70, ptr, 1); /* Please see comment below */
      	  total_size = 2*64*1024;
      	}
	  }
      fprintf(stderr, "Write Mac Address 2 to Flash? (y/n) ");
      gets(buffer);
	  if(buffer[0] == 'y')
	  {
      	fprintf(stderr, "\nformat xx-xx-xx-xx-xx-xx ");
      	gets(buffer);
	  	if (macAddrProcess(buffer, ptr))
      	{
      	  MfgSetMacAddr((void *)0xd001ff70, ptr, 2); /* Please see comment below */
      	  total_size = 2*64*1024;
      	}
	  }
      /*********************************************
      This is defined in mfg.h without PEREGRINE compile flag, 
      so regarding of cal struct (P or F), using 0xffffff70 offset 
      for mfg->MfgMacAddr does not break anything. This code needs to be cleaned up in future

        typedef struct mfg_data_t {
            UINT8 BrdDscrpt[8];      
            UINT8 Rev;
            UINT8 PAOptions;
            UINT8 ExtPA;
            UINT8 Ant;
            UINT16 IntPA[14];
            UINT8 CCA[8];
            UINT16 Domain;           
            UINT16 CstmrOpts;
            UINT8 Reserved_1[8];
            UINT16 Magic;
            UINT16 ChkSum;
            UINT8 MfgMacAddr[6];     
            UINT8 MfgMacAddr2[6];
            UINT8 PID[56];           
            UINT8 BootVersion[12];
        } PACK_END mfg_data_t;      
      ***********************************************/

      fprintf(stderr, "Domain Code to Flash? (y/n) ");
      gets(buffer);
	  if(buffer[0] == 'y')
	  {
      	fprintf(stderr, "\nformat: 0xhh ");
      	gets(buffer);
      	MfgSetDomain((void *)0xd001ff70, stringtoNum(buffer));
	  }
   }

   program_image(board_map[i].what);
   printf("Done\n");
}
__value_in_regs struct __initial_stackheap __user_initial_stackheap(
        unsigned R0, unsigned SP, unsigned R2, unsigned SL)
{
    struct __initial_stackheap config;
    int aa, bb;

    aa=0x50078;
    bb=(0x80000|(4 << 1));
    __asm{
        ldr      r3, [&bb]
        MCR	 p15, 0, r3, c9, c1, 0
        MRC	 p15, 0, r3, c1, c0, 0
	ldr      r0, [&aa]
	MCR      p15, 0, r0, c1, c0, 0
        MRC	 p15, 0, r3, c1, c0, 0
        ORR	 r3, r3, #((1<<16) | (1<<18))
        MCR	 p15, 0, r3, c1, c0, 0
    }
   
    config.heap_base = 0xc0008000;
    config.stack_base = 0xc000c000;

    return config;
}

int hextoNum(char a)
{
    if ( (a >= '0') && (a <='9') )
        return (a - '0');
    else if ( (a >= 'a') && (a <= 'f'))
        return (a - 87);
    else if ( (a >= 'A') && (a <= 'F'))
        return (a - 55);
    else
        return -1;
}
int digtoNum(char a)
{
    if ( (a >= '0') && (a <='9') )
        return (a - '0');
    else
        return -1;
}
int stringtoNum(char *src)
{
    int ret = 0;
    int hex = 0;
    while(*src)
    {
        if( hex || (*src++ == '0' && ((*src++ == 'x') || (*src == 'X'))))
        {
            hex = 1;
            ret = ret*16 + hextoNum(*src++);
        }
        else
        {
            ret = ret*10 + digtoNum(*src++);
        }
    }
    return ret;
}
int macAddrProcess(char *src, char *dst)
{
    int ch;
    int ret = 0;
    int i,j;
    while (*src)
    {
        ret++;
        i = hextoNum(*src);
        j = hextoNum(*(++src));
        ch = i*16 + j;
        *dst++ = ch; 
        *src++;
        if(*src == '-')
            *src++;
    }
    return ret;
}

