/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    eth_debug.c
 * \brief   Implements of the Ethernet Debug Utility
 *
 * Author(s):    Binh Tran 
 * Date:         2003-06-04
 *
 * $Revision: #8 $
 * $Date: 2005/07/13 $
 */
#ifndef NOETHERDEBUG
#ifdef AP5X
#include "oem.h"
#endif
#include "wltypes.h"
#include "IEEE_types.h" 
#include "wl_mib.h"
#include "wl_hal.h" 
#ifndef AP5X
#include "buildModes.h"
#endif
#include "sysinfo.h"
#include "mib.h"
#include "api.h"
#include "wl_macros.h"

#include "eth_debug.h"

#include <cyg/infra/cyg_type.h>
#include <cyg/infra/diag.h>

#ifndef AP5X
#include "bufmgt.h"
#ifndef WB_EXCEPTION
#include "dsport.h"
#endif
#endif
/*! \todo Why dependency on apio_GetEtherTxBuf?, where Reset defined? */
extern void Reset(void);
#ifndef AP5X
extern UINT32 * apio_GetEtherTxBuf(void);

extern QHANDLE ethTxQ;
#endif
/* Imported Variable */
extern unsigned char enaddr[]; 
extern UINT8 AssociatedFlag;
/*extern mib_DHCP_t mib_DHCP; */
#ifndef AP5X
#define eprintf etherBugSend
#endif
/* Global Variable */
UINT8 dbgCount = 0;
UINT8 dbgInit = 0;
UINT8 dbgPolling = 1;
UINT8 destMac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
UINT8 msg[DBG_BUF_MAX_LEN];
//UINT8 cmdBuf[DBG_BUF_MAX_LEN];
apio_handle_t ethDefaultHandle;  

/* IO Handler and Ethernet Type that you can register */
UINT16  etherBugType = DBG_DEFAULT_TYPE;
apio_handle_t *etherBugHandle = NULL;



#define pollingOn	dbgPolling=1
#define pollingOff dbgPolling=0

/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
typedef struct CNumMap
{
    UINT8 chr;
    int value;
}CNumMap;
#define HexMapL  16

#define DecMapL 10

CNumMap NumMap[HexMapL] =
{
    {'0', 0}, {'1', 1},
    {'2', 2}, {'3', 3},
    {'4', 4}, {'5', 5},
    {'6', 6}, {'7', 7},
    {'8', 8}, {'9', 9},
    {'a', 10}, {'b', 11},
    {'c', 12}, {'d', 13},
    {'e', 14}, {'f', 15}
};

static UINT32 etherBugStringToNum(UINT8 *string_p)
{
    UINT8 *s = string_p;
    UINT32 result = 0;
    BOOLEAN firsttime = TRUE;
    BOOLEAN found = FALSE;
    int i;
    if ( *s == '0' && *(s + 1) == 'x' )
    {
        s += 2;
        while ( *s != '\0' )
        {
            for ( i = 0; i < HexMapL; i++ )
            {
                if ( *s == NumMap[i].chr )
                {
                    if ( !firsttime ) result <<= 4;
                    result |= NumMap[i].value;
                    found = TRUE;
                    break;
                }
            }
            if ( !found ) break;
            s++;
            firsttime = FALSE;
        }
    } else
    {
        while ( *s != '\0' )
        {
            for ( i = 0; i < DecMapL; i++ )
            {
                if ( *s == NumMap[i].chr )
                {
                    if ( !firsttime ) result *= 10;
                    result += NumMap[i].value;
                    found = TRUE;
                    break;
                }
            }
            if ( !found ) break;
            s++;
            firsttime = FALSE;
        }
    }
    return(result);
}
void MacAddrString(UINT8 *str, UINT8* macAddr)
{
    UINT8 i, tmp[5];
    tmp[0] = '0';
    tmp[1] = 'x';
    tmp[4] = 0;
    for (i = 0; i <6; i++) {
        tmp[2] = *str++;
        tmp[3] = *str++;
        macAddr[i] = etherBugStringToNum(tmp);
    }
}

#define MAX_AGRC    6
#define MAX_STRING 32
UINT8 strArray[MAX_AGRC][MAX_STRING];
int StringToArray(UINT8 *string_p)
{
    UINT16 argc = 0;
    UINT8 *s, *s1;
    UINT8 spacefound = 0;
    s = s1 = string_p;
    memset(strArray[0], 0, MAX_AGRC*MAX_STRING);
    while ( *s != 0x00 || *s != 0x0d )
    {
        if ( *s == 0x20 || *s == 0x0d )
        {
            if ( *s == 0x0d )
            {
                *s = 0;
                strcpy(strArray[argc++], s1); 
                return (argc);
            }
            *s = 0;
            spacefound = 1;
            strcpy(strArray[argc++], s1); 
        }
        s++;
        if ( spacefound )
        {
            while ( *s == 0x20 )
                s++;
            s1 = s;
        }
        spacefound = 0;
    }
    return (argc);
}
/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
extern void wrtRF(UINT8 *s1, UINT8 *s2)
{
    UINT8 regNum = 0;
    UINT8 regVal = 0;
    UINT8  regValRead;

    if ( !s1 || !s2 )
    {
        eprintf("Fail to write Reg:: rfwrite <reg # in hex> <val in hex>\n");
    }

    regNum = etherBugStringToNum(s1);   
    regVal = etherBugStringToNum(s2);   
    sys_InitPHYAccess();
    msi_wl_WritePHY(regNum, regVal);
    msi_wl_ReadPHY(regNum, &regValRead);
    eprintf("RF Write: Reg Num = 0x%x Value = 0x%x\n", regNum, regVal);
    eprintf("RF Read: Reg 0x%x = 0x%x\n", regNum, regValRead);
}

/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
extern void rdRF(UINT8 *string_p)
{
    UINT8 regNum = 0;
    UINT8 regVal = 0;
    UINT8  regValRead;
    if ( !string_p )
    {
        eprintf("Fail to read Reg:: rfread <reg # in hex>\n");
    }
    regNum = etherBugStringToNum(string_p);
    sys_InitPHYAccess();
    msi_wl_ReadPHY(regNum, &regValRead);
    eprintf("RF Read: Reg 0x%x = 0x%x\n", regNum, regValRead);
}
static void WaitForDone(void)
{
    int i = 1000;
    while ( i-- );
}

/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
extern void wrtBB(UINT8 *s1, UINT8 *s2)
{
    UINT8 regNum = 0;
    UINT8 regVal = 0;
    UINT8  regValRead;

    if ( !s1 || !s2 )
    {
        eprintf("Fail to write Reg:: bbwrite <reg # in hex> <val in hex>\n");
    }

    regNum = etherBugStringToNum(s1);
    regVal = etherBugStringToNum(s2);
    sys_InitBBPAccess();
	if(regNum >= 0x80)
	{
    	msi_wl_WriteBBP(0, 0x80);
	}
	else
	{
	    msi_wl_WriteBBP(0, 0);
	}
	WaitForDone();
    msi_wl_WriteBBP(regNum, regVal);
	WaitForDone();
    msi_wl_ReadBBP(regNum, &regValRead);
    eprintf("BB Write: Reg Num = 0x%x Value = 0x%x\n", regNum, regVal);
    eprintf("BB Read: Reg 0x%x = 0x%x\n", regNum, regValRead);
}

/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
extern void rdBB(UINT8 *string_p)
{
    UINT32 regNum = 0;
    UINT32 regVal = 0;
    UINT8 regValRead;
    if ( !string_p )
    {
        eprintf("Fail to read Reg:: bbread <reg # in hex>\n");
    }
    regNum = etherBugStringToNum(string_p);
    sys_InitBBPAccess();
	if(regNum >= 0x80)
	{
    	msi_wl_WriteBBP(0, 0x80);
	}
	else
	{
	    msi_wl_WriteBBP(0, 0);
	}
	WaitForDone();
    msi_wl_ReadBBP(regNum, &regValRead);
    eprintf("BB Read: Reg 0x%x = 0x%x\n", regNum, regValRead);
}


#ifndef AP5X
extern UINT16 etherBugPoll(void)
{
    if ( !dbgPolling )
    {
        return (1);
    }
    eprintf("IP Address = %d.%d.%d.%d :: on channel %d %s\n",
            (mib_DHCP_p->IPAddr >> 24), 
            ((mib_DHCP_p->IPAddr & 0xffffff) >> 16),
            ((mib_DHCP_p->IPAddr & 0xffff) >> 8),
            (mib_DHCP_p->IPAddr & 0xff),
            msi_wl_GetRFChan(),
            mib_StaCfg_p->DesiredSsId);

    return (0);
}
#endif

static UINT8 etherBugCmdPoll(UINT8 *msgCmd)
{
    UINT8 *pMsg;

    if ( msgCmd == NULL )
    {
        return (1);
    }

    pMsg = msgCmd;

    if ( *pMsg == '0' )
    {
        pollingOff;
    } else if ( *pMsg == '1' )
    {
        pollingOn;
    }
    return (0);
}

static UINT8 etherBugCmdParse(int argc/*UINT8 *msgCmd*/)
{
    if ( strArray[0][0] == 0x0d || strArray[0][0] == 0x00 )
    {
        return (0);
    } else if ( !memcmp(strArray[0], "poll", 4) )
    {
        etherBugCmdPoll(strArray[1]);
    } else if ( !memcmp(strArray[0], "ver", 3) )
    {
#ifdef AP5X
        eprintf("WLAN Firmware Version: %s\n", GetLibwlandrvVersion());
#else        
        extern char* __version__;
        eprintf("ELVIS is Alive with ");
        eprintf("WLAN Firmware Version: %s\n", __version__);
#endif
    } else if ( !memcmp(strArray[0], "mlme", 4) )
    {
        eprintf("MLME State Machine Events\n");
        /*mlmeCmdHandle(strArray[0]); */
    } else if ( !memcmp(strArray[0], "ip", 2) )
    {
#ifdef AP5X
        UINT8 array[100];
        GetDebugInfo(&array[0]);
        eprintf("%s\n", array);
		//eprintf("channel selected %d\n",channelSelected(etherBugStringToNum(strArray[1])));
#else
        eprintf("IP Address = %d.%d.%d.%d %s\n", (mib_DHCP_p->IPAddr >> 24), 
                ( (mib_DHCP_p->IPAddr & 0xffffff) >> 16),
                ( (mib_DHCP_p->IPAddr & 0xffff) >> 8),
                (mib_DHCP_p->IPAddr & 0xff), mib_StaCfg_p->DesiredSsId );
#endif
#if !defined(WB_EXCEPTION)
    } else if ( !memcmp(strArray[0], "api", 3) )
    {
        UINT16 val;
        val = etherBugStringToNum(strArray[1]);
		Api_Get(val);
#endif
#if defined(BRIDGE_POLLING_1) && !defined(WB_EXCEPTION)
    } else if ( !memcmp(strArray[0], "resetmode", 9) )
    {
        extern UINT8 resetmode;
        UINT8 val;
        val = etherBugStringToNum(strArray[1]);
        eprintf("resetmode %d -> %d\n", resetmode, val);
        resetmode = val;
#endif
    } else if ( !memcmp(strArray[0], "reset", 5) )
    {
        if(etherBugStringToNum(strArray[1]) == 100)
            ResetToFactorySettings();
#ifdef AP5X
        if(etherBugStringToNum(strArray[1]) == 101)
            EraseUserData();
#endif
        eprintf("ELVIS Has Left The Building!!!!  ");
        eprintf("Device Reseting....\n");
        Reset();
    } else if ( !memcmp(strArray[0], "clock", 5) )
    {
        UINT8 cpuclock, sysclock;
        cpuclock = etherBugStringToNum(strArray[1]);
        sysclock = etherBugStringToNum(strArray[2]);
        hal_SetClock(cpuclock,sysclock);
        eprintf("cpu clock index = %d, system clock index =%d\n", cpuclock, sysclock);
    } else if ( !memcmp(strArray[0], "rfwrite", 7) )
    {
        wrtRF(strArray[1], strArray[2]);
    } else if ( !memcmp(strArray[0], "rfread", 6) )
    {
        rdRF(strArray[1]);

    } else if ( !memcmp(strArray[0], "bbwrite", 7) )
    {
        wrtBB(strArray[1], strArray[2]);

    } else if ( !memcmp(strArray[0], "bbread", 6) )
    {
        rdBB(strArray[1]);
    } else if ( !memcmp(strArray[0], "regr", 4) )
    {
        UINT32 reg = etherBugStringToNum(strArray[1]); 
        eprintf("reg %x = %x\n", reg, WL_REGS32(reg));
    } else if ( !memcmp(strArray[0], "regw", 4) )
    {
        UINT32 reg, val;
        reg = etherBugStringToNum(strArray[1]);
        val = etherBugStringToNum(strArray[2]);
#ifdef AP5X
        if(reg == 0xffffffff)
        {
			EraseMfgData(15, 1);
        }
        else if ((reg & 0xFFFFF000) == 0xfffff000)
        {
			flashWriteWord(reg, val);
        }else
#endif  
        	WL_REGS32(reg) = val;
        eprintf("write %x to reg %x\n", val, reg);
    } else if ( !memcmp(strArray[0], "dump", 4) )
    {
        UINT16 length;
        UINT32 start;
        start = etherBugStringToNum(strArray[1]);  
        length = etherBugStringToNum(strArray[2]);
        dump((unsigned char *)start,length);
#ifdef PROFILING
    } else if ( !memcmp(strArray[0], "prof", 4) )
    {
        UINT16 mode;
        mode = etherBugStringToNum(strArray[1]);
        if(mode)
        pf_enable();
		else
		pf_disable(strArray[2]);
#endif
#if 0//ndef WB_EXCEPTION        
    } else if ( !memcmp(strArray[0], "stats", 5) )
    {
        displayStats();
    } else if ( !memcmp(strArray[0], "power", 5) )
    {
        UINT16 t1, t2, t3;
        t1 = etherBugStringToNum(strArray[1]);  
        t2 = etherBugStringToNum(strArray[2]);
        t3 = etherBugStringToNum(strArray[3]);
        RFSetOutputPower(t1, t2, t3);
    } else if ( !memcmp(strArray[0], "gqid", 4) )
    {
        extern UINT32 CBP_QID;
        UINT32 tmp;
        tmp=etherBugStringToNum(strArray[1]);
        if(tmp == 6 || tmp == 7)
        {
            eprintf("G rate QID %d -> %d\n",CBP_QID, tmp);
            CBP_QID = tmp;
        }
        else
            eprintf("please select 6 or 7\n");

    } else if ( !memcmp(strArray[0], "bqid", 4) )
    {
        extern UINT32 BRATE_QID;
        UINT32 tmp;
        tmp=etherBugStringToNum(strArray[1]);
        if(tmp == 6 || tmp == 7)
        {
            eprintf("B rate QID %d -> %d\n",BRATE_QID, tmp);
            BRATE_QID = tmp;
        }
        else
            eprintf("please select 6 or 7\n");
#endif
#ifdef AP_URPTR
    } else if ( !memcmp(strArray[0], "ur", 2) )
    {
        displayUrInfo();
    }
    else if ( !memcmp(strArray[0], "brtbl", 5) )
    {
        displayBridgeTbl();
#endif
    } else
    {
        eprintf("No Valid Commands found\n");
    }

    return (0);
}

static void etherBugLowerCase(UINT8 *pStr)
{
    UINT16 i;
    UINT16 lenStr = strlen(pStr);

    for ( i=0; i < lenStr; i++ )
    {
        if ( *(pStr + i) > 0x40 && *(pStr + i) < 0x5b )
        {
            *(pStr + i) = *(pStr + i) + 0x20;

        }
    }

}

extern UINT32 etherBugRegIOHandler(apio_handle_t *ioHandler)
{
    if ( ioHandler == NULL )
    {
        return (1);
    }
    etherBugHandle = ioHandler;
    return (0);
}

static UINT8 etherBugBuildHdr(UINT8 *pack)
{
    eth_hdr_dbg *EtherHdr_p;

    if ( pack == NULL )
    {
        return (1);
    }

    EtherHdr_p = (eth_hdr_dbg *)pack;
    memset(pack, 0, sizeof(eth_hdr_dbg));
    memcpy(EtherHdr_p->sa, enaddr , 6);
    memcpy(EtherHdr_p->da, destMac, 6);
    EtherHdr_p->type[0] = etherBugType >> 8;
    EtherHdr_p->type[1] = etherBugType & 0xff;
    return (0);
}

UINT32 etherBugSend(const char *format, ... )
{
    eth_hdr_dbg *EtherHdr_p;
    apio_bufdescr_t BufDescr;
    UINT32 len;

    UINT8 *p;

    va_list args;

#ifndef AP5X
    if ( (EtherHdr_p =(eth_hdr_dbg *)apio_GetEtherTxBuf())
         == NULL )
    {
        return (0);
    }
#endif
    if ( etherBugHandle == NULL )
    {
        if ( !dbgInit )
        {
            dbgInit = 1;
#ifdef AP5X
            if(systemCfg.prod == AP51)
                apio_open("eth0",&ethDefaultHandle,NULL);
            else
                apio_open("lan0",&ethDefaultHandle,NULL);
#else
            apio_open(DBG_DEFAULT_ETH_IO,&ethDefaultHandle,NULL);
#endif
        }

        etherBugRegIOHandler(&ethDefaultHandle);
    }
#ifdef AP5X
    if ((apio_alloc(ethDefaultHandle, &BufDescr)) != APCTL_OK)
    {
        return (0);
    }
#ifdef GATEWAY
    if(systemCfg.prod == AP51)
		BufDescr.reserved = 1;/*mark descriptor for TX to WAN*/
#endif    
    EtherHdr_p = (eth_hdr_dbg *)BufDescr.framePtr;
#endif
    etherBugBuildHdr((UINT8 *)EtherHdr_p);
    va_start(args, format);
    vsprintf(msg, format, args);
    va_end(args);

    len = strlen(msg);

    if ( etherBugType == DBG_DEFAULT_TYPE )
    {
        ethbug_info *etherBugInfo = 
        (ethbug_info *)(((UINT8 *)EtherHdr_p) + ETH_DBG_HDR_LEN);

        /*need to add reserve data here */
        etherBugInfo->resrv[0] = 0x22;
        etherBugInfo->resrv[1] = 0x01;
        etherBugInfo->resrv[3] = dbgCount++;
        etherBugInfo->resrv[4] = len >> 8;;
        etherBugInfo->resrv[5] = len & 0xff;
        len += ETHERBUG_HDR_LEN;

        p = ((UINT8 *)EtherHdr_p) + ETH_DBG_HDR_LEN + ETHERBUG_HDR_LEN;

        /*       sprintf(p, "%u ", cyg_current_time()); */
#ifndef PROFILING
        sprintf(p, "%u ", WL_REGS32(0x8000a600));
        len +=strlen(p);
        p += strlen(p);
#endif
        memcpy(p,
               msg, len);
        len += ETH_DBG_HDR_LEN;
    } else
    {
        memcpy(((UINT8 *)EtherHdr_p) + ETH_DBG_HDR_LEN, msg, len);
        len += ETH_DBG_HDR_LEN;
    }

    /*msi_wl_PutPkts(ethTxQ, msi_wl_GetUsrBufPtr(EtherHdr_p),QUEUE_DATA_0,0,0); */
    BufDescr.frameLen = len;
#ifdef WB_EXCEPTION
    BufDescr.framePtr = (UINT8*)EtherHdr_p;
#endif
    apio_write(*etherBugHandle,&BufDescr);
    return (len);
}

/*************************************************************************
* Function:
*
* Description:
*
* Input:
*
* Output:
*
**************************************************************************/
extern UINT32 etherBugRecv(UINT8 *pack, UINT32 packLen)
{
    eth_hdr_dbg *EtherHdr_p;
    UINT8 cmdBuf[DBG_BUF_MAX_LEN];

    if ( (pack == NULL) || !((packLen - ETH_DBG_HDR_LEN) > 0) )
    {
        return (0);
    }

    EtherHdr_p = (eth_hdr_dbg *)pack;
    if ( memcmp(EtherHdr_p->da, enaddr, 6) )
    {
        return (0);

    }

    memset(cmdBuf, 0, DBG_BUF_MAX_LEN);
    if ( (packLen - ETH_DBG_HDR_LEN) >= DBG_BUF_MAX_LEN )
    {
        memcpy(cmdBuf, ((UINT8 *)EtherHdr_p) + ETH_DBG_HDR_LEN + ETHERBUG_HDR_LEN, 
               DBG_BUF_MAX_LEN - 1);
    } else
    {
        memcpy(cmdBuf, ((UINT8 *)EtherHdr_p) + ETH_DBG_HDR_LEN + ETHERBUG_HDR_LEN, 
               packLen - ETH_DBG_HDR_LEN);
    }
    etherBugLowerCase(cmdBuf);
    etherBugCmdParse(StringToArray(cmdBuf));
    return (packLen);
}

void etherBugError(int err)
{
    pollingOff;
    etherBugSend("err %d happenned\n", err);
}


static char array[3000];
void dump(char *start, int length)
{
    int len,j,i = 0;
    char c;
    char space=' ';
	if(length > 208 )  /*limited to 208 bytes*/
		len = 208;
	else
		len = length;
    for (j = 1; j<=len; j++)
    {
        c = *start++;
        array[i++] = (c >> 4)>9?(c>>4)+0x37:(c>>4)+0x30;
        array[i++] = (c&0xf )>9?(c&0xf)+0x37:(c&0xf)+0x30;
        if (j % 16)
        {
            space = ' ';
        } else
        {
            space = '\n';
        }
        array[i++] = space;
    }
    array[i] = 0;
    etherBugSend("dump %d bytes\n%s\n", len, array);
}
#ifndef WB_EXCEPTION
int Api_Get(UINT32 code)
{
	APICMDBUF *cmdBuf;
	UINT16 len;
	cmdBuf = malloc(12);
	cmdBuf->seqNum = 1;
	cmdBuf->cmdCode = code;
	cmdBuf->cmdBodyLen = 0;
	cmdBuf->cmdBody = (UINT8 *)((UINT32)cmdBuf+SIZEOF_APICMDHEADER);
	apctl(cmdBuf);
	if(cmdBuf->cmdStatus == APCTL_BUF_TOO_SMALL)
	{
		len = cmdBuf->cmdBodyLen;
		free(cmdBuf);
		cmdBuf = malloc(12 + len);
		cmdBuf->seqNum = 1;
		cmdBuf->cmdCode = code;
		cmdBuf->cmdBody = (UINT8 *)((UINT32)cmdBuf+SIZEOF_APICMDHEADER);
		cmdBuf->cmdBodyLen = len;
		apctl(cmdBuf);
	}
	dump((UINT8 *)((UINT32)&(cmdBuf->cmdBodyLen)+2), cmdBuf->cmdBodyLen+4);
	dump(cmdBuf->cmdBody, cmdBuf->cmdBodyLen);
	free(cmdBuf);
}
#endif
#endif /*NOETHERDEBUG */
