/******************** (c) Marvell Semiconductor, Inc., 2003 *******************
*
* $HEADER$
*
* Purpose:
*    This file contains the function prototypes and definitions for the
*    MfgTask.
*
* Public Procedures:
*
* Notes:
*    None.
*
*****************************************************************************/

#ifndef _MFGTASK_H_
#define _MFGTASK_H_

//=============================================================================
//                               INCLUDE FILES
//=============================================================================


//=============================================================================
//                            PUBLIC DEFINITIONS
//=============================================================================
#define  MFG_TASK_STACK_SIZE       4096 
/* mfg task stack size */
#ifdef AP5X
#define  MFG_TASK_PRI                12 
#else
#define  MFG_TASK_PRI                20 
#endif
/* mfg task priority */
#define  MFG_TASK_NAME        "MfgTask" 
/* mfg task name */

#define PIU_LEN         70 
/* number of byte for the PIU (PID) flash section */
#define INT_PA_SIZE     14 
/* internal PA size (number of WORD) */
#define MFG_CCA_SIZE    8 
/* CCA manufacturing size in byte */

#define OFFSET_CALTABLE (mfgDataAddress() & 0x1FFFFFL) 
/* offset calibration table for mfg data type 1 */

#define PORT 9930 
/* port number for the UDP connection */
#define IP_OFFSET_ADDR 90 
/* client IP address (that would be the PC) is AP IP address + OFFSET */

#define _REG(x) (*(volatile unsigned long *) (x)) 
/* access 32 bits register */
#define _REG16(x) (*(volatile unsigned short *) (x)) 
/* access 16 bits register */

#define NORMAL_MODE  0
/* normal (or customer mode) */
#define MFG_MODE     1
/* manufacturing mode (no beacon...) */

#define NUMOFILTERREF 17
/* number of low pass filter */

#define LEN_RELEASENOTE    256
/* release note size in byte */

/*========================================================================== */
/*                         PUBLIC TYPE DEFINITIONS                           */
/*========================================================================== */
enum {
    CMD_WR_RF        = 0x0002,        /* write MAC registers */
    RSP_WR_RF        = 0x0003,
    CMD_RD_RF        = 0x0004,        /* read MAC registers */
    RSP_RD_RF        = 0x0005,
    CMD_WR_BB        = 0x0006,        /* write BB registers */
    RSP_WR_BB        = 0x0007,
    CMD_RD_BB        = 0x0008,        /* read BB registers */
    RSP_RD_BB        = 0x0009,
    CMD_WR_MAC       = 0x000A,        /* write MAC register */
    RSP_WR_MAC       = 0x000B,
    CMD_RD_MAC       = 0x000C,        /* read MAC register */
    RSP_RD_MAC       = 0x000D,
    CMD_WR_VMR       = 0x000E,        /* write to the flash */
    RSP_WR_VMR       = 0x000F,
    CMD_RD_VMR       = 0x0010,        /* read from the flash */
    RSP_RD_VMR       = 0x0011,
    CMD_ER_VMR       = 0x0012,        /* erase sector from flash */
    RSP_ER_VMR       = 0x0013,
    CMD_ST_VMR       = 0x0014,        /* return flash status */
    RSP_ST_VMR       = 0x0015,
    CMD_ST_DISS      = 0x0016,        /* Disassociate (not implemented) */
    RSP_ST_DISS      = 0x0017,        
    CMD_SET_BCN_ENB  = 0x0018,        /* Beacon Tx on or off */
    RSP_SET_BCN_ENB  = 0x0019,        
    CMD_CHA_BCN      = 0x001A,        /* set beacon channel */
    RSP_CHA_BCN      = 0x001B,        
    CMD_TX_FRAME     = 0x001C,        /* send Tx frame */
    RSP_TX_FRAME     = 0x001D,        
    CMD_RF_ENA       = 0x001E,        /* RF Tx enable */
    RSP_RF_ENA       = 0x001F,        
    CMD_WR_RFCHA     = 0x0020,        /* set RF channel */
    RSP_WR_RFCHA     = 0x0021,        
    CMD_RD_RFCHA     = 0x0022,        /* get RF channel */
    RSP_RD_RFCHA     = 0x0023,        
    CMD_WR_RFPOW     = 0x0024,        /* set RF power */
    RSP_WR_RFPOW     = 0x0025,        
    CMD_RD_RFPOW     = 0x0026,        /* get RF power */
    RSP_RD_RFPOW     = 0x0027,        
    CMD_RF_STEP      = 0x0028,        /* step RF power */
    RSP_RF_STEP      = 0x0029,        
    CMD_TX_ANT       = 0x002A,        /* set Tx Antenna */
    RSP_TX_ANT       = 0x002B,        
    CMD_RX_ANT       = 0x002C,        /* set Rx Antenna */
    RSP_RX_ANT       = 0x002D,        
    CMD_TX_RATE      = 0x002E,        /* set Tx data rate */
    RSP_TX_RATE      = 0x002F,        
    CMD_SET_22M      = 0x0030,        /* enable 22 Mbps */
    RSP_SET_22M      = 0x0031,        
    CMD_SET_PRMBL    = 0x0032,        /* set short/long preamble */
    RSP_SET_PRMBL    = 0x0033,        
    CMD_SET_PWMODE   = 0x0034,        /* set power mode */
    RSP_SET_PWMODE   = 0x0035,        
    CMD_RSSI_PWOPT   = 0x0036,        /* rssi power option */
    RSP_RSSI_PWOPT   = 0x0037,        
    CMD_SET_TXCONT   = 0x0038,        /* set Tx continuous mode */
    RSP_SET_TXCONT   = 0x0039,        
    CMD_CLR_RSSI_MP  = 0x003A,        /* clear rssi multi packet */
    RSP_CLR_RSSI_MP  = 0x003B,        
    CMD_GET_RSSI_MP  = 0x003C,        /* get rssi multi packet */
    RSP_GET_RSSI_MP  = 0x003D,        
    CMD_CLR_RSSI_UP  = 0x003E,        /* clear rssi uni packet */
    RSP_CLR_RSSI_UP  = 0x003F,        
    CMD_GET_RSSI_UP  = 0x0040,        /* get rssi uni packet */
    RSP_GET_RSSI_UP  = 0x0041,        
    CMD_GET_RSSI     = 0x0042,        /* get rssi power */
    RSP_GET_RSSI     = 0x0043,        
    CMD_SET_BCN_INT  = 0x0044,        /* set beacon interval */
    RSP_SET_BCN_INT  = 0x0045,        
    CMD_GET_BCN_INT  = 0x0046,        /* get beacon interval */
    RSP_GET_BCN_INT  = 0x0047,        
    CMD_GET_HW       = 0x0048,        /* get Rf & BB hardware version */
    RSP_GET_HW       = 0x0049,        
    CMD_GET_MAC_ADD  = 0x004A,        /* get MAC address */
    RSP_GET_MAC_ADD  = 0x004B,        
    CMD_SET_MAC_ADD  = 0x004C,        /* set MAC address */
    RSP_SET_MAC_ADD  = 0x004D,        
    CMD_GET_CCODE    = 0x004E,        /* get Country Code */
    RSP_GET_CCODE    = 0x004F,        
    CMD_SET_CCODE    = 0x0050,        /* set Country Code */
    RSP_SET_CCODE    = 0x0051,        
    CMD_GET_PIU      = 0x0052,        /* get PIU table */
    RSP_GET_PIU      = 0x0053,        
    CMD_SET_PIU      = 0x0054,        /* set PIU table */
    RSP_SET_PIU      = 0x0055,        
    CMD_GET_MFG      = 0x0056,        /* get mfg hard infos */
    RSP_GET_MFG      = 0x0057,        
    CMD_SET_MFG      = 0x0058,        /* set mfg hard infos */
    RSP_SET_MFG      = 0x0059,        
    CMD_GET_VERS     = 0x005A,        /* get mfg firmware version */
    RSP_GET_VERS     = 0x005B,        
    CMD_ENB_SSID     = 0x005C,        /* enable BSSID filtering */
    RSP_ENB_SSID     = 0x005D,        
    CMD_SET_FW_MODE  = 0x005E,        /* fw mode (mfg or customer) */
    RSP_SET_FW_MODE  = 0x005F,        
    CMD_WR_RF_REG    = 0x0060,        /* write RF register */
    RSP_WR_RF_REG    = 0x0061,
    CMD_RD_RF_REG    = 0x0062,        /* read RF register */
    RSP_RD_RF_REG    = 0x0063,
    CMD_SET_72M      = 0x0064,        /* enable the 72 Mbps */
    RSP_SET_72M      = 0x0065,
    CMD_SET_LPF      = 0x0066,        /* set RF and BB low pass filter */
    RSP_SET_LPF      = 0x0067,
    CMD_WAKE_UP_RF   = 0x0068,        /* wake up RF and BB */
    RSP_WAKE_UP_RF   = 0x0069,
    CMD_XOSC_CAL     = 0x006A,        /* Xosc calibration */
    RSP_XOSC_CAL     = 0x006B,
    CMD_GET_BCN_ENB  = 0x006C,        /* read Beacon switch */
    RSP_GET_BCN_ENB  = 0x006D,        
    CMD_CLR_RX_ERR   = 0x006E,        /* clear RX error counter */
    RSP_CLR_RX_ERR   = 0x006F,
    CMD_GET_RX_ERR   = 0x0070,        /* read Rx error counter */
    RSP_GET_RX_ERR   = 0x0071,
    CMD_GET_TX_ANT   = 0x0072,        /* get Tx antenna */
    RSP_GET_TX_ANT   = 0x0073,
    CMD_GET_RX_ANT   = 0x0074,        /* get Rx antenna */
    RSP_GET_RX_ANT   = 0x0075,
    CMD_GET_TX_RATE  = 0x0076,        /* get Tx data rate */
    RSP_GET_TX_RATE  = 0x0077,
    CMD_EN_CST       = 0x0078,        /* enable carrier suppression test */
    RSP_EN_CST       = 0x0079,
    CMD_DUTY_CYCLE   = 0x007A,        /* duty cycle */
    RSP_DUTY_CYCLE   = 0x007B,
    CMD_SET_VGA      = 0x007C,        /* set VGA */
    RSP_SET_VGA      = 0x007D,
    CMD_GET_VGA      = 0x007E,        /* get VGA */
    RSP_GET_VGA      = 0x007F,
    CMD_SET_TCAP     = 0x0080,        /* set LC tank cap */
    RSP_SET_TCAP     = 0x0081,
    CMD_GET_TCAP     = 0x0082,        /* get LC tank cap */
    RSP_GET_TCAP     = 0x0083,
    CMD_RF_CORR      = 0x0084,        /* get RF power correction */
    RSP_RF_CORR      = 0x0085,
    CMD_RF_SETTING   = 0x0086,        /* get RF power setting */
    RSP_RF_SETTING   = 0x0087,
    CMD_SET_BAND_AG  = 0x0088,        /* set the band A or B */
    RSP_SET_BAND_AG  = 0x0089,
    CMD_GET_BAND_AG  = 0x008A,        /* get the band A or B */
    RSP_GET_BAND_AG  = 0x008B,
    CMD_SET_RFXTAL_CTRL = 0x008C,     /* set RF crystal control */
    RSP_SET_RFXTAL_CTRL = 0x008D,
    CMD_GET_RFXTAL_CTRL = 0x008E,     /* get RF crystal control */
    RSP_GET_RFXTAL_CTRL = 0x008F,
    CMD_GET_RF_RC_CAL   = 0x0090,     /* get RF RC cal */
    RSP_GET_RF_RC_CAL   = 0x0091,
    CMD_SET_RF_RCCTRL   = 0x0092,     /* set RF RC control */
    RSP_SET_RF_RCCTRL   = 0x0093,
    CMD_GET_RF_RCCTRL   = 0x0094,     /* get RF RC control */
    RSP_GET_RF_RCCTRL   = 0x0095,

    CMD_PA_LNA_MODE     = 0x00A2,     /* PA LNA mode (ext or int) */
    RSP_PA_LNA_MODE     = 0x00A3,
    CMD_SET_RF_LOOP     = 0x00A4,     /* set RF loop mode */
    RSP_SET_RF_LOOP     = 0x00A5,
    CMD_GET_RF_LOOP     = 0x00A6,     /* get RF loop mode */
    RSP_GET_RF_LOOP     = 0x00A7,
    CMD_SET_THERMAL     = 0x00A8,     /* set RF thermal scaler */
    RSP_SET_THERMAL     = 0x00A9,
    CMD_GET_THERMAL     = 0x00AA,     /* get RF thermal scaler */
    RSP_GET_THERMAL     = 0x00AB,
    CMD_SET_RANGE       = 0x00AC,     /* set RF range table */
    RSP_SET_RANGE       = 0x00AD,
    CMD_GET_RANGE       = 0x00AE,     /* get RF range table */
    RSP_GET_RANGE       = 0x00AF,
    CMD_GET_REL_NOTE    = 0x00B0,     /* get release note */
    RSP_GET_REL_NOTE    = 0x00B1,
    CMD_SETTING_EEPROM  = 0x00B2,     /* load settings from eeprom */
    RSP_SETTING_EEPROM  = 0x00B3,
    CMD_GET_PWRTBL      = 0x00B4,     /* upload power table */
    RSP_GET_PWRTBL      = 0x00B5,
    CMD_SET_PWRTBL      = 0x00B6,     /* dowload power table */
    RSP_SET_PWRTBL      = 0x00B7,
    CMD_GET_PWRVAL      = 0x00B8,     /* get RF power value */
    RSP_GET_PWRVAL      = 0x00B9,
    CMD_SET_PWRVAL      = 0x00BA,     /* set RF power value */
    RSP_SET_PWRVAL      = 0x00BB,
    CMD_HW_RF_RESET     = 0x00BC,     /* RF reset */
    RSP_HW_RF_RESET     = 0x00BD,
    CMD_WRITE_FLASH     = 0x00BE,     /* write block of data */
    RSP_WRITE_FLASH     = 0x00BF,     
    CMD_READ_FLASH      = 0x00C0,     /* read block of data */
    RSP_READ_FLASH      = 0x00C1,     

    CMD_ST_TERM         = 0x0100,     /* end of test (set DHCP mode) */
    RSP_ST_TERM         = 0x0101,
};
/* enum used by the main mfgtask to filter the different commands */

enum 
{
    SIGNAL_801_11B, 
    SIGNAL_801_11G
}; 
// 11B or 11G signal

enum 
{
    MANU_ID        = 0x0000,
    API_ID         = 0x0001
};
// enum used to distinguish between the messages.

typedef unsigned long DWORD;
/* DWORD defined as 32 bits */

typedef unsigned short WORD;
/* WORD defined as 16 bits */

typedef unsigned char BYTE;
/* BYTE defined as 8 bits */

typedef PACK_START struct
{
UINT8 antGrntTh2:
    5;
UINT8 antNoGap:
    1;
UINT8 antDivModeTx:
    2;
} PACK_END antDiversityTx_t;
/* structure used for the TX antenna diversity (register Diversity 1) */
/* the fields in the structure are: */
/*    1) field1, gradient threshold */
/*    2) field2, antenna control */
/*    3) field3, antenna diversity mode */

typedef PACK_START struct
{
UINT8 antGrntTh1:
    6;
UINT8 antDivModeRx:
    2;
} PACK_END antDiversityRx_t;
/* structure used for the RX antenna diversity (register diversity 0) */
/* the fields in the structure are: */
/*    1) field1, gradient threshold */
/*    2) field2, receive antenna diversity mode */

#ifndef CONDOR
typedef PACK_START struct
{
UINT8 reserved:
    2;
UINT8 rxExtRxclk:
    1;
UINT8 rxMdrdyFallDtnd:
    1;
UINT8 rxMdrdyRiseCrc:
    1;
UINT8 rxBypsDeScrambler:
    1;
UINT8 rxEnCont:
    1;
UINT8 rxEn22:
    1;
} PACK_END receiveControl0_t;
/* structure used to enable the 22 Mbps */
/* the fields in the structure are: */
/*    1) field1, reserved */
/*    2) field2, RX clock */
/*    3) field3, RD_RDY de-assert */
/*    4) field4, MD_RDY assert */
/*    5) field5, bypass the descrambler */
/*    6) field6, enable continuous receive mode */
/*    7) field7, enable 22 Mbps receive mode */
#else
typedef PACK_START struct
{
UINT8 reserved:
    2;
UINT8 rxLngth11gDis:
    1;
UINT8 rxMode:
    1;
UINT8 bypsSrvb411b:
    1;
UINT8 rxContEn:
    1;
UINT8 rxEn72:
    1;
UINT8 rxEn22:
    1;
} PACK_END receiveControl0_t;
/* structure used to enable the 72 Mbps for 11g */
/* the fields in the structure are: */
/*    1) field1, reserved */
/*    2) field2, disable RxRdy for zero rx-length */
/*    3) field3, Rx mode */
/*    4) field4, bypass service bit */
/*    5) field5, enable continuous receive mode */
/*    6) field6, enable 72 Mbps receive mode */
/*    7) field7, enable 22 Mbps receive mode */
#endif

#ifndef CONDOR
typedef PACK_START struct
{
UINT8 reserved:
    2;
UINT8 txClockCtrl:
    1;
UINT8 cckModCtrl:
    1;
UINT8 reserved1:
    1;
UINT8 txEn22:
    1;
UINT8 ext22M:
    1;
UINT8 ext1122:
    1;
} PACK_END transmitControl5_t;
/* structure used for the TX control (enable 22 Mbps) */
/* the fields in the structure are: */
/*    1) field1, reserved */
/*    2) field2, transmit clock control */
/*    3) field3, CCK modulation control */
/*    4) field4, reserved */
/*    5) field5, enable for 22 Mbps mode */
/*    6) field6, length extension bit for 22 Mbps */
/*    7) field7, length extension bit for 11/22 Mbps */
#else
typedef PACK_START struct
{
UINT8 reserved:
    2;
UINT8 txClockCtrl:
    1;
UINT8 cckModCtrl:
    1;
UINT8 txEn22:
    1;
UINT8 reserved1:
    1;
UINT8 ext22M:
    1;
UINT8 ext1122:
    1;
} PACK_END transmitControl5_t;
/* structure used for the TX control (enable 22 Mbps) */
/* the fields in the structure are: */
/*    1) field1, reserved */
/*    2) field2, transmit clock control */
/*    3) field3, CCK modulation control */
/*    4) field4, enable for 22 Mbps mode */
/*    5) field5, reserved */
/*    6) field6, length extension bit for 22 Mbps */
/*    7) field7, length extension bit for 11/22 Mbps */
#endif

typedef PACK_START struct
{
UINT8 reserved:
    1;
UINT8 txLenTxpe:
    1;
UINT8 txPeCont:
    1;
UINT8 txEnCw:
    1;
UINT8 txEnCst:
    1;
UINT8 txRxLenUs:
    1;
UINT8 txShrtPrmbl:
    1;
UINT8 txBypsFil11b:
    1;
} PACK_END transmitControl6_t;
/* structure used for the TX control (enable CW, short preamble) */
/* the fields in the structure are: */
/*    1) field1, reserved, */
/*    2) field2, start tail and pad bits in a TX packet */
/*    3) field3, enables continuous TX test mode */
/*    4) field4, enable CW test mode */
/*    5) field5, enable carrier suppression test */
/*    6) field6, definition for TX/RX length (us or byte) */
/*    7) field7, short preamble TX */
/*    8) field8, bypass Tx poly-phase filter */

typedef PACK_START struct
{
UINT8 ccaThEdHi:
    6;
UINT8 reserved:
    1;
UINT8 ccaRssiSel:
    1;
} PACK_END ccaControl1_t;
/* structure used for the CCA control */
/* the fields in the structure are: */
/*    1) field1, threshold for energy detection */
/*    2) field2, reserved */
/*    3) field3, RSSI definition (absolute or relative) */

typedef PACK_START struct
{
UINT8 ccaTimeRxpe:
    4;
UINT8 cca8hold:
    1;
UINT8 cca16hold:
    1;
UINT8 ccaRdbkSel:
    2;
} PACK_END ccaControl6_t;
/* structure used for the CCA control (antenna selection) */
/* the fields in the structure are: */
/*    1) field1, CCA busy delay */
/*    2) field2, hold CCA busy if 8 consecutive 1s or 0s */
/*    3) field3, hold CCA busy if 16 consecutive 1s or 0s */
/*    4) field4, antenna select for RSSI readback */

typedef PACK_START struct
{
    UINT8  boardDesc[8];
    UINT16 paOptions;
    UINT8  extPA;
    UINT8  antenna;
    UINT16 intPA[INT_PA_SIZE];
    UINT8  cca[MFG_CCA_SIZE];
    UINT16 countryCode;
    UINT16 customerOpt;
    UINT8  led[4];
    UINT16 reserved[3];
    UINT16 checksum;
} PACK_END mfgHardInfo_t;
/* structure used for the mfg - hardware information data structure type 1 */
/* the fields in the structure are: */
/*    1) field1, board description (8bytes) */
/*    2) field2, PA option */
/*    3) field3, external PA */
/*    4) field4, antenna selection */
/*    5) field5, internal PA calibration table (2 * 14 bytes) */
/*    6) field6, CCA configuration */
/*    7) field7, country/region code */
/*    8) field8, customer options */
/*    9) field9, led */
/*    10) field10, reserved bytes */
/*    11) field11, checksum */

typedef struct PKHEADER
{
    unsigned short int id;
    unsigned short int cmd;
    unsigned short int seq;
    unsigned short int off;
    unsigned short int seg;
    unsigned short int len;
} PkHeader_t;
/* structure used for the udp message */
/* the fields in the structure are: */
/*    1) field1, identificator (not used) */
/*    2) field2, command types */
/*    3) field3, seq (??? not used) */
/*    4) field4, offset (not used) */
/*    5) field5, segment (not used) */
/*    6) field6, length (not used) */

typedef struct
{
    struct PKHEADER header;
    DWORD errorcode;
} PK_Std_t;
/* structure used for the standard command messages */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code (FAIL if different from 0) */

typedef struct
{
    struct PKHEADER header;
    DWORD errorcode;
    DWORD address;
    DWORD data;
} PK_RfReg_t;
/* structure used to access the AP registers */
/* the fields in the structure are: */
/*   1) field1, frame header */
/*   2) field2, error code */
/*   3) field3, register address */
/*   4) field4, data to/from the register */

typedef struct
{
    struct PKHEADER header;
    DWORD errorcode;
    WORD manufacturer;
    WORD part;
    WORD sectors;
    WORD numMB;
} PK_VmrStatus_t;
/* structure used for the flash status */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, manufacturer type, */
/*    4) field4, part ID */
/*    5) field5, number of sectors */
/*    6) field6, flash size in MB */

typedef struct
{
    struct PKHEADER header;
    DWORD errorcode;
    DWORD data;
} PK_Data_t;
/* structure used for frame with 1 parameter */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, data */

typedef struct
{
    struct PKHEADER header;
    DWORD errorcode;
    DWORD data1;
    DWORD data2;
} PK_Data2_t;
/* structure used for frame with 2 parameters */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, first data */
/*    4) field4, second data */

typedef struct
{
    struct PKHEADER header;
    DWORD  errorcode;
    UINT32 flashOffsetPiuTable; 
    BYTE   piuTable[PIU_LEN];
} PK_PiuTable_t;
/* structure used to store the PIU mfg data */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, PIU (PID) table (70 bytes) */

typedef struct
{
    struct PKHEADER header;
    DWORD errorcode;
    DWORD dataRate;
    DWORD framePattern;
    DWORD frameLength;
    DWORD frameCount;
    DWORD shortPreamble;
    UINT8 bssid[6];
} PK_TxFrame_t;
/* structure used by the TX frame function */
/* the fields in the structure are: */
/*    1) field1, frame header, */
/*    2) field2, error code, */
/*    3) field3, data rate (1 to 22 Mbps) */
/*    4) field4, frame pattern (ex: 0x0A0A0A0A) */
/*    5) field5, payload length (byte number) */
/*    6) field6, frame count (number of frame to be sent) */
/*    7) field7, to select short or long preamble */
/*    8) field8, BSSID (for filtering) */

typedef struct
{
    struct PKHEADER header;
    DWORD  errorcode;
    UINT32 flashOffsetMacAddr; 
    UINT8  macAddr[6];
} PK_MacAddr_t;
/* structure used for the MAC address */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, MAC address (6 bytes) */
/*    4) field4, flash offset MAC address */

typedef struct
{
    struct PKHEADER header;
    DWORD errorcode;
    char version[50];
} PK_MfgVersion_t;
/* structure used for the FW version */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    4) field3, FW version ex: "1.1.1.12 (Mar 12 2003 09:42:15)" */

typedef struct
{
    struct PKHEADER header;
    DWORD  errorcode;
    DWORD  flashOffsetMfgInfo;
    DWORD  data;
    mfgHardInfo_t mfgInfo;
} PK_MfgInfo_t;
/* structure used for the manufacturing info data */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, flash offset mfg info */
/*    4) field4, data */
/*    5) field5, mfg info */

typedef struct
{
    struct PKHEADER header;
    DWORD errorcode;
    char ssid[32];
    UINT8 bssid[6];
} PK_EnableSsid_t;
/* structure used for the FW version */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    4) field3, SSID */
/*    5) field4, BSSID */

typedef struct
{
    struct PKHEADER header;
    DWORD errorcode;
    DWORD powerRefDac;
    DWORD preDriver;
    DWORD polarization;
} PK_RfPower_t;
/* structure used for setRfPower */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, power reference DAC */
/*    4) field4, pre driver */
/*    5) field5, polarization */

typedef struct
{
    struct PKHEADER header;
    DWORD  errorcode;
    DWORD  enable;
    DWORD  dataRate;
    DWORD  percent;
} PK_DutyCycle_t;
/* structure used for the FW version */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, enable (0 or 1) */
/*    4) field4, data rate 0 -> 1Mbps, 1-> 2Mbps... */
/*    5) field5, percentage */

typedef struct
{    
    struct PKHEADER header;
    DWORD  errorcode;
    DWORD  paConfig1;
    DWORD  paConfig2;
    DWORD  paConfig3;
    DWORD  vtPowerDetHi;
    DWORD  vtPowerDetLo;
    DWORD  powerTarget;
    DWORD  dacX;
} PK_RfSetting_t;
/* structure used by the getRfPowerSetting function */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, Tx PA configuration 1 */
/*    4) field4, Tx PA configuration 2 */
/*    5) field5, Tx PA configuration 3 */
/*    6) field6, Tx power detect Hi threshold */
/*    7) field7, Tx power detect Lo threshold */
/*    8) field8, Tx target output power */
/*    9) field9, power detect DAC */

typedef struct
{
    struct PKHEADER header;
    DWORD  errorcode;
    DWORD  extPA;
    DWORD  extLNA;
} PK_PaLnaMode_t;
/* structure used for the PA, LNA mode */
/* the fields in the structure are: */
/*    1) field1, frame header */
/*    2) field2, error code */
/*    3) field3, extPA (1 ext, 0 int) */
/*    4) field4, extLNA (1 ext, 0 int) */

typedef struct 
{
    struct PKHEADER header;
    DWORD  errorcode;
    DWORD  thermalOnlyControl;
} PK_RfLoop_t;
   
typedef struct 
{
    struct PKHEADER header;
    DWORD  errorcode;
    DWORD  thermalScaler;
} PK_RfThermal_t;

typedef struct
{
    UINT32 val1Eh; 
    UINT32 val1Fh; 
    UINT32 val20h; 
    UINT32 topPwr; 
    UINT32 btmPwr; 
} pwrRangeTbleRow_t;

typedef struct
{
    struct PKHEADER   header;
           DWORD      errorcode;
    pwrRangeTbleRow_t rangeTable_1_13_High1;  
    pwrRangeTbleRow_t rangeTable_1_13_High2;  
    pwrRangeTbleRow_t rangeTable_1_13_High3;  
    pwrRangeTbleRow_t rangeTable_1_13_High4;  
    pwrRangeTbleRow_t rangeTable_14_High1;  
    pwrRangeTbleRow_t rangeTable_14_High2;  
    pwrRangeTbleRow_t rangeTable_14_High3;  
    pwrRangeTbleRow_t rangeTable_14_High4;  
} PK_RfPwrRange_t;

typedef struct
{
    struct PKHEADER header;
    DWORD  errorcode;
    UINT8  relNote[LEN_RELEASENOTE]; 
} PK_RelNote_t;

typedef struct 
{
    struct PKHEADER header;
    DWORD  errorcode;
    DWORD  tableId;         /* as band 0 for G, 1 for A */
    DWORD  maxPower; 
    DWORD  minPower; 
    DWORD  sizeOfTblInByte; 	
    BYTE   tbl[1]; 
} PK_LoadPwrTbl_t;

typedef struct
{
    struct PKHEADER header;
    DWORD  errorcode;
    DWORD  rangeIndexPwrPnt;
    DWORD  top;
    DWORD  btm;
} PK_RfRange_t;

typedef struct
{
    struct PKHEADER header;
    DWORD  errorcode;
    DWORD  size;
    DWORD  address;
    DWORD  tbl[128];
} PK_BlockFlash_t;

/* ========================================================================== */
/*                    PUBLIC PROCEDURES (ANSI Prototypes) */
/* ========================================================================== */
extern void MfgInit ();
extern UINT32 setRfBbLpf (UINT32 channel);
extern UINT32 set22M (UINT32 enable22);
extern UINT32 set72M (UINT32 enable72);

#endif /* _MFGTASK_H */
