/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    mfgtask.c
 * \brief   Manufacturing/calibration task.
 *
 * $Revision: #26 $
 * $Date: 2005/10/20 $
 *
*/

/*=========================================================================== */
/*                               INCLUDE FILES */
/*=========================================================================== */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "wltypes.h"
#ifdef AP5X
#include "oem.h"
#else
#include "timer.h"
#endif
#include "osif.h"
#include <netdb.h>
#include "mfgtask.h"
#include <stdlib.h>
#include <errno.h>
#ifdef AP31HOST
#include "ap_mib.h"
#include "ap_api.h"
cyg_sem_t wait_udp_response;
APICMDBUF *udp_apicmdbuf_p;
UINT8 lastcmdCode;
#endif
#include "IEEE_types.h"
#ifdef WB_EXCEPTION
#include "wl_mib.h"
#include "api.h"
#else
#include "mib.h"
#include "api.h"
mib_DHCP_t *mib_DHCP_p;
#endif
/*=========================================================================== */
/*                              DEFINITIONS */
/*=========================================================================== */
#define SEND_TO   0
    /* use for the trace, display <- */
#define RECV_FROM 1
    /* use for the trace, display -> */

#define MTU		1532	// Ether size
#define ETHII_SIZE	14      // MAC address etc
#define IP_SIZE		20	// IP addresses
#define UDP_SIZE	 8	// UDP header

#define RELEASENOTE "Supporting:\n\
SOC version: A0,B0\n\
Mac_Doc:0.6\n\
BBU_Doc:1.8\n\
8010:B0-B5,Doc 1.6\n\
8015:A0,A1,B0, Doc 0.7\n\
Feature : disable TRPC, enable AFC\n\
ThermalOnly and Close-Loop PA ctrl and 4+ch range settings\n"

/*=========================================================================== */
/*                             TYPE DEFINITIONS */
/*=========================================================================== */
union {
    void (*branch) (void);
    int hackfest;
} u;
/* union used to reboot the AP when the mfg test is done */
/* the fields in the union are: */
/*    1) field1: to branch the PC */
/*    2) field2: where to boot (flash base address) */

/*=========================================================================== */
/*                         IMPORTED PUBLIC VARIABLES */
/*=========================================================================== */
extern UINT8 mib_currentmacaddress[6];
#ifdef ECOS
extern cyg_sem_t ip_ready;
#endif
#ifdef AP5X
extern char * FIRMWARE_VERSION;
#else
extern char * __version__;
#endif
extern int close (int);
extern void wait_ipReady (void);
extern void initiate_dhcp (int force);
extern void init_uart (void);
extern void diag_printf (const char *fmt, ... );
extern int read (int fd, char *Buff, int NumBytes);
extern int write (int fd, char *Buff, int NumBytes);
extern void flashInterrogate (void);
extern int flashWriteWord (int offset, unsigned short dat);
extern int EraseMfgData (int sblk, int nblks);
extern UINT32 readBBReg (UINT8 reg, UINT8 *val);
extern UINT32 writeBBReg (UINT8 reg, UINT8 val);
extern UINT32 readRfReg (UINT8 reg, UINT8 *val);
extern UINT32 writeRfReg (UINT8 reg, UINT8 val);
extern UINT32 setBeaconChannel (PK_Data_t *pkData);
extern UINT32 txMulticastFrame (PK_TxFrame_t *pkTxFrame);
extern UINT32 setBeaconEnable (PK_Data_t *pkData);
extern UINT32 getBeaconEnable (PK_Data_t *pkData);
extern UINT32 setRfChannel (PK_Data_t *pkdata);
extern UINT32 getRfChannel (PK_Data_t *pkdata);
extern UINT32 setRfPower (PK_RfPower_t *pkrfpower);
extern UINT32 getRfPower (PK_RfPower_t *pkrfpower);
extern UINT32 stepRfPower (SINT32 stepPower, UINT32 polarization);
extern UINT32 rfTxEnable (PK_Data_t *pkdata);
extern UINT32 setTxAntenna (PK_Data_t *pkdata);
extern UINT32 getTxAntenna (PK_Data_t *pkdata);
extern UINT32 setRxAntenna (PK_Data_t *pkdata);
extern UINT32 getRxAntenna (PK_Data_t *pkdata);
extern UINT32 setTxDataRate (PK_Data_t *pkdata);
extern UINT32 getTxDataRate (PK_Data_t *pkdata);
extern UINT32 set22M (UINT32 enable22);
extern UINT32 setShortPreamble (PK_Data_t *pkdata);
extern UINT32 setPowerMode (PK_Data_t *pkdata);
extern UINT32 setRssiPowerOption (PK_Data2_t *pkdata2);
extern UINT32 setTxContinuousMode (PK_Data2_t *pkdata2);
extern UINT32 clearRxMultiCastPckt (void);
extern UINT32 getRxMultiCastPckt (PK_Data_t *pkdata);
extern UINT32 clearRxUniCastPckt (void);
extern UINT32 getRxUniCastPckt (PK_Data_t *pkdata);
extern UINT32 getRssiPower (PK_Data_t *pkdata);
extern UINT32 setBeaconInterval (PK_Data_t *pkdata);
extern UINT32 getBeaconInterval (PK_Data_t *pkdata);
extern UINT32 getHardwareVersion (PK_Data2_t *pkdata2);
extern UINT32 setMacAddress (PK_MacAddr_t *pkmacaddr);
extern UINT32 getMacAddress (PK_MacAddr_t *pkmacaddr);
extern UINT32 setCountryCode (PK_Data2_t *pkdata2);
extern UINT32 getCountryCode (PK_Data2_t *pkdata2);
extern UINT32 setPiuTable (PK_PiuTable_t *pkpiutable);
extern UINT32 getPiuTable (PK_PiuTable_t *pkpiutable);
extern UINT32 setMfgInfo (PK_MfgInfo_t *pkmfginfo);
extern UINT32 getMfgInfo (PK_MfgInfo_t *pkmfginfo);
extern UINT32 IPAddrPlus90 (char *IPAddr);
extern UINT32 IPAddr (char *IPAddr);
extern UINT32 enableSSID (char *ssid, UINT8 bssid[6]);
extern UINT32 setFwMode (UINT32 fwModeVal);
extern UINT32 enableDHCP (UINT32 enable);
extern UINT32 set72M (UINT32 enable72);
extern UINT32 setRfBbLpf (UINT32 channel);
extern UINT32 wakeUpRfBb (void);
extern UINT32 xoscCal (UINT32 *cal, UINT32 tu);
extern UINT32 clearRxErrPckt (void);
extern UINT32 getRxErrPckt (PK_Data_t *pkdata);
extern UINT32 setTxCarrierSuppression (UINT32 enableTxTest);
extern UINT32 setTxDutyCycle (UINT32 enable, UINT32 dataRate, UINT32 percent);
extern UINT32 getMfgVersion (PK_MfgVersion_t *pkmfgversion);
extern UINT32 setTxVgaGain (UINT32 xgvga, UINT32 vgaGain);
extern UINT32 getTxVgaGain (UINT32 *xgvga, UINT32 *vgaGain);
extern UINT32 setLcTankCap (UINT32 extSelect, UINT32 capValue);
extern UINT32 getLcTankCap (UINT32 *extSelect, UINT32 *capValue);
extern UINT32 getRfPowerSetting (UINT8 *paConfig1, UINT8 *paConfig2,
                  UINT8 *paConfig3, UINT8 *vtPowerDetHi,
                  UINT8 *vtPowerDetLo, UINT8 *powerTarget, UINT8 *dacX);
extern UINT32 getRfPowerCorr (PK_RfPower_t *pkrfpower);
extern UINT32 setBandAG (UINT32 band);
extern UINT32 getBandAG (UINT32 *band);
extern UINT32 setRfXtalCtrl (UINT32 extMode, UINT32 value);
extern UINT32 getRfXtalCtrl (UINT32 *extMode, UINT32 *value);
extern UINT32 getRfRcCal (UINT32 *values);
extern UINT32 setRfRcCtrl (UINT32 extMode, UINT32 value);
extern UINT32 getRfRcCtrl (UINT32 *extMode, UINT32 *value);
extern UINT32 setPaLnaMode (UINT32 extPA, UINT32 extLNA);
// #ifndef WB_EXCEPTION
extern UINT32 setControlLoopMode (UINT32 ctrlLpMode);  
extern UINT32 getControlLoopMode (UINT32 *ctrlLpMode);
extern UINT32 setThermalScaler (UINT8 thermScaler);
extern UINT32 getThermalScaler (UINT8 *thermScaler);
extern UINT32 downloadRangeTbl ( 
               pwrRangeTbleRow_t *high1_ch1_13, 
               pwrRangeTbleRow_t *high2_ch1_13,
               pwrRangeTbleRow_t *high3_ch1_13, 
               pwrRangeTbleRow_t *high4_ch1_13, 
               pwrRangeTbleRow_t *high1_ch14, 
               pwrRangeTbleRow_t *high2_ch14, 
               pwrRangeTbleRow_t *high3_ch14, 
               pwrRangeTbleRow_t *high4_ch14);
extern UINT32 uploadRangeTbl (
               pwrRangeTbleRow_t *high1_ch1_13,
               pwrRangeTbleRow_t *high2_ch1_13,
               pwrRangeTbleRow_t *high3_ch1_13,
               pwrRangeTbleRow_t *high4_ch1_13,
               pwrRangeTbleRow_t *high1_ch14,
               pwrRangeTbleRow_t *high2_ch14,
               pwrRangeTbleRow_t *high3_ch14,
               pwrRangeTbleRow_t *high4_ch14);
extern UINT32 loadSettingsFromEeprom (void);
extern UINT32 uploadPwrTbl (
               UINT32 id, UINT32 *max, UINT32 *min, 
               UINT32 *size, UINT8 *tbl); 
extern UINT32 downloadPwrTbl (
               UINT32 id, UINT32 max, UINT32 min,
               UINT32 size, UINT8 *tbl);
extern UINT32 setRfPwrRange (UINT32 power, UINT32 signalBG);
extern UINT32 getRfPwrRange (UINT32 *powerRange, UINT32 *top, UINT32 *bottom);
extern UINT32 hwRfReset (void);
extern UINT32 writeBlockFlash (UINT32 size, UINT32 address, UINT32 *tbl);
extern UINT32 readBlockFlash (UINT32 size, UINT32 address, UINT32 *tbl);
//#endif

/*=========================================================================== */
/*                          MODULE LEVEL VARIABLES */
/*=========================================================================== */
#ifdef ECOS
static cyg_thread TaskMfg;
/* task object */
static cyg_handle_t TaskMfgHdl;
/* task handle */
static UINT8 mfgStack[MFG_TASK_STACK_SIZE];
/* task stack size */

#ifdef MFG_TIMER
#ifdef AP5X
static timer_Data_t *mfgTimer;
#else
static Timer mfgTimer;
#endif
#endif
#endif

#ifndef EEPROM
extern unsigned int flashbase;
/* flash base address: 0xFFF0 0000 for 1 MByte */
/* 0xFFE0 0000 for 2 MByte */
/* flashbase address is define in oemflash.c */

extern unsigned int flashsize;
/* flash size */
extern unsigned manu;
/* flash manufacter */
extern unsigned part;
/* flash part number */
extern unsigned int flashNsect;
/* flash sector number */
#else
#define flashbase 0x7300
#endif

/*=========================================================================== */
/*                   PRIVATE PROCEDURES (ANSI Prototypes) */
/*=========================================================================== */
static void MfgTask (os_Addrword_t Data);
static int mfgRecvFrom (int, void *, int, int, struct sockaddr *, int *);
static int mfgSendTo (int, const void *, int, int, 
               const struct sockaddr *, int);
#ifdef MFG_DBG
static void mfgDbg (UINT32 flow, void *msg);
#endif
#ifdef MFG_TIMER
static void mfgTimerTick (void);
#endif

/*=========================================================================== */
/*                         CODED PUBLIC PROCEDURES */
/*=========================================================================== */

/******************************************************************************
 *
 * Name: MfgInit
 *
 * Description:
 *    Routine to create and resume MfgTask.
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    None.
 *
 * Return Value:
 *    None.
 *
 * Notes:
 *    None.
 *
 * PDL:
 *
 * END PDL
 *
 *****************************************************************************/
extern void MfgInit ()
{
#ifdef ECOS
    cyg_thread_create (MFG_TASK_PRI,
                       MfgTask,
                       0,
                       MFG_TASK_NAME,
                       (void *) (mfgStack),
                       MFG_TASK_STACK_SIZE,
                       &TaskMfgHdl,
                       &TaskMfg);

    cyg_thread_resume(TaskMfgHdl);
#else
    sp(MfgTask);    /* Use the os-independent version for VxWorks. */
#endif
}

/*=========================================================================== */
/*                         CODED PRIVATE PROCEDURES */
/*=========================================================================== */

/******************************************************************************
 *
 * Name: MfgTask
 *
 * Description:
 *    Open an incoming and outgoing socket.
 *    Wait for incoming commands. 
 *    Execute the command and return the results
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    Arg1 (i  ): task data
 *
 * Return Value:
 *    None.
 *
 * Notes:
 *    None.
 *
 * PDL:
 *
 * END PDL
 *
 *****************************************************************************/ 
char buffer[2048];
int outgoing_socket;
int incoming_socket;
struct sockaddr_in incoming;
struct sockaddr_in outgoing, outgoing1;
int need_reboot = 0;
static void MfgTask (os_Addrword_t Data)
{
    UINT32 IPAddr_pre;			
             int bytes;
             int tmp;
    volatile int outLen;
#ifdef MFG_DBG
             char ApIpAddr[20];
#endif
 
/* Following code not needed for VxWorks since the task is 
   started after all the IP stuff is initialized - different 
   than ECOS. */
    
#ifdef ECOS
#ifdef MFG_TIMER
#ifdef AP5X
    mfgTimer = timer_Get (&mfgTimerTick); 
    timer_Start (mfgTimer->Id, 200 * 10, 0);
#else
    TimerFireIn (&mfgTimer, 1, &mfgTimerTick, NULL, 20 * 10 );
#endif
#endif
#ifdef MFG_DBG
    /* initialize the serial port */
    diag_printf ("\nMfgTask\n");
#ifdef AP5X
    diag_printf ("version: %s\n", FIRMWARE_VERSION);
#else    
    diag_printf ("version: %s\n", __version__);
#endif    
    diag_printf ("cyg_semaphore_wait %d\n", ip_ready);
#endif

    /* wait until the TCP/IP stack protocol is up and running */
    wait_ipReady ();
#ifdef AP31HOST
    cyg_semaphore_init(&wait_udp_response, 0);
#endif
#ifndef WB_EXCEPTION
	mib_DHCP_p = &mib_DHCP;
#endif
	while(1)
	{
		IPAddr_pre = mib_DHCP_p->IPAddr;
#ifdef MFG_DBG
    IPAddr ((char *) ApIpAddr);
    diag_printf ("cyg_semaphore_wait %d\n", ip_ready);
    diag_printf ("AP IP addr: %s\n", ApIpAddr);
    diag_printf ("MAC addr: %02x-%02x-%02x-%02x-%02x-%02x\n",
        mib_currentmacaddress[0], mib_currentmacaddress[1],
        mib_currentmacaddress[2], mib_currentmacaddress[3],
        mib_currentmacaddress[4], mib_currentmacaddress[5]);
#endif
#endif /* endif ECOS */

    /* UDP connection AP is the server, the PC is the client */
    bzero (&incoming, sizeof (incoming));
    bzero (&outgoing, sizeof (outgoing));
    incoming_socket = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    outgoing_socket = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);

    outLen = sizeof(outgoing);

    /* incoming packet, use of recvfrom so we can get the client IP addr */
#ifdef AP31HOST    
    {
        extern UINT32 UDPClientIP;

        outgoing.sin_family = AF_INET;
        outgoing.sin_port = htons (PORT);
        outgoing.sin_addr.s_addr = UDPClientIP;
        outgoing1 = outgoing;
    }
    incoming.sin_addr.s_addr = inet_address(mib_DHCP_p->IPAddr);
    incoming.sin_port = htons (PORT+1);
#else
    incoming.sin_addr.s_addr = htonl (INADDR_ANY);
    incoming.sin_port = htons (PORT);
#endif    
    incoming.sin_family = AF_INET;
    incoming.sin_len    = sizeof(incoming);
    tmp = bind (incoming_socket, (struct sockaddr *) & incoming,
                sizeof (incoming));
#ifdef MFG_DBG
    diag_printf ("bind: %d\n", tmp);
#endif

    /* outgoing packet, use of sendto (connection-less) */
    outgoing.sin_family = AF_INET;
    while (IPAddr_pre == mib_DHCP_p->IPAddr )
    {
        bytes = mfgRecvFrom (
                    incoming_socket, 

                    (char *) &buffer,
                    sizeof (buffer),
                 
                    0,
                    (struct sockaddr *) &outgoing,
                    (int *) &outLen);
#ifdef AP5X
        outgoing.sin_port = htons (PORT + 1);
#endif        
        if (bytes > 0)
        {
            PkHeader_t *pkhdrp;
            PK_RfReg_t *pkrfp;
            PK_VmrStatus_t *pkvmrs;
            PK_Data_t *pkdata;
            PK_Data2_t *pkdata2;
            PK_TxFrame_t *pktxframe;
            PK_Std_t *pkstd;
            PK_MacAddr_t *pkmacaddr;
            PK_PiuTable_t *pkpiutable;
            PK_MfgInfo_t *pkmfginfo;
            PK_MfgVersion_t *pkmfgversion;
            PK_EnableSsid_t *pkenablessid;
            PK_RfPower_t *pkrfpower;
            PK_DutyCycle_t *pkdutycycle;
            PK_RfSetting_t *pkrfsetting;
            PK_PaLnaMode_t *pkpalnamode;
//#ifndef WB_EXCEPTION
            PK_RfLoop_t *pkrfloop;
            PK_RfThermal_t *pkrfthermal;
            PK_RfPwrRange_t *pkrfpwrrange;
            PK_RelNote_t *pkrelnote;
            PK_LoadPwrTbl_t *pkloadpwrtbl;
            PK_RfRange_t *pkrfrange;
            PK_BlockFlash_t *pkblockflash;
//#endif
            pkhdrp = (PkHeader_t *) buffer;
	    if(pkhdrp->id == MANU_ID) {
            outgoing.sin_port = htons (PORT +1);
             switch (pkhdrp->cmd)
            {
            case CMD_WR_RF:
            case CMD_WR_MAC:  /* write MAC registers */
                pkrfp = (PK_RfReg_t *) buffer;
                * (DWORD *) pkrfp->address = pkrfp->data;
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RD_RF:
            case CMD_RD_MAC:  /* read MAC registers */
                pkrfp = (PK_RfReg_t *) buffer;
                pkrfp->data = * (DWORD *) pkrfp->address;
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_WR_BB:  /* write BaseBand registers */
                pkrfp = (PK_RfReg_t *) buffer;
                pkrfp->errorcode = writeBBReg (
                    pkrfp->address, pkrfp->data);
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RD_BB: /* read BaseBand registers */
                pkrfp = (PK_RfReg_t *) buffer;
                pkrfp->errorcode = readBBReg (
                    (UINT8) pkrfp->address, (UINT8 *) &pkrfp->data);
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;
            case CMD_WR_VMR:  /* write flash */
                pkrfp = (PK_RfReg_t *) buffer;
#ifdef EEPROM   
                tmp = spi_write4( pkrfp->address & 0xff + flashbase, pkrfp->data); 
#else                
                tmp = flashWriteWord (
                    pkrfp->address, pkrfp->data & 0xffff);
                if (tmp == TRUE)
                    tmp = flashWriteWord (
                       pkrfp->address + 2, (pkrfp->data >> 16) & 0xffff);
#endif                
                pkrfp->errorcode = tmp == TRUE ? SUCCESS : FAIL;
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RD_VMR:  /* read flash */
                pkrfp = (PK_RfReg_t *) buffer;
#ifdef EEPROM   
                    {
                        DWORD result;
                        result = spi_read4(pkrfp->address & 0xff + flashbase);
                        result |= (spi_read4(pkrfp->address & 0xff + flashbase + 4)<<32);
                        pkrfp->data = result;
                    }
#else
                pkrfp->data = * (DWORD *) (pkrfp->address | flashbase);
#endif                
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_ER_VMR:  /* eraze flash */
                pkrfp = (PK_RfReg_t *) buffer;
                tmp = EraseMfgData (pkrfp->address, 1);
                pkrfp->errorcode = tmp == TRUE ? SUCCESS : FAIL;
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

             case CMD_ST_VMR:  /* flash status */
                pkvmrs = (PK_VmrStatus_t *) buffer;
#ifdef EEPROM
                /* fake eeprom info*/
                pkvmrs->manufacturer = 0x1000;
                pkvmrs->part = 0x2000;
                pkvmrs->sectors = 16;
                pkvmrs->numMB =  8; /* In MB */
#else
                pkvmrs->manufacturer = manu;
                pkvmrs->part = part;
                pkvmrs->sectors = flashNsect;
                pkvmrs->numMB = flashsize >> 20; /* In MB */
#endif // !EEPROM
                pkvmrs->header.cmd = pkvmrs->header.cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkvmrs, sizeof (PK_VmrStatus_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;
            case CMD_ST_DISS:  /* Disassociate (not implemented) */
                pkrfp = (PK_RfReg_t *) buffer;
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                pkrfp->errorcode = FAIL;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_BCN_ENB:     /* set beacon (0 switch off) */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setBeaconEnable (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_BCN_ENB:     /* get beacon (0 switch off) */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getBeaconEnable (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_TX_FRAME:     /* send Tx frame */
                pktxframe = (PK_TxFrame_t *) buffer;
                pktxframe->errorcode = txMulticastFrame (pktxframe);
                pktxframe->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pktxframe, sizeof (PK_TxFrame_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_CHA_BCN:     /* set beacon channel */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setBeaconChannel (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RF_ENA:     /* RF Tx enable */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = rfTxEnable (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_WR_RFCHA:     /* set RF channel */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setRfChannel (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RD_RFCHA:     /* get RF channel */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getRfChannel (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_WR_RFPOW:     /* set RF power */
                pkrfpower = (PK_RfPower_t *) buffer;
                pkrfpower->errorcode = setRfPower (pkrfpower);
                pkrfpower->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfpower, sizeof (PK_RfPower_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RD_RFPOW:     /* get RF power */
                pkrfpower = (PK_RfPower_t *) buffer;
                pkrfpower->errorcode = getRfPower (pkrfpower);
                pkrfpower->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfpower, sizeof (PK_RfPower_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RF_STEP:     /* step RF power */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = stepRfPower (pkdata2->data1,
                    pkdata2->data2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_TX_ANT:     /* set Tx Antenna */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setTxAntenna (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_TX_ANT:     /* get Tx Antenna */
               pkdata = (PK_Data_t *) buffer;
               pkdata->errorcode = getTxAntenna (pkdata);
               pkdata->header.cmd = pkhdrp->cmd + 1;
               mfgSendTo (outgoing_socket,
                  (char *) pkdata, sizeof (PK_Data_t),
                  0, (const struct sockaddr *) &outgoing,
                  sizeof (outgoing));
               break;

            case CMD_RX_ANT:     /* set Rx Antenna */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setRxAntenna (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_RX_ANT:     /* get Rx Antenna */
               pkdata = (PK_Data_t *) buffer;
               pkdata->errorcode = getRxAntenna (pkdata);
               pkdata->header.cmd = pkhdrp->cmd + 1;
               mfgSendTo (outgoing_socket,
                  (char *) pkdata, sizeof (PK_Data_t),
                  0, (const struct sockaddr *) &outgoing,
                  sizeof (outgoing));
               break;

            case CMD_TX_RATE:     /* set Tx data rate */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setTxDataRate (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_TX_RATE:     /* get Tx data rate */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getTxDataRate (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_22M:     /* set 22 Mbps */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = set22M (pkdata->data);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_PRMBL:     /* set short/long preamble */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setShortPreamble (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_PWMODE:    /* set power mode */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setPowerMode (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RSSI_PWOPT:    /* set Rssi power option */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = setRssiPowerOption (pkdata2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_TXCONT:    /* set Tx continuous mode */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = setTxContinuousMode (pkdata2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_CLR_RSSI_MP:    /* clear rx multicast packet */
                pkstd = (PK_Std_t *) buffer;
                pkstd->errorcode = clearRxMultiCastPckt ();
                pkstd->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkstd, sizeof (PK_Std_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_RSSI_MP:    /* get rx multicast packet */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getRxMultiCastPckt (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_CLR_RSSI_UP:    /* clear rx unicast packet */
                pkstd = (PK_Std_t *) buffer;
                pkstd->errorcode = clearRxUniCastPckt ();
                pkstd->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkstd, sizeof (PK_Std_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_RSSI_UP:    /* get rx unicast packet */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getRxUniCastPckt (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_RSSI:       /* get rssi power */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getRssiPower (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_BCN_INT:     /* set beacon interval */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setBeaconInterval (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_BCN_INT:     /* get beacon interval */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getBeaconInterval (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_HW:          /* get hardware version */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = getHardwareVersion (pkdata2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_MAC_ADD:     /* set MAC address */
                pkmacaddr = (PK_MacAddr_t *) buffer;
                pkmacaddr->errorcode = setMacAddress (pkmacaddr);
                pkmacaddr->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkmacaddr, sizeof (PK_MacAddr_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_MAC_ADD:     /* get MAC address */
                pkmacaddr = (PK_MacAddr_t *) buffer;
                pkmacaddr->errorcode = getMacAddress (pkmacaddr);
                pkmacaddr->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkmacaddr, sizeof (PK_MacAddr_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_CCODE:       /* set country code */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = setCountryCode (pkdata2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_CCODE:       /* get country code */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = getCountryCode (pkdata2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_PIU:         /* set PIU table (70 bytes) */
                pkpiutable = (PK_PiuTable_t *) buffer;
                pkpiutable->errorcode = setPiuTable (pkpiutable);
                pkpiutable->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkpiutable, sizeof (PK_PiuTable_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_PIU:         /* get PIU table (70 bytes) */
                pkpiutable = (PK_PiuTable_t *) buffer;
                pkpiutable->errorcode = getPiuTable (pkpiutable);
                pkpiutable->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkpiutable, sizeof (PK_PiuTable_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_MFG:         /* set mfg hard info (64 bytes) */
                pkmfginfo = (PK_MfgInfo_t *) buffer;
                pkmfginfo->errorcode = setMfgInfo (pkmfginfo);
                pkmfginfo->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkmfginfo, sizeof (PK_MfgInfo_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_MFG:         /* get mfg hard info (64 bytes) */
                pkmfginfo = (PK_MfgInfo_t *) buffer;
                pkmfginfo->errorcode = getMfgInfo (pkmfginfo);
                pkmfginfo->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkmfginfo, sizeof (PK_MfgInfo_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_VERS:        /* get mfg firmware version */
                pkmfgversion = (PK_MfgVersion_t *) buffer;
                pkmfgversion->errorcode = SUCCESS;
                pkmfgversion->errorcode = getMfgVersion (pkmfgversion);
                pkmfgversion->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket, 
                   (char *) pkmfgversion, sizeof (PK_MfgVersion_t), 
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_ENB_SSID:        /* enable BSSID filtering */
                pkenablessid = (PK_EnableSsid_t *) buffer;
                pkenablessid->errorcode = enableSSID (pkenablessid->ssid,
                                                      pkenablessid->bssid);
                pkenablessid->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkenablessid, sizeof (PK_EnableSsid_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_FW_MODE:     /* set the FW mode (mfg of customer) */
#ifdef MFG_TIMER
                /* disable MFG_TIMER */
#ifdef AP5X
               timer_Stop (mfgTimer->Id);
#else
                TimerDisarm (&mfgTimer);  
#endif                
#endif

                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setFwMode (pkdata->data);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_WR_RF_REG:   /* write RF registers */
                pkrfp = (PK_RfReg_t *) buffer;
                pkrfp->errorcode = writeRfReg (
                    pkrfp->address, pkrfp->data);
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RD_RF_REG:   /* read RF registers */
                pkrfp = (PK_RfReg_t *) buffer;
                pkrfp->errorcode = readRfReg (
                    (UINT8) pkrfp->address, (UINT8 *) &pkrfp->data);
                pkrfp->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfp, sizeof (PK_RfReg_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_72M:     /* set 72 Mbps */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = set72M (pkdata->data);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_LPF:     /* set Low Pass Filter */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setRfBbLpf (pkdata->data);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_WAKE_UP_RF:    /* wake up RF and BB */
                pkstd = (PK_Std_t *) buffer;
                pkstd->errorcode = wakeUpRfBb ();
                pkstd->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkstd, sizeof (PK_Std_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;
      
            case CMD_XOSC_CAL:     /* xosc calibration */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = xoscCal ((UINT32 *) &pkdata2->data1,
                    pkdata2->data2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_RX_ERR:     /* get Rx error counter */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getRxErrPckt (pkdata);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_CLR_RX_ERR:    /* clear Rx error counter */
                pkstd = (PK_Std_t *) buffer;
                pkstd->errorcode = clearRxErrPckt ();
                pkstd->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkstd, sizeof (PK_Std_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_EN_CST:     /* enable carrier suppression test */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setTxCarrierSuppression (pkdata->data);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_DUTY_CYCLE:  /* Tx duty cycle */
                pkdutycycle = (PK_DutyCycle_t *) buffer;
                pkdutycycle->errorcode = setTxDutyCycle (
                    pkdutycycle->enable,
                    pkdutycycle->dataRate,
                    pkdutycycle->percent);
                pkdutycycle->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdutycycle, sizeof (PK_DutyCycle_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_VGA:       /* set the VGA gain */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = setTxVgaGain (
                    pkdata2->data1,
                    pkdata2->data2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_VGA:       /* get the VGA gain */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = getTxVgaGain (
                    (UINT32 *) &pkdata2->data1,
                    (UINT32 *) &pkdata2->data2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_TCAP:       /* set LC tank cap */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = setLcTankCap (
                    pkdata2->data1,
                    pkdata2->data2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_TCAP:       /* get LC tank cap */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = getLcTankCap (
                    (UINT32 *) &pkdata2->data1,
                    (UINT32 *) &pkdata2->data2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RF_CORR:       /* get Rf power correction */
                pkrfpower = (PK_RfPower_t *) buffer;
                pkrfpower->errorcode = getRfPowerCorr (pkrfpower);
                pkrfpower->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfpower, sizeof (PK_RfPower_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_RF_SETTING:       /* get Rf power setting */
                pkrfsetting = (PK_RfSetting_t *) buffer;
                pkrfsetting->errorcode = getRfPowerSetting (
                                          (UINT8 *) &pkrfsetting->paConfig1,
                                          (UINT8 *) &pkrfsetting->paConfig2,
                                          (UINT8 *) &pkrfsetting->paConfig3,
                                          (UINT8 *) &pkrfsetting->vtPowerDetHi,
                                          (UINT8 *) &pkrfsetting->vtPowerDetLo,
                                          (UINT8 *) &pkrfsetting->powerTarget,
                                          (UINT8 *) &pkrfsetting->dacX);
                pkrfsetting->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfsetting, sizeof (PK_RfSetting_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_BAND_AG:      /* set band A or G */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = setBandAG (pkdata->data);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_BAND_AG:      /* get band A or G */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getBandAG ((UINT32 *) &pkdata->data);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_RFXTAL_CTRL:      /* set RF crystal control */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = setRfXtalCtrl (
                                       pkdata2->data1,
                                       pkdata2->data2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

           case CMD_GET_RFXTAL_CTRL:      /* get RF crystal control */
               pkdata2 = (PK_Data2_t *) buffer;
               pkdata2->errorcode = getRfXtalCtrl (
                                      (UINT32 *) &pkdata2->data1,
                                      (UINT32 *) &pkdata2->data2);
               pkdata2->header.cmd = pkhdrp->cmd + 1;
               mfgSendTo (outgoing_socket,
                  (char *) pkdata2, sizeof (PK_Data2_t),
                  0, (const struct sockaddr *) &outgoing,
                  sizeof (outgoing));
               break;

            case CMD_GET_RF_RC_CAL:      /* get RF RC cal */
                pkdata = (PK_Data_t *) buffer;
                pkdata->errorcode = getRfRcCal ((UINT32 *) &pkdata->data);
                pkdata->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata, sizeof (PK_Data_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_SET_RF_RCCTRL:      /* set RF RC ctrl */
                pkdata2 = (PK_Data2_t *) buffer;
                pkdata2->errorcode = setRfRcCtrl (
                                       pkdata2->data1,
                                       pkdata2->data2);
                pkdata2->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkdata2, sizeof (PK_Data2_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

           case CMD_GET_RF_RCCTRL:      /* get RF RC ctrl */
               pkdata2 = (PK_Data2_t *) buffer;
               pkdata2->errorcode = getRfRcCtrl (
                                      (UINT32 *) &pkdata2->data1,
                                      (UINT32 *) &pkdata2->data2);
               pkdata2->header.cmd = pkhdrp->cmd + 1;
               mfgSendTo (outgoing_socket,
                  (char *) pkdata2, sizeof (PK_Data2_t),
                  0, (const struct sockaddr *) &outgoing,
                  sizeof (outgoing));
               break;

            case CMD_PA_LNA_MODE:      /* PA, LNA mode */
                pkpalnamode = (PK_PaLnaMode_t *) buffer;
                pkpalnamode->errorcode = setPaLnaMode (
                                            pkpalnamode->extPA,
                                            pkpalnamode->extLNA);
                pkpalnamode->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkpalnamode, sizeof (PK_PaLnaMode_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

// #ifndef WB_EXCEPTION
            case CMD_GET_REL_NOTE:  /* get release note */
                pkrelnote = (PK_RelNote_t *) buffer;
                strcpy ((char *) pkrelnote->relNote, RELEASENOTE);

                pkrelnote->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrelnote, sizeof (PK_RelNote_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_SET_RF_LOOP:  /* set RF close loop mode */
                pkrfloop = (PK_RfLoop_t *) buffer;
                pkrfloop->errorcode = setControlLoopMode (
                     pkrfloop->thermalOnlyControl);
                pkrfloop->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfloop, sizeof (PK_RfLoop_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_GET_RF_LOOP:  /* get RF close loop mode */
                pkrfloop = (PK_RfLoop_t *) buffer;
                pkrfloop->errorcode = getControlLoopMode (
                     (UINT32 *) &pkrfloop->thermalOnlyControl);
                pkrfloop->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfloop, sizeof (PK_RfLoop_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_SET_THERMAL:  /* set RF thermal scaler */
                pkrfthermal = (PK_RfThermal_t *) buffer;
                pkrfthermal->errorcode = setThermalScaler (
                     pkrfthermal->thermalScaler);
                pkrfthermal->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfthermal, sizeof (PK_RfThermal_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_GET_THERMAL:  /* get RF thermal scaler */
                pkrfthermal = (PK_RfThermal_t *) buffer;
                pkrfthermal->errorcode = getThermalScaler (
                     (UINT8 *) &pkrfthermal->thermalScaler);
                pkrfthermal->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfthermal, sizeof (PK_RfThermal_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_SET_RANGE:  /* set RF power range table */
                pkrfpwrrange = (PK_RfPwrRange_t *) buffer;
                pkrfpwrrange->errorcode = downloadRangeTbl (
                     &pkrfpwrrange->rangeTable_1_13_High1,
                     &pkrfpwrrange->rangeTable_1_13_High2,
                     &pkrfpwrrange->rangeTable_1_13_High3,
                     &pkrfpwrrange->rangeTable_1_13_High4,
                     &pkrfpwrrange->rangeTable_14_High1,
                     &pkrfpwrrange->rangeTable_14_High2,
                     &pkrfpwrrange->rangeTable_14_High3,
                     &pkrfpwrrange->rangeTable_14_High4);
                pkrfpwrrange->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfpwrrange, sizeof (PK_RfPwrRange_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_GET_RANGE:  /* get RF power range table */
                pkrfpwrrange = (PK_RfPwrRange_t *) buffer;
                pkrfpwrrange->errorcode = uploadRangeTbl (
                     &pkrfpwrrange->rangeTable_1_13_High1,
                     &pkrfpwrrange->rangeTable_1_13_High2,
                     &pkrfpwrrange->rangeTable_1_13_High3,
                     &pkrfpwrrange->rangeTable_1_13_High4,
                     &pkrfpwrrange->rangeTable_14_High1,
                     &pkrfpwrrange->rangeTable_14_High2,
                     &pkrfpwrrange->rangeTable_14_High3,
                     &pkrfpwrrange->rangeTable_14_High4);
                pkrfpwrrange->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfpwrrange, sizeof (PK_RfPwrRange_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_SETTING_EEPROM:  /* load settings from EEPROM */
                pkstd = (PK_Std_t *) buffer;
                pkstd->errorcode = loadSettingsFromEeprom ();
                pkstd->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkstd, sizeof (PK_Std_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;

            case CMD_GET_PWRTBL:    /* upload power table */
                pkloadpwrtbl = (PK_LoadPwrTbl_t *) buffer;
                pkloadpwrtbl->errorcode = uploadPwrTbl (
                    pkloadpwrtbl->tableId,
                    (UINT32 *) &pkloadpwrtbl->maxPower,
                    (UINT32 *) &pkloadpwrtbl->minPower,
                    (UINT32 *) &pkloadpwrtbl->sizeOfTblInByte,
                    (UINT8 *) &pkloadpwrtbl->tbl);
                pkloadpwrtbl->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkloadpwrtbl, sizeof (PK_LoadPwrTbl_t) + 
                   pkloadpwrtbl->sizeOfTblInByte,
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_SET_PWRTBL:    /* download power table */
                pkloadpwrtbl = (PK_LoadPwrTbl_t *) buffer;
                pkloadpwrtbl->errorcode = downloadPwrTbl (
                    pkloadpwrtbl->tableId,
                    pkloadpwrtbl->maxPower,
                    pkloadpwrtbl->minPower,
                    pkloadpwrtbl->sizeOfTblInByte,
                    (UINT8 *) &pkloadpwrtbl->tbl);
                pkloadpwrtbl->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkloadpwrtbl, sizeof (PK_LoadPwrTbl_t) +
                   pkloadpwrtbl->sizeOfTblInByte,
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_SET_PWRVAL:  /* set RF power value */
                pkrfrange = (PK_RfRange_t *) buffer;
                pkrfrange->errorcode = setRfPwrRange (
                     pkrfrange->rangeIndexPwrPnt,
                     pkrfrange->top);
                pkrfrange->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfrange, sizeof (PK_RfRange_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_GET_PWRVAL:  /* get RF power value */
                pkrfrange = (PK_RfRange_t *) buffer;
                pkrfrange->errorcode = getRfPwrRange (
                     (UINT32 *) &pkrfrange->rangeIndexPwrPnt,
                     (UINT32 *) &pkrfrange->top,
                     (UINT32 *) &pkrfrange->btm);
                pkrfrange->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkrfrange, sizeof (PK_RfRange_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_HW_RF_RESET:  /* reset RF chipset */
                pkstd = (PK_Std_t *) buffer;
                pkstd->errorcode = hwRfReset ();
                pkstd->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkstd, sizeof (PK_Std_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_WRITE_FLASH:  /* write block of data */
                pkblockflash = (PK_BlockFlash_t *) buffer;
                pkblockflash->errorcode = writeBlockFlash (
                     pkblockflash->size,
                     pkblockflash->address,
                     pkblockflash->tbl);
                pkblockflash->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkblockflash, sizeof (PK_BlockFlash_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;

            case CMD_READ_FLASH:  /* read block of data */
                pkblockflash = (PK_BlockFlash_t *) buffer;
                pkblockflash->errorcode = readBlockFlash (
                     pkblockflash->size,
                     pkblockflash->address,
                     pkblockflash->tbl);
                pkblockflash->header.cmd = pkhdrp->cmd + 1;
                mfgSendTo (outgoing_socket,
                   (char *) pkblockflash, sizeof (PK_BlockFlash_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
               break;
//#endif
            case CMD_ST_TERM:
                pkstd = (PK_Std_t *) buffer;
                pkstd->header.cmd = pkhdrp->cmd + 1;
                pkstd->errorcode = enableDHCP (TRUE);
                mfgSendTo (outgoing_socket,
                   (char *) pkstd, sizeof (PK_Std_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                /* End of calib. Leave back to bootrom */
                u.hackfest = 0xffff0000;
                u.branch ();
                break;

            default: /* command not yet implemented return FAIL */
                pkstd = (PK_Std_t *) buffer;
                pkstd->header.cmd = pkhdrp->cmd + 1;
                pkstd->errorcode = FAIL;
                mfgSendTo (outgoing_socket,
                   (char *) pkstd, sizeof (PK_Std_t),
                   0, (const struct sockaddr *) &outgoing,
                   sizeof (outgoing));
                break;
            }
	    } else if(pkhdrp->id == API_ID) {
#if defined(AP31HOST) || defined(AP31E) ||defined(UDP_API)
		UINT32 len;
		APICMDBUF *apicmdp = (APICMDBUF *)(buffer + sizeof(PkHeader_t));
#endif
#ifdef AP31HOST
        if(apicmdp->cmdCode == lastcmdCode +1){
            udp_apicmdbuf_p = msi_wl_AllocPktBuf(NULL);
            memcpy(udp_apicmdbuf_p, apicmdp, apicmdp->cmdBodyLen +sizeof(APICMDBUF));
            cyg_semaphore_post(&wait_udp_response);
        }
#endif  
      
#if defined(AP31E) || defined(UDP_API)
		pkhdrp->cmd = 1; // Always cmd == 1
		// pass the API request over as is from the body of the buffer
		// vvvvvvvvvvv bad code vvvvvvvvvvvvvvv
		// inspect the APICMD for CMD's that have an embedded pointer
		// revector those pointers to point to the element just after
		outgoing.sin_port = htons (PORT + 1);
		switch(apicmdp->cmdCode) {
		case APCMD_GET_SSID:
		case APCMD_SET_SSID:
		case APCMD_GET_FIRMWARE_VERSION:
		case APCMD_GET_WLAN_FILTER_LIST:
		case APCMD_SET_WLAN_FILTER_LIST:
		case APCMD_GET_STA_LIST:
		case APCMD_SET_BEACON_TX_MODE:
        case APCMD_SET_RATE_ADAPTATION_METHOD:
#ifdef  WDS_FEATURE
		case APCMD_GET_WDS_MODE:
		case APCMD_SET_WDS_MODE:
		case APCMD_GET_WDS_LIST:
        case APCMD_SET_WDS_LIST:
#endif		    
            apicmdp->cmdBody = ((unsigned char *)&apicmdp->cmdBody) 
			+ sizeof(apicmdp->cmdBody);
		    break;
        case APCMD_UPDATE_FIRMWARE:
            apicmdp->cmdStatus = compose_update_packet(apicmdp);
		    pkhdrp->seq = 0;
            pkhdrp->cmd++; // reply is always == 2
            // Replies are always even. Make sure the reply
            // is well formed. Some API commands are not well formed
            // and will reply with the wrong type of message
            if(apicmdp->cmdCode & 1) {
                apicmdp->cmdCode++;
            }
            goto send_reply;
            break;
#ifdef AP31E
        case APCMD_SET_PRIVACY_TBL_RSN_ENABLED:
        case APCMD_SET_PRIVACY_OPTION:
            need_reboot++;
            break;
        case APCMD_BSS_START:
            if(need_reboot)
            {
                need_reboot = 0;
                SetUserConfig();
                pkhdrp->seq = 0;
                pkhdrp->cmd = 1; // reply is always == 2
                // Replies are always even. Make sure the reply
                // is well formed. Some API commands are not well formed
                // and will reply with the wrong type of message
                apicmdp->cmdCode = APCMD_FW_SOFT_RESET;
                apicmdp->cmdStatus = 1;
                goto send_reply;
            }
#endif
		}
		if(apctl(apicmdp) == FAIL) {
		    pkhdrp->seq = 1; // We flag API fails here.
		} else {
		    pkhdrp->seq = 0;
		}
		pkhdrp->cmd++; // reply is always == 2
		// Replies are always even. Make sure the reply
		// is well formed. Some API commands are not well formed
		// and will reply with the wrong type of message
		if(apicmdp->cmdCode & 1) {
		    apicmdp->cmdCode++;
		}
send_reply:
		// Formulate the reply. Get the length first
		len = apicmdp->cmdBodyLen + sizeof(PkHeader_t)+sizeof(APICMDBUF);
		if(len > (MTU - (ETHII_SIZE + IP_SIZE + UDP_SIZE))) {
		    pkhdrp->seq = 2; // We flag overrage here.
		    // Fix the len
		    len = (MTU - (ETHII_SIZE + IP_SIZE + UDP_SIZE));
		}
		{
			extern BOOLEAN WlanEnabled;

			while(WlanEnabled == FALSE)
			{
               os_TaskDelay(10);			
			}
		    mfgSendTo (outgoing_socket,
			   (char *) buffer, len,
			   0, (const struct sockaddr *) &outgoing,
			   sizeof (outgoing));
		}
#endif			   
	    }
        }
    }
	//close sockets
	close(outgoing_socket);
	close(incoming_socket);	
	}
}

/******************************************************************************
 *
 * Name: mfgRecvFrom
 *
 * Description:
 *    Call recvfrom function and print debug information.
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    See function prototype recvfrom
 *
 * Return Value:
 *    See function prototype recvfrom
 *
 * Notes:
 *    None.
 *
 * PDL:
 *
 * END PDL
 *
 *****************************************************************************/
static int mfgRecvFrom (int s, void *msg, int len, int flags, 
               struct sockaddr *from, int *fromlen)
{
    int bytes;

    /* wait for UDP packets, AP is the server */
    bytes = recvfrom (
                s,
                (char *) msg,
                len,
                flags,
                (struct sockaddr *) from,
                fromlen);

#ifdef MFG_DBG
    /* print the debug info */
    if (bytes == -1)
    {
        diag_printf ("mfgRecvFrom\n");
        diag_printf ("recvfrom: bytes = %d, errno = %x\n", bytes, errno);
    }
    else
        mfgDbg (RECV_FROM, (char *) msg);
#endif

    return (bytes);
}

/******************************************************************************
 *
 * Name: mfgSendTo
 *
 * Description:
 *    Call sento function and print debug information.
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    See function prototype sendto
 *
 * Return Value:
 *    See function prototype sendto
 *
 * Notes:
 *    None.
 *
 * PDL:
 *
 * END PDL
 *
 *****************************************************************************/
static int mfgSendTo (int s, const void *msg, int len, int flags,
               const struct sockaddr *to, int tolen)
{
    int bytes;

    /* send UDP packet */
    bytes = sendto (
                s,
                (char *) msg,
                len,
                flags,
                (const struct sockaddr *) to,
                tolen);

#ifdef MFG_DBG
    /* print the debug info */
    if (bytes == -1)
    { 
        diag_printf ("mfgSendTo\n");
        diag_printf ("sendto: bytes = %d, errno = %d\n", bytes, errno);
    }
    else
        mfgDbg (SEND_TO, (char *) msg);
#endif

    return (bytes);
}

/******************************************************************************
 *
 * Name: mfgDbg
 *
 * Description:
 *    Print debug information.
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    Arg1 (i  ): either SEND_TO or RECV_FROM
 *    Arg2 (i  ): message to print
 *
 * Return Value:
 *    None. 
 *
 * Notes:
 *    None.
 *
 * PDL:
 *
 * END PDL
 *
 *****************************************************************************/
#ifdef MFG_DBG
static void mfgDbg (UINT32 flow, void *msg)
{
    int i;
    int cmd;
    char *arrow[] = {"->", "<-"};
    PkHeader_t *pkhdrp;
    PK_RfReg_t *pkrfp;
    PK_VmrStatus_t *pkvmrs;
    PK_Data_t  *pkdata;
    PK_Data2_t  *pkdata2;
    PK_TxFrame_t *pktxframe;
    PK_MacAddr_t *pkmacaddr;
    PK_PiuTable_t *pkpiutable;
    PK_MfgInfo_t *pkmfginfo;
    PK_MfgVersion_t *pkmfgversion;
    PK_EnableSsid_t *pkenablessid;
    PK_Std_t *pkstd;
    PK_RfPower_t *pkrfpower;
    PK_DutyCycle_t *pkdutycycle;
    char *cmdMsg[] = {
        "", "", "CMD_WR_RF", "RSP_WR_RF", "CMD_RD_RF", "RSP_RD_RF",
        "CMD_WR_BB", "RSP_WR_BB", "CMD_RD_BB", "RSP_RD_BB", "CMD_WR_MAC",
        "RSP_WR_MAC", "CMD_RD_MAC", "RSP_RD_MAC", "CMD_WR_VMR", "RSP_WR_VMR",
        "CMD_RD_VMR", "RSP_RD_VMR", "CMD_ER_VMR", "RSP_ER_VMR", "CMD_ST_VMR",
        "RSP_ST_VMR", "CMD_ST_DISS", "RSP_ST_DISS", "CMD_SET_BCN_ENB", 
        "RSP_SET_BCN_ENB", "CMD_CHA_BCN", "RSP_CHA_BCN", "CMD_TX_FRAME", 
        "RSP_TX_FRAME", "CMD_RF_ENA", "RSP_RF_ENA", "CMD_WR_RFCHA", 
        "RSP_WR_RFCHA", "CMD_RD_RFCHA", "RSP_RD_RFCHA", "CMD_WR_RFPOW",
        "RSP_WR_RFPOW", "CMD_RD_RFPOW", "RSP_RD_RFPOW", "CMD_RF_STEP", 
        "RSP_RF_STEP", "CMD_TX_ANT", "RSP_TX_ANT", "CMD_RX_ANT", "RSP_RX_ANT",
        "CMD_TX_RATE", "RSP_TX_RATE", "CMD_SET_22M", "RSP_SET_22M", 
        "CMD_SET_PRMBL", "RSP_SET_PRMBL", "CMD_SET_PWMODE", "RSP_SET_PWMODE",
        "CMD_RSSI_PWOPT", "RSP_RSSI_PWOPT", "CMD_SET_TXCONT", 
        "RSP_SET_TXCONT", "CMD_CLR_RSSI_MP", "RSP_CLR_RSSI_MP",
        "CMD_GET_RSSI_MP", "RSP_GET_RSSI_MP", "CMD_CLR_RSSI_UP", 
        "RSP_CLR_RSSI_UP", "CMD_GET_RSSI_UP", "RSP_GET_RSSI_UP",
        "CMD_GET_RSSI", "RSP_GET_RSSI", "CMD_SET_BCN_INT", "RSP_SET_BCN_INT",
        "CMD_GET_BCN_INT", "RSP_GET_BCN_INT", "CMD_GET_HW", "RSP_GET_HW", 
        "CMD_GET_MAC_ADD", "RSP_GET_MAC_ADD", "CMD_SET_MAC_ADD", 
        "RSP_SET_MAC_ADD", "CMD_GET_CCODE", "RSP_GET_CCODE", "CMD_SET_CCODE",
        "RSP_SET_CCODE", "CMD_GET_PIU", "RSP_GET_PIU", "CMD_SET_PIU", 
        "RSP_SET_PIU", "CMD_GET_MFG", "RSP_GET_MFG", "CMD_SET_MFG", 
        "RSP_SET_MFG", "CMD_GET_VERS", "RSP_GET_VERS", "CMD_ENB_SSID",
        "RSP_ENB_SSID", "CMD_SET_FW_MODE", "RSP_SET_FW_MODE", "CMD_WR_RF_REG",
        "RSP_WR_RF_REG", "CMD_RD_RF_REG", "RSP_RD_RF_REG", 
        "CMD_SET_72M", "RSP_SET_72M", "CMD_SET_LPF", "RSP_SET_LPF",
        "CMD_WAKE_UP_RF", "RSP_WAKE_UP_RF", "CMD_XOSC_CAL", "RSP_XOSC_CAL",
        "CMD_GET_BCN_ENB", "RSP_GET_BCN_ENB", "CMD_CLR_RX_ERR",
        "RSP_CLR_RX_ERR", "CMD_GET_RX_ERR", "RSP_GET_RX_ERR",
        "CMD_GET_TX_ANT", "RSP_GET_TX_ANT", "CMD_GET_RX_ANT",
        "RSP_GET_RX_ANT", "CMD_GET_TX_RATE", "RSP_GET_TX_RATE",
        "CMD_EN_CST", "RSP_EN_CST",
        "CMD_DUTY_CYCLE", "RSP_DUTY_CYCLE", "CMD_SET_VGA", "RSP_SET_VGA",
        "CMD_GET_VGA", "RSP_GET_VGA", "CMD_SET_TCAP", "RSP_SET_TCAP",
        "CMD_GET_TCAP", "RSP_GET_TCAP",
        "CMD_ST_TERM", "RSP_ST_TERM"};

    /* print the header information */
    pkhdrp = (PkHeader_t *) msg;
    cmd = (pkhdrp->cmd < CMD_ST_TERM) ? pkhdrp->cmd : 
        RSP_GET_TCAP + (pkhdrp->cmd - CMD_ST_TERM) + 1;
    diag_printf ("%s %s\n", arrow[flow], cmdMsg[cmd]);

    /* CMD = RSP - 1, print error code */
    if (flow == SEND_TO)
    {
        pkhdrp->cmd--;
        pkstd = (PK_Std_t *) msg;
        diag_printf ("    errorcode     %x\n", pkstd->errorcode);
    }

    /* print the data */
    switch (pkhdrp->cmd)
    {
    case CMD_RD_RF:
    case CMD_WR_RF:
    case CMD_WR_VMR:
    case CMD_RD_VMR:
    case CMD_ER_VMR:
    case CMD_RD_BB:
    case CMD_WR_BB:
    case CMD_RD_RF_REG:
    case CMD_WR_RF_REG:
    case CMD_ST_DISS:
        pkrfp = (PK_RfReg_t *) msg;
        diag_printf ("    address       %x\n", pkrfp->address);
        diag_printf ("    data          %x\n", pkrfp->data);
        break;

    case CMD_ST_VMR:
        pkvmrs = (PK_VmrStatus_t *) msg;
        diag_printf ("    manufacturer  %x\n", pkvmrs->manufacturer);
        diag_printf ("    part          %x\n", pkvmrs->part);
        diag_printf ("    sectors       %x\n", pkvmrs->sectors);
        diag_printf ("    numMB         %x\n", pkvmrs->numMB);
        break;

    case CMD_SET_BCN_ENB:
    case CMD_GET_BCN_ENB:
    case CMD_CHA_BCN:
    case CMD_RF_ENA:
    case CMD_WR_RFCHA:
    case CMD_RD_RFCHA:
    case CMD_TX_ANT:
    case CMD_RX_ANT:
    case CMD_GET_TX_ANT:
    case CMD_GET_RX_ANT:
    case CMD_TX_RATE:
    case CMD_GET_TX_RATE:
    case CMD_SET_22M:
    case CMD_SET_PRMBL:
    case CMD_SET_PWMODE:
    case CMD_GET_RSSI_MP:
    case CMD_GET_RSSI_UP:
    case CMD_GET_RSSI:
    case CMD_SET_BCN_INT:
    case CMD_GET_BCN_INT:
    case CMD_SET_FW_MODE:
    case CMD_SET_72M:
    case CMD_SET_LPF:
    case CMD_GET_RX_ERR:
    case CMD_EN_CST:
        pkdata = (PK_Data_t *) msg;
        diag_printf ("    data          %x\n", pkdata->data);
        break;

    case CMD_SET_CCODE:
    case CMD_GET_CCODE:
    case CMD_RSSI_PWOPT:
    case CMD_SET_TXCONT:
    case CMD_GET_HW:
    case CMD_RF_STEP:
    case CMD_XOSC_CAL:
    case CMD_SET_VGA:
    case CMD_GET_VGA:
    case CMD_SET_TCAP:
    case CMD_GET_TCAP:
        pkdata2 = (PK_Data2_t *) msg;
        diag_printf ("    data1         %x\n", pkdata2->data1);
        diag_printf ("    data2         %x\n", pkdata2->data2);
        break;

    case CMD_CLR_RSSI_MP:
    case CMD_CLR_RSSI_UP:
    case CMD_WAKE_UP_RF:
    case CMD_CLR_RX_ERR:
    case CMD_ST_TERM:
        /* nothing to display */
        break;

    case CMD_TX_FRAME:
        pktxframe = (PK_TxFrame_t *) msg;
        diag_printf ("    dataRate      %x\n", pktxframe->dataRate);
        diag_printf ("    framePattern  %x\n", pktxframe->framePattern);
        diag_printf ("    frameLength   %x\n", pktxframe->frameLength);
        diag_printf ("    frameCount    %x\n", pktxframe->frameCount);
        diag_printf ("    shortPreamble %x\n", pktxframe->shortPreamble);
        diag_printf ("    bssid         ");
        for (i = 0 ; i < 5 ; i++)
            diag_printf ("%x.", pktxframe->bssid[i]);
        diag_printf ("%x\n", pktxframe->bssid[i]);
        break;
  
    case CMD_SET_MAC_ADD:
    case CMD_GET_MAC_ADD:
        pkmacaddr = (PK_MacAddr_t *) msg;
        diag_printf ("    macAddr       ");
        for (i = 0 ; i < 5 ; i++)
            diag_printf ("%x.", pkmacaddr->macAddr[i]);
        diag_printf ("%x\n", pkmacaddr->macAddr[i]);
        diag_printf ("    flashOffset   %x\n", pkmacaddr->flashOffsetMacAddr);
        break;

    case CMD_SET_PIU: 
    case CMD_GET_PIU:
        pkpiutable = (PK_PiuTable_t *) msg;
        diag_printf ("    flashOffset   %x\n", pkpiutable->flashOffsetPiuTable);
        diag_printf ("    piuTable      ");
        for (i = 0 ; i < PIU_LEN ; i++)
        {
            diag_printf ("%x ", pkpiutable->piuTable[i]);
            if ((i % 10) == 0)
                diag_printf ("\n                ");
        }
        diag_printf ("\n");
        break;

    case CMD_SET_MFG:
    case CMD_GET_MFG:
        pkmfginfo = (PK_MfgInfo_t *) msg;
        diag_printf ("    flashOffset   %x\n", pkmfginfo->flashOffsetMfgInfo);
        diag_printf ("    data          %x\n", pkmfginfo->data);
        if ((cmd == RSP_GET_MFG) || (cmd == CMD_SET_MFG))
        {
            diag_printf ("    boardDesc     %s\n", 
                pkmfginfo->mfgInfo.boardDesc);
            diag_printf ("    paOptions     %x\n", 
                pkmfginfo->mfgInfo.paOptions);
            diag_printf ("    extPA         %x\n", 
                pkmfginfo->mfgInfo.extPA);
            diag_printf ("    antenna       %x\n", 
                pkmfginfo->mfgInfo.antenna);
            for (i = 0 ; i < INT_PA_SIZE ; i++)
                diag_printf ("    intPA[%x]      %x\n", i, 
                    pkmfginfo->mfgInfo.intPA[i]);
            for (i = 0 ; i < MFG_CCA_SIZE ; i++)
                diag_printf ("    cca[%d]        %x\n", i, 
                    pkmfginfo->mfgInfo.cca[i]);
            diag_printf ("    contryCode    %x\n", 
                pkmfginfo->mfgInfo.countryCode);
            diag_printf ("    customerOpt   %x\n", 
                pkmfginfo->mfgInfo.customerOpt);
            for (i = 0 ; i < 4 ; i++)
                diag_printf ("    led[%d]        %x\n", i, 
                    pkmfginfo->mfgInfo.led[i]);
            for (i = 0 ; i < 3 ; i++)
                diag_printf ("    reserved[%d]   %x\n", i, 
                    pkmfginfo->mfgInfo.reserved[i]);
        }
        diag_printf ("    checksum      %x\n", 
            pkmfginfo->mfgInfo.checksum);
        break;
  
    case CMD_GET_VERS:
	pkmfgversion = (PK_MfgVersion_t *) msg;
        diag_printf ("    version       %s\n", pkmfgversion->version);
        break;

    case CMD_ENB_SSID:
        pkenablessid = (PK_EnableSsid_t *) msg;
        diag_printf ("    ssid          %s\n", pkenablessid->ssid);
        diag_printf ("    bssid         ");
        for (i = 0 ; i < 5 ; i++)
            diag_printf ("%x.", pkenablessid->bssid[i]);
        diag_printf ("%x\n", pkenablessid->bssid[i]);
        break;

    case CMD_WR_RFPOW:
    case CMD_RD_RFPOW:
        pkrfpower = (PK_RfPower_t *) msg;
        diag_printf ("    powerRefDac   %x\n", pkrfpower->powerRefDac);
        diag_printf ("    preDriver     %x\n", pkrfpower->preDriver);
        diag_printf ("    polarization  %x\n", pkrfpower->polarization);
        break;

    case CMD_DUTY_CYCLE:
        pkdutycycle = (PK_DutyCycle_t *) msg;
        diag_printf ("    enable        %x\n", pkdutycycle->enable);
        diag_printf ("    dataRate      %x\n", pkdutycycle->dataRate);
        diag_printf ("    percent       %x\n", pkdutycycle->percent);
        break;

    default:
        diag_printf ("UNKNOWN COMMAND\n");
        break;
    }
}
#endif

/******************************************************************************
 *
 * Name: mfgTimerTick
 *
 * Description:
 *    This function is called by the mfgTimer only once and after 60 secondes.
 * If the timer was not disabled by the mfg command CMD_SET_FW_MODE, the mfg
 * task will be killed in order to get the maximum performance during the 
 * field test.
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    None.
 *
 * Return Value:
 *    None.
 *
 * Notes:
 *    None.
 *
 * PDL:
 *
 * END PDL
 *
 *****************************************************************************/
#ifdef MFG_TIMER
static void mfgTimerTick (void)
{
    cyg_thread_kill (TaskMfgHdl);    
}
#endif
#ifdef AP31HOST

int UDPSendTo(const void *msg, int len)
{
//    extern UINT32 UDPClientIP;
    int bytes;

//    outgoing.sin_family = AF_INET;
//    outgoing.sin_port = htons (PORT);
//    outgoing.sin_addr.s_addr = UDPClientIP;
    /* send UDP packet */
    bytes = sendto (
                outgoing_socket,
                (char *) msg,
                len,
                0,
                (const struct sockaddr *) &outgoing1,
                sizeof (outgoing1));
    return (bytes);

}
#endif

#if defined(UDP_API) || defined(AP31E)

int compose_update_packet( APICMDBUF *apicmdp)
{
    static UINT32 len, len1;
    static char *img_ptr,*img_ptr1=NULL;
    static UINT16 seqNum = 1;
    char *ptr;
    ptr = (char *)apicmdp;

    if(apicmdp->seqNum != seqNum++)
    {
        seqNum = 1;
        if(img_ptr1){
            free(img_ptr1);
            img_ptr1 = NULL;
        }
        return APCTL_ERROR;
    }
    if (apicmdp->seqNum == 1) {
        len = *((UINT32 *)&ptr[16]) + 12;
        len1 = len;
        img_ptr = malloc(len);
        img_ptr1 = img_ptr;
    }
    memcpy(img_ptr, &ptr[12], apicmdp->cmdBodyLen);
    img_ptr += apicmdp->cmdBodyLen;
    len -= apicmdp->cmdBodyLen;
    if (len <= 0) {
        if(ApUpgradeFirmware(img_ptr1, len1))
        {
            seqNum = 1;
            free(img_ptr1);
            img_ptr1 = NULL;
        }
        else
        {
            seqNum = 1;
            free(img_ptr1);
            img_ptr1 = NULL;
            return APCTL_ERROR;
        }
    }
    return APCTL_OK;
}
#endif
