/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    utils.c
 * \brief   Implement commonly used utilities
 *
 * $Revision: #1 $
 * $Date: 2004/09/10 $
 */

#include "wltypes.h"


/******************************************************************************
 *
 * Name: util_CopyList
 *
 * Description:
 *    Routine to copy a list of bytes that is represented by an array where
 *    an entry of 0 indicates the end of the list if the list does not
 *    contain the maximum number of entries.
 *
 * Conditions For Use:
 *    None.
 *
 * Arguments:
 *    Arg1 (  o): Dest_p  - Pointer to the location a list is to be copied to
 *    Arg2 (i  ): Src_p   - Pointer to the location of the list to be copied
 *    Arg3 (i  ): MaxSize - The maximum allowed length of the list
 *
 * Return Value:
 *    The length of the list.
 *
 * Notes:
 *    The length is computed by assuming that as soon as there is a zero
 *    in the array, that indicates the end of the data in the array.
 *
 * PDL:
 *    Initialize an index into the array to zero
 *    While the value at the index location in the array is not zero
 *    and the value of the index is not greater than MaxSize
 *       Copy the source location to the destination location
 *       Increment to the index in the array
 *    End while
 *    If the entire list has not been filled Then
 *      Enter a zero into the next unfilled location
 *    End If
 *    Return the index
 * END PDL
 *
 *****************************************************************************/
UINT8 util_CopyList(UINT8 *Dest_p, UINT8 *Src_p, UINT32 MaxSize)
{
   UINT32  i = 0;
    
   while (i < MaxSize  &&  Src_p[i] != '\0')
   {
      Dest_p[i] = Src_p[i];
      i++;
   }

   if (i < MaxSize)
   {
      Dest_p[i] = '\0';
   }

   return (i);

}

UINT8 util_CompareList(UINT8 *Dest_p, UINT8 *Src_p, UINT32 MaxSize)
{
   UINT32  i = 0;
    
   while (i < MaxSize )
   {
      if(Dest_p[i] != Src_p[i])
      {
          return (i+1);
      }
      else if (!Dest_p[i]) {
          return 0;
      }
      i++;
   }

   if (i == MaxSize)
   {
      return 0;
   }

   return (i);

}

