#ifndef __DHCPS_H__
#define __DHCPS_H__


/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    dhcps.h
 * \brief   Header file for DHCP Server Module
 *
 * Author(s):    Yevgeny Yanovsky
 * Date:         2004-07-11
 *
 * $Revision: #1 $
 * $Date: 2004/11/29 $
 */


 /*!**************************************************************************
 *!
 *$ GENERAL DESCRIPTION:  DHCP Exported Types en prototypes.
 *!
 *$ REMARKS:              None
 *!
 *!**************************************************************************
 *!*/

#include  <network.h>
#include <netinet/in.h>

#include "wltypes.h"
#include "etherlow.h"

#include "dhcps_api.h"

/******************************************************************************/

#define PKD __attribute__ (( aligned(1) , packed ))


#define _string(s) #s
#define string(s) _string(s)

/* Definitions */
#define DHCPS_Buff_Size                         1500


/*
Client identifier length,
requests with longer client identifier
will be discarded
*/

#define DHCPDBG_client_id_length_CNS        32

#define DHCPP_max_lease_time_CNS            0xFFFFFFFF

#define DHCPP_chaddr_len_CNS                16
#define DHCPP_sname_len_CNS                 64
#define DHCPP_bootfile_len_CNS              128

#define DHCPP_udp_port_bootp_srv_CNS        67
#define DHCPP_udp_port_bootp_cln_CNS        68

#define DHCPP_bootp_oper_req_CNS            0x01
#define DHCPP_bootp_oper_rep_CNS            0x02

#define DHCPP_max_msg_len_CNS               548    /* max DHCP actual data size */
#define DHCPP_max_options_len_CNS           312    /* max DHCP options size */
#define DHCPP_constant_options_len_CNS      32     /* options length that is always filled:
                                                       cookie + message type + server id +
                                                       lease time + T1 + T2 +
                                                       client mask + end tag  = 32 */
#define DHCPP_header_b4_options_len_CNS     DHCPP_max_msg_len_CNS - DHCPP_max_options_len_CNS

#define DHCPP_brdcst_flg_CNS                htons(0x8000)


/* OPTIONS */

/* Constant length */
#define DHCPP_opt_len_4_CNS                   4
#define DHCPP_opt_len_1_CNS                   1
#define DHCPP_opt_len_max_message_size_CNS    2

/* Tag values */
#define DHCPP_opt_tag_pad_CNS                 0     /* len 1 */
#define DHCPP_opt_tag_ip_mask_CNS             1     /* len 4 */
#define DHCPP_opt_tag_ip_DftRouter_CNS        3     /* Variable Length */
#define DHCPP_opt_tag_time_srv_CNS            4     /* Variable Length */
#define DHCPP_opt_tag_name_srv_CNS            5     /* Variable Length */
#define DHCPP_opt_tag_dns_srv_CNS             6     /* Variable Length */
#define DHCPP_opt_tag_host_name_CNS           12    /* Variable Length */
#define DHCPP_opt_tag_domain_name_CNS         15    /* Variable Length */
#define DHCPP_opt_tag_swap_server_CNS         16    /* len 4 */
#define DHCPP_opt_tag_wins_srv_CNS            44    /* Variable Length */
#define DHCPP_opt_tag_wins_node_type_CNS      46    /* len 1 */
#define DHCPP_opt_tag_req_ip_addr_CNS         50    /* len 4 */
#define DHCPP_opt_tag_ip_addr_ls_time_CNS     51    /* len 4 */
#define DHCPP_opt_tag_overload_CNS            52    /* len 1 */
#define DHCPP_opt_tag_msg_type_CNS            53    /* len 2 */
#define DHCPP_opt_tag_srv_id_CNS              54    /* len 4 */
#define DHCPP_opt_tag_req_list_prm_CNS        55    /* Variable Length */
#define DHCPP_opt_tag_message_CNS             56    /* Variable Length */
#define DHCPP_opt_tag_max_message_size_CNS    57    /* len 1 */
#define DHCPP_opt_tag_T1_CNS                  58    /* len 4 */
#define DHCPP_opt_tag_T2_CNS                  59    /* len 4 */
#define DHCPP_opt_tag_vendor_class_id_CNS     60    /* Variable Length */
#define DHCPP_opt_tag_client_id_CNS           61    /* Variable Length */
#define DHCPP_opt_tag_tftp_srv_name_CNS       66    /* Variable Length */
#define DHCPP_opt_tag_bootfile_name_CNS       67    /* Variable Length */
#define DHCPP_opt_tag_community_CNS           128   /* Variable Length */
#define DHCPP_opt_tag_ip_nms_CNS              129   /* len 4 */
#define DHCPP_opt_tag_root_CNS                130   /* Unused */
#define DHCPP_opt_tag_end_CNS                 255   /* len 1 */

/* END OF OPTIONS */

/* Base Data Types */

typedef PKD struct {
    UINT8       bytes[6];
} RSG_mac_addr_STC;

#define RSG_mac_addr_TYP  RSG_mac_addr_STC


/* MACROS */

#define RSG_swap32_MAC(val)   (htonl(val))

#define RSG_mac_addr_is_null_MAC(addr_PTR)                              \
    ( (((RSG_mac_addr_TYP *)(addr_PTR))->bytes[0] ==  0) &&             \
      (((RSG_mac_addr_TYP *)(addr_PTR))->bytes[1] ==  0) &&             \
      (((RSG_mac_addr_TYP *)(addr_PTR))->bytes[2] ==  0) &&             \
      (((RSG_mac_addr_TYP *)(addr_PTR))->bytes[3] ==  0) &&             \
      (((RSG_mac_addr_TYP *)(addr_PTR))->bytes[4] ==  0) &&             \
      (((RSG_mac_addr_TYP *)(addr_PTR))->bytes[5] ==  0) )

/*! Check if the given IP addresses are on the same subnet  */
#define RSG_ip_is_same_subnet( ip_addr_1, ip_addr_2, ip_mask )          \
    (((ip_addr_1) & (ip_mask)) == ((ip_addr_2) & (ip_mask)))

/* Data Types */

typedef enum
{
    DHCPS_alloc_status_valid_E,
    DHCPS_alloc_status_expired_E,
    DHCPS_alloc_status_declined_E,
    DHCPS_alloc_status_static_ghost_E
} DHCPS_alloc_status_ENT;

typedef enum
{
    DHCPS_alloc_mechanism_dynamic_E,
    DHCPS_alloc_mechanism_static_E
} DHCPS_alloc_mechanism_ENT;

typedef enum
{
    DHCPP_alloc_start_E,
    DHCPP_alloc_old_found_E,
    DHCPP_alloc_new_E,
    DHCPP_alloc_new_from_free_E,
    DHCPP_alloc_new_from_expired_E,
    DHCPP_alloc_new_from_declined_E,
    DHCPP_alloc_set_response_data_E
} DHCPP_alloc_states_ENT;


typedef enum {
    DHCPP_message_type_first_E,
    DHCPP_message_type_dscvr_E,
    DHCPP_message_type_offer_E,
    DHCPP_message_type_req_E,
    DHCPP_message_type_dcln_E,
    DHCPP_message_type_ack_E,
    DHCPP_message_type_nak_E,
    DHCPP_message_type_rls_E,
    DHCPP_message_type_inf_E,
    DHCPP_message_type_bootp_req_E,
    DHCPP_message_type_last_E
} DHCPP_message_type_ENT;



/****************************************/
/* DHCP PDU structure */
/****************************************/

typedef PKD struct
{
    UINT8               oper;           /* Operation Code */
    UINT8               htype;          /* The Hardware Type Code */
    UINT8               hlen;           /* Number of Octets in Hardware Address */
    UINT8               hops;           /* hops */
    UINT32              xid;            /* a random number */
    UINT16              secs;           /* time from boot */
    UINT16              flags;          /* flags field */
    unsigned long       ciaddr;         /* client Ip address */
    unsigned long       yiaddr;         /* "your" Ip address */
    unsigned long       siaddr;         /* server Ip address */
    unsigned long       giaddr;         /* gateway Ip address */
    UINT8               chaddr[DHCPP_chaddr_len_CNS];
                                        /* client hardware address */
    UINT8               sname[DHCPP_sname_len_CNS];
                                        /* optional server host name  */
                                        /* for rx - options or unused */
    UINT8               file[DHCPP_bootfile_len_CNS];
                                        /* boot file name             */
                                        /* for rx - options or unused */
    UINT8               options[DHCPP_max_options_len_CNS];
                                        /* options */
} DHCPP_net_dhcp_hdr_STC;



/* Server Configuration Parameters */

typedef struct
{
    unsigned long       enabled;         /* BOOLEAN     */
    unsigned long       deviceIpAddr;    /* IP Address  */
    unsigned long       deviceIpMask;    /* IP Address  */
    unsigned long       ipAddrFrom;      /* IP Address  */
    unsigned long       ipAddrTo;        /* IP Address  */
    unsigned long       primDns;         /* IP Address  */
    unsigned long       secDns;          /* IP Address  */
    unsigned long       leaseTime;       /* seconds     */

} DHCPS_conf_params_STC;


/* Allocation Table Entry */

typedef struct
{
    /* key */
    unsigned long               ipAddr;
    /* data */
    RSG_mac_addr_TYP            macAddr;
    DHCPS_alloc_mechanism_ENT   mechanism;
    UINT32                      bornTime;
    DHCPS_alloc_status_ENT      status;
    char                        hostName[DHCPS_host_name_max_len_CNS];

} DHCPS_alloc_entry_STC;


/* DHCP Rx Template */

typedef struct
{
    DHCPP_message_type_ENT      messageType;    /* DHCP message type */
    unsigned long               clientIpMask;   /* Subnet mask 1 */
    unsigned long               reqIpAddr;      /* Request IP address */
    UINT32                      leaseTime;      /* Lease Time option 51 */
    UINT32                      t1;             /* Renewal (T1) Time Value 58 */
    UINT32                      t2;             /* Rebinding (T2) Time Value 59 */
    RSG_mac_addr_TYP            macAddr;        /* Allocation table key */
    char                        hostName[DHCPS_host_name_max_len_CNS];

} DHCPP_net_dhcp_rx_tmpl_STC;


/* Global Variables */
extern unsigned long            DHCPS_started;

extern cyg_sem_t                DHCPS_db_sem;

extern unsigned long            DHCPS_dclnAddr [DHCPS_max_num_declined_addresses_CNS];

extern  DHCPS_conf_params_STC   DHCPS_conf_params;


/* Function Prototypes */

extern unsigned long DHCPS_time_get_in_sec(void);

extern void DHCPS_db_lock (void);

extern void DHCPS_db_unlock (void);

extern void DHCPS_start_timer (void);

extern void DHCPS_stop_timer (void);

extern void DHCPSTask( cyg_addrword_t Data );

extern void DHCPS_alarm_function(cyg_handle_t alarm, cyg_addrword_t data);

extern DHCPS_alloc_entry_STC *DHCPS_get_alloc_entry_by_mac_addr(RSG_mac_addr_TYP macAddr);

extern unsigned int ipToUint(char *str);

#endif /* __DHCPS_H__ */


