#ifndef __L2_H__
#define __L2_H__

/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 *
 * Author(s):    Ariel Gilman
 * Date:         2004-08-03
 *
 * $Revision: #1 $
 * $Date:
 */

#include "marvell/fast_route/fr_main/incl/main_api.h"

/******************************************************************************
 *
 * Name:   FR_L2_init
 *
 * Description:
 *   This routine is called to initialize L2 component of FR.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern FastRouting_init_ENT  FR_L2_init(void);


/******************************************************************************
 *
 * Name:   FR_L2_Frame_update_from_WAN
 *
 * Description:
 *   This routine is calls upon RX of packets from WAN.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 - buffer (may be changed).
 *   Arg2 - buffer length.
 *
 * Return Value:
 *   Action for the packet and updated buffer.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern FastRouting_action_ENT  FR_L2_Frame_update_from_WAN (
    /* input */
    /* input & output */
    UINT8                     **buff_ptr_PTR,
    UINT32                    *buff_len_PTR
    /* output */
);


/******************************************************************************
 *
 * Name:   FR_L2_update_MAC_to_WAN
 *
 * Description:
 *   This routine is called before TX packets to WAN.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 - buffer (may be changed).
 *   Arg2 - buffer length.
 *
 * Return Value:
 *   Action for the packet and updated buffer.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern FastRouting_action_ENT  FR_L2_update_MAC_to_WAN (
    /* input */
    /* input & output */
    UINT8                     **buff_ptr_PTR,
    UINT32                    *buff_len_PTR
    /* output */
);


/******************************************************************************
 *
 * Name:   L2_update_MAC_to_LAN
 *
 * Description:
 *   This routine is called before TX packets to LAN.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 - buffer.
 *   Arg2 - buffer length.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern FastRouting_action_ENT  L2_update_MAC_to_LAN (
    /* input */
    UINT8                     *buff_PTR
    /* input & output */
    /* output */
);


#endif /* __L2_H__ */
