#ifndef __FR_MAIN_API_H__
#define __FR_MAIN_API_H__

/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 *
 * Author(s):    Ariel Gilman
 * Date:         2004-08-03
 *
 * $Revision: #1 $
 * $Date:
 */

#ifdef _RND_
#include         <inc\defs.h>

/* radlan simulation */
#include    <os\exp\os.h>
#include    <lib\swap\exp\swap.h>
#include    <lib\hash\exp\hash.h>

#define swap16 RSG_swap16_MAC
#define swap32 RSG_swap32_MAC
#define UINT8 UINT_8
#define UINT16 UINT_16
#define UINT32 UINT_32
#define diag_printf OSIOG_printf
#define time_get_in_sec() ( OSTIMG_time_get_in_ms_MAC()/1000 )

#else
/* sdk */
#include "wltypes.h"
#include "etherlow.h"
#include  <network.h>
#include "..\..\..\..\marvell\hash\incl\ghash.h"
#include "IEEE_types.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <sys/endian.h>
#include <machine/endian.h>
#include <machine/types.h>
#include <cyg/kernel/kapi.h>
#include "mib.h"


#define PKD PACK_START
/*__attribute__ (( aligned(1) , packed )) */
#define PRAGMA_NOALIGN
#define PRAGMA_ALIGN

#define time_get_in_sec() (cyg_current_time() / 89 )

#endif


#define FR_MAIN_mac_WAN_exist_CNS        0x00000001
#define FR_MAIN_mac_WAN_not_exist_CNS    0x00000000
#define mac_dst_offset_CNS 0
#define mac_src_offset_CNS 6
#define mac_size_CNS       6
#define ip_src_offset_CNS  26
#define ip_dst_offset_CNS  30
#define ip_size_CNS        4


/* Defines */

PRAGMA_NOALIGN

typedef PKD union {
    UINT8   bytes[4];
    UINT32  ipAddr;
} FR_ip_addr_UNT;
typedef  FR_ip_addr_UNT  IpAddress_TYP;

typedef PKD struct {
    UINT8   bytes[6];
} FR_mac_addr_STC;
typedef  FR_mac_addr_STC  MacAddress_TYP;

PRAGMA_ALIGN

extern IpAddress_TYP    FR_MAIN_LAN_ip_addr;
extern IpAddress_TYP    FR_MAIN_LAN_ip_mask;
extern IpAddress_TYP    FR_MAIN_WAN_ip_addr;
extern IpAddress_TYP    FR_MAIN_WAN_ip_mask;
extern IpAddress_TYP    FR_MAIN_WAN_dGateway;
extern MacAddress_TYP   FR_MAIN_mac_addr_my;
extern MacAddress_TYP   FR_MAIN_mac_addr_dGateway;
extern UINT32           FR_MAIN_mac_WAN;

/* ENUMS */

typedef enum {
    FastRouting_LAN_inf_E,
    FastRouting_WAN_inf_E
} FastRouting_interface_ENT;

typedef enum {
    FastRouting_init_ok_E,
    FastRouting_init_error_E
} FastRouting_init_ENT;

typedef enum {
    FastRouting_next_E,
    FastRouting_to_Lan_E,
    FastRouting_to_Wan_E,
    FastRouting_to_Stack_E,
    FastRouting_discard_E
} FastRouting_action_ENT;

/* Data */

PRAGMA_NOALIGN

typedef PKD struct{
    MacAddress_TYP  dest_addr;
    MacAddress_TYP  src_addr;
    UINT16          eth_type;
}FR_eth_header_STC;

typedef PKD struct  {
    UINT8           ver_len;
    UINT8           service_type;
    UINT16          total_len;
    UINT16          ident;
    UINT16          frag_offest;
    UINT8           ttl;
    UINT8           protocol;
    UINT16          checksum;
    IpAddress_TYP   src_ip;
    IpAddress_TYP   dest_ip;
}FR_ip_header_STC;

PRAGMA_ALIGN

/******************************************************************************
 *
 * Name:   FR_Main_Print_IP
 *
 * Description:
 *   Print IP address or mask.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 - IP address or mask.
 *
 * Return Value:
 *   Action for the packet.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void FR_Main_Print_IP(
    /* input */
    IpAddress_TYP    IpData
    /* input & output */
    /* output */
);

/******************************************************************************
 *
 * Name:   FR_Main_Print_MAC
 *
 * Description:
 *   Print MAC address.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 - MAC address.
 *
 * Return Value:
 *   Action for the packet.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void FR_Main_Print_MAC(
    /* input */
    MacAddress_TYP    MacAddress
    /* input & output */
    /* output */
);


/******************************************************************************
 *
 * Name:   FR_Main_Init
 *
 * Description:
 *   This routine is called to initialize MAIN component of FR.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern FastRouting_init_ENT FR_Main_Init( void );



/******************************************************************************
 *
 * Name:   FR_Main_IP_rx
 *
 * Description:
 *   This routine is upon RX of IP packets.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 - incoming interface.
 *   Arg2 - buffer (may be changed).
 *   Arg3 - buffer length.
 *
 * Return Value:
 *   Action for the packet and updated buffer.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern FastRouting_action_ENT FR_Main_IP_rx(
    /* input */
    FastRouting_interface_ENT  incoming_inf,
    /* input & output */
    UINT8                       **buff_ptr_PTR,
    UINT32                       *buff_len_PTR
    /* output */
);



/******************************************************************************
 *
 * Name:   FR_Main_ARP_rx
 *
 * Description:
 *   This routine is called upon RX of ARP packets.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   Arg1 - incoming interface.
 *   Arg2 - buffer.
 *   Arg3 - buffer length.
 *
 * Return Value:
 *   Action for the packet.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void FR_Main_ARP_rx(
    /* input */
    FastRouting_interface_ENT  incoming_inf,
    UINT8                       *buff_PTR,
    UINT32                       buff_len
    /* input & output */
    /* output */
);




/******************************************************************************
 *
 * Name:   FR_Main_WanIP_update
 *
 * Description:
 *   This routine is called upon update of WAN IP interface.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 *
 *
 * END PDL
 *
 *****************************************************************************/

extern void FR_Main_WanIP_update( void );


#endif /* __FR_MAIN_API_H__ */
