#ifndef __GHASH_H__
#define __GHASH_H__


/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*/

/*!
 * \file    dhcps.h
 * \brief   Header file for DHCP Server Module
 *
 * Author(s):    Yevgeny Yanovsky
 * Date:         2004-07-11
 *
 * $Revision: #1 $
 * $Date: 2004/11/29 $
 */
#include "wltypes.h"


/* TBD what about fatal errors ? */
#define        MIN(a,b) (((a)<(b))?(a):(b))
#define        MAX(a,b) (((a)>(b))?(a):(b))

/* TBD what about fatal errors ? */
/*#ifdef _WIN32
#define PKD
#else
#define PKD __attribute__ (( aligned(1) , packed ))
#endif
*/
/* #define HASH_DEBUG_FLAG 1 */
/*!  Action has succeded */
#define  HSG_success_CNS         0

/*! No free entries due to big tables or leck of memory. */
#define  HSG_overflow_CNS        1

/*! Entry is already exist in TABLE. */
#define  HSG_already_exist_CNS   2

/*! Entry doesn't exist in table. */
#define  HSG_not_found_CNS       3

/*! Sort-by-insertion-time unavailable */
#define  HSG_no_insert_sort_CNS  4
#define HSG_SORT_MIN_VAL_CNS            0
#define HSG_NO_SORT_CNS                 0
#define HSG_SORT_BY_KEY_CNS             1
#define HSG_SORT_BY_INSERTION_TIME_CNS  2
#define HSG_SORT_BY_KEY_AND_TIME_CNS    3
#define HSG_SORT_MAX_VAL_CNS            HSG_SORT_BY_KEY_AND_TIME_CNS

/*!**************************************************************************
 *!
 *$ FUNCTION: HSG_NewInit_hash
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Routine initialize HASH table entity according to user requirements.
 *! Routine allocates memory and initialize it.
 *!
 *$ RETURNS:
 *!
 *! Routine returns initialize status (ok, fail).
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *! Using OS utilities of allocate memory.
 *! Using Error handler utility in case of FATAL error.
 *!
 *$ REMARKS:     (local)
 *!
 *!**************************************************************************
 *!*/

extern UINT32 HSG_NewInit_hash ( /* Returns init results */

    /*!     INPUTS / OUTPUTS:   */
     void*       *id_ptr,   /* Hash id for further use of hash utilities*/

    /*!     INPUTS:             */
         UINT32  key_len,  /* key size 2,4,6,8,10,12,20,50 */

         UINT32  entry_size, /* size of data in bytes (with the key) */

         UINT32  min_num_of_entries, /* Basic entry allocation */

         UINT32  max_num_of_entries,

         UINT32  alloc_portion, /* Portion for each entries allocation*/

         UINT32  header_size, /* Size is round up to the nearest power of 2 */
                               /* Big header improves hash hit ratio */

         UINT32  sort_db_val,   /*  0 - no sorting on the DB
                     1 - maintain the DB sorted by key
                     2 - maintain the DB sorted by insertion
                         time
                     3 - maintain the DB sorted both by key
                         and by insertion time.
                 */

         UINT32  free_entries_margin,/* Number of empty entries that are */
                                      /* always in the free list. This */
                                      /* margin promis us that at least  */
                                      /* this num of inserts must be done */
                                      /* before a deleted entry memory */
                                      /* area will be used again. */

         UINT32  use_sorted_header   /* Flag indicates weather to use sorted */
                                      /* header in case od sorted hash. */
)
;
/*!**************************************************************************
 *!
 *$ FUNCTION:
 *!
 *$ GENERAL DESCRIPTION:  HSG_init_hash
 *!
 *! Routine initialize HASH table entity according to user requirements.
 *! Routine allocates memory and initialize it.
 *!
 *$ RETURNS:
 *!
 *! Routine returns initialize status (ok, fail).
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *! Using OS utilities of allocate memory.
 *! Using Error handler utility in case of FATAL error.
 *!
 *$ REMARKS:     (local)
 *!
 *!**************************************************************************
 *!*/

extern UINT32 HSG_init_hash ( /* Returns init results */

    /*!     INPUTS / OUTPUTS:   */
     void*       *id_ptr,   /* Hash id for further use of hash utilities*/

    /*!     INPUTS:             */
         UINT32  key_len,  /* key size 2,4,6,8,10,12,20,50 */

         UINT32  entry_size, /* size of data in bytes (with the key) */

         UINT32  min_num_of_entries, /* Basic entry allocation */

         UINT32  max_num_of_entries,

         UINT32  alloc_portion, /* Portion for each entries allocation*/

         UINT32  header_size, /* Size is round up to the nearest power of 2 */
                               /* Big header improves hash hit ratio */

         UINT32  sort_db_val,   /* FALSE - no sorting on the DB
                    TRUE  - maintain the DB sorted by key */

         UINT32  free_entries_margin,  /* Number of empty entries that are */
                                        /* always in the free list. This */
                                        /* margin promis us that at least  */
                                        /* this num of inserts must be done */
                                        /* before a deleted entry memory */
                                        /* area will be used again. */

         UINT32  use_sorted_header   /* Flag indicates weather to use sorted */
                                      /* header in case od sorted hash. */
);
/*$ END OF HSG_init_hash  */
/*!**************************************************************************
 *!
 *$ FUNCTION: HSG_search
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Routine is used to search for an entry.
 *! Search is a protected routine.
 *!
 *$ RETURNS:
 *!
 *! Routine returns pointer to required entry.
 *! If entry not found returns NULL.
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS:     (local)
 *!
 *!
 *!**************************************************************************
 *!*/


extern void* HSG_search ( /* Returns search results */

    /*!     INPUTS:             */
        void  *id,  /* Hash table identification. */

        void  *key_ptr /* Element key */

)
;
/*!**************************************************************************
 *!
 *$ FUNCTION:
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Routine is used to insert new entry, in case entry exist data is
 *! NOT changed.
 *! In sorted DB insert update the sorted list.
 *! In large SORTED tables insert is heavy task.
 *!
 *$ RETURNS:
 *!
 *! Routine returns insert status (ok, already exist, overflow, fail).
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS:
 *! 17-Dec-2001, ShayZ
 *! Find out if the entry already exists before trying to allocate memory
 *! (if there are no free entries).
 *!
 *!**************************************************************************
 *!*/

extern UINT32 HSG_insert ( /* Returns insert results */

    /*!     INPUTS:             */
        void  *id,  /* Hash table identification. */

        void  *user_entry_ptr, /* Element ptr with key at first */

        void* *entry_ptr_ptr  /* OUTPUT: pointer to inserted entry */
)
;
/*$ END OF HSG_insert  */
/*!**************************************************************************
 *!
 *$ FUNCTION:
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Routine is used to delete entry, the memory used by the deleted entry
 *! will be used again after at least free_entries_margin inserts
 *! or updates.
 *! In sorted DB insert update the sorted list.
 *!
 *$ RETURNS:
 *!
 *! Routine returns delete status (ok, not found, fail).
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS:     (local)
 *!
 *! in large SORTED tables delete is heavy task.
 *!
 *!**************************************************************************
 *!*/


extern UINT32 HSG_delete ( /* Returns delete results */

    /*!     INPUTS:             */
        void  *id,  /* Hash table identification. */

        void  *key_ptr /* Element key */

)
;
/*$ END OF HSG_delete  */
/*!**************************************************************************
 *!
 *$ FUNCTION: HSG_update
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Routine is used to update entries data in such a way that reading
 *! data is always valid and legal.
 *! If entry doesn't exist update inserts new entry.
 *!
 *$ RETURNS:
 *!
 *! Routine returns update status (ok, overflow, fail).
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS: ShayZ 5/2/02 - when there are no free entries and only 1
 *!                            margin entry - first_free_entry = last_free_entry
 *! in large SORTED tables update is heavy task.
 *!
 *!**************************************************************************
 *!*/

extern UINT32 HSG_update ( /* Returns update results */

    /*!     INPUTS:             */
        void  *id,  /* Hash table identification. */

        void  *user_entry_ptr, /* Element ptr with key at first */

        void* *entry_ptr_ptr  /* OUTPUT: pointer to updated entry */
)
;
/*!**************************************************************************
 *!
 *$ FUNCTION:
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Routine delete all HASH table entries.
 *!
 *$ RETURNS:
 *!
 *! Routine returns initialize status (ok, fail).
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *! Using Error handler utility in case of FATAL error.
 *!
 *$ REMARKS:     (local)
 *!
 *!**************************************************************************
 *!*/

extern UINT32 HSG_clean_hash ( /* Returns clean results */

    /*!     INPUTS:             */
        void  *id  /* Hash table identification. */

)
;
/*!**************************************************************************
 *!
 *$ FUNCTION:
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Routine is used for SNMP reports and fast walk over the table.
 *! The routine returnes the next sorted (by key) entry.
 *!
 *$ RETURNS:
 *!
 *! In case input key_ptr is NULL the smallest entry, otherwise the routine
 *! returns the next sorted (by key) entry.
 *! In case input key_ptr is equal or bigger then the bigest entry in table
 *! routine returns NULL.
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS:     (local)
 *!
 *!**************************************************************************
 *!*/

extern void* HSG_get_next_sort ( /* Returns get_next results */

    /*!     INPUTS:             */
        void  *id,  /* Hash table identification. */

        void  *key_ptr  /* Element key */
)
;
/*!**************************************************************************
 *!
 *$ FUNCTION:
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Routine is used for fast walk over the table.
 *! The routine returnes as many next sorted (by key) entries as the user asked.
 *!
 *$ RETURNS:
 *!
 *! In case input key_ptr is NULL the smallest entries, otherwise the routine
 *! returns the next sorted (by key) entries.
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS:     (local)
 *!
 *!**************************************************************************
 *!*/

extern void HSG_get_next_sort_entries ( /* Returns get_next results */

    /*!     INPUTS:             */
        void     *id,  /* Hash table identification. */

        void     *key_ptr,  /* Element key */

        UINT32* num_of_entries_ptr, /* Input:  Needed enties num. */
                                     /* Output: Actual entries num. */

        void*    *entries_list_ptr  /* Pointer to list of entrie pointers. */
)
;
/*$ END OF HSG_get_next_sort_entries */
/*!**************************************************************************
 *!
 *$ FUNCTION:
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Function is used to get the entry in the hash table, which was inserted
 *! after the one given as parameter (or to get the oldest entry, if given a
 *! NULL pointer).
 *!
 *$ RETURNS:
 *!
 *! The function returns a pointer to the key of the entry that was inserted
 *! after the given one, or NULL if it is the newest entry or if the hash table
 *! is not sorted by insertion time.
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS:
 *!
 *!**************************************************************************
 *!*/

extern void *HSG_get_next_by_insertion_time (

  /*!     INPUTS:             */

  void *id,      /* pointer to hash id */
  void *key_ptr  /* Pointer to an entry key. The routine will return a pointer
                    to the entry that was inserted after this one, or NULL if it
                    is the newest or if the  hash table is not sorted by
                    insertion time.
                    Use NULL to get the oldest entry. */

  )
;
/*!**************************************************************************
 *!
 *$ FUNCTION:
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Function is used to delete the n oldest entries in the hash table.
 *!
 *$ RETURNS:
 *!
 *! Function returns in parameter num_deleted the number of entries successfully
 *! deleted (which is only smaller than n in case there are not enough
 *! elements).
 *! The function's return value is:
 *! - HSG_success_CNS, on successful deletion
 *! - HSG_not_found_CNS, on the first deletion fail
 *! - HSG_no_insert_sort_CNS, if the hash table is not sorted by insertion time.
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS:
 *!
 *!**************************************************************************
 *!*/

extern UINT32 HSG_delete_oldest_n (

  /*!     INPUTS:             */
  void*           id,           /* pointer to hash id */
  UINT32         n,            /* number of entries to delete */

  /*!     OUTPUTS:            */
  UINT32         *num_deleted  /* return parameter: number of
                                   entries successsfully deleted */
  );
/*$ END OF HSG_delete_oldest_n */
/*!**************************************************************************
 *!
 *$ FUNCTION: HSG_test_next_insert
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *$ RETURNS:
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS: THIS FUNCTION CHECKS THE HASH ID ONLY !! IT CHECKS WHETHER
    MAX ENTRIES WERE USE OR NOT. AFTER THE SUCCESS OF THIS FUNCTION YOU CAN
    STILL GET MEM ALLOACTION ERROR DURING THE INSERT FUNCTION.
 *!
 *!**************************************************************************
 *!*/

extern UINT32 HSG_test_next_insert (

    /*!     INPUTS:             */

    void * id
);
/*!**************************************************************************
 *!
 *$ FUNCTION:
 *!
 *$ GENERAL DESCRIPTION:
 *!
 *! Function is used to get the number of entries in the hash table
 *!
 *$ RETURNS:
 *!
 *! Function returns the number of entries in the hash table
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS:
 *!
 *!**************************************************************************
 *!*/

extern UINT32 HSG_get_entries_num (

  /*!     INPUTS:             */
  void *id
)
;
/*!**************************************************************************
 *!
 *$ FUNCTION: HSG_get_next_sort_by_ptr
 *!
 *$ GENERAL DESCRIPTION:
 *!      The routine is used to get the following sorted hash entry.
 *!      entry_ptr MUST point to an entry that iscurrently in the hash
 *!      table. CAUTION - if using this function in the process of deleteing
 *!      entries - first call this function and then delete the entry.
 *!
 *$ RETURNS:
 *!
 *$ ALGORITHM:   (local)
 *!
 *$ ASSUMPTIONS:
 *!
 *$ REMARKS:     (local)
 *!
 *!**************************************************************************
 *!*/
extern void * HSG_get_next_sort_by_ptr(
        /*!     INPUTS:             */

    void    * id, /* Hash table identification. */

    void    * entry_ptr /* Element key */
)
;

#endif /* __GHASH_H__  */
