#ifndef _API_H_
#define _API_H_ 
#ifdef UDP_API
#include "api_e.h"
#else
/*
 *               Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    api.h
 * \brief   This file contains the function prototypes and definitions for the
 *          AP commands API
 *
 */

/* Use these macros for port name */
/* Wireless LAN port */
#define PORT_NAME_WLAN "wl0" 
/* WAN logical port attached to physical port eth0 */
#define PORT_NAME_WAN  "wan0" 
/* LAN logical port attached to physical port eth0 */
#define PORT_NAME_LAN  "lan0" 
/** DMZ logical port **/
#define PORT_NAME_DMZ "dmz0"

#define PORT_NAME_ETH "eth0" 

/* GPIO number cannot exceed this limit */
#define MAX_GPIO_NUM 15

extern UINT32 CheckAvailablePool(void);
extern UINT32 NoPoolBufAvailable;
void Wlan_Radio(UINT8 option);


//=============================================================================
//                          API for Sending Commands
//=============================================================================

typedef	enum
{
/* please do not add #ifdef inside of this enum */
    APCMD_BSS_START                  = 0x00000001       ,/* 0x00000001 */
    APCMD_BSS_START_REPLY                               ,/* 0x00000002 */
    APCMD_GET_SSID                                      ,/* 0x00000003 */
    APCMD_GET_SSID_REPLY                                ,/* 0x00000004 */
    APCMD_SET_SSID                                      ,/* 0x00000005 */
    APCMD_SET_SSID_REPLY                                ,/* 0x00000006 */
    APCMD_GET_OPEN_SSID_EN_STATE                        ,/* 0x00000007 */
    APCMD_GET_OPEN_SSID_EN_STATE_REPLY                  ,/* 0x00000008 */
    APCMD_SET_OPEN_SSID_EN_STATE                        ,/* 0x00000009 */
    APCMD_SET_OPEN_SSID_EN_STATE_REPLY                  ,/* 0x0000000a */
    APCMD_GET_RF_MAC_ADDRESS                            ,/* 0x0000000b */
    APCMD_GET_RF_MAC_ADDRESS_REPLY                      ,/* 0x0000000c */
    APCMD_SET_RF_MAC_ADDRESS                            ,/* 0x0000000d */
    APCMD_SET_RF_MAC_ADDRESS_REPLY                      ,/* 0x0000000e */
    APCMD_GET_RF_CHANNEL                                ,/* 0x0000000f */
    APCMD_GET_RF_CHANNEL_REPLY                          ,/* 0x00000010 */
    APCMD_SET_RF_CHANNEL                                ,/* 0x00000011 */
    APCMD_SET_RF_CHANNEL_REPLY                          ,/* 0x00000012 */
    APCMD_GET_BASIC_DATA_RATES                          ,/* 0x00000013 */
    APCMD_GET_BASIC_DATA_RATES_REPLY                    ,/* 0x00000014 */
    APCMD_SET_BASIC_DATA_RATES                          ,/* 0x00000015 */
    APCMD_SET_BASIC_DATA_RATES_REPLY                    ,/* 0x00000016 */
    APCMD_GET_SUP_DATA_RATES                            ,/* 0x00000017 */
    APCMD_GET_SUP_DATA_RATES_REPLY                      ,/* 0x00000018 */
    APCMD_SET_SUP_DATA_RATES                            ,/* 0x00000019 */
    APCMD_SET_SUP_DATA_RATES_REPLY                      ,/* 0x0000001a */
    APCMD_GET_TX_DATA_RATE                              ,/* 0x0000001b */
    APCMD_GET_TX_DATA_RATE_REPLY                        ,/* 0x0000001c */
    APCMD_SET_TX_DATA_RATE                              ,/* 0x0000001d */
    APCMD_SET_TX_DATA_RATE_REPLY                        ,/* 0x0000001e */
    APCMD_GET_TX_POWER_LEVEL                            ,/* 0x0000001f */
    APCMD_GET_TX_POWER_LEVEL_REPLY                      ,/* 0x00000020 */
    APCMD_SET_TX_POWER_LEVEL                            ,/* 0x00000021 */
    APCMD_SET_TX_POWER_LEVEL_REPLY                      ,/* 0x00000022 */
    APCMD_GET_RF_PREAMBLE_OPTION                        ,/* 0x00000023 */
    APCMD_GET_RF_PREAMBLE_OPTION_REPLY                  ,/* 0x00000024 */
    APCMD_SET_RF_PREAMBLE_OPTION                        ,/* 0x00000025 */
    APCMD_SET_RF_PREAMBLE_OPTION_REPLY                  ,/* 0x00000026 */
    APCMD_GET_ANTENNA_MODE                              ,/* 0x00000027 */
    APCMD_GET_ANTENNA_MODE_REPLY                        ,/* 0x00000028 */
    APCMD_SET_ANTENNA_MODE                              ,/* 0x00000029 */
    APCMD_SET_ANTENNA_MODE_REPLY                        ,/* 0x0000002a */
    APCMD_GET_FRAG_THRESHOLD                            ,/* 0x0000002b */
    APCMD_GET_FRAG_THRESHOLD_REPLY                      ,/* 0x0000002c */
    APCMD_SET_FRAG_THRESHOLD                            ,/* 0x0000002d */
    APCMD_SET_FRAG_THRESHOLD_REPLY                      ,/* 0x0000002e */
    APCMD_GET_RTS_THRESHOLD                             ,/* 0x0000002f */
    APCMD_GET_RTS_THRESHOLD_REPLY                       ,/* 0x00000030 */
    APCMD_SET_RTS_THRESHOLD                             ,/* 0x00000031 */
    APCMD_SET_RTS_THRESHOLD_REPLY                       ,/* 0x00000032 */
    APCMD_GET_RTS_RETRY_LIMIT                           ,/* 0x00000033 */
    APCMD_GET_RTS_RETRY_LIMIT_REPLY                     ,/* 0x00000034 */
    APCMD_SET_RTS_RETRY_LIMIT                           ,/* 0x00000035 */
    APCMD_SET_RTS_RETRY_LIMIT_REPLY                     ,/* 0x00000036 */
    APCMD_GET_DATA_RETRY_LIMIT                          ,/* 0x00000037 */
    APCMD_GET_DATA_RETRY_LIMIT_REPLY                    ,/* 0x00000038 */
    APCMD_SET_DATA_RETRY_LIMIT                          ,/* 0x00000039 */
    APCMD_SET_DATA_RETRY_LIMIT_REPLY                    ,/* 0x0000003a */
    APCMD_GET_BEACON_PERIOD                             ,/* 0x0000003b */
    APCMD_GET_BEACON_PERIOD_REPLY                       ,/* 0x0000003c */
    APCMD_SET_BEACON_PERIOD                             ,/* 0x0000003d */
    APCMD_SET_BEACON_PERIOD_REPLY                       ,/* 0x0000003e */
    APCMD_GET_DTIM_PERIOD                               ,/* 0x0000003f */
    APCMD_GET_DTIM_PERIOD_REPLY                         ,/* 0x00000040 */
    APCMD_SET_DTIM_PERIOD                               ,/* 0x00000041 */
    APCMD_SET_DTIM_PERIOD_REPLY                         ,/* 0x00000042 */
    APCMD_GET_802_11_STATS                              ,/* 0x00000043 */
    APCMD_GET_802_11_STATS_REPLY                        ,/* 0x00000044 */
    APCMD_ADD_WLAN_FILTER_ENTRY                         ,/* 0x00000045 */
    APCMD_ADD_WLAN_FILTER_ENTRY_REPLY                   ,/* 0x00000046 */
    APCMD_DEL_WLAN_FILTER_ENTRY                         ,/* 0x00000047 */
    APCMD_DEL_WLAN_FILTER_ENTRY_REPLY                   ,/* 0x00000048 */
    APCMD_GET_STA_LIST                                  ,/* 0x00000049 */
    APCMD_GET_STA_LIST_REPLY                            ,/* 0x0000004a */
    APCMD_GET_STA_INFO                                  ,/* 0x0000004b */
    APCMD_GET_STA_INFO_REPLY                            ,/* 0x0000004c */
    APCMD_DEAUTH_STA                                    ,/* 0x0000004d */
    APCMD_DEAUTH_STA_REPLY                              ,/* 0x0000004e */
    APCMD_DISASSOC_STA                                  ,/* 0x0000004f */
    APCMD_DISASSOC_STA_REPLY                            ,/* 0x00000050 */
    APCMD_ADD_ASSOC_FILTER_ENTRY                        ,/* 0x00000051 */
    APCMD_ADD_ASSOC_FILTER_ENTRY_REPLY                  ,/* 0x00000052 */
    APCMD_DEL_ASSOC_FILTER_ENTRY                        ,/* 0x00000053 */
    APCMD_DEL_ASSOC_FILTER_ENTRY_REPLY                  ,/* 0x00000054 */
    APCMD_GET_MAX_NUM_STA                               ,/* 0x00000055 */
    APCMD_GET_MAX_NUM_STA_REPLY                         ,/* 0x00000056 */
    APCMD_SET_MAX_NUM_STA                               ,/* 0x00000057 */
    APCMD_SET_MAX_NUM_STA_REPLY                         ,/* 0x00000058 */
    APCMD_GET_DEFAULT_WEP_KEYS                          ,/* 0x00000059 */
    APCMD_GET_DEFAULT_WEP_KEYS_REPLY                    ,/* 0x0000005a */
    APCMD_SET_DEFAULT_WEP_KEYS                          ,/* 0x0000005b */
    APCMD_SET_DEFAULT_WEP_KEYS_REPLY                    ,/* 0x0000005c */
    APCMD_SET_STA_WEP_KEY                               ,/* 0x0000005d */
    APCMD_SET_STA_WEP_KEY_REPLY                         ,/* 0x0000005e */
    APCMD_GET_PRIVACY_OPTION                            ,/* 0x0000005f */
    APCMD_GET_PRIVACY_OPTION_REPLY                      ,/* 0x00000060 */
    APCMD_SET_PRIVACY_OPTION                            ,/* 0x00000061 */
    APCMD_SET_PRIVACY_OPTION_REPLY                      ,/* 0x00000062 */
    APCMD_GET_AUTH_MODE                                 ,/* 0x00000063 */
    APCMD_GET_AUTH_MODE_REPLY                           ,/* 0x00000064 */
    APCMD_SET_AUTH_MODE                                 ,/* 0x00000065 */
    APCMD_SET_AUTH_MODE_REPLY                           ,/* 0x00000066 */
    APCMD_GET_ETH_MAC_ADDRESS                           ,/* 0x00000067 */
    APCMD_GET_ETH_MAC_ADDRESS_REPLY                     ,/* 0x00000068 */
    APCMD_GET_ETH_IP_CONFIG                             ,/* 0x00000069 */
    APCMD_GET_ETH_IP_CONFIG_REPLY                       ,/* 0x0000006a */
    APCMD_SET_ETH_IP_CONFIG                             ,/* 0x0000006b */
    APCMD_SET_ETH_IP_CONFIG_REPLY                       ,/* 0x0000006c */
    /* Added wireless lan and WAN ip info for VxWorks. */
    APCMD_GET_WL_IP_CONFIG                              ,/* 0x0000006d */
    APCMD_GET_WL_IP_CONFIG_REPLY                        ,/* 0x0000006e */
    APCMD_SET_WL_IP_CONFIG                              ,/* 0x0000006f */
    APCMD_SET_WL_IP_CONFIG_REPLY                        ,/* 0x00000070 */

    APCMD_GET_WAN_IP_CONFIG                             ,/* 0x00000071 */
    APCMD_GET_WAN_IP_CONFIG_REPLY                       ,/* 0x00000072 */
    APCMD_SET_WAN_IP_CONFIG                             ,/* 0x00000073 */
    APCMD_SET_WAN_IP_CONFIG_REPLY                       ,/* 0x00000074 */
    /* Added wireless lan and WAN ip info for VxWorks. */

    APCMD_GET_ETH_PORT_STATUS                           ,/* 0x00000075 */
    APCMD_GET_ETH_PORT_STATUS_REPLY                     ,/* 0x00000076 */
    APCMD_SET_ETH_PORT_PARAMS                           ,/* 0x00000077 */
    APCMD_SET_ETH_PORT_PARAMS_REPLY                     ,/* 0x00000078 */
    APCMD_GET_ICMP_EN_STATE                             ,/* 0x00000079 */
    APCMD_GET_ICMP_EN_STATE_REPLY                       ,/* 0x0000007a */
    APCMD_SET_ICMP_EN_STATE                             ,/* 0x0000007b */
    APCMD_SET_ICMP_EN_STATE_REPLY                       ,/* 0x0000007c */
    APCMD_GET_802_3_STATES                              ,/* 0x0000007d */
    APCMD_GET_802_3_STATES_REPLY                        ,/* 0x0000007e */
    APCMD_READ_GPIO                                     ,/* 0x0000007f */
    APCMD_READ_GPIO_REPLY                               ,/* 0x00000080 */
    APCMD_WRITE_GPIO                                    ,/* 0x00000081 */
    APCMD_WRITE_GPIO_REPLY                              ,/* 0x00000082 */
    APCMD_SET_ETH_MAC_ADDRESS                           ,/* 0x00000083 */
    APCMD_SET_ETH_MAC_ADDRESS_REPLY                     ,/* 0x00000084 */
    APCMD_GET_WAN_MAC_ADDRESS                           ,/* 0x00000085 */
    APCMD_GET_WAN_MAC_ADDRESS_REPLY                     ,/* 0x00000086 */
    APCMD_SET_WAN_MAC_ADDRESS                           ,/* 0x00000087 */
    APCMD_SET_WAN_MAC_ADDRESS_REPLY                     ,/* 0x00000088 */
    APCMD_FW_SOFT_RESET                                 ,/* 0x00000089 */
    APCMD_FW_SOFT_RESET_REPLY                           ,/* 0x0000008a */
    APCMD_GET_WLAN_FILTER_LIST                          ,/* 0x0000008b */
    APCMD_GET_WLAN_FILTER_LIST_REPLY                    ,/* 0x0000008c */
    APCMD_SET_WLAN_FILTER_LIST                          ,/* 0x0000008d */
    APCMD_SET_WLAN_FILTER_LIST_REPLY                    ,/* 0x0000008e */
    APCMD_GET_WLAN_FILTER_MODE                          ,/* 0x0000008f */
    APCMD_GET_WLAN_FILTER_MODE_REPLY                    ,/* 0x00000090 */
    APCMD_SET_WLAN_FILTER_MODE                          ,/* 0x00000091 */
    APCMD_SET_WLAN_FILTER_MODE_REPLY                    ,/* 0x00000092 */
    APCMD_UPDATE_FIRMWARE                               ,/* 0x00000093 */
    APCMD_UPDATE_FIRMWARE_REPLY                         ,/* 0x00000094 */
    APCMD_SAVE_USER_CONFIG                              ,/* 0x00000095 */
    APCMD_SAVE_USER_CONFIG_REPLY                        ,/* 0x00000096 */
    APCMD_SET_STA_AGING_TIME_IN_MINUTES                 ,/* 0x00000097 */
    APCMD_SET_STA_AGING_TIME_IN_MINUTES_REPLY           ,/* 0x00000098 */
    APCMD_GET_FIRMWARE_VERSION                          ,/* 0x00000099 */
    APCMD_GET_FIRMWARE_VERSION_REPLY                    ,/* 0x0000009a */
    APCMD_SET_BEACON_TX_MODE                            ,/* 0x0000009b */
    APCMD_SET_BEACON_TX_MODE_REPLY                      ,/* 0x0000009c */
    APCMD_SET_RATE_ADAPTATION_METHOD                    ,/* 0x0000009d */
    APCMD_SET_RATE_ADAPTATION_METHOD_REPLY              ,/* 0x0000009e */
    APCMD_GET_RATE_ADAPTATION_PARAMS                    ,/* 0x0000009f */
    APCMD_GET_RATE_ADAPTATION_PARAMS_REPLY              ,/* 0x000000a0 */
    APCMD_SET_RATE_ADAPTATION_PARAMS                    ,/* 0x000000a1 */
    APCMD_SET_RATE_ADAPTATION_PARAMS_REPLY              ,/* 0x000000a2 */
    APCMD_SET_SDRAM_BURST_MODE                          ,/* 0x000000a3 */
    APCMD_SET_SDRAM_BURST_MODE_REPLY                    ,/* 0x000000a4 */
    APCMD_GET_DEVICE_NAME                               ,/* 0x000000a5 */
    APCMD_GET_DEVICE_NAME_REPLY                         ,/* 0x000000a6 */
    APCMD_SET_DEVICE_NAME                               ,/* 0x000000a7 */
    APCMD_SET_DEVICE_NAME_REPLY                         ,/* 0x000000a8 */
    APCMD_SET_WLAN_RADIO                                ,/* 0x000000a9 */
    APCMD_SET_WLAN_RADIO_REPLY                          ,/* 0x000000aa */
    APCMD_GET_STA_ASSOC_STATE                           ,/* 0x000000ab */
    APCMD_GET_STA_ASSOC_STATE_REPLY                     ,/* 0x000000ac */
    APCMD_GET_RSN_CFG_UNICAST_CIPHER                    ,/* 0x000000ad */
    APCMD_GET_RSN_CFG_UNICAST_CIPHER_REPLY              ,/* 0x000000ae */
    APCMD_SET_RSN_CFG_UNICAST_CIPHER                    ,/* 0x000000af */
    APCMD_SET_RSN_CFG_UNICAST_CIPHER_REPLY              ,/* 0x000000b0 */
    APCMD_GUI_SET_PRIVACY_TBL_RSN_ENABLED               ,/* 0x000000b1 */
    APCMD_GUI_SET_PRIVACY_TBL_RSN_ENABLED_REPLY         ,/* 0x000000b2 */
    APCMD_GET_WDS_BCAST_MODE                            ,/* 0x000000b3 */
    APCMD_GET_WDS_BCAST_MODE_REPLY                      ,/* 0x000000b4 */
    APCMD_SET_WDS_BCAST_MODE                            ,/* 0x000000b5 */
    APCMD_SET_WDS_BCAST_MODE_REPLY                      ,/* 0x000000b6 */

/* GET/SET Baseband Reg */
    APCMD_GET_BB_REG                                    ,/* 0x000000b7 */
    APCMD_GET_BB_REG_REPLY                              ,/* 0x000000b8 */
    APCMD_SET_BB_REG                                    ,/* 0x000000b9 */
    APCMD_SET_BB_REG_REPLY                              ,/* 0x000000ba */
/* GET/SET Baseband Reg end */

    APCMD_GET_CUR_RADIO_TX_RATE                         ,/* 0x000000bb */
    APCMD_GET_CUR_RADIO_TX_RATE_REPLY                   ,/* 0x000000bc */
/* GET/SET 11g commands */
    APCMD_GET_BASIC_DATA_RATES_G                        ,/* 0x000000bd */
    APCMD_GET_BASIC_DATA_RATES_G_REPLY                  ,/* 0x000000be */
    APCMD_SET_BASIC_DATA_RATES_G                        ,/* 0x000000bf */
    APCMD_SET_BASIC_DATA_RATES_G_REPLY                  ,/* 0x000000c0 */
    APCMD_GET_TX_DATA_RATE_G                            ,/* 0x000000c1 */
    APCMD_GET_TX_DATA_RATE_G_REPLY                      ,/* 0x000000c2 */
    APCMD_SET_TX_DATA_RATE_G                            ,/* 0x000000c3 */
    APCMD_SET_TX_DATA_RATE_G_REPLY                      ,/* 0x000000c4 */
    APCMD_GET_AP_MODE                                   ,/* 0x000000c5 */
    APCMD_GET_AP_MODE_REPLY                             ,/* 0x000000c6 */
    APCMD_SET_AP_MODE                                   ,/* 0x000000c7 */
    APCMD_SET_AP_MODE_REPLY                             ,/* 0x000000c8 */
    APCMD_GET_STA_LIST_G                                ,/* 0x000000c9 */
    APCMD_GET_STA_LIST_G_REPLY                          ,/* 0x000000ca */
    APCMD_GET_STA_INFO_G                                ,/* 0x000000cb */
    APCMD_GET_STA_INFO_G_REPLY                          ,/* 0x000000cc */
    /*GET/SET 11g commands end */
    APCMD_GET_ETH_CONFIG                                ,/* 0x000000cd */
    APCMD_GET_ETH_CONFIG_REPLY                          ,/* 0x000000ce */
    APCMD_SET_ETH_CONFIG                                ,/* 0x000000cf */
    APCMD_SET_ETH_CONFIG_REPLY                          ,/* 0x000000d0 */
    APCMD_GET_PRIVACY_TBL_RSN_ENABLED                   ,/* 0x000000d1 */
    APCMD_GET_PRIVACY_TBL_RSN_ENABLED_REPLY             ,/* 0x000000d2 */
    APCMD_SET_PRIVACY_TBL_RSN_ENABLED                   ,/* 0x000000d3 */
    APCMD_SET_PRIVACY_TBL_RSN_ENABLED_REPLY             ,/* 0x000000d4 */
    APCMD_GET_RSN_CFG_AUTH_SUITE                        ,/* 0x000000d5 */
    APCMD_GET_RSN_CFG_AUTH_SUITE_REPLY                  ,/* 0x000000d6 */
    APCMD_SET_RSN_CFG_AUTH_SUITE                        ,/* 0x000000d7 */
    APCMD_SET_RSN_CFG_AUTH_SUITE_REPLY                  ,/* 0x000000d8 */
    APCMD_GET_RSN_CFG_UNICAST_CIPHERS                   ,/* 0x000000d9 */
    APCMD_GET_RSN_CFG_UNICAST_CIPHERS_REPLY             ,/* 0x000000da */
    APCMD_SET_RSN_CFG_UNICAST_CIPHERS                   ,/* 0x000000db */
    APCMD_SET_RSN_CFG_UNICAST_CIPHERS_REPLY             ,/* 0x000000dc */
    APCMD_SET_RSN_CFG_MULTICAST_CIPHERS                 ,/* 0x000000dd */
    APCMD_SET_RSN_CFG_MULTICAST_REPLY                   ,/* 0x000000de */
    APCMD_GET_RSN_CFG                                   ,/* 0x000000df */
    APCMD_GET_RSN_CFG_REPLY                             ,/* 0x000000e0 */
    APCMD_SET_PSK_PASSPHRASE                            ,/* 0x000000e1 */
    APCMD_SET_PSK_PASSPHRASE_REPLY                      ,/* 0x000000e2 */
    APCMD_SET_GRP_REKEY_TIME                            ,/* 0x000000e3 */
    APCMD_SET_GRP_REKEY_TIME_REPLY                      ,/* 0x000000e4 */
    APCMD_GET_RSN_STATS                                 ,/* 0x000000e5 */
    APCMD_GET_RSN_STATS_REPLY                           ,/* 0x000000e6 */
                                                        
    APCMD_GET_WDS_MODE                                  ,/* 0x000000e7 */
    APCMD_GET_WDS_MODE_REPLY                            ,/* 0x000000e8 */
    APCMD_SET_WDS_MODE                                  ,/* 0x000000e9 */
    APCMD_SET_WDS_MODE_REPLY                            ,/* 0x000000ea */
    APCMD_GET_WDS_LIST                                  ,/* 0x000000eb */
    APCMD_GET_WDS_LIST_REPLY                            ,/* 0x000000ec */
    APCMD_SET_WDS_LIST                                  ,/* 0x000000ed */
    APCMD_SET_WDS_LIST_REPLY                            ,/* 0x000000ee */
    APCMD_GET_DISABLE_ASSOC                             ,/* 0x000000ef */
    APCMD_GET_DISABLE_ASSOC_REPLY                       ,/* 0x000000f0 */
    APCMD_SET_DISABLE_ASSOC                             ,/* 0x000000f1 */
    APCMD_SET_DISABLE_ASSOC_REPLY                       ,/* 0x000000f2 */
    /* WDS interop option */
    APCMD_GET_WDS_OPTION                                ,/* 0x000000f3 */
    APCMD_GET_WDS_OPTION_REPLY                          ,/* 0x000000f4 */
    APCMD_SET_WDS_OPTION                                ,/* 0x000000f5 */
    APCMD_SET_WDS_OPTION_REPLY                          ,/* 0x000000f6 */

    APCMD_GET_FORCE_PROTECTION_MODE                     ,/* 0x000000f7 */
    APCMD_GET_FORCE_PROTECTION_MODE_REPLY               ,/* 0x000000f8 */
    APCMD_SET_FORCE_PROTECTION_MODE                     ,/* 0x000000f9 */
    APCMD_SET_FORCE_PROTECTION_MODE_REPLY               ,/* 0x000000fa */

    /* WiPod 5-in-1 */
    APCMD_GET_WIPOD_MODE                                ,/* 0x000000fb */
    APCMD_GET_WIPOD_MODE_REPLY                          ,/* 0x000000fc */
    APCMD_SET_WIPOD_MODE                                ,/* 0x000000fd */
    APCMD_SET_WIPOD_MODE_REPLY                          ,/* 0x000000fe */

    APCMD_GET_BURST_MODE                                ,/* 0x000000ff */
    APCMD_GET_BURST_MODE_REPLY                          ,/* 0x00000100 */
    APCMD_SET_BURST_MODE                                ,/* 0x00000101 */
    APCMD_SET_BURST_MODE_REPLY                          ,/* 0x00000102 */

    APCMD_SET_SURVEY_SCAN                               ,/* 0x00000103 */
    APCMD_SET_SURVEY_SCAN_REPLY                         ,/* 0x00000104 */
    APCMD_GET_SURVEY_INFO                               ,/* 0x00000105 */
    APCMD_GET_SURVEY_INFO_REPLY                         ,/* 0x00000106 */
    APCMD_SET_SELF_CTS                                  ,/* 0x00000107 */
    APCMD_SET_SELF_CTS_REPLY                            ,/* 0x00000108 */
    APCMD_GET_SELF_CTS                                  ,/* 0x00000109 */
    APCMD_GET_SELF_CTS_REPLY                            ,/* 0x0000010a */
    APCMD_SET_RF_POWER_MODE                             ,/* 0x0000010b */
    APCMD_SET_RF_POWER_MODE_REPLY                       ,/* 0x0000010c */
    APCMD_SET_AUTO_LINK_MODE                            ,/* 0x0000010d */
    APCMD_SET_AUTO_LINK_MODE_REPLY                      ,/* 0x0000010e */
    APCMD_GET_AUTO_LINK_MODE                            ,/* 0x0000010f */
    APCMD_GET_AUTO_LINK_MODE_REPLY                      ,/* 0x00000110 */
    APCMD_GET_STA_CFG_RSN_OPTION_IMP                    ,/* 0x00000111 */
    APCMD_GET_STA_CFG_RSN_OPTION_IMP_REPLY              ,/* 0x00000112 */
    APCMD_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS            ,/* 0x00000113 */
    APCMD_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS_REPLY      ,/* 0x00000114 */
    APCMD_GET_RSN_CFG_TBL                               ,/* 0x00000115 */
    APCMD_GET_RSN_CFG_TBL_REPLY                         ,/* 0x00000116 */
    APCMD_SET_RSN_CFG_TBL                               ,/* 0x00000117 */
    APCMD_SET_RSN_CFG_TBL_REPLY                         ,/* 0x00000118 */
    APCMD_SET_RSN_DATA_TRAFFIC_ENABLED                  ,/* 0x00000119 */
    APCMD_SET_RSN_DATA_TRAFFIC_ENABLED_REPLY            ,/* 0x0000011a */
    APCMD_SET_RSN_PWK                                   ,/* 0x0000011b */
    APCMD_SET_RSN_PWK_REPLY                             ,/* 0x0000011c */
    APCMD_GET_RSN_PWK                                   ,/* 0x0000011d */
    APCMD_GET_RSN_PWK_REPLY                             ,/* 0x0000011e */
    APCMD_SET_RSN_GRP_KEY                               ,/* 0x0000011f */
    APCMD_SET_RSN_GRP_KEY_REPLY                         ,/* 0x00000120 */
    APCMD_GET_RSN_GRP_KEY                               ,/* 0x00000121 */
    APCMD_GET_RSN_GRP_KEY_REPLY                         ,/* 0x00000122 */
    APCMD_GET_THIS_STA_RSN_IE                           ,/* 0x00000123 */
    APCMD_GET_THIS_STA_RSN_IE_REPLY                     ,/* 0x00000124 */
    APCMD_GET_EXT_STA_RSN_IE                            ,/* 0x00000125 */
    APCMD_GET_EXT_STA_RSN_IE_REPLY                      ,/* 0x00000126 */
    APCMD_GET_PAIRWISE_TSC                              ,/* 0x00000127 */
    APCMD_GET_PAIRWISE_TSC_REPLY                        ,/* 0x00000128 */
    APCMD_SET_PAIRWISE_TSC                              ,/* 0x00000129 */
    APCMD_SET_PAIRWISE_TSC_REPLY                        ,/* 0x0000012a */
    APCMD_GET_GRP_TSC                                   ,/* 0x0000012b */
    APCMD_GET_GRP_TSC_REPLY                             ,/* 0x0000012c */
    APCMD_SET_GRP_TSC                                   ,/* 0x0000012d */
    APCMD_SET_GRP_TSC_REPLY                             ,/* 0x0000012e */
    APCMD_SET_COUNTER_MEASURE_ENABLED                   ,/* 0x0000012f */
    APCMD_SET_COUNTER_MEASURE_ENABLED_REPLY             ,/* 0x00000130 */
    APCMD_SET_RSN_PMK                                   ,/* 0x00000131 */
    APCMD_SET_RSN_PMK_REPLY                             ,/* 0x00000132 */

    /* WPA2 */
    APCMD_GET_WPA2_ENABLED                              ,/* 0x00000133 */
    APCMD_GET_WPA2_ENABLED_REPLY                        ,/* 0x00000134 */
    APCMD_SET_WPA2_ENABLED                              ,/* 0x00000135 */
    APCMD_SET_WPA2_ENABLED_REPLY                        ,/* 0x00000136 */
    APCMD_GET_WPA2ONLY_ENABLED                          ,/* 0x00000137 */
    APCMD_GET_WPA2ONLY_ENABLED_REPLY                    ,/* 0x00000138 */
    APCMD_SET_WPA2ONLY_ENABLED                          ,/* 0x00000139 */
    APCMD_SET_WPA2ONLY_ENABLED_REPLY                    ,/* 0x0000013a */
    APCMD_GET_RSN_CFG_WPA2_TBL                          ,/* 0x0000013b */
    APCMD_GET_RSN_CFG_WPA2_TBL_REPLY                    ,/* 0x0000013c */
    APCMD_SET_RSN_CFG_WPA2_TBL                          ,/* 0x0000013d */
    APCMD_SET_RSN_CFG_WPA2_TBL_REPLY                    ,/* 0x0000013e */
    APCMD_GET_RSN_CFG_WPA2_UNICAST_CIPHER               ,/* 0x0000013f */
    APCMD_GET_RSN_CFG_WPA2_UNICAST_CIPHER_REPLY         ,/* 0x00000140 */
    APCMD_SET_RSN_CFG_WPA2_UNICAST_CIPHER               ,/* 0x00000141 */
    APCMD_SET_RSN_CFG_WPA2_UNICAST_CIPHER_REPLY         ,/* 0x00000142 */
    APCMD_GET_RSN_CFG_WPA2_AUTH_SUITE                   ,/* 0x00000143 */
    APCMD_GET_RSN_CFG_WPA2_AUTH_SUITE_REPLY             ,/* 0x00000144 */
    APCMD_SET_RSN_CFG_WPA2_AUTH_SUITE                   ,/* 0x00000145 */
    APCMD_SET_RSN_CFG_WPA2_AUTH_SUITE_REPLY             ,/* 0x00000146 */
    APCMD_GET_THIS_STA_WPA2_RSN_IE                      ,/* 0x00000147 */
    APCMD_GET_THIS_STA_WPA2_RSN_IE_REPLY                ,/* 0x00000148 */
    APCMD_GET_RSN_CFG_WPA2_UNICAST_CIPHER2              ,/* 0x00000149 */
    APCMD_GET_RSN_CFG_WPA2_UNICAST_CIPHER2_REPLY        ,/* 0x0000014a */
    APCMD_SET_RSN_CFG_WPA2_UNICAST_CIPHER2              ,/* 0x0000014b */
    APCMD_SET_RSN_CFG_WPA2_UNICAST_CIPHER2_REPLY        ,/* 0x0000014c */
    APCMD_SET_WPA2_PREAUTH_ENABLED                      ,/* 0x0000014d */
    APCMD_SET_WPA2_PREAUTH_ENABLED_REPLY                ,/* 0x0000014e */
    APCMD_SET_CLOCK                                     ,/* 0x0000014f */
    APCMD_SET_CLOCK_REPLY                               ,/* 0x00000150 */
    APCMD_GET_CLOCK                                     ,/* 0x00000151 */
    APCMD_GET_CLOCK_REPLY                               ,/* 0x00000152 */
    APCMD_GET_UREPEATER                                 ,/* 0x00000153 */
    APCMD_GET_UREPEATER_REPLY                           ,/* 0x00000154 */
    APCMD_SET_UREPEATER                                 ,/* 0x00000155 */
    APCMD_SET_UREPEATER_REPLY                           ,/* 0x00000156 */
    APCMD_SET_BOOSTER_MODE                              ,/* 0x00000157 */
    APCMD_SET_BOOSTER_MODE_REPLY                        ,/* 0x00000158 */
    APCMD_GET_BOOSTER_MODE                              ,/* 0x00000159 */
    APCMD_GET_BOOSTER_MODE_REPLY                        ,/* 0x0000015a */
    APCMD_SET_SWITCH_IP_PRIO_MAP_EN                     ,/* 0x0000015b */
    APCMD_SET_SWITCH_IP_PRIO_MAP_EN_REPLY               ,/* 0x0000015c */
    APCMD_GET_SWITCH_IP_PRIO_MAP_EN                     ,/* 0x0000015d */
    APCMD_GET_SWITCH_IP_PRIO_MAP_EN_REPLY               ,/* 0x0000015e */
    APCMD_SET_SWITCH_USER_PRIO_MAP_EN                   ,/* 0x0000015f */
    APCMD_SET_SWITCH_USER_PRIO_MAP_EN_REPLY             ,/* 0x00000160 */
    APCMD_GET_SWITCH_USER_PRIO_MAP_EN                   ,/* 0x00000161 */
    APCMD_GET_SWITCH_USER_PRIO_MAP_EN_REPLY             ,/* 0x00000162 */
    APCMD_SET_SWITCH_PRIO_MAP_RULE                      ,/* 0x00000163 */
    APCMD_SET_SWITCH_PRIO_MAP_RULE_REPLY                ,/* 0x00000164 */
    APCMD_GET_SWITCH_PRIO_MAP_RULE                      ,/* 0x00000165 */
    APCMD_GET_SWITCH_PRIO_MAP_RULE_REPLY                ,/* 0x00000166 */
    APCMD_SET_SWITCH_IP_PRIO_MAP                        ,/* 0x00000167 */
    APCMD_SET_SWITCH_IP_PRIO_MAP_REPLY                  ,/* 0x00000168 */
    APCMD_GET_SWITCH_IP_PRIO_MAP                        ,/* 0x00000169 */
    APCMD_GET_SWITCH_IP_PRIO_MAP_REPLY                  ,/* 0x0000016a */
    APCMD_SET_SWITCH_USER_PRIO_MAP                      ,/* 0x0000016b */
    APCMD_SET_SWITCH_USER_PRIO_MAP_REPLY                ,/* 0x0000016c */
    APCMD_GET_SWITCH_USER_PRIO_MAP                      ,/* 0x0000016d */
    APCMD_GET_SWITCH_USER_PRIO_MAP_REPLY                ,/* 0x0000016e */
    APCMD_GET_UREPEATER_LINK                            ,/* 0x0000016f */
    APCMD_GET_UREPEATER_LINK_REPLY                      ,/* 0x00000170 */
    APCMD_GET_AUTH_MODE_G                               ,/* 0x00000171 */
    APCMD_GET_AUTH_MODE_G_REPLY                         ,/* 0x00000172 */
    APCMD_SET_AUTH_MODE_G,
    APCMD_SET_AUTH_MODE_G_REPLY,
    APCMD_GET_PRIVACY_OPTION_G,
    APCMD_GET_PRIVACY_OPTION_G_REPLY,
    APCMD_SET_PRIVACY_OPTION_G,
    APCMD_SET_PRIVACY_OPTION_G_REPLY,
    APCMD_GET_DEFAULT_WEP_KEYS_G,
    APCMD_GET_DEFAULT_WEP_KEYS_G_REPLY,
    APCMD_SET_DEFAULT_WEP_KEYS_G,
    APCMD_SET_DEFAULT_WEP_KEYS_G_REPLY,

/** Hack new for A&g **/
    APCMD_GET_RF_CHANNEL2,
    APCMD_GET_RF_CHANNEL2_REPLY,
    APCMD_SET_RF_CHANNEL2,
    APCMD_SET_RF_CHANNEL2_REPLY,
    APCMD_GET_SSID2 ,
    APCMD_GET_SSID2_REPLY,
    APCMD_SET_SSID2 ,
    APCMD_SET_SSID2_REPLY,

    APCMD_GET_TX_DATA_RATE_A,
    APCMD_GET_TX_DATA_RATE_A_REPLY,
    APCMD_SET_TX_DATA_RATE_A,
    APCMD_SET_TX_DATA_RATE_A_REPLY,
    APCMD_SET_QOS_MODE,
    APCMD_SET_QOS_MODE_REPLY,
    APCMD_GET_QOS_MODE,
    APCMD_GET_QOS_MODE_REPLY,
    APCMD_GET_QOS_TS,
    APCMD_GET_QOS_TS_REPLY,
    APCMD_SET_EDCA_PARAM,
    APCMD_SET_EDCA_PARAM_REPLY,
    APCMD_GET_EDCA_PARAM,
    APCMD_GET_EDCA_PARAM_REPLY,
    APCMD_SET_NOACK_MODE,
    APCMD_SET_NOACK_MODE_REPLY,
    APCMD_GET_NOACK_MODE,
    APCMD_GET_NOACK_MODE_REPLY,
    
    APCMD_GET_ETH_LAN_IP_CONFIG,
    APCMD_GET_ETH_LAN_IP_CONFIG_REPLY,
    APCMD_SET_ETH_LAN_IP_CONFIG,
    APCMD_SET_ETH_LAN_IP_CONFIG_REPLY,
    APCMD_GET_ETH_WAN_IP_CONFIG,
    APCMD_GET_ETH_WAN_IP_CONFIG_REPLY,
    APCMD_SET_ETH_WAN_IP_CONFIG,
    APCMD_SET_ETH_WAN_IP_CONFIG_REPLY,
    APCMD_GET_ETH_DHCPS_CONFIG,
    APCMD_GET_ETH_DHCPS_CONFIG_REPLY,
    APCMD_SET_ETH_DHCPS_CONFIG,
    APCMD_SET_ETH_DHCPS_CONFIG_REPLY,
    APCMD_GET_SPECTRUM_MGMT,
    APCMD_GET_SPECTRUM_MGMT_REPLY,
    APCMD_SET_SPECTRUM_MGMT,
    APCMD_SET_SPECTRUM_MGMT_REPLY,
    APCMD_GET_MITIGATION,
    APCMD_GET_MITIGATION_REPLY,
    APCMD_SET_MITIGATION,
    APCMD_SET_MITIGATION_REPLY,
    APCMD_SEND_SWITCH_CHANNEL_ANNOUNCEMENT,
    APCMD_SEND_SWITCH_CHANNEL_ANNOUNCEMENT_REPLY,
    APCMD_GET_REGULATORY_DOMAIN,
    APCMD_GET_REGULATORY_DOMAIN_REPLY,
    APCMD_SET_REGULATORY_DOMAIN,
    APCMD_SET_REGULATORY_DOMAIN_REPLY,
    APCMD_SET_WPA_PSK_VALUE, 		 
    APCMD_SET_WPA_PSK_VALUE_REPLY, 	 
    APCMD_SET_WPA2_PSK_VALUE, 		 
    APCMD_SET_WPA2_PSK_VALUE_REPLY,
    APCMD_GET_CHANNEL_SPACING,
    APCMD_GET_CHANNEL_SPACING_REPLY,
    APCMD_SET_CHANNEL_SPACING,
    APCMD_SET_CHANNEL_SPACING_REPLY,

    APCMD_SET_RSN_CFG_WPA2_MULTICAST_CIPHERS,
    APCMD_SET_RSN_CFG_WPA2_MULTICAST_CIPHERS_REPLY,
/* GET/SET RF Reg */
    APCMD_GET_RF_REG                                    ,
    APCMD_GET_RF_REG_REPLY                              ,
    APCMD_SET_RF_REG                                    ,
    APCMD_SET_RF_REG_REPLY                              ,
/* GET/SET RF Reg end */

    /*AutoLink Commands*/
    APCMD_GET_PSK_PASSPHRASE,    
    APCMD_GET_PSK_PASSPHRASE_REPLY,

    APCMD_GET_AL_LOW_POWER_STATE,
    APCMD_GET_AL_LOW_POWER_STATE_REPLY,
    APCMD_SET_AL_LOW_POWER_STATE,
    APCMD_SET_AL_LOW_POWER_STATE_REPLY,

    APCMD_GET_AL_ENABLE_STATE,
    APCMD_GET_AL_ENABLE_STATE_REPLY,
    APCMD_SET_AL_ENABLE_STATE,
    APCMD_SET_AL_ENABLE_STATE_REPLY,

    APCMD_GET_AL_ON_TIME,
    APCMD_GET_AL_ON_TIME_REPLY,
    APCMD_SET_AL_ON_TIME,
    APCMD_SET_AL_ON_TIME_REPLY,

    APCMD_GET_AL_PP_ENABLE_STATE,
    APCMD_GET_AL_PP_ENABLE_STATE_REPLY,
    APCMD_SET_AL_PP_ENABLE_STATE,
    APCMD_SET_AL_PP_ENABLE_STATE_REPLY,

    APCMD_GET_AL_SEC_MODE,
    APCMD_GET_AL_SEC_MODE_REPLY,
    APCMD_SET_AL_SEC_MODE,
    APCMD_SET_AL_SEC_MODE_REPLY,

    /*DCS mode commands*/
    APCMD_SET_DCS_MODE,
    APCMD_SET_DCS_MODE_REPLY,
    APCMD_GET_DCS_MODE,
    APCMD_GET_DCS_MODE_REPLY,
    APCMD_SET_BKGND_SCAN,
    APCMD_SET_BKGND_SCAN_REPLY,
    APCMD_GET_BGSCAN_DATA,
    APCMD_GET_BGSCAN_DATA_REPLY,
    APCMD_GET_CHWEIGHT_DATA,
    APCMD_GET_CHWEIGHT_DATA_REPLY,
    APCMD_SET_DCS_CHANNEL,
    APCMD_SET_DCS_CHANNEL_REPLY,

/* GUEST ACCESS */
    APCMD_GET_GUEST_ACCESS,
    APCMD_GET_GUEST_ACCESS_REPLY,
    APCMD_SET_GUEST_ACCESS,
    APCMD_SET_GUEST_ACCESS_REPLY,

    /* Alway add items above this line */
    /* please do not add #ifdef inside of this enum */
    MAX_APCMD
} APCMD;
#if defined(UDP_API)
    #define APICMDHEADER   unsigned short cmdCode; \
                       unsigned short seqNum; \
                       unsigned short cmdStatus; \
                       unsigned short cmdBodyLen; \
                       unsigned int *cmdBody
#else
    #define APICMDHEADER   unsigned short cmdCode; \
                       unsigned short seqNum; \
                       unsigned short cmdStatus; \
                       unsigned short cmdBodyLen
#define APICMDHEADERLEN  sizeof(unsigned short)*4
#endif
#define SIZEOF_APICMDHEADER 12


typedef enum
{
    APCTL_OK = 1,
    APCTL_BUF_TOO_SMALL,
    APCTL_BAD_CMD,
    APCTL_CMD_FAILED,
    APCTL_TIMEOUT,
    APCTL_ERROR,
}APCMD_STATUS;

typedef struct _APICMDBUF
{
    UINT16 cmdCode;
    UINT16 seqNum;
    UINT16 cmdStatus;
    UINT16 cmdBodyLen;
    UINT8 *cmdBody;
}
APICMDBUF;

#define apio_handle_t UINT32
#define APIO_NOERROR 1
#define APIO_ERROR  0
#define apio_ErrCode_t UINT8

typedef struct apio_bufdescr_t
{
    UINT16 frameLen;  //Length of the frame
    UINT16 reserved;  //reserved field
    UINT8 *framePtr;  //Pointer to the frame
}
apio_bufdescr_t;

extern WL_STATUS apctl(APICMDBUF * apicmdbuf);
extern apio_ErrCode_t apio_open(char *pName, apio_handle_t *pHandle, void (*) (apio_bufdescr_t *));
extern apio_ErrCode_t apio_close(apio_handle_t);
extern apio_ErrCode_t apio_read(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_write(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_alloc(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_free(apio_handle_t, apio_bufdescr_t *pBufDescr);
extern UINT32 apio_select(void);

typedef enum
{
    OPEN_SYSTEM_ONLY = 1,
    SHARED_KEY_ONLY,
    OPEN_SYSTEM_AND_SHARED_KEY,
    NO_WEP,
}AUTH_MODE;

typedef enum
{
    APIO_POLL_RADIO_DATA_RDY = 1,
    APIO_POLL_ETH_DATA_RDY = 2,    /* Ethernet port has received data */
    APIO_POLL_WAN_DATA_RDY = (1 << 7),


}POLL_DATA;

typedef enum
{
    AP_MODE_B_ONLY,
    AP_MODE_G_ONLY,
    AP_MODE_MIXED,
    AP_MODE_A_ONLY,
    AP_MODE_AandG,
    
}AP_RATE_MODE;

typedef enum
{
	CLIENT_MODE_B,
	CLIENT_MODE_G,
	CLIENT_MODE_BG,
	CLIENT_MODE_A,
    CLIENT_MODE_AUTO,
}CLIENT_MODE;

#if defined(GATEWAY)
typedef struct _APICMDBUF_GET_ETH_LAN_IP_CONFIG
{
    APICMDHEADER;
    unsigned long ipAddress;
    unsigned long subnetMask;
}
APICMDBUF_GET_ETH_LAN_IP_CONFIG;

typedef struct _APICMDBUF_SET_ETH_LAN_IP_CONFIG
{
    APICMDHEADER;
    unsigned long ipAddress;
    unsigned long subnetMask;
}
APICMDBUF_SET_ETH_LAN_IP_CONFIG;

typedef struct _APICMDBUF_GET_ETH_WAN_IP_CONFIG
{
    APICMDHEADER;
    unsigned long dhcpEnabled;
    unsigned long ipAddress;
    unsigned long subnetMask;
    unsigned long gatewayAddress;
    unsigned long primaryDNSAddress;
    unsigned long secondaryDNSAddress;
}
APICMDBUF_GET_ETH_WAN_IP_CONFIG;

typedef struct _APICMDBUF_SET_ETH_WAN_IP_CONFIG
{
    APICMDHEADER;
    unsigned long dhcpEnabled;
    unsigned long ipAddress;
    unsigned long subnetMask;
    unsigned long gatewayAddress;
    unsigned long primaryDNSAddress;
    unsigned long secondaryDNSAddress;
}
APICMDBUF_SET_ETH_WAN_IP_CONFIG;
#endif

#if defined(GATEWAY) && defined(DHCPS)
#include "dhcps_api.h"
typedef struct _APICMDBUF_SET_ETH_DHCPS_CONFIG
{
    APICMDHEADER;
    unsigned long isEnable;
    unsigned long rangeStartAddr;
    unsigned long rangeEndAddr;
    unsigned long leaseTime;
    DHCPS_STATION_PROPERTIES station;
}

APICMDBUF_SET_ETH_DHCPS_CONFIG;

typedef struct _APICMDBUF_GET_ETH_DHCPS_CONFIG
{
    APICMDHEADER;
    unsigned long isEnable;
    unsigned long rangeStartAddr;
    unsigned long rangeEndAddr;
    unsigned long leaseTime;
    unsigned long numOfStations;
    DHCPS_STATION_PROPERTIES stationsTbl[DHCPS_MAX_NUM_OF_STATIONS];
}

APICMDBUF_GET_ETH_DHCPS_CONFIG;

#endif
typedef struct _APICMDBUF_GET_RF_CHANNEL
{
    APICMDHEADER;
    unsigned short rfModulationType;
    char	availChanList[IEEEtypes_MAX_CHANNELS+IEEEtypes_MAX_CHANNELS_A];
#ifdef AUTOCHANNEL
    unsigned short curChanNum : 15;
	unsigned short isAutoChan : 1;
#else
    unsigned short curChanNum;
#endif
}
APICMDBUF_GET_RF_CHANNEL;

typedef struct _APICMDBUF_GET_OPEN_SSID_EN_STATE
{
    APICMDHEADER;
    unsigned char openSSIDEnabled;
}
APICMDBUF_GET_OPEN_SSID_EN_STATE;

typedef struct _APICMDBUF_SET_OPEN_SSID_EN_STATE
{
    APICMDHEADER;
    unsigned char openSSIDEnabled;
}
APICMDBUF_SET_OPEN_SSID_EN_STATE;


typedef struct _APICMDBUF_SET_RF_CHANNEL
{
    APICMDHEADER;
    unsigned char curChanNum;

}
APICMDBUF_SET_RF_CHANNEL;

typedef struct _APICMDBUF_GET_RF_MACADDR
{
    APICMDHEADER;
    unsigned char physAddr[6];
}
APICMDBUF_GET_RF_MACADDR;

typedef struct _APICMDBUF_SET_RF_MACADDR
{
    APICMDHEADER;
    unsigned char physAddr[6];
}
APICMDBUF_SET_RF_MACADDR;

typedef struct _APICMDBUF_GET_BASIC_DATA_RATES
{
    APICMDHEADER;
    unsigned char basicDataRates[8];
}
APICMDBUF_GET_BASIC_DATA_RATES;

typedef struct _APICMDBUF_SET_BASIC_DATA_RATES
{
    APICMDHEADER;
    unsigned char basicDataRates[8];
}
APICMDBUF_SET_BASIC_DATA_RATES;

typedef struct _APICMDBUF_GET_SUP_DATA_RATE
{
    APICMDHEADER;
    unsigned char txDataRate[14];
}
APICMDBUF_GET_SUP_DATA_RATE;

typedef struct _APICMDBUF_SET_SUP_DATA_RATE
{
    APICMDHEADER;
    unsigned char txDataRate[14];
}
APICMDBUF_SET_SUP_DATA_RATE;

typedef struct _APICMDBUF_GET_TX_DATA_RATE
{
    APICMDHEADER;
    unsigned char txDataRate;
}
APICMDBUF_GET_TX_DATA_RATE;

typedef struct _APICMDBUF_SET_TX_DATA_RATE
{
    APICMDHEADER;
    unsigned char txDataRate;
}
APICMDBUF_SET_TX_DATA_RATE;

typedef struct _APICMDBUF_GET_TX_POWER_LEVEL
{
    APICMDHEADER;
    unsigned char txPowerLevel;
    unsigned char powerLevelList[8];
}
APICMDBUF_GET_TX_POWER_LEVEL;

typedef struct _APICMDBUF_SET_TX_POWER_LEVEL
{
    APICMDHEADER;
    unsigned char txPowerLevel;
}
APICMDBUF_SET_TX_POWER_LEVEL;

typedef struct _APICMBUF_GET_RF_PREAMBLE_OPTION
{
    APICMDHEADER;
    unsigned char rfPreambleOption;
}
APICMBUF_GET_RF_PREAMBLE_OPTION;

typedef struct _APICMBUF_SET_RF_PREAMBLE_OPTION
{
    APICMDHEADER;
    unsigned char rfPreambleOption;
}
APICMBUF_SET_RF_PREAMBLE_OPTION;

typedef enum
{
    PREAMBLE_SHORT,
    PREAMBLE_LONG,
    PREAMBLE_AUTO_SELECT,
} PREAMBLE_TYPE;

typedef struct _APICMDBUF_GET_ANTENNA_MODE
{
    APICMDHEADER;
    unsigned char rxAntennaMode;
    unsigned char txAntennaMode;
    unsigned char whichAnt;
}
APICMDBUF_GET_ANTENNA_MODE;

typedef struct _APICMDBUF_SET_ANTENNA_MODE
{
    APICMDHEADER;
    unsigned char whichAnt;
    unsigned char antennaMode;
}
APICMDBUF_SET_ANTENNA_MODE;

typedef enum
{
    AP_ANT_A,
    AP_ANT_B,
    AP_ANT_DIVERSITY,
} ANTENNA_TYPE;

typedef enum
{
    AP_RX_ANTENNA,
    AP_TX_ANTENNA,
    AP_BOTH_ANTENNA,
} ANTENNA_MODE_TYPE;


typedef struct _APICMDBUF_GET_FRAG_THRESHOLD
{
    APICMDHEADER;
    unsigned short fragThreshold;
}
APICMDBUF_GET_FRAG_THRESHOLD;

typedef struct _APICMDBUF_SET_FRAG_THRESHOLD
{
    APICMDHEADER;
    unsigned short fragThreshold;
}
APICMDBUF_SET_FRAG_THRESHOLD;

typedef struct _APICMDBUF_GET_RTS_THRESHOLD
{
    APICMDHEADER;
    unsigned short rtsThreshold;
}
APICMDBUF_GET_RTS_THRESHOLD;

typedef struct _APICMDBUF_SET_RTS_THRESHOLD
{
    APICMDHEADER;
    unsigned short rtsThreshold;
}
APICMDBUF_SET_RTS_THRESHOLD;


typedef struct _APICMDBUF_GET_RTS_RETRY_LIMIT
{
    APICMDHEADER;
    unsigned short rtsRetryLimit;
}
APICMDBUF_GET_RTS_RETRY_LIMIT;

typedef struct _APICMDBUF_SET_RTS_RETRY_LIMIT
{
    APICMDHEADER;
    unsigned short rtsRetryLimit;
}
APICMDBUF_SET_RTS_RETRY_LIMIT;

typedef struct _APICMDBUF_GET_DATA_RETRY_LIMIT
{
    APICMDHEADER;
    unsigned short dataRetryLimit;
}
APICMDBUF_GET_DATA_RETRY_LIMIT;

typedef struct _APICMDBUF_SET_DATA_RETRY_LIMIT
{
    APICMDHEADER;
    unsigned short dataRetryLimit;
}
APICMDBUF_SET_DATA_RETRY_LIMIT;

typedef struct _APICMDBUF_GET_BEACON_PERIOD
{
    APICMDHEADER;
    unsigned long beaconPeriod;
}
APICMDBUF_GET_BEACON_PERIOD;

typedef struct _APICMDBUF_SET_BEACON_PERIOD
{
    APICMDHEADER;
    unsigned long beaconPeriod;
}
APICMDBUF_SET_BEACON_PERIOD;

typedef struct _APICMDBUF_GET_DTIM_PERIOD
{
    APICMDHEADER;
    unsigned long dtimPeriod;
}
APICMDBUF_GET_DTIM_PERIOD;


typedef struct _APICMDBUF_SET_DTIM_PERIOD
{
    APICMDHEADER;
    unsigned long dtimPeriod;
}
APICMDBUF_SET_DTIM_PERIOD;

typedef struct _APICMDBUF_GET_802_11_STATS
{
    APICMDHEADER;
    unsigned long totaltxBytes;
    unsigned long cntMcastPackets;
    unsigned long cntUcastPackets;
    unsigned long cntTxBad;
    unsigned long cntRetry;
    unsigned long cntMultRetry;
    unsigned long rtsOKCnt;
    unsigned long rtsBadCnt;
    unsigned long ackBadCnt;
    unsigned long totalRxBytes;
    unsigned long rxMcastOkCnt;
    unsigned long rxUcastOkCnt;
    unsigned long fcsErrorCnt;
    unsigned long duplicateCnt;
    unsigned long wepDecryptBadCnt;
}
APICMDBUF_GET_802_11_STATS;

typedef struct _APICMDBUF_GET_ETH_MAC_ADDRESS
{
    APICMDHEADER;
    unsigned char phyAddress[6];
}
APICMDBUF_GET_ETH_MAC_ADDRESS;

typedef struct _APICMDBUF_SET_ETH_MAC_ADDRESS
{
    APICMDHEADER;
    unsigned char phyAddress[6];
}
APICMDBUF_SET_ETH_MAC_ADDRESS;

typedef struct _APICMDBUF_GET_WAN_MAC_ADDRESS
{
    APICMDHEADER;
    unsigned char phyAddress[6];
}
APICMDBUF_GET_WAN_MAC_ADDRESS;

typedef struct _APICMDBUF_SET_WAN_MAC_ADDRESS
{
    APICMDHEADER;
    unsigned char phyAddress[6];
}
APICMDBUF_SET_WAN_MAC_ADDRESS;

typedef struct _APICMDBUF_GET_ETH_IP_CONFIG
{
    APICMDHEADER;
    unsigned long dhcpEnabled;
    unsigned long ipAddress;
    unsigned long subnetMask;
    unsigned long gatewayAddress;
}
APICMDBUF_GET_ETH_IP_CONFIG;

typedef struct _APICMDBUF_SET_ETH_IP_CONFIG
{
    APICMDHEADER;
    unsigned long dhcpEnabled;
    unsigned long ipAddress;
    unsigned long subnetMask;
    unsigned long gatewayAddress;
}
APICMDBUF_SET_ETH_IP_CONFIG;

typedef struct _APICMDBUF_GET_ETH_PORT_STATUS
{
    APICMDHEADER;
    unsigned char portnumber;
    unsigned char ethOperStatus;
    unsigned char ethDuplex;
    unsigned char ethLinkSpeed;
}
APICMDBUF_GET_ETH_PORT_STATUS;

typedef enum
{
    ETH_OPER_STATUS_UP,
    ETH_OPER_STATUS_DOWN,
}ETHER_PORTSTATUS;


typedef struct _APICMDBUF_SET_ETH_PORT_PARAMS
{
    APICMDHEADER;
    unsigned short linkSpeed;
    unsigned short ethDuplex;
}
APICMDBUF_SET_ETH_PORT_PARAMS;

typedef struct _APICMDBUF_GET_ICMP_EN_STATE
{
    APICMDHEADER;
    unsigned char icmpEnabled;
}
APICMDBUF_GET_ICMP_EN_STATE;

typedef struct _APICMDBUF_SET_ICMP_EN_STATE
{
    APICMDHEADER;
    unsigned char icmpEnabled;
}
APICMDBUF_SET_ICMP_EN_STATE;


typedef struct _APICMDBUF_GET_802_3_STATS
{
    APICMDHEADER;
    unsigned long clearStats;
    unsigned long totalTxBytes;
    unsigned long cntMcastPackets;
    unsigned long cntUcastPackets;
    unsigned long cntTxBad;
    unsigned long totalRxBytes;
    unsigned long rxBadCnt;
    unsigned long rxMcastOKCnt;
    unsigned long rxUcastOKCnt;
    unsigned long crcErrorCnt;
}
APICMDBUF_GET_802_3_STATS;


typedef struct _APICMDBUF_GET_STA_INFO
{
    APICMDHEADER;
    unsigned char staMACAddress[6];
    unsigned char staIPAddress;
    unsigned char staPowerSaveMode;
    unsigned char staWEPStatus;
    UINT32 staRSSI;
    UINT32 staSQ;
}
APICMDBUF_GET_STA_INFO;

typedef struct _APICMDBUF_DEAUTH_STA
{
    APICMDHEADER;
    unsigned char staMACAddress[6];
}
APICMDBUF_DEAUTH_STA;

typedef struct _APICMDBUF_DISASSOC_STA
{
    APICMDHEADER;
    unsigned char staMACAddress[6];
}
APICMDBUF_DISASSOC_STA;

/* SECURITY COMMANDS */
typedef struct _APICMDBUF_GET_DEFAULT_WEP_KEYS
{
    APICMDHEADER;
    unsigned short keyindex; /*cntWEPKEYS; */
    unsigned short keyType0;
    unsigned char keyValue0[16];
    unsigned short keyType1;
    unsigned char keyValue1[16];
    unsigned short keyType2;
    unsigned char keyValue2[16];
    unsigned short keyType3;
    unsigned char keyValue3[16];
}
APICMDBUF_GET_DEFAULT_WEP_KEYS;

typedef struct _APICMDBUF_SET_DEFAULT_WEP_KEYS
{
    APICMDHEADER;
    unsigned short keyindex;
    unsigned short keyType0;
    unsigned char keyValue0[16];
    unsigned short keyType1;
    unsigned char keyValue1[16];
    unsigned short keyType2;
    unsigned char keyValue2[16];
    unsigned short keyType3;
    unsigned char keyValue3[16];
}
APICMDBUF_SET_DEFAULT_WEP_KEYS;

typedef struct _APICMDBUF_SET_STA_WEP_KEY
{
    APICMDHEADER;
    unsigned char staMACAddress[6];
    unsigned short keyType;
    unsigned short KeyValue[16];
}
APICMDBUF_SET_STA_WEP_KEY;

typedef enum
{
    WEP_NOT_SET,
    WEP_40_BIT,
    WEP_104_BIT,
} WEP_KEY_TYPE;

typedef struct _APICMDBUF_GET_PRIVACY_OPTION
{
    APICMDHEADER;
    unsigned char dataEncrypt;
}
APICMDBUF_GET_PRIVACY_OPTION;

typedef struct _APICMDBUF_SET_PRIVACY_OPTION
{
    APICMDHEADER;
    unsigned char dataEncrypt;
}
APICMDBUF_SET_PRIVACY_OPTION;

typedef enum
{
    DATA_ENCRYPT_NOT_REQUIRED,
    DATA_ENCRYPT_OPTIONA,
    FULL_DATA_ENCRYPT_REQUIRED,
} PRIVACY_TYPE;

typedef struct _APICMDBUF_GET_AUTH_MODE
{
    APICMDHEADER;
    unsigned char authMode;
}
APICMDBUF_GET_AUTH_MODE;

typedef struct _APICMDBUF_SET_AUTH_MODE
{
    APICMDHEADER;
    unsigned char authMode;
}
APICMDBUF_SET_AUTH_MODE;

/*AP SYSTEM COMMANDS */
typedef struct _APICMDBUF_GET_API_VER
{
    APICMDHEADER;
    unsigned char apAPIVer[32];
}
APICMDBUF_GET_API_VER;

typedef struct _APICMDBUF_GET_AP_STATUS
{
    APICMDHEADER;
    unsigned short macStatus;
    unsigned short rfPortStatus;
    unsigned short ethPortStatus;
    unsigned short curChannel;
    unsigned char apname[32];
    unsigned char macAddress[6];
    unsigned short cntAssocSta;
    unsigned long maxLinkSpeed;
    unsigned char apFwVer[32];
    unsigned long apFwChecksum;
}
APICMDBUF_GET_AP_STATUS;

typedef enum
{
    OPER_STATUS_UP,
    OPER_STATUS_DOWN,
} AP_STATUS_TYPE;

typedef struct _APICMDBUF_GET_EVENT_LOG
{
    APICMDHEADER;
    unsigned char eventlog;
}
APICMDBUF_GET_EVENT_LOG;

typedef struct _APICMDBUF_PURGE_EVENT_LOG
{
    APICMDHEADER;
}
APICMDBUF_PURGE_EVENT_LOG;

typedef struct _APICMDBUF_UPDATE_FIRMWARE
{
    APICMDHEADER;
    unsigned char *imgPtr;
    unsigned char apFwVer[32];
    unsigned long apChecksum;
}
APICMDBUF_UPDATE_FIRMWARE;

typedef enum
{
    FW_UPDATE_OK,
    FW_UPDATE_FAIL_FLASH,
    FW_UPDATE_FAIL_CHECKUSM,
    FW_UPDATE_SAME_VER,
} UPDATE_FIRMWARE_TYPE;

typedef struct _APICMDBUF_RUN_DIAG
{
    APICMDHEADER;
    unsigned short testNum;
    unsigned short testResult;
}
APICMDBUF_RUN_DIAG;

typedef enum
{
    TEST_PASS,
    TEST_FAIL,
    TEST_INCONCLUSIVE,
} DIAG_TYPE;

typedef struct _APICMDBUF_START_LINK_TEST
{
    APICMDHEADER;
    unsigned char staMACAdress[6];
    unsigned long cntTxPacketsOK;
    unsigned long totalErrors;
    unsigned long totalWEPErrors;
}
APICMDBUF_START_LINK_TEST;

typedef struct _APICMDBUF_READ_GPIO
{
    APICMDHEADER;
    unsigned char gpioNum;
    unsigned char gpioVal;
}
APICMDBUF_READ_GPIO;

typedef struct _APICMDBUF_WRITE_GPIO
{
    APICMDHEADER;
    unsigned char gpioNum;
    unsigned char gpioVal;
}
APICMDBUF_WRITE_GPIO;

typedef struct _APICMDBUF_SET_STA_AGING_TIME
{
    APICMDHEADER;
    unsigned short timeInMinutes;

}
APICMDBUF_SET_STA_AGING_TIME;


typedef struct _APICMDBUF_SET_WLAN_RADIO
{
    APICMDHEADER;
    unsigned char radio;
}
APICMDBUF_SET_WLAN_RADIO;


typedef enum{
    APEVT_STA_AUTHENTICATED,
    APEVT_STA_ASSOCIATED,
    APEVT_STA_DEAUTHENTICATED,
    APEVT_STA_DISASSOCIATED,
    APEVT_STA_AUTHENTICATE_FAIL,
}STA_ASSOC_TYPE;

typedef enum{
    STA_8021X_NOT_AUTHENTICATED,
    STA_8021X_AUTHENTICATED,
} STA_8021X_STATUS;

/** use for mac access list **/
typedef enum{
    ACCESS_MODE,
    FILTER_MODE,
    DISABLE_MODE,
}FILTER_TYPE;

typedef struct _APICMDBUF_GET_BASIC_DATA_RATES_G
{
    APICMDHEADER;
    unsigned char basicDataRates[16];
}
APICMDBUF_GET_BASIC_DATA_RATES_G;

typedef struct _APICMDBUF_SET_BASIC_DATA_RATES_G
{
    APICMDHEADER;
    unsigned char basicDataRates[16];
}
APICMDBUF_SET_BASIC_DATA_RATES_G;

typedef struct _APICMDBUF_GET_AP_MODE
{
    APICMDHEADER;
    UINT32 apMode;
}
APICMDBUF_GET_AP_MODE;

typedef struct _APICMDBUF_SET_AP_MODE
{
    APICMDHEADER;
    UINT32 apMode;
}
APICMDBUF_SET_AP_MODE;


#define SDRAM_BURST_REG (W81_SDRAM_CFG_BASE+0x14)
#define SDRAM_BURST_1   0
#define SDRAM_BURST_2   1
#define SDRAM_BURST_4   2
#define SDRAM_BURST_8   3
typedef struct _APICMDBUF_SDRAM_BURST_MODE
{
    APICMDHEADER;
    UINT8 sdramBurstMode;

}
APICMDBUF_SDRAM_BURST_MODE;
typedef enum
{
    FILTER_MODE_ALLOW,
    FILTER_MODE_BLOCK,
    FILTER_MODE_DISABLE,
}FILTER_MODE_TYPE;

typedef struct _APICMDBUF_FILTER_ENTRY
{
    APICMDHEADER;
    UINT8  macAddr[6];
}
APICMDBUF_FILTER_ENTRY;
typedef struct _APICMDBUF_GET_FILTER_MODE
{
    APICMDHEADER;
    unsigned char  mode; /* use FILTER_MODE_TYPE enum */
}
APICMDBUF_GET_FILTER_MODE;

typedef struct _APICMDBUF_SET_FILTER_MODE
{
    APICMDHEADER;
    unsigned char  mode; /* use FILTER_MODE_TYPE enum */
}
APICMDBUF_SET_FILTER_MODE;

typedef struct _APICMDBUF_FILTER_MODE
{
    APICMDHEADER;
    unsigned char  mode; 
}
APICMDBUF_FILTER_MODE;

typedef struct _APICMDBUF_SET_BURST_MODE
{
    APICMDHEADER;
    UINT8 BurstMode;
    UINT8 BurstRate;
}
APICMDBUF_SET_BURST_MODE;

typedef struct _APICMDBUF_GET_BURST_MODE
{
    APICMDHEADER;
    UINT8 BurstMode;
    UINT8 BurstRate;
}
APICMDBUF_GET_BURST_MODE;
#ifdef WPA

/**************** 802.11i MIB supported *********************/
typedef struct _APICMDBUF_GET_STA_CFG_RSN_OPTION_IMP
{
    APICMDHEADER;
    unsigned char RSN_Capable;
}
APICMDBUF_GET_STA_CFG_RSN_OPTION_IMP;

typedef struct _APICMDBUF_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS
{
    APICMDHEADER;
    unsigned long NumOfCounters;
}
APICMDBUF_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS;

typedef struct _APICMDBUF_GET_PRIVACY_TBL_RSN_ENABLED
{
    APICMDHEADER;
    unsigned long Enabled;
}
APICMDBUF_GET_PRIVACY_TBL_RSN_ENABLED;

typedef struct _APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED
{
    APICMDHEADER;
    unsigned long Enabled;
}
APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED;

typedef struct _APICMDBUF_GET_RSN_CFG_TBL
{
    APICMDHEADER;
    unsigned long Version;
    unsigned long PairwiseKeysSupported;
    unsigned char MulticastCipher[4];
    unsigned char GroupRekeyMethod;
    unsigned long GroupRekeyTime;
    unsigned long GroupRekeyPackets;
    unsigned char GroupRekeyStrict;
    //unsigned char  PSKValue[32];  // write only
#ifdef AUTOLINK
    unsigned char  PSKPassPhrase[64]; // write only
#endif
    unsigned char TSNEnabled;
    unsigned long GroupMasterRekeyTime;
    unsigned long GroupUpdateTimeOut;
    unsigned long GroupUpdateCount;
    unsigned long PairwiseUpdateTimeOut;
    unsigned long PairwiseUpdateCount;
}
APICMDBUF_GET_RSN_CFG_TBL;
typedef struct _APICMDBUF_GET_CLOCK
{
    APICMDHEADER;
    unsigned short cpuFreq;  /*MHz*/
    unsigned short sysFreq;  /*MHz*/
}
APICMDBUF_GET_CLOCK;
typedef struct _APICMDBUF_SET_CLOCK
{
    APICMDHEADER;
    unsigned short cpuFreq;  /*MHz*/
    unsigned short sysFreq;  /*MHz*/
}
APICMDBUF_SET_CLOCK;

typedef struct _APICMDBUF_SET_RSN_CFG_TBL
{
    APICMDHEADER;
    //unsigned long  Version;   // read only
    unsigned long PairwiseKeysSupported;
    unsigned char MulticastCipher[4];
    unsigned char GroupRekeyMethod;
    unsigned long GroupRekeyTime;
    unsigned long GroupRekeyPackets;
    unsigned char GroupRekeyStrict;
    unsigned char PSKValue[32]; //write only
    unsigned char PSKPassPhrase[64]; // write only
    unsigned char TSNEnabled;
    unsigned long GroupMasterRekeyTime;
    unsigned long GroupUpdateTimeOut;
    unsigned long GroupUpdateCount;
    unsigned long PairwiseUpdateTimeOut;
    unsigned long PairwiseUpdateCount;
}
APICMDBUF_SET_RSN_CFG_TBL;

typedef struct _APICMDBUF_GET_RSN_CFG_UNICAST_CIPHER
{
    APICMDHEADER;
    unsigned char UnicastCipher[4];
    unsigned char Enabled;
}
APICMDBUF_GET_RSN_CFG_UNICAST_CIPHER;

typedef struct _APICMDBUF_SET_RSN_CFG_UNICAST_CIPHER
{
    APICMDHEADER;
    unsigned char UnicastCipher[4];
    unsigned char Enabled;
}
APICMDBUF_SET_RSN_CFG_UNICAST_CIPHER;

typedef struct _APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS
{
    APICMDHEADER;
    unsigned char MulticastCipher[4];
}
APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS;

typedef struct _APICMDBUF_SET_RSN_CFG_WPA2_MULTICAST_CIPHERS
{
    APICMDHEADER;
    unsigned char MulticastCipher[4];
}
APICMDBUF_SET_RSN_CFG_WPA2_MULTICAST_CIPHERS;

typedef struct _APICMDBUF_SET_PSK_PASSPHRASE
{
    APICMDHEADER;
    unsigned char passphrase[64];
}
APICMDBUF_SET_PSK_PASSPHRASE;

typedef struct _APICMDBUF_SET_GRP_REKEY_TIME
{
    APICMDHEADER;
    unsigned long rekey_time;
}
APICMDBUF_SET_GRP_REKEY_TIME;

typedef struct _APICMDBUF_GET_RSN_CFG_AUTH_SUITE
{
    APICMDHEADER;
    unsigned char AuthSuites[4];
    unsigned char Enabled;
}
APICMDBUF_GET_RSN_CFG_AUTH_SUITE;

typedef struct _APICMDBUF_SET_RSN_CFG_AUTH_SUITE
{
    APICMDHEADER;
    unsigned char AuthSuites[4];
    unsigned char Enabled;
}
APICMDBUF_SET_RSN_CFG_AUTH_SUITE;

typedef struct _APICMDBUF_GET_RSN_STATS
{
    APICMDHEADER;
    unsigned char MacAddr[6];
    unsigned char Version;
    unsigned char SelectedUnicastCipher[4];
    unsigned long TKIPICVErrors;
    unsigned long TKIPLocalMICFailures;
    unsigned long TKIPRemoteMICFailures;
    unsigned long TKIPCounterMeasuresInvoked;
    unsigned long WRAPFormatErrors;
    unsigned long WRAPReplays;
    unsigned long WRAPDecryptErrors;
    unsigned long CCMPFormatErrors;
    unsigned long CCMPReplays;
    unsigned long CCMPDecryptErrors;
}
APICMDBUF_GET_RSN_STATS;

typedef struct _APICMDBUF_GET_UREPEATER
{
    APICMDHEADER;
    unsigned char urMode; /* 1=enable; else=disable */
    unsigned char urSsid[32];
    unsigned short urRsvd;
    unsigned char urBssid[6];
    unsigned char urPrefBssid;
    unsigned char wbMode;
}
APICMDBUF_GET_UREPEATER;

typedef struct _APICMDBUF_SET_UREPEATER
{
    APICMDHEADER;
    unsigned char urMode; /* 1=enable; else=disable */
    unsigned char urSsid[32];
    unsigned short urRsvd;
    unsigned char urBssid[6];
    unsigned char urPrefBssid;
    unsigned char wbMode;
}
APICMDBUF_SET_UREPEATER;

typedef struct _IPMAC_ENTRY
{
	unsigned long    IpAddr;
	unsigned char    Addr[6];
}
IPMAC_ENTRY;
typedef struct _APICMDBUF_IPMAC_TABLE
{
    APICMDHEADER;
    unsigned short NoOfEntries;
	IPMAC_ENTRY entries[1];
}
APICMDBUF_IPMAC_TABLE;

enum 
{
    UR_DISCONNECTED = 0,
    UR_CONNECTED = 1
};
typedef struct _APICMDBUF_GET_UREPEATER_LINK
{
    APICMDHEADER;
    unsigned char urLink; /* 1=connected; 0=disconnected */
    unsigned char urBssid[6];
#ifdef APCFGUR
	unsigned char RateToBeUsedForTx;
    char 		  AvgRSSI;
#endif
}
APICMDBUF_GET_UREPEATER_LINK;

/* end of 802.11i MIB supported */

/************ Marvell private MIB **************************/

// Enable data traffic for a station. This will be enabled if authentication and key installation
// for the station is done.
typedef struct _APICMDBUF_SET_RSN_DATA_TRAFFIC_ENABLED
{
    APICMDHEADER;
    unsigned char StaMacAddr[6];
    unsigned char Enabled;
}
APICMDBUF_SET_RSN_DATA_TRAFFIC_ENABLED;

// Set Pairwise Key (PWK) for a station
typedef struct _APICMDBUF_SET_RSN_PWK
{
    APICMDHEADER;
    UINT8 StaMacAddr[6];
    UINT8 EncryptKey[16];
    UINT32 TxMICKey[2];
    UINT32  RxMICKey[2];
    //unsigned long  TempKey1[4];
    //unsigned long  TempKey2[4];
}
APICMDBUF_SET_RSN_PWK;

// Get Pairwise Key (PWK) for a station
typedef struct _APICMDBUF_GET_RSN_PWK
{
    APICMDHEADER;
    UINT8 StaMacAddr[6];
    UINT8 EncryptKey[16];
    UINT32 TxMICKey[2];
    UINT32 RxMICKey[2];
    //unsigned long  TempKey1[4];
    //unsigned long  TempKey2[4];
}
APICMDBUF_GET_RSN_PWK;

// Set group key
typedef struct _APICMDBUF_SET_RSN_GRP_KEY
{
    APICMDHEADER;
    unsigned char EncryptKey[16];
    UINT32 TxMICKey[2];
    UINT32 RxMICKey[2];
    //unsigned long  TempKey1[4];
    //unsigned long  TempKey2[4];
}
APICMDBUF_SET_RSN_GRP_KEY;

// Get group key
typedef struct _APICMDBUF_GET_RSN_GRP_KEY
{
    APICMDHEADER;
    unsigned char EncryptKey[16];
    UINT32 TxMICKey[2];
    UINT32 RxMICKey[2];
    //unsigned long  TempKey1[4];
    //unsigned long  TempKey2[4];
}
APICMDBUF_GET_RSN_GRP_KEY;
/* End of Marvell private MIB */

// Get a station's association state. The states include APEVT_STA_ASSOCIATED and
// APEVT_STA_DISASSOCIATED defined in enum STA_ASSOC_TYPE. Whenever a station gets
// associated or deassociated, MacMgmt task will send the state to a message queue.
// This function can be called to check the state in the message queue.
typedef struct _APICMDBUF_GET_STA_ASSOC_STATE
{
    APICMDHEADER;
    unsigned char staMACAddr[6];
    unsigned short assocType;
}
APICMDBUF_GET_STA_ASSOC_STATE;

// Get this station's RSN IE
typedef struct _APICMDBUF_GET_THIS_STA_RSN_IE
{
    APICMDHEADER;
    //unsigned char  staMACAddr[6];
    unsigned char RsnIEBuf[MAX_SIZE_RSN_IE_BUF];
    unsigned char RsnIEWPA2Buf[MAX_SIZE_RSN_IE_BUF];
}
APICMDBUF_GET_THIS_STA_RSN_IE;

// Get an external station's RSN IE
typedef struct _APICMDBUF_GET_EXT_STA_RSN_IE
{
    APICMDHEADER;
    unsigned char staMACAddr[6];
    unsigned char RsnIEBuf[MAX_SIZE_RSN_IE_BUF];
}
APICMDBUF_GET_EXT_STA_RSN_IE;

// Get pairwise TSC
typedef struct _APICMDBUF_GET_PAIRWISE_TSC
{
    APICMDHEADER;
    unsigned char staMACAddr[6];
    unsigned long TxIV32;
    unsigned short TxIV16;
}
APICMDBUF_GET_PAIRWISE_TSC;

// Set pairwise TSC
typedef struct _APICMDBUF_SET_PAIRWISE_TSC
{
    APICMDHEADER;
    unsigned char staMACAddr[6];
    unsigned long TxIV32;
    unsigned short TxIV16;
}
APICMDBUF_SET_PAIRWISE_TSC;

// Get Group TSC
typedef struct _APICMDBUF_GET_GRP_TSC
{
    APICMDHEADER;
    unsigned long g_IV32;
    unsigned short g_IV16;
}
APICMDBUF_GET_GRP_TSC;

// Set Group TSC
typedef struct _APICMDBUF_SET_GRP_TSC
{
    APICMDHEADER;
    unsigned long g_IV32;
    unsigned short g_IV16;
}
APICMDBUF_SET_GRP_TSC;

typedef struct _APICMDBUF_SET_COUNTER_MEASURE_ENABLED
{
    APICMDHEADER;
    unsigned char Enabled;
}
APICMDBUF_SET_COUNTER_MEASURE_ENABLED;

// Set Pairwise Master Key (PMK) for a station
typedef struct _APICMDBUF_SET_RSN_PMK
{
    APICMDHEADER;
    unsigned char StaMacAddr[6];
    unsigned char PMK[32];
}
APICMDBUF_SET_RSN_PMK;

UINT8 disableStaAssociate;

// Set WPA PSK Value
typedef struct _APICMDBUF_SET_WPA_PSK_VALUE
{
	APICMDHEADER;
    unsigned char PSKValueEnabled;
    unsigned char PSKValue[32];
}
APICMDBUF_SET_WPA_PSK_VALUE;

#ifdef AP_WPA2
/* WPA2 */
// Set WPA PSK Value
typedef struct _APICMDBUF_SET_WPA2_PSK_VALUE
{
	APICMDHEADER;
    unsigned char PSKValueEnabled;
    unsigned char PSKValue[32];
}
APICMDBUF_SET_WPA2_PSK_VALUE;

typedef struct _APICMDBUF_GET_RSN_CFG_WPA2_TBL
{
    APICMDHEADER;
    unsigned long Version;
    unsigned long PairwiseKeysSupported;
    unsigned char MulticastCipher[4];
    unsigned char GroupRekeyMethod;
    unsigned long GroupRekeyTime;
    unsigned long GroupRekeyPackets;
    unsigned char GroupRekeyStrict;
    //unsigned char  PSKValue[40];  // write only
    //unsigned char  PSKPassPhrase[64]; // write only
    unsigned char TSNEnabled;
    unsigned long GroupMasterRekeyTime;
    unsigned long GroupUpdateTimeOut;
    unsigned long GroupUpdateCount;
    unsigned long PairwiseUpdateTimeOut;
    unsigned long PairwiseUpdateCount;
}
APICMDBUF_GET_RSN_CFG_WPA2_TBL;

typedef struct _APICMDBUF_SET_RSN_CFG_WPA2_TBL
{
    APICMDHEADER;
    //unsigned long  Version;   // read only
    unsigned long PairwiseKeysSupported;
    unsigned char MulticastCipher[4];
    unsigned char GroupRekeyMethod;
    unsigned long GroupRekeyTime;
    unsigned long GroupRekeyPackets;
    unsigned char GroupRekeyStrict;
    unsigned char PSKValue[40]; //write only
    unsigned char PSKPassPhrase[64]; // write only
    unsigned char TSNEnabled;
    unsigned long GroupMasterRekeyTime;
    unsigned long GroupUpdateTimeOut;
    unsigned long GroupUpdateCount;
    unsigned long PairwiseUpdateTimeOut;
    unsigned long PairwiseUpdateCount;
}
APICMDBUF_SET_RSN_CFG_WPA2_TBL;

// Get this station's RSN IE
typedef struct _APICMDBUF_GET_THIS_STA_WPA2_RSN_IE
{
    APICMDHEADER;
    //unsigned char  staMACAddr[6];
    unsigned char RsnIEBuf[MAX_SIZE_RSN_IE_BUF];
    unsigned char RsnIEWPA2Buf[MAX_SIZE_RSN_IE_BUF];
}
APICMDBUF_GET_THIS_STA_WPA2_RSN_IE;

typedef struct _APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER
{
    APICMDHEADER;
    unsigned char UnicastCipher[4];
    unsigned char Enabled;
}
APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER;

typedef struct _APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER
{
    APICMDHEADER;
    unsigned char UnicastCipher[4];
    unsigned char Enabled;
}
APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER;

typedef struct _APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER2
{
    APICMDHEADER;
    unsigned char UnicastCipher[4];
    unsigned char Enabled;
}
APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER2;

typedef struct _APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER2
{
    APICMDHEADER;
    unsigned char UnicastCipher[4];
    unsigned char Enabled;
}
APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER2;


typedef struct _APICMDBUF_GET_RSN_CFG_WPA2_AUTH_SUITE
{
    APICMDHEADER;
    unsigned char AuthSuites[4];
    unsigned char Enabled;
}
APICMDBUF_GET_RSN_CFG_WPA2_AUTH_SUITE;

typedef struct _APICMDBUF_SET_RSN_CFG_WPA2_AUTH_SUITE
{
    APICMDHEADER;
    unsigned char AuthSuites[4];
    unsigned char Enabled;
}
APICMDBUF_SET_RSN_CFG_WPA2_AUTH_SUITE;


typedef struct _APICMDBUF_GET_WPA2_ENABLED
{
    APICMDHEADER;
    unsigned long Enabled;
}
APICMDBUF_GET_WPA2_ENABLED;

typedef struct _APICMDBUF_SET_WPA2_ENABLED
{
    APICMDHEADER;
    unsigned long Enabled;
}
APICMDBUF_SET_WPA2_ENABLED;

typedef struct _APICMDBUF_SET_WPA2_PREAUTH_ENABLED
{
    APICMDHEADER;
    unsigned long Enabled;
}
APICMDBUF_SET_WPA2_PREAUTH_ENABLED;

typedef struct _APICMDBUF_GET_WPA2ONLY_ENABLED
{
    APICMDHEADER;
    unsigned long Enabled;
}
APICMDBUF_GET_WPA2ONLY_ENABLED;

typedef struct _APICMDBUF_SET_WPA2ONLY_ENABLED
{
    APICMDHEADER;
    unsigned long Enabled;
}
APICMDBUF_SET_WPA2ONLY_ENABLED;

extern IEEEtypes_RSN_IE_WPA2_t thisStaRsnIEWPA2;
extern IEEEtypes_RSN_IE_WPA2MixedMode_t thisStaRsnIEWPA2MixedMode;

#endif

typedef enum {
    COUNTER_MEASURE_START,
    COUNTER_MEASURE_STOP,
} COUNTER_MEASURE_EVENT;
 /* GET/SET Baseband Reg */
typedef struct _APICMDBUF_GET_BB_REG{

    APICMDHEADER;
unsigned short regNum;
unsigned char regVal;

}
APICMDBUF_GET_BB_REG;

typedef struct _APICMDBUF_SET_BB_REG{

    APICMDHEADER;
unsigned short regNum;
unsigned char regVal;
 
}
APICMDBUF_SET_BB_REG;
typedef struct _APICMDBUF_GET_RF_REG{

    APICMDHEADER;
	unsigned short regNum;
	unsigned char regVal;
}
APICMDBUF_GET_RF_REG;

typedef struct _APICMDBUF_SET_RF_REG{

    APICMDHEADER;
	unsigned short regNum;
	unsigned char regVal; 
}
APICMDBUF_SET_RF_REG;

typedef struct _APICMDBUF_GET_FORCE_PROTECTION_MODE
{
    APICMDHEADER;
    unsigned short forceProtectMode;
}
APICMDBUF_GET_FORCE_PROTECTION_MODE;

typedef struct _APICMDBUF_SET_FORCE_PROTECTION_MODE
{
    APICMDHEADER;
    unsigned short forceProtectMode;
}
APICMDBUF_SET_FORCE_PROTECTION_MODE;

 /* GET/SET Baseband Reg end*/
 #ifdef SELF_CTS
typedef struct _APICMDBUF_GET_SELF_CTS
{
    APICMDHEADER;
    unsigned char SelfCts;
}
APICMDBUF_GET_SELF_CTS;
typedef struct _APICMDBUF_SET_SELF_CTS
{
    APICMDHEADER;
    unsigned char SelfCts;
}

APICMDBUF_SET_SELF_CTS;
#endif

typedef struct _APICMDBUF_GET_BOOSTER_MODE
{
    APICMDHEADER;
    unsigned char BoosterMode;
}APICMDBUF_GET_BOOSTER_MODE;

typedef struct _APICMDBUF_SET_BOOSTER_MODE
{
    APICMDHEADER;
    unsigned char BoosterMode;
}APICMDBUF_SET_BOOSTER_MODE;


 /* GET/SET Baseband Reg end*/

#ifdef WDS_FEATURE
typedef enum{
    WDS_MODE_DISABLE,
    WDS_MODE_STATIC,
    WDS_MODE_DYNAMIC,
}WDS_MODE_TYPE;
typedef enum {
	BCAST_AS_WDS_BCAST,
	BCAST_AS_WDS_UNICAST,
} WDS_BCAST_MODE_TYPE;
#endif // WDS_FEATURE
typedef struct _APICMDBUF_SET_TX_DATA_RATE_G
{
    APICMDHEADER;
    unsigned char txDataRate;
}APICMDBUF_SET_TX_DATA_RATE_G;

typedef struct _APICMDBUF_GET_RSN_CFG
{
    APICMDHEADER;
    unsigned long Version;
    unsigned long PairwiseKeysSupported;
    unsigned char MulticastCipher[4];
    unsigned char GroupRekeyMethod;
    unsigned long GroupRekeyTime;
    unsigned long GroupRekeyPackets;
    unsigned char GroupRekeyStrict;
    //unsigned char  PSKValue[32];  // write only
    //unsigned char  PSKPassPhrase[64]; // write only
    unsigned char TSNEnabled;
    unsigned long GroupMasterRekeyTime;
    unsigned long GroupUpdateTimeOut;
    unsigned long GroupUpdateCount;
    unsigned long PairwiseUpdateTimeOut;
    unsigned long PairwiseUpdateCount;
}
APICMDBUF_GET_RSN_CFG;

#ifdef AP_SITE_SURVEY
typedef struct _APICMDBUF_GET_SURVEY_INFO
{
    APICMDHEADER;
    unsigned char  busyScan;
    unsigned char  maxEntry;
    unsigned char  *infoAddr;

}APICMDBUF_GET_SURVEY_INFO;

typedef struct _API_SURVEY_ENTRY
{
    unsigned char dirty;
    unsigned char BssId[6];
    unsigned char SsId[32];
    unsigned char RSSI;
    unsigned char channel;
    unsigned char IBSS:1;
    unsigned char wepEnabled:1;
    unsigned char B_Support:1;
    unsigned char G_Support:1;
    unsigned char wpaEnabled:2;
    unsigned char A_Support:1;
    unsigned char reserve:1;

}API_SURVEY_ENTRY;
#endif /* AP_SITE_SURVEY */

#ifdef STA_INFO_DB
typedef struct _STA_INFO
{
    UINT8 macAdd[6];
    UINT8 assocState;
    UINT8 staMode;
    UINT8 rate;
    UINT8 sq2;
    UINT8 sq1;
    UINT8 RSSI;
    UINT8 powerSave;
}
PACK_END STA_INFO;
#else
typedef struct _STA_INFO
{
    UINT8 macAdd[6];
    UINT16 assocState;
}
PACK_END STA_INFO;
#endif
#if defined(WIPOD)
    #define WIPOD_TYPE_AP 0x80
typedef enum
{
    WIPOD_MODE_CLIENT_BRIDGE = 0x01,
    WIPOD_MODE_AP = 0x82,
    WIPOD_MODE_REPEATER = 0x83,
    WIPOD_MODE_PT_2_PT = 0x84,
    WIPOD_MODE_PT_2_MULTIPT = 0x85
}
WIPOD_MODE;

typedef struct _APICMDBUF_GET_WIPOD_MODE
{
    APICMDHEADER;
    unsigned char  mode; /* use WIPOD_MODE enum */
    unsigned char  body[63];
}
APICMDBUF_GET_WIPOD_MODE;

typedef struct _APICMDBUF_SET_WIPOD_MODE
{
    APICMDHEADER;
    unsigned char mode; /* use WIPOD_MODE enum */
    unsigned char body[63];
}
APICMDBUF_SET_WIPOD_MODE;

typedef struct _APICMDBUF_WIPOD_REPEATER_MODE
{
    APICMDHEADER;
    unsigned char  mode; /* use WIPOD_MODE enum */
    unsigned char  macAddr1[6]; /* Set to all zero if not used */
    unsigned char  macAddr2[6]; /* Set to all zero if not used */
    unsigned char  enableApFunc; /* True, False */
}
APICMDBUF_WIPOD_REPEATER_MODE;

typedef struct _APICMDBUF_WIPOD_P2P_MODE
{
    APICMDHEADER;
    unsigned char  mode; /* use WIPOD_MODE enum */
    unsigned char  macAddr[6]; /* Set to all zero if not used */
}
APICMDBUF_WIPOD_P2P_MODE;

typedef struct _APICMDBUF_WIPOD_P2MP_MODE
{
    APICMDHEADER;
    unsigned char  mode; /* use WIPOD_MODE enum */
    unsigned char  macAddr1[6]; /* Set to all zero if not used */
    unsigned char  macAddr2[6]; /* Set to all zero if not used */
    unsigned char  macAddr3[6]; /* Set to all zero if not used */
    unsigned char  macAddr4[6]; /* Set to all zero if not used */
    unsigned char  macAddr5[6]; /* Set to all zero if not used */
    unsigned char  macAddr6[6]; /* Set to all zero if not used */
    unsigned char  enableDynamic; /* True or False */
}
APICMDBUF_WIPOD_P2MP_MODE;

#endif /* WIPOD */
enum
{
    POWER_ON,
    POWER_OFF,
};
typedef struct _APICMDBUF_SET_RF_POWER_MODE
{
    APICMDHEADER;
    unsigned char PowerMode;    /* POWER_ON, POWER_OFF */
}APICMDBUF_SET_RF_POWER_MODE;

#ifdef QOS_FEATURE

typedef struct _APICMDBUF_SET_EDCA_PARAM
{
    APICMDHEADER;
    mib_QAPEDCATable_t QAPEDCATable;
}
APICMDBUF_SET_EDCA_PARAM;

typedef struct _APICMDBUF_GET_EDCA_PARAM
{
    APICMDHEADER;
    mib_QAPEDCATable_t QAPEDCATable;
}
APICMDBUF_GET_EDCA_PARAM;

typedef struct _APICMDBUF_GET_QOS_TS_INFO
{
    APICMDHEADER;
    unsigned char  maxEntry;
    unsigned char   *pInfo;

}APICMDBUF_GET_QOS_TS_INFO;

typedef struct _APICMDBUF_SET_QOS_MODE
{
    APICMDHEADER;
    unsigned char qos_mode;
}
APICMDBUF_SET_QOS_MODE;

typedef struct _APICMDBUF_GET_QOS_MODE
{
    APICMDHEADER;
    unsigned char qos_mode;
}
APICMDBUF_GET_QOS_MODE;

typedef struct _APICMDBUF_SET_NOACK_MODE

{
    APICMDHEADER;
    unsigned char noack_mode;

}

APICMDBUF_SET_NOACK_MODE;



typedef struct _APICMDBUF_GET_NOACK_MODE

{
    APICMDHEADER;
    unsigned char noack_mode;

}

APICMDBUF_GET_NOACK_MODE;

#endif

void (*MICCounterMeasureInvokedCbFunc)(COUNTER_MEASURE_EVENT event);

#endif

#ifdef IEEE80211H
typedef struct _APICMDBUF_GET_SPECTRUM_MGMT
{
    APICMDHEADER;
    UINT8 isEnable;
}
APICMDBUF_GET_SPECTRUM_MGMT;

typedef struct _APICMDBUF_SET_SPECTRUM_MGMT
{
    APICMDHEADER;
    UINT8 isEnable;
}
APICMDBUF_SET_SPECTRUM_MGMT;

typedef struct _APICMDBUF_GET_MITIGATION
{
    APICMDHEADER;
    UINT32 mitigationValue;
}
APICMDBUF_GET_MITIGATION;

typedef struct _APICMDBUF_SET_MITIGATION
{
    APICMDHEADER;
    UINT32 mitigationValue;
}
APICMDBUF_SET_MITIGATION;

typedef struct _APICMDBUF_SEND_SWITCH_CHANNEL_ANNOUNCEMENT
{
    APICMDHEADER;
    UINT8 channel; 
}
APICMDBUF_SEND_SWITCH_CHANNEL_ANNOUNCEMENT;

typedef struct _APICMDBUF_GET_REGULATORY_DOMAIN
{
    APICMDHEADER;
    UINT8 domain;
}
APICMDBUF_GET_REGULATORY_DOMAIN;

typedef struct _APICMDBUF_SET_REGULATORY_DOMAIN
{
    APICMDHEADER;
    UINT8 domain;
}
APICMDBUF_SET_REGULATORY_DOMAIN;
#endif /* IEEE80211H*/

#ifdef JAPAN_CHANNEL_SPACING_10_SUPPORT
typedef struct _APICMDBUF_GET_CHANNEL_SPACING
{
    APICMDHEADER;
    unsigned short curChanSpacing;
}
APICMDBUF_GET_CHANNEL_SPACING;



typedef struct _APICMDBUF_SET_CHANNEL_SPACING
{
    APICMDHEADER;
    unsigned char curChanSpacing;

}
APICMDBUF_SET_CHANNEL_SPACING;

#endif
typedef struct _APICMDBUF_SWITCH_PRIO_MAP_EN
{
    APICMDHEADER;
    unsigned long port;
    unsigned long mode;
}
APICMDBUF_SWITCH_PRIO_MAP_EN;

typedef struct _APICMDBUF_SWITCH_PRIO_MAP
{
    APICMDHEADER;
    unsigned long tag;
    unsigned long prio;
}
APICMDBUF_SWITCH_PRIO_MAP;

#ifdef AUTOLINK
typedef struct _APICMDBUF_GET_PSK_PASSPHRASE
{
    APICMDHEADER;
    unsigned char passphrase[64];
}
APICMDBUF_GET_PSK_PASSPHRASE;

typedef struct _APICMDBUF_SET_SSID
{
    APICMDHEADER;
    unsigned char ssid[33];
}
APICMDBUF_SET_SSID;

typedef struct _APICMDBUF_GET_AL_LOW_POWER_STATE
{
    APICMDHEADER;
    unsigned char ALLowTxPowerEnable;
}
APICMDBUF_GET_AL_LOW_POWER_STATE;

typedef struct _APICMDBUF_SET_AL_LOW_POWER_STATE
{
    APICMDHEADER;
    unsigned char ALLowTxPowerEnable;
}
APICMDBUF_SET_AL_LOW_POWER_STATE;

typedef struct _APICMDBUF_GET_AL_ENABLE_STATE
{
    APICMDHEADER;
    unsigned char ALEnable;
}
APICMDBUF_GET_AL_ENABLE_STATE;

typedef struct _APICMDBUF_SET_AL_ENABLE_STATE
{
    APICMDHEADER;
    unsigned char ALEnable;
}
APICMDBUF_SET_AL_ENABLE_STATE;

typedef struct _APICMDBUF_GET_AL_ON_TIME
{
    APICMDHEADER;
    unsigned int ALONTime;
}
APICMDBUF_GET_AL_ON_TIME;

typedef struct _APICMDBUF_SET_AL_ON_TIME
{
    APICMDHEADER;
    unsigned int ALONTime;
}
APICMDBUF_SET_AL_ON_TIME;

typedef struct _APICMDBUF_GET_AL_PP_ENABLE_STATE
{
    APICMDHEADER;
    unsigned char ALPPEnable;
}
APICMDBUF_GET_AL_PP_ENABLE_STATE;

typedef struct _APICMDBUF_SET_AL_PP_ENABLE_STATE
{
    APICMDHEADER;
    unsigned char ALPPEnable;
}
APICMDBUF_SET_AL_PP_ENABLE_STATE;

typedef struct _APICMDBUF_GET_AL_SEC_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char ALSecmode;
}
APICMDBUF_GET_AL_SEC_MODE;

typedef struct _APICMDBUF_SET_AL_SEC_MODE
{
    unsigned short cmdCode;
    unsigned short seqNum;
    unsigned short cmdStatus;
    unsigned short cmdBodyLen;
    unsigned char ALSecMode;
}
APICMDBUF_SET_AL_SEC_MODE;
#endif /*AUTOLINK*/

#ifdef DCS
typedef struct _APICMDBUF_GET_DCS_MODE
{
    APICMDHEADER;
    UINT8 DCSMode;
}APICMDBUF_GET_DCS_MODE;

typedef struct _APICMDBUF_SET_DCS_MODE
{
    APICMDHEADER;
    UINT8 DCSMode;
}APICMDBUF_SET_DCS_MODE;

#endif
#ifdef GUEST_ACCESS
typedef struct _APICMDBUF_GET_GUEST_ACCESS
{
	APICMDHEADER;
    unsigned char GuestAccessEnable;
    unsigned char GuestAccessPassPhrase[64];
    unsigned char GuestAccessPassPhrase2[64];
}
APICMDBUF_GET_GUEST_ACCESS;
typedef APICMDBUF_GET_GUEST_ACCESS   APICMDBUF_SET_GUEST_ACCESS;
#endif
/* Timer API */
/**
 * Create a periodic timer. The callback function will run in the
 * Delayed Service Routine of the real-time clock interrupt.
 * 
 * @param period  The period in milliseconds
 * @param timerCb The callback function
 * @return 0: failed to create timer because exceed maximum
 *         number of timers supported
 *         >0: the timer index, to be used in
 *         subsequent call to apctlTimerDelete()
 */
unsigned long apctlTimerCreate(int period, void (*timerCb)(void));

/**
 * Delete a timer
 * 
 * @param timerIndex The timer index returned by apctlTimerCreate()
 * @return TRUE: success
 *         FALSE: failure because timer index is invalid
 */
int apctlTimerDelete(unsigned long timerIndex);

/* Interrupt control API */

/**
 * Get interrupt enable status
 * 
 * @return Interrupt status
 */
int apctlGetIntStatus(void);

/**
 * Enable all interrupts
 * 
 * @param status Status word returned by apctlIntDisable()
 * @return Interrupt status
 */
int apctlIntEnable(int status);
/**
 * Disable all interrupts
 * 
 * @return Interrupt status
 */
int apctlIntDisable(void);

/* GPIO control API */

/**
 * Initialize the GPIO pin for output or input
 * 
 * @param dirMap The GPIO direction bit map.
 *               Bit set to 1: the corresponding GPIO pin is configured
 *               for input.
 *               Bit set to 0: the corresponding GPIO pin is configured
 *               for output
 */
void apctlGPIOInit(unsigned long dirMap);

/**
 * Read the state of a GPIO pin
 * 
 * @param gpioNum the GPIO pin number
 * @return 0: GPIO state is "low"
 *         1: GPIO state is "high"
 *        -1: Error: invalid GPIO number
 */
int apctlGPIORead(int gpioNum);

/**
 * Set the state of a GPIO pin. The new GPIO state
 * after write is returned.
 * 
 * @param gpioNum The GPIO pin number
 * @param state   The state to set:
 *                0: set to "low"
 *                1: set to "high"
 * @return 0: the GPIO state is "low"
 *         1: the GPIO state is "high"
 *        -1: Error: invalid GPIO number
 *        -2: Error: GPIO pin is in input mode
 */
int apctlGPIOWrite(int gpioNum, int state);

void apctlMsTimerCreate(int period, void (*timerCbFunc)(void));

#ifdef WAN_PHY_INIT

UINT8 Wan_PhyConfig;

typedef enum
{
    WAN_AUTO_NEG,
    WAN_10_HALF_DUPLEX,
    WAN_10_FULL_DUPLEX,
    WAN_100_HALF_DUPLEX,
    WAN_100_FULL_DUPLEX,
}WAN_PHY_CONFIG;

#endif
/*
#ifdef WPA
typedef struct _StaAssocStateMsg
{
    unsigned char staMACAddr[6];
    unsigned short assocType;
}
StaAssocStateMsg_t;

#endif
*/

#ifdef RESET_BUTTON_EVENT
typedef struct _ApEvtMsg
{
    UINT16 evtCode;
    UINT16 evtBodyLen;
    UINT32 evtTimeStamp;
    UINT8 *evtBody;
}
ApEvtMsg_t;

//EVENTS MESSAGES
typedef struct _APEVT_STA_ASSOC_TYPE
{
    unsigned short evtCode;
    unsigned short evtBodyLen;
    unsigned long evtTimeStamp;
    unsigned char staMACAddress[6];
    unsigned short assocType;
}
APEVT_STA_ASSOC_TYPE;

typedef enum
{
    APEVT_RST_BTN_PRESSED = 1,
    APEVT_UTIL_BTN_PRESSED = (1<<1),
    APEVT_ETH_PORT_STATUS_CHANGE = (1 << 2),
    APEVT_STA_ASSOC = (1 << 3),
                              
}APEVT;

typedef enum
{
    GPIO_PIN_0 = (1 << 0),
    GPIO_PIN_1 = (1 << 1),
    GPIO_PIN_2 = (1 << 2),
    GPIO_PIN_3 = (1 << 3),
    GPIO_PIN_4 = (1 << 4),
    GPIO_PIN_5 = (1 << 5),
    GPIO_PIN_6 = (1 << 6),
    GPIO_PIN_7 = (1 << 7),
    GPIO_PIN_8 = (1 << 8),
    GPIO_PIN_9 = (1 << 9),
    GPIO_PIN_10 = (1 << 10),
    GPIO_PIN_11 = (1 << 11),
    GPIO_PIN_12 = (1 << 12),
    GPIO_PIN_13 = (1 << 13),
    GPIO_PIN_14 = (1 << 14),
    GPIO_PIN_15 = (1 << 15),
}GPIO;


// maximum events
#define API_EVENT_MAX_EVENT 4

// maximun subscribers
#define API_EVENT_MAX_SUB 2*API_EVENT_MAX_EVENT

/*
 ** apEvtSub
 *
 *  PARAMETERS: 
 *      eventSet - the set of events the caller want to subcribe. This parameter is a bit map
 *          each bit corresponds to an event type. Set the bit to 1 to subscribe for the event
 *      *evtCbFunc - Pointer to the callback function
 *      pSubID - Pointer to the variable that will receive the subscription ID. The caller must 
 *          save the subscription ID for later use to unscribe for the event
 *
 *  DESCRIPTION:
 *
 *  RETURNS: TRUE - the subscription is successful
 *           FALSE - the subscription fail
 *
 */
BOOLEAN apEvtSub(unsigned long eventSet, void (*evtCbFunc)(ApEvtMsg_t *), unsigned int *pSubID);

/*
 ** apEvtUnsub
 *
 *  PARAMETERS:
 *      eventSet - the set of events the caller want to subcribe. This parameter is a bit map
 *          each bit corresponds to an event type. Set the bit to 1 to subscribe for the event
 *      subID - the subscription ID
 *
 *  DESCRIPTION:
 *
 *  RETURNS: TRUE - the unsubscription is successful
 *           FALSE - the unsubscription fail
 *
 */
BOOLEAN apEvtUnsub(unsigned long eventSet, unsigned int subID);

#endif /*RESET_BUTTON_EVENT*/

void WriteRfReg(unsigned char offset, unsigned char value);
unsigned char ReadRfReg(unsigned char offset);
void Reset(void);
void Reset2();
#endif /* UDP_API */
#endif /* _API_H_ */
