/******************** (c) Marvell Semiconductor, Inc., 2002 *******************
*
* Purpose:
*    This file contains the function prototypes and definitions for the
*    AP command APIs.
*
* Public Procedures:
*
* Notes:
*    None.
*
*****************************************************************************/

#ifndef _API2_H_
#define _API2_H_

//=============================================================================
//                               INCLUDE FILES
//=============================================================================

//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================
#define api_RESP_RCVD      (1<<0)




/*--------------------------------*/
/* API Commands  & Responses      */
/*--------------------------------*/
typedef struct api_Cmd_t
{
    host_MsgHdr_t MsgHdr;
    union
    {
        host_getHwSpec_t HwSpec;
        host_802_11SetWep_t SetWep;
        APCmd_DS_802_11_QUERY_STATUS QueryStatus;
        APCmd_CMD_802_11_STA_LIST StaList;
        APCmd_CMD_802_11_GET_AUTH_STA_LIST AuthStaList;
        APCmd_CMD_802_11_GET_ASSOC_STA_LIST AssocStaList;
        APCmd_CMD_802_11_QUERY_STA_INFO StaInfo;
        APCmd_CMD_802_11_SSID SsId;
        APCmd_CMD_802_11_PRIVACY_OPTION PrivOption;
        APCmd_CMD_802_11_AUTHENTICATION_MODE AuthMode;
        APCmd_CMD_802_11_FRAGMENTATION_THRESHOLD FragThresh;
        APCmd_CMD_802_11_RTS_THRESHOLD RtsThreshold;
        APCmd_CMD_802_11_RTS_RETRY_LIMIT RetryLimit;
        APCmd_DS_802_11_DATA_RATE DataRate;
        APCmd_DS_802_11_RADIO_PREAMBLE RadioPreamble;
        APCmd_DS_802_11_BEACON_PERIOD BeaconPeriod;
        APCmd_DS_802_11_DTIM_PERIOD DtimPeriod;
        APCmd_DS_802_3_GET_STAT GetStat;
        APCmd_DS_802_3_LINK_SPEED LinkSpeed;
        APCmd_DS_802_3_IP_ADDRESS IpAddr;
        APCmd_DS_802_3_SUBNET_MASK SubnetMask;
        APCmd_DS_802_3_GATEWAY Gateway;
        APCmd_DS_802_3_DHCP Dhcp;
        APCmd_DS_802_3_ICMP Icmp;
        APCmd_CMD_802_11_LINK_TEST LinkTest;
#ifdef IEEE80211H
        APCmd_CMD_802_11_H_CHANNELSWITCH ChannelSwitchCmd;
#endif
    } Msg;
}
api_Cmd_t;






//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================




/******************************************************************************
 *
 * Name: api_RespReadNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where IEEE 802.11
 *   management messages are placed for the SME Task. If writing to the
 *   queue cannot immediately occur, then the routine returns with a failure
 *   status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling smeQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern api_Cmd_t *api_RespReadNoBlock(void);

/******************************************************************************
 *
 * Name: smeQ_CbProcWriteNoBlock
 *
 * Description:
 
 *
 * Conditions For Use:
 *   The queue has been initialized by calling smeQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): 
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern WL_STATUS api_CmdWriteNoBlock( api_Cmd_t *Cmd_p );
extern WL_STATUS api_RespWriteNoBlock( api_Cmd_t *Cmd_p );

extern api_Cmd_t *api_FreeCmdBuf(void);
extern api_Cmd_t *api_GetCmdBuf(void);

//=============================================================================
//                          API for Sending Commands
//=============================================================================
#endif

