#ifndef _apio_h_
#define _apio_h_ 

extern apio_ErrCode_t apio_wlanRead(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_wlanWrite(apio_bufdescr_t *pBufDescr);
extern void apio_FreeEtherRxBuf(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_write_wan(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_write_lan(apio_bufdescr_t *pBufDescr);
#ifdef DMZ_PORT
extern apio_ErrCode_t apio_write_dmz(apio_bufdescr_t *pBufDescr);
#endif
extern apio_ErrCode_t apio_wlanRead_NoWPA(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_wlanWrite_noWPA(apio_bufdescr_t *pBufDescr);
extern UINT8 *apio_GetEtherTxBuf(void);
extern apio_ErrCode_t apio_etherRead(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_ethAlloc(apio_handle_t handle, apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_ethFree(apio_handle_t handle, apio_bufdescr_t *pBufDescr);

#ifdef NO_VLAN
extern apio_ErrCode_t apio_etherRead_noVlan(apio_bufdescr_t *pBufDescr);
extern apio_ErrCode_t apio_write_lan_noVlan(apio_bufdescr_t *pBufDescr);
#endif

/* Port type enumeration */
enum portType_e
{
    portType_wlan,
    portType_wan,
    portType_lan,
#ifdef DMZ_PORT
    portType_dmz,
#endif
#ifdef WDS_FEATURE
    portType_wds,
    portType_llc,
#endif // WDS_FEATURE
};

/* Index of packet filter function (see ether.c) */
/* Do not use 0. It's reserved for the default do-nothing filter */
#define RXPKTFILTER_WAN 1
#define RXPKTFILTER_LAN 2
#define RXPKFILTER_DMZ 4

#ifdef WDS_FEATURE
apio_ErrCode_t Empty_Deal(apio_bufdescr_t *pBufDescr)
{
	return APIO_NOERROR;
}

void Empty_Deal_Done(apio_bufdescr_t *pBufDescr)
{
	return;
}
#endif // WDS_FEATURE

/************************************************
* The network port objects, statically defined  *
************************************************/

/* WLAN port object */
class_port_t wlanPort =
    {
        portType_wlan,
        PORT_STATUS_CLOSED,
        apio_wlanRead,
        apio_wlanWrite,
        default_close,
        apio_ethAlloc,
        apio_ethFree,
        apio_FreeEtherRxBuf,
        NULL
    };

#ifdef DMZ_PORT 
/* WAN port object */
class_port_t dmzPort =
    {
        portType_dmz,
        PORT_STATUS_CLOSED,
        apio_etherRead,
        apio_write_dmz,
        default_close,
        apio_ethAlloc,
        apio_ethFree,
        apio_FreeEtherRxBuf,
        NULL
    };
#endif


/* WAN port object */
class_port_t wanPort =
    {
        portType_wan,
        PORT_STATUS_CLOSED,
        apio_etherRead,
        apio_write_wan,
        default_close,
        apio_ethAlloc,
        apio_ethFree,
        apio_FreeEtherRxBuf,
        NULL
    };

/* LAN port object */
class_port_t lanPort =
    {
        portType_lan,
        PORT_STATUS_CLOSED,
        apio_etherRead,
        apio_write_lan,
        default_close,
        apio_ethAlloc,
        apio_ethFree,
        apio_FreeEtherRxBuf,
        NULL
    };

/* WLAN port object */
#if 1
class_port_t wlanPortnoWPA =
    {
        portType_wlan,
        PORT_STATUS_CLOSED,
        apio_wlanRead_NoWPA,
        apio_wlanWrite_noWPA,
        default_close,
        apio_ethAlloc,
        apio_ethFree,
        apio_FreeEtherRxBuf,
        NULL
    };
#endif

#ifdef NO_VLAN
/* LAN port object */
class_port_t lanPort_noVlan = 
{
  portType_lan,
  PORT_STATUS_CLOSED,
  apio_etherRead_noVlan,
  apio_write_lan_noVlan,
  default_close,
  apio_ethAlloc,
  apio_ethFree,
  apio_FreeEtherRxBuf,
  NULL
};
#endif
#ifdef WDS_FEATURE
#ifndef MAX_WDS_PORT
#define MAX_WDS_PORT   6
#endif

class_port_t wdsPort[MAX_WDS_PORT+1];
//bellow variable is used to init wdsPort[...]
class_port_t wdsPortModule =
    {
        portType_wds,
        PORT_STATUS_CLOSED,
        apio_wlanRead,
        apio_wlanWrite,
        default_close,
        apio_ethAlloc,
        apio_ethFree,
        apio_FreeEtherRxBuf,
        NULL //this filed will point to each port specified private struct in apio_open()
    };
/* llc simulate port object */
class_port_t llcPort =
    {
        portType_llc,
        PORT_STATUS_CLOSED,
        Empty_Deal,
        Empty_Deal,
        Empty_Deal,
        Empty_Deal,
        Empty_Deal,
        Empty_Deal_Done,
        NULL
    };
apio_handle_t llcHandle = (apio_handle_t ) &llcPort;
#endif // WDS_FEATURE

/**
 * Allocate a buffer for the WAN and LAN port
 * 
 * @param handle     The WAN or LAN port handle
 * @param ppBufDescr Pointer to buffer descriptor
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
inline apio_ErrCode_t apio_ethAlloc(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
{
#ifdef NEW_APIO_ALLOC
    if ((pBufDescr->framePtr = apio_GetEtherTxBuf()) != NULL)
    {
        pBufDescr->framePtr += 2;
#ifdef USE_BUFFER_OFFSET
        *(pBufDescr->framePtr - 1) = 128;   //Offset is 128
#endif
        return APIO_NOERROR;
    }
#else
    if ((pBufDescr->framePtr = apio_GetEtherTxBuf()) != NULL)
        return APIO_NOERROR;
#endif
    else
        return APIO_ERROR;
}

/**
 * Free a buffer allocated by apio_ethAlloc
 * 
 * @param handle    Handle of the WAN or LAN port
 * @param pBufDescr Pointer to the buffer descriptor
 * @return APIO_NOERROR
 *         APIO_ERROR
 */
inline apio_ErrCode_t apio_ethFree(apio_handle_t handle, apio_bufdescr_t *pBufDescr)
{
    apio_FreeEtherRxBuf(pBufDescr);
    return APIO_NOERROR;
}

#endif /* _apio_h_ */
