/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
 * 
 * Purpose: This file contains data structures relevant to AutoLink
 * 
 * Public Procedures: 
 * 
 * Notes: 
 *    None. 
 * 
 *****************************************************************************/
#ifndef _AUTOLINK_H_
#define _AUTOLINK_H_

typedef	enum
{
    ALCMD_START = 0x0001,
    ALCMD_END,
    ALCMD_CHALLENGE,
    ALCMD_STATUS,
	ALCMD_INITIATE,
	ALCMD_KEYMSG
} AL_CMD;

typedef	enum
{
    ALSTATUS_OFF = 0x00,
    ALSTATUS_ON =   0x02,
    ALSTATUS_DONE = 0x04,
	ALSTATUS_PROPAGATE = 0x06
} AL_STATUS;

typedef struct AutoLinkCmd_Hdr_t
{
    UINT16  CmdCode;
    UINT16  Size;
    UINT16  SeqNum;
    UINT16  Result;
}
PACK_END AutoLinkCmd_Hdr_t;

#ifdef AL_DH
typedef  struct AL_Start_Req_t
{
    UINT8 CfgPsw[72]; //64 bytes to accomodate WPA passphrase + 8 bytes for AES encrypted passphrase
	UINT8 CfgPswWPA2[72]; //64 bytes to accomodate WPA2 passphrase + 8 bytes for AES encrypted passphrase
    UINT16 SecMode;
    UINT16 KeyIndex;
    UINT16 IPAddressOption;
    UINT32 CurrentIPAddress;
    UINT16 SuggestChannel;
	UINT8 Ssid[32];
	UINT8 MacAddress[6];
	UINT32 GroupRekeyTime;
}
PACK_END AL_Start_Req_t;
#else
typedef  struct AL_Start_Req_t
{
    UINT8 CfgPsw[64]; //64 bytes to accomodate WPA passphrase
	UINT8 CfgPswWPA2[64]; //64 bytes to accomodate WPA2 passphrase
    UINT16 SecMode;
    UINT16 KeyIndex;
    UINT16 IPAddressOption;
    UINT32 CurrentIPAddress;
    UINT16 SuggestChannel;
	UINT8 Ssid[32];
	UINT8 MacAddress[6];
	UINT32 GroupRekeyTime;
}
PACK_END AL_Start_Req_t;
#endif

#define AL_SEC_MODE_OPEN 			0
#define AL_SEC_MODE_WPA1 			1
#define AL_SEC_MODE_WEP64 			2
#define AL_SEC_MODE_WEP128 			3
#define AL_SEC_MODE_WPA1_CCMP 		4
#define AL_SEC_MODE_WPA2 			5
#define AL_SEC_MODE_WPA2_MIXED 		6
#define AL_IP_FIXD    0x0001
#define AL_IP_DHCP    0x0002


typedef  struct AL_Challenge_t
{
    UINT8 PlainText[128];
}
PACK_END AL_Challenge_t;

typedef  struct AL_KeyMsg_t
{
    UINT8 PublicKey[128];
    UINT16 GroupIndex;
    UINT16 Reserved;
}
PACK_END AL_KeyMsg_t;

typedef  struct AL_Status_t
{
    UINT8 Status;
    UINT8 Action;
}
PACK_END AL_Status_t;
#define AL_STATUS_SUCCESS	1
#define AL_STATUS_FAIL		0
#define AL_ACTION_END		0
#define AL_STATUS_RESTART	1

//AutoLink protocol type is 0x02, common to all products
#define AL_PROTOCOL_PRODUCT_TYPE_UR 		0x0202
#define AL_PROTOCOL_PRODUCT_TYPE_CLIENT   	0x0201
#define AL_PROTOCOL_PRODUCT_TYPE_AP 		0x0200

#define AL_MSG_TIMEOUT			100 /*AutoLink message timeouts*/
#define AL_SHUTOFF_TIMEOUT		600 /*60 seconds*/

#define ENH_AL		1 /*enhanced autoLink implementation*/

#define AL_LIFELINE_KEY					1
#define AL_OPERATIONAL_KEY 				2
#define AL_OPERATIONAL_PASSPHRASE 		3

#ifdef LED_SUPPORT
#if PROD_NAME == AP52
#define WLAN_TX_LED 0
#define WLAN_RX_LED 1
#else
#define WLAN_TX_LED 2
#define WLAN_RX_LED 2
#endif
#define LAN_TX_LED  3
#define LAN_RX_LED  3
#define UR_LINK_STATUS_LED	 0
#define LAN_LINK_STATUS_LED 1
#endif

Timer ALTimer;
Timer LEDTimer;
Timer ALInitiateTimer;
Timer ALProtectTimer;

#ifndef ENH_AL
static UINT8 ChallengeText[128]; 
#endif
#endif /*_AUTOLINK_H_*/
