#if !defined(_CSU_H_)
#define _CSU_H_

/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    csu.h
 * \brief   This file contains the Clock Serial Unit Registers for the 88W8100 hardware.
 *
 */


/*--------------------------------------------------------------*/
/*    Clock Serial Unit Registers                              */
/*--------------------------------------------------------------*/

#define WL_CSU_PRER          (WL_LBU_CSU_CFG_BASE+0x100)
#define WL_CSU_CTRL          (WL_LBU_CSU_CFG_BASE+0x104)
#define WL_CSU_SPI_CTRL      (WL_LBU_CSU_CFG_BASE+0x108)
#define WL_CSU_UW_CTRL       (WL_LBU_CSU_CFG_BASE+0x10C)
#define WL_CSU_I2C_CTRL      (WL_LBU_CSU_CFG_BASE+0x110)
#define WL_CSU_UW_OPCADDR    (WL_LBU_CSU_CFG_BASE+0x114)
#define WL_CSU_STATUS        (WL_LBU_CSU_CFG_BASE+0x11C)
#define WL_CSU_INTR_ACK      (WL_LBU_CSU_CFG_BASE+0x120)
#define WL_CSU_DATA          (WL_LBU_CSU_CFG_BASE+0x124)
#define WL_CSU_ADDR          (WL_LBU_CSU_CFG_BASE+0x128)

#define WL_CSU_CODEC_REG_ACCESS_CMD_LO 	(WL_LBU_CSU_CFG_BASE + 0x130)
#define WL_CSU_CODEC_REG_ACCESS_CMD_HI  (WL_LBU_CSU_CFG_BASE + 0x134)
#define WL_CSU_CODEC_REG_ACCESS_CTRL  	(WL_LBU_CSU_CFG_BASE + 0x138)
#define WL_CSU_CODEC_DATA  				(WL_LBU_CSU_CFG_BASE + 0x13C)
#define WL_CSU_FST_BURDATA				(WL_LBU_CSU_CFG_BASE + 0x200)

#define WRSR   0x01            /* write status register */
#define WRITE  0x02            /* page write to upper 256 bytes address space */
#define READ   0x03            /* read from upper 256 bytes address space */
#define WRDI   0x04            /* reset write enable */
#define RDSR   0x05            /* read status reg */
#define WREN   0x06            /* set write enable */
#define BE     0xc7            /* bulk erase */

/* CSU 16-bit SPI commd's */
#define SPI_WREN    0x0420
#define SPI_ERASE   0x0420
#define SPI_WRDI    0x0410
#define SPI_RDSR    0x0408
#define SPI_WRSR    0x0404
#define SPI_ADDR1_READ1  0x0442       /* read back 1 byte */
#define SPI_ADDR1_WRITE1 0x0441       /* 1 bytes write */
#define SPI_ADDR3_READ1  0x04c2       /* read for 3 byte addr */
#define SPI_ADDR3_WRITE1 0x04c1       /* write for 3 byte addr */
#define SPI_ADDR3_READ4  0x0582       /* read for 3 byte addr */
#define SPI_ADDR3_WRITE4 0x0581       /* write for 3 byte addr */

#define SPI_ADDR16       (1 << 4)	  /* 1->16bit , 0->24bit spi parts */

#endif /* _CSU_H_ */

