/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
* 
* Purpose: 
*    This file contains the function prototypes and definitions for the 
*    Distribution Service Module. 
* 
* Public Procedures: 
*    ds_Init       Initialzies Distribution Service Task and 
*                           related components 
*    ds_Start      Starts running the Distribution Service Task 
*    ds_SendEvent  Mechanism used to trigger an event indicating that 
*                           messages are present in the queue or receive
*                           buffers 
* 
* Notes: 
*    None. 
* 
*****************************************************************************/

#ifndef _DS_H_
#define _DS_H_

//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================
#define ds_802_11_DATA_RCVD         (1<<0)
#define ds_802_3_DATA_RCVD          (1<<1)
#define ds_MGMT_NOTIFICATION_RCVD   (1<<2)
#define ds_LLC_DATA_RCVD            (1<<3)
#define ds_802_3_TX_DONE            (1<<4)
#define ds_802_11_TX_DONE           (1<<5)
#define ds_POLLING_TIMEOUT_RCVD     (1<<6)
#define ds_802_3_WAN_DATA_RCVD           (1<<7)

extern UINT8 NO_OF_PKTS_WLAN;
extern UINT8 NO_OF_PKTS_LAN;

#define ETHER_HDR_SIZE 14
#define MAX_ETHER_PKT_SIZE 1800

#define HEADER_ALLOC_OFFSET 2

#define IS_BROADCAST(macaddr) ((*(UINT32*)macaddr == 0xffff ) &&  \
                          *(UINT16 *)((UINT8*)macaddr+4) == 0xff)

#define IS_MULTICAST(macaddr)  ((*(UINT8*)macaddr & 0x01) == 0x01)

#define IS_GROUP(macaddr)  ((*(UINT8*)macaddr & 0x01) == 0x01)

#define MACADDR_CMP(macaddr1, macaddr2)         \
		((*(UINT32*)macaddr1 == *(UINT32*)macaddr2) && \
		(*(UINT16 *)((UINT8*)macaddr1+4) == (*(UINT16 *)((UINT8*)macaddr2+4))) ? \
		0 : 1)

#define MACADDR_CPY(macaddr1,macaddr2) { *(UINT16*)macaddr1 = *(UINT16*)macaddr2; \
                    *(UINT16 *)((UINT16*)macaddr1+1) = *(UINT16 *)((UINT16*)macaddr2+1); \
                    *(UINT16 *)((UINT16*)macaddr1+2) = *(UINT16 *)((UINT16*)macaddr2+2);}

typedef enum {
    ASSOCIATE_NOTIFICATION,
    DISASSOCIATE_NOTIFICATION
} ds_mgmt_notify_msg_e;


typedef struct
{
    UINT32 Key;
    UINT32 totalLen;
#ifdef GATEWAY
    unsigned int interface;/*0 - LAN ;1 - WAN*/
#endif
    UINT8 data[1600];
}
dsQ_LlcData_t;

typedef struct
{
    IEEEtypes_MacAddr_t da;
    IEEEtypes_MacAddr_t sa;
    UINT16 type;
}
PACK_END ether_hdr_t;

#ifdef QOS_FEATURE
typedef PACK_START struct 
{
   
   UINT8 ihl:4; //Note that the MS nibble is Version.
                //the LS nibble is Header Length.
   UINT8 ver:4;
   UINT8 tos;
   UINT16 total_length;
   UINT16 identification;
   UINT16 flag_fragoffset; //the MS three bits are for flag and rest is fragment offset.
   UINT8 ttl;
   UINT8 protocol;
   UINT16 header_chksum;
   UINT8 src_IP_addr[4];
   UINT8 dst_IP_addr[4];
}
PACK_END IEEEtypes_IPv4_Hdr_t;

typedef PACK_START struct
{
   UINT32 flow_label:16;
   UINT32 traffic_class:8;
   UINT32 ver:4;
   UINT16 payload_length;
   UINT8 next_header;
   UINT8 hop_limit;
   UINT8 src_IP_addr[4];
   UINT8 dst_IP_addr[4];
}
PACK_END IEEEtypes_IPv6_Hdr_t;

typedef PACK_START struct
{
  UINT16 src_port;
  UINT16 dst_port;
  UINT32 seq_num;
  UINT32 ack_num;
  UINT32 datoffset_control_window; //data offset is MS four bits
  UINT16 chksum;
  UINT16 urgent_ptr;
}
PACK_END IEEEtypes_TCP_Hdr_t;

typedef PACK_START struct
{
  UINT16 src_port;
  UINT16 dst_port;
  UINT16 len;
  UINT16 chksum;
}
PACK_END IEEEtypes_UDP_Hdr_t;

#endif //QOS_FEATURE

#ifdef WDS_FEATURE
#define WLAN_BSS_PACKET    0
#define WDS_BROADCAST       1
#endif
#ifdef GUEST_ACCESS
#define WLAN_BSS_GUEST       (MAX_WDS_PORT+1)       /* packet from guest account */
#endif


typedef PACK_START struct
{
    ether_hdr_t Hdr;
    UINT8 Body[1800 - ETHER_HDR_SIZE];
}
PACK_END IEEEtypes_8023_Frame_t;

typedef PACK_START struct
{
    //   SwInfo_t SwInfo;
    //   UINT8  reserved[40];  /* reserved for 802.11 Hdr */
    IEEEtypes_8023_Frame_t DataFrame;
}
PACK_END rx8023_DataFrame_t;

typedef struct llc_snap_hdr
{
    UINT8 Dsap;
    UINT8 Ssap;
    UINT8 Control;
    UINT8 Org[3];
    UINT16 Type;
}
llc_snap_hdr_t;

#if 0 //WPA 
typedef enum
{
    ERROR,
    KEYMGMTTIMEOUTEVENT, //key management timeout
    MICERRORTIMEOUTEVENT, //MIC error timeout
    MICERRORRSTTIMEOUTEVENT, //MIC error timeout within 60 secs
    GRPKEYTIMEOUTEVENT
} dist_timout_msg_type_e;

//Rahul
typedef struct
{
    //timer_Data_t timerData;
    //UINT32   Id;
    IEEEtypes_MacAddr_t StnAddr;
    dist_timout_msg_type_e type;
}
dist_PendingData_t;

/*----------------*/
/* Timer Messages */
/*----------------*/
typedef struct
{
    dist_PendingData_t *PendingData_p;
    UINT8 Id;
}
distQ_TimerMsg_t;

typedef enum
{
    ASSOMSGRECVD,
    TIMERMSGRECVD
}MsgType_e;

typedef struct
{
    MsgType_e MsgType;
    union
    {
        StaAssocStateMsg_t StaAssocStateMsg;
        distQ_TimerMsg_t distQ_TimerMsg;
    }msg;
}
DistTaskMsg_t;
#endif

#ifdef QOS_FEATURE
typedef struct 
{
    UINT16 Type;
    UINT16 Control;
}
IEEE802_1QTag_t;

#define IEEE802_11Q_TYPE    0x8100
#define IPV6_VERSION 0x06

#endif //QOS_FEATURE

extern IEEEtypes_MacAddr_t bcast;

extern WL_STATUS ds_Init(void);
extern void ds_Start(void);
extern WL_STATUS ds_SendEvent(UINT8 Event);
//extern void distTask_Timeout(distQ_TimerMsg_t *TimerMsg_p);

#endif /* _DS_H_ */
