#ifndef _W8100_ETH_
#define _W8100_ETH_ 
/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file    ether.h
 * \brief   This file contains the ethernet driver definitions
 */

#ifdef AP32GST
#define NT 48 /* define for GST */
#else
#define NT 128 /* Number of Transmit buffers */
               /* changed default from 32 to 128 per customer requests */
#endif

#define NR 64 /* Number of Receive buffers */
#define W8100_INT_THRE 1
#define MAX_BUFF_SIZE 1536
#define ETH_ADDR_GAP (ETHER1_PORT_CONFIG_REG - ETHER0_PORT_CONFIG_REG)

#define NUM_TX_QUEUES 2
#define NUM_RX_QUEUES 1

#define RX_MATCHED_FRAMES   1
#define DISCARD_MATCHED_FRAMES 0

#define SKIP_ADDR_TABLE_ENTRY 1
#define DONOT_SKIP_ADDR_TABLE_ENTRY 0
#define GTREGREAD WL_REGS32

//#define IFF_PROMISC   0x1
//#define IFF_RUNNING   0x2

#define PACKET_HOST   1
#define CHECKSUM_NONE 1

struct w8100_ethernet_priv;

struct net_device
{
    unsigned char dev_addr[6];
    struct w8100_ethernet_priv *priv;
    unsigned int flags;
    int trans_start;
};


struct sk_buff
{
    unsigned int *data;
    unsigned int len;
    unsigned int pkt_type;
    unsigned int protocol;
    struct net_device *dev;
};

struct pt_regs
    {}
;

extern struct net_device dev;
int w8100_eth_add_mac_address ( UINT8 *addr) ;
int w8100_eth_remove_mac_address(UINT8 * addr);
int w8100_eth_init (struct net_device *);
int w8100_eth_open (struct net_device *);
int w8100_eth_start_xmit (struct sk_buff * , struct net_device *);
int w8100_eth_xmit (void *data , UINT32 len, struct net_device *dev);
int w8100_eth_stop (struct net_device *);
void w8100_eth_int_handler (int , void * , struct pt_regs *);
int w8100_eth_set_mac_address (struct net_device *, void *);
struct net_device_stats* w8100_eth_get_stats (struct net_device *);
int w8100_eth_receive ( struct net_device *dev, int queue , int * data,
                        int * len, int pktFilterSel);
unsigned int w8100_eth_free_tx_queue (struct net_device *dev , int queue);
int w8100_eth_receive2 ( struct net_device *dev, int queue , int * data,
                         int * len, int *pktFilterSel);

struct eth_tx_desc_struct
{
    volatile UINT32 command_status;
    volatile UINT32 bytecount_reserved;
    volatile unsigned char * buff_pointer; /* Note - This will not work for 64 bit addressing */
    ;
    volatile struct eth_tx_desc_struct * next_desc;
};

struct eth_rx_desc_struct
{
    volatile UINT32 command_status;
    volatile UINT32 buff_size_byte_count;
    volatile unsigned char * buff_pointer;
    volatile struct eth_rx_desc_struct * next_desc;
};

struct net_device_stats
{
    UINT32 rx_packets;
    UINT32 tx_packets;
    UINT32 rx_bytes;
    UINT32 tx_bytes;
    UINT32 rx_errors;
    UINT32 rx_dropped;
    UINT32 tx_dropped;
    UINT32 multicast;
    UINT32 collisions;
    UINT32 rx_length_errors;
    UINT32 rx_crc_errors;
    UINT32 tx_fifo_errors;

};

struct w8100_ethernet_priv
{
    /* other stuff can be added here such as GT96100 counters etc... */
    struct eth_tx_desc_struct *eth_tx_queue[NUM_TX_QUEUES];
    struct eth_rx_desc_struct *eth_rx_queue[NUM_RX_QUEUES];
    struct sk_buff *eth_tx_skb_pointer[NUM_TX_QUEUES][NT];
    struct sk_buff *eth_rx_skb_pointer[NUM_RX_QUEUES][NR];
    unsigned int TDN_ETH[NUM_TX_QUEUES] , TDN_ETH_TMP[NUM_TX_QUEUES] , RDN_ETH[NUM_RX_QUEUES];
    unsigned int ethernet_irq;
    struct net_device_stats stats;
    unsigned char port;
    /*  spinlock_t lock; */
};

extern struct net_device eth_dev;
#endif /* _W8100_ETH_ */

