/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* Purpose:
*    This file contains low level definitions of ethernet.
*
* Public Procedures:
*    None.
*
* Notes:
*    An exception has been made to not always follow the conventional naming
*    conventions for some types in this file. Thus, for example, the BOOLEAN
*    type does not include the file prefix and the enumeration suffix. This
*    is to allow for the useage of shorter and more common type names.
*
*****************************************************************************/

#ifndef _ETHERLOW_H_
#define _ETHERLOW_H_ 
#include "wltypes.h"
#include "osif.h"
#define LE
extern unsigned int internalRegBaseAddr;

/* 2003/04/04, fred chang */
/* since we try to seperate 802.3 driver to release to customer, so define ether */
/* mac registers independantly */
/****************************************/
/*     Base Address Definitions         */
/****************************************/
#ifndef WL_FEMAC_CFG_BASE
#define WL_FEMAC_CFG_BASE         (0x80008000)
#define WL_FEMAC_CFG_SIZE         (0x1000)
#endif

/****************************************/
/*          GENERAL Definitions			*/
/****************************************/

#define NO_BIT          0x00000000
#define BIT0            0x00000001
#define BIT1            0x00000002
#define BIT2            0x00000004
#define BIT3            0x00000008
#define BIT4            0x00000010
#define BIT5            0x00000020
#define BIT6            0x00000040
#define BIT7            0x00000080
#define BIT8            0x00000100
#define BIT9            0x00000200
#define BIT10           0x00000400
#define BIT11           0x00000800
#define BIT12           0x00001000
#define BIT13           0x00002000
#define BIT14           0x00004000
#define BIT15           0x00008000
#define BIT16           0x00010000
#define BIT17           0x00020000
#define BIT18           0x00040000
#define BIT19           0x00080000
#define BIT20           0x00100000
#define BIT21           0x00200000
#define BIT22           0x00400000
#define BIT23           0x00800000
#define BIT24           0x01000000
#define BIT25           0x02000000
#define BIT26           0x04000000
#define BIT27           0x08000000
#define BIT28           0x10000000
#define BIT29           0x20000000
#define BIT30           0x40000000
#define BIT31           0x80000000

#define _1K             0x00000400
#define _2K             0x00000800
#define _4K             0x00001000
#define _8K             0x00002000
#define _16K            0x00004000
#define _32K            0x00008000
#define _64K            0x00010000
#define _128K           0x00020000
#define _256K           0x00040000
#define _512K           0x00080000

#define _1M             0x00100000
#define _2M             0x00200000
#define _3M             0x00300000
#define _4M             0x00400000
#define _5M             0x00500000
#define _6M             0x00600000
#define _7M             0x00700000
#define _8M             0x00800000
#define _9M             0x00900000
#define _10M            0x00a00000
#define _11M            0x00b00000
#define _12M            0x00c00000
#define _13M            0x00d00000
#define _14M            0x00e00000
#define _15M            0x00f00000
#define _16M            0x01000000


#ifdef _EAGLEHW_H_  /* 2003/04/04, fred chang */
#define GT_REG_READ(x,y)  WL_READ_REGS32(x, *(y))
#define GT_REG_WRITE WL_WRITE_REGS32
#else
#define GT_REG_READ(x,y)   ((*(y)) = (*(volatile unsigned long *)(x)))
#define GT_REG_WRITE(x,y)  ((*(volatile unsigned long *)(x)) = y)
#endif

#define NUMBER_OF_MIB_COUNTERS              25

#define 		TIME_OUT					10  /* 1/6 SEC */

#define MAX_NUMBER_OF_DEFINED_REGISTERS     200

/*  SET_REG_BITS(regOffset,bits) -
   gets register offset and bits: a 32bit value. It set to logic '1' in the  
   internal register the bits which given as an input example:
   SET_REG_BITS(0x840,BIT3 | BIT24 | BIT30) - set bits: 3,24 and 30 to logic
   '1' in register 0x840 while the other bits stays as is. */
#define SET_REG_BITS(regOffset,bits)                     \
        (volatile unsigned int)*(unsigned int*)           \
        ( internalRegBaseAddr | regOffset) |=            \
        (unsigned int)WORD_SWAP(bits)

/*  RESET_REG_BITS(regOffset,bits) -
   gets register offset and bits: a 32bit value. It set to logic '0' in the  
   internal register the bits which given as an input example:
   RESET_REG_BITS(0x840,BIT3 | BIT24 | BIT30) - set bits: 3,24 and 30 to logic
   '0' in register 0x840 while the other bits stays as is. */
#define RESET_REG_BITS(regOffset,bits)                     \
        (volatile unsigned int)*(unsigned int*)            \
        ( internalRegBaseAddr |regOffset) &=                \
        ~( (unsigned int)WORD_SWAP(bits) )

#define REG_ADDR(offset) (internalRegBaseAddr | (offset))
#define REG_CONTENT(offset) (volatile unsigned int)     \
        *(unsigned int*)REG_ADDR(offset)

/*typedef enum _bool{false,true} bool; */


typedef enum RegDefaultTable
{
    /* ETHERNET */
    PCR0 = 0,
    PCR1,
    PCXR0,
    PCXR1,
    PCOM0,
    PCOM1,

    SDCR0,
    SDCR
} DEFAULT_TABLE_REGISTERS;

#define     PHY_ADD_REG_VALUE    0x1ee


#define MAX_NUMBER_OF_ETHERNET_PORTS            2

#define MRR_REG_VALUE                           0x00fc0000


#define LL_QUEUE_PRIORITY                       1
#define L_QUEUE_PRIORITY                        2
#define H_QUEUE_PRIORITY                        3
#define HH_QUEUE_PRIORITY                       4


#define ETHERNET0_CAUSE_REGISTER        (WL_FEMAC_CFG_BASE + 0x0450)
#define ETHERNET0_IW2CR_REGISTER        (WL_FEMAC_CFG_BASE + 0x0454)
#define ETHERNET0_MASK_REGISTER         (WL_FEMAC_CFG_BASE + 0x0458)
#define ETHERNET1_CAUSE_REGISTER        (WL_FEMAC_CFG_BASE + 0x0850)
#define ETHERNET1_MASK_REGISTER         (WL_FEMAC_CFG_BASE + 0x0858)
#define SDMA_CAUSE_REGISTER             (WL_FEMAC_CFG_BASE + 0x103A10)
#define SDMA_MASK_REGISTER              (WL_FEMAC_CFG_BASE + 0x103A90)


/* Ethernet Ports */
#define ETHER_PHY_ADDR_REG              (WL_FEMAC_CFG_BASE + 0x0000)
#define ETHER_SMI_REG                   (WL_FEMAC_CFG_BASE + 0x0010)

/* Ethernet0 Ports */
#define ETHER0_PORT_CONFIG_REG          (WL_FEMAC_CFG_BASE + 0x0400)
#define ETHER0_PORT_CONFIG_EXT_REG      (WL_FEMAC_CFG_BASE + 0x0408)
#define ETHER0_PORT_COMM_REG            (WL_FEMAC_CFG_BASE + 0x0410)
#define ETHER0_PORT_STATUS_REG          (WL_FEMAC_CFG_BASE + 0x0418)
#define ETHER0_SER_PARAM_REG            (WL_FEMAC_CFG_BASE + 0x0420)
#define ETHER0_HASH_TBL_PTR_REG         (WL_FEMAC_CFG_BASE + 0x0428)
#define ETHER0_FLOW_CNTRL_SRC_ADDR_L    (WL_FEMAC_CFG_BASE + 0x0430)
#define ETHER0_FLOW_CNTRL_SRC_ADDR_H    (WL_FEMAC_CFG_BASE + 0x0438)
#define ETHER0_SDMA_CONFIG_REG          (WL_FEMAC_CFG_BASE + 0x0440)
#define ETHER0_SDMA_COMM_REG            (WL_FEMAC_CFG_BASE + 0x0448)
#define ETHER0_INT_CAUSE_REG            (WL_FEMAC_CFG_BASE + 0x0450)
#define ETHER0_IW2CR_REG                (WL_FEMAC_CFG_BASE + 0x0454)
#define ETHER0_INT_MASK_REG             (WL_FEMAC_CFG_BASE + 0x0458)

#define ETHER0_TOS_PRIORITY_0_LOW       (WL_FEMAC_CFG_BASE + 0x0460)
#define ETHER0_TOS_PRIORITY_0_HIGH      (WL_FEMAC_CFG_BASE + 0x0464)
#define ETHER0_TOS_PRIORITY_1_LOW       (WL_FEMAC_CFG_BASE + 0x0468)
#define ETHER0_TOS_PRIORITY_1_HIGH      (WL_FEMAC_CFG_BASE + 0x046c)
#define ETHER0_VLAN_TO_PRIORITY         (WL_FEMAC_CFG_BASE + 0x0470)

#define ETHER0_1ST_RX_DESC_PTR0         (WL_FEMAC_CFG_BASE + 0x0480)
#define ETHER0_1ST_RX_DESC_PTR1         (WL_FEMAC_CFG_BASE + 0x0484)
#define ETHER0_1ST_RX_DESC_PTR2         (WL_FEMAC_CFG_BASE + 0x0488)
#define ETHER0_1ST_RX_DESC_PTR3         (WL_FEMAC_CFG_BASE + 0x048C)
#define ETHER0_CURR_RX_DESC_PTR0        (WL_FEMAC_CFG_BASE + 0x04A0)
#define ETHER0_CURR_RX_DESC_PTR1        (WL_FEMAC_CFG_BASE + 0x04A4)
#define ETHER0_CURR_RX_DESC_PTR2        (WL_FEMAC_CFG_BASE + 0x04A8)
#define ETHER0_CURR_RX_DESC_PTR3        (WL_FEMAC_CFG_BASE + 0x04AC)
#define ETHER0_CURR_TX_DESC_PTR0        (WL_FEMAC_CFG_BASE + 0x04E0)
#define ETHER0_CURR_TX_DESC_PTR1        (WL_FEMAC_CFG_BASE + 0x04E4)
#define ETHER0_MIB_COUNT_BASE           (WL_FEMAC_CFG_BASE + 0x0500)

/* Ethernet1 Ports */
#define ETHER1_PORT_CONFIG_REG          (WL_FEMAC_CFG_BASE + 0x0800)
#define ETHER1_PORT_CONFIG_EXT_REG      (WL_FEMAC_CFG_BASE + 0x0808)
#define ETHER1_PORT_COMM_REG            (WL_FEMAC_CFG_BASE + 0x0810)
#define ETHER1_PORT_STATUS_REG          (WL_FEMAC_CFG_BASE + 0x0818)
#define ETHER1_SER_PARAM_REG            (WL_FEMAC_CFG_BASE + 0x0820)
#define ETHER1_HASH_TBL_PTR_REG         (WL_FEMAC_CFG_BASE + 0x0828)
#define ETHER1_FLOW_CNTRL_SRC_ADDR_L    (WL_FEMAC_CFG_BASE + 0x0830)
#define ETHER1_FLOW_CNTRL_SRC_ADDR_H    (WL_FEMAC_CFG_BASE + 0x0838)
#define ETHER1_SDMA_CONFIG_REG          (WL_FEMAC_CFG_BASE + 0x0840)
#define ETHER1_SDMA_COMM_REG            (WL_FEMAC_CFG_BASE + 0x0848)
#define ETHER1_INT_CAUSE_REG            (WL_FEMAC_CFG_BASE + 0x0850)
#define ETHER1_INT_MASK_REG             (WL_FEMAC_CFG_BASE + 0x0858)
#define ETHER1_1ST_RX_DESC_PTR0         (WL_FEMAC_CFG_BASE + 0x0880)
#define ETHER1_1ST_RX_DESC_PTR1         (WL_FEMAC_CFG_BASE + 0x0884)
#define ETHER1_1ST_RX_DESC_PTR2         (WL_FEMAC_CFG_BASE + 0x0888)
#define ETHER1_1ST_RX_DESC_PTR3         (WL_FEMAC_CFG_BASE + 0x088C)
#define ETHER1_CURR_RX_DESC_PTR0        (WL_FEMAC_CFG_BASE + 0x08A0)
#define ETHER1_CURR_RX_DESC_PTR1        (WL_FEMAC_CFG_BASE + 0x08A4)
#define ETHER1_CURR_RX_DESC_PTR2        (WL_FEMAC_CFG_BASE + 0x08A8)
#define ETHER1_CURR_RX_DESC_PTR3        (WL_FEMAC_CFG_BASE + 0x08AC)
#define ETHER1_CURR_TX_DESC_PTR0        (WL_FEMAC_CFG_BASE + 0x08E0)
#define ETHER1_CURR_TX_DESC_PTR1        (WL_FEMAC_CFG_BASE + 0x08E4)
#define ETHER1_MIB_COUNT_BASE           (WL_FEMAC_CFG_BASE + 0x0900)

/* macros for wancom device */
#define ETHERNET_DOWNLOADING_PORT           ETHERNET_PORT1
#define ETHERNET_DOWNLOADING_TX_PORT        TX_ETHERNET_0_LOW+\
                                            ETHERNET_DOWNLOADING_PORT
#define ETHERNET_DOWNLOADING_RX_PORT        RX_ETHERNET_0_LL+\
                                            ETHERNET_DOWNLOADING_PORT
#define ETHERNET_PORT1                      1
#define ETHERNET_PORT0                      0
#define TX_ETHERNET_0_LOW                   8
#define RX_ETHERNET_0_LL                    8
#define TX_ETHERNET_1_LOW                   9
#define RX_ETHERNET_1_LL                    9


#define NUMBER_OF_ETHERNET_RX_PRIO              4
#define NUMBER_OF_ETHERNET_TX_PRIO              2

#define ETHERNET_PORTS_DIFFERENCE_OFFSETS       0x4000

#define ETHERNET0_HH_PRIORITY                   14
#define ETHERNET0_LL_PRIORITY                   8
#define ETHERNET1_HH_PRIORITY                   15

/* this macros are used to enable access to SMI_REG */
#define SMI_OP_CODE_BIT_READ                    1
#define SMI_OP_CODE_BIT_WRITE                   0
#define SMI_BUSY                                1<<28
#define READ_VALID                              1<<27

#define PHY_ADD0                                0x1C
#define PHY_ADD1                                0x1D


/* this is macros for the port configuration register */
#define DUPLEX_MODE             1<<15
#define VLAN_ENABLE             1<<16
#define TOS_ENABLE              1<<17

#define HALF_DUPLEX   0
#define FULL_DUPLEX   1
#define SPEED_100M    1
#define SPEED_10M     0
#define AUTO_ENABLE   1
#define AUTO_DISABLE  0

/* this macros are used to enable access to ETHERNET_PCXR */
#define OVERRIDE_RX_PRIORITY    1<<8
#define DUPLEX_ENABLE           1<<9
#define MIB_CLEAR_MODE          1<<16
#define PORT_SPEED              1<<18
#define RMII_ENABLE             1<<20 /* MII Port1 functions as RMII port0,1 */
#define DSCP_ENABLE             1<<21
#define SPEED_ENABLE            1<<19  /* AUTO negotiate */
#define BYTE_STUFFING_ENABLE    1<<28



/* this macros are used to enable access to ETHERNET_SDCMR */
#define START_TX_HIGH       1<<23
#define START_TX_LOW        1<<24
#define ENABLE_RX_DMA       1<<7
#define ABORT_RECEIVE       1<<15
#define STOP_TX_HIGH        1<<16
#define STOP_TX_LOW         1<<17
#define ABORT_TRANSMIT      1<<31



/* this macros are used to enable access to ETHERNET_SDCR */
#define ETHERNET_SDMA_BURST_SIZE          3




typedef unsigned int ETHERNET_PCR;
typedef unsigned int ETHERNET_PCXR;
typedef unsigned int ETHERNET_PCMR;
typedef unsigned int ETHERNET_PSR;
typedef unsigned int ETHERNET_SDCMR;
typedef unsigned int ETHERNET_SDCR;

typedef unsigned int PHY_ADD_REG;
typedef unsigned int SMI_REG;


typedef struct mibCounters
{
    unsigned int	byteReceived;
    unsigned int	byteSent;
    unsigned int	framesReceived;
    unsigned int	framesSent;
    unsigned int	totalByteReceived;
    unsigned int	totalFramesReceived;
    unsigned int	broadcastFramesReceived;
    unsigned int	multicastFramesReceived;
    unsigned int	cRCError;
    unsigned int	oversizeFrames;
    unsigned int	fragments;
    unsigned int	jabber;
    unsigned int collision;
    unsigned int lateCollision;
    unsigned int	frames64;
    unsigned int	frames65_127;
    unsigned int	frames128_255;
    unsigned int	frames256_511;
    unsigned int	frames512_1023;
    unsigned int	frames1024_MaxSize;
    unsigned int macRxError;
    unsigned int	droppedFrames;
    unsigned int outMulticastFrames;
    unsigned int outBroadcastFrames;
    unsigned int undersizeFrames;
}
STRUCT_MIB_COUNTERS;

/* functions definitions */
bool etherGetPortConfReg (ETHERNET_PCR* portConfReg , unsigned int portNumber);
bool etherSetPortConfReg (ETHERNET_PCR* portConfReg, unsigned int portNumber);
bool etherGetDefaultPortConfReg (ETHERNET_PCR* portConfReg,
                                 unsigned int portNumber);
bool etherModifyDefaultPortConfReg (ETHERNET_PCR* portConfReg,
                                    unsigned int portNumber);
bool etherGetPortConfExtReg (ETHERNET_PCXR* portConfExtReg ,
                             unsigned int portNumber);
bool etherSetPortConfExtReg (ETHERNET_PCXR* portConfExtReg ,
                             unsigned int portNumber);
bool etherGetDefaultPortConfExtReg (ETHERNET_PCXR* portConfExtReg ,
                                    unsigned int portNumber);
bool etherModifyDefaultPortConfExtReg (ETHERNET_PCXR* portConfExtReg ,
                                       unsigned int portNumber);
bool etherGetPortCommandReg (ETHERNET_PCMR* portCommandReg ,
                             unsigned int portNumber);
bool etherSetPortCommandReg (ETHERNET_PCMR* portCommandReg ,
                             unsigned int portNumber);
bool etherGetDefaultPortCommandReg (ETHERNET_PCMR* portCommandReg ,
                                    unsigned int portNumber);
bool etherModifyDefaultPortCommandReg (ETHERNET_PCMR* portCommandReg ,
                                       unsigned int portNumber);
bool etherGetPortboolReg (ETHERNET_PSR* portboolReg, unsigned int portNumber);
bool etherReadMibCounters (STRUCT_MIB_COUNTERS* mibCountersSt ,
                           unsigned int portNumber);
bool etherClearMibCounters (unsigned int portNumber);
bool etherReadMIIReg (unsigned int portNum , unsigned int miiReg,
                      unsigned int* value);
bool etherWriteMIIReg (unsigned int portNum , unsigned int miiReg,
                       unsigned int value);




#define ETHER_GET_PORT_CONF_REG                etherGetPortConfReg
#define ETHER_SET_PORT_CONF_REG                etherSetPortConfReg
#define ETHER_GET_DEFAULT_PORT_CONF_REG        etherGetDefaultPortConfReg
#define ETHER_MODIFY_DEFAULT_PORT_CONF_REG     etherModifyDefaultPortConfReg
#define ETHER_GET_PORT_CONF_EXT_REG            etherGetPortConfExtReg
#define ETHER_SET_PORT_CONF_EXT_REG            etherSetPortConfExtReg
#define ETHER_GET_DEFAULT_PORT_CONF_EXT_REG    etherGetDefaultPortConfExtReg
#define ETHER_MODIFY_DEFAULT_PORT_CONF_EXT_REG etherModifyDefaultPortConfExtReg
#define ETHER_GET_PORT_COMMAND_REG             etherGetPortCommandReg
#define ETHER_SET_PORT_COMMAND_REG             etherSetPortCommandReg
#define ETHER_GET_DEFAULT_PORT_COMMAND_REG     etherGetDefaultPortCommandReg
#define ETHER_MODIFY_DEFAULT_PORT_COMMAND_REG  etherModifyDefaultPortCommandReg
#define ETHER_GET_PORT_STATUS_REG              etherGetPortStatusReg
#define ETHER_READ_MIB_COUNTERS                etherReadMibCounters


/* function declaration */

unsigned int GetDefaultRegisterValue(unsigned int Register);
void SetDefaultRegisterValue(unsigned int Register, unsigned int Value);

#endif

