#if !defined(_HAL_PHY_H_)
#define _HAL_PHY_H_

/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*
*
* Filename: hal_phy.h
* Description: This file contains all the registers settings related to
*              the PHY layer of Condor+Peregrine. 
*
******************************************************************************
*/
//#include "hal_phy_def.h"
//#include "prod_hal_phy.h"

/*-------------------------------------------------------*/
/* [Board Specific Section]                              */
/* Settings for CB32p                                    */
/*-------------------------------------------------------*/
// TBD

/*-------------------------------------------------------*/
/* RF channel settings                                   */
/* Harrier                                               */
/*-------------------------------------------------------*/

#define PHY_REG_VER 0x11032003

typedef struct _ChnlSelRegs_t {
	UINT8 r3c;
	UINT8 r3b;
	UINT8 r3a;
} ChnlSelRegs_t;

#ifdef PEREGRINE
#define	RF_FALCON       0			/* Bill Modify and JL Write it */  
#define	RF_PEREGRINE    1			/* Bill Modify and JL Write it */
typedef struct _PHY_REGS
{
    unsigned char reg;
    unsigned char val;
} PHY_REGS;

typedef struct _SOC2PHY_REGS
{
    unsigned long reg;
    unsigned long val;
} SOC2PHY_REGS;
#endif
extern ChnlSelRegs_t rfch_11g_Regs[];

extern ChnlSelRegs_t rfch_11a_Regs[];


/*-------------------------------------------------------*/
/* RF channel settings                                   */
/* Peregrine rev A and B                                 */
/*-------------------------------------------------------*/
extern  ChnlSelRegs_t rfch_A20Mhz_Regs[];

extern  ChnlSelRegs_t rfch_A40Mhz_Regs[];

extern  ChnlSelRegs_t rfch_A44Mhz_Regs[]; 

extern  ChnlSelRegs_t rfch_B20Mhz_Regs[]; 
extern ChnlSelRegs_t rfch_B40Mhz_Regs[];

extern  ChnlSelRegs_t rfch_B44Mhz_Regs[]; 

#ifdef CHANNEL14_SUPPORT
PHY_REGS rfCh14Regs[2][2]={
  {
    {0x39, 0x1f},
    {0xff, 0xff} },
  {
    {0x39, 0x10},
    {0xff, 0xff} }
};

PHY_REGS bbpCh14Regs[2][18]={
  {
    {0x59, 0x80},
    {0x5a, 0xbe},
    {0x5b, 0xbb},
    {0x5c, 0xb8},
    {0x5d, 0xb4},
    {0x5e, 0xb3},
    {0x61, 0xb6},
    {0x62, 0x80},
    {0x63, 0x84},
    {0x64, 0x8b},
    {0x65, 0x93},
    {0x66, 0x9b},
    {0x67, 0xa4},
    {0x68, 0xab},
    {0x6a, 0x70},
    {0x6b, 0x72},
    {0x6c, 0x40},
    {0xff, 0xff} },
  {
    {0x59, 0xbb},
    {0x5a, 0x87},
    {0x5b, 0x8e},
    {0x5c, 0x94},
    {0x5d, 0x8e},
    {0x5e, 0x83},
    {0x61, 0xb0},
    {0x62, 0xa0},
    {0x63, 0xb5},
    {0x64, 0xb7},
    {0x65, 0xbd},
    {0x66, 0x88},
    {0x67, 0x95},
    {0x68, 0xa2},
    {0x6a, 0x6b},
    {0x6b, 0x6f},
    {0x6c, 0x40},
    {0xff, 0xff} }
};
#endif
#ifdef PEREGRINE
/*-------------------------------------------------------*/
/* [RF Section]                                          */
/* Peregrine initial register settings                   */
/*-------------------------------------------------------*/
// Version A2 Peregrine
PHY_REGS rfInitRegs[] = {
   {0x02, 0x00},
   {0x02, 0x07},
   {0x07, 0xc8},
   {0x09, 0x18},
   {0x0c, 0x05},     /* Register 0c - TX EVM related change */
   {0x0d, 0xd2},
   {0x0e, 0x12},
   {0x0f, 0x0b},
   {0x13, 0x25},     /* Initial Gain in Peregrine to 54dB */
   {0x14, 0x05},
   {0x18, 0x22},
   {0x19, 0x00},
   {0x1e, 0xcf},
   {0x1f, 0xce},
   {0x20, 0x18},
   {0x3d, 0x9c},     /* Set the reference clock to 40mhz */
   {0x3e, 0x00},
   {0xff, 0xff}      /* end of list */
};

// Version B1 Peregrine /* David Modify and JL Write it */
PHY_REGS rfInitRegs_Bx[] = {
   {0x02, 0x00},
   {0x02, 0x07},
   {0x06, 0xC0},     // Continuing sample power control
   {0x08, 0x05},
   {0x0b, 0x04},
   {0x0c, 0x05},     /* Register 0c - TX EVM related change */
   {0x0d, 0xd2},
   {0x0e, 0x12},
   {0x13, 0x25},     /* Initial Gain in Peregrine to 54dB */
   {0x14, 0x05},
   {0x18, 0x22},
   {0x19, 0x02},
   {0x1e, 0x8f},
   {0x1f, 0xce},
   {0x20, 0x18},
   {0x23, 0x54},
   {0x30, 0xb5},
   {0x3a, 0xd7},     /* Set the reference clock to 40mhz */
   {0x3b, 0xeb},
   {0x3c, 0x00},
   {0x49, 0x1b},
   {0xff, 0xff}      /* end of list */
};

/*-------------------------------------------------------*/
/* SOC settings                                          */
/*-------------------------------------------------------*/
#include "ciu.h"
SOC2PHY_REGS soc2phyInitRegs[] = { 
   {0x8000a240, 0x3000},      /* TX_PE Rising edge */
   {0x8000a244, 0x50142020},  /* TX_PE Falling edge */
   {0x8000a660, 0x28},        /* Clock divisor (UFactor) */
#ifndef CONDOR2
   {WL_CIU_CONTROL,0x7136},
#endif
   {0xffffffff, 0xffffffff}
};
#endif

/*-------------------------------------------------------*/
/* [BBP Section]                                         */
/* Condor Baseband initial register settion              */
/*-------------------------------------------------------*/
#ifdef PEREGRINE
PHY_REGS bbpInitRegs_p[] = {
#ifdef CONDOR2
    {0x00, 0x00},   /* BBP_PAGE_REG_0 to page 0 */
    {0x03, 0xff},   /* BBP_SW_RESET soft reset */
    {0x03, 0x00},   /* BBP_SW_RESET out of soft reset */
    {0x07, 0x82},   /* BBP_DEV_CTRL0 11g mode( 11b or 11a) */ //tocheck foo
    {0x17, 0x00},   /* to fix: Frequent TX_WATCHDOG interrupts occurs in noisy environment. 9/24/2003 */
    {0x1b, 0x70},   /* BBP_AGC_REG6 */
    {0x1d, 0x32},   /* BBP_AGC_REG6 */
    {0x1e, 0x0e},   /* BBP_AGC_REG6 */
    {0x1f, 0x87},   /* BBP_CCA_REG2 */
    {0x27, 0x60},   /* BBP_CCA_REG2 */
    {0x28, 0x60},   /* BBP_CCA_REG2 */
    {0x29, 0xe0},   /* BBP_CCA_REG2 */
    {0x2b, 0x28},   /* BBP_PREDISTORTION_1 */
    {0x2c, 0x2c},   /* BBP_PREDISTORTION_1 */
    {0x2f, 0x78},   /* BBP_PREDISTORTION_1 */
    {0x30, 0x64},   /* BBP_PREDISTORTION_1 */
    {0x31, 0x90},   /* BBP_PREDISTORTION_1 */
    {0x35, 0x03},   /* BBP_PREDISTORTION_1 */
    {0x37, 0xa0},   /* BBP_PREDISTORTION_1 */
    {0x38, 0xa0},   /* BBP_PREDISTORTION_1 */
    {0x3f, 0x30},   /* BBP_PREDISTORTION_1 */
    {0x40, 0x30},   /* BBP_PREDISTORTION_1 */
    {0x41, 0xa0},   /* BBP_PREDISTORTION_1 */
    {0x42, 0x41},   /* BBP_PREDISTORTION_1 */
    {0x45, 0x88},   /* BBP_PREDISTORTION_1 */
    {0x4b, 0x40},   /* BBP_PREDISTORTION_1 */
    {0x4d, 0x10},   /* BBP_PREDISTORTION_1 */
    {0x4f, 0x08},   /* BBP_PREDISTORTION_1 */
    {0x50, 0x4c},   /* BBP_PREDISTORTION_1 */
    {0x51, 0x06},   /* BBP_PREDISTORTION_1 */
    {0x52, 0x34},   /* BBP_PREDISTORTION_1 */
    {0x55, 0x02},   /* BBP_PREDISTORTION_1 */
    {0x56, 0x01},   /* BBP_PREDISTORTION_1 */
    {0x57, 0xff},   /* BBP_PREDISTORTION_1 */
    {0x00, 0x80},   /* BBP_PAGE_REG_0 set to page 1 */
    {0x82, 0xfc},   /* reserved ? */
    {0x83, 0xb7},   /* reserved ? */
    {0x87, 0x94},   /* reserved ? */
    {0xa4,0x22},   /* reserved ? */
    {0xb8, 0x02},   /* BBP_CCA_REG13 */
    {0xbf, 0x16},  /* BBP_CCA_REG13 */
    {0xcb, 0x75},   /* Extend short sync for OFDM */
    {0xce, 0x59},   /* Extend short sync for OFDM */
    {0xd6, 0x54},   /* BBP_TIME_RECOV_REG0 new equalizer settings */
    {0xde, 0xa0},   /* BBP_CCA_REG13 */
    {0xe4, 0x7f},   /* BBP_CCA_REG14 */
    {0x00, 0x00},   /* BBP_PAGE_REG_0 to page 0 */
    {0xff, 0xff}
#else
   {0x00, 0x00},     /* BBP_PAGE_REG_0 to page 0 */
   {0x03, 0xff},     /* BBP_SW_RESET soft reset */
   {0x03, 0x00},
   {0x07, 0x88},     /* Register 07 is programmed to 11g mode */
   {0x17, 0x00},
   {0x29, 0xe0},
   //{0x2b, 0x28},   /* AGC Gain Target *//* David Modify and JL Write it */
   {0x2c, 0x2c},     /* AGC initial gain to 54dB *//* David Modify and JL Write it 0x36->0x2f */
   //{0x2f, 0x78},   /* David Modify and JL Write it */
   {0x35, 0x03},     /* rx sensitivity to -93 dBm from -89 dBm, per jackh */
   {0x37, 0xa0},
   {0x38, 0xa0},
   {0x3f, 0x30},     /* TX and RX antennas set to Antenna-0 */
   {0x40, 0x30},     /* David Modify and JL Write it 0x10->0x30 */
   {0x42, 0x41},
   {0x4c, 0x4f},
   {0x00, 0x80},     /* BBP_PAGE_REG_0 set to page 1 */
   {0x82, 0xfc},     /* David Modify and JL Write it */
   {0x83, 0xd7},
   {0x87, 0x94},
   {0xa4, 0x22},     /* TX_PE latency in baseband set to 1us */
   //{0xc3, 0x90},   /* 90: Disable DC loop */
   {0xcb, 0x75},     /* David Modify and JL Write it */
   {0xce, 0x59},
   {0xd6, 0x60},     /* David Modify and JL Write it 0x54->0x60 */
   {0xe4, 0x7f},
   {0x00, 0x00},     /* BBP_PAGE_REG_0 to page 0 */
   {0xff, 0xff}      /* end of list */
#endif
};
#endif

#endif /* _HAL_PHY_H_ */

