/*
*                Copyright 2003, Marvell Semiconductor, Inc.
* This code contains confidential information of Marvell Semiconductor, Inc.
* No rights are granted herein under any patent, mask work right or copyright
* of Marvell or any third party.
* Marvell reserves the right at its sole discretion to request that this code
* be immediately returned to Marvell. This code is provided "as is".
* Marvell makes no warranties, express, implied or otherwise, regarding its
* accuracy, completeness or performance.
*
*
* Filename: hal_rfcal.h
* Description: This file mainly provides data structures for the manufacture HW specified in EEPROM.
*
*
* Public Procedures:
*    
* Notes:
*
*****************************************************************************/
#if !defined(_HAL_RFCAL_H_)
#define _HAL_RFCAL_H_

#ifdef NEWCALDATA

#define MFG_ANNEXTYP1	1
#define MFG_ANNEXTYP2	2
#define MFG_ANNEXTYP3	3
#define MFG_ANNEXTYP4	4
#define MFG_ANNEXTYP5	5
#define MFG_ANNEXTYP6	6
#define MFG_ANNEXTYP7	7
#define MFG_ANNEXTYP8	8
#define MFG_ANNEXTYP9	9
#define MFG_ANNEXTYP10	10
#define MFG_ANNEXTYP11	11
#define MFG_ANNEXTYP12	12
#define MFG_ANNEXTYP13	13

#define MAX_NUMBER_SUBBANDS 		5
#define MAX_POWER_RANGES 			4
#define MAX_POWER_LEVELS			21
#define MIN_POWER_LEVELS 			5
#define MAX_ChX_SETTINGS_TYPE12		14
#define MAX_ChX_SETTINGS_TYPE13		43
#define MAX_CHANNEL_INDEX			16
#define MAX_CALCH_A_STRUCTS_TYPE10	16

#define MFG_CHIPID_8010     0x01
#define MFG_CHIPID_8020     0x02
#define MFG_CHIPID_8030     0x03
#define MFG_CHIPID_8015     0x05

#define MFG_RF_8000			0x00		/* Falcon */
#define MFG_RF_8010			0x01		/* Peregrine (8010 and 8015) */
#define MFG_RF_8030			0x02		/* Harrier (8020 and 8030) */

#define MAX_POWER_ADAPT_LEVELS	 3

#ifdef EEPROM
extern UINT32 MFG_HW_DATA_ADDRESS;
extern UINT32 mfg_start_address_in_eeprom;
#else
#if defined (PEREGRINE) || defined (AP_PGRINE)
#define MFG_HW_DATA_ADDRESS	0xFFFFF000/*0xFFFFFF70 */
#else
#define MFG_HW_DATA_ADDRESS	0xFFFFF000/*0xFFFFFD00 */
#endif
#endif /* end of EEPROM 
*/
#ifdef WB_EXCEPTION
#define MFG_MODE_B	CLIENT_MODE_B
#define MFG_MODE_G	CLIENT_MODE_G
#define MFG_MODE_BG	CLIENT_MODE_BG
#define MFG_MODE_A	CLIENT_MODE_A
#define MFG_MODE_AG	CLIENT_MODE_AUTO
#else /* For AP */
#define MFG_MODE_B	AP_MODE_B_ONLY
#define MFG_MODE_G	AP_MODE_G_ONLY
#define MFG_MODE_BG	AP_MODE_MIXED
#define MFG_MODE_A	AP_MODE_A_ONLY
#define MFG_MODE_AG	AP_MODE_AandG
#endif
typedef  struct _POWER_ADAPT_DATA
{
    UINT8 state;	/* 1: available, 0: not available.*/
    UINT8 Power;	
    UINT8 Reg1e;
    UINT8 Reg1f;
    UINT8 Reg20;
	UINT8 Reg50;
    UINT8 Reg1a;
    UINT8 Reg1b;
    UINT8 Reg1c;
	UINT8 Reg1d;
}
POWER_ADAPT_DATA;

#ifdef MFG_OLD_STRUCT
/* For  PEREGRINE Rev. 0x0A */
typedef struct _PAOPTIONBYTE_VA
{
 	UINT8 PaOption_ExtPAPolar:1;
	UINT8 PaOption_IntExtPa:1;
	UINT8 PaOption_ReserveBits:2;
	UINT8 PaOption_ThermalScaler:2;
	UINT8 PaOption_ReserveBits_2:2;
} 
PACK_END PAOPTIONBYTE_VA; 

typedef struct _EXTPABYTE_VA
{
	UINT8 ExterPA_partId:8;
} 
PACK_END EXTPABYTE_VA; 

typedef struct _ANTBYTE_VA
{
 	UINT8 antDivModeTx:2;
	UINT8 antDivModeRx:2;
	UINT8 antBtConf:4;
} 
PACK_END ANTBYTE_VA; 

typedef struct _TRIBYTE_VA
{ 
 	UINT8		savedCh0:4; 
 	UINT8		savedCh1:4; 
} 
PACK_END TRIBYTE_VA;

#define MAXSAVEDCH 6 
typedef struct _CALVALUE_M_0_WORD
{
 	UINT8		Corr1A; 
 	UINT8		Corr1B1C;  
} 
PACK_END CALVALUE_M_0_WORD_VA;   

typedef struct _CALVALUE_M_1_WORD
{
 	UINT8		Value1A; 
 	UINT8		Step1dB;  
} 
PACK_END CALVALUE_M_1_WORD_VA;   

typedef union _CALVALUE_M_WORD_VA 
{
	CALVALUE_M_0_WORD_VA CalValuePerCh0;
	CALVALUE_M_1_WORD_VA CalValuePerCh1;
} 
PACK_END CALVALUE_M_WORD_VA ; 

typedef struct _PACONFDWORD_VA
{
	UINT8			PaConfig1Eh;
	UINT8			PaConfig1Fh;
	UINT8			PaConfig20h;
	UINT8			CalPnt_pwr;
} 
PACK_END PACONFDWORD_VA; 

typedef struct _CALVALUE_RNG_VA
{
	CALVALUE_M_WORD_VA	PaCal[MAXSAVEDCH];
	UINT8				TopPwr1_13;
	UINT8				BtmPwr1_13;
	UINT8				TopPwr14;
	UINT8				BtmPwr14; 
	PACONFDWORD_VA		PaConfig1_13;
	PACONFDWORD_VA		PaConfig14;
} 
PACK_END CALVALUE_RNG_VA;

typedef struct _ASNDWORD_VA
{
 	UINT32		SN:24; 
 	UINT32		AssemblyHouse:8; 
} 
PACK_END ASNDWORD_VA; 

typedef struct _CALOPTIONBYTE_VA
{
 	UINT8 CalOption_Range1:1;
 	UINT8 CalOption_Range2:1;
 	UINT8 CalOption_Range3:1;
 	UINT8 CalOption_Range4:1;
 	UINT8 CalOption_signal_B:1;
 	UINT8 CalOption_signal_G:1;
	UINT8 CalOption_CalDataOption:1;
	UINT8 CalOption_ReserveBits:1;
} 
PACK_END CALOPTIONBYTE_VA; 

typedef struct _MISC_FLAG_REVA
{
	UINT8 extSleepClock:1;
	UINT8 wakeUp:1;
	UINT8 reserved:6;
} 
PACK_END MISC_FLAG_REVA; 

typedef struct _LCCAPBYTE_VA
{
 	UINT8		LCCapChH:3; 
 	UINT8		LCCapChL:3; 
 	UINT8		LCCapCaled:1;   
 	UINT8		LCCapCaledInv:1;   
} 
PACK_END LCCAPBYTE_VA;  

typedef struct _SPISIZEBYTE_VA
{
 	UINT8		size:7; 
 	UINT8		unit:1; 
} 
PACK_END SPISIZEBYTE_VA; 

typedef struct _TOOLVERDWORD_VA
{
	UINT16			TestToolVersion;
	UINT16			TestDllVersion;
} 
PACK_END TOOLVERDWORD_VA; 

/************************************************************ 
*		Marvell HW informational (main) structure from here
*************************************************************/
typedef PACK_START struct _HW_INFO_HEAD
{
    UINT8               Type;
    UINT8               CheckSum;
    UINT16              Length;
    UINT32              Next;
}
PACK_END HW_INFO_HEAD;

typedef PACK_START struct _ANT_CONF
{
    UINT8 Tx:2;
    UINT8 Rx:2;
    UINT8 BtCfg:4;
}
PACK_END ANT_CONF;

typedef PACK_START struct _PA_OPTION
{
    UINT8 ExtPAPolarG:1; 
    UINT8 ExtPAPolarA:1;    
    UINT8 IntExtPaG:1;
    UINT8 IntExtPaA:1;        
    UINT8 ThermalScalar:2;
    UINT8 Reserved:2;
}
PACK_END PA_OPTION;

typedef PACK_START struct _EXT_PA_ID
{
    UINT8 ExtPAVendorInfoG:4; 
    UINT8 ExtPAVendorInfoA:4; 
}
PACK_END EXT_PA_ID;

typedef PACK_START struct _MISC_FLAG
{
    UINT8 ExtXTALOscSrc:1; 
    UINT8 ExtSleepClk:1;
    UINT8 WlanWakeUp:1;
    UINT8 Reserved:5;
}
PACK_END MISC_FLAG;

typedef PACK_START struct _LNA_OPTION
{
    UINT8 IntExtLnaG:1; 
    UINT8 IntExtLnaA:1;
    UINT8 Reserved:6;
}
PACK_END LNA_OPTION;

typedef PACK_START struct _EXT_LNA_ID
{
    UINT8 ExtLnaVenderInfoG:4; 
    UINT8 ExtLnaVenderInfoA:4; 
}
PACK_END EXT_LNA_ID;

typedef PACK_START struct _ASSEMBLY_CODE
{
    UINT8 House:4;
    UINT8 GeoLc:4;
}
PACK_END ASSEMBLY_CODE;

typedef PACK_START struct _BBU_RF_VER
{
    UINT8 MinorVer:4;
    UINT8 MajorVer:4;
}
PACK_END BBU_RF_VER;

typedef PACK_START struct _SPI_SIZE
{
    UINT8 Value:7;
    UINT8 InMegabit:1;
}
PACK_END SPI_SIZE;

typedef PACK_START struct _HW_INFO_MAIN_VA
{
	UINT8				PcbRev[8];
	UINT8				StructureRevision;
	PAOPTIONBYTE_VA		PaOption; 
	EXTPABYTE_VA		ExterPA;
	ANTBYTE_VA			AntCfg;
	UINT32				PtrFirstAnnex; 
	UINT8				numOfSavedCh; 
	TRIBYTE_VA			savedChannel[3]; 
	CALVALUE_RNG_VA		range1CalValue; 
 	ASNDWORD_VA			Asn; 
	UINT8				CountryCode;
	CALOPTIONBYTE_VA	CalOption;
	MISC_FLAG_REVA		miscFlag;
	LCCAPBYTE_VA		LCCap;
	UINT8				SocVers;
	UINT8				RfVers;
 	SPISIZEBYTE_VA		SpiSize;
	UINT8				AntPartInfo;
	TOOLVERDWORD_VA		ToolVersion; 
 	UINT16				Xosc;
	UINT8				reserved;
	UINT8				Cs;
}
PACK_END HW_INFO_MAIN_VA;

typedef PACK_START struct _HW_INFO_MAIN
{
	HW_INFO_HEAD		Head;
	UINT8 				DesignType;
	UINT8 				MajorRev;
	UINT8 				MinorRev;
	UINT8 				Variation;
	ANT_CONF			AntConf;
	UINT8				AntPartId;
	PA_OPTION			PaOption;
	EXT_PA_ID			ExtPaId;
	UINT8				RegionCode;	
	MISC_FLAG			MiscFlag;
	LNA_OPTION			LNAOption;
	EXT_LNA_ID			LNAId;
	ASSEMBLY_CODE		AssHouse;
	UINT8				SN[3];
	UINT16				SocID;
	UINT8				SocVersion;
	SPI_SIZE			SpiSize;
	UINT8				BbuId;
	BBU_RF_VER			BbuVersion;	
	UINT8				RfId;
	BBU_RF_VER			RfVersion;
	UINT32				TestVersion;
	UINT32				MfgVersion;									
}
PACK_END HW_INFO_MAIN;

/************************************************************ 
*			Marvell Annex type1
*************************************************************/
typedef PACK_START struct _PA_CONFIG
{
    UINT8 Reg1e;
    UINT8 Reg1f;
    UINT8 Reg20;
    SINT8 PwrSet4Cal;
}
PACK_END PA_CONFIG;

typedef PACK_START struct _PA_CAL_DATA
{
        SINT8 ChxRgN_1a;
        SINT8 ChxRgN_1b1c;
}
PACK_END PA_CAL_DATA;
#if 0
typedef PACK_START struct _PWR_RANGEX_CALCH_B
{
	PA_CAL_DATA	  PaCalData[6];
}
PACK_END PWR_RANGEX_CALCH_B;
#endif
typedef PACK_START struct _PWR_RANGEX_DATA
{
    PA_CAL_DATA			PaCalData[6];
    UINT8         		TopPwrRangeCh1_13;
    UINT8         		BtmPwrRangeCh1_13;
    UINT8         		TopPwrRangeCh14;
    UINT8         		BtmPwrRangeCh14;
    PA_CONFIG     		PaConfigCh1_13;
    PA_CONFIG     		PaConfigCh14;
}
PACK_END PWR_RANGEX_DATA;

typedef PACK_START struct _ANNEX_TYPE1
{
	HW_INFO_HEAD		Head;
	PWR_RANGEX_DATA  	PwrRangeData[3];
}
PACK_END ANNEX_TYPE1;

/************************************************************ 
*			Marvell Annex type2
*************************************************************/
typedef PACK_START struct _ANNEX_TYPE2
{
	HW_INFO_HEAD		Head;
	UINT32 				RFregistersPtr; 
}
PACK_END ANNEX_TYPE2;

/************************************************************ 
*			Marvell Annex type3
*************************************************************/
typedef PACK_START struct _ANNEX_TYPE3
{
	HW_INFO_HEAD		Head;
	UINT32 				BBPregistersPtr; 
}
PACK_END ANNEX_TYPE3;

/************************************************************ 
*			Marvell Annex type4
*************************************************************/
typedef PACK_START struct _PA_CAL_DATA_G
{
        SINT8 ChxRgN_Byte0;
}
PACK_END PA_CAL_DATA_G;

typedef PACK_START struct _PWR_RANGEX_CALCH_G
{
    PA_CAL_DATA_G  PaCalData[6];
}
PACK_END PWR_RANGEX_CALCH_G;

typedef PACK_START struct _ANNEX_TYPE4
{
	HW_INFO_HEAD		Head;
	PWR_RANGEX_CALCH_G  PwrRangeDataG[MAX_POWER_RANGES];
}
PACK_END ANNEX_TYPE4;

/************************************************************ 
*			Marvell Annex type5
*************************************************************/
typedef PACK_START struct _ANNEX_TYPE5
{
	HW_INFO_HEAD	Head;
	UINT8			LedPolority[8];
}
PACK_END ANNEX_TYPE5;

/************************************************************ 
*			Marvell Annex type6
*************************************************************/
typedef PACK_START struct _ANNEX_TYPE6
{
	HW_INFO_HEAD	Head;
	UINT8			UsbVendorProductStr[24];
}
PACK_END ANNEX_TYPE6;

/************************************************************ 
*			Marvell Annex type7 and type8
*************************************************************/
typedef struct _PWR_REGS_1A1D
{
   UINT8 r1d:1;
   UINT8 Reserved:1;
   UINT8 r1a:6;
}
PACK_END PWR_REGS_1A1D;

typedef struct _PWR_REGS
{
   UINT8 RwrInDbm;
   UINT8 r1b;
   UINT8 r1c;
   PWR_REGS_1A1D R1a1d;
}
PACK_END PWR_REGS;

typedef PACK_START struct _ANNEX_TYPE7
{
	HW_INFO_HEAD		Head;
	PWR_REGS			pwr_regs[MAX_POWER_LEVELS];
}
PACK_END ANNEX_TYPE7;

#define ANNEX_TYPE8 ANNEX_TYPE7

/************************************************************ 
*			Marvell Annex type9
*************************************************************/
typedef PACK_START struct _CAL_OPTION_TYPE9
{
    UINT8 RangexCalibrated:4;
    UINT8 bDataRateCalibrated:1;
    UINT8 gDataRateCalibrated:1;
    UINT8 CalMethodOption:1;
	UINT8 LCTank_Flag:1;
}
PACK_END CAL_OPTION_TYPE9;

typedef PACK_START struct _LCTANKBYTE_TPYE9_
{
    UINT8 CalCh1To7:4;      
    UINT8 CalCh8To14:4;        
}
PACK_END LCTANKBYTE_TPYE9;

typedef PACK_START struct _CH_PAIR
{
    UINT8 First:4;
    UINT8 Secnd:4;
}
PACK_END CH_PAIR;

typedef PACK_START struct _ANNEX_TYPE9
{
	HW_INFO_HEAD		Head;
	CAL_OPTION_TYPE9    CalOption;
	LCTANKBYTE_TPYE9    LctCal;
	UINT8 				RfXtalCal;
	UINT8				Reserve1;
	UINT8 				LNA_12;
	UINT8				LNA_13;
	UINT8				Reserve2;
	UINT8				Reserve3;
	UINT8               NumSavedCh;
	CH_PAIR             ChPair[3];
	PWR_RANGEX_DATA     PwrRangeData[MAX_POWER_RANGES];
}
PACK_END ANNEX_TYPE9;

/************************************************************ 
*			Marvell Annex type10
*************************************************************/
typedef PACK_START struct _CAL_OPTION_TYPE10
{
    UINT8 RangexCalibrated:4;
	UINT8 Reserved1:2;
    UINT8 CalMethodOption:1;
	UINT8 Reserved2:1;
}
PACK_END CAL_OPTION_TYPE10;

typedef PACK_START struct _POWER_RANGE_STRUCT_
{
	UINT8 		   Calibrated_Power[MAX_POWER_RANGES];
	UINT8		   Max_Power[MAX_POWER_RANGES];
	UINT8		   Min_Power[MAX_POWER_RANGES];
}
PACK_END POWER_RANGE_STRUCT;

typedef PACK_START struct _PWR_RANGEX_CALCH_A
{
	PA_CAL_DATA	PaCalData[MAX_POWER_RANGES];
} 
PACK_END PWR_RANGEX_CALCH_A;

typedef PACK_START struct _ANNEX_TYPE10
{
	HW_INFO_HEAD		Head;
	CAL_OPTION_TYPE10   CalOption;
	UINT8 				RfXtalCal;
	UINT8 				LNA_12;
	UINT8 				LNA_13;
	UINT8 				dot11J_Offset_1AH;
	UINT8				Reserve;
	UINT8               NumSubBands;
	UINT8				NumSavedCh_Band[MAX_NUMBER_SUBBANDS];
	UINT8	            ChannelIndex[MAX_CHANNEL_INDEX];
	POWER_RANGE_STRUCT  Power_SubBand[MAX_NUMBER_SUBBANDS];
	PWR_RANGEX_CALCH_A	PwrRangeDataA[MAX_CALCH_A_STRUCTS_TYPE10];
}
PACK_END ANNEX_TYPE10;

/************************************************************ 
*			Marvell Annex type11
*************************************************************/
typedef PACK_START struct _SUBBAND_POWER_TABLE_
{
	UINT8				  SubBandIndex;
	UINT8				  PaConfig_50;
	UINT8				  NumLevels;
	UINT8				  Reserved;
	UINT16				  HighestFrequency;
	UINT16				  LowestFrequency;
	PWR_REGS			  pwr_regs[MAX_POWER_LEVELS];
} PACK_END SUBBAND_POWER_TABLE;

typedef PACK_START struct ANNEX_TYPE11
{
	HW_INFO_HEAD		Head;
	SUBBAND_POWER_TABLE	SubBand[MAX_NUMBER_SUBBANDS];
}
PACK_END ANNEX_TYPE11;

/************************************************************ 
*			Marvell Annex type12
*************************************************************/
typedef PACK_START struct _CHANNEL_SETTINGS_STRUCTURE
{
	UINT8 ChannelIndex;
	UINT8 Reg3a;
	UINT8 Reg3b;
	UINT8 Reg3c;
	UINT8 Reg3d;
	UINT8 BWOptionsFlag;
	UINT16 Ch_Frequency;
}
PACK_END CHANNEL_SETTINGS_STRUCTURE;

typedef PACK_START struct ANNEX__TYPE12
{
	HW_INFO_HEAD		Head;
	UINT8				LO_ReadBack;
	UINT8				RFReg07;
	UINT8				RFReg3e;
	UINT8				Reserved;
	CHANNEL_SETTINGS_STRUCTURE ChX_Setting[MAX_ChX_SETTINGS_TYPE12];
}
PACK_END ANNEX_TYPE12;

/************************************************************ 
*			Marvell Annex type13
*************************************************************/
typedef PACK_START struct ANNEX_TYPE13
{
	HW_INFO_HEAD		Head;
	UINT8				LO_ReadBack;
	UINT8				RFReg07;
	UINT8				RFReg3e;
	UINT8				Reserved;
	CHANNEL_SETTINGS_STRUCTURE ChX_Setting[MAX_ChX_SETTINGS_TYPE13];
}
PACK_END ANNEX_TYPE13;
#else /* MFG_NEW_STRUCT	*/
/* For  PEREGRINE Rev. 0x0A */
typedef struct _PAOPTIONBYTE_VA
{
 	UINT8 PaOption_ExtPAPolar:1;
	UINT8 PaOption_IntExtPa:1;
	UINT8 PaOption_ReserveBits:2;
	UINT8 PaOption_ThermalScaler:2;
	UINT8 PaOption_ReserveBits_2:2;
} 
PACK_END PAOPTIONBYTE_VA; 

typedef struct _EXTPABYTE_VA
{
	UINT8 ExterPA_partId:8;
} 
PACK_END EXTPABYTE_VA; 

typedef struct _ANTBYTE_VA
{
 	UINT8 antDivModeTx:2;
	UINT8 antDivModeRx:2;
	UINT8 antBtConf:4;
} 
PACK_END ANTBYTE_VA; 

typedef struct _TRIBYTE_VA
{ 
 	UINT8		savedCh0:4; 
 	UINT8		savedCh1:4; 
} 
PACK_END TRIBYTE_VA;

#define MAXSAVEDCH 6 
typedef struct _CALVALUE_M_0_WORD
{
 	UINT8		Corr1B1C;
 	UINT8		Corr1A;   
} 
PACK_END CALVALUE_M_0_WORD_VA;   

typedef struct _CALVALUE_M_1_WORD
{
 	UINT8		Step1dB; 
 	UINT8		Value1A;  
} 
PACK_END CALVALUE_M_1_WORD_VA;   

typedef union _CALVALUE_M_WORD_VA 
{
	CALVALUE_M_0_WORD_VA CalValuePerCh0;
	CALVALUE_M_1_WORD_VA CalValuePerCh1;
} 
PACK_END CALVALUE_M_WORD_VA ; 

typedef struct _PACONFDWORD_VA
{
	UINT8			CalPnt_pwr;
	UINT8			PaConfig20h;
	UINT8			PaConfig1Fh;
	UINT8			PaConfig1Eh;
} 
PACK_END PACONFDWORD_VA; 

typedef struct _CALVALUE_RNG_VA
{
	CALVALUE_M_WORD_VA	PaCal[MAXSAVEDCH];
	UINT8				BtmPwr14;
	UINT8				TopPwr14;
	UINT8				BtmPwr1_13;
	UINT8				TopPwr1_13;	 
	PACONFDWORD_VA		PaConfig1_13;
	PACONFDWORD_VA		PaConfig14;
} 
PACK_END CALVALUE_RNG_VA;

typedef struct _ASNDWORD_VA
{
#if 1
 	UINT32		SN:24; 
 	UINT32		AssemblyHouse:8; 
#else
 	UINT32		AssemblyHouse:8; 
 	UINT32		SN:24; 
#endif
} 
PACK_END ASNDWORD_VA; 

typedef struct _CALOPTIONBYTE_VA
{
 	UINT8 CalOption_Range1:1;
 	UINT8 CalOption_Range2:1;
 	UINT8 CalOption_Range3:1;
 	UINT8 CalOption_Range4:1;
 	UINT8 CalOption_signal_B:1;
 	UINT8 CalOption_signal_G:1;
	UINT8 CalOption_CalDataOption:1;
	UINT8 CalOption_ReserveBits:1;
} 
PACK_END CALOPTIONBYTE_VA; 

typedef struct _MISC_FLAG_REVA
{
	UINT8 extSleepClock:1;
	UINT8 wakeUp:1;
	UINT8 reserved:6;
} 
PACK_END MISC_FLAG_REVA; 

typedef struct _LCCAPBYTE_VA
{
 	UINT8		LCCapChH:3; 
 	UINT8		LCCapChL:3; 
 	UINT8		LCCapCaled:1;   
 	UINT8		LCCapCaledInv:1;   
} 
PACK_END LCCAPBYTE_VA;  

typedef struct _SPISIZEBYTE_VA
{
 	UINT8		size:7; 
 	UINT8		unit:1; 
} 
PACK_END SPISIZEBYTE_VA; 

typedef struct _TOOLVERDWORD_VA
{
	UINT16			TestDllVersion;
	UINT16			TestToolVersion;
} 
PACK_END TOOLVERDWORD_VA; 

/************************************************************ 
*		Marvell HW informational (main) structure from here
*************************************************************/
typedef PACK_START struct _HW_INFO_HEAD
{
    UINT16              Length;
    UINT8               CheckSum;
    UINT8               Type;
    UINT32              Next;
}
PACK_END HW_INFO_HEAD;

typedef PACK_START struct _ANT_CONF
{
    UINT8 Tx:2;
    UINT8 Rx:2;
    UINT8 BtCfg:4;
}
PACK_END ANT_CONF;

typedef PACK_START struct _PA_OPTION
{
    UINT8 ExtPAPolarG:1; 
    UINT8 ExtPAPolarA:1;    
    UINT8 IntExtPaG:1;
    UINT8 IntExtPaA:1;        
    UINT8 ThermalScalar:2;
    UINT8 Reserved:2;
}
PACK_END PA_OPTION;

typedef PACK_START struct _EXT_PA_ID
{
    UINT8 ExtPAVendorInfoG:4; 
    UINT8 ExtPAVendorInfoA:4; 
}
PACK_END EXT_PA_ID;

typedef PACK_START struct _MISC_FLAG
{
    UINT8 ExtXTALOscSrc:1; 
    UINT8 ExtSleepClk:1;
    UINT8 WlanWakeUp:1;
    UINT8 Reserved:5;
}
PACK_END MISC_FLAG;

typedef PACK_START struct _LNA_OPTION
{
    UINT8 IntExtLnaG:1; 
    UINT8 IntExtLnaA:1;
    UINT8 Reserved:6;
}
PACK_END LNA_OPTION;

typedef PACK_START struct _EXT_LNA_ID
{
    UINT8 ExtLnaVenderInfoG:4; 
    UINT8 ExtLnaVenderInfoA:4; 
}
PACK_END EXT_LNA_ID;

typedef PACK_START struct _ASSEMBLY_CODE
{
    UINT8 House:4;
    UINT8 GeoLc:4;
}
PACK_END ASSEMBLY_CODE;

typedef PACK_START struct _BBU_RF_VER
{
    UINT8 MinorVer:4;
    UINT8 MajorVer:4;
}
PACK_END BBU_RF_VER;

typedef PACK_START struct _SPI_SIZE
{
    UINT8 Value:7;
    UINT8 InMegabit:1;
}
PACK_END SPI_SIZE;

typedef PACK_START struct _HW_INFO_MAIN_VA
{
	UINT8				PcbRev[8]; 
	ANTBYTE_VA			AntCfg;
	EXTPABYTE_VA		ExterPA;
	PAOPTIONBYTE_VA		PaOption;
	UINT8				StructureRevision;
	UINT32				PtrFirstAnnex;  
	TRIBYTE_VA			savedChannel[3];
	UINT8				numOfSavedCh;		 
	CALVALUE_RNG_VA		range1CalValue; 
 	ASNDWORD_VA			Asn; 
	LCCAPBYTE_VA		LCCap;
	MISC_FLAG_REVA		miscFlag;
	CALOPTIONBYTE_VA	CalOption;
	UINT8				CountryCode;
	UINT8				AntPartInfo;
 	SPISIZEBYTE_VA		SpiSize;
	UINT8				RfVers;
	UINT8				SocVers;
	TOOLVERDWORD_VA		ToolVersion; 
	UINT8				Cs;
	UINT8				reserved;
 	UINT16				Xosc;
}
PACK_END HW_INFO_MAIN_VA;

typedef PACK_START struct _HW_INFO_MAIN
{
	HW_INFO_HEAD		Head;
	UINT8 				Variation;
	UINT8 				MinorRev;
	UINT8 				MajorRev;
	UINT8 				DesignType;
	EXT_PA_ID			ExtPaId;
	PA_OPTION			PaOption;
	UINT8				AntPartId;
	ANT_CONF			AntConf;
	EXT_LNA_ID			LNAId;
	LNA_OPTION			LNAOption;
	MISC_FLAG			MiscFlag;
	UINT8				RegionCode;
	UINT8				SN[3];
	ASSEMBLY_CODE		AssHouse;
	SPI_SIZE			SpiSize;
	UINT8				SocVersion;
	UINT16				SocID;
	BBU_RF_VER			RfVersion;
	UINT8				RfId;
	BBU_RF_VER			BbuVersion;
	UINT8				BbuId;
	UINT32				TestVersion;
	UINT32				MfgVersion;									
}
PACK_END HW_INFO_MAIN;

/************************************************************ 
*			Marvell Annex type1
*************************************************************/
typedef PACK_START struct _PA_CONFIG
{
    SINT8 PwrSet4Cal;
    UINT8 Reg20;
    UINT8 Reg1f;
    UINT8 Reg1e;
}
PACK_END PA_CONFIG;

typedef PACK_START struct _PA_CAL_DATA
{
        SINT8 ChxRgN_1b1c;
        SINT8 ChxRgN_1a;
}
PACK_END PA_CAL_DATA;
#if 0
typedef PACK_START struct _PWR_RANGEX_CALCH_B
{
	PA_CAL_DATA	  PaCalData[6];
}
PACK_END PWR_RANGEX_CALCH_B;
#endif
typedef PACK_START struct _PWR_RANGEX_DATA
{
    PA_CAL_DATA			PaCalData[6];
    UINT8         		BtmPwrRangeCh14;
    UINT8         		TopPwrRangeCh14;
    UINT8         		BtmPwrRangeCh1_13;
    UINT8         		TopPwrRangeCh1_13;
    PA_CONFIG     		PaConfigCh1_13;
    PA_CONFIG     		PaConfigCh14;
}
PACK_END PWR_RANGEX_DATA;

typedef PACK_START struct _ANNEX_TYPE1
{
	HW_INFO_HEAD		Head;
	PWR_RANGEX_DATA  	PwrRangeData[3];
}
PACK_END ANNEX_TYPE1;

/************************************************************ 
*			Marvell Annex type2
*************************************************************/
typedef PACK_START struct _ANNEX_TYPE2
{
	HW_INFO_HEAD		Head;
	UINT32 				RFregistersPtr; 
}
PACK_END ANNEX_TYPE2;

/************************************************************ 
*			Marvell Annex type3
*************************************************************/
typedef PACK_START struct _ANNEX_TYPE3
{
	HW_INFO_HEAD		Head;
	UINT32 				BBPregistersPtr; 
}
PACK_END ANNEX_TYPE3;

/************************************************************ 
*			Marvell Annex type4
*************************************************************/
typedef PACK_START struct _PA_CAL_DATA_G
{
        SINT8 ChxRgN_Byte0;
}
PACK_END PA_CAL_DATA_G;

typedef PACK_START struct _PWR_RANGEX_CALCH_G
{
    PA_CAL_DATA_G  PaCalData[6];
}
PACK_END PWR_RANGEX_CALCH_G;

typedef PACK_START struct _ANNEX_TYPE4
{
	HW_INFO_HEAD		Head;
	PWR_RANGEX_CALCH_G  PwrRangeDataG[MAX_POWER_RANGES];
}
PACK_END ANNEX_TYPE4;

/************************************************************ 
*			Marvell Annex type5
*************************************************************/
typedef PACK_START struct _ANNEX_TYPE5
{
	HW_INFO_HEAD	Head;
	UINT8			LedPolority[8];
}
PACK_END ANNEX_TYPE5;

/************************************************************ 
*			Marvell Annex type6
*************************************************************/
typedef PACK_START struct _ANNEX_TYPE6
{
	HW_INFO_HEAD	Head;
	UINT8			UsbVendorProductStr[24];
}
PACK_END ANNEX_TYPE6;

/************************************************************ 
*			Marvell Annex type7 and type8
*************************************************************/
typedef struct _PWR_REGS_1A1D
{
   UINT8 r1d:1;
   UINT8 Reserved:1;
   UINT8 r1a:6;
}
PACK_END PWR_REGS_1A1D;

typedef struct _PWR_REGS
{
   PWR_REGS_1A1D R1a1d;
   UINT8 r1c;
   UINT8 r1b;
   UINT8 RwrInDbm;
}
PACK_END PWR_REGS;

typedef PACK_START struct _ANNEX_TYPE7
{
	HW_INFO_HEAD		Head;
	PWR_REGS			pwr_regs[MAX_POWER_LEVELS];
}
PACK_END ANNEX_TYPE7;

#define ANNEX_TYPE8 ANNEX_TYPE7

/************************************************************ 
*			Marvell Annex type9
*************************************************************/
typedef PACK_START struct _CAL_OPTION_TYPE9
{
    UINT8 RangexCalibrated:4;
    UINT8 bDataRateCalibrated:1;
    UINT8 gDataRateCalibrated:1;
    UINT8 CalMethodOption:1;
	UINT8 LCTank_Flag:1;
}
PACK_END CAL_OPTION_TYPE9;

typedef PACK_START struct _LCTANKBYTE_TPYE9_
{
    UINT8 CalCh1To7:4;      
    UINT8 CalCh8To14:4;        
}
PACK_END LCTANKBYTE_TPYE9;

typedef PACK_START struct _CH_PAIR
{
    UINT8 First:4;
    UINT8 Secnd:4;
}
PACK_END CH_PAIR;

typedef PACK_START struct _ANNEX_TYPE9
{
	HW_INFO_HEAD		Head;
	UINT8				Reserve1;
	UINT8 				RfXtalCal;
	LCTANKBYTE_TPYE9    LctCal;
	CAL_OPTION_TYPE9    CalOption;
	UINT8				Reserve3;
	UINT8				Reserve2;
	UINT8				LNA_13;
	UINT8 				LNA_12;
	CH_PAIR             ChPair[3];
	UINT8               NumSavedCh;
	PWR_RANGEX_DATA     PwrRangeData[MAX_POWER_RANGES];
}
PACK_END ANNEX_TYPE9;

/************************************************************ 
*			Marvell Annex type10
*************************************************************/
typedef PACK_START struct _CAL_OPTION_TYPE10
{
    UINT8 RangexCalibrated:4;
	UINT8 Reserved1:2;
    UINT8 CalMethodOption:1;
	UINT8 Reserved2:1;
}
PACK_END CAL_OPTION_TYPE10;

typedef PACK_START struct _POWER_RANGE_STRUCT_
{
	UINT8 		   Calibrated_Power[MAX_POWER_RANGES];
	UINT8		   Max_Power[MAX_POWER_RANGES];
	UINT8		   Min_Power[MAX_POWER_RANGES];
}
PACK_END POWER_RANGE_STRUCT;

typedef PACK_START struct _PWR_RANGEX_CALCH_A
{
	PA_CAL_DATA	PaCalData[MAX_POWER_RANGES];
} 
PACK_END PWR_RANGEX_CALCH_A;

typedef PACK_START struct _ANNEX_TYPE10
{
	HW_INFO_HEAD		Head;
	UINT8 				LNA_13;
	UINT8 				LNA_12;
	UINT8 				RfXtalCal;
	CAL_OPTION_TYPE10   CalOption;
	UINT8				NumSavedCh_Band1;
	UINT8               NumSubBands;
	UINT8				Reserve;
	UINT8 				dot11J_Offset_1AH;	
	UINT8				NumSavedCh_Band[4];
	UINT8	            ChannelIndex[MAX_CHANNEL_INDEX];
	POWER_RANGE_STRUCT  Power_SubBand[MAX_NUMBER_SUBBANDS];
	PWR_RANGEX_CALCH_A	PwrRangeDataA[MAX_CALCH_A_STRUCTS_TYPE10];
}
PACK_END ANNEX_TYPE10;

/************************************************************ 
*			Marvell Annex type11
*************************************************************/
typedef PACK_START struct _SUBBAND_POWER_TABLE_
{
	UINT8				  Reserved;
	UINT8				  NumLevels;
	UINT8				  PaConfig_50;
	UINT8				  SubBandIndex;
	UINT16				  LowestFrequency;
	UINT16				  HighestFrequency;
	PWR_REGS			  pwr_regs[MAX_POWER_LEVELS];
} PACK_END SUBBAND_POWER_TABLE;

typedef PACK_START struct ANNEX_TYPE11
{
	HW_INFO_HEAD		Head;
	SUBBAND_POWER_TABLE	SubBand[MAX_NUMBER_SUBBANDS];
}
PACK_END ANNEX_TYPE11;

/************************************************************ 
*			Marvell Annex type12
*************************************************************/
typedef PACK_START struct _CHANNEL_SETTINGS_STRUCTURE
{
	UINT8 Reg3c;
	UINT8 Reg3b;
	UINT8 Reg3a;
	UINT8 ChannelIndex;
	UINT16 Ch_Frequency;
	UINT8 BWOptionsFlag;
	UINT8 Reg3d;
}
PACK_END CHANNEL_SETTINGS_STRUCTURE;

typedef PACK_START struct ANNEX__TYPE12
{
	HW_INFO_HEAD		Head;
	UINT8				Reserved;
	UINT8				RFReg3e;
	UINT8				RFReg07;
	UINT8				LO_ReadBack;
	CHANNEL_SETTINGS_STRUCTURE ChX_Setting[MAX_ChX_SETTINGS_TYPE12];
}
PACK_END ANNEX_TYPE12;

/************************************************************ 
*			Marvell Annex type13
*************************************************************/
typedef PACK_START struct ANNEX_TYPE13
{
	HW_INFO_HEAD		Head;
	UINT8				Reserved;
	UINT8				RFReg3e;
	UINT8				RFReg07;
	UINT8				LO_ReadBack;
	CHANNEL_SETTINGS_STRUCTURE ChX_Setting[MAX_ChX_SETTINGS_TYPE13];
}
PACK_END ANNEX_TYPE13;
#endif /* end of MFG_OLD_STRUCT */

typedef PACK_START struct _MFG_HW_INFO
{
    HW_INFO_MAIN	*hwInfo;
	HW_INFO_MAIN_VA	*hwInfoA;
	ANNEX_TYPE1		*type1;
	ANNEX_TYPE2		*type2;
	ANNEX_TYPE3		*type3;
	ANNEX_TYPE4		*type4;
	ANNEX_TYPE5		*type5;
	ANNEX_TYPE6		*type6;
	ANNEX_TYPE7		*type7;
	ANNEX_TYPE8		*type8;
	ANNEX_TYPE9		*type9;
	ANNEX_TYPE10	*type10;
	ANNEX_TYPE11	*type11;
	ANNEX_TYPE12	*type12;
	ANNEX_TYPE13	*type13;
}
PACK_END MFG_HW_INFO;

#if defined (PEREGRINE) || defined (AP_PGRINE)
#ifdef MFG_OLD_STRUCT
#define mfg_GetHwInfoDataRev(addr) (*((UINT8 *)addr+8))
#else  /* for MFG_NEW_STRUCT*/
#define mfg_GetHwInfoDataRev(addr) (*((UINT8 *)addr+11))
#endif /* MFG_OLD_STRUCT */
#define mfg_GetHwInfoRev(pMfgHwData) (pMfgHwData->hwInfoA->StructureRevision)
#define mfg_GetFirAnnexAddr(pMfgHwData) (pMfgHwData->hwInfoA->PtrFirstAnnex)
#define mfg_GetExtPAPolar(mode) \
	    (sSysCfg.mfgHwData->hwInfoA->PaOption.PaOption_ExtPAPolar)
#define mfg_GetIntExtPa(mode) \
	 	(sSysCfg.mfgHwData->hwInfoA->PaOption.PaOption_IntExtPa)

#else  /* HARRIER */
#ifdef HARRIER
#ifdef MFG_OLD_STRUCT
#define mfg_GetHwInfoDataRev(addr) (*((UINT8 *)addr))
#else  /* for MFG_NEW_STRUCT */
#define mfg_GetHwInfoDataRev(addr) (*((UINT8 *)addr+3))
#endif /* end of MFG_OLD_STRUCT */

#define mfg_GetHwInfoRev(pMfgHwData) (pMfgHwData->hwInfo->Head.Type)
#define mfg_GetFirAnnexAddr(pMfgHwData) (pMfgHwData->hwInfo->Head.Next)
#define mfg_GetExtPAPolar(mode) \
	((mode == MFG_MODE_A) ? (sSysCfg.mfgHwData->hwInfo->PaOption.ExtPAPolarA) : \
	(pMfgHwData->hwInfo->PaOption.ExtPAPolarG))

#define mfg_GetIntExtPa(mode) \
	((mode == MFG_MODE_A) ? (sSysCfg.mfgHwData->hwInfo->PaOption.IntExtPaA) : \
	(sSysCfg.mfgHwData->hwInfo->PaOption.IntExtPaG))

#endif /* end of HARRIER */
#endif /* end of defined (PEREGRINE) || defined (AP_PGRINE) */

#define mfg_IsAnnex8Available(pMfgHwData, ch) ((ch == 14) && (pMfgHwData->type8 != NULL))

#define mfg_IsAbsoluteCalMethod(mode)	\
	((mode == MFG_MODE_A) ? (sSysCfg.mfgHwData->type10->CalOption.CalMethodOption) : \
	(sSysCfg.mfgHwData->type9->CalOption.CalMethodOption))

#define mfg_IsCh14Calibration()	\
	((sSysCfg.mfgHwData->type8 == NULL) ? FALSE : TRUE) 

#define mfg_IsRangeXCalibrated(bitmap, mode) \
	((mode == MFG_MODE_A) ? \
	(sSysCfg.mfgHwData->type10->CalOption.RangexCalibrated & (1 << bitmap)) : \
	(sSysCfg.mfgHwData->type9->CalOption.RangexCalibrated & (1 << bitmap)))

/* For .11a correction, if both b and g are calibrated,
use Annex 4 for G rates and Annex 9 for B rates.
If only G rate calibrated, use only Annex 4.
If only B rate calibrated, use only annex 9. */
#define mfg_IsDot11G_CalibrationAvailable()	\
	((sSysCfg.mfgHwData->type9->CalOption.gDataRateCalibrated) ? TRUE : FALSE)

#define mfg_IsDot11B_CalibrationAvailable()	\
    ((sSysCfg.mfgHwData->type9->CalOption.bDataRateCalibrated) ? TRUE : FALSE)

#ifdef WB_EXCEPTION
#define mfg_GetRFChan() msi_wl_GetRFChan() 
#else /* for Ap */
#define mfg_GetRFChan() PhyDSSSTable_p->CurrChan
#endif

#ifdef MFG_OLD_STRUCT
#define ARRAY_INDEX_SWAP(x, y) (x)
#else
#define ARRAY_INDEX_SWAP(x, y) (y+x-1-2*(x%y))
#endif

#ifdef EEPROM
#define FIRST_ANNEX_ADDR (MFG_HW_DATA_ADDRESS + \
		(mfg_GetFirAnnexAddr(pMfgHwInfo)) - mfg_start_address_in_eeprom)
#define NEXT_ANNEX_ADDR ((*((UINT32 *)nextLink + 1) == 0xffffffff) ? 0xffffffff : \	
		(MFG_HW_DATA_ADDRESS - mfg_start_address_in_eeprom +(*((UINT32 *)nextLink + 1))))

#else	
#define FIRST_ANNEX_ADDR  mfg_GetFirAnnexAddr(pMfgHwInfo)
#define NEXT_ANNEX_ADDR (*((UINT32 *)nextLink + 1))
#endif	

/* Offer external API here */
#define msi_wl_rfCalGetHwDataRev(addr) mfg_GetHwInfoDataRev(addr)
#define msi_wl_rfCalGetIntExtPa(mode)	mfg_GetIntExtPa(mode)

extern MFG_HW_INFO *msi_wl_rfCalInitHwInfoTable();										 
extern Status_e msi_wl_rfCalInit(MFG_HW_INFO *pMfgHwInfo, UINT8 mode);
extern Status_e msi_wl_rfCalSetAdaptPower(UINT8 dBm, UINT8 ch, UINT8 mode);
#endif /* #ifdef NEWCALDATA	*/

#endif /* #ifdef _HAL_RFCAL_H_ */
