/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
 *
 * Purpose:
 *    This file contains the definitions needed to perform a hash.
 *
 * Public Procedures:
 *    hash_GetHashResult   The routine that performs the hash
 *
 * Notes:
 *    None.
 *
 *****************************************************************************/

#ifndef _HASH_H_
#define _HASH_H_

//=============================================================================
//                               INCLUDE FILES
//=============================================================================

//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================
// None.


//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================

/******************************************************************************
 *
 * Name: hash_GetHashResult
 *
 * Description:
 *   Routine that takes a key, performs a hash on that key, and then
 *   scales the resulting index down to the size of the hash table.
 *
 * Conditions For Use:
 *   None.
 *
 * Arguments:
 *    Arg1 (i  ): Key       - 32 bit value that will be hashed
 *    Arg2 (i  ): TableSize - The size of the hash table
 *
 * Return Value:
 *   The index produced from hashing and scaling
 *
 * Notes:
 *   The values returned by the hashing functions are 32 bit unsigned
 *   values. Hence, to get an index for the hash table, the result from
 *   the hash must be normalized by the largest 32 bit number and then
 *   multiplied by the table size to yield an index appropriate for the
 *   hash table.
 *
 *****************************************************************************/
extern UINT32 hash_GetHashResult ( UINT32 Key, UINT32 TableSize );

#endif /* _HASH_H_ */