#if !defined(_INTR_H_)
#define _INTR_H_

/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */


/*!
 * \file    intr.h
 * \brief   This file contains the interrupt controller definitions.
 *
 */


/*--------------------------------------------------------------*/
/*  Interrupt Controller Registers                              */
/*--------------------------------------------------------------*/
#define WL_INTR_REQ_STAT    (WL_INTR_BASE)
#define WL_INTR_SRC_STAT    ((WL_INTR_BASE)|(0x0004))
#define WL_INTR_ENABLE      ((WL_INTR_BASE)|(0x0008))  /*enable status; read */
/*only */
#define WL_INTR_SET         ((WL_INTR_BASE)|(0x0008))  /*set to enable intr */
#define WL_INTR_CLEAR       ((WL_INTR_BASE)|(0x000C))  /*set to clear intr */
#define WL_INTR_SWI_STAT    ((WL_INTR_BASE)|(0x0010))
#define WL_INTR_FIQ_STAT    ((WL_INTR_BASE)|(0x0100))
#define WL_INTR_FIQ_SRC     ((WL_INTR_BASE)|(0x0104))
#define WL_INTR_FIQ_SET     ((WL_INTR_BASE)|(0x0108))
#define WL_INTR_FIQ_ENABLE  ((WL_INTR_BASE)|(0x0108))
#define WL_INTR_FIQ_CLEAR   ((WL_INTR_BASE)|(0x010C))


/* Interrupt mask bit definitions. */
#define INTR_FIQ_SHARE   (1<<0)  /* may share the same int source as FIQ      */
#define INTR_SOFT        (1<<1)  /* software interrupt                        */
#define INTR_COMMS_RX    (1<<2)  /* communication receiveing channel          */
#define INTR_COMMS_TX  	 (1<<3)  /* communication transmitting channel        */
#define INTR_TIMER_1     (1<<4)  /* timer 1 interrupt request                 */
#define INTR_TIMER_2     (1<<5)  /* timer 2 interrupt request                 */
#define INTR_TIMER_3     (1<<6)  /* timer 3 interrupt request                 */
#define INTR_TIMER_4     (1<<7)  /* timer 4 interrupt request                 */
#define INTR_HOST_IF     (1<<8)  /* host interface interrupt request          */
#define INTR_FST_ETH_MAC (1<<9)  /* fast Ethernet MAC interrupt request       */
#define INTR_WL_ETH_MAC  (1<<10) /* Wireless Ethernet MAC interrupt request   */
#define INTR_UART        (1<<11) /* UART interrupt request                    */
#define INTR_GPIO        (1<<12) /* GPIO interrupt request                    */
#define INTR_CSIU        (1<<13) /* Clocked Serial Interface interrupt request*/
#define INTR_PMU         (1<<14) /* Power Managment Unit interrupt request    */
#define INTR_DMA_0       (1<<15) /* DMA Channel 0 interrupt request           */
#define INTR_DMA_1       (1<<16) /* DMA Channel 1 interrupt request           */
#define INTR_AEU         (1<<17) /* Advanced Encryption Unit interrupt request*/

#define INTR_TDU         (1<<21) /* TDU interrtup request */


#define INTR_SET_ALL        (INTR_FIQ_SHARE   | \
                             INTR_SOFT        | \
                             INTR_COMMS_RX    | \
                             INTR_COMMS_TX    | \
                             INTR_TIMER_1     | \
                             INTR_TIMER_2     | \
                             INTR_TIMER_3     | \
                             INTR_TIMER_4     | \
                             INTR_HOST_IF     | \
                             INTR_FST_ETH_MAC | \
                             INTR_WL_ETH_MAC  | \
                             INTR_UART        | \
                             INTR_GPIO        | \
                             INTR_CSIU        | \
                             INTR_PMU         | \
                             INTR_DMA_0       | \
                             INTR_DMA_1       | \
                             INTR_AEU)
                                

/* Default - Disable INTR_FIQ_SHARE, INTR_SOFT, INTR_COMMS_RX, INTR_COMMS_TX,
             INTR_HOST_IF, INTR_UART, INTR_CSIU, INTR_PMU, INTR_AEU. */
#ifndef B0_CHIP
#define INTR_DEFAULT_ENABLE  (INTR_SET_ALL & \
                             ~(INTR_FIQ_SHARE   | \
                               INTR_SOFT        | \
                               INTR_COMMS_RX    | \
                               INTR_COMMS_TX    | \
                               INTR_HOST_IF     | \
                               INTR_UART        | \
                               INTR_CSIU        | \
                               INTR_PMU         | \
                               INTR_AEU))
#else
#define INTR_DEFAULT_ENABLE  (INTR_SET_ALL & \
                             ~(INTR_FIQ_SHARE   | \
                               INTR_SOFT        | \
                               INTR_COMMS_RX    | \
                               INTR_COMMS_TX    | \
                               INTR_HOST_IF     | \
                               INTR_UART        | \
                               INTR_CSIU        | \
                               INTR_AEU))

#endif
                               
#define INTR_DEFAULT_DIS_GPIO_ENABLE (INTR_DEFAULT_ENABLE & ~INTR_GPIO)



#endif /* _INTR_H_ */

