#if !defined(_MEMMAP_H_)
#define _MEMMAP_H_ 
/*
 *                Copyright 2003, Marvell Semiconductor, Inc.
 * This code contains confidential information of Marvell Semiconductor, Inc.
 * No rights are granted herein under any patent, mask work right or copyright
 * of Marvell or any third party.
 * Marvell reserves the right at its sole discretion to request that this code
 * be immediately returned to Marvell. This code is provided "as is".
 * Marvell makes no warranties, express, implied or otherwise, regarding its
 * accuracy, completeness or performance.
 */

/*!
 * \file  memmap.h
 * \brief Defines for Memory Map
 */


/*--------------------------------------------------------------*/
/*!  Memory map                                                 */
/*--------------------------------------------------------------*/
#define WL_SQRAM_BASE             (0xC0000000)      /*interal sram 'SQ' */
#define WL_SQRAM_SIZE             (0x10000)         /*64K byte */
#define WL_SDRAM0_BASE            (0xD0000000)      /*exteral sram 'SD'0 */
#define WL_SDRAM0_SIZE            (0x400000)        /*4M byte */
#define WL_SDRAM1_BASE            (0xE0000000)      /*exteral sram 'SD'1 */
#define WL_SDRAM1_SIZE            (0x400000)        /*4M byte */

#define WL_AMBA_BASE              (0x90000000)
#define WL_AMBA_SIZE              (0x10000000)

#define WL_SRAM_BASE              WL_AMBA_BASE
#define WL_INTR_BASE              (0x90008000)
#define WL_TIMER_BASE             (0x90009000)
#define WL_CFG_BASE               (0x80000000)      /*change from 0x80... to */
/*0x81... for Armulator */
/*testing */

#define WL_HIU_CFG_BASE           WL_CFG_BASE      /*Host interface (pci & */
/*usb) */
#define WL_HIU_CFG_SIZE           (0x2000)

#define WL_TDU_CFG_BASE			  (WL_CFG_BASE+0x1000)

#define WL_CIU_CFG_BASE           (WL_CFG_BASE+0x2000)  /*AMBA, G-bus bridge */
#define WL_CIU_CFG_SIZE           (0x2000)

#define WL_SQRAM_CFG_BASE         (WL_CFG_BASE+0x4000)
#define WL_SQRAM_CFG_SIZE         (0x2000)

#define WL_SDRAM_CFG_BASE         (WL_CFG_BASE+0x6000)
#define WL_SDRAM_CFG_SIZE         (0x2000)

#ifndef WL_FEMAC_CFG_BASE
#define WL_FEMAC_CFG_BASE         (WL_CFG_BASE+0x8000)
#endif
#define WL_FEMAC_CFG_SIZE         (0x1000)

#define WL_WLMAC_CFG_BASE         (WL_CFG_BASE+0xA000)
#define WL_WLMAC_CFG_SIZE         (0x1000)

#define WL_LBU_CFG_BASE           (WL_CFG_BASE+0xC000)  /*low speed bus unit */
#define WL_LBU_CFG_SIZE           (0x2000)
#define WL_LBU_CSU_CFG_BASE       WL_LBU_CFG_BASE     /*low speed bus unit */
#define WL_LBU_CSU_CFG_SIZE       (0x800)
#define WL_LBU_CSU_INTERNAL_BASE  WL_LBU_CFG_BASE     /*internal register */
#define WL_LBU_CSU_INTERNAL_SIZE  (0x200)
#define WL_LBU_CSU_SEEPROM_BASE   (WL_CFG_BASE+0xC200)  /*serial eeprom */
#define WL_LBU_CSU_SEEPROM_SIZE   (0x800)
#define WL_LBU_CSU_W80_BASE       (WL_CFG_BASE+0xC400)  /*RF */
#define WL_LBU_CSU_W80_SIZE       (0x800)
#define WL_LBU_CSU_W82_BASE       (WL_CFG_BASE+0xC600)  /*base band */
#define WL_LBU_CSU_W82_SIZE       (0x800)
#define WL_LBU_SIU_CFG_BASE       (WL_CFG_BASE+0xC800) /*low speed bus unit */
#define WL_LBU_SIU_CFG_SIZE       (0x800)
#define WL_LBU_GPU_CFG_BASE       (WL_CFG_BASE+0xD000) /*GP I/O unit */
#define WL_LBU_GPU_CFG_SIZE       (0x800)
#define WL_LBU_PMU_CFG_BASE       (WL_CFG_BASE+0xD800) /*Pwr Mgmt unit */
#define WL_LBU_PMU_CFG_SIZE       (0x800)

#define WL_IDMA_CFG_BASE          (WL_CFG_BASE+0xE000)  /*dma unit */
#define WL_IDMA_CFG_SIZE          (0x2000)

#define WL_WLTXMAC_CFG_BASE       WL_WLMAC_CFG_BASE
#define WL_WLRXMAC_CFG_BASE       (WL_WLMAC_CFG_BASE+0x200)

#define TIMER1_LEN    (WL_TIMER_BASE + 0x0)
#define TIMER2_LEN    (WL_TIMER_BASE + 0x4)
#define TIMER3_LEN    (WL_TIMER_BASE + 0x8)
#define TIMER4_LEN    (WL_TIMER_BASE + 0xc)
#define TIMER_CONTROL (WL_TIMER_BASE + 0x10)
#define TIMER_ISR     (WL_TIMER_BASE + 0x24)
#define TIMER_IMR     (WL_TIMER_BASE + 0x28)

//Bit maps for the timers
#define TIMER1_AUTORST      (1<<0)
#define TIMER1_ACTIVATE     (1<<1)
#define TIMER1_LD_VAL       (1<<3)

#define TIMER2_AUTORST      (1<<4)
#define TIMER2_ACTIVATE     (1<<5)
#define TIMER2_LD_VAL       (1<<7)

#define TIMER3_AUTORST      (1<<8)
#define TIMER3_ACTIVATE     (1<<9)
#define TIMER3_LD_VAL       (1<<11)

#define TIMER4_AUTORST      (1<<12)
#define TIMER4_ACTIVATE     (1<<13)
#define TIMER4_LD_VAL       (1<<15)

#endif /* _MEMMAP_H_ */

