#ifndef _MFG_H_
#define _MFG_H_

/*============================================================================= */
/*                            PUBLIC DEFINITIONS */
/*============================================================================= */
#define MFG_TYPE1       0 
/* manufacturing structure type for the hardware info data structure */
#define MFG_TYPE1_SIZE  64 
/* size for the manufacturing structure of type 1 */

#define POW_BOUNDARY_B0 11
#define POW_BOUNDMARGINE 1

#define POW_MIN      5 
/* power min in dBm */
#define POW_MAX      20 
/* power max in dBm */

#define ANT_SEL_MIN  1 
/* antenna selection minimum value */
#define ANT_SEL_MAX  3 
/* antenna selection maximum value */

#define DATARATE_1M  0 
/* data rate 1 Mbps */
#define DATARATE_2M  1 
/* data rate 2 Mbps */
#define DATARATE_5M  2 
/* data rate 5.5 Mbps */
#define DATARATE_11M 3 
/* data rate 11 Mbps */
#define DATARATE_22M 4 
/* data rate 22 Mbps */
#define DATARATE_6M  5 
/* data rate 6 Mbps */
#define DATARATE_9M  6 
/* data rate 9 Mbps */
#define DATARATE_12M 7  
/* data rate 12 Mbps */
#define DATARATE_18M 8  
/* data rate 18 Mbps */
#define DATARATE_24M 9  
/* data rate 24 Mbps */
#define DATARATE_36M 10  
/* data rate 36 Mbps */
#define DATARATE_48M 11  
/* data rate 48 Mbps */
#define DATARATE_54M 12  
/* data rate 54 Mbps */
#define DATARATE_72M 13  
/* data rate 72 Mbps */

#define TX_EN_22     0x20 
/* enable 22 Mbps */
#define TX_RX_ACTIVE 0x00 
/* power mode TX/RX active */
#define PW_MODE_ACTIVE 0 
/* select power mode active */
#define PW_MODE_STDBY  1 
/* select power mode stand bye */
#define PW_MODE_DONE   2 
/* select power done mode */
#define RDBACK_ANT0    0 
/* select read back antenna 0 */
#define RDBACK_ANT1    1 
/* select read back antenna 1 */
#define RDBACK_ACTIVE  2 
/* read back by antenna selection */

/*--------------------------------------------------------------*/
/*    Baseband processor Registers                              */
/*--------------------------------------------------------------*/
#define W81_BBP_ID            (0x01)
#define W81_BBP_RX_CTRL0      (0x08)
#define W81_BBP_TX_CTRL2      (0x13)
#define W81_BBP_TX_CTRL5      (0x16)
#define W81_BBP_TX_CTRL6      (0x17)
#define W81_BBP_CCA_CTRL1     (0x35)
#define W81_BBP_CCA_CTRL2     (0x36)
#define W81_BBP_CCA_CTRL6     (0x3A)
#define W81_BBP_CCA_CTRL7     (0x3B)
#define W81_BBP_ANT_DIV0      (0x3F)
#define W81_BBP_ANT_DIV1      (0x40)

/*============================================================================= */
/*                          PUBLIC TYPE DEFINITIONS */
/*============================================================================= */
/* mfg data struct*/
#define SZ_INTERNAL_PA_CFG  14
#define SZ_EXTERNAL_PA_CFG  1

#ifdef PEREGRINE
typedef struct PaOpt_t
{
   UINT8  IntPaCalTabOpt:1;  /* 0 = Delta Method, 1 = Absolute Method */
   UINT8  ant2NotCalSep:1;
   UINT8  ExtPaNegPol:1;     /* Ext PA polarization. 0 = Positive, 1 = Negative */
   UINT8  ExtPaUsed:1;
   UINT8  Reserved:4;
} PACK_END PaOpt_t;

typedef struct Ant_t
{
   UINT8   Tx:2;        /* AntDivModeTx[7:6](BBP:0x38),  00 = ant0; 01 = ant1; 1x = diversity */
   UINT8   Rx:2;        /* AntDivModeRx[7:6](BBP:0x37),  00 = ant0; 01 = ant1; 1x = diversity */
   UINT8   Reserved:4;
} PACK_END Ant_t;

typedef struct Delta_t
{
   UINT8 CalVal_C1;
   UINT8 CalVal_C0;
} PACK_END Delta_t;

typedef struct Absolute_t
{
   UINT8 Reg_1B;
   UINT8 Reg_1C;
} PACK_END Absolute_t;

typedef union Pa_t
{
   Delta_t    Delta;
   Absolute_t Absolute;
} PACK_END Pa_t;

typedef struct PaConfig_t
{
   UINT32 Reg_1E:8;
   UINT32 Reg_1F:8;
   UINT32 Reg_20:8;
   UINT32 Reg_1A:6;
   UINT32 Reg_1D_0:1;
   UINT32 _2CalValTbl:1;
} PACK_END PaConfig_t;

typedef struct AssmSN_t
{
   UINT32 AssmCode:8;
   UINT32 SerialNum:24;
} PACK_END AssmSN_t;

typedef struct LCTCal_t
{
   UINT8 LctCalCh11:3;
   UINT8 LctCalCh4:3;
   UINT8 Valid0:1;
   UINT8 Valid1:1;	
} PACK_END LCTCal_t; 

/* mfg data struct*/
typedef struct mfg_data_t
{
    UINT8 BrdDscrpt[8];      // 8 byte ASCII to descript the type and version of the board
    UINT8 Rev;
    union {
        struct mfg_peregrine_s {
            PaOpt_t         PaOpt_p;
            UINT8           ExtPA_p[SZ_EXTERNAL_PA_CFG];
            Ant_t           Ant_p;
            Pa_t            Pa_p[SZ_INTERNAL_PA_CFG];;
            PaConfig_t      PaConfig_p;
            AssmSN_t        AssmSN_p;
            UINT16          Domain_p;            /* Wireless domain */
            UINT8	        ECO_p;
            LCTCal_t        LCT_cal_p;
            UINT8           SocVer_p;
            UINT8           RFVer_p;
            UINT8	        SPISize_p;
            UINT8    	    AntPart_p;
            UINT16          TestVer_p;
            UINT16          DllVer_p;
            UINT16          Xosc_p;
        } PACK_END mfg_peregrine;
        struct mfg_falcon_s {
            UINT8 PAOptions_f;
            UINT8 ExtPA_f;
            UINT8 Ant_f;
            UINT16 IntPA_f[14];
            UINT8 CCA_f[8];
            UINT16 Domain_f;            // Wireless domain
            UINT16 CstmrOpts_f;
            UINT8 Reserved_1_f[8];
            UINT16 Magic_f;
        } PACK_END mfg_falcon;
    } PACK_END mfg_u;
    UINT16 ChkSum;
    UINT8 MfgMacAddr[6];     // Mfg mac address
    UINT8 MfgMacAddr2[6];
    UINT8 PID[56];           // Production ID
    UINT8 BootVersion[12];
} PACK_END mfg_data_t;

#define PAOptions   mfg_u.mfg_falcon.PAOptions_f
#define ExtPA       mfg_u.mfg_falcon.ExtPA_f
#define Ant         mfg_u.mfg_falcon.Ant_f
#define IntPA       mfg_u.mfg_falcon.IntPA_f
#define CCA         mfg_u.mfg_falcon.CCA_f
#define Domain      mfg_u.mfg_falcon.Domain_f
#define CstmrOpts   mfg_u.mfg_falcon.CstmrOpts_f
#define Reserved_1  mfg_u.mfg_falcon.Reserved_1_f
#define Magic       mfg_u.mfg_falcon.Magic_f

#define PaOpt       mfg_u.mfg_peregrine.PaOpt_p       
#define ExtPAp      mfg_u.mfg_peregrine.ExtPA_p       
#define Antp        mfg_u.mfg_peregrine.Ant_p         
#define Pa          mfg_u.mfg_peregrine.Pa_p          
#define PaConfig    mfg_u.mfg_peregrine.PaConfig_p    
#define AssmSN      mfg_u.mfg_peregrine.AssmSN_p      
#define Domainp     mfg_u.mfg_peregrine.Domain_p      
#define ECO         mfg_u.mfg_peregrine.ECO_p         
#define LCT_cal     mfg_u.mfg_peregrine.LCT_cal_p     
#define SocVer      mfg_u.mfg_peregrine.SocVer_p      
#define RFVer       mfg_u.mfg_peregrine.RFVer_p       
#define SPISize     mfg_u.mfg_peregrine.SPISize_p     
#define AntPart     mfg_u.mfg_peregrine.AntPart_p     
#define TestVer     mfg_u.mfg_peregrine.TestVer_p     
#define DllVer      mfg_u.mfg_peregrine.DllVer_p      
#define Xosc        mfg_u.mfg_peregrine.Xosc_p        

#else

/* mfg data struct*/
typedef struct mfg_data_t
{
    UINT8 BrdDscrpt[8];      // 8 byte ASCII to descript the type and version of the board
    UINT8 Rev;
    UINT8 PAOptions;
    UINT8 ExtPA;
    UINT8 Ant;
    UINT16 IntPA[14];
    UINT8 CCA[8];
    UINT16 Domain;            // Wireless domain
    UINT16 CstmrOpts;
    UINT8 Reserved_1[8];
    UINT16 Magic;
    UINT16 ChkSum;
    UINT8 MfgMacAddr[6];     // Mfg mac address
    UINT8 MfgMacAddr2[6];
    UINT8 PID[56];           // Production ID
    UINT8 BootVersion[12];
} PACK_END mfg_data_t;

#define PAOPTIONS_USE_TABLE_1   0x01
#define PAOPTIONS_ANT2_NOT_SPRT 0x02
#define PAOPTIONS_EXT_PA_POL    0x04
#define PAOPTIONS_EXTERNAL_PA   0x08
#endif


/*============================================================================= */
/*                                PUBLIC DATA */
/*============================================================================= */
/* None. */

/*============================================================================= */
/*                    PUBLIC PROCEDURES (ANSI Prototypes) */
/*============================================================================= */
/* None. */

#endif /* _MFG_H */
