/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* Purpose:
*    This file contains the definitions for the Management Informaton Base
*    (MIB).
*
* Public Procedures:
*    None.
*
* Notes:
*    The structure of the MIB is given in the IEEE 802.11 standard.
*
*****************************************************************************/

#ifndef _MIB_H_
#define _MIB_H_
#include "wl_mib.h"

//=============================================================================
//                               INCLUDE FILES
//=============================================================================

//=============================================================================
//                            PUBLIC DEFINITIONS
//=============================================================================
#define mib_MAXIMUM_DATA_RATES  8
#define mib_MAX_DEFAULT_KEYS    4


//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================

/*---------------------------------------------------------------------------*/
/*                     Enumerated types used for the MIB                     */
/*---------------------------------------------------------------------------*/
typedef enum
{
    RSSI_ONLY = 1,
    SIG1_AND_RSSI_A = 2,
    RSSI_AND_POS_SIG1 = 3,
    SIG1_AND_SIG2_AND_RSSI = 4,
    SIG1_AND_RSSI_B = 5
} snmp_CcaMode_e;

//=============================================================================
//                       STATION MANAGEMENT ATTRIBUTES
//=============================================================================



//=============================================================================
//                             MAC ATTRIBUTES
//=============================================================================

/*----------------*/
/* Counters Table */
/*----------------*/
typedef struct mib_Counters_t
{
    UINT32 RxFrmCnt;
    UINT32 MulticastTxFrmCnt;
    UINT32 FailedCnt;
    UINT32 RetryCnt;
    UINT32 MultRetryCnt;
    UINT32 FrmDupCnt;
    UINT32 RtsSuccessCnt;
    UINT32 RtsFailCnt;
    UINT32 AckFailCnt;
    UINT32 RxFragCnt;
    UINT32 MulticastRxFrmCnt;
    UINT32 FcsErrCnt;
    UINT32 TxFrmCnt;
    UINT32 WepUndecryptCnt;
}
mib_Counters_t;

/*-----------------------*/
/* Group Addresses Table */
/*-----------------------*/
typedef struct mib_GroupAddr_t
{
    UINT32 GroupAddrIdx;
    IEEEtypes_MacAddr_t Addr;
    UINT8 GroupAddrStatus;  // SNMP_Rowstatus_e values
}
mib_GroupAddr_t;

/*----------------------------*/
/* Resource Information Table */
/*----------------------------*/
typedef struct mib_RsrcInfo_t
{
    UINT8 ManufOui[3];         // 3 byte string
    UINT8 ManufName[128];      // 128 byte string
    UINT8 ManufProdName[128];  // 128 byte string
    UINT8 ManufProdVer[128];   // 128 byte string
}
mib_RsrcInfo_t;

extern UINT8 mib_preAmble;
extern UINT8 mib_rxAntenna;
extern UINT8 mib_txAntenna;
extern UINT8 mib_AntDiversity;
extern UINT8 mib_broadcastssid;
extern UINT8 mib_defaultkeyindex;
extern UINT8 mib_defaultkeyindex_G;
extern UINT8 mib_dhcpenable;
extern UINT8 mib_dhcpstatechange;
extern UINT8 mib_txDataRate;
extern UINT8 mib_currentmacaddress[6];
extern UINT8 mib_wep; // special mode
extern UINT8 mib_wlanfilterno;
extern UINT8 mib_wlanfiltermac[256*6];
extern UINT8 mib_wlanfiltertype;
extern UINT8 mib_txDataRateG;
extern UINT8 mib_txDataRateA;
extern UINT8 mib_shortSlotTime;
extern UINT8 mib_ApMode;
extern UINT8 mib_burstmode;
extern UINT8 mib_systembus;
extern UINT8 mib_burstrate;
extern UINT8 mib_clock;
extern UINT8 mib_disable6060clock;

#ifdef WDS_FEATURE
#ifndef MAX_WDS_PORT
#define MAX_WDS_PORT   6
#endif
extern UINT8 mib_wdsEnable;
extern UINT8 mib_wdsno;
extern UINT8 mib_wdsmac[];
#define WDS_CONFIGURED_KEY          0x2345a55a
extern UINT8 mib_disableAssoc;
extern UINT8 mib_wdsBcastMode;
#endif // WDS_FEATURE

#ifdef WIPOD
extern UINT8 mib_wipodMode;
#endif
//=============================================================================
//                             PHY ATTRIBUTES
//=============================================================================


/*---------------------------------------------*/
/* PHY Frequency Hopping Spread Spectrum Table */
/*---------------------------------------------*/
typedef struct mib_PhyFreqHopData_t
{
    UINT8 HopTime;        // 224?
    UINT8 CurrChanNum;    // 0 to 99
    UINT16 MaxDwellTime;   // 0 to 65535
    UINT16 CurrDwellTime;  // 0 to 65535
    UINT16 CurrSet;        // 0 to 255
    UINT16 CurrPattern;    // 0 to 255
    UINT16 CurrIdx;        // 0 to 255
}
mib_PhyFreqHopData_t;

/*--------------*/
/* PHY IR Table */
/*--------------*/
typedef struct mib_PhyIRData_t
{
    UINT32 CcaWatchDogTmrMax;
    UINT32 CcaWatchDogCntMax;
    UINT32 CcaWatchDogTmrMin;
    UINT32 CcaWatchDogCntMin;
}
mib_PhyIRData_t;

/*----------------------------------------*/
/* PHY Regulatory Domains Supported Table */
/*----------------------------------------*/
typedef struct mib_PhySuppRegDomains_t
{
    UINT32 RegDomainsSuppIdx;
    UINT8 RegDomainsSuppVal;  //SNMP_RegDomainsSuppVal_e values
}
mib_PhySuppRegDomains_t;


/*----------------------------------------*/
/* PHY Supported Receive Data Rates Table */
/*----------------------------------------*/
typedef struct mib_PhySuppRxDataRates_t
{
    UINT8 SuppDataRatesRxIdx;  //1 to 8
    UINT8 SuppDataRatesRxVal;  //2 to 127
}
mib_PhySuppRxDataRates_t;

#ifdef WPA 
//=============================================================================
//                             802.11i ATTRIBUTES
//=============================================================================


/* Marvell private MIB */
typedef struct mib_MrvlRSNDataTrafficEnabled_t
{
    UINT8 StaMacAddr[6];
    UINT8 Enabled;
}
mib_MrvlRSNDataTrafficEnabled_t;

typedef struct mib_MrvlRSN_PWK_t
{
    UINT8 StaMacAddr[6];
    UINT8 EncryptKey[16];
    UINT32 TxMICKey[2];
    UINT32 RxMICKey[2];
    //UINT32  TempKey1[4];
    //UINT32  TempKey2[4];
}
mib_MrvlRSN_PWK_t;
/*
typedef struct mib_MrvlRSN_GrpKey_t
{
    UINT8 EncryptKey[16];
    UINT32 TxMICKey[2];
    UINT32 RxMICKey[2];
    //UINT32  TempKey1[4];
    //UINT32  TempKey2[4];
}
mib_MrvlRSN_GrpKey_t;
*/
typedef struct mib_DHCP_t
{
    UINT32  IPAddr;
    UINT32 SubnetMask;
    UINT32 GwyAddr;
#ifdef GATEWAY    
    UINT32 PrimaryDNS;
    UINT32 SecondaryDNS;
#endif    
}
mib_DHCP_t;

#ifdef QOS_FEATURE
typedef struct
{
     UINT32 EDCATblIndx;
     UINT32 EDCATblCWmin;
     UINT32 EDCATblCWmax;
     UINT32 EDCATblAIFSN;
     UINT32 EDCATblTXOPLimit;
     UINT32 EDCATblMSDULifeTime;
     UINT32 EDCATblMandatory;

}mib_EDCATable_t;

typedef struct
{
     UINT32 QAPEDCATblIndx;
     UINT32 QAPEDCATblCWmin;
     UINT32 QAPEDCATblCWmax;
     UINT32 QAPEDCATblAIFSN;
     UINT32 QAPEDCATblTXOPLimitBAP;
     UINT32 QAPEDCATblTXOPLimit;
//     UINT32 QAPEDCATblMSDULifeTime;
     UINT32 QAPEDCATblMandatory;
}mib_QAPEDCATable_t;

typedef struct
{
     UINT32 QStaEDCATblIndx;
     UINT32 QStaEDCATblCWmin;
     UINT32 QStaEDCATblCWmax;
     UINT32 QStaEDCATblAIFSN;
     UINT32 QStaEDCATblTXOPLimitBSta;
     UINT32 QStaEDCATblTXOPLimit;
	 UINT32 QStaEDCATblMandatory;
}mib_QStaEDCATable_t;

typedef struct
{
    UINT32 QoSCounterIndx;
    UINT32 QoSTxFracCnt;
    UINT32 QoSFailedCnt;
    UINT32 QoSRetryCnt;
    UINT32 QoSMultipleRetryCnt;
    UINT32 QoSFrmDupCnt; //FrameDuplicateCount
    UINT32 QoSRTSSuccessCnt;
    UINT32 QoSRTSFailureCnt;
    UINT32 QoSAckFailureCnt;
    UINT32 QoSRcvdFragCnt;
    UINT32 QoSTxFrmCnt;
    UINT32 QoSDiscardedFrmCnt;
    UINT32 QoSMPDURecvdCnt;
    UINT32 QoSRetryRecvdCnt;

}mib_QoSCounters_t;
#endif


/* End of Marvell private MIB */


extern MIB_RSNCONFIG *mib_RSNConfig_p;

extern MIB_RSNCONFIG_UNICAST_CIPHERS  *mib_RSNConfigUnicastCiphers_p;
extern MIB_RSNCONFIG_AUTH_SUITES  *mib_RSNConfigAuthSuites_p;
extern MIB_RSNSTATS *mib_RSNStats_p;

extern mib_MrvlRSNDataTrafficEnabled_t mib_MrvlRSNDataTrafficEnabled;
extern mib_MrvlRSN_PWK_t mib_MrvlRSN_PWK;
//extern mib_MrvlRSN_GrpKey_t mib_MrvlRSN_GrpKey;
extern MRVL_MIB_RSN_GRP_KEY mib_MrvlRSN_GrpKey;

#endif

#ifdef AP_WPA2
extern MIB_RSNCONFIGWPA2              *mib_RSNConfigWPA2_p;
extern MIB_RSNCONFIGWPA2_UNICAST_CIPHERS   *mib_RSNConfigWPA2UnicastCiphers_p;
extern MIB_RSNCONFIGWPA2_UNICAST_CIPHERS   *mib_RSNConfigWPA2UnicastCiphers2_p;
extern MIB_RSNCONFIGWPA2_AUTH_SUITES       *mib_RSNConfigWPA2AuthSuites_p;

extern UINT8 mib_WPAPSKValueEnabled;
extern UINT8 mib_WPA2PSKValueEnabled;
#endif

extern MIB_STA_CFG *mib_StaCfg_p;
extern MIB_OP_DATA *mib_OpData_p;
extern MIB_AUTH_ALG *mib_AuthAlg_p;
extern MIB_AUTH_ALG *mib_AuthAlg_G;
extern MIB_PHY_DSSS_TABLE *PhyDSSSTable_p;

extern MIB_PRIVACY_TABLE *mib_PrivacyTable_p;
extern MIB_PHY_TX_POWER_TABLE *mib_PhyTxPwrData_p;
extern MIB_PHY_SUPP_DATA_RATES_TX *mib_PhySuppTxDataRates_p;
extern MIB_PHY_ANT_LIST  *mib_PhyAntlist_p;

extern MIB_PHY_ANT_TABLE *mib_PhyAntData_p;
extern mib_Counters_t mib_Counters;
extern MIB_WEP_DEFAULT_KEYS *mib_WepDefaultKeys_p;
extern MIB_WEP_DEFAULT_KEYS *mib_WepDefaultKeys_G;
extern mib_DHCP_t mib_DHCP;

extern UINT8 mib_enableFixedRateTx;
extern UINT8 mib_forceProtectiondisable;
#ifdef QOS_FEATURE
extern mib_QAPEDCATable_t mib_QAPEDCATable[4];
extern mib_QStaEDCATable_t mib_QStaEDCATable[4];
#endif
#ifdef QOS_FEATURE
#ifdef STA_QOS
extern MIB_QSTAEDCATABLE          *mib_URQStaEDCATable;
#endif
#endif

#ifdef IEEE80211H
extern MIB_SPECTRUM_MGMT *mib_SpectrumMagament_p;
#endif /*IEEE80211H*/

#ifdef AUTOLINK
extern UINT8	mib_ALOperationalKey[16]; /* AutoLink 128 bit operational key*/
extern UINT8 	mib_ALSecMode; /*AutoLink encrypt mode can be OPEN, WEP 64, WEP 128, WPA or WPA2*/
extern UINT8	mib_ALStatus; /*Status of AutoLink: OFF:0, ON:2, DONE:4, PROPAGATE:6*/
extern UINT8	mib_ALLowPowerEnable; /*Low Power during AutoLink*/
extern UINT8	mib_ALEnable; /*To enable and disable AutoLink*/
extern UINT32	mib_ALONTime; /*To control time period after which AutoLink is turned OFF*/
extern UINT8	mib_ALPPEnable; /*User defined pass phrase to be used by AutoLink*/
#endif
#ifdef GUEST_ACCESS
extern UINT8 mib_GuestAccessEnable;
extern UINT8 mib_GuestAccessPassPhrase[64];
extern UINT8 mib_GuestAccessPassPhrase2[64];
// Use mib_GuestAccessPassPhrase[63]/mib_GuestAccessPassPhrase2[63]
// to indicate 64 HEX
#define GUEST_ACCESS_WITH_64HEX 0x01
#endif
extern WL_STATUS mib_Init();
extern void mib_RestoreDefaults();
extern void mib_Reset();

#ifdef DCS
extern UINT8 mib_dcsMode;
#endif
extern UINT8 mib_AgingTime;	   
#endif /* _MIB_H_ */
