/******************** (c) Marvell Semiconductor, Inc., 2004 *******************
* 
* $Header: //depot/MRVL/main/firmware/AP5x/wlandrv/incl/qosq.h#4 $ 
* 
* Purpose: 
*    This file contains the function prototypes and definitions for the 
*    Qos Q module. 
* 
* Public Procedures: 
* 
* Notes: 
*    None. 
* 
*****************************************************************************/

#ifndef _QOSQ_H_
#define _QOSQ_H_

#ifdef QOS_FEATURE
//=============================================================================
//                               INCLUDE FILES
//=============================================================================

//=============================================================================
//                            PUBLIC DEFINITIONS
//=============================================================================
// None.

//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================


//=============================================================================
//                                PUBLIC DATA
//=============================================================================
// None.

//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================

/******************************************************************************
 * 
 * Name: qosq_Init 
 * 
 * Description: 
 *   This routine is called to initialize the the Qos Q module. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   None. 
 * 
 * Return Value: 
 *   Status indicating success or failure 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
extern Status_e qosq_Init( void );

/******************************************************************************
 * 
 * Name: qosq_WrTsQ
 * 
 * Description: 
 *   This routine is called to write to Traffic Stream sw Q. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   uint32 gTID  : Global TID for the stream.
 *   txInfo_t *   : packet to be queuex 
 * 
 * Return Value: 
 *   void 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
extern Status_e qosq_WrTsQ(UINT32 gTID,txInfo_t * TxInfo_p);
/******************************************************************************
 * 
 * Name: qosq_RdTsQ
 * 
 * Description: 
 *   This routine is called to read from  Traffic Stream  sw Q. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   uint32 gTID  : Global TID for the stream.
 *   
 * 
 * Return Value: 
 *   txInfo_t *   : packet read from the queue 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
extern txInfo_t *qosq_RdTsQ(UINT32 gTID);


/******************************************************************************
 * 
 * Name: qosq_WrEdcaQ
 * 
 * Description: 
 *   This routine is called to write to edca sw Q. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   uint32 ac  : access category.
 *   txInfo_t *   : packet to be queuex 
 * 
 * Return Value: 
 *   void 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
extern Status_e qosq_WrEdcaQ(txInfo_t *txInfo_p);

/******************************************************************************
 * 
 * Name: qosq_RdEdcaQ
 * 
 * Description: 
 *   This routine is called to read from  Edca sw Q. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   txInfo+t *  : Ptr to the packet.
 *   
 * 
 * Return Value: 
 *   txInfo_t *   : packet read from the queue 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
txInfo_t * qosq_RdEdcaQ(UINT32 ac);
/******************************************************************************
 * 
 * Name: qosq_WrCapQ
 * 
 * Description: 
 *   This routine is called to queue the packets in CAP Q,for transmission in
 *   next service interval. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   uint32 Qnum  : queue number.
 *   txInfo_t *   : packet to be transmitted
 * 
 * Return Value: 
 *   txInfo_t *   : packet read from the queue 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
Status_e qosq_WrCapQ(txInfo_t* TxInfo_p);
/******************************************************************************
 * 
 * Name: qosq_ActivateCapQ
 * 
 * Description: 
 *   This routine is called to start tx from the CAP q. 
 * 
 * Conditions For Use: 
 *   None. 
 * 
 * Arguments: 
 *   uint32 Qnum  : queue number.
 *   
 * 
 * Return Value: 
 *   none   : 
 * 
 * Notes: 
 *   None. 
 * 
 *****************************************************************************/
void qosq_ActivateCapQ(UINT32 Qnum);
UINT32 qosq_GetACq(txInfo_t *TxInfo_p);
UINT32 qosq_SetSeqNum(txInfo_t *txInfo_p);

#endif //QOS_FEATURE
#endif /* _QOSQ_H_ */
