/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
* 
* Purpose: 
*    This file contains the structures for rate adaptation. 
* 
* Public Procedures: 

* 
* Private Procedures: 
*      
* 
* Notes: 
*    None. 
* 
**************************************************************************/

#ifndef _RATEADAPT_H_
#define _RATEADAPT_H_

#include "tx80211.h"
 
//If we are in debugging mode.
//Will disable this in actual mode.
//#define RA_DEBUG

//Only if we wanna compute the Margins
//#define ENABLE_MARGIN_COMPUTATION
//#define PRINTDEBUG
//#define SQM_BASED
#define ENABLE_ADAPTIVE_BAILOUT 1 //Adaptive Bailout Rate
#define ADAPT_ON_TX_ERR
#define ENABLE_POWER_ADAPT
//#define LOW_RSSI_FOLLOW
//#define UPDATE_TX_QUEUE
#define MULTIPATH_FIX
#define STATISTICAL_METHOD

//Mapping of Rates to indices.
#define RATE_36MBPS 10
#define RATE_11MBPS 3
#define RATE_1MBPS 0
#define RATE_12MBPS 7
#define RATE_9MBPS 6
#define RATE_54MBPS 12
#define RATE_6MBPS  5
#define RATE_22MBPS 4
#define RATE_18MBPS 8

#define PER_THRESHOLD 30 // % per for going down in rate


//Fill teh Adapt Row for teh Table.
void SetupAdaptRow();

//Fill teh Assoc Table with initial values.
void SetupAssocTableRateAdapt(AssocReqData_t* pAssocTableOfStation);

void PrintStationStats(AssocReqData_t* pAssocTableOfStation, rx80211_Frame_t * Data11Frame_p);

UINT8 GetStationRate(AssocReqData_t* pAssocTableOfStation, UINT8 ClientMode, UINT8 Aid, UINT8 ApMode);// __attribute__ ((section (".itcmbuf")));
extern UINT8 (*GetStationRateFp)(AssocReqData_t* pAssocTableOfStation, UINT8 ClientMode, UINT8 Aid, UINT8 ApMode) ;
void UpdateStationTxStats(AssocReqData_t* pAssocTableOfStation, txInfo_t * txInfo_p, UINT16 stnId);// __attribute__ ((section (".itcmbuf")));;
extern void (*UpdateStationTxStatsFp)(AssocReqData_t* pAssocTableOfStation, txInfo_t * txInfo_p, UINT16 stnId);
void UpdateStationRxStats(AssocReqData_t* pAssocTableOfStation, rx80211_Frame_t * Data11Frame_p); // __attribute__ ((section (".itcmbuf")));;
extern void (*UpdateStationRxStatsFp)(AssocReqData_t* pAssocTableOfStation, rx80211_Frame_t * Data11Frame_p);
UINT8 RECV_PKT_TYPE; //0=>both data and Acks. 1=> Only Data. 2=> Only Acks.
extern inline UINT32 UpdatePower(UINT8 Rate, UINT8 Aid, SINT32 Result, UINT8 ApMode);
extern inline void UpdateBailOutRate(UINT8 Rate, UINT8 Aid);

UINT16 lastTxStnId; //to use when ENABLE_DASEBAND_ACKS. To get reading from Ack pkt

void EnablePowerAdapt(bool flag);

void UpdateStationAckStats(AssocReqData_t* pAssocTableOfStation);

void UpdateTxQueue();

void UpdateAssocStnData(UINT8 Aid, UINT8 ApMode);

void (*UpdateBailOutRateFp)(UINT8 Rate, UINT8 Aid);
UINT32 (*UpdatePowerFp)(UINT8 Rate, UINT8 Aid, SINT32 Result);

void ResetRateIncreaseParam();

#ifdef ENABLE_POWER_ADAPT
void SetupPower();
#endif
void apiSetHiPower();
//Some rate adaptation Variables.
#ifdef ENABLE_ADAPTIVE_BAILOUT
extern UINT32 BailOutRate;
extern UINT32 BailOutFlag; //If 1 => BailOutRateset to 11Mbps.
// If 0 => BailOutRate set to 1Mbps.
extern UINT32 LowestRate;
extern UINT32 LowestRateAid;
#endif //ENABLE_ADAPTIVE_BAILOUT
#ifdef ENABLE_POWER_ADAPT
extern UINT8 POWER_ADAPT; //1 to near/far range power adaptation
extern UINT32 POWER_LEVEL; //0 means PA is adjusted to 74 (near range).. 1 means it is 68. (far range)
extern UINT8 RFReg20;
extern UINT8 RFReg20Calibrated;
extern UINT8 RFReg26;
extern UINT8 RFReg26Calibrated;
extern UINT32 HighestRate;
extern UINT32 HighestRateAid;
#endif //POWER_ADAPT
extern UINT32 RAIdleCnt;

typedef enum 
{
    STEADY_STATE,
    UNDER_EVAL_STATE
} eval_state_e;

extern eval_state_e eval_state;

extern UINT8 ExternalPA;
extern Status_e StopRateAdaptTimer();

#ifdef AP_URPTR
typedef WLAN_RX_INFO rate_RxSign_t;

typedef struct rate_AssocReqData_t
{
   UINT8    RateToBeUsedForTx;
   SINT8    AvgRSSI;
   UINT8    InvAlpha;       //to get running average.
   SINT8    Correction;     //To account for fading.
                            //Correction is in 8.1 format.
                            //and can be negative
   UINT8    TxSuccess;      //For Rate Adaptation
   UINT8    TxFailures;     //For Rate Adaptation.
   UINT8    TxFailuresCorrection; //For Correction handling.
   UINT8    TxRetry;        //For Rate Adaptation.
   #ifdef ENABLE_OSCILLATION_CODE
   UINT8    OSC_FLAG;       //if 1, correction won't be decremented.
   UINT8    RSSI_Store;
   //Since recording the RSSI is not perfect, we will use this 
   //counter to disable the Osc Flag if the flag set for a long time.
   UINT32   DisableOscFlagCounter;
   #endif //ENABLE_OSCILLATION_CODE
   //For maintaining the statistical counter for logging failures.
   UINT8    PktTxSinceFail;
   UINT8    TxFailuresStats;
   UINT8    RateIncreaseIncrement;
} rate_AssocReqData_t;

extern void rate_SetupAdaptRow(void);
extern void rate_SetupAssocTableRateAdapt(void);
extern void rate_UpdateRxStats(rate_RxSign_t *RxSignOfStation_p, int len);
extern void rate_UpdateTxStats(MRVL_TX_DONE_INFO *TxInfo_p);
extern UINT8 rate_GetTxRate(void);

extern UINT8 msi_wl_GetRfTxPower();


#endif /* AP_URPTR */

#endif
