/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
* 
* $Header: //depot/MRVL/main/firmware/AP5x/wlandrv/incl/rateadaptur.h#3 $ 
* 
* Purpose: 
*    This file contains the structures for rate adaptation. 
* 
* Public Procedures: 

* 
* Private Procedures: 
*      
* 
* Notes: 
*    None. 
* 
**************************************************************************/

#ifndef _RATEADAPT_H_
#define _RATEADAPT_H_


#ifdef WB_EXCEPTION /* This is the rate adaption for Station client */

#include "wltypes.h"
#include "buildModes.h"
#include "IEEE_types.h"
#include "tkip.h"
#include "StaDb.h"
#include "wl_mib.h"
#include "wlmac.h"
#include "wl_hal.h"
#include "ap_mib.h"
//#include "rateadapt.h"



//=============================================================================
//                          PUBLIC TYPE DEFINITIONS
//=============================================================================

//Mapping of Rates to indices.
#define RATEINDEX_1MBPS 0

#ifdef NEW_RATE_ADAPT
#define RATE_ADAPT_STNCNT_THRESH   5
#endif
#define NUM_MARGINS 6
//#define SQM_BASED
//#define RA_DEBUG
#define MULTIPATH_FIX
#define ADAPT_ON_TX_ERR
#define ENABLE_OSCILLATION_CODE
#define STATISTICAL_METHOD
#define ENABLE_POWER_ADAPT 

//Mapping of Rates to indices.
#define RATEINDEX_1MBPS 0
#define RATEINDEX_6MBPS 5
#define RATEINDEX_11MBPS 3
#define RATEINDEX_22MBPS 4
#define RATEINDEX_36MBPS 10
#define RATEINDEX_54MBPS 12
#define RATEINDEX_72MBPS 13

                      
typedef WLAN_RX_INFO rate_RxSign_t;

typedef struct rate_AdaptRow_t
{
  SINT8   offset;            //to be added to the margin. Scale is 8.1.Can be negative
} rate_AdaptRow_t;

typedef struct rate_AssocReqData_t
{
   UINT8    RateToBeUsedForTx;
   SINT8     AvgRSSI;
   UINT8    InvAlpha;     //to get running average.
   SINT8     Correction;   //To account for fading.
                          //Correction is in 8.1 format.
                          //and can be negative
   UINT8    TxSuccess;    //For Rate Adaptation
   UINT8    TxFailures;   //For Rate Adaptation.
   UINT8    TxFailuresCorrection; //For Correction handling.
   UINT8    TxRetry;      //For Rate Adaptation.
   #ifdef ENABLE_OSCILLATION_CODE
   UINT8 OSC_FLAG; //if 1, correction won't be decremented.
   UINT8 RSSI_Store;
   //Since recording the RSSI is not perfect, we will use this 
   //counter to disable the Osc Flag if the flag set for a long time.
   UINT32 DisableOscFlagCounter;
   #endif //ENABLE_OSCILLATION_CODE
   //For maintaining the statistical counter for logging failures.
   UINT8 PktTxSinceFail;
   UINT8 TxFailuresStats;
   UINT8 RateIncreaseIncrement;
} rate_AssocReqData_t;


//=============================================================================
//                    PUBLIC PROCEDURES (ANSI Prototypes)
//=============================================================================


/******************************************************************************
 *
 * Name: rate_SetupAdaptRow
 *
 * Description:
 *   Routine to initialize the table.
 *
 * Conditions For Use:
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void rate_SetupAdaptRow(void);

/******************************************************************************
 *
 * Name: rate_SetupAssocTableRateAdapt
 *
 * Description:
 *   Routine to initialize the rate adaptation variables.
 *
 * Conditions For Use:
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void rate_SetupAssocTableRateAdapt(void);


/******************************************************************************
 *
 * Name: rate_UpdateRxStats
 *
 * Description:
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): RxSignOfStation_p
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern void rate_UpdateRxStats(rate_RxSign_t *RxSignOfStation_p, int len);

/******************************************************************************
 *
 * Name: rate_UpdateTxStats
 *
 * Description:
 *
 * Conditions For Use:
 *
 * Arguments:
 *   Arg1 (i  ): TxInfo_p
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
#if 1
extern void rate_UpdateTxStats(MRVL_TX_DONE_INFO *TxInfo_p);
#else
extern void rate_UpdateTxStats(MacTxInfo_t *TxInfo_p);
#endif /* bt_todo */
/******************************************************************************
 *
 * Name: rate_GetTxRate
 *
 * Description:
 *
 * Conditions For Use:
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
extern UINT8 rate_GetTxRate(void);

/******************************************************************************
 *
 * Name: ratae_UpdateStationAckStats
 *
 * Description:
 *   Routine to check if the Ack was received at the Bail Out Rate.
 *   If so, consider the rate at which we transmitted as a failure.
 *
 * Conditions For Use:
 *
 * Arguments:
 *   date_rate.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 *****************************************************************************/
void ratae_UpdateStationAckStats(UINT8 rate);



#else   /* WB_EXCEPTION */

#ifdef AP_URPTR

typedef enum 
{
    STEADY_STATE,
    UNDER_EVAL_STATE
} eval_state_e;

typedef WLAN_RX_INFO rate_RxSign_t;

typedef struct rate_AssocReqData_t
{
   UINT8    RateToBeUsedForTx;
   SINT8     AvgRSSI;
   UINT8    InvAlpha;     //to get running average.
   SINT8     Correction;   //To account for fading.
                          //Correction is in 8.1 format.
                          //and can be negative
   UINT8    TxSuccess;    //For Rate Adaptation
   UINT8    TxFailures;   //For Rate Adaptation.
   UINT8    TxFailuresCorrection; //For Correction handling.
   UINT8    TxRetry;      //For Rate Adaptation.
   #ifdef ENABLE_OSCILLATION_CODE
   UINT8 OSC_FLAG; //if 1, correction won't be decremented.
   UINT8 RSSI_Store;
   //Since recording the RSSI is not perfect, we will use this 
   //counter to disable the Osc Flag if the flag set for a long time.
   UINT32 DisableOscFlagCounter;
   #endif //ENABLE_OSCILLATION_CODE
   //For maintaining the statistical counter for logging failures.
   UINT8 PktTxSinceFail;
   UINT8 TxFailuresStats;
   UINT8 RateIncreaseIncrement;
} rate_AssocReqData_t;

extern void rate_SetupAdaptRow(void);
extern void rate_SetupAssocTableRateAdapt(void);
extern void rate_UpdateRxStats(rate_RxSign_t *RxSignOfStation_p, int len);
extern void rate_UpdateTxStats(MRVL_TX_DONE_INFO *TxInfo_p);
extern UINT8 rate_GetTxRate(void);

#endif /* AP_URPTR */

/*#define PRINTDEBUG */
#define ADAPT_ON_TX_ERR
#define ENABLE_POWER_ADAPT 
#define ENABLE_ADAPTIVE_BAILOUT 1 
/*#define UPDATE_TX_QUEUE */
#define MULTIPATH_FIX
#define STATISTICAL_METHOD

//Mapping of Rates to indices.
#define RATEINDEX_1MBPS 0

#define RATEINDEX_6MBPS 5
#define RATEINDEX_11MBPS 3
#define RATEINDEX_22MBPS 4
#define RATEINDEX_36MBPS 10
#define RATEINDEX_54MBPS 12
#define RATEINDEX_72MBPS 13


/*Fill teh Adapt Row for teh Table. */
void SetupAdaptRow();

/*Fill teh Assoc Table with initial values. */
void SetupAssocTableRateAdapt(AssocReqData_t* pAssocTableOfStation);

/*void PrintStationStats(AssocReqData_t* pAssocTableOfStation, rx80211_Frame_t * Data11Frame_p); */
#ifdef ECOS
UINT8 GetStationRate(AssocReqData_t* pAssocTableOfStation, UINT8 ClientMode, UINT16 Aid) __attribute__ ((section (".itcmbuf")));
#else
UINT8 GetStationRate(AssocReqData_t* pAssocTableOfStation, UINT8 ClientMode, UINT16 Aid);
#endif

#ifdef ECOS /* Be careful not to modify ECOS functions */
//extern UINT8 (*GetStationRateFp)(AssocReqData_t* pAssocTableOfStation, UINT8 ClientMode, UINT16 Aid) ;
void UpdateStationTxStats(AssocReqData_t* pAssocTableOfStation, MRVL_TX_DONE_INFO * doneInfo_p, UINT16 stnId) __attribute__ ((section (".itcmbuf"))); 
extern void (*UpdateStationTxStatsFp)(AssocReqData_t* pAssocTableOfStation, MRVL_TX_DONE_INFO * doneInfo_p, UINT16 stnId);
void UpdateStationRxStats(AssocReqData_t* pAssocTableOfStation, WLAN_RX_FRAME * Data11Frame_p) __attribute__ ((section (".itcmbuf")));
extern void (*UpdateStationRxStatsFp)(AssocReqData_t* pAssocTableOfStation, WLAN_RX_FRAME * Data11Frame_p);
#else
extern UINT8 (*GetStationRateFp)(AssocReqData_t* pAssocTableOfStation, UINT8 ClientMode, UINT16 Aid) ;
void UpdateStationTxStats(AssocReqData_t* pAssocTableOfStation, MRVL_TX_DONE_INFO * doneInfo_p, UINT16 stnId);
extern void (*UpdateStationTxStatsFp)(AssocReqData_t* pAssocTableOfStation, MRVL_TX_DONE_INFO * doneInfo_p, UINT16 stnId);
void UpdateStationRxStats(AssocReqData_t* pAssocTableOfStation, WLAN_RX_FRAME * Data11Frame_p);
extern void (*UpdateStationRxStatsFp)(AssocReqData_t* pAssocTableOfStation, WLAN_RX_FRAME * Data11Frame_p);
#endif
UINT8 RECV_PKT_TYPE; /*0=>both data and Acks. 1=> Only Data. 2=> Only Acks. */

UINT16 lastTxStnId; /*to use when ENABLE_DASEBAND_ACKS. To get reading from Ack pkt */

void EnablePowerAdapt(BOOLEAN flag);

void UpdateStationAckStats(AssocReqData_t* pAssocTableOfStation);

void UpdateTxQueue();

void UpdateAssocStnData(UINT8 Aid);
#endif /* WB_EXCEPTION */

void (*UpdateBailOutRateFp)(UINT8 Rate, UINT16 Aid);
void (*UpdatePowerFp)(UINT8 Rate, UINT16 Aid, SINT32 Result);
void apiSetHiPower();

#endif
