#ifndef _RX80211_H_
#define _RX80211_H_

// The portion of the header that is Marvell specific:
typedef struct w8100_MrvlRxHdr_t
{
#ifdef CONDOR2
   UINT8    RxParam;          // Receive parameters 0: received using OFDM
                              //   1: received using CCK
   UINT8    Sq2;              // Signal Quality 2
   UINT16   Signature;        // Signature to distinguish fragmented or whole frame
   UINT8    AntPream;         // Antenna/Preamble
   UINT8    Sq1;              // Signal Quality 1
   UINT8    Rate;             // rate at which frame was received 
   UINT8    RSSI;             // RF Signal Strength Indicator
   UINT8    croffset;   /** 4 extra byte on condor 2 here **/
   UINT8    troffset;
   UINT8    Noiseflr;
   UINT8    Sqm;
#ifdef QOS_FEATURE
   IEEEtypes_QoS_Ctl_t QosControl;
#else
   UINT16   QosControl;       // QoS Control field
#endif
   UINT16   Service;
#else
   UINT8    RxParam;          // Receive parameters 0: received using OFDM
                              //   1: received using CCK
   UINT8    Sq2;              // Signal Quality 2
   UINT16   Signature;        // Signature to distinguish fragmented or whole frame
   UINT8    AntPream;         // Antenna/Preamble
   UINT8    Sq1;              // Signal Quality 1
   UINT8    Rate;             // rate at which frame was received 
   UINT8    RSSI;             // RF Signal Strength Indicator
#ifdef QOS_FEATURE
   IEEEtypes_QoS_Ctl_t QosControl;
#else
   UINT16   QosControl;       // QoS Control field
#endif
   UINT16   Service;
#endif
} PACK_END w8100_MrvlRxHdr_t;

#define RxSign_t w8100_MrvlRxHdr_t

typedef struct rx80211_MgmtMsg_t
{
    union
    {
        RxSign_t RxSign;
        SwInfo_t SwInfo;
    } u;
    macmgmtQ_MgmtMsg_t MgmtFrame;
}
PACK_END rx80211_MgmtMsg_t;

typedef struct rx80211_DataFrame_t
{
    union
    {
        RxSign_t RxSign;
        SwInfo_t SwInfo;
    } u;
    IEEEtypes_DataFrame_t DataFrame;
}
PACK_END rx80211_DataFrame_t;

typedef struct rx80211_Frame_t
{
    union
    {
        RxSign_t RxSign;
        SwInfo_t SwInfo;
    } u;
    IEEEtypes_GenHdr_t Hdr;
    UINT8 FrmBody[1];
}
PACK_END rx80211_Frame_t;

typedef struct statistics
{
    UINT32 UpdateQCnt;
    UINT32 DoneQCnt;
    UINT32 RxBufErrCnt;
    UINT32 DataRecvCnt;
    UINT32 DataFreedCnt;
    UINT32 DataFreedOutofSeq;
    UINT32 DataDoubleFreeCnt;
    UINT32 MgmtRecvCnt;
    UINT32 MgmtFreedCnt;
    UINT32 MgmtFreedOutofSeq;
    UINT32 MgmtDoubleFreeCnt;
    UINT32 DataQCnt;
    UINT32 DataTxFailCnt;
    UINT32 RetryCnt;
    UINT32 DataTxSuccessCnt;
    UINT32 UnassociatedSrcCnt;
    UINT32 UnassociatedDstCnt;
    UINT32 DroppedCnt;
    UINT32 RxMemFragErrCnt;
    UINT32 TxErrCnt;
    UINT32 TxWatchDogTimeouts;
    UINT32 EthRecvCnt;
    UINT32 EthTxCnt;
    UINT32 EthRxResourceErrCnt;
    UINT32 EthRxErrCnt;
    UINT32 EthTxFailCnt;
    UINT32 DmaNotFinishedCnt;
    UINT32 LockFailCnt;
    UINT32 TkipICVCnt;
    UINT32 TkipMICCnt;
#ifdef B0_CHIP
    UINT32 RxWatchdogTimeouts;
#endif
} Statistics;

extern Statistics Stats;
#endif /* _RX80211_H_ */
