#ifndef _SHA1_H_
#define _SHA1_H_

enum
{
    shaSuccess = 0,
    shaNull,            /* Null pointer parameter */
    shaInputTooLong,    /* input data too long */
    shaStateError       /* called Input after Result */
};

#define SHA1HashSize 20
#define uint32_t unsigned int
#define uint8_t unsigned char
#define int_least16_t short
#define A_SHA_CTX SHA1Context
/*
 *  This structure will hold context information for the SHA-1
 *  hashing operation
 */
typedef struct
{
    uint32_t Intermediate_Hash[SHA1HashSize/4]; /* Message Digest  */

    uint32_t Length_Low;            /* Message length in bits      */
    uint32_t Length_High;           /* Message length in bits      */

                               /* Index into message block array   */
    int_least16_t Message_Block_Index;
    uint8_t Message_Block[64];      /* 512-bit message blocks      */

    int Computed;               /* Is the digest computed?         */
    int Corrupted;             /* Is the message digest corrupted? */
} Mrvl_SHA1_CTX;

/*
 *  Function Prototypes
 */

int Mrvl_SHA1Init(Mrvl_SHA1_CTX *);
int Mrvl_SHA1Update(Mrvl_SHA1_CTX *, const uint8_t *, unsigned int);
int Mrvl_SHA1Final(Mrvl_SHA1_CTX *, uint8_t Message_Digest[SHA1HashSize]);
void Mrvl_PRF(unsigned char *key, int key_len, unsigned char *prefix,
         int prefix_len, unsigned char *data, int data_len, 
         unsigned char *output, int len);
void Mrvl_hmac_sha1(
              unsigned char *text, int text_len,
              unsigned char *key, int key_len,
              unsigned char *digest);


#endif

