/******************** (c) Marvell Semiconductor, Inc., 2001 *******************
*
* Purpose:
*    This file contains the configuration parameters used for tasks, queues,
*    and other operating system components.
*
* Public Procedures:
*    None.
*
* Notes:
*    None.
*
*****************************************************************************/
#ifndef _SYSINFO_H_
#define _SYSINFO_H_

//=============================================================================
//                            PUBLIC DEFINITIONS
//=============================================================================

/*--------------*/
/* System Sizes */
/*--------------*/
#define  sysinfo_TOTAL_TASKS        8// 10
#define  sysinfo_TOTAL_QUEUES        64
#ifdef VOIP
#define  sysinfo_TOTAL_INTERRUPTS    11 // Mehrdad Increased for TDU interrupt
#else
#define  sysinfo_TOTAL_INTERRUPTS    10
#endif
#define  sysinfo_MAX_STATIONS   64//64+32/* max stations per AP */
#define  sysinfo_TOTAL_TIMERS   8//sysinfo_MAX_STATIONS+32//8--> Rahul. The number of software timers need to be equal to the MAX # of Stations for the group Key to work.
#define  sysinfo_TOTAL_SEMAPHORES    16
#define  sysinfo_TOTAL_EVENTS        15

/*----------*/
/* Task IDs */
/*----------*/
#define  sysinfo_ROOT_TASK                0
#define  sysinfo_IDLE_TASK                1
#define  sysinfo_SME_TASK                 2
#define  sysinfo_MAC_MGMT_TASK            3
#define  sysinfo_802_11_MSG_TX_TASK       4
#define  sysinfo_DS_TASK                  5
#define  sysinfo_TEST_TASK               6
#define  sysinfo_BCNGEN_TASK            7
#define  sysinfo_WEB_TASK               8
#define sysinfo_DHCP_TASK       9

/*------------*/
/* Task Names */
/*------------*/
#define  sysinfo_ROOT_TASK_NAME              "Root"
#define  sysinfo_IDLE_TASK_NAME              "Idle"
#define  sysinfo_SME_TASK_NAME               "SME"
#define  sysinfo_MAC_MGMT_TASK_NAME          "MAC Mgmt"
#define  sysinfo_802_11_MSG_TX_TASK_NAME     "MAC Tx"
#define  sysinfo_DS_TASK_NAME                "Distribution"
#define  sysinfo_TEST_TASK_NAME              "Test"
#define  sysinfo_BCNGEN_TASK_NAME            "Bcngen"
#define  sysinfo_WEB_TASK_NAME               "Web"
#define  sysinfo_DHCP_TASK_NAME               "DHCPTask"



/*-----------------*/
/* Task Priorities */
/*-----------------*/
#define  sysinfo_ROOT_TASK_PRI                1
#define  sysinfo_IDLE_TASK_PRI                31
#define  sysinfo_SME_TASK_PRI                15
#define  sysinfo_MAC_MGMT_TASK_PRI           5
#define  sysinfo_802_11_MSG_TX_TASK_PRI       4
#define  sysinfo_DS_TASK_PRI                  6
#define  sysinfo_TEST_TASK_PRI               23
#define  sysinfo_BCNGEN_TASK_PRI             10
#define  sysinfo_WEB_TASK_PRI                21
#define  sysinfo_DHCP_TASK_PRI                18

/*------------------*/
/* Task Stack Sizes */
/*------------------*/

#define  sysinfo_ROOT_TASK_STACK_SIZE            CYGNUM_HAL_STACK_SIZE_TYPICAL
#define  sysinfo_IDLE_TASK_STACK_SIZE            CYGNUM_HAL_STACK_SIZE_TYPICAL 
#define  sysinfo_SME_TASK_STACK_SIZE             CYGNUM_HAL_STACK_SIZE_TYPICAL 
#define  sysinfo_MAC_MGMT_TASK_STACK_SIZE        CYGNUM_HAL_STACK_SIZE_TYPICAL 
#define  sysinfo_802_11_MSG_TX_TASK_STACK_SIZE   CYGNUM_HAL_STACK_SIZE_TYPICAL
#define  sysinfo_DS_TASK_STACK_SIZE              (CYGNUM_HAL_STACK_SIZE_TYPICAL+8192)
#define  sysinfo_TEST_TASK_STACK_SIZE            2048 
#define  sysinfo_BCNGEN_TASK_STACK_SIZE          CYGNUM_HAL_STACK_SIZE_TYPICAL
#define  sysinfo_WEB_TASK_STACK_SIZE             (CYGNUM_HAL_STACK_SIZE_TYPICAL+8192)

#ifdef TRIGGEREVENT
#define sysinfo_TOTAL_STACK_SIZE      (sysinfo_ROOT_TASK_STACK_SIZE + \
                                      sysinfo_SME_TASK_STACK_SIZE  + \
                                      sysinfo_MAC_MGMT_TASK_STACK_SIZE + \
                                      sysinfo_802_11_MSG_TX_TASK_STACK_SIZE + \
                                      sysinfo_DS_TASK_STACK_SIZE  + \
                                      sysinfo_TEST_TASK_STACK_SIZE + \
                                      sysinfo_DHCP_TASK_STACK_SIZE + \
                                      sysinfo_BCNGEN_TASK_STACK_SIZE)

#else
#define sysinfo_TOTAL_STACK_SIZE      (sysinfo_ROOT_TASK_STACK_SIZE + \
                                      sysinfo_SME_TASK_STACK_SIZE  + \
                                      sysinfo_MAC_MGMT_TASK_STACK_SIZE + \
                                      sysinfo_802_11_MSG_TX_TASK_STACK_SIZE + \
                                      sysinfo_TEST_TASK_STACK_SIZE + \
                                      sysinfo_BCNGEN_TASK_STACK_SIZE)
                                      
#endif



/*-----------*/
/* Queue IDs */
/*-----------*/

#define  sysinfo_SME_MGMT_MSG_Q           0
#define  sysinfo_SME_HOST_CMD_Q           1
#define  sysinfo_AP_API_RESP_Q           2
#define  sysinfo_SME_ERROR_MSG_Q          3
#define  sysinfo_MAC_MGMT_802_11_MSG_Q    4
#define  sysinfo_MAC_MGMT_SME_MSG_Q       5
#define  sysinfo_802_11_MSG_TX_Q          6
#define  sysinfo_DS_NOTIFY_Q              7
#define  sysinfo_802_11_DATA_TX_Q         8
#define  sysinfo_802_11_RX_PS_POLL_Q      9
#define  sysinfo_MAC_MGMT_TIMER_MSG_Q     10
#define  sysinfo_DS_LLC_DATA_Q            11
//#define  sysinfo_STA_ASSOC_STATE_Q        12 
#define  sysinfo_WPA_Q                    12
#define  sysinfo_ROAMING_TX_Q    13
#define  sysinfo_TIMER_CALL_BACK_Q        14  


/*-----------*/
/* Event IDs */
/*-----------*/
#define  sysinfo_HOST_CMD_Q_EVENT             0
#define  sysinfo_SME_MAIN_Q_EVENT            1
#define  sysinfo_MAC_MGMT_MAIN_Q_EVENT       2
#define  sysinfo_802_11_TX_NOTIFY_Q_EVENT    3
#define  sysinfo_802_11_RX_EVENT             4
#define  sysinfo_802_11_TX_Q_EVENT           5
#define  sysinfo_BCN_Q_EVENT                 6
#define  sysinfo_DS_EVENT                    7
#define  sysinfo_BCNGEN_EVENT              8
#define  sysinfo_API_RESP_Q_EVENT               9

/*---------------*/
/* Semaphore IDs */
/*---------------*/
#define  sysinfo_EXT_STA_SEM   0
#define  sysinfo_TIMER_SEM     1
#define  sysinfo_TIMER_DB_SEM     2
#define  sysinfo_MIC_FAIL_SEM     3
#define  sysinfo_STALIST_SEM   4

/*---------------*/
/* Interrupt IDs */
/*---------------*/
#define  sysinfo_RX_MGMT_MSG_INT          0
#define  sysinfo_RX_DATA_MSG_INT          1
#define  sysinfo_RX_CTL_MSG_INT           2
#define  sysinfo_RX_BCN_INT               3
#define  sysinfo_RX_HOST_MSG_INT          4
#define  sysinfo_TX_NOTIFICATION_INT      5
#define  sysinfo_PWR_MODE_UPDATE_INT      6

/*-----------*/
/* Timer IDs */
/*-----------*/
#define  sysinfo_TIMER_1    0
#define  sysinfo_TIMER_2    1
#define  sysinfo_BCNTIMER   3

#define  sysinfo_MAX_STATIONS   64 /* max stations per AP */


#endif /* _SYSINFO_H_ */
