#ifndef UART_H
#define UART_H

typedef struct {
    UINT8  *data;
    UINT32 put;
    UINT32 get;
    UINT32 nbytes;     /* number of bytes currently in buffer */
    UINT32 size;
} UART_CB_T;

/*--------------------------------------------------------------*/
/*  UART registers                        */
/*--------------------------------------------------------------*/
#define WL_UART_RBR           ((WL_LBU_SIU_CFG_BASE)|(0x0040))
#define WL_UART_THR           ((WL_LBU_SIU_CFG_BASE)|(0x0040))
#define WL_UART_DLL           ((WL_LBU_SIU_CFG_BASE)|(0x0040))
#define WL_UART_IER           ((WL_LBU_SIU_CFG_BASE)|(0x0044))
#define WL_UART_DLH           ((WL_LBU_SIU_CFG_BASE)|(0x0044))
#define WL_UART_ISR_ID        ((WL_LBU_SIU_CFG_BASE)|(0x0048))
#define WL_UART_FIFO_CTRL     ((WL_LBU_SIU_CFG_BASE)|(0x0048))
#define WL_UART_LCR           ((WL_LBU_SIU_CFG_BASE)|(0x004c))
#define WL_UART_MCR           ((WL_LBU_SIU_CFG_BASE)|(0x0050))
#define WL_UART_LSR           ((WL_LBU_SIU_CFG_BASE)|(0x0054))
#define WL_UART_SCR           ((WL_LBU_SIU_CFG_BASE)|(0x005c))

/* Interrupt ID regiser */
#define SIO_ISR_ID_RX   0x04            /* receive data available */
#define SIO_ISR_ID_RXTO 0x0C            /* character timeout */
#define SIO_ISR_ID_TX   0x02            /* tx holding register empty */

/* The line status register bits. */
#define SIO_LSR_DR      0x01            /* data ready */
#define SIO_LSR_OE      0x02            /* overrun error */
#define SIO_LSR_PE      0x04            /* parity error */
#define SIO_LSR_FE      0x08            /* framing error */
#define SIO_LSR_BI      0x10            /* break interrupt */
#define SIO_LSR_THRE    0x20            /* transmitter holding register empty */
#define SIO_LSR_TEMT    0x40            /* transmitter register empty */
#define SIO_LSR_ERR     0x80            /* any error condition */

/* The modem status register bits. */
#define SIO_MSR_DCTS  0x01              /* delta clear to send */
#define SIO_MSR_DDSR  0x02              /* delta data set ready */
#define SIO_MSR_TERI  0x04              /* trailing edge ring indicator */
#define SIO_MSR_DDCD  0x08              /* delta data carrier detect */
#define SIO_MSR_CTS   0x10              /* clear to send */
#define SIO_MSR_DSR   0x20              /* data set ready */
#define SIO_MSR_RI    0x40              /* ring indicator */
#define SIO_MSR_DCD   0x80              /* data carrier detect */

/* The line control register bits. */
#define SIO_LCR_WLS0   0x01             /* word length select bit 0 */
#define SIO_LCR_WLS1   0x02             /* word length select bit 1 */
#define SIO_LCR_STB    0x04             /* number of stop bits */
#define SIO_LCR_PEN    0x08             /* parity enable */
#define SIO_LCR_EPS    0x10             /* even parity select */
#define SIO_LCR_SP     0x20             /* stick parity */
#define SIO_LCR_SB     0x40             /* set break */
#define SIO_LCR_DLAB   0x80             /* divisor latch access bit */

/* Baud rate = 38400 */
#define WL_UART_BAUD_MSB_38400        0x00
#define WL_UART_BAUD_LSB_38400        0x0D

/* Baud rate = 19200 */
#define WL_UART_BAUD_MSB_19200        0x00
#define WL_UART_BAUD_LSB_19200        0x1A

/* Baud rate = 9600 */
#define WL_UART_BAUD_MSB_9600        0x00
#define WL_UART_BAUD_LSB_9600        0x34

/* Baud rate = 4800 */
#define WL_UART_BAUD_MSB_4800        0x00
#define WL_UART_BAUD_LSB_4800        0x68

/* Baud rate = 2400 */
#define WL_UART_BAUD_MSB_2400        0x00
#define WL_UART_BAUD_LSB_2400        0xD0

#define SEL_8MHZ_CLK_FROM_132MHZ     0x00
#define SEL_8MHZ_CLK_FROM_88MHZ      0x01
#define SEL_8MHZ_CLK_FROM_44MHZ      0x02
#define SEL_8MHZ_CLK_FROM_22MHZ      0x03

#endif /* UART_H */
