/******************** (c) Marvell Semiconductor, Inc., 2004 *******************
*
* Purpose:
*    This file contains the implementations of the AP API.
*   
*
* Public Procedures:
*
* Private Procedures:
*  
*
* Notes:
*    None.
*
*****************************************************************************/

//=============================================================================
//                               INCLUDE FILES
//=============================================================================
#include "oem.h"
#include "wltypes.h"
#include "IEEE_types.h"
#include "osif.h"
#include "host.h"
#include "sysinfo.h"
#include "host.h"
#include "pool.h"
#include "mib.h"
#include "smeMain.h"
#include "api2.h"
#include "api.h"
#include "util.h"
#include "mib.h"
#include "memmap.h"
#include "qos.h"
#include "wlmac.h"
#include "ds.h"
#include "keyMgmtCommon.h"
#include "keyMgmt.h"
#include "tkip.h"
#include "StaDb.h"
#include "macmgmtap.h"
#include "tx80211.h"
#include "txsched.h"
#if !defined(AP22) && !defined(GST_AP31) 
 #include <network.h>
 #include <dhcp.h>
#endif
#include "etherswitch.h"
#include "msApi.h"
#include "gtDrvSwRegs.h"
#include "gtHwCntl.h"
#include "macMgmtMlme.h"
#include "ether.h"
#include "etherLow.h"
#include "bufferinit.h"
#include "wl_macros.h"
#include "wpa.h"
#include "rateadapt.h"
#include "sad.h"
#include "buildModes.h"
#include "usrcfg.h"
/*============================================================================= */
/*                                DEFINITIONS */
/*============================================================================= */
#define MAX_API_DATA_THRESHOLD 1024

/* GPIO is accessible through the LBU register space */



#define GPIO_REG(offset) (WL_LBU_GPU_CFG_BASE | offset)

#define GPIO_REG_SELECT_0_7         WL_LBU_GPU_CFG_BASE
#define GPIO_REG_SELECT_8_15        GPIO_REG(0x04)
#define GPIO_REG_OUTPUT_ENABLE      GPIO_REG(0x08)
#define GPIO_REG_OUTPUTS            GPIO_REG(0x0C)
#define GPIO_REG_INPUTS             GPIO_REG(0x10)
#define GPIO_REG_IER                GPIO_REG(0x14)
#define GPIO_REG_IMR                GPIO_REG(0x18)
#define GPIO_REG_ISR                GPIO_REG(0x20)
#define GPIO_REG_STRETCH            GPIO_REG(0x4c)

#define SW_LED_OUTPUT_ENABLE GPIO_REG(0x24)
#define SW_LED_CYCLE_3_0  GPIO_REG(0x28)
#define SW_LED_CYCLE_7_4 GPIO_REG(0x2c)
#define SW_LED_CYCLE_11_8 GPIO_REG(0x30)
#define SW_LED_CYCLE_15_12 GPIO_REG(0x34)
#define SW_LED_DUTY_CYCLE_3_0 GPIO_REG(0x38)
#define SW_LED_DUTY_CYCLE_7_4 GPIO_REG(0x3C)
#define SW_LED_DUTY_CYCLE_11_8 GPIO_REG(0x40)
#define SW_LED_DUTY_CYCLE_15_12 GPIO_REG(0x44)
#define LED_OUTPUT_ENABLE_FOR_STATUS_SIGNALS GPIO_REG(0x48)
#define LED_STRETCH_DURATION_FOR_STATUS_SIGNALS GPIO_REG(0x4c)
#define PCLK_CLOCK_DIVIDER_TO_GENERATE_LED_CLK GPIO_REG(0x50)



#define ETHER_HANDLE 1
#define WLAN_HANDLE 2

#define DS_EVENTS     (APIO_POLL_ETH_DATA_RDY|APIO_POLL_RADIO_DATA_RDY)

/*============================================================================= */
/*                   PRIVATE PROCEDURES (ANSI Prototypes) */
/*============================================================================= */

/*============================================================================= */
/*                         CODED PUBLIC PROCEDURES */
/*============================================================================= */
extern char *FIRMWARE_VERSION;

#ifdef WATCHDOG_SIMULATION
extern UINT8 tmp_cnt;
#endif

#ifdef AP_URPTR
extern UINT8 mib_urMode;
extern UINT8 mib_wbMode;
extern UINT8 mib_urModeConfig;
extern UINT16 mib_urRsvd;
extern UINT8  mib_urSsid[33];
extern IEEEtypes_MacAddr_t  mib_urBssid;
extern UINT8 mib_urPrefBssid;
extern MIB_STA_CFG mib_StaCfg_ur;
#endif

extern IEEEtypes_MacAddr_t macStaAddr;
extern UINT32 mib_ErpProtEnabled;
extern UINT32 NoTxInfoPoolAvailable;
extern IEEEtypes_RSN_IE_t thisStaRsnIE;
#ifdef AP_WPA2
extern IEEEtypes_RSN_IE_WPA2_t thisStaRsnIEWPA2;
extern IEEEtypes_RSN_IE_WPA2MixedMode_t thisStaRsnIEWPA2MixedMode;
#endif

extern UINT32 apio_wlanRead(apio_bufdescr_t *ppBufDescr);
extern UINT32 apio_wlanWrite(apio_bufdescr_t *ppBufDescr);
extern UINT8 *apio_GetEtherTxBuf(void);
extern inline void apio_FreeEtherRxBuf(apio_bufdescr_t *pBufDescr);
extern void apiSetHiPower();

/* Timer alarm */
static short activeAlarmCnt = 0;

#define MAX_TIMER_ALARM 3

/* Memory for the timer alarm object */
static cyg_alarm timerAlarm;

/* The alarm handle, use when delete alarm */
static cyg_handle_t hAlarm;

static unsigned int alarmData;

/* Global interrupt enable status */
static int intStatus = 0;
#ifdef AP5X
extern UINT8 WepType[];
extern UINT8 WepType_G[];
#else
extern UINT32 WepType;
extern UINT32 WepType_G;
#endif
extern GT_QD_DEV etherDev;

extern GT_STATUS gprtGetLinkState(IN GT_QD_DEV * dev, IN GT_LPORT port, OUT GT_BOOL * state);
extern GT_STATUS gprtGetDuplex(IN GT_QD_DEV * dev, IN GT_LPORT port, OUT GT_BOOL * mode);
extern GT_STATUS gprtGetSpeed(IN GT_QD_DEV * dev, IN GT_LPORT port, OUT GT_BOOL * speed);
extern GT_STATUS  gprtPortPowerDown(IN GT_QD_DEV * dev,IN GT_LPORT port, IN GT_BOOL state);

#ifdef WDS_FEATURE
UINT8 getDynamicWdsMac(UINT8 *buf);
#endif

#ifdef AP_URPTR
extern UINT32 msi_wl_CheckURSupport(void);
#endif

extern UINT32 EtherPoolStart;

#ifdef WPA
extern UINT32 g_IV32;
extern UINT16 g_IV16;
#endif
extern UINT8 MICCounterMeasureEnabled;
extern UINT8 EtherInitComplete;
UINT8 Radio_off=0;
extern UINT8 PowerReg20;
extern UINT8  PowerReg26;
extern BOOLEAN gGrpKeyInstalled;
extern int MfgTestModeOn;

#ifdef QOS_FEATURE
extern TSPECEntry_t TSpecEntryDb[MAX_QOS_STRMS];
extern mib_QAPEDCATable_t mib_QAPEDCATable[4];
#endif
extern txInfo_t *BcnTxInfo;
extern txBcnInfo_t *PrbRspTxInfo;

#ifdef UR_WPA
extern UINT8   PSKValueUr[40];
inline rx80211_Frame_t *(*ProcessWPAPcktStaUnicastFp)(rx80211_Frame_t *pRxPckt, UINT32 Len);
inline rx80211_Frame_t *(*ProcessWPAPcktStaBcastFp)(rx80211_Frame_t *pRxPckt, UINT32 Len);
extern UINT32 (*DoWPAAndSchedFrameStaFp)(tx_pktbuf_t *tx80211_p, IEEEtypes_8023_Frame_t  *DataEtherFrame_p, UINT8 * llcSnap_p);
inline UINT32 DoTKIPAndSchedFrameStaUr(tx_pktbuf_t *tx80211_p, IEEEtypes_8023_Frame_t  *DataEtherFrame_p, UINT8 * llcSnap_p);
inline rx80211_Frame_t *ProcessTKIPPcktStaUr(rx80211_Frame_t *pRxPckt);
inline UINT32 DoCCMPAndSchedFrameStaUr(tx_pktbuf_t *tx80211_p, IEEEtypes_8023_Frame_t  *DataEtherFrame_p, UINT8 * llcSnap_p);
inline rx80211_Frame_t *ProcessCCMPPcktStaUr(rx80211_Frame_t *pRxPckt, UINT32 Len);
#endif


#ifdef JAPAN_CHANNEL_SPACING_10_SUPPORT
extern UINT8 mib_channelspacing;
#endif

#ifdef DCS
extern UINT8 BkgndScanActive;
extern API_BGSCAN_BSS_ENTRY    bkgndScanBSSData[BGSCAN_ENTRY_MAX];
extern API_BGSCAN_CH_ENTRY     bkgndScanCHData[IEEEtypes_MAX_CHANNELS+IEEEtypes_MAX_CHANNELS_A];
extern API_CH_WEIGHT_ENTRY  CHWeightData[IEEEtypes_MAX_CHANNELS+IEEEtypes_MAX_CHANNELS_A];

UINT8 SetDCSChannel(UINT8 channel);
#endif

static void dummy()
{
}

#if defined(AP_SITE_SURVEY) || defined(AUTOCHANNEL)
/* For Site Survey */
#define DEFAULT_CHAN_MIN        1
#define DEFAULT_CHAN_MAX        11
extern UINT8  busyScanning;

extern void SendScanCmd(UINT8 *channels);

#if defined(AP_SITE_SURVEY)
#define SITE_SURVEY_ENTRY_MAX   50
extern API_SURVEY_ENTRY  siteSurveyInfo[SITE_SURVEY_ENTRY_MAX];
#endif /* AP_SITE_SURVEY */
extern void SetupScan(void)
{
    if(!busyScanning)
    {
        UINT8  scanChannel[IEEEtypes_MAX_CHANNELS+IEEEtypes_MAX_CHANNELS_A] =
         {1,2,3,4,5,6,7,8,9,10,11,12,13,0,36,40,44,48,52,56,60,64,149,153,157,161,165,0,0,0,0,0,0};
        /* get range to scan */
        #ifdef DOMAIN_SUPPORTED
        if (domainGetInfo(scanChannel));
        #endif
        SendScanCmd(scanChannel);
    }
}
#endif

#if defined(GATEWAY) || defined(DHCPS)
typedef struct MIB_IP_LAN_s
{
    unsigned long IPAddr;
    unsigned long SubnetMask;
}
MIB_IP_LAN;
extern MIB_IP_LAN mib_IP_LAN;
#endif

#ifdef IEEE80211H
extern UINT8 bcn_reg_domain;
#endif /* IEEE80211H*/

/******************************************************************************
 *
 * Name: smeMain_SendEvent
 *
 * Description:
 *   This routine provides the mechanism to trigger an event indicating to
 *   the api called Task that a message has been written to one of its queues.
 *
 * Conditions For Use: 
 *   
 *
 * Arguments:
 *   Arg1 (i  ): Event - value indicating which queue has received a message
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS api_SendEvent( UINT8 Event )
{


    if (Event == api_RESP_RCVD )
    {
#ifdef USE_NEW_OSIF
        os_EventTrigger(apiQH, Event);
#else
        os_EventTrigger(sysinfo_API_RESP_Q_EVENT, Event);
#endif
        return(OS_SUCCESS);
    }
    else
    {
        return(OS_FAIL);
    }

}

/******************************************************************************
 *
 * Name: api_CmdWriteNoBlock
 *
 * Description:
 *
 * Conditions For Use:
 *   
 *
 * Arguments:
 *   Arg1 (i  ): Cmd_p - a pointer to the message to be placed on the
 *                         queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS api_CmdWriteNoBlock( api_Cmd_t *Cmd_p )
{
    WL_STATUS status;
    //if(systemCfg.prod==AP52)
    {
#ifdef USE_NEW_OSIF
        if ( (status = os_QueueWriteNoBlock(smeQH,
#else
        if ( (status = os_QueueWriteNoBlock(sysinfo_SME_HOST_CMD_Q,
#endif
                                            Cmd_p)) == OS_SUCCESS ) {
            smeMain_SendEvent(smeMain_CMD_RCVD);
            return(OS_SUCCESS);
        }
        else {
            return(OS_FAIL);
        }
    }    
    return(OS_FAIL);
}

/******************************************************************************
 *
 * Name: smeQ_ApiRestWriteNoBlock
 *
 * Description:
 *   This routine is called to write a message to the queue where API responses
 *    are placed for Command originator. If writing
 *   to the queue cannot immediately occur, then the routine returns with
 *   a failure status (non-blocking).
 *
 * Conditions For Use:
 *   The queue has been initialized by calling smeQ_Init().
 *
 * Arguments:
 *   Arg1 (i  ): ApiResp_p - a pointer to the message to be placed on the
 *                         queue
 *
 * Return Value:
 *   Status indicating success or failure
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern WL_STATUS api_RespWriteNoBlock( api_Cmd_t *ApiResp_p )
{
    WL_STATUS status;
    if(systemCfg.prod==AP52)
    {
#ifdef USE_NEW_OSIF
        if ( (status = os_QueueWriteNoBlock(apiQH,
#else
        if ( (status = os_QueueWriteNoBlock(sysinfo_AP_API_RESP_Q,
#endif
                                            ApiResp_p)) == OS_SUCCESS ) {
            smeMain_SendEvent(api_RESP_RCVD);
            return(OS_SUCCESS);
        }
        else {
            return(OS_FAIL);
        }
    }
    return(OS_FAIL);
}

/******************************************************************************
 *
 * Name: smeQ_ApiRestReadNoBlock
 *
 * Description:
 
 *
 * Conditions For Use:
 *   
 *
 * Arguments:
 *   Arg1 (i  ): 
 *
 * Return Value:
 *   
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern api_Cmd_t *api_RespReadBlock(void)
{
    if(systemCfg.prod==AP52)
#ifdef USE_NEW_OSIF
        return((api_Cmd_t *) os_QueueReadNoBlock(apiQH));
#else
        return ((api_Cmd_t *) os_QueueReadNoBlock(sysinfo_AP_API_RESP_Q));
#endif
    return NULL;
}
/******************************************************************************
 *
 * Name:  
 *
 * Description:
 *
 * Conditions For Use:
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
extern api_Cmd_t *api_GetCmdBuf(void)
{
    return ((api_Cmd_t *)pool_GetBuf(smeHostCmdPoolId));
}

extern void smeQ_FreeHostCmdBuf(smeQ_HostCmd_t *HostCmd_p)
{
    pool_FreeBuf((UINT8 *)HostCmd_p);
}

int CMHz2Index(UINT16 MHz)
{
    int index = CPU_MHZ_44;
    if (MHz > 44 ) 
    {
        index = CPU_MHZ_64;
    }
    if (MHz > 64 ) 
    {
        index = CPU_MHZ_80;
    }
    if (MHz > 80 ) 
    {
        index = CPU_MHZ_106;
    }
    if (MHz > 106 ) 
    {
        index = CPU_MHZ_128;
    }    
    if (MHz > 128 ) 
    {
        index = CPU_MHZ_160;
    }
    if (MHz > 160 ) 
    {
        index = CPU_MHZ_176;
    }
    return index;
}
UINT16 Index2CMHz(int index)
{
    UINT16 MHz;
    switch (index) {
    case CPU_MHZ_22:
        MHz = 22;
        break;
    case CPU_MHZ_44:
        MHz = 44;
        break;
    case CPU_MHZ_64:
        MHz = 64;
        break;
    case CPU_MHZ_80:
        MHz = 80;
        break;
    case CPU_MHZ_106:
        MHz = 106;
        break;
    case CPU_MHZ_128:
        MHz = 128;
        break;
    case CPU_MHZ_160:
        MHz = 160;
        break;
    case CPU_MHZ_176:
        MHz = 176;
        break;
    default:
        MHz = 44;
        break;
    }
    return MHz;
}
/******************************************************************************
 *
 * Name:  apctl
 *
 * Description: This function executes the command indicated in the requested structure
 * 
 * Conditions For Use:
 *
 * Arguments:
 *   None.
 *
 * Return Value:
 *   None.
 *
 * Notes:
 *   None.
 *
 * PDL:
 * END PDL
 *
 *****************************************************************************/
static UINT8 *apibuf=NULL;
static UINT16 apidataleft, apidataindex;
#ifdef APCFGUR
UINT8 *AssembleBuf(APICMDBUF *apicmdbuf_p)
{
	static UINT8 *buf=NULL;
	static UINT16 cmdcode;
	static UINT16 cmdLength=0;
	static UINT16 seqNum=0;
	static UINT16 alloclength;
	if(apicmdbuf_p->seqNum == 1)
	{	
		seqNum = apicmdbuf_p->seqNum; 
		cmdcode = apicmdbuf_p->cmdCode;
		cmdLength = apicmdbuf_p->cmdBodyLen;
		alloclength = apicmdbuf_p->cmdBodyLen * 2; 
		buf = malloc(alloclength);
		if (buf)
		{
			memcpy(buf, apicmdbuf_p, apicmdbuf_p->cmdBodyLen + APICMDHEADERLEN);
		}	
	}else
	{
		if((++seqNum == apicmdbuf_p->seqNum) && buf && (apicmdbuf_p->cmdCode == cmdcode) )
		{
			if(	cmdLength + apicmdbuf_p->cmdBodyLen + APICMDHEADERLEN > alloclength)
			{
				UINT8 *tmp;
				alloclength = cmdLength + apicmdbuf_p->cmdBodyLen + APICMDHEADERLEN;
				tmp = malloc(alloclength);
				memcpy(tmp, buf, cmdLength + APICMDHEADERLEN);
				free(buf);
				buf = tmp;
			}
			memcpy(&buf[APICMDHEADERLEN+cmdLength],	(UINT8 *)apicmdbuf_p + APICMDHEADERLEN, apicmdbuf_p->cmdBodyLen);
			cmdLength +=apicmdbuf_p->cmdBodyLen;
		}
		else
		{
			free(buf);
			buf=NULL;
			seqNum =0;
			cmdLength = 0;
		}
	}
	if(apicmdbuf_p->cmdStatus == APCTL_LAST_DATA && buf)
	{
		((APICMDBUF *)buf)->cmdBodyLen = cmdLength;
		((APICMDBUF *)buf)->cmdBody = buf + APICMDHEADERLEN;
		return buf;
	}
	return NULL;
}
#endif
WL_STATUS apctl(APICMDBUF *apicmdbuf_p)
{

    api_Cmd_t *apiCmdQueue;
    unsigned int i;
	APICMDBUF *apicmdbuf;
    UINT16 cmdCode;
    UINT8 tmp;

#ifdef APCFGUR
	if(apicmdbuf_p->cmdStatus == APCTL_MORE_DATA || apicmdbuf_p->cmdStatus == APCTL_LAST_DATA)
	{
		UINT8 *buf;
        buf = AssembleBuf(apicmdbuf_p);
		if(!buf)
		{
        	apicmdbuf_p->cmdStatus = APCTL_OK;
    		return OS_SUCCESS;
		}
		else
		{
			apicmdbuf = (APICMDBUF*)buf;
		}
	}
	else
	{
#else
	{
#endif
		apicmdbuf = apicmdbuf_p;
	}
	cmdCode = apicmdbuf->cmdCode;
	apicmdbuf->cmdCode++;
    switch (cmdCode)
    {
    case APCMD_BSS_START:
        if(systemCfg.prod==AP51)
        {
            SendResetCmd(0);
            apicmdbuf->cmdStatus = APCTL_OK;
        }
        else
        {
            if ((apiCmdQueue = api_GetCmdBuf()) != NULL)
            {
                apiCmdQueue->MsgHdr.Msg = APCMD_BSS_START;
                if (api_CmdWriteNoBlock(apiCmdQueue) == FAIL)
                {
                    apicmdbuf->cmdStatus = APCTL_ERROR;
                    return FAIL;
                }
                else
                {
                    // 1 tick = 100us, so task delay for about  100ms
                    //Wait for task to finish
#if 1				
//move this delay to smeTask when reading APCMD_BSS_START to let api reply 
//get a chance to sent out when in remote config mode.
                    os_TaskDelay(200);
#endif
                    apicmdbuf->cmdStatus = APCTL_OK;
                }
            }
            else
            {
                apicmdbuf->cmdStatus = APCTL_ERROR;
                return OS_FAIL;
            }
        }
        break;
    case APCMD_GET_SSID:
        if (apicmdbuf->cmdBodyLen < IEEEtypes_SSID_SIZE)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = IEEEtypes_SSID_SIZE;
            return OS_FAIL;
        }
        else
        {
#ifdef APCFGUR
			apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
            util_CopyList(apicmdbuf->cmdBody, mib_StaCfg_p->DesiredSsId, IEEEtypes_SSID_SIZE);
            apicmdbuf->cmdBody[IEEEtypes_SSID_SIZE] = '\0';
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_SSID:
        if (apicmdbuf->cmdBodyLen < IEEEtypes_SSID_SIZE)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = IEEEtypes_SSID_SIZE;
            return OS_FAIL;
        }
        else
        {
#ifdef APCFGUR
			apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
            util_CopyList( mib_StaCfg_p->DesiredSsId, apicmdbuf->cmdBody, IEEEtypes_SSID_SIZE);
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;
	 case APCMD_GET_SSID2:
        if (apicmdbuf->cmdBodyLen < IEEEtypes_SSID_SIZE)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = IEEEtypes_SSID_SIZE;
            return OS_FAIL;
        }
        else
        {
#ifdef APCFGUR
			apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
            util_CopyList(apicmdbuf->cmdBody, mib_StaCfg_p->DesiredSsId2, IEEEtypes_SSID_SIZE);
            apicmdbuf->cmdBody[IEEEtypes_SSID_SIZE] = '\0';
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_SSID2:
        if (apicmdbuf->cmdBodyLen < IEEEtypes_SSID_SIZE)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = IEEEtypes_SSID_SIZE;
            return OS_FAIL;
        }
        else
        {
#ifdef APCFGUR
			apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
            util_CopyList( mib_StaCfg_p->DesiredSsId2, apicmdbuf->cmdBody, IEEEtypes_SSID_SIZE);
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_GET_OPEN_SSID_EN_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_OPEN_SSID_EN_STATE* apicmdbuf_get_open_ssid_en_state;
            apicmdbuf_get_open_ssid_en_state = (APICMDBUF_GET_OPEN_SSID_EN_STATE *)apicmdbuf;
            apicmdbuf_get_open_ssid_en_state->cmdCode = APCMD_GET_OPEN_SSID_EN_STATE_REPLY;
            apicmdbuf_get_open_ssid_en_state->openSSIDEnabled = mib_broadcastssid;
            apicmdbuf_get_open_ssid_en_state->cmdStatus = APCTL_OK;
        }

        break;



    case APCMD_SET_OPEN_SSID_EN_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_OPEN_SSID_EN_STATE* apicmdbuf_set_open_ssid_en_state;
            apicmdbuf_set_open_ssid_en_state = (APICMDBUF_SET_OPEN_SSID_EN_STATE *)apicmdbuf;
            apicmdbuf_set_open_ssid_en_state->cmdCode = APCMD_SET_OPEN_SSID_EN_STATE_REPLY;
            mib_broadcastssid = apicmdbuf_set_open_ssid_en_state->openSSIDEnabled;
            apicmdbuf_set_open_ssid_en_state->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_GET_RF_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_RF_MACADDR* apicmdbuf_get_rf_macaddr;
            apicmdbuf_get_rf_macaddr = (APICMDBUF_GET_RF_MACADDR *)apicmdbuf;
            apicmdbuf_get_rf_macaddr->cmdCode = APCMD_GET_RF_MAC_ADDRESS_REPLY;
            memcpy(apicmdbuf_get_rf_macaddr->physAddr, mib_OpData_p->StaMacAddr, 6);
            apicmdbuf_get_rf_macaddr->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_RF_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_RF_MACADDR* apicmdbuf_set_rf_macaddr;
            apicmdbuf_set_rf_macaddr = (APICMDBUF_SET_RF_MACADDR *)apicmdbuf;
            apicmdbuf_set_rf_macaddr->cmdCode = APCMD_SET_RF_MAC_ADDRESS_REPLY;
            memcpy(mib_OpData_p->StaMacAddr, apicmdbuf_set_rf_macaddr->physAddr, 6);
            //	memcpy(enaddr,apicmdbuf_set_rf_macaddr->physAddr,6);
            apicmdbuf_set_rf_macaddr->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_GET_RF_CHANNEL:
        if (apicmdbuf->cmdBodyLen < 18)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 18;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_RF_CHANNEL *apicmdbuf_get_rf_channel;
            apicmdbuf_get_rf_channel = (APICMDBUF_GET_RF_CHANNEL *)apicmdbuf;
            apicmdbuf_get_rf_channel->cmdCode = APCMD_GET_RF_CHANNEL_REPLY;
            apicmdbuf_get_rf_channel->rfModulationType = 2;
            apicmdbuf_get_rf_channel->curChanNum = PhyDSSSTable_p->CurrChan;
#ifdef DOMAIN_SUPPORTED
			domainGetInfo(apicmdbuf_get_rf_channel->availChanList);
#ifndef HARRIER
			memset(&(apicmdbuf_get_rf_channel->availChanList[IEEEtypes_MAX_CHANNELS]), 0, IEEEtypes_MAX_CHANNELS_A);
#endif
#endif
            apicmdbuf_get_rf_channel->cmdStatus = APCTL_OK;
#ifdef AUTOCHANNEL
			if(PhyDSSSTable_p->CurrChan ==0)
			{
            	UINT8 mode;
				if(mib_ApMode>=AP_MODE_A_ONLY)
					mode = 1;
				else
					mode = 0;
            	apicmdbuf_get_rf_channel->isAutoChan = 1;
             	apicmdbuf_get_rf_channel->curChanNum = channelSelected(mode);
			}
			else
            	apicmdbuf_get_rf_channel->isAutoChan = 0;
#endif
        }

        break;

    case APCMD_GET_RF_CHANNEL2:
        if (apicmdbuf->cmdBodyLen < 18)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 18;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_RF_CHANNEL *apicmdbuf_get_rf_channel;
            apicmdbuf_get_rf_channel = (APICMDBUF_GET_RF_CHANNEL *)apicmdbuf;
            apicmdbuf_get_rf_channel->cmdCode = APCMD_GET_RF_CHANNEL2_REPLY;
            apicmdbuf_get_rf_channel->rfModulationType = 2;
            apicmdbuf_get_rf_channel->curChanNum = PhyDSSSTable_p->CurrChan2;
#ifdef DOMAIN_SUPPORTED
			domainGetInfo(apicmdbuf_get_rf_channel->availChanList);
#ifndef HARRIER
			memset(&(apicmdbuf_get_rf_channel->availChanList[IEEEtypes_MAX_CHANNELS]), 0, IEEEtypes_MAX_CHANNELS_A);
#endif
#endif
            apicmdbuf_get_rf_channel->cmdStatus = APCTL_OK;
#ifdef AUTOCHANNEL
            apicmdbuf_get_rf_channel->isAutoChan = 0;
			if(PhyDSSSTable_p->CurrChan2 ==0)
			{
				if(mib_ApMode==AP_MODE_AandG)
				{
            		apicmdbuf_get_rf_channel->isAutoChan = 1;
             		apicmdbuf_get_rf_channel->curChanNum = channelSelected(0);
				}
			}
#endif
        }

        break;

    case APCMD_SET_RF_CHANNEL:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_RF_CHANNEL* apicmdbuf_set_rf_channel;
            apicmdbuf_set_rf_channel = (APICMDBUF_SET_RF_CHANNEL *)apicmdbuf;
            apicmdbuf_set_rf_channel->cmdCode = APCMD_SET_RF_CHANNEL_REPLY;
            PhyDSSSTable_p->CurrChan = apicmdbuf_set_rf_channel->curChanNum;
            apicmdbuf_set_rf_channel->cmdStatus = APCTL_OK;
        }
#ifdef AUTOCHANNEL
        if(PhyDSSSTable_p->CurrChan ==0 )
        {
#ifdef AP_URPTR
            if (mib_urMode != 1)
#endif
			    SetupScan();
        }
#endif
        break;

    case APCMD_SET_RF_CHANNEL2:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_RF_CHANNEL* apicmdbuf_set_rf_channel;
            apicmdbuf_set_rf_channel = (APICMDBUF_SET_RF_CHANNEL *)apicmdbuf;
            apicmdbuf_set_rf_channel->cmdCode = APCMD_SET_RF_CHANNEL2_REPLY;
            PhyDSSSTable_p->CurrChan2 = apicmdbuf_set_rf_channel->curChanNum;
            apicmdbuf_set_rf_channel->cmdStatus = APCTL_OK;
        }
#ifdef AUTOCHANNEL
        if(PhyDSSSTable_p->CurrChan2 ==0 )
        {
#ifdef AP_URPTR
            if (mib_urMode != 1)
#endif
			    SetupScan();
        }
#endif
        break;
#ifndef CONDOR
    case APCMD_GET_BASIC_DATA_RATES:
        if (apicmdbuf->cmdBodyLen < 8)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_BASIC_DATA_RATES * apicmdbuf_get_basic_data_rates;
            apicmdbuf_get_basic_data_rates = (APICMDBUF_GET_BASIC_DATA_RATES *)apicmdbuf;
            apicmdbuf_get_basic_data_rates->cmdCode = APCMD_GET_BASIC_DATA_RATES;
            apicmdbuf_get_basic_data_rates->basicDataRates[0] = mib_StaCfg_p->OpRateSet[0];
            apicmdbuf_get_basic_data_rates->basicDataRates[1] = mib_StaCfg_p->OpRateSet[1];
            apicmdbuf_get_basic_data_rates->basicDataRates[2] = mib_StaCfg_p->OpRateSet[2];
            apicmdbuf_get_basic_data_rates->basicDataRates[3] = mib_StaCfg_p->OpRateSet[3];
            apicmdbuf_get_basic_data_rates->basicDataRates[4] = mib_StaCfg_p->OpRateSet[4];
            apicmdbuf_get_basic_data_rates->basicDataRates[5] = mib_StaCfg_p->OpRateSet[5];
            apicmdbuf_get_basic_data_rates->basicDataRates[6] = mib_StaCfg_p->OpRateSet[6];
            apicmdbuf_get_basic_data_rates->basicDataRates[7] = mib_StaCfg_p->OpRateSet[7];

            apicmdbuf_get_basic_data_rates->cmdStatus = APCTL_OK;

        }

        break;
    case APCMD_SET_BASIC_DATA_RATES:
        if (apicmdbuf->cmdBodyLen < 8)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_SET_BASIC_DATA_RATES * apicmdbuf_set_basic_data_rates;
            apicmdbuf_set_basic_data_rates = (APICMDBUF_SET_BASIC_DATA_RATES *)apicmdbuf;

            apicmdbuf_set_basic_data_rates->cmdCode = APCMD_SET_BASIC_DATA_RATES_REPLY;
            if (apicmdbuf_set_basic_data_rates->basicDataRates[0] != 0x82)
                //mib_StaCfg_p->OpRateSet[0]=0x82;  //Mandatory rate
                return FAIL;
            if (apicmdbuf_set_basic_data_rates->basicDataRates[1] != 0x84)
                return FAIL; //Mandatory rate
            //mib_StaCfg_p->OpRateSet[1]=0x84;
#if 0

            if (apicmdbuf_set_basic_data_rates->basicDataRates[2] == 0x0B)
                mib_StaCfg_p->OpRateSet[2] = 0x8B;
            else
                mib_StaCfg_p->OpRateSet[2] = apicmdbuf_set_basic_data_rates->basicDataRates[2];
            if (apicmdbuf_set_basic_data_rates->basicDataRates[3] == 0x16)
                mib_StaCfg_p->OpRateSet[3] = 0x96;
            else
                mib_StaCfg_p->OpRateSet[3] = apicmdbuf_set_basic_data_rates->basicDataRates[3];
#else

            mib_StaCfg_p->OpRateSet[2] = apicmdbuf_set_basic_data_rates->basicDataRates[2];
            mib_StaCfg_p->OpRateSet[3] = apicmdbuf_set_basic_data_rates->basicDataRates[3];
#endif
#ifdef CONDOR

            apicmdbuf_set_basic_data_rates->cmdStatus = APCTL_ERROR;
#else

            apicmdbuf_set_basic_data_rates->cmdStatus = APCTL_OK;
#endif


        }

        break;
    case APCMD_GET_SUP_DATA_RATES:
        if (apicmdbuf->cmdBodyLen < 8)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {

            APICMDBUF_GET_SUP_DATA_RATE * apicmdbuf_get_sup_data_rates;
            apicmdbuf_get_sup_data_rates = (APICMDBUF_GET_SUP_DATA_RATE *)apicmdbuf;
            apicmdbuf_get_sup_data_rates->cmdCode = APCMD_GET_SUP_DATA_RATES_REPLY;

            apicmdbuf_get_sup_data_rates->txDataRate[0] = mib_StaCfg_p->OpRateSet[0];
            apicmdbuf_get_sup_data_rates->txDataRate[1] = mib_StaCfg_p->OpRateSet[1];
            apicmdbuf_get_sup_data_rates->txDataRate[2] = mib_StaCfg_p->OpRateSet[2];
            apicmdbuf_get_sup_data_rates->txDataRate[3] = mib_StaCfg_p->OpRateSet[3];
            apicmdbuf_get_sup_data_rates->txDataRate[4] = mib_StaCfg_p->OpRateSet[4];
            apicmdbuf_get_sup_data_rates->txDataRate[5] = mib_StaCfg_p->OpRateSet[5];
            apicmdbuf_get_sup_data_rates->txDataRate[6] = mib_StaCfg_p->OpRateSet[6];
            apicmdbuf_get_sup_data_rates->txDataRate[7] = mib_StaCfg_p->OpRateSet[7];
            apicmdbuf_get_sup_data_rates->cmdStatus = APCTL_OK;

        }
        break;
    case APCMD_SET_SUP_DATA_RATES:
        if (apicmdbuf->cmdBodyLen < 8)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {


            APICMDBUF_SET_SUP_DATA_RATE * apicmdbuf_set_sup_data_rates;

            apicmdbuf_set_sup_data_rates = (APICMDBUF_SET_SUP_DATA_RATE *)apicmdbuf;

            apicmdbuf_set_sup_data_rates->cmdCode = APCMD_SET_SUP_DATA_RATES_REPLY;
#if 0 //Currently not supported

            mib_StaCfg_p->OpRateSet[0] = apicmdbuf_set_sup_data_rates->txDataRate[0];
            mib_StaCfg_p->OpRateSet[1] = apicmdbuf_set_sup_data_rates->txDataRate[1];
            mib_StaCfg_p->OpRateSet[2] = apicmdbuf_set_sup_data_rates->txDataRate[2];
            mib_StaCfg_p->OpRateSet[3] = apicmdbuf_set_sup_data_rates->txDataRate[3];
            mib_StaCfg_p->OpRateSet[4] = apicmdbuf_set_sup_data_rates->txDataRate[4];
            mib_StaCfg_p->OpRateSet[5] = apicmdbuf_set_sup_data_rates->txDataRate[5];
            mib_StaCfg_p->OpRateSet[6] = apicmdbuf_set_sup_data_rates->txDataRate[6];
            mib_StaCfg_p->OpRateSet[7] = apicmdbuf_set_sup_data_rates->txDataRate[7];
#endif



            apicmdbuf_set_sup_data_rates->cmdStatus = APCTL_OK;
        }
        break;
#endif
    case APCMD_GET_BASIC_DATA_RATES_G:
        if (apicmdbuf->cmdBodyLen < 16)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {
            int i;
            APICMDBUF_GET_BASIC_DATA_RATES_G * apicmdbuf_get_basic_data_rates;
            apicmdbuf_get_basic_data_rates = (APICMDBUF_GET_BASIC_DATA_RATES_G *)apicmdbuf;
            apicmdbuf_get_basic_data_rates->cmdCode = APCMD_GET_BASIC_DATA_RATES_G;
            for ( i = 0; i < 16; i++)
            {
                apicmdbuf_get_basic_data_rates->basicDataRates[i] =
                    mib_StaCfg_p->OpRateSet[i];
            }
            apicmdbuf_get_basic_data_rates->cmdStatus = APCTL_OK;

        }

        break;
    case APCMD_SET_BASIC_DATA_RATES_G:
        if (apicmdbuf->cmdBodyLen < 16)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_SET_BASIC_DATA_RATES_G * apicmdbuf_set_basic_data_rates;
            int i;
            apicmdbuf_set_basic_data_rates = (APICMDBUF_SET_BASIC_DATA_RATES_G *)apicmdbuf;
            apicmdbuf_set_basic_data_rates->cmdCode = APCMD_SET_BASIC_DATA_RATES_G_REPLY;
            for (i = 0;i < 16; i++)
            {
                mib_StaCfg_p->OpRateSet[i] = apicmdbuf_set_basic_data_rates->basicDataRates[i];
            }



		mib_StaCfg_p->OpRateSet2[0] = 0x82;
       	mib_StaCfg_p->OpRateSet2[1] = 0x84;
		mib_StaCfg_p->OpRateSet2[2] = 0x8b;
		mib_StaCfg_p->OpRateSet2[3] = 0x96;
		mib_StaCfg_p->OpRateSet2[4] = 12 | 0x80;
		mib_StaCfg_p->OpRateSet2[5] = 18;
		mib_StaCfg_p->OpRateSet2[6] =24 | 0x80;
		mib_StaCfg_p->OpRateSet2[7] = 36;
		mib_StaCfg_p->OpRateSet2[8] = 48 | 0x80;
		mib_StaCfg_p->OpRateSet2[9] = 72;
	       mib_StaCfg_p->OpRateSet2[10] =  96;
		mib_StaCfg_p->OpRateSet2[11] = 108;
		mib_StaCfg_p->OpRateSet2[12] = 0;






            apicmdbuf_set_basic_data_rates->cmdStatus = APCTL_OK;
        }

        break;
#if 0
    case APCMD_GET_TX_POWER_LEVEL:
        if (apicmdbuf->cmdBodyLen < 9)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 9;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_TX_POWER_LEVEL * apicmdbuf_get_tx_power_level;
            apicmdbuf_get_tx_power_level = (APICMDBUF_GET_TX_POWER_LEVEL *)apicmdbuf;
            apicmdbuf_get_tx_power_level->cmdCode = APCMD_GET_TX_POWER_LEVEL_REPLY;
            apicmdbuf_get_tx_power_level->txPowerLevel = mib_PhyTxPwrData_p->CurrTxPwrLevel;
            apicmdbuf_get_tx_power_level->powerLevelList[0] = mib_PhyTxPwrData_p->TxPwrLevel1;
            apicmdbuf_get_tx_power_level->powerLevelList[1] = mib_PhyTxPwrData_p->TxPwrLevel2;
            apicmdbuf_get_tx_power_level->powerLevelList[2] = mib_PhyTxPwrData_p->TxPwrLevel3;
            apicmdbuf_get_tx_power_level->powerLevelList[3] = mib_PhyTxPwrData_p->TxPwrLevel4;
            apicmdbuf_get_tx_power_level->powerLevelList[4] = mib_PhyTxPwrData_p->TxPwrLevel5;
            apicmdbuf_get_tx_power_level->powerLevelList[5] = mib_PhyTxPwrData_p->TxPwrLevel6;
            apicmdbuf_get_tx_power_level->powerLevelList[6] = mib_PhyTxPwrData_p->TxPwrLevel7;
            apicmdbuf_get_tx_power_level->powerLevelList[7] = mib_PhyTxPwrData_p->TxPwrLevel8;
            apicmdbuf_get_tx_power_level->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_TX_POWER_LEVEL:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_TX_POWER_LEVEL * apicmdbuf_set_tx_power_level;
            apicmdbuf_set_tx_power_level = (APICMDBUF_SET_TX_POWER_LEVEL *)apicmdbuf;
            apicmdbuf_set_tx_power_level->cmdCode = APCMD_SET_TX_POWER_LEVEL_REPLY;
            mib_PhyTxPwrData_p->CurrTxPwrLevel = apicmdbuf_set_tx_power_level->txPowerLevel;
            apicmdbuf_set_tx_power_level->cmdStatus = APCTL_OK;
        }
        break;
#endif
    case APCMD_GET_RF_PREAMBLE_OPTION:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMBUF_GET_RF_PREAMBLE_OPTION * apicmdbuf_get_rf_preamble_option;
            apicmdbuf_get_rf_preamble_option = (APICMBUF_GET_RF_PREAMBLE_OPTION *)apicmdbuf;
            apicmdbuf_get_rf_preamble_option->cmdCode = APCMD_GET_RF_PREAMBLE_OPTION_REPLY;
            apicmdbuf_get_rf_preamble_option->rfPreambleOption = mib_preAmble;
            apicmdbuf_get_rf_preamble_option->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_RF_PREAMBLE_OPTION:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMBUF_SET_RF_PREAMBLE_OPTION * apicmdbuf_set_rf_preamble_option;
            apicmdbuf_set_rf_preamble_option = (APICMBUF_SET_RF_PREAMBLE_OPTION *)apicmdbuf;
            apicmdbuf_set_rf_preamble_option->cmdCode = APCMD_SET_RF_PREAMBLE_OPTION_REPLY;
            mib_preAmble = apicmdbuf_set_rf_preamble_option->rfPreambleOption;
            apicmdbuf_set_rf_preamble_option->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_ANTENNA_MODE:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_ANTENNA_MODE* apicmdbuf_get_antenna_mode;
            apicmdbuf_get_antenna_mode = (APICMDBUF_GET_ANTENNA_MODE *)apicmdbuf;
            apicmdbuf_get_antenna_mode->cmdCode = APCMD_GET_ANTENNA_MODE_REPLY;
#ifndef PEREGRINE                        
            apicmdbuf_get_antenna_mode->rxAntennaMode = mib_rxAntenna;
            apicmdbuf_get_antenna_mode->txAntennaMode = mib_txAntenna;
#else
            if(mib_rxAntenna == AP_ANT_A)
            {
                apicmdbuf_get_antenna_mode->rxAntennaMode = AP_ANT_B;
            }
            else
            {
                apicmdbuf_get_antenna_mode->rxAntennaMode = AP_ANT_A;                
            }

            if(mib_txAntenna == AP_ANT_A)
            {
                apicmdbuf_get_antenna_mode->txAntennaMode = AP_ANT_B;
            }
            else
            {
                apicmdbuf_get_antenna_mode->txAntennaMode = AP_ANT_A;                
            }
            
#endif
            if(mib_AntDiversity)
                apicmdbuf_get_antenna_mode->whichAnt = AP_ANT_DIVERSITY;
            else
#ifndef PEREGRINE                
                apicmdbuf_get_antenna_mode->whichAnt = mib_txAntenna;
#else
            {
                if(mib_txAntenna == AP_ANT_A)
                {
                    apicmdbuf_get_antenna_mode->whichAnt = AP_ANT_B;
                }
                else
                {
                    apicmdbuf_get_antenna_mode->whichAnt = AP_ANT_A;                
                }
            }
#endif
            apicmdbuf_get_antenna_mode->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_ANTENNA_MODE:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_ANTENNA_MODE* apicmdbuf_set_antenna_mode;
            apicmdbuf_set_antenna_mode = (APICMDBUF_SET_ANTENNA_MODE *)apicmdbuf;
            apicmdbuf_set_antenna_mode->cmdCode = APCMD_SET_ANTENNA_MODE_REPLY;
            if (apicmdbuf_set_antenna_mode->whichAnt == AP_RX_ANTENNA)
            {
#ifndef PEREGRINE                
                mib_rxAntenna = apicmdbuf_set_antenna_mode->antennaMode;
#else
                if(apicmdbuf_set_antenna_mode->antennaMode == AP_ANT_A)
                {
                    mib_rxAntenna = AP_ANT_B;
                }
                else
                {
                    mib_rxAntenna = AP_ANT_A;                            
                }
                
#endif
            }
            else if(apicmdbuf_set_antenna_mode->whichAnt == AP_TX_ANTENNA)
            {
#ifndef PEREGRINE                
                mib_txAntenna = apicmdbuf_set_antenna_mode->antennaMode;
#else
                if(apicmdbuf_set_antenna_mode->antennaMode == AP_ANT_A)
                {
                    mib_txAntenna = AP_ANT_B;
                }
                else
                {
                    mib_txAntenna = AP_ANT_A;                            
                }
                
#endif
                
            }
            else if(apicmdbuf_set_antenna_mode->whichAnt == AP_BOTH_ANTENNA)
            {
                if(apicmdbuf_set_antenna_mode->antennaMode == AP_ANT_DIVERSITY)
                {
                        mib_AntDiversity = 1;
                }
                else
                {
                    mib_AntDiversity = 0;
#ifndef PEREGRINE                    
                    if(mib_txAntenna != apicmdbuf_set_antenna_mode->antennaMode)
#else
                    if(mib_txAntenna == apicmdbuf_set_antenna_mode->antennaMode)
#endif
                    {
#ifndef PEREGRINE                        
                       mib_txAntenna = apicmdbuf_set_antenna_mode->antennaMode;
                       mib_rxAntenna = apicmdbuf_set_antenna_mode->antennaMode;
#else
                        if(apicmdbuf_set_antenna_mode->antennaMode == AP_ANT_A)
                        {
                            mib_txAntenna = AP_ANT_B;
                            mib_rxAntenna = AP_ANT_B;
                        }
                        else
                        {
                            mib_txAntenna = AP_ANT_A;
                            mib_rxAntenna = AP_ANT_A;                            
                        }
                          
#endif
                    }
                }
            }
            apicmdbuf_set_antenna_mode->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_GET_FRAG_THRESHOLD:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_FRAG_THRESHOLD* apicmdbuf_get_frag_threshold;
            apicmdbuf_get_frag_threshold = (APICMDBUF_GET_FRAG_THRESHOLD *)apicmdbuf;
            apicmdbuf_get_frag_threshold->cmdCode = APCMD_GET_FRAG_THRESHOLD_REPLY;
            apicmdbuf_get_frag_threshold->fragThreshold = mib_OpData_p->FragThresh;
            apicmdbuf_get_frag_threshold->cmdStatus = APCTL_OK;

        }
        break;

    case APCMD_SET_FRAG_THRESHOLD:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_FRAG_THRESHOLD* apicmdbuf_set_frag_threshold;
            apicmdbuf_set_frag_threshold = (APICMDBUF_SET_FRAG_THRESHOLD *)apicmdbuf;
            apicmdbuf_set_frag_threshold->cmdCode = APCMD_SET_FRAG_THRESHOLD_REPLY;
            if(apicmdbuf_set_frag_threshold->fragThreshold < 256)
            {
                mib_OpData_p->FragThresh = 256;
            }
            else if (apicmdbuf_set_frag_threshold->fragThreshold > 2346)
            {
                mib_OpData_p->FragThresh = 2346;
            }
            else
            {
                mib_OpData_p->FragThresh = apicmdbuf_set_frag_threshold->fragThreshold;
            }
            apicmdbuf_set_frag_threshold->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_RTS_THRESHOLD:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_RTS_THRESHOLD* apicmdbuf_get_rts_threshold;
            apicmdbuf_get_rts_threshold = (APICMDBUF_GET_RTS_THRESHOLD *)apicmdbuf;
            apicmdbuf_get_rts_threshold->cmdCode = APCMD_GET_RTS_THRESHOLD_REPLY;
            apicmdbuf_get_rts_threshold->rtsThreshold = mib_OpData_p->RtsThresh;
            apicmdbuf_get_rts_threshold->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_RTS_THRESHOLD:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_RTS_THRESHOLD* apicmdbuf_set_rts_threshold;
            apicmdbuf_set_rts_threshold = (APICMDBUF_SET_RTS_THRESHOLD *)apicmdbuf;
            apicmdbuf_set_rts_threshold->cmdCode = APCMD_SET_RTS_THRESHOLD_REPLY;
            if(apicmdbuf_set_rts_threshold->rtsThreshold == 0)
            {
                mib_OpData_p->RtsThresh = 0;
            }
            else if (apicmdbuf_set_rts_threshold->rtsThreshold > 2432)
            {
                mib_OpData_p->RtsThresh =  2432;
            }
            else
            {
                mib_OpData_p->RtsThresh = apicmdbuf_set_rts_threshold->rtsThreshold;
            }
            apicmdbuf_set_rts_threshold->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_RTS_RETRY_LIMIT:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_RTS_RETRY_LIMIT * apicmdbuf_get_rts_retry_limit;
            apicmdbuf_get_rts_retry_limit = (APICMDBUF_GET_RTS_RETRY_LIMIT *)apicmdbuf;
            apicmdbuf_get_rts_retry_limit->cmdCode = APCMD_GET_RTS_RETRY_LIMIT_REPLY;
            apicmdbuf_get_rts_retry_limit->rtsRetryLimit = mib_OpData_p->ShortRetryLim;
            apicmdbuf_get_rts_retry_limit->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_RTS_RETRY_LIMIT:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_RTS_RETRY_LIMIT * apicmdbuf_set_rts_retry_limit;
            apicmdbuf_set_rts_retry_limit = (APICMDBUF_SET_RTS_RETRY_LIMIT *)apicmdbuf;
            apicmdbuf_set_rts_retry_limit->cmdCode = APCMD_SET_RTS_THRESHOLD_REPLY;
            mib_OpData_p->ShortRetryLim = apicmdbuf_set_rts_retry_limit->rtsRetryLimit;
            apicmdbuf_set_rts_retry_limit->cmdStatus = APCTL_OK;
        }
        break;
#if 0
    case APCMD_GET_DATA_RETRY_LIMIT:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            apicmdbuf->cmdStatus = APCTL_OK;
            //Todo (UINT16 *)apicmdbuf->cmdBody =  mib_OpData_p.;

        }
        break;

    case APCMD_SET_DATA_RETRY_LIMIT:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            apicmdbuf->cmdStatus = APCTL_OK;
            /*Todo mib_StaCfg_p->BcnPeriod = (UINT16 *)apicmdbuf->cmdBody; */
        }
        break;
#endif
    case APCMD_GET_BEACON_PERIOD:
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_BEACON_PERIOD* apicmdbuf_get_beacon_period;
            apicmdbuf_get_beacon_period = (APICMDBUF_GET_BEACON_PERIOD *)apicmdbuf;
            apicmdbuf_get_beacon_period->cmdCode = APCMD_GET_BEACON_PERIOD_REPLY;
            apicmdbuf_get_beacon_period->beaconPeriod = mib_StaCfg_p->BcnPeriod;
            apicmdbuf_get_beacon_period->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_BEACON_PERIOD:
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_BEACON_PERIOD* apicmdbuf_set_beacon_period;
            apicmdbuf_set_beacon_period = (APICMDBUF_SET_BEACON_PERIOD *)apicmdbuf;
            apicmdbuf_set_beacon_period->cmdCode = APCMD_SET_BEACON_PERIOD_REPLY;
            if(apicmdbuf_set_beacon_period->beaconPeriod < 1)
            {
                mib_StaCfg_p->BcnPeriod = 1;
            }
            else if (apicmdbuf_set_beacon_period->beaconPeriod > 65535)
            {
                mib_StaCfg_p->BcnPeriod = 65535;
            }
            else
            {
                mib_StaCfg_p->BcnPeriod = apicmdbuf_set_beacon_period->beaconPeriod;
            }

            apicmdbuf_set_beacon_period->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_DTIM_PERIOD:
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_DTIM_PERIOD* apicmdbuf_get_dtim_period;
            apicmdbuf_get_dtim_period = (APICMDBUF_GET_DTIM_PERIOD *)apicmdbuf;
            apicmdbuf_get_dtim_period->cmdCode = APCMD_GET_DTIM_PERIOD_REPLY;
            apicmdbuf_get_dtim_period->dtimPeriod = mib_StaCfg_p->DtimPeriod;
            apicmdbuf_get_dtim_period->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_DTIM_PERIOD:
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_DTIM_PERIOD* apicmdbuf_set_dtim_period;
            apicmdbuf_set_dtim_period = (APICMDBUF_SET_DTIM_PERIOD *)apicmdbuf;
            apicmdbuf_set_dtim_period->cmdCode = APCMD_SET_DTIM_PERIOD_REPLY;
            if( apicmdbuf_set_dtim_period->dtimPeriod <= 0)
                mib_StaCfg_p->DtimPeriod = 1;
            else if(apicmdbuf_set_dtim_period->dtimPeriod > 255)
                mib_StaCfg_p->DtimPeriod = 255;
            else
                mib_StaCfg_p->DtimPeriod = apicmdbuf_set_dtim_period->dtimPeriod;
            

            apicmdbuf_set_dtim_period->cmdStatus = APCTL_OK;
        }
        break;
#if 0
    case APCMD_GET_802_11_STATS:
        if (apicmdbuf->cmdBodyLen < 60)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 60;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_802_11_STATS* apicmdbuf_get_802_11_stats;
            apicmdbuf_get_802_11_stats = (APICMDBUF_GET_802_11_STATS *)apicmdbuf;
            apicmdbuf_get_802_11_stats->cmdCode = APCMD_GET_802_11_STATS_REPLY;
//            apicmdbuf_get_802_11_stats->rxFrmCnt = Stats.DataRecvCnt;
 //           apicmdbuf_get_802_11_stats->txFrmCnt = Stats.DataTxSuccessCnt;
            Stats.DataRecvCnt = 0;
            Stats.DataTxSuccessCnt = 0;
    //        apicmdbuf_get_802_11_stats->totaltxBytes = Stats.WlTxDataBytes;
          //  Stats.WlTxDataBytes = 0;
            apicmdbuf_get_802_11_stats->totalRxBytes = (*(volatile unsigned long *)(0x8000a220));
            /*todo */
            apicmdbuf_get_802_11_stats->cntMcastPackets = 0;
            apicmdbuf_get_802_11_stats->cntUcastPackets = 0; /* count of uniticastpackets */
            apicmdbuf_get_802_11_stats->cntTxBad = 0;
            apicmdbuf_get_802_11_stats->cntRetry = 0;
            apicmdbuf_get_802_11_stats->rtsBadCnt = 0;
            apicmdbuf_get_802_11_stats->ackBadCnt = 0;
            apicmdbuf_get_802_11_stats->rxMcastOkCnt = 0; /*mib_Counters. */
            apicmdbuf_get_802_11_stats->fcsErrorCnt = 0;
            apicmdbuf_get_802_11_stats->duplicateCnt = 0; /*todo */
            apicmdbuf_get_802_11_stats->wepDecryptBadCnt = 0; /*todo */
            apicmdbuf_get_802_11_stats->cmdStatus = APCTL_OK;

        }
        break;
#endif
    case APCMD_ADD_WLAN_FILTER_ENTRY:
        {
            if (apicmdbuf->cmdBodyLen < 6)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 6;
                apicmdbuf->cmdCode = APCMD_ADD_WLAN_FILTER_ENTRY_REPLY;
                return OS_FAIL;
            }
            else
            {
                APICMDBUF_FILTER_ENTRY* apicmdbuf_filter_entry;
                apicmdbuf_filter_entry =(APICMDBUF_FILTER_ENTRY*)apicmdbuf;
#ifdef APCFGUR
                SendCtrlMsgTo(apicmdbuf, apicmdbuf->cmdBodyLen + APICMDHEADERLEN);
#endif
                memcpy(mib_wlanfiltermac + mib_wlanfilterno*6, apicmdbuf_filter_entry->macAddr, 6);
                mib_wlanfilterno++;
                apicmdbuf->cmdCode = APCMD_ADD_WLAN_FILTER_ENTRY_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
                if(mib_wlanfiltertype== FILTER_MODE_BLOCK)
                {
                    extStaDb_StaInfo_t *StaInfo_p;
                    if ((StaInfo_p = extStaDb_GetStaInfo(&apicmdbuf_filter_entry->macAddr,  0)))
                    {
                        StaInfo_p->TimeStamp = 1;
                        extStaDb_ProcessAgeEvt();
                    }
                }
            }
        }
        break;
    case APCMD_DEL_WLAN_FILTER_ENTRY:
        {
            if (apicmdbuf->cmdBodyLen < 6)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 6;
                apicmdbuf->cmdCode = APCMD_DEL_WLAN_FILTER_ENTRY_REPLY;
                return OS_FAIL;
            }
            else
            {
                APICMDBUF_FILTER_ENTRY* apicmdbuf_filter_entry;
                int i;
                apicmdbuf_filter_entry =(APICMDBUF_FILTER_ENTRY*)apicmdbuf;
#ifdef APCFGUR
                SendCtrlMsgTo(apicmdbuf, apicmdbuf->cmdBodyLen + APICMDHEADERLEN);
#endif
                for (i = 0; i < mib_wlanfilterno; i++)
                {
                    if(!memcmp(mib_wlanfiltermac + i*6, apicmdbuf_filter_entry->macAddr, 6))
                    {
                        mib_wlanfilterno--;
                        memcpy(mib_wlanfiltermac + i*6, mib_wlanfiltermac + i*6 + 6, (mib_wlanfilterno-i)*6);
                        if(mib_wlanfiltertype== FILTER_MODE_ALLOW)
                        {
                            extStaDb_StaInfo_t *StaInfo_p;
                            if ((StaInfo_p = extStaDb_GetStaInfo(&apicmdbuf_filter_entry->macAddr,  0)))
                            {
                                StaInfo_p->TimeStamp = 1;
                                extStaDb_ProcessAgeEvt();
                            }
                        }
                    }
                }
                apicmdbuf->cmdCode = APCMD_DEL_WLAN_FILTER_ENTRY_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_STA_LIST:
        {
            UINT16 nostation = extStaDb_entries(0);
            UINT16 minbodysize = 2 + extStaDb_entries(0) * sizeof(STA_INFO);
            if (apicmdbuf->cmdBodyLen < minbodysize)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdCode = APCMD_GET_STA_LIST_REPLY;
                apicmdbuf->cmdBodyLen = minbodysize;
                return OS_FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                //Number of associated station
#ifdef STA_INFO_DB
                apicmdbuf->cmdBody[1] = 0; //reserved field
                apicmdbuf->cmdBody[0] = nostation % 0x100;
#else
                apicmdbuf->cmdBody[0] = 0; //reserved field
                apicmdbuf->cmdBody[1] = nostation % 0x100;
#endif                
                extStaDb_list(&apicmdbuf->cmdBody[2], 0);
                apicmdbuf->cmdCode = APCMD_GET_STA_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
#if 0
    case APCMD_GET_STA_INFO:
        /*Todo */
        break;
#endif 
    case APCMD_DEAUTH_STA:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_DEAUTH_STA* apicmdbuf_deauth_sta;
            apicmdbuf_deauth_sta = (APICMDBUF_DEAUTH_STA *)apicmdbuf;
            apicmdbuf_deauth_sta->cmdCode = APCMD_DEAUTH_STA_REPLY;
            SendDeauthCmd((IEEEtypes_MacAddr_t *)apicmdbuf_deauth_sta->staMACAddress);
            apicmdbuf_deauth_sta->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_DISASSOC_STA:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_DISASSOC_STA* apicmdbuf_disassoc_sta;
            apicmdbuf_disassoc_sta = (APICMDBUF_DISASSOC_STA *)apicmdbuf;
            apicmdbuf_disassoc_sta->cmdCode = APCMD_DISASSOC_STA_REPLY;
            SendDeauthCmd((IEEEtypes_MacAddr_t *)apicmdbuf_disassoc_sta->staMACAddress);
            apicmdbuf_disassoc_sta->cmdStatus = APCTL_OK;
        }
        break;
#if 0
    case APCMD_ADD_ASSOC_FILTER_ENTRY:
        /*Todo */
        break;
    case APCMD_DEL_ASSOC_FILTER_ENTRY:
        /*Todo */
        break;
    case APCMD_GET_MAX_NUM_STA:
        /*Todo */
        break;
    case APCMD_SET_MAX_NUM_STA:
        /*Todo */
        break;
#endif
    case APCMD_GET_DEFAULT_WEP_KEYS:
        if (apicmdbuf->cmdBodyLen < 74)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 74;
            return OS_FAIL;
        }
        else
        {
            int weptype = WEP_NOT_SET;
            APICMDBUF_GET_DEFAULT_WEP_KEYS *apicmdbuf_get_default_wep_keys;
            apicmdbuf_get_default_wep_keys = (APICMDBUF_GET_DEFAULT_WEP_KEYS *)apicmdbuf;
            apicmdbuf_get_default_wep_keys->cmdCode = APCMD_GET_DEFAULT_WEP_KEYS_REPLY;
            apicmdbuf_get_default_wep_keys->keyindex = mib_defaultkeyindex;
#ifdef AP5X
            apicmdbuf_get_default_wep_keys->keyType0 = WepType[0];
#else
            if (WepType == 0)
                weptype = WEP_NOT_SET;
            else if (WepType == 1)
                weptype = WEP_40_BIT;
            else if (WepType == 2)
                weptype = WEP_104_BIT;
            apicmdbuf_get_default_wep_keys->keyType0 = weptype;
#endif
            memset(apicmdbuf_get_default_wep_keys->keyValue0, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue0, mib_WepDefaultKeys_p[0].WepDefaultKeyValue, 13);
#ifdef AP5X
            apicmdbuf_get_default_wep_keys->keyType1 = WepType[1];
#else
            apicmdbuf_get_default_wep_keys->keyType1 = weptype;
#endif
            memset(apicmdbuf_get_default_wep_keys->keyValue1, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue1, mib_WepDefaultKeys_p[1].WepDefaultKeyValue, 13);
#ifdef AP5X
            apicmdbuf_get_default_wep_keys->keyType2 = WepType[2];
#else
            apicmdbuf_get_default_wep_keys->keyType2 = weptype;
#endif
            memset(apicmdbuf_get_default_wep_keys->keyValue2, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue2, mib_WepDefaultKeys_p[2].WepDefaultKeyValue, 13);
#ifdef AP5X
            apicmdbuf_get_default_wep_keys->keyType3 = WepType[3];
#else
            apicmdbuf_get_default_wep_keys->keyType3 = weptype;
#endif
            memset(apicmdbuf_get_default_wep_keys->keyValue3, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue3, mib_WepDefaultKeys_p[3].WepDefaultKeyValue, 13);
            apicmdbuf_get_default_wep_keys->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_DEFAULT_WEP_KEYS:
        if (apicmdbuf->cmdBodyLen < 72)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 72;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_DEFAULT_WEP_KEYS *apicmdbuf_set_default_wep_keys;
            apicmdbuf_set_default_wep_keys = (APICMDBUF_SET_DEFAULT_WEP_KEYS *)apicmdbuf;
            apicmdbuf_set_default_wep_keys->cmdCode = APCMD_SET_DEFAULT_WEP_KEYS_REPLY;

            mib_defaultkeyindex = apicmdbuf_set_default_wep_keys->keyindex;

            memcpy(mib_WepDefaultKeys_p[0].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue0, 13);
            memcpy(mib_WepDefaultKeys_p[1].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue1, 13);
            memcpy(mib_WepDefaultKeys_p[2].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue2, 13);
            memcpy(mib_WepDefaultKeys_p[3].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue3, 13);
#ifdef AP5X
            WepType[0] =apicmdbuf_set_default_wep_keys->keyType0;
            WepType[1] =apicmdbuf_set_default_wep_keys->keyType1;
            WepType[2] =apicmdbuf_set_default_wep_keys->keyType2;
            WepType[3] =apicmdbuf_set_default_wep_keys->keyType3;
#else            
            if (apicmdbuf_set_default_wep_keys->keyType0 == WEP_40_BIT)
            {
                WepType = 1;
            }
            else if (apicmdbuf_set_default_wep_keys->keyType0 == WEP_104_BIT)
            {
                WepType = 2;
            }
            else if (apicmdbuf_set_default_wep_keys->keyType0 == WEP_NOT_SET)
            {
                WepType = 0;
            }
#endif
            apicmdbuf_set_default_wep_keys->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_STA_WEP_KEY:
        /*todo */
        break;
        
    case APCMD_GET_PRIVACY_OPTION:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_PRIVACY_OPTION *apicmdbuf_get_privacy_option;
            apicmdbuf_get_privacy_option = (APICMDBUF_GET_PRIVACY_OPTION*)apicmdbuf;
            apicmdbuf_get_privacy_option->cmdCode = APCMD_GET_PRIVACY_OPTION_REPLY;
            apicmdbuf_get_privacy_option->dataEncrypt = mib_PrivacyTable_p->PrivInvoked;
            apicmdbuf_get_privacy_option->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_PRIVACY_OPTION:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_PRIVACY_OPTION *apicmdbuf_set_privacy_option;
            apicmdbuf_set_privacy_option = (APICMDBUF_SET_PRIVACY_OPTION *)apicmdbuf;
            apicmdbuf_set_privacy_option->cmdCode = APCMD_SET_PRIVACY_OPTION_REPLY;
            if (apicmdbuf_set_privacy_option->dataEncrypt == DATA_ENCRYPT_NOT_REQUIRED)
            {
                mib_StaCfg_p->PrivOption = mib_PrivacyTable_p->PrivInvoked = 0;
            }
            else
            {
                mib_StaCfg_p->PrivOption = mib_PrivacyTable_p->PrivInvoked = 1;
            }
            apicmdbuf_set_privacy_option->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_AUTH_MODE:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AUTH_MODE *apicmdbuf_get_auth_mode;
            apicmdbuf_get_auth_mode = (APICMDBUF_GET_AUTH_MODE *)apicmdbuf;
            apicmdbuf_get_auth_mode->cmdCode = APCMD_GET_AUTH_MODE_REPLY;
            if (mib_AuthAlg_p->Enable == 0x0)
            {
                apicmdbuf_get_auth_mode->authMode = OPEN_SYSTEM_ONLY;
            }
            else
            {
                apicmdbuf_get_auth_mode->authMode = mib_AuthAlg_p->Type + 1;
            }
            apicmdbuf_get_auth_mode->cmdStatus = APCTL_OK;
        }
        break;
        
    case APCMD_SET_AUTH_MODE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_AUTH_MODE *apicmdbuf_set_auth_mode;
            apicmdbuf_set_auth_mode = (APICMDBUF_SET_AUTH_MODE *)apicmdbuf;
            apicmdbuf_set_auth_mode->cmdCode = APCMD_SET_AUTH_MODE_REPLY;

            if (apicmdbuf_set_auth_mode->authMode == NO_WEP)
            {
                mib_AuthAlg_p->Enable = 0;
                mib_AuthAlg_p->Type = AUTH_OPEN_SYSTEM;
            }
            else
            {
                mib_AuthAlg_p->Enable = 1;
                mib_AuthAlg_p->Type = apicmdbuf_set_auth_mode->authMode - 1;
            }
            apicmdbuf_set_auth_mode->cmdStatus = APCTL_OK;
        }
        break;

#ifdef AG_MIXED_SECURITY        
    case APCMD_GET_DEFAULT_WEP_KEYS_G:
        if (apicmdbuf->cmdBodyLen < 74)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 74;
            return OS_FAIL;
        }
        else
        {
            int weptype = WEP_NOT_SET;
            APICMDBUF_GET_DEFAULT_WEP_KEYS *apicmdbuf_get_default_wep_keys;
            apicmdbuf_get_default_wep_keys = (APICMDBUF_GET_DEFAULT_WEP_KEYS *)apicmdbuf;
            apicmdbuf_get_default_wep_keys->cmdCode = APCMD_GET_DEFAULT_WEP_KEYS_G_REPLY;
            apicmdbuf_get_default_wep_keys->keyindex = mib_defaultkeyindex_G;
#ifdef AP5X
            apicmdbuf_get_default_wep_keys->keyType0 = WepType_G[0];
#else
            if (WepType_G == 0)
                weptype = WEP_NOT_SET;
            else if (WepType_G == 1)
                weptype = WEP_40_BIT;
            else if (WepType_G == 2)
                weptype = WEP_104_BIT;
            apicmdbuf_get_default_wep_keys->keyType0 = weptype;
#endif
            memset(apicmdbuf_get_default_wep_keys->keyValue0, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue0, mib_WepDefaultKeys_G[0].WepDefaultKeyValue, 13);

#ifdef AP5X
            apicmdbuf_get_default_wep_keys->keyType1 = WepType_G[1];
#else
            apicmdbuf_get_default_wep_keys->keyType1 = weptype;
#endif
            memset(apicmdbuf_get_default_wep_keys->keyValue1, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue1, mib_WepDefaultKeys_G[1].WepDefaultKeyValue, 13);

#ifdef AP5X
            apicmdbuf_get_default_wep_keys->keyType2 = WepType_G[2];
#else
            apicmdbuf_get_default_wep_keys->keyType2 = weptype;
#endif
            memset(apicmdbuf_get_default_wep_keys->keyValue2, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue2, mib_WepDefaultKeys_G[2].WepDefaultKeyValue, 13);

#ifdef AP5X
            apicmdbuf_get_default_wep_keys->keyType3 = WepType_G[3];
#else
            apicmdbuf_get_default_wep_keys->keyType3 = weptype;
#endif
            memset(apicmdbuf_get_default_wep_keys->keyValue3, 0, 16);
            memcpy(apicmdbuf_get_default_wep_keys->keyValue3, mib_WepDefaultKeys_G[3].WepDefaultKeyValue, 13);
            apicmdbuf_get_default_wep_keys->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_DEFAULT_WEP_KEYS_G:
        if (apicmdbuf->cmdBodyLen < 72)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 72;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_DEFAULT_WEP_KEYS *apicmdbuf_set_default_wep_keys;
            apicmdbuf_set_default_wep_keys = (APICMDBUF_SET_DEFAULT_WEP_KEYS *)apicmdbuf;
            apicmdbuf_set_default_wep_keys->cmdCode = APCMD_SET_DEFAULT_WEP_KEYS_G_REPLY;

            mib_defaultkeyindex_G = apicmdbuf_set_default_wep_keys->keyindex;

            memcpy(mib_WepDefaultKeys_G[0].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue0, 13);
            memcpy(mib_WepDefaultKeys_G[1].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue1, 13);
            memcpy(mib_WepDefaultKeys_G[2].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue2, 13);
            memcpy(mib_WepDefaultKeys_G[3].WepDefaultKeyValue, apicmdbuf_set_default_wep_keys->keyValue3, 13);
#ifdef AP5X
            WepType_G[0] =apicmdbuf_set_default_wep_keys->keyType0;
            WepType_G[1] =apicmdbuf_set_default_wep_keys->keyType1;
            WepType_G[2] =apicmdbuf_set_default_wep_keys->keyType2;
            WepType_G[3] =apicmdbuf_set_default_wep_keys->keyType3;
#else            
            if (apicmdbuf_set_default_wep_keys->keyType0 == WEP_40_BIT)
            {
                WepType_G = 1;
            }
            else if (apicmdbuf_set_default_wep_keys->keyType0 == WEP_104_BIT)
            {
                WepType_G = 2;
            }
            else if (apicmdbuf_set_default_wep_keys->keyType0 == WEP_NOT_SET)
            {
                WepType_G = 0;
            }
#endif
            apicmdbuf_set_default_wep_keys->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_PRIVACY_OPTION_G:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_PRIVACY_OPTION *apicmdbuf_get_privacy_option;
            apicmdbuf_get_privacy_option = (APICMDBUF_GET_PRIVACY_OPTION *)apicmdbuf;
            apicmdbuf_get_privacy_option->cmdCode = APCMD_GET_PRIVACY_OPTION_G_REPLY;
            apicmdbuf_get_privacy_option->dataEncrypt = mib_PrivacyTable_p->PrivInvoked_G;
            apicmdbuf_get_privacy_option->cmdStatus = APCTL_OK;
        }
        break;  

    case APCMD_SET_PRIVACY_OPTION_G:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_PRIVACY_OPTION *apicmdbuf_set_privacy_option;
            apicmdbuf_set_privacy_option = (APICMDBUF_SET_PRIVACY_OPTION *)apicmdbuf;
            apicmdbuf_set_privacy_option->cmdCode = APCMD_SET_PRIVACY_OPTION_G_REPLY;
            if (apicmdbuf_set_privacy_option->dataEncrypt == DATA_ENCRYPT_NOT_REQUIRED)
            {
                mib_PrivacyTable_p->PrivInvoked_G = 0;
            }
            else
            {
                mib_PrivacyTable_p->PrivInvoked_G = 1;
            }
            apicmdbuf_set_privacy_option->cmdStatus = APCTL_OK;
        }
        break;
        
    case APCMD_GET_AUTH_MODE_G:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AUTH_MODE *apicmdbuf_get_auth_mode;
            apicmdbuf_get_auth_mode = (APICMDBUF_GET_AUTH_MODE *)apicmdbuf;
            apicmdbuf_get_auth_mode->cmdCode = APCMD_GET_AUTH_MODE_G_REPLY;
            if (mib_AuthAlg_G->Enable == 0x0)
            {
                apicmdbuf_get_auth_mode->authMode = OPEN_SYSTEM_ONLY;
            }
            else
            {
                apicmdbuf_get_auth_mode->authMode = mib_AuthAlg_G->Type + 1;
            }
            apicmdbuf_get_auth_mode->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_AUTH_MODE_G:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_AUTH_MODE *apicmdbuf_set_auth_mode;
            apicmdbuf_set_auth_mode = (APICMDBUF_SET_AUTH_MODE *)apicmdbuf;
            apicmdbuf_set_auth_mode->cmdCode = APCMD_SET_AUTH_MODE_G_REPLY;

            if (apicmdbuf_set_auth_mode->authMode == NO_WEP)
            {
                mib_AuthAlg_G->Enable = 0;
                mib_AuthAlg_G->Type = AUTH_OPEN_SYSTEM;
            }
            else
            {
                mib_AuthAlg_G->Enable = 1;
                mib_AuthAlg_G->Type = apicmdbuf_set_auth_mode->authMode - 1;
            }
            apicmdbuf_set_auth_mode->cmdStatus = APCTL_OK;
        }
        break;
#endif
        
    case APCMD_GET_ETH_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            int i;
            APICMDBUF_GET_ETH_MAC_ADDRESS* apicmdbuf_get_eth_mac_address;
            apicmdbuf_get_eth_mac_address = (APICMDBUF_GET_ETH_MAC_ADDRESS *)apicmdbuf;
            apicmdbuf_get_eth_mac_address->cmdCode = APCMD_GET_ETH_MAC_ADDRESS_REPLY;
            for (i = 0;i < 6;i++)
                apicmdbuf_get_eth_mac_address->phyAddress[i] = mib_currentmacaddress[i];
            apicmdbuf_get_eth_mac_address->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_ETH_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            int i;
            APICMDBUF_SET_ETH_MAC_ADDRESS* apicmdbuf_set_eth_mac_address;
            apicmdbuf_set_eth_mac_address = (APICMDBUF_SET_ETH_MAC_ADDRESS *)apicmdbuf;
            apicmdbuf_set_eth_mac_address->cmdCode = APCMD_SET_ETH_MAC_ADDRESS_REPLY;
            for (i = 0;i < 6;i++)
                mib_currentmacaddress[i] = apicmdbuf_set_eth_mac_address->phyAddress[i];
            apicmdbuf_set_eth_mac_address->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_GET_WAN_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            int i;
            APICMDBUF_GET_WAN_MAC_ADDRESS* apicmdbuf_get_wan_mac_address;
            apicmdbuf_get_wan_mac_address = (APICMDBUF_GET_WAN_MAC_ADDRESS *)apicmdbuf;
            apicmdbuf_get_wan_mac_address->cmdCode = APCMD_GET_WAN_MAC_ADDRESS_REPLY;
            for (i = 0;i < 6;i++)
                apicmdbuf_get_wan_mac_address->phyAddress[i] = mib_currentmacaddress[i];
            apicmdbuf_get_wan_mac_address->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_WAN_MAC_ADDRESS:
        if (apicmdbuf->cmdBodyLen < 6) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 6;
            return OS_FAIL;
        }
        else
        {
            int i;
            APICMDBUF_SET_WAN_MAC_ADDRESS* apicmdbuf_set_wan_mac_address;
            apicmdbuf_set_wan_mac_address = (APICMDBUF_SET_WAN_MAC_ADDRESS *)apicmdbuf;
            apicmdbuf_set_wan_mac_address->cmdCode = APCMD_SET_WAN_MAC_ADDRESS_REPLY;
            for (i = 0;i < 6;i++)
                mib_currentmacaddress[i] = apicmdbuf_set_wan_mac_address->phyAddress[i];
            apicmdbuf_set_wan_mac_address->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_GET_ETH_IP_CONFIG:
        if (apicmdbuf->cmdBodyLen < 16) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_ETH_IP_CONFIG* apicmdbuf_get_eth_ip_config;
            apicmdbuf_get_eth_ip_config = (APICMDBUF_GET_ETH_IP_CONFIG *)apicmdbuf;
            apicmdbuf_get_eth_ip_config->cmdCode = APCMD_GET_ETH_IP_CONFIG_REPLY;

/*#ifdef TASKDHCP*/

            apicmdbuf_get_eth_ip_config->dhcpEnabled = mib_dhcpenable;
            apicmdbuf_get_eth_ip_config->subnetMask = inet_address(mib_DHCP.SubnetMask);
            apicmdbuf_get_eth_ip_config->ipAddress = inet_address(mib_DHCP.IPAddr);
            apicmdbuf_get_eth_ip_config->gatewayAddress = inet_address(mib_DHCP.GwyAddr);
/*#endif*/

            apicmdbuf_get_eth_ip_config->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_ETH_IP_CONFIG:
        if (apicmdbuf->cmdBodyLen < 16) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_SET_ETH_IP_CONFIG* apicmdbuf_set_eth_ip_config;
            apicmdbuf_set_eth_ip_config = (APICMDBUF_SET_ETH_IP_CONFIG *)apicmdbuf;
            apicmdbuf_set_eth_ip_config->cmdCode = APCMD_SET_ETH_IP_CONFIG_REPLY;

/*#ifdef TASKDHCP*/

            mib_dhcpenable = apicmdbuf_set_eth_ip_config->dhcpEnabled;
            mib_DHCP.SubnetMask = inet_address(apicmdbuf_set_eth_ip_config->subnetMask);
            mib_DHCP.IPAddr = inet_address(apicmdbuf_set_eth_ip_config->ipAddress);
            mib_DHCP.GwyAddr = inet_address(apicmdbuf_set_eth_ip_config->gatewayAddress);


            apicmdbuf_set_eth_ip_config->cmdStatus = APCTL_OK;
            mib_dhcpstatechange = TRUE;
            initiate_dhcp(1);
/*#endif*/

            apicmdbuf_set_eth_ip_config->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_GET_ETH_PORT_STATUS:
        if (apicmdbuf->cmdBodyLen < 5) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 5;
            return FAIL;
        }
        else
#ifdef AP22
        {
            GT_LPORT port;
            GT_BOOL mode;

            APICMDBUF_GET_ETH_PORT_STATUS* apicmdbuf_get_eth_port_status;
            apicmdbuf_get_eth_port_status = (APICMDBUF_GET_ETH_PORT_STATUS *)apicmdbuf;
            apicmdbuf_get_eth_port_status->cmdCode = APCMD_GET_ETH_PORT_STATUS_REPLY;
            port = apicmdbuf_get_eth_port_status->portnumber;
            gprtGetLinkState(&etherDev, port, &mode);
            if (mode == GT_TRUE)
            {
                apicmdbuf_get_eth_port_status->ethOperStatus = ETH_OPER_STATUS_UP;
            }
            else
            {
                apicmdbuf_get_eth_port_status->ethOperStatus = ETH_OPER_STATUS_DOWN;
            }
            gprtGetSpeed(&etherDev, port, &mode);
            if (mode == GT_TRUE)
            {
                apicmdbuf_get_eth_port_status->ethLinkSpeed = TRUE;
            }
            else
            {
                apicmdbuf_get_eth_port_status->ethLinkSpeed = FALSE;
            }
            gprtGetDuplex(&etherDev, port, &mode);
            if (mode == GT_TRUE)
            {
                apicmdbuf_get_eth_port_status->ethDuplex = 100;
            }
            else
            {
                apicmdbuf_get_eth_port_status->ethDuplex = 10;
            }

            apicmdbuf_get_eth_port_status->cmdStatus = APCTL_OK;
        }
#else
        {
            APICMDBUF_GET_ETH_PORT_STATUS* apicmdbuf_get_eth_port_status;

            int status;

            apicmdbuf_get_eth_port_status = (APICMDBUF_GET_ETH_PORT_STATUS *)apicmdbuf;
            apicmdbuf_get_eth_port_status->cmdCode = APCMD_GET_ETH_PORT_STATUS_REPLY;
            apicmdbuf_get_eth_port_status->portnumber = 0;

            status = (WL_REGS32(ETHER0_PORT_STATUS_REG) & 0xff);
            if (status & 0x8)
            {
                apicmdbuf_get_eth_port_status->ethOperStatus = ETH_OPER_STATUS_UP;
            }
            else
            {
                apicmdbuf_get_eth_port_status->ethOperStatus = ETH_OPER_STATUS_DOWN;
            }
            if ( status & 0x01)
            {
                apicmdbuf_get_eth_port_status->ethLinkSpeed = 100;
            }
            else
            {
                apicmdbuf_get_eth_port_status->ethLinkSpeed = 10;
            }

            if (status & 0x02)
            {
                apicmdbuf_get_eth_port_status->ethDuplex = 1;
            }
            else
            {
                apicmdbuf_get_eth_port_status->ethDuplex = 0;
            }
            apicmdbuf_get_eth_port_status->cmdStatus = APCTL_OK;
        }
#endif //AP22
        break;

    case APCMD_SET_ETH_PORT_PARAMS:
        /*todo */
        break;
    case APCMD_GET_ICMP_EN_STATE:
        /*todo */
        break;
    case APCMD_SET_ICMP_EN_STATE:
        /*todo */
        break;
    case APCMD_GET_802_3_STATES:
        /*todo */
        break;
        
    case APCMD_GET_TX_DATA_RATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_TX_DATA_RATE * apicmdbuf_get_tx_data_rate;
            apicmdbuf_get_tx_data_rate = (APICMDBUF_GET_TX_DATA_RATE *)apicmdbuf;
            apicmdbuf_get_tx_data_rate->cmdCode = APCMD_GET_TX_DATA_RATE_REPLY;
            apicmdbuf_get_tx_data_rate->cmdStatus = APCTL_OK;
            if (mib_enableFixedRateTx == 0)
            {
                apicmdbuf_get_tx_data_rate->txDataRate = 0;
            }
            else
            {
                apicmdbuf_get_tx_data_rate->txDataRate = util_GetRateByIndex(mib_txDataRate);
            }
        }
        break;

    case APCMD_SET_TX_DATA_RATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_TX_DATA_RATE * apicmdbuf_set_tx_data_rate;
            apicmdbuf_set_tx_data_rate = (APICMDBUF_SET_TX_DATA_RATE *)apicmdbuf;
            apicmdbuf_set_tx_data_rate->cmdCode = APCMD_SET_TX_DATA_RATE_REPLY;
            apicmdbuf_set_tx_data_rate->cmdStatus = APCTL_OK;
            if (apicmdbuf_set_tx_data_rate->txDataRate == 0)
            {
                mib_enableFixedRateTx = 0;
            	msi_wl_SetAutoRateDrop(1);
            }
            else
            {
                mib_enableFixedRateTx = 1;
                mib_txDataRate = util_GetIndexByRate(apicmdbuf_set_tx_data_rate->txDataRate);
            	msi_wl_SetAutoRateDrop(0);
            }
        }
        break;
        
    case APCMD_GET_TX_DATA_RATE_G:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_TX_DATA_RATE * apicmdbuf_get_tx_data_rate;
            apicmdbuf_get_tx_data_rate = (APICMDBUF_GET_TX_DATA_RATE *)apicmdbuf;
            apicmdbuf_get_tx_data_rate->cmdCode = APCMD_GET_TX_DATA_RATE_G_REPLY;
            apicmdbuf_get_tx_data_rate->cmdStatus = APCTL_OK;
            if (mib_enableFixedRateTx == 0)
            {
                apicmdbuf_get_tx_data_rate->txDataRate = 0;
            }
            else
            {
                apicmdbuf_get_tx_data_rate->txDataRate = util_GetRateByIndex(mib_txDataRateG);
            }
        }
        break;

    case APCMD_SET_TX_DATA_RATE_G:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_TX_DATA_RATE * apicmdbuf_set_tx_data_rate;
            apicmdbuf_set_tx_data_rate = (APICMDBUF_SET_TX_DATA_RATE *)apicmdbuf;
            apicmdbuf_set_tx_data_rate->cmdCode = APCMD_SET_TX_DATA_RATE_G_REPLY;
            apicmdbuf_set_tx_data_rate->cmdStatus = APCTL_OK;
            if (apicmdbuf_set_tx_data_rate->txDataRate == 0)
            {
                mib_enableFixedRateTx = 0;
            	msi_wl_SetAutoRateDrop(1);
            }
            else
            {
                mib_enableFixedRateTx = 1;
                mib_txDataRateG = util_GetIndexByRate(apicmdbuf_set_tx_data_rate->txDataRate);
            	msi_wl_SetAutoRateDrop(0);
            }
        }
        break;
        
	case APCMD_GET_TX_DATA_RATE_A:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_TX_DATA_RATE * apicmdbuf_get_tx_data_rate;
            apicmdbuf_get_tx_data_rate = (APICMDBUF_GET_TX_DATA_RATE *)apicmdbuf;
            apicmdbuf_get_tx_data_rate->cmdCode = APCMD_GET_TX_DATA_RATE_A_REPLY;
            apicmdbuf_get_tx_data_rate->cmdStatus = APCTL_OK;
            if (mib_enableFixedRateTx == 0)
            {
                apicmdbuf_get_tx_data_rate->txDataRate = 0;
            }
            else
            {
            	apicmdbuf_get_tx_data_rate->txDataRate = util_GetRateByIndex(mib_txDataRateA);
            }
        }
        break;

    case APCMD_SET_TX_DATA_RATE_A:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_TX_DATA_RATE * apicmdbuf_set_tx_data_rate;
            apicmdbuf_set_tx_data_rate = (APICMDBUF_SET_TX_DATA_RATE *)apicmdbuf;
            apicmdbuf_set_tx_data_rate->cmdCode = APCMD_SET_TX_DATA_RATE_A_REPLY;
            apicmdbuf_set_tx_data_rate->cmdStatus = APCTL_OK;
            if (apicmdbuf_set_tx_data_rate->txDataRate == 0)
            {
	            mib_enableFixedRateTx = 0;
				msi_wl_SetAutoRateDrop(1);
	        }
	        else
	        {
	            mib_enableFixedRateTx = 1;
                mib_txDataRateA = util_GetIndexByRate(apicmdbuf_set_tx_data_rate->txDataRate);	        	
	        	msi_wl_SetAutoRateDrop(0);
	        }
        }
        break;

    case APCMD_READ_GPIO:
        if (apicmdbuf->cmdBodyLen < 1) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_READ_GPIO * apicmdbuf_read_gpio;
            apicmdbuf_read_gpio = (APICMDBUF_READ_GPIO *)apicmdbuf;
            apicmdbuf_read_gpio->cmdCode = APCMD_READ_GPIO_REPLY;
            /*todo; */
            apicmdbuf_read_gpio->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_WRITE_GPIO:
        if (apicmdbuf->cmdBodyLen < 1) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_WRITE_GPIO * apicmdbuf_write_gpio;
            apicmdbuf_write_gpio = (APICMDBUF_WRITE_GPIO *)apicmdbuf;
            apicmdbuf_write_gpio->cmdCode = APCMD_WRITE_GPIO_REPLY;
            /*	mib_txDataRate=apicmdbuf_set_tx_data_rate->txDataRate; */
            apicmdbuf_write_gpio->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_FW_SOFT_RESET:
        {
            apicmdbuf->cmdStatus = APCTL_OK;
            Reset();
        }

        break;

#ifdef MACACCESSTABLE

    case APCMD_GET_WLAN_FILTER_LIST:
        {
            UINT16 nostation = mib_wlanfilterno;
            UINT16 minbodysize = 2 + mib_wlanfilterno * 6;
            if (apicmdbuf->cmdBodyLen < minbodysize)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = minbodysize;
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_LIST_REPLY;
                return OS_FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                /*Number of associated station */
#if defined(UDP_API)
                apicmdbuf->cmdBody[1] = 0; /*Cannot be more than 256 for now */
                apicmdbuf->cmdBody[0] = nostation % 0x100;
#else
                //Number of associated station
                apicmdbuf->cmdBody[0] = 0; //Cannot be more than 256 for now
                apicmdbuf->cmdBody[1] = nostation % 0x100;
#endif
                memcpy(&apicmdbuf->cmdBody[2], mib_wlanfiltermac, mib_wlanfilterno*6);
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
#endif

    case APCMD_SET_WLAN_FILTER_LIST:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_WLAN_FILTER_LIST_REPLY;
                return FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
                SendCtrlMsgTo(apicmdbuf, apicmdbuf->cmdBodyLen + APICMDHEADERLEN);
#endif
                /*Number of associated station */
                /* as the number can not be more than 256 for now,
                 no matter what the order of bytes, we can replace the original as */
                mib_wlanfilterno = apicmdbuf->cmdBody[0]+apicmdbuf->cmdBody[1];
                if (mib_wlanfiltertype == FILTER_MODE )
                {
                    SendResetCmd(1);
                    os_TaskDelay(100);
                    SendStartCmd();
			        os_TaskDelay(200);
                }
                memcpy(mib_wlanfiltermac, &apicmdbuf->cmdBody[2], mib_wlanfilterno*6);
                apicmdbuf->cmdCode = APCMD_SET_WLAN_FILTER_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_WLAN_FILTER_MODE:
        {
#ifdef WIPOD_NEW_GUI
            if (apicmdbuf->cmdBodyLen < sizeof(APICMDBUF_GET_FILTER_MODE) - 4*sizeof(unsigned short))
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = sizeof(APICMDBUF_GET_FILTER_MODE) - 4*sizeof(unsigned short);
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_MODE_REPLY;
                return OS_FAIL;
            }
#else
            if (apicmdbuf->cmdBodyLen < 1 )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_GET_WLAN_FILTER_MODE_REPLY;
                return OS_FAIL;
            }
#endif
            else
            {
                APICMDBUF_GET_FILTER_MODE *apicmdbuf_get_filter_mode;
                apicmdbuf_get_filter_mode = (APICMDBUF_GET_FILTER_MODE *)apicmdbuf; 
                apicmdbuf_get_filter_mode->mode = mib_wlanfiltertype;
                apicmdbuf_get_filter_mode->cmdCode = APCMD_GET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf_get_filter_mode->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_WLAN_FILTER_MODE:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_WLAN_FILTER_MODE_REPLY;
                return OS_FAIL;
            }
            else
            {
                APICMDBUF_SET_FILTER_MODE *apicmdbuf_set_filter_mode;
                apicmdbuf_set_filter_mode = (APICMDBUF_SET_FILTER_MODE *)apicmdbuf;
#ifdef APCFGUR
                SendCtrlMsgTo(apicmdbuf, apicmdbuf->cmdBodyLen + APICMDHEADERLEN);
#endif
                mib_wlanfiltertype = apicmdbuf_set_filter_mode->mode;
                apicmdbuf_set_filter_mode->cmdCode = APCMD_SET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf_set_filter_mode->cmdStatus = APCTL_OK;
                //SendResetCmd(0);
                macMgmtMlme_SendDeauthenticateMsg(&bcast, MCBC_STN_ID, IEEEtypes_REASON_DEAUTH_LEAVING);
            }
        }
        break;

    case APCMD_UPDATE_FIRMWARE:
        {
            APICMDBUF_UPDATE_FIRMWARE * apicmdbuf_update_fw;
            apicmdbuf_update_fw = (APICMDBUF_UPDATE_FIRMWARE *)apicmdbuf;
            if (apicmdbuf_update_fw->cmdBodyLen < 100*1024)
            {
                apicmdbuf_update_fw->cmdCode = APCMD_SET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf_update_fw->cmdStatus = APCTL_BUF_TOO_SMALL;
                return OS_FAIL;
            }
            else
            {
                FirmwareUpgrade(apicmdbuf_update_fw->imgPtr, apicmdbuf_update_fw->cmdBodyLen);
                apicmdbuf_update_fw->cmdCode = APCMD_SET_WLAN_FILTER_MODE_REPLY;
                apicmdbuf_update_fw->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SAVE_USER_CONFIG:
        {
            apicmdbuf->cmdCode = APCMD_SAVE_USER_CONFIG_REPLY;
            apicmdbuf->cmdStatus = APCTL_OK;
            UserConfigModified();
        }
        break;
    case APCMD_SET_STA_AGING_TIME_IN_MINUTES:
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_STA_AGING_TIME* apicmdbuf_set_sta_aging_time;
            apicmdbuf_set_sta_aging_time = (APICMDBUF_SET_STA_AGING_TIME *)apicmdbuf;
            apicmdbuf_set_sta_aging_time->cmdCode = APCMD_SET_STA_AGING_TIME_IN_MINUTES_REPLY;
            set_sta_aging_time(apicmdbuf_set_sta_aging_time->timeInMinutes);
			mib_AgingTime = apicmdbuf_set_sta_aging_time->timeInMinutes; 
            apicmdbuf_set_sta_aging_time->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_SET_WLAN_RADIO:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_WLAN_RADIO* apicmdbuf_set_wlan_radio;
            apicmdbuf_set_wlan_radio = (APICMDBUF_SET_WLAN_RADIO *)apicmdbuf;
            apicmdbuf_set_wlan_radio->cmdCode = APCMD_SET_WLAN_RADIO_REPLY;
            Wlan_Radio(apicmdbuf_set_wlan_radio->radio);
            apicmdbuf_set_wlan_radio->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_SET_SDRAM_BURST_MODE:
        {
            apicmdbuf->cmdCode++;   // Make it a reply
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_SDRAM_BURST_MODE *p =
                    (APICMDBUF_SDRAM_BURST_MODE *)apicmdbuf;
                if ( p->sdramBurstMode > SDRAM_BURST_8 )
                {
                    apicmdbuf->cmdStatus = APCTL_ERROR;
                    apicmdbuf->cmdBodyLen = 1;
                    return FAIL;
                }
#ifdef SDRAMBURST
                SetSdramBurstMode(p->sdramBurstMode);
#endif

                apicmdbuf->cmdStatus = APCTL_OK;
            }
        }

#ifdef WPA
    case APCMD_GET_STA_CFG_RSN_OPTION_IMP:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_STA_CFG_RSN_OPTION_IMP* apicmdbuf_get_sta_cfg_rsn_option_imp;
                apicmdbuf_get_sta_cfg_rsn_option_imp = (APICMDBUF_GET_STA_CFG_RSN_OPTION_IMP*)apicmdbuf;
                apicmdbuf_get_sta_cfg_rsn_option_imp->cmdCode = APCMD_GET_STA_CFG_RSN_OPTION_IMP_REPLY;
                apicmdbuf_get_sta_cfg_rsn_option_imp->RSN_Capable = mib_StaCfg_p->RSNOptionImplemented;
                apicmdbuf_get_sta_cfg_rsn_option_imp->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS:
        {
            if (apicmdbuf->cmdBodyLen < 4)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 4;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS* apicmdbuf_get_sta_cfg_tkip_num_rply_counters;
                apicmdbuf_get_sta_cfg_tkip_num_rply_counters = (APICMDBUF_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS*)apicmdbuf;
                apicmdbuf_get_sta_cfg_tkip_num_rply_counters->cmdCode = APCMD_GET_STA_CFG_TKIP_NUM_RPLY_COUNTERS_REPLY;
                apicmdbuf_get_sta_cfg_tkip_num_rply_counters->NumOfCounters = mib_StaCfg_p->TKIPNumberOfReplayCounters;
                apicmdbuf_get_sta_cfg_tkip_num_rply_counters->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_GET_PRIVACY_TBL_RSN_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_PRIVACY_TBL_RSN_ENABLED* apicmdbuf_get_privacy_tbl_rsn_enabled;
                apicmdbuf_get_privacy_tbl_rsn_enabled = (APICMDBUF_GET_PRIVACY_TBL_RSN_ENABLED*)apicmdbuf;
                apicmdbuf_get_privacy_tbl_rsn_enabled->cmdCode = APCMD_GET_PRIVACY_TBL_RSN_ENABLED_REPLY;
                apicmdbuf_get_privacy_tbl_rsn_enabled->Enabled = mib_PrivacyTable_p->RSNEnabled;
                apicmdbuf_get_privacy_tbl_rsn_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_PRIVACY_TBL_RSN_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED* apicmdbuf_set_privacy_tbl_rsn_enabled;
                apicmdbuf_set_privacy_tbl_rsn_enabled = (APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED*)apicmdbuf;
                apicmdbuf_set_privacy_tbl_rsn_enabled->cmdCode = APCMD_SET_PRIVACY_TBL_RSN_ENABLED_REPLY;
							mib_PrivacyTable_p->RSNEnabled = apicmdbuf_set_privacy_tbl_rsn_enabled->Enabled; 
                apicmdbuf_set_privacy_tbl_rsn_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_GET_RSN_CFG_TBL:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG_TBL) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG_TBL* apicmdbuf_get_rsn_cfg_tbl;
                apicmdbuf_get_rsn_cfg_tbl = (APICMDBUF_GET_RSN_CFG_TBL*)apicmdbuf;
                apicmdbuf_get_rsn_cfg_tbl->cmdCode = APCMD_GET_RSN_CFG_TBL_REPLY;
                apicmdbuf_get_rsn_cfg_tbl->Version = mib_RSNConfig_p->Version;
                apicmdbuf_get_rsn_cfg_tbl->PairwiseKeysSupported = mib_RSNConfig_p->PairwiseKeysSupported;
                memcpy( apicmdbuf_get_rsn_cfg_tbl->MulticastCipher, mib_RSNConfig_p->MulticastCipher, 4);
                apicmdbuf_get_rsn_cfg_tbl->GroupRekeyMethod = mib_RSNConfig_p->GroupRekeyMethod;
                apicmdbuf_get_rsn_cfg_tbl->GroupRekeyTime = mib_RSNConfig_p->GroupRekeyTime;
                apicmdbuf_get_rsn_cfg_tbl->GroupRekeyPackets = mib_RSNConfig_p->GroupRekeyPackets;
                apicmdbuf_get_rsn_cfg_tbl->GroupRekeyStrict = mib_RSNConfig_p->GroupRekeyStrict;
                apicmdbuf_get_rsn_cfg_tbl->TSNEnabled = mib_RSNConfig_p->TSNEnabled;
                apicmdbuf_get_rsn_cfg_tbl->GroupMasterRekeyTime = mib_RSNConfig_p->GroupMasterRekeyTime;
                apicmdbuf_get_rsn_cfg_tbl->GroupUpdateTimeOut = mib_RSNConfig_p->GroupUpdateTimeOut;
                apicmdbuf_get_rsn_cfg_tbl->GroupUpdateCount = mib_RSNConfig_p->GroupUpdateCount;
                apicmdbuf_get_rsn_cfg_tbl->PairwiseUpdateTimeOut = mib_RSNConfig_p->PairwiseUpdateTimeOut;
                apicmdbuf_get_rsn_cfg_tbl->PairwiseUpdateCount = mib_RSNConfig_p->PairwiseUpdateCount;
                apicmdbuf_get_rsn_cfg_tbl->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_RSN_CFG_TBL:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_TBL) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_TBL* apicmdbuf_set_rsn_cfg_tbl;
                apicmdbuf_set_rsn_cfg_tbl = (APICMDBUF_SET_RSN_CFG_TBL*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_tbl->cmdCode = APCMD_SET_RSN_CFG_TBL_REPLY;
                memcpy( mib_RSNConfig_p->PSKPassPhrase, apicmdbuf_set_rsn_cfg_tbl->PSKPassPhrase, 64 );
                apicmdbuf_set_rsn_cfg_tbl->cmdStatus = APCTL_OK;
            }
        }
        break;

    /* WPA2 */
    case APCMD_GET_RSN_CFG_WPA2_TBL:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG_WPA2_TBL) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG_WPA2_TBL* apicmdbuf_get_rsn_cfg_wpa2_tbl;
                apicmdbuf_get_rsn_cfg_wpa2_tbl = (APICMDBUF_GET_RSN_CFG_WPA2_TBL*)apicmdbuf;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->cmdCode = APCMD_GET_RSN_CFG_WPA2_TBL_REPLY;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->Version = mib_RSNConfigWPA2_p->Version;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->PairwiseKeysSupported = mib_RSNConfigWPA2_p->PairwiseKeysSupported;
                memcpy( apicmdbuf_get_rsn_cfg_wpa2_tbl->MulticastCipher, mib_RSNConfigWPA2_p->MulticastCipher, 4);
                apicmdbuf_get_rsn_cfg_wpa2_tbl->GroupRekeyMethod = mib_RSNConfigWPA2_p->GroupRekeyMethod;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->GroupRekeyTime = mib_RSNConfigWPA2_p->GroupRekeyTime;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->GroupRekeyPackets = mib_RSNConfigWPA2_p->GroupRekeyPackets;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->GroupRekeyStrict = mib_RSNConfigWPA2_p->GroupRekeyStrict;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->TSNEnabled = mib_RSNConfigWPA2_p->TSNEnabled;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->GroupMasterRekeyTime = mib_RSNConfigWPA2_p->GroupMasterRekeyTime;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->GroupUpdateTimeOut = mib_RSNConfigWPA2_p->GroupUpdateTimeOut;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->GroupUpdateCount = mib_RSNConfigWPA2_p->GroupUpdateCount;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->PairwiseUpdateTimeOut = mib_RSNConfigWPA2_p->PairwiseUpdateTimeOut;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->PairwiseUpdateCount = mib_RSNConfigWPA2_p->PairwiseUpdateCount;
                apicmdbuf_get_rsn_cfg_wpa2_tbl->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_RSN_CFG_WPA2_TBL:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_WPA2_TBL) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_WPA2_TBL* apicmdbuf_set_rsn_cfg_wpa2_tbl;
                apicmdbuf_set_rsn_cfg_wpa2_tbl = (APICMDBUF_SET_RSN_CFG_WPA2_TBL*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_wpa2_tbl->cmdCode = APCMD_SET_RSN_CFG_WPA2_TBL_REPLY;
                memcpy( mib_RSNConfigWPA2_p->PSKPassPhrase, apicmdbuf_set_rsn_cfg_wpa2_tbl->PSKPassPhrase, 64 );
                apicmdbuf_set_rsn_cfg_wpa2_tbl->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_GET_RSN_STATS:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_STATS) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_STATS* apicmdbuf_get_rsn_stats;
                apicmdbuf_get_rsn_stats = (APICMDBUF_GET_RSN_STATS*)apicmdbuf;
                apicmdbuf_get_rsn_stats->cmdCode = APCMD_GET_RSN_STATS_REPLY;

                memcpy( apicmdbuf_get_rsn_stats->MacAddr, mib_RSNStats_p->MacAddr, 6);
                apicmdbuf_get_rsn_stats->Version = mib_RSNStats_p->Version;
                memcpy( apicmdbuf_get_rsn_stats->SelectedUnicastCipher, mib_RSNStats_p->SelectedUnicastCipher, 4);
                apicmdbuf_get_rsn_stats->TKIPICVErrors = mib_RSNStats_p->TKIPICVErrors;
                apicmdbuf_get_rsn_stats->TKIPLocalMICFailures = mib_RSNStats_p->TKIPLocalMICFailures;
                apicmdbuf_get_rsn_stats->TKIPRemoteMICFailures = mib_RSNStats_p->TKIPRemoteMICFailures;
                apicmdbuf_get_rsn_stats->TKIPCounterMeasuresInvoked = mib_RSNStats_p->TKIPCounterMeasuresInvoked;
                apicmdbuf_get_rsn_stats->WRAPFormatErrors = mib_RSNStats_p->WRAPFormatErrors;
                apicmdbuf_get_rsn_stats->WRAPReplays = mib_RSNStats_p->WRAPReplays;
                apicmdbuf_get_rsn_stats->WRAPDecryptErrors = mib_RSNStats_p->WRAPDecryptErrors;
                apicmdbuf_get_rsn_stats->CCMPFormatErrors = mib_RSNStats_p->CCMPFormatErrors;
                apicmdbuf_get_rsn_stats->CCMPReplays = mib_RSNStats_p->CCMPReplays;
                apicmdbuf_get_rsn_stats->CCMPDecryptErrors = mib_RSNStats_p->CCMPDecryptErrors;

                apicmdbuf_get_rsn_stats->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_RSN_DATA_TRAFFIC_ENABLED:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_DATA_TRAFFIC_ENABLED) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_DATA_TRAFFIC_ENABLED* apicmdbuf_set_rsn_data_traffic_enabled;
                apicmdbuf_set_rsn_data_traffic_enabled = (APICMDBUF_SET_RSN_DATA_TRAFFIC_ENABLED*)apicmdbuf;
                apicmdbuf_set_rsn_data_traffic_enabled->cmdCode = APCMD_SET_RSN_DATA_TRAFFIC_ENABLED_REPLY;

                // set ext sta db
                if ( STATE_SUCCESS == extStaDb_SetRSNDataTrafficEnabled(
                            (IEEEtypes_MacAddr_t *) apicmdbuf_set_rsn_data_traffic_enabled->StaMacAddr,
                            apicmdbuf_set_rsn_data_traffic_enabled->Enabled ) )
                    apicmdbuf_set_rsn_data_traffic_enabled->cmdStatus = APCTL_OK;
                else
                    apicmdbuf_set_rsn_data_traffic_enabled->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;

    case APCMD_SET_RSN_PWK:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_PWK) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_PWK* apicmdbuf_set_rsn_pwk;
                apicmdbuf_set_rsn_pwk = (APICMDBUF_SET_RSN_PWK*)apicmdbuf;
                apicmdbuf_set_rsn_pwk->cmdCode = APCMD_SET_RSN_PWK_REPLY;

                // set ext sta db
                if ( STATE_SUCCESS == extStaDb_SetRSNPwk(
                            (IEEEtypes_MacAddr_t *)apicmdbuf_set_rsn_pwk->StaMacAddr,
                            apicmdbuf_set_rsn_pwk->EncryptKey, apicmdbuf_set_rsn_pwk->TxMICKey,
                            apicmdbuf_set_rsn_pwk->RxMICKey ) )
                    apicmdbuf_set_rsn_pwk->cmdStatus = APCTL_OK;
                else
                    apicmdbuf_set_rsn_pwk->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;

    case APCMD_GET_RSN_PWK:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_PWK) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_PWK* apicmdbuf_get_rsn_pwk;
                apicmdbuf_get_rsn_pwk = (APICMDBUF_GET_RSN_PWK*)apicmdbuf;
                apicmdbuf_get_rsn_pwk->cmdCode = APCMD_GET_RSN_PWK_REPLY;

                // set ext sta db
                if ( STATE_SUCCESS == extStaDb_GetRSNPwk(
                            (IEEEtypes_MacAddr_t *)apicmdbuf_get_rsn_pwk->StaMacAddr,
                            apicmdbuf_get_rsn_pwk->EncryptKey, apicmdbuf_get_rsn_pwk->TxMICKey,
                            apicmdbuf_get_rsn_pwk->RxMICKey ) )
                    apicmdbuf_get_rsn_pwk->cmdStatus = APCTL_OK;
                else
                    apicmdbuf_get_rsn_pwk->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_SET_RSN_GRP_KEY:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_GRP_KEY) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_GRP_KEY* apicmdbuf_set_rsn_grp_key;
                apicmdbuf_set_rsn_grp_key = (APICMDBUF_SET_RSN_GRP_KEY*)apicmdbuf;
                apicmdbuf_set_rsn_grp_key->cmdCode = APCMD_SET_RSN_GRP_KEY_REPLY;

                memcpy(mib_MrvlRSN_GrpKey.EncryptKey, apicmdbuf_set_rsn_grp_key->EncryptKey, 16);
                memcpy(mib_MrvlRSN_GrpKey.TxMICKey, apicmdbuf_set_rsn_grp_key->TxMICKey, 8);
                memcpy(mib_MrvlRSN_GrpKey.RxMICKey, apicmdbuf_set_rsn_grp_key->RxMICKey, 8);
                gGrpKeyInstalled = TRUE;
                apicmdbuf_set_rsn_grp_key->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_GET_RSN_GRP_KEY:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_GRP_KEY) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_GRP_KEY* apicmdbuf_get_rsn_grp_key;
                apicmdbuf_get_rsn_grp_key = (APICMDBUF_GET_RSN_GRP_KEY*)apicmdbuf;
                apicmdbuf_get_rsn_grp_key->cmdCode = APCMD_GET_RSN_GRP_KEY_REPLY;

                memcpy(apicmdbuf_get_rsn_grp_key->EncryptKey, mib_MrvlRSN_GrpKey.EncryptKey, 16);
                memcpy(apicmdbuf_get_rsn_grp_key->TxMICKey, mib_MrvlRSN_GrpKey.TxMICKey, 8);
                memcpy(apicmdbuf_get_rsn_grp_key->RxMICKey, mib_MrvlRSN_GrpKey.RxMICKey, 8);
                apicmdbuf_get_rsn_grp_key->cmdStatus = APCTL_OK;
            }
        }
        break;
/*
    case APCMD_GET_STA_ASSOC_STATE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_STA_ASSOC_STATE) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                StaAssocStateMsg_t * pStaMsg;
                APICMDBUF_GET_STA_ASSOC_STATE* apicmdbuf_get_sta_assoc_state;
                apicmdbuf_get_sta_assoc_state = (APICMDBUF_GET_STA_ASSOC_STATE*)apicmdbuf;
                apicmdbuf_get_sta_assoc_state->cmdCode = APCMD_GET_STA_ASSOC_STATE_REPLY;

                if ( (pStaMsg = (StaAssocStateMsg_t *)os_QueueReadNoBlock(sysinfo_STA_ASSOC_STATE_Q)) != NULL )
                {
                    memcpy(apicmdbuf_get_sta_assoc_state->staMACAddr, pStaMsg->staMACAddr, 6);
                    apicmdbuf_get_sta_assoc_state->assocType = pStaMsg->assocType;
                    pool_FreeBuf((UINT8 *)pStaMsg);
                    //free(pStaMsg);
                    apicmdbuf_get_sta_assoc_state->cmdStatus = APCTL_OK;
                }
                else
                {
                    apicmdbuf_get_sta_assoc_state->cmdStatus = APCTL_CMD_FAILED;
                }
            }
        }
        break;
*/
    case APCMD_GET_THIS_STA_RSN_IE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_THIS_STA_RSN_IE) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_THIS_STA_RSN_IE* apicmdbuf_get_this_sta_rsn_ie;
                apicmdbuf_get_this_sta_rsn_ie = (APICMDBUF_GET_THIS_STA_RSN_IE*)apicmdbuf;
                apicmdbuf_get_this_sta_rsn_ie->cmdCode = APCMD_GET_THIS_STA_RSN_IE_REPLY;

                memcpy( &apicmdbuf_get_this_sta_rsn_ie->RsnIEBuf, &thisStaRsnIE, sizeof(IEEEtypes_RSN_IE_t) );
                if (mib_RSNConfigWPA2_p->WPA2Enabled) 
                    memcpy( &apicmdbuf_get_this_sta_rsn_ie->RsnIEWPA2Buf, &thisStaRsnIEWPA2MixedMode, thisStaRsnIEWPA2MixedMode.Len + 2 );
                else
                    memcpy( &apicmdbuf_get_this_sta_rsn_ie->RsnIEWPA2Buf, &thisStaRsnIEWPA2, thisStaRsnIEWPA2.Len + 2 );

                apicmdbuf_get_this_sta_rsn_ie->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_GET_EXT_STA_RSN_IE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_EXT_STA_RSN_IE) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_EXT_STA_RSN_IE* apicmdbuf_get_ext_sta_rsn_ie;
                apicmdbuf_get_ext_sta_rsn_ie = (APICMDBUF_GET_EXT_STA_RSN_IE*)apicmdbuf;
                apicmdbuf_get_ext_sta_rsn_ie->cmdCode = APCMD_GET_EXT_STA_RSN_IE_REPLY;

                if ( STATE_SUCCESS == extStaDb_GetRSN_IE(
                            (IEEEtypes_MacAddr_t *)apicmdbuf_get_ext_sta_rsn_ie->staMACAddr,
                            apicmdbuf_get_ext_sta_rsn_ie->RsnIEBuf) )
                    apicmdbuf_get_ext_sta_rsn_ie->cmdStatus = APCTL_OK;
                else
                    apicmdbuf_get_ext_sta_rsn_ie->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_SET_RSN_CFG_AUTH_SUITE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_AUTH_SUITE) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_AUTH_SUITE* apicmdbuf_set_rsn_cfg_auth_suite;
                apicmdbuf_set_rsn_cfg_auth_suite =
                    (APICMDBUF_SET_RSN_CFG_AUTH_SUITE*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_auth_suite->cmdCode = APCMD_SET_RSN_CFG_AUTH_SUITE;

                memcpy(mib_RSNConfigAuthSuites_p->AuthSuites,
                       apicmdbuf_set_rsn_cfg_auth_suite->AuthSuites, 4);
                mib_RSNConfigAuthSuites_p->Enabled = apicmdbuf_set_rsn_cfg_auth_suite->Enabled;
                apicmdbuf_set_rsn_cfg_auth_suite->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_RSN_CFG_AUTH_SUITE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG_AUTH_SUITE) -
                                         4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG_AUTH_SUITE* apicmdbuf_get_rsn_cfg_auth_suite;
                apicmdbuf_get_rsn_cfg_auth_suite =
                    (APICMDBUF_GET_RSN_CFG_AUTH_SUITE*)apicmdbuf;
                apicmdbuf_get_rsn_cfg_auth_suite->cmdCode =
                    APCMD_GET_RSN_CFG_AUTH_SUITE_REPLY;
                memcpy(apicmdbuf_get_rsn_cfg_auth_suite->AuthSuites,
                       mib_RSNConfigAuthSuites_p->AuthSuites, 4);
                apicmdbuf_get_rsn_cfg_auth_suite->Enabled = mib_RSNConfigAuthSuites_p->Enabled;
                apicmdbuf_get_rsn_cfg_auth_suite->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_RSN_CFG_UNICAST_CIPHER:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_UNICAST_CIPHER) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_UNICAST_CIPHER* apicmdbuf_set_rsn_cfg_unicast_cipher;
                apicmdbuf_set_rsn_cfg_unicast_cipher =
                    (APICMDBUF_SET_RSN_CFG_UNICAST_CIPHER*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_unicast_cipher->cmdCode = APCMD_SET_RSN_CFG_UNICAST_CIPHER_REPLY;

                memcpy(mib_RSNConfigUnicastCiphers_p->UnicastCipher,
                       apicmdbuf_set_rsn_cfg_unicast_cipher->UnicastCipher, 4);
                mib_RSNConfigUnicastCiphers_p->Enabled =
                    apicmdbuf_set_rsn_cfg_unicast_cipher->Enabled;
#ifdef AP_URPTR
                if (mib_urMode == 1)
                {
                    if (!mib_RSNConfigWPA2_p->WPA2OnlyEnabled && !mib_RSNConfigWPA2_p->WPA2Enabled
                        && mib_RSNConfigUnicastCiphers_p->UnicastCipher[3] == 2)
                    {
                        //ProcessWPAPcktStaFp = ProcessTKIPPcktStaUr;
                        ProcessWPAPcktStaUnicastFp = ProcessTKIPPcktStaUr;
                        DoWPAAndSchedFrameStaFp = DoTKIPAndSchedFrameStaUr;
                        ProcessWPAPcktStaBcastFp = ProcessTKIPPcktStaUr;  // UR_WPA2_TEST
                    }
                    else
                    {
                        //ProcessWPAPcktStaFp = ProcessCCMPPcktStaUr;
                        ProcessWPAPcktStaUnicastFp = ProcessCCMPPcktStaUr;
                        DoWPAAndSchedFrameStaFp = DoCCMPAndSchedFrameStaUr;

                        ProcessWPAPcktStaBcastFp = ProcessCCMPPcktStaUr; // UR_WPA_TEST
                    }
                }
#endif /* AP_URPTR */
                apicmdbuf_set_rsn_cfg_unicast_cipher->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_RSN_CFG_UNICAST_CIPHER:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG_UNICAST_CIPHER) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG_UNICAST_CIPHER* apicmdbuf_get_rsn_cfg_unicast_cipher;
                apicmdbuf_get_rsn_cfg_unicast_cipher =
                    (APICMDBUF_GET_RSN_CFG_UNICAST_CIPHER*)apicmdbuf;
                apicmdbuf_get_rsn_cfg_unicast_cipher->cmdCode = APCMD_GET_RSN_CFG_UNICAST_CIPHER;

                memcpy(apicmdbuf_get_rsn_cfg_unicast_cipher->UnicastCipher,
                       mib_RSNConfigUnicastCiphers_p->UnicastCipher, 4);
                apicmdbuf_get_rsn_cfg_unicast_cipher->Enabled = mib_RSNConfigUnicastCiphers_p->Enabled;
                apicmdbuf_get_rsn_cfg_unicast_cipher->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_RSN_CFG_MULTICAST_CIPHERS:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS) -
                                         4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS* apicmdbuf_set_rsn_cfg_mcast_cipher;
                apicmdbuf_set_rsn_cfg_mcast_cipher =
                (APICMDBUF_SET_RSN_CFG_MULTICAST_CIPHERS*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_mcast_cipher->cmdCode =
                                                APCMD_SET_RSN_CFG_MULTICAST_CIPHERS;
                memcpy(mib_RSNConfig_p->MulticastCipher,
                       apicmdbuf_set_rsn_cfg_mcast_cipher->MulticastCipher,4);
                apicmdbuf_set_rsn_cfg_mcast_cipher->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_RSN_CFG_WPA2_MULTICAST_CIPHERS:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_WPA2_MULTICAST_CIPHERS) -
                                         4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_WPA2_MULTICAST_CIPHERS* apicmdbuf_set_rsn_cfg_wpa2_mcast_cipher;
                apicmdbuf_set_rsn_cfg_wpa2_mcast_cipher =
                (APICMDBUF_SET_RSN_CFG_WPA2_MULTICAST_CIPHERS*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_wpa2_mcast_cipher->cmdCode =
                                                APCMD_SET_RSN_CFG_WPA2_MULTICAST_CIPHERS;
                memcpy(mib_RSNConfigWPA2_p->MulticastCipher,
                       apicmdbuf_set_rsn_cfg_wpa2_mcast_cipher->MulticastCipher,4);
                apicmdbuf_set_rsn_cfg_wpa2_mcast_cipher->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_PSK_PASSPHRASE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_PSK_PASSPHRASE* apicmdbuf_set_psk_passph;
            apicmdbuf_set_psk_passph = (APICMDBUF_SET_PSK_PASSPHRASE *)apicmdbuf;
            apicmdbuf_set_psk_passph->cmdCode = APCMD_SET_PSK_PASSPHRASE_REPLY;
            strcpy(mib_RSNConfig_p->PSKPassPhrase, apicmdbuf_set_psk_passph->passphrase);

            apicmdbuf_set_psk_passph->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_GRP_REKEY_TIME:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_GRP_REKEY_TIME* apicmdbuf_set_grp_rekey_time;
            apicmdbuf_set_grp_rekey_time = (APICMDBUF_SET_GRP_REKEY_TIME *)apicmdbuf;
            apicmdbuf_set_grp_rekey_time->cmdCode = APCMD_SET_GRP_REKEY_TIME_REPLY;
            mib_RSNConfig_p->GroupRekeyTime = apicmdbuf_set_grp_rekey_time->rekey_time;

            apicmdbuf_set_grp_rekey_time->cmdStatus = APCTL_OK;
        }
        break;

    /* WPA2*/
    case APCMD_SET_RSN_CFG_WPA2_AUTH_SUITE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_WPA2_AUTH_SUITE) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_WPA2_AUTH_SUITE* apicmdbuf_set_rsn_cfg_wpa2_auth_suite;
                apicmdbuf_set_rsn_cfg_wpa2_auth_suite =
                    (APICMDBUF_SET_RSN_CFG_WPA2_AUTH_SUITE*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_wpa2_auth_suite->cmdCode = APCMD_SET_RSN_CFG_WPA2_AUTH_SUITE_REPLY;

                memcpy(mib_RSNConfigWPA2AuthSuites_p->AuthSuites,
                       apicmdbuf_set_rsn_cfg_wpa2_auth_suite->AuthSuites, 4);
                mib_RSNConfigWPA2AuthSuites_p->Enabled = apicmdbuf_set_rsn_cfg_wpa2_auth_suite->Enabled;
                apicmdbuf_set_rsn_cfg_wpa2_auth_suite->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_RSN_CFG_WPA2_AUTH_SUITE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG_WPA2_AUTH_SUITE) -
                                         4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG_WPA2_AUTH_SUITE* apicmdbuf_get_rsn_cfg_wpa2_auth_suite;
                apicmdbuf_get_rsn_cfg_wpa2_auth_suite =
                    (APICMDBUF_GET_RSN_CFG_WPA2_AUTH_SUITE*)apicmdbuf;
                apicmdbuf_get_rsn_cfg_wpa2_auth_suite->cmdCode =
                    APCMD_GET_RSN_CFG_WPA2_AUTH_SUITE_REPLY;
                memcpy(apicmdbuf_get_rsn_cfg_wpa2_auth_suite->AuthSuites,
                       mib_RSNConfigWPA2AuthSuites_p->AuthSuites, 4);
                apicmdbuf_get_rsn_cfg_wpa2_auth_suite->Enabled = mib_RSNConfigWPA2AuthSuites_p->Enabled;
                apicmdbuf_get_rsn_cfg_wpa2_auth_suite->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_RSN_CFG_WPA2_UNICAST_CIPHER:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER* apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher;
                apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher =
                    (APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher->cmdCode = APCMD_SET_RSN_CFG_WPA2_UNICAST_CIPHER_REPLY;

                memcpy(mib_RSNConfigWPA2UnicastCiphers_p->UnicastCipher,
                       apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher->UnicastCipher, 4);
                mib_RSNConfigWPA2UnicastCiphers_p->Enabled =
                    apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher->Enabled;
                apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_RSN_CFG_WPA2_UNICAST_CIPHER:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER* apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher;
                apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher =
                    (APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER*)apicmdbuf;
                apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher->cmdCode = APCMD_GET_RSN_CFG_WPA2_UNICAST_CIPHER_REPLY;

                memcpy(apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher->UnicastCipher,
                       mib_RSNConfigWPA2UnicastCiphers_p->UnicastCipher, 4);
                apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher->Enabled = mib_RSNConfigWPA2UnicastCiphers_p->Enabled;
                apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_RSN_CFG_WPA2_UNICAST_CIPHER2:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER2) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER2* apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher2;
                apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher2 =
                    (APICMDBUF_SET_RSN_CFG_WPA2_UNICAST_CIPHER2*)apicmdbuf;
                apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher2->cmdCode = APCMD_SET_RSN_CFG_WPA2_UNICAST_CIPHER2_REPLY;

                memcpy(mib_RSNConfigWPA2UnicastCiphers2_p->UnicastCipher,
                       apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher2->UnicastCipher, 4);
                mib_RSNConfigWPA2UnicastCiphers2_p->Enabled =
                    apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher2->Enabled;
                apicmdbuf_set_rsn_cfg_wpa2_unicast_cipher2->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_GET_RSN_CFG_WPA2_UNICAST_CIPHER2:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER2) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER2* apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher2;
                apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher2 =
                    (APICMDBUF_GET_RSN_CFG_WPA2_UNICAST_CIPHER2*)apicmdbuf;
                apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher2->cmdCode = APCMD_GET_RSN_CFG_WPA2_UNICAST_CIPHER2_REPLY;

                memcpy(apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher2->UnicastCipher,
                       mib_RSNConfigWPA2UnicastCiphers2_p->UnicastCipher, 4);
                apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher2->Enabled = mib_RSNConfigWPA2UnicastCiphers2_p->Enabled;
                apicmdbuf_get_rsn_cfg_wpa2_unicast_cipher2->cmdStatus = APCTL_OK;
            }
        }
        break;


    case APCMD_GET_PAIRWISE_TSC:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_PAIRWISE_TSC) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_PAIRWISE_TSC* apicmdbuf_get_pairwise_tsc;
                apicmdbuf_get_pairwise_tsc = (APICMDBUF_GET_PAIRWISE_TSC*)apicmdbuf;
                apicmdbuf_get_pairwise_tsc->cmdCode = APCMD_GET_PAIRWISE_TSC_REPLY;

                if ( STATE_SUCCESS == extStaDb_GetPairwiseTSC(
                            (IEEEtypes_MacAddr_t *)apicmdbuf_get_pairwise_tsc->staMACAddr,
                            (UINT32 *)&(apicmdbuf_get_pairwise_tsc->TxIV32), &(apicmdbuf_get_pairwise_tsc->TxIV16)) )
                    apicmdbuf_get_pairwise_tsc->cmdStatus = APCTL_OK;
                else
                    apicmdbuf_get_pairwise_tsc->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;

    case APCMD_SET_PAIRWISE_TSC:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_PAIRWISE_TSC) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_PAIRWISE_TSC* apicmdbuf_set_pairwise_tsc;
                apicmdbuf_set_pairwise_tsc = (APICMDBUF_SET_PAIRWISE_TSC*)apicmdbuf;
                apicmdbuf_set_pairwise_tsc->cmdCode = APCMD_SET_PAIRWISE_TSC_REPLY;

                if ( STATE_SUCCESS == extStaDb_SetPairwiseTSC(
                            (IEEEtypes_MacAddr_t *)apicmdbuf_set_pairwise_tsc->staMACAddr,
                            apicmdbuf_set_pairwise_tsc->TxIV32, apicmdbuf_set_pairwise_tsc->TxIV16) )
                    apicmdbuf_set_pairwise_tsc->cmdStatus = APCTL_OK;
                else
                    apicmdbuf_set_pairwise_tsc->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;

    case APCMD_GET_GRP_TSC:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_GET_GRP_TSC) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_GRP_TSC* apicmdbuf_get_grp_tsc;
                apicmdbuf_get_grp_tsc = (APICMDBUF_GET_GRP_TSC*)apicmdbuf;
                apicmdbuf_get_grp_tsc->cmdCode = APCMD_GET_GRP_TSC_REPLY;

                apicmdbuf_get_grp_tsc->g_IV32 = g_IV32;
                apicmdbuf_get_grp_tsc->g_IV16 = g_IV16;

                apicmdbuf_get_grp_tsc->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_GRP_TSC:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_GRP_TSC) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_GRP_TSC* apicmdbuf_set_grp_tsc;
                apicmdbuf_set_grp_tsc = (APICMDBUF_SET_GRP_TSC*)apicmdbuf;
                apicmdbuf_set_grp_tsc->cmdCode = APCMD_SET_GRP_TSC_REPLY;


                g_IV32 = apicmdbuf_set_grp_tsc->g_IV32;
                g_IV16 = apicmdbuf_set_grp_tsc->g_IV16;
                apicmdbuf_set_grp_tsc->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_COUNTER_MEASURE_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_COUNTER_MEASURE_ENABLED* apicmdbuf_set_counter_measure_enabled;
                apicmdbuf_set_counter_measure_enabled = (APICMDBUF_SET_COUNTER_MEASURE_ENABLED*)apicmdbuf;
                apicmdbuf_set_counter_measure_enabled->cmdCode = APCMD_SET_COUNTER_MEASURE_ENABLED_REPLY;
                MICCounterMeasureEnabled = apicmdbuf_set_counter_measure_enabled->Enabled;
                apicmdbuf_set_counter_measure_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_RSN_PMK:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_RSN_PMK) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_RSN_PMK* apicmdbuf_set_rsn_pmk;
                apicmdbuf_set_rsn_pmk = (APICMDBUF_SET_RSN_PMK*)apicmdbuf;
                apicmdbuf_set_rsn_pmk->cmdCode = APCMD_SET_RSN_PMK_REPLY;

                // set ext sta db
                if ( STATE_SUCCESS == extStaDb_SetRSNPmk(
                            (IEEEtypes_MacAddr_t *)apicmdbuf_set_rsn_pmk->StaMacAddr,
                            apicmdbuf_set_rsn_pmk->PMK) )
                    apicmdbuf_set_rsn_pmk->cmdStatus = APCTL_OK;
                else
                    apicmdbuf_set_rsn_pmk->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;

    case APCMD_SET_WPA_PSK_VALUE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_WPA_PSK_VALUE) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_WPA_PSK_VALUE* apicmdbuf_set_wpa_psk_value;
                apicmdbuf_set_wpa_psk_value = (APICMDBUF_SET_WPA_PSK_VALUE*)apicmdbuf;
                apicmdbuf_set_wpa_psk_value->cmdCode = APCMD_SET_WPA_PSK_VALUE_REPLY;

                mib_WPAPSKValueEnabled = apicmdbuf_set_wpa_psk_value->PSKValueEnabled;
                memcpy(mib_RSNConfig_p->PSKValue, 
                       apicmdbuf_set_wpa_psk_value->PSKValue, 32);
                apicmdbuf_set_wpa_psk_value->cmdStatus = APCTL_OK;
            }
        }
        break;
    case APCMD_SET_WPA2_PSK_VALUE:
        {
            unsigned short expectedLen = sizeof(APICMDBUF_SET_WPA2_PSK_VALUE) - 4 * sizeof(unsigned short);
            if (apicmdbuf->cmdBodyLen < expectedLen )
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = expectedLen;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_WPA2_PSK_VALUE* apicmdbuf_set_wpa2_psk_value;
                apicmdbuf_set_wpa2_psk_value = (APICMDBUF_SET_WPA2_PSK_VALUE*)apicmdbuf;
                apicmdbuf_set_wpa2_psk_value->cmdCode = APCMD_SET_WPA2_PSK_VALUE_REPLY;

                mib_WPA2PSKValueEnabled = apicmdbuf_set_wpa2_psk_value->PSKValueEnabled;
                memcpy(mib_RSNConfigWPA2_p->PSKValue, 
                       apicmdbuf_set_wpa2_psk_value->PSKValue, 32);
                apicmdbuf_set_wpa2_psk_value->cmdStatus = APCTL_OK;
            }
        }
        break;

#endif
    case APCMD_GET_CLOCK:
        if ( apicmdbuf->cmdBodyLen < 4 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return FAIL;
        }
        else
        {
            UINT16 cpu, sys;
            APICMDBUF_GET_CLOCK * apicmdbuf_get_CLOCK;
            apicmdbuf_get_CLOCK = (APICMDBUF_GET_CLOCK *)apicmdbuf;
            apicmdbuf_get_CLOCK->cmdCode = APCMD_GET_CLOCK_REPLY;
            hal_GetClock(&cpu, &sys);
            apicmdbuf_get_CLOCK->cpuFreq = Index2CMHz(cpu);
            apicmdbuf_get_CLOCK->sysFreq = Index2CMHz(sys);
            apicmdbuf_get_CLOCK->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_CLOCK:
        /* currently supports only mixed mode */
        if ( apicmdbuf->cmdBodyLen < 4 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_CLOCK* apicmdbuf_set_CLOCK;
            apicmdbuf_set_CLOCK = (APICMDBUF_SET_CLOCK*)apicmdbuf;
            apicmdbuf_set_CLOCK->cmdCode = APCMD_SET_CLOCK_REPLY;
            hal_SetClock(CMHz2Index(apicmdbuf_set_CLOCK->cpuFreq), CMHz2Index(apicmdbuf_set_CLOCK->sysFreq));
            apicmdbuf_set_CLOCK->cmdStatus = APCTL_OK;
        }
        break;

#ifdef CONDOR

    case APCMD_GET_AP_MODE:
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_AP_MODE* apicmdbuf_get_ap_mode;
            apicmdbuf_get_ap_mode = (APICMDBUF_GET_AP_MODE *)apicmdbuf;
            apicmdbuf_get_ap_mode->cmdCode = APCMD_GET_AP_MODE_REPLY;

            apicmdbuf_get_ap_mode->apMode = mib_ApMode;
            apicmdbuf_get_ap_mode->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_SET_AP_MODE:
        /* currently supports only mixed mode */
        if (apicmdbuf->cmdBodyLen < 4)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 4;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_AP_MODE* apicmdbuf_set_ap_mode;
            apicmdbuf_set_ap_mode = (APICMDBUF_SET_AP_MODE *)apicmdbuf;
            apicmdbuf_set_ap_mode->cmdCode = APCMD_SET_AP_MODE_REPLY;
            mib_ApMode = apicmdbuf_set_ap_mode->apMode;
            if (mib_ApMode == AP_MODE_B_ONLY  || mib_ApMode == AP_MODE_A_ONLY)
                mib_ErpProtEnabled = FALSE;
            else
                mib_ErpProtEnabled = TRUE;
            apicmdbuf_set_ap_mode->cmdStatus = APCTL_OK;
        }

        break;

#endif

    case APCMD_GET_BURST_MODE:
        /* currently supports only mixed mode */
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_BURST_MODE * apicmdbuf_get_burst_mode;
            apicmdbuf_get_burst_mode = (APICMDBUF_GET_BURST_MODE *)apicmdbuf;
            apicmdbuf_get_burst_mode->cmdCode = APCMD_GET_BURST_MODE_REPLY;
            apicmdbuf_get_burst_mode->BurstMode = mib_burstmode;
            apicmdbuf_get_burst_mode->BurstRate = mib_burstrate;

            apicmdbuf_get_burst_mode->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_SET_BURST_MODE:
        /* currently supports only mixed mode */
        if (apicmdbuf->cmdBodyLen < 2)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_BURST_MODE * apicmdbuf_set_burst_mode;
            apicmdbuf_set_burst_mode = (APICMDBUF_SET_BURST_MODE *)apicmdbuf;
            apicmdbuf_set_burst_mode->cmdCode = APCMD_SET_BURST_MODE_REPLY;
            mib_burstmode = apicmdbuf_set_burst_mode->BurstMode;
            mib_burstrate = apicmdbuf_set_burst_mode->BurstRate;

            apicmdbuf_set_burst_mode->cmdStatus = APCTL_OK;
        }

        break;

    case APCMD_GUI_SET_PRIVACY_TBL_RSN_ENABLED:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED* apicmdbuf_set_privacy_tbl_rsn_enabled;
            apicmdbuf_set_privacy_tbl_rsn_enabled = (APICMDBUF_SET_PRIVACY_TBL_RSN_ENABLED*)apicmdbuf;
            apicmdbuf_set_privacy_tbl_rsn_enabled->cmdCode = APCMD_SET_PRIVACY_TBL_RSN_ENABLED_REPLY;
            if (mib_PrivacyTable_p->RSNEnabled == FALSE)
            {
                if (apicmdbuf_set_privacy_tbl_rsn_enabled->Enabled == TRUE)
                {
                    //
                    //Since system will reboot
                    //no need to set mib_PrivacyTable_p->RSNEnabled = TRUE, 
                    // if set to TRUE, FreeRxDataBuf in Eaglehw.c will get 
                    // crash easily
                    //
                    os_EnterCriticalSection;
                    mib_PrivacyTable_p->ApiUpdateWpa = TRUE;
                    os_ExitCriticalSection;   
                }
                else
                {
                    mib_PrivacyTable_p->RSNEnabled = apicmdbuf_set_privacy_tbl_rsn_enabled->Enabled; 

                }                       
            }
            else
            {
                mib_PrivacyTable_p->RSNEnabled = apicmdbuf_set_privacy_tbl_rsn_enabled->Enabled;
            }                   

            apicmdbuf_set_privacy_tbl_rsn_enabled->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_FIRMWARE_VERSION:
        apicmdbuf->cmdCode = APCMD_GET_FIRMWARE_VERSION_REPLY;

        if (apicmdbuf->cmdBodyLen < 60)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 60;
            return FAIL;
        }
        else
        {
            util_CopyList(apicmdbuf->cmdBody, FIRMWARE_VERSION, 60);
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;
#ifdef FORCE_ERP_PROTECTION_SUPPORT
	case APCMD_GET_FORCE_PROTECTION_MODE:
	      if (apicmdbuf->cmdBodyLen < 2)
       	 {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 2;
            return FAIL;
        	}
        	else
        	{
          	  APICMDBUF_GET_FORCE_PROTECTION_MODE * apicmdbuf_get_force_protection_mode;
            apicmdbuf_get_force_protection_mode = (APICMDBUF_GET_FORCE_PROTECTION_MODE *)apicmdbuf;
            apicmdbuf_get_force_protection_mode->cmdCode = APCMD_GET_FORCE_PROTECTION_MODE_REPLY;

            apicmdbuf_get_force_protection_mode->forceProtectMode = mib_forceProtectiondisable;
            apicmdbuf_get_force_protection_mode->cmdStatus = APCTL_OK;
        }

        break;
	    case APCMD_SET_FORCE_PROTECTION_MODE:
        	/* currently supports only mixed mode */
        	if (apicmdbuf->cmdBodyLen < 2)
        	{
            	apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            	apicmdbuf->cmdBodyLen = 2;
            return OS_FAIL;
        	}
        	else
        	{
            	APICMDBUF_SET_FORCE_PROTECTION_MODE* apicmdbuf_set_force_protection_mode;
            	apicmdbuf_set_force_protection_mode = (APICMDBUF_SET_FORCE_PROTECTION_MODE*)apicmdbuf;
            	apicmdbuf_set_force_protection_mode->cmdCode = APCMD_SET_AP_MODE_REPLY;
            	mib_forceProtectiondisable= apicmdbuf_set_force_protection_mode->forceProtectMode;
            	apicmdbuf_set_force_protection_mode->cmdStatus = APCTL_OK;
        	}

        break;


#endif
#ifdef WDS_FEATURE
        case APCMD_GET_WDS_MODE:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_GET_WDS_MODE_REPLY;
                return FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                apicmdbuf->cmdBody[0] = mib_wdsEnable;
		   		apicmdbuf->cmdCode = APCMD_GET_WDS_MODE_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
	      break;
        case APCMD_SET_WDS_MODE:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_WDS_MODE_REPLY;
                return FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                mib_wdsEnable = apicmdbuf->cmdBody[0];
                apicmdbuf->cmdCode = APCMD_SET_WDS_MODE_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
		break;
        case APCMD_GET_WDS_LIST:
            if (apicmdbuf->cmdBodyLen < 2 + mib_wdsno * 6)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 2 + mib_wdsno * 6;
                apicmdbuf->cmdCode = APCMD_GET_WDS_LIST_REPLY;
                return FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                if(mib_wdsEnable != WDS_MODE_DYNAMIC)
                {
                apicmdbuf->cmdBody[0] = mib_wdsno;
                memcpy(&apicmdbuf->cmdBody[2], mib_wdsmac, mib_wdsno*6);
                }
                else
                {
                    apicmdbuf->cmdBody[0] = getDynamicWdsMac(&apicmdbuf->cmdBody[2]); 
                }
                apicmdbuf->cmdCode = APCMD_GET_WDS_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
		break;

        case APCMD_SET_WDS_LIST:
            if (apicmdbuf->cmdBodyLen < 2)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 2;
                apicmdbuf->cmdCode = APCMD_SET_WDS_LIST_REPLY;
                return FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                if(mib_wdsEnable == 1)
                {
                mib_wdsno = apicmdbuf->cmdBody[0];
                memcpy(mib_wdsmac, &apicmdbuf->cmdBody[2],mib_wdsno*6);
                }
                apicmdbuf->cmdCode = APCMD_SET_WDS_LIST_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
#ifndef NO_GUI
                SetUserConfig();
#endif
               // Reset();
            }
		break;
        case APCMD_GET_DISABLE_ASSOC:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_GET_DISABLE_ASSOC_REPLY;
                return FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                apicmdbuf->cmdBody[0] = mib_disableAssoc;
		   		apicmdbuf->cmdCode = APCMD_GET_DISABLE_ASSOC_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
	        break;

        case APCMD_SET_DISABLE_ASSOC:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_DISABLE_ASSOC_REPLY;
                return FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                mib_disableAssoc = apicmdbuf->cmdBody[0];
                apicmdbuf->cmdCode = APCMD_SET_DISABLE_ASSOC_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
		    break;
        case APCMD_GET_WDS_BCAST_MODE:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_GET_WDS_BCAST_MODE_REPLY;
                return FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                apicmdbuf->cmdBody[0] = mib_wdsBcastMode;
		   		apicmdbuf->cmdCode = APCMD_GET_WDS_BCAST_MODE_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
	        break;

        case APCMD_SET_WDS_BCAST_MODE:
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                apicmdbuf->cmdCode = APCMD_SET_WDS_BCAST_MODE_REPLY;
                return FAIL;
            }
            else
            {
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                mib_wdsBcastMode = apicmdbuf->cmdBody[0];
                apicmdbuf->cmdCode = APCMD_SET_WDS_BCAST_MODE_REPLY;
                apicmdbuf->cmdStatus = APCTL_OK;
            }
		    break;
#endif // WDS_FEATURE

#ifdef AP_URPTR
        case APCMD_GET_UREPEATER:
            {
                if (apicmdbuf->cmdBodyLen < sizeof(APICMDBUF_GET_UREPEATER) - 4*sizeof(unsigned short))
                {
                    apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                    apicmdbuf->cmdBodyLen = sizeof(APICMDBUF_GET_UREPEATER) - 4*sizeof(unsigned short);
                    apicmdbuf->cmdCode = APCMD_GET_UREPEATER_REPLY;
                    return OS_FAIL;
                }
                else
                {
                    APICMDBUF_GET_UREPEATER *apicmdbuf_get_ur;
                    apicmdbuf_get_ur = (APICMDBUF_GET_UREPEATER *)apicmdbuf; 
                    apicmdbuf_get_ur->urMode = mib_urMode;
                    apicmdbuf_get_ur->wbMode = mib_wbMode;
                    apicmdbuf_get_ur->urRsvd = mib_urRsvd;                    
                    util_CopyList(apicmdbuf_get_ur->urSsid, mib_urSsid, IEEEtypes_SSID_SIZE);
                    memcpy(apicmdbuf_get_ur->urBssid, mib_urBssid, 6);
                    apicmdbuf_get_ur->urPrefBssid = mib_urPrefBssid;
                    apicmdbuf_get_ur->cmdCode = APCMD_GET_UREPEATER_REPLY;
                    apicmdbuf_get_ur->cmdStatus = APCTL_OK;
                }
                break;
            }
        case APCMD_SET_UREPEATER:

            {
                if (apicmdbuf->cmdBodyLen < sizeof(APICMDBUF_SET_UREPEATER) - 4*sizeof(unsigned short))
                {
                    apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                    apicmdbuf->cmdBodyLen = sizeof(APICMDBUF_SET_UREPEATER) - 4*sizeof(unsigned short);
                    apicmdbuf->cmdCode = APCMD_SET_UREPEATER_REPLY;
                    return OS_FAIL;
                }
                else
                {
                    APICMDBUF_SET_UREPEATER *apicmdbuf_set_ur;
                    apicmdbuf_set_ur = (APICMDBUF_SET_UREPEATER *)apicmdbuf;
#ifdef W8515
                    if ( !msi_wl_CheckURSupport() )
                    {
                        /* Wrong chip version! */
                        /* UR support available only on 8515. */
                        apicmdbuf_set_ur->cmdCode = APCMD_SET_UREPEATER_REPLY;
                        apicmdbuf_set_ur->cmdStatus = APCTL_ERROR;
                    }
                    else
#endif /* W8515 */
                    {
                        mib_urModeConfig = 0;
                        mib_urMode = apicmdbuf_set_ur->urMode;
                        mib_wbMode = apicmdbuf_set_ur->wbMode;
                        mib_urRsvd = apicmdbuf_set_ur->urRsvd;
                        util_CopyList(mib_urSsid, apicmdbuf_set_ur->urSsid, IEEEtypes_SSID_SIZE);
                        util_CopyList(mib_StaCfg_ur.DesiredSsId, apicmdbuf_set_ur->urSsid, IEEEtypes_SSID_SIZE); 
                        memcpy(mib_urBssid, apicmdbuf_set_ur->urBssid,  6);
                        mib_urPrefBssid = apicmdbuf_set_ur->urPrefBssid;
                        apicmdbuf_set_ur->cmdCode = APCMD_SET_UREPEATER_REPLY;
                        apicmdbuf_set_ur->cmdStatus = APCTL_OK;
                    }
                }
                break;
            }
        case APCMD_GET_UREPEATER_LINK:
            {
#if 0 /* #ifdef APCFGUR -- The UR rate adapt had changed; This info can be obtained this from AP side. */
				extern rate_AssocReqData_t AdaptSta;
#endif
                int urLink;
                unsigned char urBssid[6];

                if (apicmdbuf->cmdBodyLen < sizeof(APICMDBUF_GET_UREPEATER_LINK) - 4*sizeof(unsigned short))
                {
                    apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                    apicmdbuf->cmdBodyLen = sizeof(APICMDBUF_GET_UREPEATER_LINK) - 4*sizeof(unsigned short);
                    apicmdbuf->cmdCode = APCMD_GET_UREPEATER_LINK_REPLY;
                    return OS_FAIL;
                }
                else
                {
                    APICMDBUF_GET_UREPEATER_LINK *apicmdbuf_get_urLink;
                    apicmdbuf_get_urLink = (APICMDBUF_GET_UREPEATER_LINK *)apicmdbuf; 

                    if( (urAssocState(&urLink,urBssid) == 0) /* 0==ok */
                        && urLink == 1 )
                    {
                        apicmdbuf_get_urLink->urLink = UR_CONNECTED;
                        memcpy(apicmdbuf_get_urLink->urBssid, urBssid, 6);
                    }
                    else
                    {
                        apicmdbuf_get_urLink->urLink = UR_DISCONNECTED;
                        memset(apicmdbuf_get_urLink->urBssid,0,6);
                    }
#if 0 /* #ifdef APCFGUR -- The UR rate adapt had changed; This info can be obtained this from AP side. */
					apicmdbuf_get_urLink->RateToBeUsedForTx = AdaptSta.RateToBeUsedForTx;
					apicmdbuf_get_urLink->AvgRSSI = AdaptSta.AvgRSSI;
#endif
                    apicmdbuf_get_urLink->cmdCode = APCMD_GET_UREPEATER_LINK_REPLY;
                    apicmdbuf_get_urLink->cmdStatus = APCTL_OK;
                }
                break;
            }
#ifdef UDP_API
    case APCMD_GET_IP_MAC_TABLE:
        {
            UINT16 entries;
            UINT16 minbodysize;

			entries = NumberOfIPMacTableEntries();
			minbodysize = 2 + entries * 10;
            if (apicmdbuf->cmdBodyLen < minbodysize && apicmdbuf->cmdBodyLen < MAX_API_DATA_THRESHOLD
                && apicmdbuf->cmdStatus != APCTL_MORE_DATA)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = minbodysize;
                apicmdbuf->cmdCode = APCMD_GET_IP_MAC_TABLE_REPLY;
                return FAIL;
            }
            else
            {
            	APICMDBUF_IPMAC_TABLE	*apicmdbuf_ipmac_table; 
#ifdef APCFGUR
				apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
                apicmdbuf_ipmac_table = (APICMDBUF_IPMAC_TABLE *)apicmdbuf;
                apicmdbuf->cmdCode = APCMD_GET_IP_MAC_TABLE_REPLY;
				if(apicmdbuf->cmdStatus != APCTL_MORE_DATA)
				{
					apicmdbuf_ipmac_table->NoOfEntries = entries;
					if(minbodysize > apicmdbuf->cmdBodyLen)
					{
                		if(apibuf){
							free(apibuf);
							apibuf = NULL;
                		}
                		apibuf = malloc(minbodysize);
						GetIPMacTable(apibuf);
                		apicmdbuf->cmdBodyLen = MAX_API_DATA_THRESHOLD;					
						memcpy(apicmdbuf->cmdBody+2, apibuf, apicmdbuf->cmdBodyLen);
                		apicmdbuf->cmdStatus = APCTL_MORE_DATA;
						apidataleft = minbodysize - MAX_API_DATA_THRESHOLD;
						apidataindex = MAX_API_DATA_THRESHOLD;
					}
					else
					{
                		apicmdbuf->cmdBodyLen = minbodysize;
						apicmdbuf_ipmac_table->NoOfEntries = GetIPMacTable(apicmdbuf->cmdBody+2);
                		apicmdbuf->cmdStatus = APCTL_OK;
					}
				}
				else
				{
					memcpy(apicmdbuf->cmdBody + apidataindex, apibuf + apidataindex, apicmdbuf->cmdBodyLen);					
					apidataindex += apicmdbuf->cmdBodyLen;
					if(apicmdbuf->cmdBodyLen < apidataleft)
					{
                		apicmdbuf->cmdStatus = APCTL_MORE_DATA;
						apidataleft -= apicmdbuf->cmdBodyLen;							
					}
					else
					{
                		apicmdbuf->cmdStatus = APCTL_LAST_DATA;
						free(apibuf);
						apibuf = NULL;
					}	
				} 
            }
        }
        break;
#endif
#endif /* AP_URPTR */

#ifdef SELF_CTS
    case APCMD_GET_SELF_CTS:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_SELF_CTS * apicmdbuf_get_SELF_CTS;
            apicmdbuf_get_SELF_CTS = (APICMDBUF_GET_SELF_CTS *)apicmdbuf;
            apicmdbuf_get_SELF_CTS->cmdCode = APCMD_GET_SELF_CTS_REPLY;

            apicmdbuf_get_SELF_CTS->SelfCts = mib_StaCfg_p->SelfCts;
            apicmdbuf_get_SELF_CTS->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_SELF_CTS:
        /* currently supports only mixed mode */
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_SELF_CTS* apicmdbuf_set_SELF_CTS;
            apicmdbuf_set_SELF_CTS = (APICMDBUF_SET_SELF_CTS*)apicmdbuf;
            apicmdbuf_set_SELF_CTS->cmdCode = APCMD_SET_SELF_CTS_REPLY;
            mib_StaCfg_p->SelfCts= apicmdbuf_set_SELF_CTS->SelfCts;
            apicmdbuf_set_SELF_CTS->cmdStatus = APCTL_OK;
        }
        break;
#endif
#ifdef BOOSTER_MODE
    case APCMD_GET_BOOSTER_MODE:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_BOOSTER_MODE * apicmdbuf_get_BOOSTER_MODE;
            apicmdbuf_get_BOOSTER_MODE = (APICMDBUF_GET_BOOSTER_MODE *)apicmdbuf;
            apicmdbuf_get_BOOSTER_MODE->cmdCode = APCMD_GET_BOOSTER_MODE;

            apicmdbuf_get_BOOSTER_MODE->BoosterMode = mib_StaCfg_p->mib_BoosterMode;
            apicmdbuf_get_BOOSTER_MODE->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_BOOSTER_MODE:
        /* currently supports only mixed mode */
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_BOOSTER_MODE * apicmdbuf_set_BOOSTER_MODE;
            apicmdbuf_set_BOOSTER_MODE = (APICMDBUF_SET_BOOSTER_MODE*)apicmdbuf;
            apicmdbuf_set_BOOSTER_MODE->cmdCode = APCMD_SET_BOOSTER_MODE;
            mib_StaCfg_p->mib_BoosterMode= apicmdbuf_set_BOOSTER_MODE->BoosterMode;
            apicmdbuf_set_BOOSTER_MODE->cmdStatus = APCTL_OK;
        }
        break;

#endif

#ifdef QOS_FEATURE
    case APCMD_GET_QOS_MODE:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_QOS_MODE* apicmdbuf_get_qos_mode;
                apicmdbuf_get_qos_mode = (APICMDBUF_GET_QOS_MODE*)apicmdbuf;
                apicmdbuf_get_qos_mode->cmdCode = APCMD_GET_QOS_MODE_REPLY;
			    apicmdbuf_get_qos_mode->qos_mode = mib_StaCfg_p->QoSOptImpl; 
                apicmdbuf_get_qos_mode->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_QOS_MODE:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_QOS_MODE* apicmdbuf_set_qos_mode;
                apicmdbuf_set_qos_mode = (APICMDBUF_SET_QOS_MODE*)apicmdbuf;
                apicmdbuf_set_qos_mode->cmdCode = APCMD_SET_QOS_MODE_REPLY;
			    mib_StaCfg_p->QoSOptImpl = apicmdbuf_set_qos_mode->qos_mode; 
                apicmdbuf_set_qos_mode->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_GET_NOACK_MODE:

        {

            if (apicmdbuf->cmdBodyLen < 1)

            {

                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;

                apicmdbuf->cmdBodyLen = 1;

                return FAIL;

            }

            else

            {

                APICMDBUF_GET_NOACK_MODE* apicmdbuf_get_noack_mode;

                apicmdbuf_get_noack_mode = (APICMDBUF_GET_NOACK_MODE*)apicmdbuf;

                apicmdbuf_get_noack_mode->cmdCode = APCMD_GET_NOACK_MODE_REPLY;

			    apicmdbuf_get_noack_mode->noack_mode = mib_StaCfg_p->NoAckMode; 

                apicmdbuf_get_noack_mode->cmdStatus = APCTL_OK;

            }

        }

        break;



    case APCMD_SET_NOACK_MODE:

        {

            if (apicmdbuf->cmdBodyLen < 1)

            {

                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;

                apicmdbuf->cmdBodyLen = 1;

                return FAIL;

            }

            else

            {

                APICMDBUF_SET_NOACK_MODE* apicmdbuf_set_noack_mode;

                apicmdbuf_set_noack_mode = (APICMDBUF_SET_NOACK_MODE*)apicmdbuf;

                apicmdbuf_set_noack_mode->cmdCode = APCMD_SET_NOACK_MODE_REPLY;

			    mib_StaCfg_p->NoAckMode = apicmdbuf_set_noack_mode->noack_mode; 

                apicmdbuf_set_noack_mode->cmdStatus = APCTL_OK;

            }

        }

        break;
    case APCMD_GET_QOS_TS:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_QOS_TS_INFO* apicmdbuf_get_qos_ts;
                apicmdbuf_get_qos_ts = (APICMDBUF_GET_QOS_TS_INFO*)apicmdbuf;
                apicmdbuf_get_qos_ts->cmdCode = APCMD_GET_QOS_TS_REPLY;
			    apicmdbuf_get_qos_ts->maxEntry = MAX_QOS_STRMS; 
                apicmdbuf_get_qos_ts->pInfo = (UINT8*)TSpecEntryDb;
                apicmdbuf_get_qos_ts->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_EDCA_PARAM:
        {
#ifdef PEREGRINE   /* shouldn't configure EDCA paramters when QOS is disabled */
            if(mib_StaCfg_p->QoSOptImpl == 0)
            {
                apicmdbuf->cmdStatus = APCTL_BAD_CMD;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
                
            }
#endif            
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_EDCA_PARAM* apicmdbuf_set_edca_param;
                apicmdbuf_set_edca_param = (APICMDBUF_SET_EDCA_PARAM*)apicmdbuf;
                if(apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx <= 3)
                {
                    mib_QAPEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblCWmin =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblCWmin;
                    mib_QAPEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblCWmax =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblCWmax;
                    mib_QAPEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblAIFSN =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblAIFSN;
                    mib_QAPEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblTXOPLimit =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblTXOPLimit;
                     mib_QAPEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblTXOPLimitBAP =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblTXOPLimitBAP;
                }
                else
                {
                    mib_QStaEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblCWmin =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblCWmin;
                    mib_QStaEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblCWmax =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblCWmax;
                    mib_QStaEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblAIFSN =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblAIFSN;
                    mib_QStaEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblTXOPLimit =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblTXOPLimit;
                      mib_QStaEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblTXOPLimitBSta =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblTXOPLimitBAP;
                    mib_QStaEDCATable[apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblMandatory =
                        apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblMandatory;

                }
                if((apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx == 3)
                    || (apicmdbuf_set_edca_param->QAPEDCATable.QAPEDCATblIndx == 7))
                {
                    if(QoS_UpdateEDCAParameters())
                        apicmdbuf_set_edca_param->cmdStatus = APCTL_OK;
                    else
                        apicmdbuf_set_edca_param->cmdStatus = APCTL_CMD_FAILED;
                }
                else
                   apicmdbuf_set_edca_param->cmdStatus = APCTL_OK;
                    
                apicmdbuf_set_edca_param->cmdCode = APCMD_SET_EDCA_PARAM_REPLY;
            }
        }
        break;
    case APCMD_GET_EDCA_PARAM:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_EDCA_PARAM* apicmdbuf_get_edca_param;
                apicmdbuf_get_edca_param = (APICMDBUF_GET_EDCA_PARAM*)apicmdbuf;
               if(((APICMDBUF_GET_EDCA_PARAM*)apicmdbuf)->QAPEDCATable.QAPEDCATblIndx <= 3)
               {
               
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblCWmin = 
					mib_QAPEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblCWmin;
                    
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblCWmax = 
					mib_QAPEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblCWmax;
                    
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblAIFSN = 
					mib_QAPEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblAIFSN;
                    
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblTXOPLimit = 
					mib_QAPEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblTXOPLimit;
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblTXOPLimitBAP = 
					mib_QAPEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx].QAPEDCATblTXOPLimitBAP;

               }
               else
               {
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblCWmin = 
					mib_QStaEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblCWmin;
                    
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblCWmax = 
					mib_QStaEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblCWmax;
                    
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblAIFSN = 
					mib_QStaEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblAIFSN;
                    
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblTXOPLimit = 
					mib_QStaEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblTXOPLimit;
                apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblTXOPLimitBAP = 
                   mib_QStaEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblTXOPLimitBSta;


				apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblMandatory = 
					mib_QStaEDCATable[apicmdbuf_get_edca_param->QAPEDCATable.QAPEDCATblIndx - 4].QStaEDCATblMandatory;
               }
                
               
                    apicmdbuf_get_edca_param->cmdStatus = APCTL_OK;
                
                   
                apicmdbuf_get_edca_param->cmdCode = APCMD_GET_EDCA_PARAM_REPLY;
            }
        }
        break;

#endif

    case APCMD_GET_DEVICE_NAME:
        if (apicmdbuf->cmdBodyLen < 16)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 16;
            return OS_FAIL;
        }
        else
        {
#ifdef APCFGUR
			apicmdbuf->cmdBody = (UINT8 *)apicmdbuf + APICMDHEADERLEN;
#endif
            apicmdbuf->cmdCode =APCMD_GET_DEVICE_NAME_REPLY;
            util_CopyList(apicmdbuf->cmdBody, "APConfig", 16);
            apicmdbuf->cmdStatus = APCTL_OK;
        }

        break;

#ifdef AP_SITE_SURVEY
    case APCMD_SET_SURVEY_SCAN:
        {

            apicmdbuf->cmdCode = APCMD_SET_SURVEY_SCAN_REPLY;        
            memset((UINT8 *)&siteSurveyInfo, 0, sizeof(API_SURVEY_ENTRY) * SITE_SURVEY_ENTRY_MAX);
			SetupScan();
            apicmdbuf->cmdStatus = APCTL_OK;
        }
        break;
    
    case APCMD_GET_SURVEY_INFO:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_SURVEY_INFO* apicmdbuf_get_survey;
            apicmdbuf_get_survey = (APICMDBUF_GET_SURVEY_INFO *)apicmdbuf;
            apicmdbuf_get_survey->cmdCode =APCMD_GET_SURVEY_INFO_REPLY;

            if(busyScanning)
            {
                apicmdbuf_get_survey->busyScan = 1;
            }
            else
            {
                apicmdbuf_get_survey->busyScan = 0;
            }
            apicmdbuf_get_survey->maxEntry = SITE_SURVEY_ENTRY_MAX;
            apicmdbuf_get_survey->infoAddr = (UINT8 *)&siteSurveyInfo;
            apicmdbuf_get_survey->cmdStatus = APCTL_OK;
        }
        break;
#endif /* AP_SITE_SURVEY */

    case APCMD_GET_WPA2_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_WPA2_ENABLED* apicmdbuf_get_wpa2_enabled;
                apicmdbuf_get_wpa2_enabled = (APICMDBUF_GET_WPA2_ENABLED*)apicmdbuf;
                apicmdbuf_get_wpa2_enabled->cmdCode = APCMD_GET_WPA2_ENABLED_REPLY;
				apicmdbuf_get_wpa2_enabled->Enabled = mib_RSNConfigWPA2_p->WPA2Enabled; 
                apicmdbuf_get_wpa2_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_WPA2_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_WPA2_ENABLED* apicmdbuf_set_wpa2_enabled;
                apicmdbuf_set_wpa2_enabled = (APICMDBUF_SET_WPA2_ENABLED*)apicmdbuf;
                apicmdbuf_set_wpa2_enabled->cmdCode = APCMD_SET_WPA2_ENABLED_REPLY;
				mib_RSNConfigWPA2_p->WPA2Enabled = apicmdbuf_set_wpa2_enabled->Enabled; 
                apicmdbuf_set_wpa2_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_GET_WPA2ONLY_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_GET_WPA2ONLY_ENABLED* apicmdbuf_get_wpa2only_enabled;
                apicmdbuf_get_wpa2only_enabled = (APICMDBUF_GET_WPA2ONLY_ENABLED*)apicmdbuf;
                apicmdbuf_get_wpa2only_enabled->cmdCode = APCMD_GET_WPA2ONLY_ENABLED_REPLY;
				apicmdbuf_get_wpa2only_enabled->Enabled = mib_RSNConfigWPA2_p->WPA2OnlyEnabled; 
                apicmdbuf_get_wpa2only_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_WPA2ONLY_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_WPA2ONLY_ENABLED* apicmdbuf_set_wpa2only_enabled;
                apicmdbuf_set_wpa2only_enabled = (APICMDBUF_SET_WPA2ONLY_ENABLED*)apicmdbuf;
                apicmdbuf_set_wpa2only_enabled->cmdCode = APCMD_SET_WPA2ONLY_ENABLED_REPLY;
				mib_RSNConfigWPA2_p->WPA2OnlyEnabled = apicmdbuf_set_wpa2only_enabled->Enabled; 
                apicmdbuf_set_wpa2only_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;

    case APCMD_SET_WPA2_PREAUTH_ENABLED:
        {
            if (apicmdbuf->cmdBodyLen < 1)
            {
                apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen = 1;
                return FAIL;
            }
            else
            {
                APICMDBUF_SET_WPA2_PREAUTH_ENABLED* apicmdbuf_set_wpa2_preauth_enabled;
                apicmdbuf_set_wpa2_preauth_enabled = (APICMDBUF_SET_WPA2_PREAUTH_ENABLED*)apicmdbuf;
                apicmdbuf_set_wpa2_preauth_enabled->cmdCode = APCMD_SET_WPA2_PREAUTH_ENABLED_REPLY;
				mib_RSNConfigWPA2_p->WPA2PreAuthEnabled = apicmdbuf_set_wpa2_preauth_enabled->Enabled; 
                apicmdbuf_set_wpa2_preauth_enabled->cmdStatus = APCTL_OK;
            }
        }
        break;
        
#if defined(GATEWAY) || defined(DHCPS)
    case APCMD_GET_ETH_LAN_IP_CONFIG:
        if (apicmdbuf->cmdBodyLen < 8) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_ETH_LAN_IP_CONFIG* apicmdbuf_get_lan_ip_config;
            apicmdbuf_get_lan_ip_config = (APICMDBUF_GET_ETH_LAN_IP_CONFIG *)apicmdbuf;
            apicmdbuf_get_lan_ip_config->cmdCode = APCMD_GET_ETH_LAN_IP_CONFIG_REPLY;

            apicmdbuf_get_lan_ip_config->subnetMask = inet_address(mib_IP_LAN.SubnetMask);
            apicmdbuf_get_lan_ip_config->ipAddress = inet_address(mib_IP_LAN.IPAddr);

            apicmdbuf_get_lan_ip_config->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_ETH_LAN_IP_CONFIG:
        if (apicmdbuf->cmdBodyLen < 8) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_SET_ETH_LAN_IP_CONFIG* apicmdbuf_set_lan_ip_config;
            apicmdbuf_set_lan_ip_config = (APICMDBUF_SET_ETH_LAN_IP_CONFIG *)apicmdbuf;
            apicmdbuf_set_lan_ip_config->cmdCode = APCMD_SET_ETH_LAN_IP_CONFIG_REPLY;

            mib_IP_LAN.SubnetMask = inet_address(apicmdbuf_set_lan_ip_config->subnetMask);
            mib_IP_LAN.IPAddr = inet_address(apicmdbuf_set_lan_ip_config->ipAddress);

            apicmdbuf_set_lan_ip_config->cmdStatus = APCTL_OK;

#if defined(DHCPS)
            /* notify DHCP Server about new IP */
            dhcpsSetIpAddress(mib_IP_LAN.IPAddr, mib_IP_LAN.SubnetMask);
#endif
        }
        break;
#if defined(GATEWAY)
    case APCMD_GET_ETH_WAN_IP_CONFIG:
        if (apicmdbuf->cmdBodyLen < 24) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 24;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_ETH_WAN_IP_CONFIG* apicmdbuf_get_wan_ip_config;
            apicmdbuf_get_wan_ip_config = (APICMDBUF_GET_ETH_WAN_IP_CONFIG *)apicmdbuf;
            apicmdbuf_get_wan_ip_config->cmdCode = APCMD_GET_ETH_WAN_IP_CONFIG_REPLY;

            apicmdbuf_get_wan_ip_config->dhcpEnabled = mib_dhcpenable;

            apicmdbuf_get_wan_ip_config->subnetMask = inet_address(mib_DHCP.SubnetMask);
            apicmdbuf_get_wan_ip_config->ipAddress = inet_address(mib_DHCP.IPAddr);
            apicmdbuf_get_wan_ip_config->gatewayAddress = inet_address(mib_DHCP.GwyAddr);
            apicmdbuf_get_wan_ip_config->primaryDNSAddress = inet_address(mib_DHCP.PrimaryDNS);
            apicmdbuf_get_wan_ip_config->secondaryDNSAddress = inet_address(mib_DHCP.SecondaryDNS);
            apicmdbuf_get_wan_ip_config->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_ETH_WAN_IP_CONFIG:
        if (apicmdbuf->cmdBodyLen < 24) /** size of mac address **/
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 24;
            return OS_FAIL;
        }
        else
        {

            APICMDBUF_SET_ETH_WAN_IP_CONFIG* apicmdbuf_set_wan_ip_config;
            apicmdbuf_set_wan_ip_config = (APICMDBUF_SET_ETH_WAN_IP_CONFIG *)apicmdbuf;
            apicmdbuf_set_wan_ip_config->cmdCode = APCMD_SET_ETH_WAN_IP_CONFIG_REPLY;

            mib_dhcpenable = apicmdbuf_set_wan_ip_config->dhcpEnabled;
            mib_DHCP.SubnetMask = inet_address(apicmdbuf_set_wan_ip_config->subnetMask);
            mib_DHCP.IPAddr = inet_address(apicmdbuf_set_wan_ip_config->ipAddress);
            mib_DHCP.GwyAddr = inet_address(apicmdbuf_set_wan_ip_config->gatewayAddress);
            mib_DHCP.PrimaryDNS = inet_address(apicmdbuf_set_wan_ip_config->primaryDNSAddress);
            mib_DHCP.SecondaryDNS = inet_address(apicmdbuf_set_wan_ip_config->secondaryDNSAddress);
            apicmdbuf_set_wan_ip_config->cmdStatus = APCTL_OK;
            mib_dhcpstatechange = TRUE;
            initiate_dhcp(1);

        }
        break;
#endif
#endif

#if defined(GATEWAY) || defined(DHCPS)

    case APCMD_GET_ETH_DHCPS_CONFIG:
        {
            unsigned int bodyLen = sizeof(APICMDBUF_GET_ETH_DHCPS_CONFIG) - 12;
            if (apicmdbuf->cmdBodyLen < bodyLen)
            {
                apicmdbuf->cmdStatus    = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen   = bodyLen;
                return OS_FAIL;
            }
            else
            {

                APICMDBUF_GET_ETH_DHCPS_CONFIG* apicmdbuf_get_eth_dhcps_config;
                apicmdbuf_get_eth_dhcps_config = (APICMDBUF_GET_ETH_DHCPS_CONFIG *)apicmdbuf;
                apicmdbuf_get_eth_dhcps_config->cmdCode = APCMD_GET_ETH_DHCPS_CONFIG_REPLY;

                apicmdbuf_get_eth_dhcps_config->isEnable = dhcpsIsEnable();
                dhcpsGetRange(&(apicmdbuf_get_eth_dhcps_config->rangeStartAddr),
                              &(apicmdbuf_get_eth_dhcps_config->rangeEndAddr));
                dhcpsGetLeaseTime(&(apicmdbuf_get_eth_dhcps_config->leaseTime));
                dhcpsGetBulkStations(&(apicmdbuf_get_eth_dhcps_config->numOfStations),
                                     apicmdbuf_get_eth_dhcps_config->stationsTbl);

                apicmdbuf_get_eth_dhcps_config->cmdStatus = APCTL_OK;

                /* Swap to network order */

                apicmdbuf_get_eth_dhcps_config->rangeStartAddr =
                                inet_address (apicmdbuf_get_eth_dhcps_config->rangeStartAddr);
                apicmdbuf_get_eth_dhcps_config->rangeEndAddr =
                                inet_address (apicmdbuf_get_eth_dhcps_config->rangeEndAddr);

                i = 0;
                while (apicmdbuf_get_eth_dhcps_config->stationsTbl[i].ip_address != 0)
                {
                    apicmdbuf_get_eth_dhcps_config->stationsTbl[i].ip_address =
                        inet_address (apicmdbuf_get_eth_dhcps_config->stationsTbl[i].ip_address);
                    apicmdbuf_get_eth_dhcps_config->stationsTbl[i].st_type =
                        apicmdbuf_get_eth_dhcps_config->stationsTbl[i].st_type;
                    i++;
                }

            }
        }
        break;
    case APCMD_SET_ETH_DHCPS_CONFIG:
        {
            unsigned int bodyLen = sizeof(APICMDBUF_SET_ETH_DHCPS_CONFIG) - 12;
            if (apicmdbuf->cmdBodyLen < bodyLen)
            {
                apicmdbuf->cmdStatus    = APCTL_BUF_TOO_SMALL;
                apicmdbuf->cmdBodyLen   = bodyLen;
                return OS_FAIL;
            }
            else
            {

                APICMDBUF_SET_ETH_DHCPS_CONFIG* apicmdbuf_set_eth_dhcps_config;
                apicmdbuf_set_eth_dhcps_config = (APICMDBUF_SET_ETH_DHCPS_CONFIG *)apicmdbuf;
                apicmdbuf_set_eth_dhcps_config->cmdCode = APCMD_SET_ETH_DHCPS_CONFIG_REPLY;

                if (dhcpsEnable(apicmdbuf_set_eth_dhcps_config->isEnable) != OS_SUCCESS)
                {
                    apicmdbuf->cmdStatus    = APCTL_CMD_FAILED;
                    return OS_FAIL;
                }

                if (dhcpsSetRange(inet_address (apicmdbuf_set_eth_dhcps_config->rangeStartAddr),
                                  inet_address (apicmdbuf_set_eth_dhcps_config->rangeEndAddr)) != OS_SUCCESS)
                {
                    apicmdbuf->cmdStatus    = APCTL_CMD_FAILED;
                    return OS_FAIL;
                }

                if (dhcpsSetLeaseTime(apicmdbuf_set_eth_dhcps_config->leaseTime) != OS_SUCCESS)
                {
                    apicmdbuf->cmdStatus    = APCTL_CMD_FAILED;
                    return OS_FAIL;
                }

                if (dhcpsSetStation(inet_address (apicmdbuf_set_eth_dhcps_config->station.ip_address),
                                    apicmdbuf_set_eth_dhcps_config->station.mac_addr,
                    (apicmdbuf_set_eth_dhcps_config->station.st_type ==
                                                    DHCPS_NO_ENTRY)) != OS_SUCCESS)
                {
                    apicmdbuf->cmdStatus    = APCTL_CMD_FAILED;
                    return OS_FAIL;
                }

            }
        }
        break;

#endif /* DHCPS */

#ifdef IEEE80211H
    case APCMD_GET_SPECTRUM_MGMT:
        if (apicmdbuf->cmdBodyLen < sizeof(UINT8))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = sizeof(BOOLEAN);
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_SPECTRUM_MGMT *apicmdbuf_get_spectrum_management;
            apicmdbuf_get_spectrum_management = (APICMDBUF_GET_SPECTRUM_MGMT *)apicmdbuf;
            apicmdbuf_get_spectrum_management->cmdCode = APCMD_GET_SPECTRUM_MGMT_REPLY;
            apicmdbuf_get_spectrum_management->isEnable= mib_StaCfg_p->SpectrumManagementRequired;            
            apicmdbuf_get_spectrum_management->cmdStatus = APCTL_OK;
        }
        break;
        
    case APCMD_SET_SPECTRUM_MGMT:
        if (apicmdbuf->cmdBodyLen < sizeof(UINT8))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = sizeof(UINT8);
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_SPECTRUM_MGMT *apicmdbuf_set_spectrum_management;
            apicmdbuf_set_spectrum_management = (APICMDBUF_SET_SPECTRUM_MGMT*)apicmdbuf;
            apicmdbuf_set_spectrum_management->cmdCode = APCMD_SET_SPECTRUM_MGMT_REPLY;
		    mib_StaCfg_p->SpectrumManagementRequired = apicmdbuf_set_spectrum_management->isEnable; 
            apicmdbuf_set_spectrum_management->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_MITIGATION:
        if (apicmdbuf->cmdBodyLen < sizeof(UINT32))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = sizeof(UINT32);
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_MITIGATION *apicmdbuf_get_mitigation;
            apicmdbuf_get_mitigation = (APICMDBUF_GET_MITIGATION *)apicmdbuf;
            apicmdbuf_get_mitigation->cmdCode = APCMD_GET_MITIGATION_REPLY;
            apicmdbuf_get_mitigation->mitigationValue= mib_SpectrumMagament_p->mitigationRequirement;
            apicmdbuf_get_mitigation->cmdStatus = APCTL_OK;
        }
        break;        
        
    case APCMD_SET_MITIGATION:
        if (apicmdbuf->cmdBodyLen < sizeof(UINT32))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = sizeof(UINT32);
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_MITIGATION *apicmdbuf_set_mitigation;
            apicmdbuf_set_mitigation = (APICMDBUF_SET_MITIGATION *)apicmdbuf;
            apicmdbuf_set_mitigation->cmdCode = APCMD_SET_MITIGATION_REPLY;
		    mib_SpectrumMagament_p->mitigationRequirement = apicmdbuf_set_mitigation->mitigationValue; 
            apicmdbuf_set_mitigation->cmdStatus = APCTL_OK;
        }
        break;  

    case APCMD_SEND_SWITCH_CHANNEL_ANNOUNCEMENT:     
        if (apicmdbuf->cmdBodyLen < sizeof(UINT8))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = sizeof(UINT8);
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SEND_SWITCH_CHANNEL_ANNOUNCEMENT *apicmdbuf_send_SCA;
            apicmdbuf_send_SCA = (APICMDBUF_SEND_SWITCH_CHANNEL_ANNOUNCEMENT *)apicmdbuf;
            if ( WL_STATUS_OK != StartSendChannelSwitchCmd(1, apicmdbuf_send_SCA->channel))
            {
                apicmdbuf->cmdStatus = APCTL_ERROR;
                return OS_FAIL;
            }
            else
            {                    
                os_TaskDelay(200);
                apicmdbuf->cmdStatus = APCTL_OK;
            }
            
        }
        break;                    

    case APCMD_GET_REGULATORY_DOMAIN:        
        if (apicmdbuf->cmdBodyLen < sizeof(UINT8))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = sizeof(BOOLEAN);
            return OS_FAIL;
        }
        else
        {            
            APICMDBUF_GET_REGULATORY_DOMAIN *apicmdbuf_get_regulartory_domain;
            apicmdbuf_get_regulartory_domain = (APICMDBUF_GET_SPECTRUM_MGMT *)apicmdbuf;
            apicmdbuf_get_regulartory_domain->cmdCode = APCMD_GET_REGULATORY_DOMAIN_REPLY;
            apicmdbuf_get_regulartory_domain->domain = bcn_reg_domain;            
            apicmdbuf_get_regulartory_domain->cmdStatus = APCTL_OK;
        }
        break;
        
    case APCMD_SET_REGULATORY_DOMAIN:        
        if (apicmdbuf->cmdBodyLen < sizeof(UINT8))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = sizeof(UINT8);            
            return OS_FAIL;
        }
        else
        {
            extern UINT8 bcn_reg_domain;
            
            APICMDBUF_SET_REGULATORY_DOMAIN *apicmdbuf_set_regulartory_domain;
            apicmdbuf_set_regulartory_domain = (APICMDBUF_SET_SPECTRUM_MGMT*)apicmdbuf;
            apicmdbuf_set_regulartory_domain->cmdCode = APCMD_SET_REGULATORY_DOMAIN_REPLY;
		    bcn_reg_domain = apicmdbuf_set_regulartory_domain->domain; 
            apicmdbuf_set_regulartory_domain->cmdStatus = APCTL_OK;            
        }
        break;
#endif /* IEEE80211H */
#ifdef JAPAN_CHANNEL_SPACING_10_SUPPORT
    case APCMD_GET_CHANNEL_SPACING:
        if (apicmdbuf->cmdBodyLen < 3)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 3;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_CHANNEL_SPACING *apicmdbuf_get_channel_spacing;
            apicmdbuf_get_channel_spacing = (APICMDBUF_GET_CHANNEL_SPACING *)apicmdbuf;
            apicmdbuf_get_channel_spacing->cmdCode = APCMD_GET_CHANNEL_SPACING;
            apicmdbuf_get_channel_spacing->curChanSpacing = mib_channelspacing;
            apicmdbuf_get_channel_spacing->cmdStatus = APCTL_OK;
        }

        break;


    case APCMD_SET_CHANNEL_SPACING:
        if (apicmdbuf->cmdBodyLen < 3)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 3;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_CHANNEL_SPACING* apicmdbuf_set_channel_spacing;
            apicmdbuf_set_channel_spacing = (APICMDBUF_SET_CHANNEL_SPACING *)apicmdbuf;
            apicmdbuf_set_channel_spacing->cmdCode = APCMD_SET_RF_CHANNEL_REPLY;
            mib_channelspacing = apicmdbuf_set_channel_spacing->curChanSpacing;
            apicmdbuf_set_channel_spacing->cmdStatus = APCTL_OK;
        }
        break;


#endif
    case APCMD_SET_RF_POWER_MODE:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_RF_POWER_MODE* apicmdbuf_set_RF_POWER_MODE;
            apicmdbuf_set_RF_POWER_MODE = (APICMDBUF_SET_RF_POWER_MODE*)apicmdbuf;
            apicmdbuf_set_RF_POWER_MODE->cmdCode = APCMD_SET_RF_POWER_MODE_REPLY;
            apicmdbuf_set_RF_POWER_MODE->cmdStatus = APCTL_OK;
            if(apicmdbuf_set_RF_POWER_MODE->PowerMode == POWER_ON)
            {
#ifdef RFSLEEP
				RfSleep(1);
#endif
                msi_wl_PowerupRF();
            }
            else if(apicmdbuf_set_RF_POWER_MODE->PowerMode == POWER_OFF)
            {
#ifdef RFSLEEP
				RfSleep(0);
#endif
                msi_wl_PowerdownRF();
            }
        }
        break;
#ifdef SOHO_GATEWAY
    case APCMD_SET_SWITCH_IP_PRIO_MAP_EN:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP_EN* apicmdbuf_switch_prio_map_en;
            apicmdbuf_switch_prio_map_en = (APICMDBUF_SWITCH_PRIO_MAP_EN*)apicmdbuf;
            apicmdbuf_switch_prio_map_en->cmdCode = APCMD_SET_SWITCH_IP_PRIO_MAP_EN_REPLY;
            if ( SUCCESS == EthSw_Port_SetIpPrioMapEn(apicmdbuf_switch_prio_map_en->port,
                apicmdbuf_switch_prio_map_en->mode))
            {
                apicmdbuf_switch_prio_map_en->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map_en->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_GET_SWITCH_IP_PRIO_MAP_EN:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP_EN* apicmdbuf_switch_prio_map_en;
            apicmdbuf_switch_prio_map_en = (APICMDBUF_SWITCH_PRIO_MAP_EN*)apicmdbuf;
            apicmdbuf_switch_prio_map_en->cmdCode = APCMD_GET_SWITCH_IP_PRIO_MAP_EN_REPLY;
            if ( SUCCESS == EthSw_Port_GetIpPrioMapEn(apicmdbuf_switch_prio_map_en->port,
                &(apicmdbuf_switch_prio_map_en->mode)) )
            {
                apicmdbuf_switch_prio_map_en->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map_en->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_SET_SWITCH_USER_PRIO_MAP_EN:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP_EN* apicmdbuf_switch_prio_map_en;
            apicmdbuf_switch_prio_map_en = (APICMDBUF_SWITCH_PRIO_MAP_EN*)apicmdbuf;
            apicmdbuf_switch_prio_map_en->cmdCode = APCMD_SET_SWITCH_USER_PRIO_MAP_EN_REPLY;
            if ( SUCCESS == EthSw_Port_SetUserPrioMapEn(apicmdbuf_switch_prio_map_en->port,
                apicmdbuf_switch_prio_map_en->mode))
            {
                apicmdbuf_switch_prio_map_en->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map_en->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_GET_SWITCH_USER_PRIO_MAP_EN:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP_EN* apicmdbuf_switch_prio_map_en;
            apicmdbuf_switch_prio_map_en = (APICMDBUF_SWITCH_PRIO_MAP_EN*)apicmdbuf;
            apicmdbuf_switch_prio_map_en->cmdCode = APCMD_GET_SWITCH_USER_PRIO_MAP_EN_REPLY;
            if ( SUCCESS == EthSw_Port_GetUserPrioMapEn(apicmdbuf_switch_prio_map_en->port,
                &(apicmdbuf_switch_prio_map_en->mode)) )
            {
                apicmdbuf_switch_prio_map_en->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map_en->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_SET_SWITCH_PRIO_MAP_RULE:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP_EN* apicmdbuf_switch_prio_map_rule;
            apicmdbuf_switch_prio_map_rule = (APICMDBUF_SWITCH_PRIO_MAP_EN*)apicmdbuf;
            apicmdbuf_switch_prio_map_rule->cmdCode = APCMD_SET_SWITCH_PRIO_MAP_RULE_REPLY;
            if ( SUCCESS == EthSw_Port_SetPrioMapRule(apicmdbuf_switch_prio_map_rule->port,
                apicmdbuf_switch_prio_map_rule->mode))
            {
                apicmdbuf_switch_prio_map_rule->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map_rule->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_GET_SWITCH_PRIO_MAP_RULE:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP_EN* apicmdbuf_switch_prio_map_rule;
            apicmdbuf_switch_prio_map_rule = (APICMDBUF_SWITCH_PRIO_MAP_EN*)apicmdbuf;
            apicmdbuf_switch_prio_map_rule->cmdCode = APCMD_GET_SWITCH_PRIO_MAP_RULE_REPLY;
            if ( SUCCESS == EthSw_Port_SetPrioMapRule(apicmdbuf_switch_prio_map_rule->port,
                &(apicmdbuf_switch_prio_map_rule->mode)) )
            {
                apicmdbuf_switch_prio_map_rule->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map_rule->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_SET_SWITCH_IP_PRIO_MAP:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP* apicmdbuf_switch_prio_map;
            apicmdbuf_switch_prio_map = (APICMDBUF_SWITCH_PRIO_MAP*)apicmdbuf;
            apicmdbuf_switch_prio_map->cmdCode = APCMD_SET_SWITCH_IP_PRIO_MAP_REPLY;
            if ( SUCCESS == EthSw_Port_SetIpPrioMap(apicmdbuf_switch_prio_map->tag,
                apicmdbuf_switch_prio_map->prio))
            {
                apicmdbuf_switch_prio_map->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_GET_SWITCH_IP_PRIO_MAP:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP* apicmdbuf_switch_prio_map;
            apicmdbuf_switch_prio_map = (APICMDBUF_SWITCH_PRIO_MAP*)apicmdbuf;
            apicmdbuf_switch_prio_map->cmdCode = APCMD_GET_SWITCH_IP_PRIO_MAP_REPLY;
            if ( SUCCESS == EthSw_Port_GetIpPrioMap(apicmdbuf_switch_prio_map->tag,
                &(apicmdbuf_switch_prio_map->prio)) )
            {
                apicmdbuf_switch_prio_map->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_SET_SWITCH_USER_PRIO_MAP:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP* apicmdbuf_switch_prio_map;
            apicmdbuf_switch_prio_map = (APICMDBUF_SWITCH_PRIO_MAP*)apicmdbuf;
            apicmdbuf_switch_prio_map->cmdCode = APCMD_SET_SWITCH_USER_PRIO_MAP_REPLY;
            if ( SUCCESS == EthSw_Port_SetUserPrioMap(apicmdbuf_switch_prio_map->tag,
                apicmdbuf_switch_prio_map->prio))
            {
                apicmdbuf_switch_prio_map->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_GET_SWITCH_USER_PRIO_MAP:
        if ( apicmdbuf->cmdBodyLen < 8 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 8;
            return FAIL;
        }
        else
        {
            APICMDBUF_SWITCH_PRIO_MAP* apicmdbuf_switch_prio_map;
            apicmdbuf_switch_prio_map = (APICMDBUF_SWITCH_PRIO_MAP*)apicmdbuf;
            apicmdbuf_switch_prio_map->cmdCode = APCMD_GET_SWITCH_USER_PRIO_MAP_REPLY;
            if ( SUCCESS == EthSw_Port_GetUserPrioMap(apicmdbuf_switch_prio_map->tag,
                &(apicmdbuf_switch_prio_map->prio)) )
            {
                apicmdbuf_switch_prio_map->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_switch_prio_map->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;

#endif
	case APCMD_GET_RF_REG:
        if ( apicmdbuf->cmdBodyLen < 3 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 3;
            return FAIL;
        }
        else
        {
			APICMDBUF_GET_RF_REG *tmp;
			tmp = (APICMDBUF_GET_RF_REG *)apicmdbuf;
			tmp->cmdCode = APCMD_GET_RF_REG_REPLY;
    		sys_InitPHYAccess();
    		msi_wl_ReadPHY(tmp->regNum, &tmp->regVal);
		}
		break;
	case APCMD_SET_RF_REG:
        if ( apicmdbuf->cmdBodyLen < 3 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 3;
            return FAIL;
        }
        else
        {
			APICMDBUF_SET_RF_REG *tmp;
			tmp = (APICMDBUF_SET_RF_REG *)apicmdbuf;
			tmp->cmdCode = APCMD_SET_RF_REG_REPLY;
    		sys_InitPHYAccess();
    		msi_wl_WritePHY(tmp->regNum, tmp->regVal);
		}
		break;
	case APCMD_GET_BB_REG:
        if ( apicmdbuf->cmdBodyLen < 3 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 3;
            return FAIL;
        }
        else
        {
			APICMDBUF_GET_BB_REG *tmp;
			tmp = (APICMDBUF_GET_BB_REG *)apicmdbuf;
			tmp->cmdCode = APCMD_GET_BB_REG_REPLY;
    		sys_InitBBPAccess();
			if(tmp->regNum >= 0x80)
			{
    			msi_wl_WriteBBP(0, 0x80);
			}
			else
			{
			    msi_wl_WriteBBP(0, 0);
			}
			WaitForDone();
    		msi_wl_ReadBBP(tmp->regNum, &tmp->regVal);
		}
		break;
	case APCMD_SET_BB_REG:
        if ( apicmdbuf->cmdBodyLen < 3 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 3;
            return FAIL;
        }
        else
        {
			APICMDBUF_SET_BB_REG *tmp;
			tmp = (APICMDBUF_SET_BB_REG *)apicmdbuf;
			tmp->cmdCode = APCMD_SET_BB_REG_REPLY;
			if(tmp->regNum >= 0x80)
			{
    			msi_wl_WriteBBP(0, 0x80);
			}
			else
			{
			    msi_wl_WriteBBP(0, 0);
			}
			WaitForDone();
    		msi_wl_WriteBBP(tmp->regNum, tmp->regVal);
		}
		break;

#ifdef AUTOLINK
    case APCMD_GET_PSK_PASSPHRASE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_PSK_PASSPHRASE* apicmdbuf_get_psk_passph;
            apicmdbuf_get_psk_passph = (APICMDBUF_GET_PSK_PASSPHRASE *)apicmdbuf;
            apicmdbuf_get_psk_passph->cmdCode = APCMD_GET_PSK_PASSPHRASE_REPLY;
            strcpy(apicmdbuf_get_psk_passph->passphrase,mib_RSNConfig_p->PSKPassPhrase);

            apicmdbuf_get_psk_passph->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_AL_LOW_POWER_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AL_LOW_POWER_STATE* apicmdbuf_get_al_low_power_state;
            apicmdbuf_get_al_low_power_state = (APICMDBUF_GET_AL_LOW_POWER_STATE *)apicmdbuf;
            apicmdbuf_get_al_low_power_state->cmdCode = APCMD_GET_AL_LOW_POWER_STATE_REPLY;
            apicmdbuf_get_al_low_power_state->ALLowTxPowerEnable = mib_ALLowPowerEnable;
            apicmdbuf_get_al_low_power_state->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_AL_LOW_POWER_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_AL_LOW_POWER_STATE* apicmdbuf_set_al_low_power_state;
            apicmdbuf_set_al_low_power_state = (APICMDBUF_SET_AL_LOW_POWER_STATE *)apicmdbuf;
            apicmdbuf_set_al_low_power_state->cmdCode = APCMD_SET_AL_LOW_POWER_STATE_REPLY;
            mib_ALLowPowerEnable = apicmdbuf_set_al_low_power_state->ALLowTxPowerEnable;
            apicmdbuf_set_al_low_power_state->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_AL_ENABLE_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AL_ENABLE_STATE* apicmdbuf_get_al_enable_state;
            apicmdbuf_get_al_enable_state = (APICMDBUF_GET_AL_ENABLE_STATE *)apicmdbuf;
            apicmdbuf_get_al_enable_state->cmdCode = APCMD_GET_AL_ENABLE_STATE_REPLY;
            apicmdbuf_get_al_enable_state->ALEnable = mib_ALEnable;
            apicmdbuf_get_al_enable_state->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_AL_ENABLE_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_AL_ENABLE_STATE* apicmdbuf_set_al_enable_state;
            apicmdbuf_set_al_enable_state = (APICMDBUF_SET_AL_ENABLE_STATE *)apicmdbuf;
            apicmdbuf_set_al_enable_state->cmdCode = APCMD_SET_AL_ENABLE_STATE_REPLY;
            mib_ALEnable = apicmdbuf_set_al_enable_state->ALEnable;
            apicmdbuf_set_al_enable_state->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_AL_ON_TIME:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AL_ON_TIME* apicmdbuf_get_al_on_time;
            apicmdbuf_get_al_on_time = (APICMDBUF_GET_AL_ON_TIME*)apicmdbuf;
            apicmdbuf_get_al_on_time->cmdCode = APCMD_GET_AL_ON_TIME_REPLY;
            apicmdbuf_get_al_on_time->ALONTime = mib_ALONTime;
            apicmdbuf_get_al_on_time->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_AL_ON_TIME:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_AL_ON_TIME* apicmdbuf_set_al_on_time;
            apicmdbuf_set_al_on_time = (APICMDBUF_SET_AL_ON_TIME *)apicmdbuf;
            apicmdbuf_set_al_on_time->cmdCode = APCMD_SET_AL_ON_TIME_REPLY;
            mib_ALONTime = apicmdbuf_set_al_on_time->ALONTime;
            apicmdbuf_set_al_on_time->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_AL_PP_ENABLE_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AL_PP_ENABLE_STATE* apicmdbuf_get_al_pp_enable_state;
            apicmdbuf_get_al_pp_enable_state = (APICMDBUF_GET_AL_PP_ENABLE_STATE *)apicmdbuf;
            apicmdbuf_get_al_pp_enable_state->cmdCode = APCMD_GET_AL_PP_ENABLE_STATE_REPLY;
            apicmdbuf_get_al_pp_enable_state->ALPPEnable = mib_ALPPEnable;
            apicmdbuf_get_al_pp_enable_state->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_AL_PP_ENABLE_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_AL_PP_ENABLE_STATE* apicmdbuf_set_al_pp_enable_state;
            apicmdbuf_set_al_pp_enable_state = (APICMDBUF_SET_AL_PP_ENABLE_STATE *)apicmdbuf;
            apicmdbuf_set_al_pp_enable_state->cmdCode = APCMD_SET_AL_PP_ENABLE_STATE_REPLY;
            mib_ALPPEnable = apicmdbuf_set_al_pp_enable_state->ALPPEnable;
            apicmdbuf_set_al_pp_enable_state->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_AL_SEC_MODE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AL_SEC_MODE* apicmdbuf_get_al_sec_mode;
            apicmdbuf_get_al_sec_mode = (APICMDBUF_GET_AL_SEC_MODE *)apicmdbuf;
            apicmdbuf_get_al_sec_mode->cmdCode = APCMD_GET_AL_SEC_MODE_REPLY;
            apicmdbuf_get_al_sec_mode->ALSecmode= mib_ALSecMode;
            apicmdbuf_get_al_sec_mode->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_AL_SEC_MODE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_SET_AL_SEC_MODE* apicmdbuf_set_al_sec_mode;
            apicmdbuf_set_al_sec_mode = (APICMDBUF_SET_AL_SEC_MODE*)apicmdbuf;
            apicmdbuf_set_al_sec_mode->cmdCode = APCMD_SET_AL_SEC_MODE_REPLY;
            mib_ALSecMode= apicmdbuf_set_al_sec_mode->ALSecMode;
            apicmdbuf_set_al_sec_mode->cmdStatus = APCTL_OK;
        }
        break;
#endif /*AUTOLINK*/

#ifdef DCS
    case APCMD_GET_DCS_MODE:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_DCS_MODE * apicmdbuf_get_DCS_MODE;
            apicmdbuf_get_DCS_MODE = (APICMDBUF_GET_DCS_MODE *)apicmdbuf;
            apicmdbuf_get_DCS_MODE->cmdCode = APCMD_GET_DCS_MODE_REPLY;
            apicmdbuf_get_DCS_MODE->DCSMode = mib_dcsMode;
            apicmdbuf_get_DCS_MODE->cmdStatus = APCTL_OK;
        }

        break;
    case APCMD_SET_DCS_MODE:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_DCS_MODE * apicmdbuf_set_DCS_MODE;
            apicmdbuf_set_DCS_MODE = (APICMDBUF_SET_DCS_MODE*)apicmdbuf;
            apicmdbuf_set_DCS_MODE->cmdCode = APCMD_SET_DCS_MODE_REPLY;
            mib_dcsMode = apicmdbuf_set_DCS_MODE->DCSMode;
            apicmdbuf_set_DCS_MODE->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_BKGND_SCAN:
        if (apicmdbuf->cmdBodyLen < (sizeof(APICMDBUF_SET_BKGND_SCAN) - APICMDHEADERLEN))
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = (sizeof(APICMDBUF_SET_BKGND_SCAN) - APICMDHEADERLEN);
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_BKGND_SCAN* apicmdbuf_set_bkgnd_scan;
            apicmdbuf_set_bkgnd_scan = (APICMDBUF_SET_BKGND_SCAN*)apicmdbuf;
            apicmdbuf_set_bkgnd_scan->cmdCode = APCMD_SET_BKGND_SCAN_REPLY;
            if (SUCCESS == SetupBkgndScan(apicmdbuf_set_bkgnd_scan))
            {
                apicmdbuf_set_bkgnd_scan->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_set_bkgnd_scan->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
    case APCMD_GET_BGSCAN_DATA:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_BGSCAN_DATA* apicmdbuf_get_bgscan_data;
            apicmdbuf_get_bgscan_data = (APICMDBUF_GET_BGSCAN_DATA *)apicmdbuf;
            apicmdbuf_get_bgscan_data->cmdCode =APCMD_GET_BGSCAN_DATA_REPLY;

            if (BkgndScanActive)
            {
                apicmdbuf_get_bgscan_data->busyScan = 1;
            }
            else
            {
                apicmdbuf_get_bgscan_data->busyScan = 0;
            }
            apicmdbuf_get_bgscan_data->maxEntry = BGSCAN_ENTRY_MAX;
            apicmdbuf_get_bgscan_data->BSSInfoAddr = (UINT8 *)&bkgndScanBSSData;
            apicmdbuf_get_bgscan_data->CHInfoAddr = (UINT8 *)&bkgndScanCHData;
            apicmdbuf_get_bgscan_data->curCHTxCount = Stats.DataTxSuccessCnt;
            apicmdbuf_get_bgscan_data->curCHTxRetryCnt = Stats.RetryCnt;
            apicmdbuf_get_bgscan_data->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_GET_CHWEIGHT_DATA:
        if (apicmdbuf->cmdBodyLen < 6)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_CHWGT_DATA* apicmdbuf_get_chwgt_data;
            apicmdbuf_get_chwgt_data = (APICMDBUF_GET_CHWGT_DATA *)apicmdbuf;
            apicmdbuf_get_chwgt_data->cmdCode =APCMD_GET_CHWEIGHT_DATA_REPLY;
            apicmdbuf_get_chwgt_data->CHWgtInfoAddr = (UINT8 *)&CHWeightData;
            apicmdbuf_get_chwgt_data->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_DCS_CHANNEL:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_DCS_CHANNEL* apicmdbuf_set_DCS_Channel;
            apicmdbuf_set_DCS_Channel = (APICMDBUF_SET_DCS_CHANNEL*)apicmdbuf;
            apicmdbuf_set_DCS_Channel->cmdCode = APCMD_SET_DCS_CHANNEL_REPLY;
            if (SUCCESS == SetDCSChannel(apicmdbuf_set_DCS_Channel->channel))
            {
                apicmdbuf_set_DCS_Channel->cmdStatus = APCTL_OK;
            }
            else
            {
                apicmdbuf_set_DCS_Channel->cmdStatus = APCTL_CMD_FAILED;
            }
        }
        break;
#endif /*DCS*/

    case APCMD_GET_STA_AVERSSI:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            extStaDb_StaInfo_t *StaInfo_p;

            APICMDBUF_GET_STA_AVERSSI * apicmdbuf_get_sta_aveRSSI;
            apicmdbuf_get_sta_aveRSSI = (APICMDBUF_GET_STA_AVERSSI*)apicmdbuf;
            apicmdbuf_get_sta_aveRSSI->cmdCode = APCMD_GET_STA_AVERSS_REPLY;
            
            if ((StaInfo_p = extStaDb_GetStaInfo(&apicmdbuf_get_sta_aveRSSI->aveRssiInfo.Addr,  0)))
            {
              	apicmdbuf_get_sta_aveRSSI->aveRssiInfo.AveRssi = getAvgRSSI(&(StaInfo_p->aveRSSI));
            }
            else
            {
				apicmdbuf_get_sta_aveRSSI->cmdStatus = APCTL_ERROR;
				return FAIL;
            }			            
            apicmdbuf_get_sta_aveRSSI->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_SET_STA_FIXEDRATE:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            extStaDb_StaInfo_t *StaInfo_p;

            APICMDBUF_SET_STA_FIXEDRATE * apicmdbuf_set_sta_fixedRate;
            apicmdbuf_set_sta_fixedRate = (APICMDBUF_SET_STA_FIXEDRATE*)apicmdbuf;
            apicmdbuf_set_sta_fixedRate->cmdCode = APCMD_SET_STA_FIXEDRATE_REPLY;
            
            if ((StaInfo_p = extStaDb_GetStaInfo(&apicmdbuf_set_sta_fixedRate->fixedRateInfo.Addr,  0)) &&
            	(apicmdbuf_set_sta_fixedRate->fixedRateInfo.FixedRate <= 144))
            {
              	AssocTable[StaInfo_p->Aid].fixedRate = util_GetIndexByRate(apicmdbuf_set_sta_fixedRate->fixedRateInfo.FixedRate);
            }
            else
            {
				apicmdbuf_set_sta_fixedRate->cmdStatus = APCTL_ERROR;
				return FAIL;
            }			            
            apicmdbuf_set_sta_fixedRate->cmdStatus = APCTL_OK;
        }
        break;

    case APCMD_GET_AGING_STATE:
        if (apicmdbuf->cmdBodyLen < 1)
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return OS_FAIL;
        }
        else
        {
            APICMDBUF_GET_AGING_STATE* apicmdbuf_get_aging_state;
            apicmdbuf_get_aging_state = (APICMDBUF_GET_AGING_STATE *)apicmdbuf;
            apicmdbuf_get_aging_state->cmdCode = APCMD_GET_AGING_STATE_REPLY;
            apicmdbuf_get_aging_state->agingTime = mib_AgingTime;	 
            apicmdbuf_get_aging_state->cmdStatus = APCTL_OK;
        }

        break;
#ifdef GUEST_ACCESS
    case APCMD_GET_GUEST_ACCESS:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_GET_GUEST_ACCESS *tmp;
            tmp = (APICMDBUF_GET_GUEST_ACCESS *)apicmdbuf;
            tmp->cmdCode = APCMD_GET_GUEST_ACCESS_REPLY;
            tmp->GuestAccessEnable = mib_GuestAccessEnable;
            memcpy(tmp->GuestAccessPassPhrase, mib_GuestAccessPassPhrase, 64);
            memcpy(tmp->GuestAccessPassPhrase2, mib_GuestAccessPassPhrase2, 64);
            tmp->cmdStatus = APCTL_OK;
        }
        break;
    case APCMD_SET_GUEST_ACCESS:
        if ( apicmdbuf->cmdBodyLen < 1 )
        {
            apicmdbuf->cmdStatus = APCTL_BUF_TOO_SMALL;
            apicmdbuf->cmdBodyLen = 1;
            return FAIL;
        }
        else
        {
            APICMDBUF_SET_GUEST_ACCESS *tmp;
            tmp = (APICMDBUF_SET_GUEST_ACCESS *)apicmdbuf;
            tmp->cmdCode = APCMD_SET_GUEST_ACCESS_REPLY;
            mib_GuestAccessEnable = tmp->GuestAccessEnable;
            memcpy(mib_GuestAccessPassPhrase, tmp->GuestAccessPassPhrase, 64);
            memcpy(mib_GuestAccessPassPhrase2, tmp->GuestAccessPassPhrase2, 64);
            tmp->cmdStatus = APCTL_OK;
        }
        break;
#endif

    default:
        apicmdbuf->cmdStatus = APCTL_BAD_CMD;
        return FAIL;
    }



    return SUCCESS;


}

void Wlan_Radio(UINT8 option)
{
#ifdef CONDOR
    if (option == 0)  //Shutoff Radio
    {
        WL_REGS32(0x8000A874) = 0x9 ;
	 Radio_off=1;

    }

    else if (option == 1)
    {
        WL_REGS32(0x8000A874) = 0x1;
	Radio_off=0;

    }


#else

    if (option == 0)  //Shutoff Radio
    {
        WL_REGS32(0x8000A444) = 0x9;

    }

    else if (option == 1)
    {
        WL_REGS32(0x8000A444) = 0x1;

    }
#endif

}
void Reset(void)
{
    union
    {
        int x;
        void (*f)(void);
    }u;

    if (MfgTestModeOn)
    {
        return;
    }

	Wlan_Radio(0);
    u.x = 0xffff0000;
    u.f();
}

void Reset2()
{
Reset();

}

#ifdef TESTCMD
unsigned char ReadBbReg(unsigned char offset)
{
   // return (W81_BBPRead(offset));
}
void WriteBbReg(unsigned char offset, unsigned char value)
{
    W81_BBPWrite(offset, value);
}
unsigned char ReadRfReg(unsigned char offset)
{
    return (W81_w8000Read(offset));
}
void WriteRfReg(unsigned char offset, unsigned char value)
{
    W81_w8000Write(offset, value);
}
unsigned long ReadMacReg(unsigned long offset)
{
    unsigned long value;

    WL_READ_WORD(WL_WLMAC_CFG_BASE | offset, value); /* for later SW use */
    return value;
}
void WriteMacReg(unsigned long offset, unsigned long value)
{

    WL_WRITE_WORD(WL_WLMAC_CFG_BASE | offset, value);
}
#endif

/* Interrupt control API */

/**
 * Disable all interrupts
 * 
 * @return Interrupt status
 */
int apctlIntDisable(void)
{
#ifdef WATCHDOG_ENABLE
        WL_WRITE_REGS16(0x90009034,0x130) ;  /** disable watchdog timer **/
#endif

    cyg_interrupt_disable();
    intStatus = -1;
    return intStatus;
}

/**
 * Enable all interrupts
 * 
 * @param status Status word returned by apctlIntDisable()
 * @return Interrupt status
 */
int apctlIntEnable(int status)
{
#ifdef WATCHDOG_ENABLE
    WL_WRITE_REGS16(0x90009034,0x131) ;  /** enable watchdog timer **/
#endif

    cyg_interrupt_enable();

    intStatus = 0;
    return intStatus;
}

/**
 * Get interrupt enable status
 * 
 * @return Interrupt status
 */
int apctlGetIntStatus(void)
{
    return intStatus;
}
#ifdef LED_SUPPORT
unsigned char apctlSwLedDisable(unsigned char gpioport)
{
    unsigned long dirMap = 0;
    unsigned long val;

    WL_READ_REGS32(GPIO_REG_OUTPUT_ENABLE, val);
    dirMap = 1 << gpioport;
    WL_WRITE_REGS32(GPIO_REG_OUTPUT_ENABLE, val | dirMap);
    return 0;
}

void apctlSwLedEnable(unsigned char gpioport)
{
    unsigned long dirMap = 0;
    unsigned long val;

    WL_READ_REGS32(GPIO_REG_OUTPUT_ENABLE, val);
    dirMap = 1 << gpioport;
    WL_WRITE_REGS32(GPIO_REG_OUTPUT_ENABLE, ~dirMap & val);

    if ( gpioport < 8)
    {
        WL_READ_REGS32(GPIO_REG_SELECT_0_7, val);
        WL_WRITE_WORD(GPIO_REG_SELECT_0_7, (3 << (2*gpioport)) | val);
    }
    else
    {
        WL_READ_REGS32(GPIO_REG_SELECT_8_15, val);
        WL_WRITE_WORD(GPIO_REG_SELECT_8_15, (3 << (2*(gpioport - 8))) | val);
    }
}

void apctlSwLedSetCycle(unsigned char gpioport, unsigned int cycle, unsigned int duty)
{
    unsigned int cycle_b = (cycle << (gpioport/4)) & 0xffff;
    unsigned int duty_b = (duty << (gpioport/4)) & 0xffff;
    unsigned int mask = ~(0xf << (gpioport/4)); 
    switch (gpioport)
    {
    case 0:
    case 1:
    case 2:
    case 3:
        WL_WRITE_WORD(SW_LED_CYCLE_3_0, (WL_REGS32(SW_LED_CYCLE_3_0) & mask) | cycle_b ); 
        WL_WRITE_WORD(SW_LED_DUTY_CYCLE_3_0, (WL_REGS32(SW_LED_DUTY_CYCLE_3_0) & mask) | duty_b );
        break;
    case 4:
    case 5:
    case 6:
    case 7:
        WL_WRITE_WORD(SW_LED_CYCLE_3_0, (WL_REGS32(SW_LED_CYCLE_7_4) & mask) | cycle_b ); 
        WL_WRITE_WORD(SW_LED_DUTY_CYCLE_3_0, (WL_REGS32(SW_LED_DUTY_CYCLE_7_4) & mask) | duty_b );
        break;
    case 8:
    case 9:
    case 10:
    case 11:
        WL_WRITE_WORD(SW_LED_CYCLE_3_0, (WL_REGS32(SW_LED_CYCLE_11_8) & mask) | cycle_b ); 
        WL_WRITE_WORD(SW_LED_DUTY_CYCLE_3_0, (WL_REGS32(SW_LED_DUTY_CYCLE_11_8) & mask) | duty_b );
        break;
    case 12:
    case 13:
    case 14:
    case 15:
        WL_WRITE_WORD(SW_LED_CYCLE_3_0, (WL_REGS32(SW_LED_CYCLE_15_12) & mask) | cycle_b ); 
        WL_WRITE_WORD(SW_LED_DUTY_CYCLE_3_0, (WL_REGS32(SW_LED_DUTY_CYCLE_15_12) & mask) | duty_b );
        break;
    }
}
#endif
/* GPIO control API */

/**
 * Initialize the GPIO pin for output or input
 * 
 * @param dirMap The GPIO direction bit map.
 *               Bit set to 1: the corresponding GPIO pin is configured
 *               for input.
 *               Bit set to 0: the corresponding GPIO pin is configured
 *               for output
 */
void apctlGPIOInit(unsigned long dirMap)
{
    UINT32 idx;
    UINT32 regVal;

    WL_WRITE_REGS32(GPIO_REG_OUTPUT_ENABLE, dirMap & 0xFFFF);
    /* Have two GPIO output source select registers, so do two loops */
    regVal = 0;
    for (idx = 0; idx < 8; idx++)
    {
        if (((dirMap >> idx) & 0x01) == 0)
        {
            /* Set to output, so select output source */
            regVal |= (1 << (2 * idx));
        }
    }
    WL_WRITE_WORD(GPIO_REG_SELECT_0_7, regVal);
    dirMap = dirMap >> 8;

    regVal = 0;
    for (idx = 0; idx < 8; idx++)
    {
        if (((dirMap >> idx) & 0x01) == 0)
        {
            /* Set to output, so select output source */
            regVal |= (1 << (2 * idx));
        }
    }
    WL_WRITE_WORD(GPIO_REG_SELECT_8_15, regVal);
}

/**
 * Test if GPIO number is configured as output mode
 * 
 * @param gpioNum the GPIO pin number
 * @return TRUE: output mode
 *         FALSE: input mode
 */
inline int gpioInOutputMode(int gpioNum)
{
    UINT32 dirMap;

    WL_READ_WORD(GPIO_REG_OUTPUT_ENABLE, dirMap);
    
    if (dirMap & (1 << gpioNum))
        return FALSE;
    else
        return TRUE;
}

/**
 * Read the state of a GPIO pin
 * 
 * @param gpioNum the GPIO pin number
 * @return 0: GPIO state is "low"
 *         1: GPIO state is "high"
 *        -1: Error: invalid GPIO number
 */
int apctlGPIORead(int gpioNum)
{
    UINT32 regVal;

    if (gpioNum >= MAX_GPIO_NUM)
    {
        return -1;
    }

    if (gpioInOutputMode(gpioNum))
    {
        /* GPIO pin is output */
        WL_READ_WORD(GPIO_REG_OUTPUTS, regVal);
        regVal = (regVal >> gpioNum) & 0x01;
    }
    else
    {
        /* GPIO pin is input */
        WL_READ_WORD(GPIO_REG_INPUTS, regVal);
        regVal = (regVal >> gpioNum) & 0x01;
    }

    return regVal;
}

/**
 * Set the state of a GPIO pin. The new GPIO state
 * after write is returned.
 * 
 * @param gpioNum The GPIO pin number
 * @param state   The state to set:
 *                0: set to "low"
 *                1: set to "high"
 * @return 0: the GPIO state is "low"
 *         1: the GPIO state is "high"
 *        -1: Error: invalid GPIO number
 *        -2: Error: GPIO pin is in input mode
 */
int apctlGPIOWrite(int gpioNum, int state)
{
    UINT32 regVal;

    if (gpioNum >= MAX_GPIO_NUM)
    {
        return -1;
    }

    if (gpioInOutputMode(gpioNum))
    {
        /* GPIO pin is output */

        /* Read the current setting and preserve what we do not touch */
        WL_READ_WORD(GPIO_REG_OUTPUTS, regVal);

        /* Set the output pin according to state */
        if (state)
        {
            /* Must return state value of 1 or 0 */
            state = 1;
            regVal |= (1 << gpioNum);
        }
        else
        {
            regVal &= ~(1 << gpioNum);
        }

        WL_WRITE_WORD(GPIO_REG_OUTPUTS, regVal);
    }
    else
    {
        /* GPIO pin is in input mode */
        return ( -2);
    }

    return state;
}

/* Timer API */
/**
 * Create a periodic timer. The callback function will run in the
 * Delayed Service Routine of the real-time clock interrupt.
 * 
 * @param period  The period in milliseconds
 * @param timerCbFunc The callback function
 * @return 0: failed to create timer because exceed maximum
 *         number of timers supported
 *         >0: the timer index, to be used in
 *         subsequent call to apctlTimerDelete()
 */
unsigned long apctlTimerCreate(int period, void (*timerCbFunc)(void))
{
#define MILLISECONDS_PER_TICK 10
    cyg_handle_t hSysClk;
    cyg_handle_t hCounter;

    int interval = period / MILLISECONDS_PER_TICK;

    /* For now, only support one timer */
    if (activeAlarmCnt > 0)
    {
        return 0;
    }

    /* Attach the timer to the real-time clock */
    hSysClk = cyg_real_time_clock();

    cyg_clock_to_counter(hSysClk, &hCounter);

    cyg_alarm_create(hCounter, (cyg_alarm_t *)timerCbFunc,
                     (cyg_addrword_t) &alarmData,
                     &hAlarm, &timerAlarm);

    /* This creates a periodic timer */
    cyg_alarm_initialize(hAlarm, cyg_current_time() + interval, interval);

    activeAlarmCnt++;

    return activeAlarmCnt;
}

/**
 * Delete a timer
 * 
 * @param timerIndex The timer index returned by apctlTimerCreate()
 * @return TRUE: success
 *         FALSE: failure because timer index is invalid
 */
int apctlTimerDelete(unsigned long timerIndex)
{
    /* For now, only support one timer */
    if (timerIndex != 1)
    {
        return FALSE;
    }

    --activeAlarmCnt;

    cyg_alarm_delete(hAlarm);

    return TRUE;
}

#ifdef RESET_BUTTON_EVENT
typedef struct
{
    void (*evtCbFunc)(ApEvtMsg_t *);   /* pointer to the event call back function */
    UINT8 apiEvtTbl[API_EVENT_MAX_EVENT]; /* List of events waiting for calling callback */
    UINT8 apiEvtIDAssigned;               /* id assigned flag */
}
API_EVT_CTRL;

API_EVT_CTRL apiEvtCtrl[API_EVENT_MAX_SUB];


void apEvtInit(void)
{
    return ;
}

BOOLEAN apEvtSub(unsigned long eventSet, void (*evtCbFunc)(ApEvtMsg_t *), unsigned int *pSubID)
{
    int i;
    static int apEvtInitFlag = 0;
    UINT32 val, mask;

    /*self contained initialization */
    if ( !apEvtInitFlag )
    {
        apEvtInit();
        apEvtInitFlag++;
    }
    /*safe guard */
    if (!evtCbFunc || !eventSet)
        return FALSE;

    if (eventSet == APEVT_RST_BTN_PRESSED)
    {
        WL_READ_WORD(GPIO_REG_INPUTS, val);
        mask = GPIO_PIN_SW_RESET;
#ifndef CONDOR2
        mask <<= 3;
#endif
        if (val & GPIO_PIN_SW_RESET)
            WL_REGS32_SETBITS(GPIO_REG_IER, mask);
        else
            WL_REGS32_CLRBITS(GPIO_REG_IER, mask);
        WL_REGS32_SETBITS(GPIO_REG_IMR, mask);
    }    
#ifdef AUTOLINK
    else if (eventSet == APEVT_UTIL_BTN_PRESSED)
    {
        WL_READ_WORD(GPIO_REG_INPUTS, val);
        mask = GPIO_PIN_SW_UTIL;
#ifndef CONDOR2
        mask <<= 3;
#endif
        if (val & GPIO_PIN_SW_UTIL)
            WL_REGS32_SETBITS(GPIO_REG_IER, mask);
        else
            WL_REGS32_CLRBITS(GPIO_REG_IER, mask);
        WL_REGS32_SETBITS(GPIO_REG_IMR, mask);
    }
#endif

    for ( i = 0; i < API_EVENT_MAX_SUB; i++)
    {
        if ( !apiEvtCtrl[i].apiEvtIDAssigned)
        {
            /*found an available id space */
            break;
        }
    }
    if ( i == API_EVENT_MAX_SUB )   /* out of id space */
        return FALSE;

    apiEvtCtrl[i].evtCbFunc = evtCbFunc;
    apiEvtCtrl[i].apiEvtIDAssigned = 1;
    *pSubID = i;

    /* set events in list table to active */
    for (i = 0; i < API_EVENT_MAX_EVENT; i++)
    {
        if ( eventSet & (1 << i) )
        {
            apiEvtCtrl[*pSubID].apiEvtTbl[i] = 1;
        }
    }
    return TRUE;
}


BOOLEAN apEvtUnsub(unsigned long eventSet, unsigned int subID)
{
    int i, left = 0;
    if (subID >= API_EVENT_MAX_SUB)
        return FALSE;

    for (i = 0; i < API_EVENT_MAX_EVENT; i++)
    {
        if ( eventSet & (1 << i) )
        {
            /*deactivate the event */
            apiEvtCtrl[subID].apiEvtTbl[i] = 0;
        }
        if (apiEvtCtrl[subID].apiEvtTbl[i])
        {
            /*count active events left */
            left++;
        }
    }
    if (!left)
    {
        /*no active events left, release the id space */
        apiEvtCtrl[subID].evtCbFunc = NULL;
        apiEvtCtrl[subID].apiEvtIDAssigned = 0;
    }
    return TRUE;
}

void apEvtRun(UINT16 evtCode, UINT8 *body, UINT8 status)
{
    int i;
    APEVT_STA_ASSOC_TYPE apevtmsg;
    UINT16 evtIndex = 0;
    apevtmsg.evtCode = evtCode;
    apevtmsg.evtTimeStamp = cyg_current_time();
    switch (evtCode)
    {
    case APEVT_STA_ASSOC:
        apevtmsg.evtBodyLen = 7;
        memcpy(apevtmsg.staMACAddress, body, 6);
        apevtmsg.assocType = status;
        break;
    case APEVT_RST_BTN_PRESSED:
        apevtmsg.evtBodyLen = 0;
        break;
    case APEVT_UTIL_BTN_PRESSED:
        apevtmsg.evtBodyLen = 0;
        break;
#if 0//def WDS_FEATURE
    case APEVT_UNKNOWN_WDS_ROUTE:
        apevtmsg.evtBodyLen = 6;
        memcpy(apevtmsg.staMACAddress, body, 6);
        break;
#endif
    default:
        apevtmsg.evtBodyLen = 0;
        break;
    }
    while ( !((evtCode >> evtIndex) == 1) )
    {
        evtIndex++;
    }
    if (evtIndex > API_EVENT_MAX_EVENT)
        return ;
    for (i = 0; i < API_EVENT_MAX_SUB; i++)
    {
        if (apiEvtCtrl[i].apiEvtIDAssigned)
        {
            if (apiEvtCtrl[i].apiEvtTbl[evtIndex])
                apiEvtCtrl[i].evtCbFunc((ApEvtMsg_t *)&apevtmsg);
        }
    }
}
#endif

#ifdef AP22c
UINT32 CheckAvailablePool(void)
{

    return NoPoolBufAvailable;

}
#endif

/** Set up gpio 1 to be use for blinking led during rx_rdy 
 **  0 = no pulse stretching
 **  1 = 17ms to 19ms
 **  2 = 36ms to 37ms
 **  3 = 73ms to 74ms
 **  4 = 148ms to 149ms
 **  5 = 297ms to 298ms
 **  6 = 596ms
 **  7 = 1191ms to 1192 ms
 **  8 = 2382ms to 2384 ms
**/       
void apctlLEDEnable(UINT16 pulse)
{

    UINT16 regVal;
    
    
    WL_READ_WORD(GPIO_REG_SELECT_0_7,regVal); 
    regVal=regVal&0xfff3;
    
    WL_WRITE_WORD(GPIO_REG_SELECT_0_7,regVal);  

    WL_READ_WORD(GPIO_REG_STRETCH,regVal);
    regVal=regVal&0xff0f;
    WL_WRITE_WORD(GPIO_REG_STRETCH,regVal|pulse<<4);
}


extern UINT8 StepValue;

#ifdef AP22N
void SetPowerRegValue(UINT8 reg20, UINT8 reg26,UINT8 tempStepValue)
{

	PowerReg20 = reg20;
	PowerReg26 = reg26;
	StepValue=tempStepValue;
}
#else
void SetPowerRegValue(UINT8 reg20, UINT8 reg26)
{

	PowerReg20 = reg20;
	PowerReg26 = reg26;
}



#endif

#if defined(NO_GUI) || defined (NO_ECOS_TCP_IP_STACK) 
/* Dummy function needed to link with latest libwlandrv.a */
int inet_address(int addr)
{
}

void initiate_dhcp(int flag)
{
}

#endif
#ifdef APCFGUR
void SendFilterCtrlMsgTo(extStaDb_StaInfo_t *StaInfo_p)
{
	UINT16 code, dataleft;
	UINT8 *ptr, *data;
	APICMDBUF *apicmdbuf;

	code =  APCMD_SET_WLAN_FILTER_LIST;
	{
		/*init*/
		dataleft = 2 + mib_wlanfilterno * 6;
		ptr = data = malloc(dataleft + APICMDHEADERLEN);
		apicmdbuf = (APICMDBUF *)data;
		apicmdbuf->seqNum = 1;
		apicmdbuf->cmdBody = (UINT8 *)(data + APICMDHEADERLEN);
		apicmdbuf->cmdCode = code;
		apicmdbuf->cmdBodyLen = dataleft;
		/*copy data */
	#if defined(UDP_API)
	    apicmdbuf->cmdBody[1] = 0; /*Cannot be more than 256 for now */
	    apicmdbuf->cmdBody[0] = mib_wlanfilterno % 0x100;
	#else
	    apicmdbuf->cmdBody[0] = 0; /*Cannot be more than 256 for now */
	    apicmdbuf->cmdBody[1] = mib_wlanfilterno % 0x100;
	#endif
		memcpy(&apicmdbuf->cmdBody[2], mib_wlanfiltermac, mib_wlanfilterno * 6);
	}
	RmSrvCtlEvent(StaInfo_p, data);
	code = APCMD_SET_WLAN_FILTER_MODE;
    {    
        APICMDBUF_SET_FILTER_MODE *apicmdbuf_set_filter_mode;
		/*init*/			
		dataleft = 1;
		ptr = data = malloc(dataleft+APICMDHEADERLEN);
		apicmdbuf = (APICMDBUF *)data;
		apicmdbuf->seqNum = 1;
		apicmdbuf->cmdCode = code;
		apicmdbuf->cmdBody = (UINT8 *)(data + APICMDHEADERLEN);
		/*copy data */
        apicmdbuf_set_filter_mode = (APICMDBUF_SET_FILTER_MODE *)apicmdbuf;
        apicmdbuf_set_filter_mode->mode = mib_wlanfiltertype;
        apicmdbuf_set_filter_mode->cmdBodyLen = 1;
	}
	RmSrvCtlEvent(StaInfo_p, data);
}
void SendMrvlCtlPkt(UINT8 *ToAddress, UINT8 *data, UINT16 length, UINT8 tods)
{
	extern apio_handle_t WlanHandle;
    apio_bufdescr_t BufDescr;
    if(apio_alloc(WlanHandle, &BufDescr) == APCTL_OK)
    {
        memcpy(BufDescr.framePtr, ToAddress, sizeof(IEEEtypes_MacAddr_t));
        memcpy(BufDescr.framePtr+6, mib_OpData_p->StaMacAddr, sizeof(IEEEtypes_MacAddr_t));
        BufDescr.framePtr[12] = 0x00;
        BufDescr.framePtr[13] = 0x03;
        BufDescr.framePtr[14] = 0xAA;
        BufDescr.framePtr[15] = 0xAA;
        BufDescr.framePtr[16] = 0x03;
        BufDescr.framePtr[17] = 0x00;
        BufDescr.framePtr[18] = 0x50;
        BufDescr.framePtr[19] = 0x43;
        BufDescr.framePtr[20] = 0x00;
        BufDescr.framePtr[21] = 0x03;
        memcpy(BufDescr.framePtr+22, data, length);
        BufDescr.frameLen = 22+length;
#ifdef AP_URPTR 
        if(tods)
		{
            apio_urWlanWrite(&BufDescr);
		}
		else
#endif
    	apio_write(WlanHandle,&BufDescr);
    }
}

void SendApiDataTo(UINT8 *addr, UINT8* data)
{
	UINT16 dataleft, code, seqnum=0, maxlen= MAX_API_DATA_THRESHOLD;
	UINT8 *ptr;
	APICMDBUF *apicmdbuf;
	UINT8 flag = 0;
	
	apicmdbuf = (APICMDBUF*)data;
	dataleft = apicmdbuf->cmdBodyLen;
	code = apicmdbuf->cmdCode;
	 
	while(dataleft > maxlen)
	{
		flag =1;
	   	apicmdbuf->cmdCode = code;
    	apicmdbuf->seqNum = ++seqnum;
		apicmdbuf->cmdBodyLen = maxlen;
		apicmdbuf->cmdStatus = APCTL_MORE_DATA;
		SendMrvlCtlPkt(addr,data, apicmdbuf->cmdBodyLen	+ APICMDHEADERLEN, 0);
		data += apicmdbuf->cmdBodyLen;
		apicmdbuf = (APICMDBUF *)data;
		dataleft -= maxlen;
	}
	apicmdbuf->cmdCode = code;
    apicmdbuf->seqNum= ++seqnum;
	apicmdbuf->cmdBodyLen = dataleft;
	if(flag)
		apicmdbuf->cmdStatus = APCTL_LAST_DATA;
	else
		apicmdbuf->cmdStatus = APCTL_OK;
   	SendMrvlCtlPkt(addr, data, apicmdbuf->cmdBodyLen + APICMDHEADERLEN, 0);
}

int SendCtrlMsgTo(UINT8* data, UINT16 len)
{
	UINT8 *buf, *copy;
    extStaDb_StaInfo_t *StaInfo_p;
	UINT16 i, nostation = extStaDb_entries(1);

	if(nostation)
	{
		buf = malloc(nostation*4);

    	extStaDb_list(buf, 2);
		i = 0;
		do
		{
			StaInfo_p = (extStaDb_StaInfo_t *)(*(UINT32 *)(buf+4*i));
			copy = malloc(len);
			memcpy(copy, data, len); 
			RmSrvCtlEvent(StaInfo_p, copy);
		}while(++i < nostation);
		free(buf);
	}
	return SUCCESS; 	
}
#endif

#ifdef DCS
UINT8 SetupBkgndScan(APICMDBUF_SET_BKGND_SCAN* cmd)
{
        SendBkgndScanCmd(cmd->enable, cmd->scanInterval,
                    cmd->channelsPerScan, cmd->channelScanTime,
                    cmd->scanType, cmd->enableCTS, cmd->channelList);
 
        return SUCCESS;
}
UINT8 SetDCSChannel(UINT8 channel)
{
    UINT8 cData = 0;
    UINT8 cData_old = 0;
    UINT32 nData = 0;
    UINT32 nData_old = 0;
    UINT8 rData = 0;
    UINT8 rData_old = 0;
    BOOLEAN restart = FALSE;

    /* Set Spectrum management to enable */
    cData =1; 
    {
	APICMDBUF_GET_SPECTRUM_MGMT cmdBuf;
	cmdBuf.cmdCode = APCMD_GET_SPECTRUM_MGMT;
    cmdBuf.seqNum = 1;
    cmdBuf.cmdBodyLen = 1;
    apctl(&cmdBuf);
    cData_old = cmdBuf.isEnable;
	}
    {
	APICMDBUF_SET_SPECTRUM_MGMT cmdBuf;
	cmdBuf.cmdCode = APCMD_SET_SPECTRUM_MGMT;
    cmdBuf.seqNum = 1;
    cmdBuf.cmdBodyLen = 1;
    cmdBuf.isEnable = cData;
    apctl(&cmdBuf);
	}
    restart = (cData_old != cData)?TRUE:FALSE;

    /* Set Mitigation to default value of 3*/
    nData = 3;
    {
	APICMDBUF_GET_MITIGATION cmdBuf;
	cmdBuf.cmdCode = APCMD_GET_MITIGATION;
    cmdBuf.seqNum = 1;
    cmdBuf.cmdBodyLen = sizeof(UINT32);
    apctl(&cmdBuf);
    nData_old = cmdBuf.mitigationValue;
	}
    {
	APICMDBUF_SET_MITIGATION cmdBuf;
	cmdBuf.cmdCode = APCMD_SET_MITIGATION;
    cmdBuf.seqNum = 1;
    cmdBuf.cmdBodyLen = 1;
    cmdBuf.mitigationValue = nData;
    apctl(&cmdBuf);
	}
    restart |= (nData_old != nData)?TRUE:FALSE;            
    
    /* Set regulatory domain to FCC */
    rData = 0x10;
    {
	APICMDBUF_GET_REGULATORY_DOMAIN cmdBuf;
	cmdBuf.cmdCode = APCMD_GET_REGULATORY_DOMAIN;
    cmdBuf.seqNum = 1;
    cmdBuf.cmdBodyLen = 1;
    apctl(&cmdBuf);
    rData_old = cmdBuf.domain;
	}
    {
	APICMDBUF_SET_REGULATORY_DOMAIN cmdBuf;
	cmdBuf.cmdCode = APCMD_SET_REGULATORY_DOMAIN;
    cmdBuf.seqNum = 1;
    cmdBuf.cmdBodyLen = 1;
    cmdBuf.domain = rData;
    apctl(&cmdBuf);
	}
    restart |= (rData_old != rData)?TRUE:FALSE;            
                    
    if (restart)
    {            
        //SendResetCmd(1);
    } 

    {
	APICMDBUF_SEND_SWITCH_CHANNEL_ANNOUNCEMENT cmdBuf;
	cmdBuf.cmdCode = APCMD_SEND_SWITCH_CHANNEL_ANNOUNCEMENT;
    cmdBuf.seqNum = 1;
    cmdBuf.cmdBodyLen = 1;
    cmdBuf.channel= channel;
    apctl(&cmdBuf);
	}

    return SUCCESS;
}
#endif /* DCS */
